# coding: utf-8
import logging

from django.conf import settings
from django.utils import translation

from horizon.utils.memoized import memoized

from neutronclient.neutron.client import Client as NeutronClient

from glanceclient.client import Client as GlanceClient

from openstack_dashboard import api
from openstack_dashboard.api import keystone
from openstack_dashboard.api.base import url_for

from tionix_client.os_clients.cinder_client import get_cinder_client
from tionix_client.os_clients.keystone_client import get_keystone_client
from tionix_client.os_clients.nova_client import (
    get_nova_client, get_admin_nova_client)

from tionix_dashboard.clients.helpers import expected_402_404
from tionix_dashboard.settings import CONF


logger = logging.getLogger(__name__)


@memoized
def novaclient(request, **kwargs):
    auth_token = kwargs.pop('auth_token', request.user.token.id)
    project_id = kwargs.pop('project_id', None) or request.user.tenant_id
    use_admin = kwargs.pop('use_admin', False)

    if use_admin:
        nova_client = get_admin_nova_client(CONF)
    else:
        if project_id != request.user.tenant_id:
            keystone_client = keystoneclient(request, project_id=project_id)
            auth_token = keystone_client.auth_token

        language = translation.get_language()
        current_locale = translation.to_locale(language)

        nova_client = get_nova_client(
            token=auth_token,
            tenant_id=project_id,
            user_locale=current_locale,
            **kwargs
        )
    return nova_client


def keystoneclient(request, **kwargs):
    keystone_url = url_for(
        request, settings.CONF.KEYSTONE['identity_service_name'])

    auth_token = kwargs.get('auth_token', request.user.token.id)
    project_id = kwargs.get('project_id', request.user.tenant_id)

    keystone_client = get_keystone_client(
        auth_url=keystone_url,
        tenant_id=project_id,
        token=auth_token)

    return keystone_client


def domain_keystoneclient(request, **kwargs):
    keystone_url = url_for(
        request, settings.CONF.KEYSTONE['identity_service_name'])
    domain_id = kwargs.get('domain_id', request.user.user_domain_id)
    domain_token = request.session.get('domain_token')
    auth_token = getattr(domain_token, 'auth_token')

    keystone_client = get_keystone_client(
        auth_url=keystone_url,
        token=auth_token,
        domain_id=domain_id)

    return keystone_client


def neutronclient(request, **kwargs):
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)

    auth_token = kwargs.get('auth_token', request.user.token.id)

    if not keystone.is_domain_admin(request):
        project_id = kwargs.get('project_id')
        if project_id != request.user.tenant_id:
            keystone_client = keystoneclient(request, project_id=project_id)
            auth_token = keystone_client.auth_token

    keystone_url = url_for(
        request, settings.CONF.KEYSTONE['identity_service_name'])
    neutron_url = url_for(
        request, settings.CONF.KEYSTONE['network_service_name'])

    client = NeutronClient(settings.CONF.NEUTRON_VERSION,
                           token=auth_token,
                           auth_url=keystone_url,
                           endpoint_url=neutron_url,
                           insecure=insecure, ca_cert=cacert)
    return client


def glanceclient(request, **kwargs):
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)

    auth_token = kwargs.get('auth_token', request.user.token.id)

    if not keystone.is_domain_admin(request):
        project_id = kwargs.get('project_id')
        if project_id != request.user.tenant_id:
            keystone_client = keystoneclient(request, project_id=project_id)
            auth_token = keystone_client.auth_token

    keystone_url = url_for(
        request, settings.CONF.KEYSTONE['identity_service_name'])
    glance_url = url_for(request, settings.CONF.KEYSTONE['image_service_name'])

    client = GlanceClient(settings.CONF.GLANCE_VERSION,
                          token=auth_token,
                          auth_url=keystone_url,
                          endpoint=glance_url,
                          insecure=insecure, ca_cert=cacert)
    return client


def cinderclient(request, **kwargs):

    project_id = kwargs.pop('project_id', None) or request.user.tenant_id
    auth_token = kwargs.get('auth_token', request.user.token.id)
    if project_id != request.user.tenant_id:
        keystone_client = keystoneclient(request, project_id=project_id)
        auth_token = keystone_client.auth_token

    language = translation.get_language()
    current_locale = translation.to_locale(language)

    cinder_client = get_cinder_client(
        token=auth_token,
        tenant_id=project_id,
        user_locale=current_locale,
        **kwargs
    )
    return cinder_client


class TNXBaseClient(object):
    """Base client for rest requests"""

    resource = None

    @classmethod
    def get_client(cls, request):
        raise NotImplementedError()

    @classmethod
    def get_resource(cls, request):
        return getattr(cls.get_client(request), cls.resource)


class TNXBaseNovaClient(TNXBaseClient):
    """Base nova client for rest requests"""

    @classmethod
    @expected_402_404(redirect=settings.WEBROOT, lic_type='all')
    def get(cls, request, identifier, project_id=None):
        return cls.get_resource(request, project_id).get(identifier)

    @classmethod
    @expected_402_404(redirect=settings.WEBROOT, lic_type='all')
    def list(cls, request, **kwargs):
        return cls.get_resource(request).list(**kwargs)

    @classmethod
    @expected_402_404(redirect=settings.WEBROOT, lic_type='all')
    def count(cls, request, **kwargs):
        return cls.get_resource(request).count(**kwargs)

    @classmethod
    @expected_402_404(redirect=settings.WEBROOT, lic_type='all')
    def update(cls, request, identifier, **kwargs):
        return cls.get_resource(request).update(identifier, **kwargs)

    # pylint: disable=arguments-differ
    @classmethod
    def get_resource(cls, request, project_id=None):
        return getattr(cls.get_client(request, project_id), cls.resource)

    # pylint: disable=arguments-differ
    @classmethod
    def get_client(cls, request, project_id=None):
        return novaclient(request, project_id=project_id)


class TNXBaseCinderClient(TNXBaseClient):
    """Base client for cinder"""

    @classmethod
    def get_client(cls, request):
        return cinderclient(request)


def tenant_get(request, project_id):
    if keystone.is_domain_admin(request):
        return keystone.tenant_get(request, project_id)
    else:
        client = keystoneclient(request, project_id=project_id)
        if keystone.VERSIONS.active < 3:
            manager = client.tenants
        else:
            manager = client.projects
        return manager.get(project_id)


def tenant_list(request, filters=None):
    if keystone.is_cloud_admin(request):
        manager = api.keystone.VERSIONS.get_project_manager(request,
                                                            admin=True)
        tenants = manager.list()
    elif keystone.is_domain_admin(request):
        tenants, _has_more_data = keystone.tenant_list(request,
                                                       filters=filters)
    else:
        client = keystoneclient(request)
        if keystone.VERSIONS.active < 3:
            manager = client.tenants
        else:
            manager = client.projects
        tenants = manager.list()
    return tenants
