# coding: utf-8

from openstack_dashboard.api import base, neutron, nova
from tionix_dashboard.clients.base import novaclient, neutronclient


class ProjectNeutronSecurityGroupManager(neutron.SecurityGroupManager):

    def __init__(self, request, project_id=None):
        super(ProjectNeutronSecurityGroupManager, self).__init__(request)
        self.project_id = project_id
        self.client = neutronclient(request, project_id=project_id)

    def list(self):
        tenant_id = self.project_id or self.request.user.tenant_id
        return self._list(tenant_id=tenant_id)

    def get_port_list(self, **params):
        ports = self.client.list_ports(**params).get('ports')
        return [neutron.Port(p) for p in ports]

    def update_instance_security_group(self, instance_id,
                                       new_security_group_ids):
        ports = self.get_port_list(device_id=instance_id)
        for p in ports:
            params = {'security_groups': new_security_group_ids}
            kwargs = neutron.unescape_port_kwargs(**params)
            self.client.update_port(p.id, body={'port': kwargs})

    def list_by_instance(self, instance_id):
        """Gets security groups of an instance."""
        ports = self.get_port_list(device_id=instance_id)
        sg_ids = []
        for p in ports:
            sg_ids += p.security_groups
        return self._list(id=set(sg_ids)) if sg_ids else []


class ProjectNovaSecurityGroupManager(nova.SecurityGroupManager):

    def __init__(self, request, project_id=None):
        super(ProjectNovaSecurityGroupManager, self).__init__(request)
        self.client = novaclient(request, project_id=project_id)


class ProjectSecurityGroupManager(object):
    def __init__(self, request, project_id=None):

        if (base.is_service_enabled(request, 'network') and
                neutron.is_extension_supported(request, 'security-group')):
            self.secgroups = ProjectNeutronSecurityGroupManager(request,
                                                                project_id)
        else:
            self.secgroups = ProjectNovaSecurityGroupManager(request,
                                                             project_id)


def get_security_groups(request, project_id=None):
    """Retrieve security group"""
    return ProjectSecurityGroupManager(request, project_id).secgroups.list()


def get_security_groups_by_instance(request, instance_id, project_id=None):
    """Retrieve security group filtered by instance"""
    return ProjectSecurityGroupManager(
        request, project_id).secgroups.list_by_instance(instance_id)


def server_update_security_groups(request, instance_id,
                                  new_security_group_ids, project_id=None):
    return ProjectSecurityGroupManager(
        request, project_id
    ).secgroups.update_instance_security_group(instance_id,
                                               new_security_group_ids)
