# coding: utf-8
# pylint: disable=too-many-arguments

import logging

from django.conf import settings
from django.core.urlresolvers import reverse_lazy
from django.utils.functional import cached_property

from horizon.utils import functions as utils

from novaclient.exceptions import NotFound

from openstack_dashboard.api import nova as nova_api

from tionix_dashboard.clients.base import TNXBaseNovaClient
from tionix_dashboard.clients.helpers import (
    encode_search_opts, expected_402_404)
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.helpers.utils import generate_timestamp_name
from tionix_dashboard.helpers.constants import (
    VM_SCSI_PROPERTIES_PREFIX, SCSI_PROPERTIES)
from tionix_dashboard.url_resolve_map import ADMIN_SERVERS_INDEX

logger = logging.getLogger(__name__)


class TNXServersClient(TNXBaseNovaClient):
    """Client for tnx servers"""

    resource = 'tnx_servers'

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.CLOUD_CONTROL)
    def get(cls, request, identifier, project_id=None):
        server = super(TNXServersClient, cls).get(request,
                                                  identifier,
                                                  project_id)
        return TnxServer(server, request)

    @classmethod
    @expected_402_404(
        redirect=reverse_lazy(ADMIN_SERVERS_INDEX),
        lic_type=LicenseController.CLOUD_CONTROL)
    def list(cls, request, **kwargs):
        servers = super(TNXServersClient, cls).list(request, **kwargs)
        return [TnxServer(server, request) for server in servers]

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.CLOUD_CONTROL)
    def update(cls, request, identifier, **kwargs):
        return super(TNXServersClient, cls).update(
            request, identifier, **kwargs)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.CLOUD_CONTROL)
    def get_paginator(cls, request, search_opts=None, all_tenants=False,
                      sort_dir=None, sort_key='created_at', last_page=False):
        """Copied from openstack api + return previous page info"""
        page_size = utils.get_page_size(request)
        paginate = False
        if search_opts is None:
            search_opts = {}
        elif 'paginate' in search_opts:
            paginate = search_opts.pop('paginate')
            if paginate:
                search_opts['limit'] = page_size + 1

        if all_tenants:
            search_opts['all_tenants'] = True
        else:
            search_opts['project_id'] = request.user.tenant_id

        total_count = cls.count(
            request, search_opts=encode_search_opts(search_opts))

        if last_page:
            search_opts['limit'] = total_count % page_size or page_size

        kwargs = {'search_opts': search_opts}
        if sort_dir is not None:
            kwargs['sort_dirs'] = (sort_dir, )

        if sort_key is not None:
            kwargs['sort_keys'] = (sort_key, )

        servers = cls.list(request, **kwargs)

        has_prev_data = has_more_data = False
        marker = search_opts.get('marker')

        if paginate:
            count = len(servers)
            # first and middle page condition
            if count > page_size:
                servers.pop(-1)
                has_more_data = True
                # middle page condition
                if marker is not None:
                    has_prev_data = True
            # first page condition when reached via prev back,
            # but not last page
            elif count == page_size and sort_dir == 'asc' and not last_page:
                has_more_data = True
            # last page condition
            elif marker is not None or last_page:
                has_prev_data = True

        if sort_dir == 'asc':
            servers.reverse()

        return servers, has_more_data, has_prev_data, total_count

    @classmethod
    @expected_402_404(redirect=settings.WEBROOT,
                      lic_type=LicenseController.MONITOR)
    def get_instance_metric(cls, request, instance_id, project_id=None):
        try:
            instance = cls.get(request, instance_id, project_id=project_id)
        except NotFound:
            logger.debug(u"Instance was not found by id={0}".format(
                instance_id))
            metrics = {}
        else:
            metrics = cls.get_client(
                request, project_id=project_id
            ).tnx_servers.metrics(instance)
            response_message = metrics.get('message')
            if response_message:
                logger.debug(response_message)

        return metrics

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT,
        lic_type=(LicenseController.NODE_CONTROL,
                  LicenseController.CLOUD_CONTROL))
    def create_image_for_instance(cls, request, instance_id):
        instance = cls.get(request, instance_id)
        image_name = generate_timestamp_name(instance.name)

        logger.debug(
            u'create image for instance, with name = "{0}". '
            u'image name = "{1}"'.format(instance.name, image_name))

        # TODO For images, image_type must be empty.
        metadata = {'image_type': 'image'}
        instance.apiresource.create_image(image_name=image_name,
                                          metadata=metadata)

        return instance

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.CLOUD_CONTROL)
    def count(cls, request, **kwargs):
        return super(TNXServersClient, cls).count(request, **kwargs)

    @classmethod
    def create(cls, request, name, image, flavor, user_data=None,
               description=None, **kwargs):
        return TnxServer(
            cls.get_resource(request).create(
                name, image, flavor, userdata=user_data,
                description=description, **kwargs),
            request
        )

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.CLOUD_CONTROL)
    def reset_with_reboot(cls, request, identifier, reboot, project_id=None):
        return cls.get_resource(request, project_id).reset_with_reboot(
            identifier, reboot=reboot
        )

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.CLOUD_CONTROL)
    def balance(cls, request, identifier):
        return cls.get_resource(request).balance(identifier)

    @classmethod
    def get_metrics_names(cls, request):
        return cls.get_client(request).tnx_servers.metrics_names().get(
            'metrics_names'
        )

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.MONITOR)
    def add_backup_templates(cls, request, instance_id, template_ids):
        return cls.get_client(request).tnx_servers.add_backup_templates(
            instance_id, template_ids
        )

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.MONITOR)
    def remove_backup_templates(cls, request, instance_id, template_ids):
        return cls.get_client(request).tnx_servers.remove_backup_templates(
            instance_id, template_ids
        )

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def add_user(cls, request, instance_id, user_id):
        return cls.get_client(request).tnx_servers.add_user(instance_id,
                                                            user_id)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def remove_user(cls, request, instance_id, user_id):
        return cls.get_client(request).tnx_servers.remove_user(instance_id,
                                                               user_id)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def add_user_group(cls, request, instance_id, group_id):
        return cls.get_client(request).tnx_servers.add_user_group(instance_id,
                                                                  group_id)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def remove_user_group(cls, request, instance_id, group_id):
        return cls.get_client(request).tnx_servers.remove_user_group(
            instance_id, group_id)


# noinspection PyClassHasNoInit
class TnxServer(nova_api.Server):

    _attrs = ['addresses', 'attrs', 'id', 'image', 'links',
              'metadata', 'name', 'private_ip', 'public_ip', 'status', 'uuid',
              'image_name', 'VirtualInterfaces', 'flavor', 'key_name', 'fault',
              'tenant_id', 'user_id', 'created', 'OS-EXT-STS:power_state',
              'OS-EXT-STS:vm_state', 'description', 'security_groups',
              'OS-EXT-STS:task_state', 'OS-EXT-SRV-ATTR:instance_name',
              'OS-EXT-SRV-ATTR:host', 'OS-EXT-AZ:availability_zone',
              'OS-DCF:diskConfig', 'tnx_node_title', 'tnx_node_id',
              'tnx_operation_mode', 'tnx_type', 'node_id', 'tnx_movable',
              'tnx_groups', 'tnx_users', 'tnx_backup_templates']

    @property
    def apiresource(self):
        return self._apiresource

    @cached_property
    def is_direct_scsi(self):
        metadata = getattr(self, 'metadata', {})
        return all(metadata.get(VM_SCSI_PROPERTIES_PREFIX + key) == val
                   for key, val in SCSI_PROPERTIES.items())
