# coding: utf-8

import logging

from django.conf import settings
from django.core.urlresolvers import reverse_lazy

from horizon.utils import functions as utils

from openstack_dashboard.api import base, keystone

from tionix_dashboard.clients.base import TNXBaseNovaClient
from tionix_dashboard.clients.helpers import expected_402_404
from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.url_resolve_map import ADMIN_SERVERS_INDEX

logger = logging.getLogger(__name__)


class TNXTenantClient(TNXBaseNovaClient):
    """Client for tnx servers"""

    resource = 'tnx_tenants'

    # pylint: disable=arguments-differ
    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def get(cls, request, identifier, only_vdi_vm=False):
        server = cls.get_resource(request).get(identifier, only_vdi_vm)
        return TnxTenant(server, request)

    @classmethod
    @expected_402_404(
        redirect=reverse_lazy(ADMIN_SERVERS_INDEX),
        lic_type=LicenseController.VDI)
    def list(cls, request, **kwargs):
        tenants = super(TNXTenantClient, cls).list(request, **kwargs)
        return [TnxTenant(tenant, request) for tenant in tenants]

    @classmethod
    @expected_402_404(
        redirect=reverse_lazy(ADMIN_SERVERS_INDEX),
        lic_type=LicenseController.VDI)
    def list_vdi(cls, request, **kwargs):
        filters = kwargs.pop('filters', {})
        filters.update({'tnx_project_type': TenantTypeEnum.VDI,
                        'only_vdi_vm': True})
        kwargs.update({'filters': filters})
        return cls.list(request, **kwargs)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def update(cls, request, identifier, **kwargs):
        return super(TNXTenantClient, cls).update(
            request, identifier, **kwargs)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def delete(cls, request, identifier):
        return cls.get_resource(request).delete(identifier)

    @classmethod
    @expected_402_404(
        redirect=settings.WEBROOT, lic_type=LicenseController.VDI)
    def get_paginator(cls, request, marker=None, is_vdi=False, **kwargs):

        page_size = utils.get_page_size(request)

        limit, has_more_data = page_size + 1, False

        kwargs['limit'] = limit

        if marker:
            kwargs['marker'] = marker

        if is_vdi:
            tenants = cls.list_vdi(request, **kwargs)
        else:
            tenants = cls.list(request, **kwargs)

        if keystone.VERSIONS.active < 3 and len(tenants) > page_size:
            tenants.pop(-1)
            has_more_data = True

        return tenants, has_more_data


class TnxTenant(base.APIResourceWrapper):
    _attrs = [
        'id', 'description', 'enabled', 'tnx_operation_mode', 'name',
        'tnx_network_id', 'tnx_image_id', 'tnx_flavor_id',
        'tnx_instances_count', 'domain_id', 'tnx_project_type',
        'tnx_availability_zone'
    ]

    def __init__(self, apiresource, request):
        super(TnxTenant, self).__init__(apiresource)
        self.request = request

    @property
    def tnx_project_type_name(self):
        return TenantTypeEnum.values.get(
            getattr(self, "tnx_project_type", TenantTypeEnum.TYPICAL)
        )
