# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, workflows

from novaclient.exceptions import NotFound

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.flavors import (
    views as admin_views)

from tionix_dashboard.dashboards.admin.flavors.tables import FlavorsTable
from tionix_dashboard.dashboards.admin.flavors.workflows import (
    CreateFlavor, ModifyAccessFlavor, UpdateFlavor)
from tionix_dashboard.helpers.views import LinkBatchWorkflowViewMixin
from tionix_dashboard.helpers.views import DTVTnxPaginationMixin

logger = logging.getLogger(__name__)


# noinspection PyClassHasNoInit
class IndexView(DTVTnxPaginationMixin,
                admin_views.IndexView):
    table_class = FlavorsTable

    # noinspection PyBroadException,PyAttributeOutsideInit,PyProtectedMember
    # pylint: disable=no-member, attribute-defined-outside-init
    def get_data(self):
        offset, limit = self.get_offset_limit()

        request = self.request
        try:
            # Removing the pagination params and adding "is_public=None"
            # will return all flavors.
            flavors, _more, _prev = api.nova.flavor_list_paged(
                request, None,
                sort_dir='asc'
            )
        except Exception:
            flavors = []
            self._prev = self._more = False
            exceptions.handle(request,
                              _('Unable to retrieve flavor list.'))

        self.table.kwargs.update({'offset': offset, 'limit': limit})
        return flavors

    def has_filters(self):
        return self.table.get_filter_field() and self.table.get_filter_string()


# noinspection PyClassHasNoInit
class ModifyAccessView(LinkBatchWorkflowViewMixin, workflows.WorkflowView):
    workflow_class = ModifyAccessFlavor
    submit_url = 'horizon:admin:flavors:modify_access'

    def get_initial(self):
        initial = super(ModifyAccessView, self).get_initial()
        flavors = []
        for flavor_id in self.get_object_ids():
            try:
                flavor = api.nova.flavor_get(self.request, flavor_id)
                flavors.append((flavor.id, flavor.name))
            except Exception as err:
                logger.error(err)
                continue
        if flavors:
            self.request.session['flavors'] = flavors
        else:
            flavors = self.request.session.pop('flavors', [])
        initial['flavors'] = flavors
        return initial


# noinspection PyClassHasNoInit
class UpdateView(admin_views.UpdateView):
    workflow_class = UpdateFlavor

    def get_initial(self):
        initial = super(UpdateView, self).get_initial()

        flavor_id, flavor_access = self.kwargs['id'], []
        try:
            flavor_access = [project.tenant_id for project in
                             api.nova.flavor_access_list(self.request,
                                                         flavor_id)]
        except NotFound:
            pass
        except Exception as err:
            logger.error(err)
            exceptions.handle(self.request,
                              _('Unable to retrieve flavor access list. '
                                'Please try again later.'))

        initial['flavor_access'] = flavor_access
        return initial


# noinspection PyClassHasNoInit
class CreateView(admin_views.CreateView):
    workflow_class = CreateFlavor
