# coding: utf-8

"""Admin flavors workflows"""

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, messages, workflows

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.flavors import (
    workflows as admin_workflows)

from tionix_dashboard.helpers.fields import AutoGenNameRegexField


logger = logging.getLogger(__name__)


# pylint: disable=super-on-old-class
class UpdateFlavorAccessAction(workflows.MembershipAction):
    def __init__(self, request, context, *args, **kwargs):
        super(UpdateFlavorAccessAction, self).__init__(
            request, context, *args, **kwargs)

        # Get list of available projects.
        all_projects = []
        try:
            all_projects, _has_more = api.keystone.tenant_list(request)
        except Exception as err:
            err_msg = _('Unable to retrieve flavor access list. '
                        'Please try again later.')
            logger.error(err)
            exceptions.handle(request, err_msg)

        projects_list = [(project.id, project.name)
                         for project in all_projects]
        # sort by name
        projects_list.sort(key=lambda x: x[1].lower())

        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)

        self.fields[field_name].choices = projects_list

        # If we have a POST from the CreateFlavor workflow, the flavor id
        # isn't an existing flavor. For the UpdateFlavor case, we don't care
        # about the access list for the current flavor anymore as we're about
        # to replace it.
        if request.method == 'POST':
            return

        self.fields[field_name].initial = context.get('flavor_access', [])

    class Meta(object):
        name = _("Flavor Access")
        slug = "update_flavor_access"


# noinspection PyClassHasNoInit
class UpdateFlavorAccess(admin_workflows.UpdateFlavorAccess):
    action_class = UpdateFlavorAccessAction


# noinspection PyClassHasNoInit
class UpdateFlavor(admin_workflows.UpdateFlavor):

    default_steps = (admin_workflows.UpdateFlavorInfo,
                     UpdateFlavorAccess)


# noinspection PyClassHasNoInit
class MultiUpdateFlavorAccess(admin_workflows.UpdateFlavorAccess):
    action_class = UpdateFlavorAccessAction
    depends_on = ('flavors',)
    contributes = ('flavor_access',)


class ModifyAccessFlavor(workflows.Workflow):
    """Modify access workflow"""
    slug = "modify_access"
    name = _("Modify Access")
    finalize_button_name = _("Modify Access")
    handle_success_message = _('Modified access for flavors: "{0}".')
    handle_failure_message = _('Unable to modify access for flavors: "{0}".')
    success_url = "horizon:admin:flavors:index"
    default_steps = (MultiUpdateFlavorAccess,)

    # pylint: disable=no-member
    def handle(self, request, context):
        success_flavors, failed_flavors = [], []
        flavors = context.get('flavors', [])
        for flavor_id, flavor_name in flavors:
            for project in context['flavor_access']:
                try:
                    api.nova.add_tenant_to_flavor(
                        request, flavor_id, project)
                except Exception as err:
                    logger.error(unicode(err))
                    failed_flavors.append(flavor_name)
                else:
                    success_flavors.append(flavor_name)

        if success_flavors:
            messages.success(request, self.handle_success_message.format(
                u'", "'.join(success_flavors)))

        if failed_flavors:
            messages.error(request, self.handle_failure_message.format(
                u'", "'.join(failed_flavors)))
        return True


# noinspection PyClassHasNoInit
class CreateFlavorInfoAction(admin_workflows.CreateFlavorInfoAction):
    name = AutoGenNameRegexField(
        label=_("Name"),
        max_length=255,
        regex=r'^[\w\.\- ]+$',
        error_messages={'invalid': _('Name may only contain letters, numbers, '
                                     'underscores, periods and hyphens.')})
    swap_mb = forms.IntegerField(label=_("Swap Disk (MB)"),
                                 required=False,
                                 initial=0,
                                 min_value=0)

    class Meta(object):
        name = _("Flavor Information")
        help_text = _("Flavors define the sizes for RAM, disk, number of "
                      "cores, and other resources and can be selected when "
                      "users deploy instances.")


# noinspection PyClassHasNoInit
class CreateFlavorInfo(admin_workflows.CreateFlavorInfo):
    action_class = CreateFlavorInfoAction


# noinspection PyClassHasNoInit
class CreateFlavor(admin_workflows.CreateFlavor):
    default_steps = (CreateFlavorInfo,
                     admin_workflows.UpdateFlavorAccess)


# noinspection PyClassHasNoInit
class UpdateFlavorInfoAction(CreateFlavorInfoAction):

    class Meta(object):
        name = _("Flavor Information")
        slug = 'update_info'
        help_text = _("Edit the flavor details. Flavors define the sizes for "
                      "RAM, disk, number of cores, and other resources. "
                      "Flavors are selected when users deploy instances.")
