# coding: utf-8
# pylint: disable=no-member

import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import (
    ugettext_lazy as _, pgettext_lazy, ungettext_lazy)

from horizon import tables, messages, exceptions

from openstack_dashboard import api

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, LinkBatchAction, TnxUpdateAction, TnxBatchActionMixin)
from tionix_dashboard.dashboards.admin.hypervisors.power_actions import (
    POWER_OFF_STATE, POWER_ON_STATE)
from tionix_dashboard.dashboards.admin.hypervisors.states import (
    STATE_UP, STATE_DOWN)

logger = logging.getLogger(__name__)


class TnxHypervisorUpdateRow(tables.Row):
    """ Action update hypervisor row in grid. """
    ajax = True

    def get_data(self, request, obj_id):
        try:
            hypervisor = TNXHypervisorsClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.critical(err)
            raise ObjectDoesNotExist()
        return hypervisor


class TnxImageUpdateCell(tables.UpdateAction):
    """
    Action updated hypervisor's record with new value of cell "Image for PXE".
    """
    def allowed(self, request, datum, cell):
        return api.keystone.keystone_can_edit_project()

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """ Inline update of image-cell. """
        if cell_name == 'image':
            if new_cell_value.isdigit():
                new_cell_value = int(new_cell_value)
            try:
                TNXHypervisorsClient.assign_hypervisor_image(
                    request, obj_id, int(new_cell_value))
            except Exception as err:
                logger.error(unicode(err))
                msg = _('Failed to assign image '
                        'for hypervisors: "{0}"').format(new_cell_value)
                messages.error(request, msg)


class TnxNodeUpdateCell(TnxUpdateAction):
    """ Action update hypervisor title """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        TNXHypervisorsClient.update(
            request, obj_id, **{cell_name: new_cell_value})


class TnxSetReserveAction(tables.UpdateAction):
    """ Action set hypervisor as reserved. """
    def allowed(self, request, datum, cell):
        return api.keystone.keystone_can_edit_project()

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """ Inline update of is_reserved-cell. """
        if cell_name == 'is_reserved':
            TNXHypervisorsClient.set_reserved_flag_hypervisor(
                request, obj_id, new_cell_value)


# noinspection PyAbstractClass
class HypervisorExtraAction(tables.LinkAction):
    """ Action show extra information for selected hypervisor. """
    name = 'hypervisors'
    verbose_name = _('More')
    url = "horizon:admin:hypervisors:extra"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )

    def get_link_url(self, datum=None):
        hypervisor_id = datum.id if datum else None
        return reverse(self.url, args=[hypervisor_id])


# noinspection PyAbstractClass,PyUnusedLocal
class RestartHypervisorAction(LinkBatchAction):
    """ Action restart hypervisor. """
    name = 'restart'
    policy_rules = (('identity', 'admin_required'), )
    multiple_next_view_name = "horizon:admin:hypervisors:restart"
    classes = ("btn-danger",)

    def action_present(self, count):
        return _('Restart')

    def allowed(self, request, datum):
        allowed = datum is None

        if datum and datum.tnx_devices:
            if datum.tnx_power_state is None:
                allowed = datum.state == STATE_UP
            else:
                allowed = datum.tnx_power_state == POWER_ON_STATE

        return allowed


# noinspection PyAbstractClass,PyUnusedLocal
class PowerOnHypervisorAction(TnxBatchActionMixin, tables.BatchAction):
    """ Action power on hypervisor. """
    name = 'power_on'
    verbose_name = _('Power on')
    policy_rules = (('identity', 'admin_required'), )

    @staticmethod
    def action_present(count):
        return _("Power on")

    @staticmethod
    def action_past(count):
        return _("Power on")

    def action(self, request, datum_id):
        datum = self.table.get_object_by_id(datum_id)
        if datum.has_default_ssh_device:
            msg = _('The action is not available for SSH.')
            raise exceptions.NotAvailable(msg)
        TNXHypervisorsClient.power_on_hypervisor(request, datum_id)

    def allowed(self, request, datum):
        allowed = True
        if datum:
            allowed = False
            if datum.tnx_devices:
                if datum.tnx_power_state is None:
                    allowed = datum.state == STATE_DOWN
                else:
                    allowed = datum.tnx_power_state == POWER_OFF_STATE

        return allowed


# noinspection PyAbstractClass,PyUnusedLocal
class PowerOffHypervisorAction(LinkBatchAction):
    """ Action power off hypervisor. """
    name = 'power_off'
    multiple_next_view_name = "horizon:admin:hypervisors:power_off"
    classes = ("btn-danger",)
    policy_rules = (('identity', 'admin_required'), )

    def action_present(self, count):
        return _('Power off')

    def allowed(self, request, datum):
        allowed = datum is None

        if datum and datum.tnx_devices:
            if datum.tnx_power_state is None:
                allowed = datum.state == STATE_UP
            else:
                allowed = datum.tnx_power_state == POWER_ON_STATE

        return allowed


# noinspection PyAbstractClass,PyUnusedLocal
class StartReservationHypervisorAction(LinkBatchAction):
    """ Action start reservation hypervisor. """
    name = 'markreserved'
    policy_rules = (('identity', 'admin_required'), )
    multiple_next_view_name = "horizon:admin:hypervisors:markreserved"

    def action_present(self, count):
        return _('Move to reserve')

    def allowed(self, request, datum):
        allowed = datum is None

        if datum and datum.tnx_devices:
            # move to reserv only if device state is known
            allowed = datum.tnx_power_state == POWER_ON_STATE

        return allowed


# noinspection PyAbstractClass,PyUnusedLocal
class HypervisorShutdownAction(LinkBatchAction):
    """ Action shut down hypervisor. """
    name = 'shutdown'
    multiple_next_view_name = "horizon:admin:hypervisors:shutdown"
    classes = ("btn-danger",)
    policy_rules = (('identity', 'admin_required'), )

    def action_present(self, count):
        return _('Shutdown')

    def allowed(self, request, datum):
        allowed = datum is None

        if datum and datum.tnx_devices:
            if datum.tnx_power_state is None:
                allowed = datum.state == STATE_UP
            else:
                allowed = datum.tnx_power_state == POWER_ON_STATE
        return allowed


# noinspection PyAbstractClass,PyUnusedLocal
class HypervisorAssignImageAction(LinkBatchAction):
    """ Multi action assign image for hypervisor. """
    name = 'assign_image'
    multiple_next_view_name = "horizon:admin:hypervisors:assign_image"
    classes = ("btn-danger",)
    policy_rules = (('identity', 'admin_required'), )

    def action_present(self, count):
        return _('Assign Image')


# noinspection PyAbstractClass
class TnxAdminHypervisorsFilterAction(TnxFilterAction, tables.FilterAction):

    name = "filter_admin_hypervisors"
    filter_type = "server"

    filter_choices = (
        ('hostname', _("Hostname"), False),
        ('node_title', pgettext_lazy("Hypervisor", "Node title"), False),
        ('hypervisor_type', _("Type"), False),
        ('image', _("Image"), False),
        ('state', _("State"), False),
        ('power_state', _("Power state"), False),
        ('is_reserved', _("Reserved"), False),
    )


# noinspection PyAbstractClass,PyUnusedLocal
class HypervisorUpdateAction(LinkBatchAction):
    """Hypervisor update action"""
    name = 'update'
    multiple_next_view_name = "horizon:admin:hypervisors:update"
    policy_rules = (('identity', 'admin_required'), )

    def action_present(self, count):
        return ungettext_lazy(
            u"Edit hypervisor",
            u"Edit hypervisors",
            count
        )

    def action_past(self, count):
        return ungettext_lazy(
            u"Edited hypervisor",
            u"Edited hypervisors",
            count
        )


class TnxConnectedDeviceUpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        update_device = None
        try:
            hypervisor = TNXHypervisorsClient.get(
                request,
                self.table.kwargs['hypervisor']
            )
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, _('Unable to get hypervisor.'))
        else:
            for device in hypervisor.devices:
                if obj_id == str(device.id):
                    update_device = device
                    break
        return update_device


class SetPowerDevicePortUpdateCell(tables.UpdateAction):
    def allowed(self, request, datum, cell):
        allowed = True
        coil_numbers = cell.row.table.devices_coil_map.get(datum.id, [])
        if len(coil_numbers) > 1:
            coil_numbers = coil_numbers.keys()
            try:
                coil_numbers.sort(key=int)
            except Exception as err:
                logger.error(unicode(err))
            cell.column.form_field.choices = zip(coil_numbers, coil_numbers)
        else:
            allowed = False
        return allowed

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        result = True
        if datum.device_port != new_cell_value:
            new_cell_value = (
                int(new_cell_value) if new_cell_value.isdigit() else
                new_cell_value
            )
            try:
                TNXHypervisorsClient.set_power_device_port_for_node(
                    request=request,
                    node_id=datum.hypervisor_id,
                    power_device_id=datum.id,
                    power_device_port=new_cell_value
                )
            except Exception as err:
                logger.error(unicode(err))
                messages.error(request, _('Failed to set coil number.'))
                result = False

        return result


# noinspection PyAbstractClass
class AssignPowerDeviceAction(tables.LinkAction):
    """Assign power device action"""
    name = "assign_power_device"
    verbose_name = _("Assign Power Control")
    url = "horizon:admin:hypervisors:assign_power_device"
    classes = ("ajax-modal",)
    policy_rules = (('identity', 'admin_required'), )
