# coding: utf-8
# pylint: disable=no-member
# pylint: disable=abstract-method
# pylint: disable=deprecated-lambda

import logging

from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import forms, messages

from tionix_dashboard.clients import TNXHypervisorsClient, TNXHostsClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.helpers.fields import NullableCharField

logger = logging.getLogger(__name__)


class HypervisorExtra(forms.Form):
    """Hypervisor Extra information"""

    name = _("Hypervisor extra information")
    id = forms.CharField(label=_('Node id'),
                         show_hidden_initial=True)
    human_id = forms.CharField(label=_('Verbose name'))
    host_ip = forms.CharField(label=_('Host IP'))
    current_workload = forms.CharField(label=_('Current workload'))
    free_ram_mb = forms.CharField(label=_('Free RAM (MB)'))
    free_disk_gb = forms.CharField(label=_('Free disk space (GB)'))
    disk_available_least = forms.CharField(
        label=_('Available disk space (GB)')
    )
    hypervisor_hostname = forms.CharField(label=_('Hypervisor hostname'))
    hypervisor_type = forms.CharField(label=_('Hypervisor type'))
    hypervisor_version = forms.CharField(label=_('Hypervisor version'))
    memory_mb = forms.CharField(label=_('Total RAM (MB)'))
    memory_mb_used = forms.CharField(label=_('Used RAM (MB)'))
    running_vms = forms.CharField(
        label=pgettext_lazy('Hypervisor', 'Instances')
    )
    vcpus = forms.CharField(label=_('Total vCPU'))
    vcpus_used = forms.CharField(label=_('Used vCPU'))
    tnx_inventory_code = forms.CharField(label=_('Inventory number'))
    tnx_dtc_number = forms.CharField(label=_('Data Center number'))
    tnx_row_number = forms.CharField(label=_('Row number'))
    tnx_rack_number = forms.CharField(label=_('Rack number'))
    tnx_unit_number = forms.CharField(label=_('Place in rack'))
    node_name = forms.CharField(label=_('Node name'))
    node_port = forms.CharField(label=_('Node port'))
    node_mac_address = forms.CharField(label=_('Node MAC address'))
    tnx_image_name = forms.CharField(label=_('Image name'))

    def __init__(self, request, *args, **kwargs):
        super(HypervisorExtra, self).__init__(request, *args, **kwargs)
        self.init_values()

        self.places_fields_caption = _("Location information")
        self.places_fields = [
            self.fields['tnx_dtc_number'],
            self.fields['tnx_row_number'],
            self.fields['tnx_rack_number'],
            self.fields['tnx_unit_number'],
            self.fields['tnx_inventory_code'],
        ]

        self.physical_info_caption = _("Physical information")
        self.physical_info_fields = filter(lambda x: hasattr(x, 'value'), [
            self.fields['id'],
            self.fields['node_name'],
            self.fields['node_port'],
            self.fields['node_mac_address'],
            self.fields['human_id'],
            self.fields['host_ip'],
            self.fields['current_workload'],
            self.fields['free_disk_gb'],
            self.fields['disk_available_least'],
            self.fields['hypervisor_type'],
            self.fields['hypervisor_version'],
            self.fields['memory_mb'],
            self.fields['free_ram_mb'],
            self.fields['memory_mb_used'],
            self.fields['running_vms'],
            self.fields['vcpus'],
            self.fields['vcpus_used'],
        ])

        self.image_info_caption = _("Image information")
        self.image_fields = [
            self.fields['tnx_image_name'],
        ]

        self.columns_count = 2

        self.devices_info_columns = [
            _(u'Control title'), _(u'Coil number'), _(u'Is default')
        ]

    def init_values(self):
        """Write initial value to each field in form from object"""
        if self.initial:
            for field in self.fields:
                if field in self.initial:
                    field_obj = self.fields[field]
                    field_obj.value = self.initial[field]

    @classmethod
    def get_fields(cls):
        return cls.base_fields.keys()


# noinspection PyAbstractClass
class HypervisorWithInstancesForm(forms.SelfHandlingForm):

    table_caption = _('Instances list')

    columns = [
        _('Name'), _('Project'), _('Host name'), _('State'),
        _('Internal IP'), _('Type')
    ]

    action_name = None
    async_action = False
    sync_message = _('Action "{0}" successfully done for hypervisors: {1}')
    async_message = _(
        'Request to perform "{0}" action has been submitted '
        'for hypervisors: {1}')

    def do_action(self, request, data, _id):
        pass

    def handle(self, request, data):
        hypervisors = self.initial['hypervisors']

        success_hyps, failed_hyps = [], []
        for _id, name in hypervisors:
            try:
                self.do_action(request, data, _id)
            except Exception as err:
                logger.error(unicode(err))
                failed_hyps.append(name)
            else:
                success_hyps.append(name)

        if failed_hyps:
            msg = _('Failed to perform "{0}" for hypervisors: {1}')
            messages.error(
                request, msg.format(self.action_name,
                                    '"' + u'", "'.join(failed_hyps) +
                                    '"'))

        if success_hyps:
            if self.async_action:
                msg = self.async_message
            else:
                msg = self.sync_message

            messages.success(
                request, msg.format(self.action_name,
                                    '"' + u'", "'.join(success_hyps) +
                                    '"'))

        return True


# noinspection PyAbstractClass
class HypervisorShutdown(HypervisorWithInstancesForm):
    """Hypervisor Shutdown form"""

    name = _("Shutdown hypervisor")
    allow_off_vm = forms.BooleanField(
        required=False,
        label=_('Allow to stop and migrate instance if live '
                'migration fails'))
    allow_evacuate = forms.BooleanField(
        required=False,
        label=_('Allow to evacuate instance if migration fails'))

    action_name = _('Shutdown')
    async_action = True

    def do_action(self, request, data, _id):
        TNXHypervisorsClient.shut_down_hypervisor(
            request,
            _id,
            data['allow_off_vm'],
            data['allow_evacuate'])


# noinspection PyAbstractClass
class HypervisorPowerOff(HypervisorWithInstancesForm):
    """Hypervisor Power off"""

    name = _("Power off hypervisor")

    action_name = _('Power off')

    def do_action(self, request, data, _id):
        TNXHypervisorsClient.power_off_hypervisor(request, _id)


# noinspection PyAbstractClass
class HypervisorRestart(HypervisorWithInstancesForm):
    """Hypervisor Restart"""

    name = _("Restart hypervisor")

    action_name = _('Restart')

    def handle(self, request, data):
        hard_reboot = request.POST.get('power_off', 'hard') == 'hard'
        in_sequence = request.POST.get('in_sequence', 'off') == 'on'
        allow_evacuate = (
            request.POST.get('allow_evacuate', 'off') == 'on')
        allow_off_vm = (
            request.POST.get('allow_off_vm', 'off') == 'on')
        hypervisor_ids = [h[0] for h in self.initial['hypervisors']]
        hypervisor_hosts = [h[1] for h in self.initial['hypervisors']]
        failed_hyps = []
        success_hyps = []
        if hard_reboot:
            try:
                _result, body = TNXHypervisorsClient.reset_all(
                    request, hypervisor_ids)
            except Exception as err:
                failed_ids = hypervisor_ids
                logger.error(err.message)
            else:
                failed_ids = body.get('reset', {}).get('failed_ids', [])

            for hypervisor in self.initial['hypervisors']:
                if hypervisor[0] in failed_ids:
                    failed_hyps.append(hypervisor[1])
                else:
                    success_hyps.append(hypervisor[1])

        else:
            TNXHostsClient.reset_hosts(
                request,
                hypervisor_hosts,
                allow_off_vm=allow_off_vm,
                allow_evacuate=allow_evacuate,
                in_sequence=in_sequence)
            success_hyps.extend(hypervisor_hosts)

        if failed_hyps:
            msg = _('Failed to perform "{0}" for hypervisors: {1}')
            messages.error(
                request, msg.format(self.action_name,
                                    '"' + u'", "'.join(failed_hyps) +
                                    '"'))

        if success_hyps:
            if hard_reboot:
                msg = self.sync_message
            else:
                msg = self.async_message
            messages.success(
                request, msg.format(self.action_name,
                                    '"' + u'", "'.join(success_hyps) +
                                    '"'))
        return True


# noinspection PyAbstractClass
class HypervisorMarkReserved(HypervisorShutdown):
    """Hypervisor mark reserved"""

    name = _("Mark reserved")

    action_name = _('Move to reserve')

    def do_action(self, request, data, _id):
        # shutdown node
        super(HypervisorMarkReserved, self).do_action(request, data, _id)

        # mark reserved
        TNXHypervisorsClient.set_reserved_flag_hypervisor(request, _id, True)


class HypervisorUpdateForm(forms.SelfHandlingForm):
    """Update form"""

    tnx_dtc_number = forms.IntegerField(
        label=_('Data Center number'),
        required=False,
        min_value=1,
        max_value=99,
        help_text=_('Input data center number. Must be in the range '
                    'from 1 to 99.'))
    tnx_row_number = forms.IntegerField(
        label=_('Row number'),
        required=False,
        min_value=1,
        max_value=99,
        help_text=_('Input row number. Must be in the range from 1 to 99.'))
    tnx_rack_number = forms.IntegerField(
        label=_('Rack number'),
        required=False,
        min_value=1,
        max_value=99,
        help_text=_('Input rack number. Must be in the range from 1 to 99.'))
    tnx_unit_number = forms.IntegerField(
        label=_('Place in rack'),
        required=False,
        min_value=1,
        max_value=99,
        help_text=_('Input place number in rack. Must be in the range '
                    'from 1 to 99.'))
    tnx_inventory_code = NullableCharField(
        label=_('Inventory number'),
        required=False,
        max_length=128,
        help_text=_('Input inventory number of hypervisor. Inventory number '
                    'can contain letters and numbers, but no more than 128 '
                    'characters.'))
    tnx_is_reserved = forms.BooleanField(
        label=_('Reserved'),
        required=False,
        help_text=_('Flag hypervisor as a reserved / Unflag. Hypervisor is '
                    'reserved if it flagged as reserved and shut down.'))

    def __init__(self, request, *args, **kwargs):
        reserved_field = self.declared_fields.get('tnx_is_reserved')
        hypervisors = kwargs.get('initial').get('hypervisors')
        disable_reserved_field = False

        if len(hypervisors) == 1:
            hypervisor = hypervisors[0]
            if (hypervisor.tnx_power_state is None and
                    not hypervisor.tnx_is_reserved):
                disable_reserved_field = True

        if disable_reserved_field:
            reserved_field.widget.attrs.update({'disabled': ''})
        else:
            # undo disable
            reserved_field.widget.attrs.pop('disabled', None)

        super(HypervisorUpdateForm, self).__init__(request, *args, **kwargs)

    def handle(self, request, tnx_data):
        success_hyps, failed_hyps = [], []

        is_reserved = tnx_data.pop('tnx_is_reserved', None)

        hypervisors_count = len(self.initial['hypervisors'])
        data = {field.replace('tnx_', ''): value
                for field, value in tnx_data.items()
                if field not in ('id',) and (hypervisors_count == 1 or value)}

        for hypervisor in self.initial['hypervisors']:
            name = getattr(hypervisor, hypervisor.NAME_ATTR, '')
            if (is_reserved and not hypervisor.tnx_is_reserved and
                    hypervisor.tnx_power_state is None):
                failed_hyps.append(name)
                msg = _('Power management is not available '
                        'for hypervisor "{0}"').format(name)
                messages.error(request, msg)
                continue
            try:
                TNXHypervisorsClient.update(self.request,
                                            hypervisor.id,
                                            **data)
                TNXHypervisorsClient.set_reserved_flag_hypervisor(
                    self.request, hypervisor.id, is_reserved)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(err.message)
                failed_hyps.append(name)
            else:
                success_hyps.append(name)

        if failed_hyps:
            msg = _('Failed to update hypervisors: "{0}"')
            messages.error(request, msg.format(u'", "'.join(failed_hyps)))

        if success_hyps:
            msg = _('Successfully updated hypervisors: "{0}"')
            messages.success(request, msg.format(u'", "'.join(success_hyps)))

        return True
