# coding: utf-8

import logging

from django.forms import widgets
from django.forms.fields import BooleanField, ChoiceField
from django.template import defaultfilters as filters
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import tables
from horizon.templatetags import sizeformat

from openstack_dashboard.dashboards.admin.hypervisors import (
    tables as admin_tables
)
from tionix_dashboard.clients import TNXPowerDevicesClient

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.pxe_images import TNXPXEImagesClient
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    HypervisorSchedulerAction)
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import (
    TnxPaginationDataTable, ColoredRowMixin)
from tionix_dashboard.dashboards.admin.hypervisors import actions
from tionix_dashboard.helpers.views import RefererRedirectMixin

logger = logging.getLogger(__name__)


POWER_STATE_DISPLAY_CHOICES = (
    ('None', '-'),
    ('power_on', _('On')),
    ('power_off', _('Off'))
)

STATE_DISPLAY_CHOICES = (
    ('up', _('Up')),
    ('down', _('Down'))
)

STATUS_DISPLAY_CHOICES = (
    ('enabled', _('enabled')),
    ('disabled', _('disabled'))
)


def is_hyp_initialized(hypervisor):
    for device in hypervisor.devices:
        if device.is_default:
            return device.is_initialized
    return False


class TnxAdminHypervisorsTable(ColoredRowMixin,
                               TnxPaginationDataTable):
    """
    Hypervisors table extension.
    """

    POWER_STATE_CHOICES = (
        (None, None),
        ('power_on', True),
        ('power_off', False)
    )

    hostname = tables.Column(
        "hypervisor_hostname",
        link="horizon:admin:hypervisors:detail",
        attrs={'data-type': 'naturalSort'},
        verbose_name=_("Hostname"))

    node_title = tables.Column(
        "tnx_node_title",
        form_field=EmptyCharField(max_length=100),
        verbose_name=pgettext_lazy("Hypervisor", "Host title"),
        update_action=actions.TnxNodeUpdateCell)

    hypervisor_type = tables.Column(
        "hypervisor_type",
        verbose_name=_("Type"))

    image = tables.Column(
        "tnx_image_name",
        verbose_name=_("Image"),
        form_field=ChoiceField(),
        update_action=actions.TnxImageUpdateCell)

    state = tables.Column(
        "state",
        verbose_name=_('State'),
        classes=('hypervisor_state', ),
        display_choices=STATE_DISPLAY_CHOICES,
    )

    power_state = tables.Column(
        lambda x: unicode(x.tnx_power_state),
        verbose_name=_('Power state'),
        status_choices=POWER_STATE_CHOICES,
        display_choices=POWER_STATE_DISPLAY_CHOICES,
        classes=('hypervisor_power_state', ))

    is_reserved = tables.Column(
        "tnx_is_reserved",
        verbose_name=_('Reserved'),
        empty_value=False,
        status=True,
        filters=(filters.yesno, filters.capfirst),
        form_field=BooleanField(label=_('Reserved'), required=False),
    )

    running_vms = tables.Column(
        "running_vms",
        verbose_name=pgettext_lazy('Hypervisor', 'Instances')
    )

    is_initialized = tables.Column(
        is_hyp_initialized,
        verbose_name='',
        status=True,
        hidden=True
    )

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(TnxAdminHypervisorsTable, self).__init__(
            request, data, needs_form_wrapper, **kwargs)

        self._populate_images(request)

    def _populate_images(self, request):
        """
        populate images for assigned image column
        """
        try:
            self.columns['image'].form_field.choices = [
                (image.id, image.image_name)
                for image in TNXPXEImagesClient.list(request)]
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.debug(err)
            self.columns['image'].form_field.choices = [('default', 'default')]

    def get_object_display(self, datum):
        return getattr(datum, datum.NAME_ATTR)

    @property
    def needs_form_wrapper(self):
        """
        csrf-token into template and so on in request.Meta
        """
        return True

    def calculate_row_status(self, statuses):
        return statuses

    def get_row_status_class(self, status):
        _statuses = []
        if status.get('is_initialized', False) is True:
            if all((status.get('state', None) is False,
                    status.get('power_state', None) is True)):
                _statuses.append('red_color')
            else:
                if status.get('state', None) is False:
                    _statuses.append('tnx_state_cell_orange')
                else:
                    _statuses.append('tnx_state_cell_green')

                if status.get('power_state', None) is False:
                    _statuses.append('tnx_power_state_cell_orange')
                else:
                    _statuses.append('tnx_power_state_cell_green')

        return u' '.join(_statuses)

    class Meta(object):
        name = 'tnx_hypervisors'
        verbose_name = _('Hypervisors')
        row_class = actions.TnxHypervisorUpdateRow
        multi_select = True
        status_columns = ['power_state', 'state', 'is_initialized']

        table_actions = (actions.TnxAdminHypervisorsFilterAction,)

        table_actions_menu = (
            actions.PowerOnHypervisorAction,
            actions.HypervisorShutdownAction,
            actions.PowerOffHypervisorAction,
            actions.RestartHypervisorAction,
            actions.StartReservationHypervisorAction,
            actions.HypervisorAssignImageAction,
            HypervisorSchedulerAction,
            actions.HypervisorUpdateAction)

        row_actions = (
            actions.HypervisorExtraAction,
            actions.HypervisorShutdownAction,
            HypervisorSchedulerAction,
            actions.RestartHypervisorAction,
            actions.PowerOnHypervisorAction,
            actions.PowerOffHypervisorAction,
            actions.StartReservationHypervisorAction,
            actions.HypervisorUpdateAction,
            actions.AssignPowerDeviceAction)


# noinspection PyClassHasNoInit
class AdminHypervisorsTable(admin_tables.AdminHypervisorsTable):
    memory_used = tables.Column('memory_mb_used',
                                verbose_name=_("RAM (used)"),
                                attrs={'data-type': 'size'},
                                filters=(sizeformat.mb_float_format,))

    memory = tables.Column('memory_mb',
                           verbose_name=_("RAM (total)"),
                           attrs={'data-type': 'size'},
                           filters=(sizeformat.mb_float_format,))

    class Meta(object):
        name = "hypervisors"
        verbose_name = _("Hypervisors")
        multi_select = True
        table_actions_menu = (HypervisorSchedulerAction,)

        row_actions = (HypervisorSchedulerAction,)


def is_default_device(device):
    return widgets.RadioSelect(choices=[(device.id, '')]).render(
        'radio_default_pwd', None, attrs={'checked': device.is_default}
    )


class ConnectedDevicesTable(ColoredRowMixin,
                            RefererRedirectMixin,
                            tables.DataTable):

    STATUS_CLASSES = {
        True: 'green_color'
    }

    name = tables.Column("device_name", verbose_name=_("Control title"))
    number = tables.Column(
        "device_port",
        verbose_name=_("Coil number"),
        form_field=ChoiceField(),
        update_action=actions.SetPowerDevicePortUpdateCell
    )
    is_default = tables.Column(
        is_default_device,
        verbose_name=_("Default")
    )
    is_initialized = tables.Column(
        'is_initialized',
        verbose_name=_('Initialized'),
        hidden=True
    )

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(ConnectedDevicesTable, self).__init__(
            request, data=data, needs_form_wrapper=True, **kwargs
        )

    @cached_property
    def devices_coil_map(self):

        try:
            power_devices = TNXPowerDevicesClient.list(self.request)
        except Exception as err:
            logger.error(unicode(err))
            power_devices = []

        return {power_device.id: power_device.coil_map
                for power_device in power_devices}

    class Meta(object):
        name = "connected_devices"
        verbose_name = _("Connected controls")
        row_class = actions.TnxConnectedDeviceUpdateRow
        status_columns = ['is_initialized']
