# coding: utf-8

import logging
import operator

from django.core.context_processors import csrf
from django.utils.translation import ugettext_lazy as _

from horizon import tabs, exceptions

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.hypervisors import (
    tabs as admin_tabs
)
from openstack_dashboard.dashboards.admin.hypervisors import (
    tables as admin_tables)
from openstack_dashboard.dashboards.admin.hypervisors.compute import (
    tabs as admin_cmp_tabs
)

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients import TNXHypervisorsClient, TNXServersClient
from tionix_dashboard.dashboards.admin.hypervisors import tables
from tionix_dashboard.dashboards.admin.hypervisors.compute import (
    tabs as cmp_tabs)
from tionix_dashboard.dashboards.admin.hypervisors.images import (
    tabs as images_tabs)
from tionix_dashboard.dashboards.admin.hypervisors.resources import (
    tabs as resources_tabs)
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    ConnectedDevicesTable)
from tionix_dashboard.dashboards.admin.instances import (
    tables as instances_tables)
from tionix_dashboard.dashboards.admin.instances.helpers import (
    get_instances_for_admin)
from tionix_dashboard.dashboards.tionix.metrics.tabs import (
    HypervisorMetricsTab)
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.helpers.views import (
    ServerFilterTabMixin, DTVTnxPaginationMixin)
from tionix_dashboard.journal.helpers import HypervisorJournal
from tionix_dashboard.dashboards.admin.hypervisors.audit_tables import (
    AuditTable)

logger = logging.getLogger(__name__)


class HypervisorStates(object):
    UP = u"up"
    DOWN = u"down"


# noinspection PyAbstractClass
class TnxHypervisorTab(ServerFilterTabMixin, DTVTnxPaginationMixin,
                       admin_tabs.HypervisorTab):
    """Tionix Hypervisors Tab."""
    table_classes = (tables.TnxAdminHypervisorsTable, )

    def get_tnx_hypervisors_data(self):
        hypervisors = []
        offset, limit = self.get_offset_limit()
        try:
            self.table.total_count = TNXHypervisorsClient.count(self.request)
            hypervisors = TNXHypervisorsClient.list(
                self.request, offset=offset, limit=limit + 1)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve hypervisor information.'))
        else:
            total = len(hypervisors)
            if offset == 0:
                self._prev = False
            if total <= limit:
                self._more = False
            else:
                hypervisors.pop()

            self.table.first_marker, self.table.last_marker = (
                0, self.calc_last_marker(self.table.total_count, limit)
            )

        return hypervisors


# noinspection PyAbstractClass, PyClassHasNoInit
class TnxHypervisorHostTabs(admin_tabs.HypervisorHostTabs):
    tabs = (TnxHypervisorTab,
            resources_tabs.TnxResourcesTab,
            cmp_tabs.TnxComputeHostTab,
            images_tabs.HypervisorImagesTab)


# noinspection PyAbstractClass
class OverviewTab(tabs.TableTab):
    name = _("Overview")
    slug = "overview"
    template_name = "hypervisors/_detail_overview.html"
    table_classes = (ConnectedDevicesTable,)
    preload = False

    def get_context_data(self, request, **kwargs):
        context = super(OverviewTab, self).get_context_data(request, **kwargs)
        hypervisor = self.tab_group.kwargs['hypervisor']

        context.update({
            'hypervisor': hypervisor
        })
        if hypervisor.state == HypervisorStates.DOWN:
            host_name = getattr(hypervisor, hypervisor.NAME_ATTR)
            vms_real_amount = TNXServersClient.count(
                self.request,
                search_opts={'host': host_name, 'all_tenants': True}
            )
            vms_on_host = hypervisor.running_vms
            if vms_real_amount < vms_on_host:
                context['vms_missing'] = vms_on_host - vms_real_amount

        context.update(csrf(request))

        return context

    def get_connected_devices_data(self):
        hypervisor = self.tab_group.kwargs['hypervisor']
        return hypervisor.devices


# noinspection PyAbstractClass
class AuditTab(tabs.TableTab):
    name = _("Action Log")
    slug = "audit"
    table_classes = (AuditTable,)
    template_name = "hypervisors/_detail_audit.html"
    preload = False

    def get_audit_data(self):
        hypervisor = self.tab_group.kwargs['hypervisor']
        journal = []

        try:
            journal = HypervisorJournal(self.request, hypervisor).list()
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request, _('Unable to retrieve hypervisor journal.'))
        else:
            journal.sort(key=operator.attrgetter('event_time'), reverse=True)

        return journal


class SLATab(tabs.Tab):
    name = _("SLA")
    slug = "sla"
    template_name = "hypervisors/_detail_sla.html"


# noinspection PyAbstractClass
class TnxInstancesTab(ServerFilterTabMixin,
                      DTVTnxPaginationMixin,
                      tabs.TableTab):
    table_classes = (instances_tables.AdminHypervisorInstancesTable,)
    name = _("Instances list")
    slug = "tnx_servers"
    template_name = "hypervisors/_instances_tab.html"
    preload = False

    @property
    def table(self):
        return self._tables['hypervisor_instances']

    # noinspection PyProtectedMember
    # pylint: disable=protected-access
    def get_hypervisor_instances_data(self):
        total_count = 0
        offset, limit = self.get_offset_limit()
        search_opts = self.get_filters({'sort_dirs': ('asc',),
                                        'sort_keys': ('created_at',)})
        hypervisor = self.tab_group.kwargs['hypervisor']
        instances = []
        if hypervisor:
            search_opts.update({
                'host': getattr(hypervisor, hypervisor.NAME_ATTR)
            })

            instances = get_instances_for_admin(
                self.request,
                search_opts.get('sort_dir'),
                search_opts
            )

            total_count = len(instances)
            range_end = offset + limit
            instances = instances[offset:range_end]

        self.set_table_footer(offset, limit, total_count)

        return instances

    def allowed(self, request):
        return LicenseController.is_valid(
            request, LicenseController.CLOUD_CONTROL)

    def get_context_data(self, request, **kwargs):
        context = super(TnxInstancesTab, self).get_context_data(
            request, **kwargs
        )
        context['disable_actions_menu'] = len(context['table'].data) == 0

        return context


# noinspection PyAbstractClass
class InstancesTab(tabs.TableTab):
    table_classes = (admin_tables.AdminHypervisorInstancesTable,)
    name = _("Instances list")
    slug = "servers"
    template_name = "horizon/common/_detail_table.html"

    def allowed(self, request):
        return not LicenseController.is_valid(
            request, LicenseController.CLOUD_CONTROL)

    def get_hypervisor_instances_data(self):
        instances = []
        try:
            hypervisor = self.tab_group.kwargs['hypervisor']
            result = api.nova.hypervisor_search(
                self.request,
                getattr(hypervisor, hypervisor.NAME_ATTR, '')
            )
            for hyp in result:
                if hyp.id == hypervisor.id:
                    try:
                        instances += hyp.servers
                    except AttributeError:
                        pass
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve hypervisor instances list.'))
        return instances


# noinspection PyAbstractClass
class TnxHypervisorDetailTabs(tabs.TabGroup):
    slug = "hypervisor_details"
    tabs = (OverviewTab,
            TnxInstancesTab,
            InstancesTab,
            AuditTab,
            ActiveTasksTab,
            HypervisorMetricsTab,
            SLATab)
    sticky = True
    obj_type = ObjectTypeEnum.hypervisor
    obj_group = ObjectGroupEnum.admin


# noinspection PyAbstractClass
class HypervisorDetailTabs(tabs.TabGroup):
    slug = "hypervisor_details"
    tabs = (InstancesTab,
            TnxInstancesTab,
            HypervisorMetricsTab)
    sticky = True
    obj_type = ObjectTypeEnum.hypervisor
    obj_group = ObjectGroupEnum.admin


# noinspection PyAbstractClass
class HypervisorTab(admin_tabs.HypervisorTab):
    table_classes = (tables.AdminHypervisorsTable,)


# noinspection PyAbstractClass
class HypervisorHostTabs(admin_tabs.HypervisorHostTabs):
    tabs = (HypervisorTab, admin_cmp_tabs.ComputeHostTab)
