# coding: utf-8

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.dashboards.admin.hypervisors import actions
from tionix_dashboard.dashboards.admin.hypervisors.actions import (
    TnxImageUpdateCell, TnxNodeUpdateCell, TnxSetReserveAction,
    RestartHypervisorAction, PowerOnHypervisorAction,
    PowerOffHypervisorAction, StartReservationHypervisorAction,
    HypervisorShutdownAction)
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    TnxAdminHypervisorsTable)
from tionix_dashboard.tests.helpers import BaseClientTestCase, FakeUser


class AdminHypervisorsTestCase(BaseClientTestCase):

    def test_update_row(self):

        for hypervisor in self.tnx_hypervisors.list():
            update_row_action = actions.TnxHypervisorUpdateRow(
                table=TnxAdminHypervisorsTable,
                datum=None)
            _hypervisor = update_row_action.get_data(
                request=None,
                obj_id=hypervisor.id)

            self.assertEqual(hypervisor.id, _hypervisor.id)

            self.assertRaises(
                ObjectDoesNotExist,
                update_row_action.get_data,
                request=None,
                obj_id=1000000)

    def test_assign_image(self):

        for hypervisor in self.tnx_hypervisors.list():
            hypervisor_images = self.tnx_hypervisors.get_images()
            hypervisor_image = 'default'

            while hypervisor_image not in hypervisor_images:
                hypervisor_image += '1'

            update_image_cell_action = TnxImageUpdateCell()

            request = get_fake_request(
                user=FakeUser(),
                is_ajax=True,
                horizon={
                    'async_messages': []
                }
            )

            self.assertTrue(
                update_image_cell_action.allowed(
                    request=request,
                    datum=None,
                    cell=None))

            update_image_cell_action.update_cell(
                request=request,
                datum=None,
                obj_id=hypervisor.id,
                cell_name='image',
                new_cell_value=hypervisor_image)

    def test_update_cell(self):

        for hypervisor in self.tnx_hypervisors.list():
            update_cell_action = TnxNodeUpdateCell()

            update_cell_action.update_cell(
                request=None,
                datum=None,
                obj_id=hypervisor.id,
                cell_name='Node Title',
                new_cell_value='node_title')

    def test_set_reserve_action(self):
        for hypervisor in self.tnx_hypervisors.list():
            set_reserve_action = TnxSetReserveAction()

            self.assertTrue(
                set_reserve_action.allowed(
                    request=None,
                    datum=None,
                    cell=None))

            set_reserve_action.update_cell(
                request=None,
                datum=None,
                obj_id=hypervisor.id,
                cell_name='is_reserved',
                new_cell_value=True)

    def test_reset_action(self):
        reset_action = RestartHypervisorAction()

        hypervisor1 = self.tnx_hypervisors.get(1)

        self.assertTrue(
            reset_action.allowed(
                request=None,
                datum=hypervisor1))

        hypervisor2 = self.tnx_hypervisors.get(2)

        self.assertTrue(
            reset_action.allowed(
                request=None,
                datum=hypervisor2))

    def test_reset_action_no_tnx(self):
        reset_action = RestartHypervisorAction()

        hypervisor1 = self.tnx_hypervisors.get(5)

        self.assertFalse(
            reset_action.allowed(
                request=None,
                datum=hypervisor1))

        hypervisor2 = self.tnx_hypervisors.get(6)

        self.assertFalse(
            reset_action.allowed(
                request=None,
                datum=hypervisor2))

    def test_power_on_action(self):
        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)

        power_on_action = PowerOnHypervisorAction()

        self.assertEqual(power_on_action.action_present(1), _(u'Power on'))
        self.assertEqual(power_on_action.action_past(1), _(u'Power on'))
        self.assertFalse(
            power_on_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertFalse(
            power_on_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertTrue(
            power_on_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertTrue(
            power_on_action.allowed(
                request=None,
                datum=hypervisor4))

        power_on_action.table = Empty(get_object_by_id=lambda x: hypervisor1)
        self.assertRaises(
            exceptions.NotAvailable,
            power_on_action.action,
            request=None, datum_id=hypervisor1.id
        )

    def test_power_on_action_no_tnx(self):
        hypervisor1 = self.tnx_hypervisors.get(5)
        hypervisor2 = self.tnx_hypervisors.get(6)
        hypervisor3 = self.tnx_hypervisors.get(7)
        hypervisor4 = self.tnx_hypervisors.get(8)

        power_on_action = PowerOnHypervisorAction()

        self.assertEqual(power_on_action.action_present(1), _(u'Power on'))
        self.assertEqual(power_on_action.action_past(1), _(u'Power on'))
        self.assertFalse(
            power_on_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertFalse(
            power_on_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            power_on_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            power_on_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_power_off_action(self):
        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)

        power_off_action = PowerOffHypervisorAction()

        self.assertTrue(
            power_off_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertTrue(
            power_off_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            power_off_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            power_off_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_power_off_action_no_tnx(self):
        hypervisor1 = self.tnx_hypervisors.get(5)
        hypervisor2 = self.tnx_hypervisors.get(6)
        hypervisor3 = self.tnx_hypervisors.get(7)
        hypervisor4 = self.tnx_hypervisors.get(8)

        power_off_action = PowerOffHypervisorAction()

        self.assertFalse(
            power_off_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertFalse(
            power_off_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            power_off_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            power_off_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_reservation_action(self):
        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)

        reservation_action = StartReservationHypervisorAction()

        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertTrue(
            reservation_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_reservation_action_no_tnx(self):
        hypervisor1 = self.tnx_hypervisors.get(5)
        hypervisor2 = self.tnx_hypervisors.get(6)
        hypervisor3 = self.tnx_hypervisors.get(7)
        hypervisor4 = self.tnx_hypervisors.get(8)

        reservation_action = StartReservationHypervisorAction()

        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_shutdown_action(self):
        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)

        reservation_action = HypervisorShutdownAction()

        self.assertTrue(
            reservation_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertTrue(
            reservation_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_shutdown_action_no_tnx(self):
        hypervisor1 = self.tnx_hypervisors.get(5)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)

        reservation_action = HypervisorShutdownAction()

        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor1))
        self.assertTrue(
            reservation_action.allowed(
                request=None,
                datum=hypervisor2))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor3))
        self.assertFalse(
            reservation_action.allowed(
                request=None,
                datum=hypervisor4))

    def test_power_device_port_update_allowed(self):
        hypervisor = self.tnx_hypervisors.get(1)

        update_action = actions.SetPowerDevicePortUpdateCell()

        devices_coil_map = {
            hypervisor.id: {'1': hypervisor.hypervisor_hostname,
                            '2': hypervisor.hypervisor_hostname}
        }
        cell = Empty(
            row=Empty(
                table=Empty(devices_coil_map=devices_coil_map)
            ),
            column=Empty(form_field=Empty())
        )

        self.assertTrue(update_action.allowed(self.request, hypervisor, cell))

    def test_power_device_port_update_not_allowed(self):
        hypervisor = self.tnx_hypervisors.get(1)

        update_action = actions.SetPowerDevicePortUpdateCell()

        devices_coil_map = {
            hypervisor.id: {'1': hypervisor.hypervisor_hostname}
        }
        cell = Empty(
            row=Empty(
                table=Empty(devices_coil_map=devices_coil_map)
            ),
            column=Empty(form_field=Empty())
        )

        self.assertFalse(update_action.allowed(self.request, hypervisor, cell))
