# coding: utf-8

import unittest

import mock

from openstack_dashboard.test import helpers as test

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.dashboards.admin.hypervisors.forms import (
    HypervisorExtra, HypervisorWithInstancesForm, HypervisorShutdown,
    HypervisorPowerOff, HypervisorRestart, HypervisorMarkReserved,
    HypervisorUpdateForm)
from tionix_dashboard.tests.helpers import BaseClientTestCase
from tionix_dashboard.tests.fake_cloud_state import cloud_state


class ExtraFormTestCase(test.TestCase):

    def test(self):

        form = HypervisorExtra(request=None)
        form.initial = {
            'id': 5
        }
        form.init_values()
        self.assertListEqual(
            sorted(form.get_fields()),
            sorted(['id', 'human_id', 'host_ip', 'free_ram_mb', 'free_disk_gb',
                    'disk_available_least', 'hypervisor_hostname',
                    'hypervisor_type', 'hypervisor_version', 'memory_mb',
                    'memory_mb_used', 'running_vms', 'vcpus', 'vcpus_used',
                    'tnx_dtc_number', 'tnx_rack_number', 'tnx_unit_number',
                    'node_name', 'node_port', 'node_mac_address',
                    'tnx_inventory_code', 'tnx_image_name',
                    'current_workload', 'tnx_row_number'])
        )


class HypervisorWithInstancesFormTestCase(test.TestCase):

    def test(self):

        for hypervisor in cloud_state['hypervisors']:
            form = HypervisorWithInstancesForm(request=None)
            form.initial = {
                'hypervisors': [(str(hypervisor['id']),
                                 hypervisor['hypervisor_hostname'])]
            }
            self.assertTrue(
                form.handle(
                    request=get_fake_request(
                        is_ajax=True,
                        horizon={
                            'async_messages': []
                        }),
                    data=None))


class HypervisorShutdownTestCase(test.TestCase):

    def test(self):

        form = HypervisorShutdown(request=None)
        form.initial = {
            'hypervisors': [('1', 'hypervisor1')]
        }
        self.assertTrue(
            form.handle(
                request=get_fake_request(
                    is_ajax=True,
                    horizon={
                        'async_messages': []
                    }),
                data={
                    'allow_off_vm': True,
                    'allow_evacuate': True}))

    def test_no_tnx(self):

        form = HypervisorShutdown(request=None)
        form.initial = {
            'hypervisors': [('5', 'hypervisor5')]
        }
        self.assertTrue(
            form.handle(
                request=get_fake_request(
                    is_ajax=True,
                    horizon={
                        'async_messages': []
                    }),
                data={
                    'allow_off_vm': True,
                    'allow_evacuate': True}))


class HypervisorPowerOffTestCase(unittest.TestCase):

    def test(self):

        form = HypervisorPowerOff(request=None)
        form.initial = {
            'hypervisors': [('1', 'hypervisor1')]
        }
        self.assertTrue(
            form.handle(
                request=get_fake_request(
                    is_ajax=True,
                    horizon={
                        'async_messages': []
                    }),
                data={
                    'allow_off_vm': True,
                    'allow_evacuate': True}))


class HypervisorRestartTestCase(BaseClientTestCase):

    def test(self):
        hypervisor = self.tnx_hypervisors.get(1)

        form = HypervisorRestart(request=None)
        form.initial = {
            'hypervisors': [(hypervisor.id, hypervisor.hypervisor_hostname)]
        }
        self.assertTrue(
            form.handle(
                request=get_fake_request(
                    is_ajax=True,
                    horizon={
                        'async_messages': []
                    }),
                data={
                    'allow_off_vm': True,
                    'allow_evacuate': True}))


class HypervisorMarkReservedTestCase(BaseClientTestCase):

    def test(self):
        hypervisor = self.tnx_hypervisors.get(1)

        form = HypervisorMarkReserved(request=None)
        form.initial = {
            'hypervisors': [(hypervisor.id, hypervisor.hypervisor_hostname)]
        }
        self.assertTrue(
            form.handle(
                request=get_fake_request(
                    is_ajax=True,
                    horizon={
                        'async_messages': []
                    }),
                data={
                    'allow_off_vm': True,
                    'allow_evacuate': True}))


class HypervisorUpdateFormTestCase(BaseClientTestCase):
    @mock.patch.object(TNXHypervisorsClient, 'update')
    def test(self, update):
        hypervisor = self.tnx_hypervisors.get(1)
        unit_number = None
        rack_number = 10

        request = get_fake_request(
            is_ajax=True,
            horizon={'async_messages': []}
        )
        initial = {'hypervisors': [hypervisor]}
        form = HypervisorUpdateForm(request=request, initial=initial)

        form.handle(request, {'tnx_unit_number': unit_number,
                              'tnx_rack_number': rack_number})

        update.assert_called_with(request,
                                  hypervisor.id,
                                  unit_number=unit_number,
                                  rack_number=rack_number)

    @mock.patch.object(TNXHypervisorsClient, 'update')
    def test_multi(self, update):
        hypervisors = self.tnx_hypervisors.list()[:2]
        unit_number = None
        rack_number = 10

        request = get_fake_request(
            is_ajax=True,
            horizon={'async_messages': []}
        )
        initial = {'hypervisors': hypervisors}
        form = HypervisorUpdateForm(request=request, initial=initial)

        form.handle(request, {'tnx_unit_number': unit_number,
                              'tnx_rack_number': rack_number})

        for hypervisor in hypervisors:
            update.assert_any_call(request,
                                   hypervisor.id,
                                   rack_number=rack_number)
