# coding: utf-8

import mock

from openstack_dashboard import api

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients.hypervisors import TnxHypervisor
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    TnxAdminHypervisorsTable)
from tionix_dashboard.dashboards.admin.hypervisors import tabs
from tionix_dashboard.dashboards.admin.hypervisors.tabs import (
    TnxHypervisorDetailTabs)
from tionix_dashboard.dashboards.admin.instances.tables import (
    AdminHypervisorInstancesTable)
from tionix_dashboard.tests.helpers import BaseAdminClientTestCase, FakeUser
from tionix_dashboard.clients import TNXJournalClient, TNXServersClient
from tionix_dashboard.tests.fake_cloud_state import SERVERS_COUNT


class TnxHypervisorTabTestCase(BaseAdminClientTestCase):

    def test1(self):

        request = get_fake_request(
            user=FakeUser(),
            session={},
            COOKIES={}
        )

        hypervisors = self.tnx_hypervisors.list()

        table = TnxAdminHypervisorsTable(request=request)
        tab = tabs.TnxHypervisorTab(tab_group=table, request=request)

        self.assertEqual(unicode(tab.table), unicode(table.Meta.verbose_name))

        self.assertEqual(len(tab.get_tnx_hypervisors_data()), len(hypervisors))

    def test2(self):
        horizon_pagesize = 3

        request = get_fake_request(
            user=FakeUser(),
            session={
                'horizon_pagesize': horizon_pagesize
            },
            COOKIES={})

        table = TnxAdminHypervisorsTable(request=request)
        tab = tabs.TnxHypervisorTab(tab_group=table, request=request)

        self.assertEqual(len(tab.get_tnx_hypervisors_data()), horizon_pagesize)


class OverviewTabTestCase(BaseAdminClientTestCase):

    def test(self):

        request = get_fake_request(
            user=FakeUser())
        for hypervisor in self.tnx_hypervisors.list():
            table = TnxAdminHypervisorsTable(
                request=request,
                hypervisor=TnxHypervisor(hypervisor))
            tab = tabs.OverviewTab(tab_group=table, request=request)

            result = tab.get_context_data(request)

            self.assertListEqual(sorted(result.keys()),
                                 sorted(['csrf_token', 'hypervisor', 'table',
                                         'connected_devices_table']))
            self.assertEqual(result['hypervisor'].id, hypervisor.id)


@mock.patch.object(TNXServersClient, 'get_metrics_names', mock.Mock())
class TnxInstancesTabTestCase(BaseAdminClientTestCase):

    @mock.patch('tionix_dashboard.dashboards.admin.instances.helpers.'
                'is_cloud_admin', new=mock.Mock(return_value=True))
    @mock.patch.object(api.keystone, 'get_default_domain')
    @mock.patch('tionix_dashboard.dashboards.admin.instances.helpers.'
                'tenant_list')
    def test1(self, tenant_list_mocked, get_default_domain_mocked):
        domain = self.tnx_domains.list()[0]
        tenants = self.tnx_tenants.list()

        get_default_domain_mocked.return_value = domain.to_dict()
        tenant_list_mocked.return_value = (tenants, None)

        hypervisor = self.tnx_hypervisors.list()[0]

        request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            user=FakeUser(
                tenant_id='1'
            ),
            get_params={
                'marker': '0'
            },
            session={},
            COOKIES={}
        )

        table = AdminHypervisorInstancesTable(request=request)
        tab_group = TnxHypervisorDetailTabs(request=request)
        tab_group.kwargs = {'hypervisor': hypervisor}
        tab = tabs.TnxInstancesTab(tab_group=tab_group, request=request)

        self.assertEqual(unicode(tab.table), unicode(table.Meta.verbose_name))

        servers = tab.get_hypervisor_instances_data()
        self.assertIsInstance(servers, list)
        self.assertEqual(len(servers), 9)

    @mock.patch('tionix_dashboard.dashboards.admin.instances.helpers.'
                'is_cloud_admin', new=mock.Mock(return_value=False))
    @mock.patch.object(api.keystone, 'get_default_domain')
    @mock.patch('tionix_dashboard.dashboards.admin.instances.helpers.'
                'tenant_list')
    def test2(self, tenant_list_mocked, get_default_domain_mocked):
        domain = self.tnx_domains.list()[0]
        tenants = self.tnx_tenants.list()

        get_default_domain_mocked.return_value = domain.to_dict()
        tenant_list_mocked.return_value = (tenants, None)

        prev_marker = 1

        request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            session={},
            COOKIES={},
            user=FakeUser(
                tenant_id='1'
            ),
            get_params={
                'prev_marker': str(prev_marker)
            }
        )

        for hypervisor in self.tnx_hypervisors.list():
            table = AdminHypervisorInstancesTable(
                request=request,
                hypervisor=hypervisor)
            tab = tabs.TnxInstancesTab(tab_group=table, request=request)

            self.assertEqual(unicode(tab.table),
                             unicode(table.Meta.verbose_name))

            servers = tab.get_hypervisor_instances_data()
            self.assertIsInstance(servers, list)

            count_by_hyp_domain = (
                SERVERS_COUNT[hypervisor.hypervisor_hostname][domain.id]
            ) or 0

            self.assertEqual(
                len(servers),
                count_by_hyp_domain - prev_marker if count_by_hyp_domain else 0
            )


class InstancesTabTestCase(BaseAdminClientTestCase):

    @mock.patch('tionix_dashboard.dashboards.license.LicenseController.'
                'is_valid', new=mock.Mock(return_value=False))
    def test(self):

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(),
            horizon={
                'async_messages': []
            }
        )
        table = AdminHypervisorInstancesTable(request=request)

        tab = tabs.InstancesTab(tab_group=table, request=request)
        self.assertTrue(tab.allowed(request=request))

        servers = tab.get_hypervisor_instances_data()
        self.assertIsInstance(servers, list)
        self.assertEqual(len(servers), 0)

    @mock.patch.object(api.nova, 'hypervisor_search')
    @mock.patch('tionix_dashboard.dashboards.license.LicenseController.'
                'is_valid', new=mock.Mock(return_value=False))
    def test2(self, hypervisor_search):
        hypervisors = self.hypervisors.list()[:2]  # pylint: disable=no-member

        hypervisor_search.return_value = hypervisors

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(
                username='admin',
                token='admin'
            ),
            horizon={
                'async_messages': []
            }
        )

        table = AdminHypervisorInstancesTable(
            request=request,
            hypervisor=hypervisors[0])

        tab = tabs.InstancesTab(tab_group=table, request=request)
        self.assertTrue(tab.allowed(request=request))

        servers = tab.get_hypervisor_instances_data()
        self.assertIsInstance(servers, list)
        self.assertEqual(len(servers), len(hypervisors[0].servers))


class AuditTabTestCase(BaseAdminClientTestCase):

    request = None
    test_node_actions = None

    def setUp(self):
        super(AuditTabTestCase, self).setUp()

        self.request = get_fake_request(user=FakeUser())
        self.test_node_actions = [
            Empty(id=1,
                  action="power-off",
                  event_time="2016-05-11T10:24:25.000000",
                  user_name=None),
            Empty(id=2,
                  action="power-on",
                  event_time="2016-05-05T12:08:26.000000",
                  user_name=None)
        ]

    @mock.patch.object(TNXJournalClient, 'list')
    def test(self, journal_list):
        journal_list.return_value = self.test_node_actions

        for hypervisor in self.tnx_hypervisors.list():
            table = TnxAdminHypervisorsTable(
                request=self.request,
                hypervisor=hypervisor)
            tab = tabs.AuditTab(tab_group=table, request=self.request)

            result = tab.get_audit_data()

            self.assertEqual(len(result),
                             len(self.test_node_actions))


class TnxAdminTabsTestCase(BaseAdminClientTestCase):
    def test_tabs(self):

        tab_group = tabs.TnxHypervisorHostTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['compute_host', 'hypervisor', 'images_tab', 'resources'])
        )
