# coding: utf-8

import mock

from django.http import HttpResponse

from horizon.exceptions import Http302

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.clients.hypervisors import TnxHypervisor
from tionix_dashboard.dashboards.admin.hypervisors import views
from tionix_dashboard.dashboards.admin.hypervisors.views import (
    HypervisorExtraView, HypervisorShutdownView, HypervisorsViewMixin,
    HypervisorMarkReservedView, mark_device_default, HypervisorDetailView,
    AdminDetailView, HypervisorMultiAssignImageView, HypervisorUpdateView)
from tionix_dashboard.tests.helpers import BaseClientTestCase, FakeUser


class TestTenantsAtt(object):
    name = 'Tenant 1'


class HypervisorExtraViewTestCase(BaseClientTestCase):

    def test_get_object1(self):
        view = HypervisorExtraView()
        view.request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        view.kwargs = {
            'hypervisor_id': None
        }
        self.assertIsNone(view.get_object())

    def test_get_object2(self):
        for hypervisor in self.tnx_hypervisors.list():
            view = HypervisorExtraView()
            view.request = get_fake_request(
                is_ajax=True,
                horizon={
                    'async_messages': []
                }
            )

            view.kwargs = {
                'hypervisor_id': str(hypervisor.id)
            }
            # pylint: disable=no-member
            self.assertEqual(
                view.get_object().id, hypervisor.id)

    def test_get_initial(self):
        hypervisor = self.tnx_hypervisors.get(1)

        view = HypervisorExtraView()
        view.request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        view.kwargs = {
            'hypervisor_id': str(hypervisor.id)
        }
        initial = view.get_initial()

        self.assertDictEqual(
            initial,
            {'host_ip': None,
             'tnx_unit_number': None,
             'hypervisor_hostname': 'Host1',
             'free_disk_gb': None,
             'hypervisor_version': None,
             'disk_available_least': None,
             'free_ram_mb': 0,
             'id': 1,
             'tnx_row_number': None,
             'tnx_image_name': 'image name',
             'vcpus_used': None,
             'memory_mb_used': 4608,
             'hypervisor_type': None,
             'tnx_dtc_number': None,
             'tnx_inventory_code': None,
             'memory_mb': None,
             'current_workload': None,
             'vcpus': None,
             'running_vms': 0,
             'tnx_rack_number': None})


class HypervisorShutdownViewTestCase(BaseClientTestCase):

    def test_get_context_data(self):
        view = HypervisorShutdownView()
        view.request = get_fake_request(
            get_params={
                'id': '1'
            },
            META={}
        )

        result = view.get_context_data()
        _result_keys = [
            'cancel_label', 'form_id', 'modal_backdrop', 'submit_url',
            'cancel_url', 'submit_label', 'view', 'modal_id', 'modal_header']
        self.assertEqual(len(result.keys()), len(_result_keys))

    def test_format_hypervisor(self):
        view = HypervisorShutdownView()

        hypervisor = self.tnx_hypervisors.get(1)
        result = view.format_hypervisor(hypervisor)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], hypervisor.id)
        self.assertEqual(result[1], hypervisor.hypervisor_hostname)

    def test_is_allowed(self):
        view = HypervisorShutdownView()
        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)

        self.assertTrue(view.is_allowed(hypervisor1))
        self.assertTrue(view.is_allowed(hypervisor2))
        self.assertFalse(view.is_allowed(hypervisor3))
        self.assertFalse(view.is_allowed(hypervisor4))

    def test_get_initial(self):
        view = HypervisorShutdownView()
        view.request = get_fake_request(
            get_params={
                'id': '2'
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        _initial = view.get_initial()
        self.assertListEqual(_initial['hypervisors'], [(2, 'Host2')])
        self.assertTrue(_initial['rows'])

    def test_get_initial2(self):
        view = HypervisorShutdownView()
        view.request = get_fake_request(
            get_params={
                'id': ''
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        self.assertDictEqual(
            view.get_initial(),
            {
                'hypervisors': [],
                'rows': []
            })


@mock.patch.object(views.HypervisorRestartView,
                   'tenants',
                   mock.Mock(return_value={'1': TestTenantsAtt()}))
class HypervisorRestartViewTestCase(BaseClientTestCase):

    def test_get_context_data(self):
        hypervisors = {'1': 'Host1', '2': 'Host2'}

        view = views.HypervisorRestartView()

        view.request = get_fake_request(
            get_params={
                'id': hypervisors.keys()
            },
            META={}
        )

        class FormInitial(object):
            initial = {
                'hypervisors': [(key, value) for key, value
                                in hypervisors.items()]
            }

        kwargs = {
            'form': FormInitial()
        }

        result = view.get_context_data(**kwargs)
        _result_keys = [
            'cancel_label', 'form_id', 'modal_backdrop', 'submit_url',
            'cancel_url', 'submit_label', 'view', 'modal_id', 'modal_header',
            'form', 'hypervisors', 'hypervisors_count', 'servers_count']
        self.assertEqual(len(result.keys()), len(_result_keys))


class HypervisorsViewMixinTestCase(BaseClientTestCase):

    def test_format_hypervisor(self):
        view = HypervisorsViewMixin()
        hypervisor = self.tnx_hypervisors.get(1)
        self.assertEqual(view.format_hypervisor(hypervisor), hypervisor)

    def test_is_allowed(self):
        view = HypervisorsViewMixin()
        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)
        self.assertTrue(view.is_allowed(hypervisor1))
        self.assertTrue(view.is_allowed(hypervisor2))
        self.assertTrue(view.is_allowed(hypervisor3))
        self.assertTrue(view.is_allowed(hypervisor4))


class HypervisorMarkReservedViewTestCase(BaseClientTestCase):

    def test_is_allowed(self):
        view = HypervisorMarkReservedView()

        hypervisor1 = self.tnx_hypervisors.get(1)
        hypervisor2 = self.tnx_hypervisors.get(2)
        hypervisor3 = self.tnx_hypervisors.get(3)
        hypervisor4 = self.tnx_hypervisors.get(4)
        self.assertFalse(view.is_allowed(hypervisor1))
        self.assertTrue(view.is_allowed(hypervisor2))
        self.assertFalse(view.is_allowed(hypervisor3))
        self.assertFalse(view.is_allowed(hypervisor4))


class MarkDeviceDefaultTestCase(BaseClientTestCase):

    def test_mark_device_default(self):
        hypervisor = self.tnx_hypervisors.get(1)
        pwd = self.power_devices.get(1)

        result = mark_device_default(
            request=get_fake_request(
                post_params={
                    'node_id': hypervisor.id,
                    'power_device_id': pwd.id
                }
            ))
        self.assertIsInstance(result, HttpResponse)
        self.assertEqual(result.status_code, 200)


class HypervisorDetailViewTestCase(BaseClientTestCase):

    def test_get_context_data(self):
        hypervisor = self.tnx_hypervisors.get(1)

        view = HypervisorDetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        kwargs = {
            'hypervisor_id': hypervisor.id
        }
        view.kwargs = kwargs

        result = view.get_context_data(**kwargs)
        _result_keys = ('hypervisor_id', 'tab_group', 'hypervisor', 'view',
                        'actions', 'url')

        self.assertEqual(len(result.keys()), len(_result_keys))
        for key in _result_keys:
            self.assertIn(key, result)

        self.assertEqual(result['hypervisor'].id, hypervisor.id)

    def test_get_context_data2(self):

        view = HypervisorDetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        view.kwargs = {
            'hypervisor_id': 1000000
        }

        with self.assertRaises(Http302) as cm:
            view.get_context_data()

        self.assertEqual(u'/admin/hypervisors/', cm.exception.location)
        self.assertEqual(302, cm.exception.status_code)

    def test_get_context_data_state_up(self):
        hypervisor = self.tnx_hypervisors.get(1)

        view = HypervisorDetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        kwargs = {
            'hypervisor_id': hypervisor.id
        }
        view.kwargs = kwargs

        result = view.get_context_data(**kwargs)

        self.assertNotIn('Power on', result['actions'])
        self.assertIn('Power off', result['actions'])

    def test_get_context_data_state_down(self):
        hypervisor = self.tnx_hypervisors.get(4)

        view = HypervisorDetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        kwargs = {
            'hypervisor_id': hypervisor.id
        }
        view.kwargs = kwargs

        result = view.get_context_data(**kwargs)

        self.assertIn('Power on', result['actions'])
        self.assertNotIn('Power off', result['actions'])


class AdminDetailViewTestCase(BaseClientTestCase):

    def test_get_context_data(self):
        hypervisor = self.tnx_hypervisors.get(1)

        view = AdminDetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            session={},
            COOKIES={}
        )
        kwargs = {
            'hypervisor_id': u'{0}_{0}'.format(hypervisor.id)
        }
        view.kwargs = kwargs

        result = view.get_context_data(**kwargs)
        _result_keys = ('hypervisor_id', 'tab_group', 'hypervisor', 'view',
                        'actions', 'url')
        self.assertEqual(len(result.keys()), len(_result_keys))
        for key in _result_keys:
            self.assertIn(key, result)

        self.assertEqual(result['hypervisor'].id, hypervisor.id)

    def test_get_context_data2(self):

        view = AdminDetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            session={},
            COOKIES={}
        )
        kwargs = {
            'hypervisor_id': '1000000_1000000'
        }
        view.kwargs = kwargs

        result = view.get_context_data(**kwargs)
        _result_keys = ('hypervisor_id', 'tab_group', 'hypervisor', 'view',
                        'actions')
        self.assertEqual(len(result.keys()), len(_result_keys))
        for key in _result_keys:
            self.assertIn(key, result)

        self.assertIsNone(result['hypervisor'])


class HypervisorMultiAssignImageViewTestCase(BaseClientTestCase):

    def test_get_initial(self):
        view = HypervisorMultiAssignImageView()
        view.kwargs = {}
        view.request = get_fake_request()

        result = view.get_initial()
        self.assertIsInstance(result['instances'], list)
        self.assertIsInstance(result['hypervisors'], list)

    def test_format_hypervisor(self):
        hypervisor = self.tnx_hypervisors.get(1)

        view = HypervisorMultiAssignImageView()
        view.request = get_fake_request(
            method='POST')
        result = view.format_hypervisor(hypervisor)
        _result = (1, 'Host1', 'Host1', None, 'image name', '', '-', u'No', 0)
        self.assertSequenceEqual(result[:5], _result[:5])
        self.assertSequenceEqual(result[6:], _result[6:])


class HypervisrEditViewTest(BaseClientTestCase):

    def test_get_initial(self):
        hypervisor = TnxHypervisor(self.tnx_hypervisors.get(1))

        view = HypervisorUpdateView()
        view.request = get_fake_request(
            get_params={
                'id': str(hypervisor.id)
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        initial = view.get_initial()

        data = {'hypervisors': [hypervisor]}
        data.update(**{key: value
                       for key, value in hypervisor.to_dict().items()
                       if key not in ('id',)})

        self.assertListEqual(initial.keys(), data.keys())

    def test_multi_get_initial(self):
        hypervisor = self.tnx_hypervisors.get(1)
        hypervisor_2 = self.tnx_hypervisors.get(1)

        view = HypervisorUpdateView()
        view.request = get_fake_request(
            get_params={
                'id': [str(hypervisor.id), str(hypervisor_2.id)]
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        initial = view.get_initial()

        self.assertListEqual(initial.keys(), ['hypervisors'])
