# coding: utf-8

import json
import logging
import re

from django.core.urlresolvers import reverse_lazy, reverse
from django.http import HttpResponse
from django.utils.datastructures import SortedDict
from django.utils.translation import ugettext_lazy as _
from django.template import defaultfilters as filters
from django.views import generic

from horizon import exceptions, forms, messages, workflows, tables
from horizon.tabs import views
from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard.api.rest import (
    urls as rest_urls, utils as rest_utils)
from openstack_dashboard.dashboards.admin.hypervisors import (
    views as admin_view)
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)

from tionix_dashboard.clients import (
    TNXHypervisorsClient, TNXServersClient, TNXPowerDevicesClient)
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.admin.hypervisors.states import STATE_UP
from tionix_dashboard.dashboards.admin.hypervisors.workflow import (
    AssignImageWorkflow, AssignPowerDeviceWorkflow)
from tionix_dashboard.dashboards.admin.hypervisors import (
    tables as admin_tables)
from tionix_dashboard.helpers.views import (
    ServerFilterViewMixin, ServerSortMultiSelect, RefererRedirectMixin,
    LinkBatchFormView, LinkBatchWorkflowViewMixin)
from tionix_dashboard.helpers.vm_constants import VMTypes
from tionix_dashboard.dashboards.admin.hypervisors.forms import (
    HypervisorExtra, HypervisorShutdown, HypervisorPowerOff,
    HypervisorMarkReserved, HypervisorRestart, HypervisorUpdateForm)
from tionix_dashboard.dashboards.admin.hypervisors.power_actions import (
    POWER_ON_STATE)
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    POWER_STATE_DISPLAY_CHOICES, STATE_DISPLAY_CHOICES,
    STATUS_DISPLAY_CHOICES, TnxAdminHypervisorsTable, ConnectedDevicesTable)
from tionix_dashboard.dashboards.admin.hypervisors.tabs import (
    TnxHypervisorHostTabs, TnxHypervisorDetailTabs, HypervisorDetailTabs,
    HypervisorHostTabs)
from tionix_dashboard.helpers.workflows import WorkflowVerticalView
from tionix_dashboard.url_resolve_map import HYPERVISORS_INDEX

logger = logging.getLogger(__name__)


class HypervisorExtraView(tables.DataTableView, forms.ModalFormView):
    """View represents hypervisors extra information"""
    table_class = ConnectedDevicesTable
    form_class = HypervisorExtra
    modal_header = _('Hypervisor extra information: "{0}"')
    template_name = 'hypervisors/extra.html'
    modal_id = "show_hypervisors_extra_info"
    page_title = _("Hypervisor extra information")
    cancel_label = _('Close')

    @memoized.memoized_method
    def get_form(self, form_class=None):
        form_class = form_class or self.get_form_class()
        return super(HypervisorExtraView, self).get_form(form_class)

    def get_context_data(self, **kwargs):
        context = super(HypervisorExtraView, self).get_context_data(**kwargs)
        context['form'] = self.get_form()
        hypervisor = self.get_object()
        context['hypervisor'] = hypervisor
        context['hypervisor_name'] = getattr(hypervisor,
                                             hypervisor.NAME_ATTR,
                                             '')
        return context

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_object(self, *args, **kwargs):
        hypervisor_id = self.kwargs['hypervisor_id'] or ''
        if hypervisor_id.isdigit():
            hypervisor_id = int(hypervisor_id)

        hypervisor = None
        try:
            hypervisor = TNXHypervisorsClient.get(self.request,
                                                  hypervisor_id)
        except UrlNotAvailableError:
            raise
        except Exception as e:
            logger.error(e.message)
            msg = _('Unable to retrieve hypervisor details.')
            exceptions.handle(self.request, msg)

        return hypervisor

    def get_initial(self):
        hypervisor = self.get_object()
        data = {}
        if hypervisor:
            try:
                cpu_info = json.loads(hypervisor.cpu_info)
            except ValueError:
                cpu_info = {}
            data.update(cpu_info)

            data.update(
                {k: v for k, v in hypervisor.to_dict().iteritems()
                 if k in self.form_class.get_fields()})

        return data

    def get_data(self):
        hypervisor = self.get_object()
        return hypervisor.devices

    def get(self, request, *args, **kwargs):
        handled = self.construct_tables()
        if handled:
            return handled
        return self.render_to_response(self.get_context_data(**kwargs))

    def post(self, request, *args, **kwargs):
        handled = self.construct_tables()
        if handled:
            return handled

        return super(HypervisorExtraView, self).post(request, *args, **kwargs)


class HypervisorsViewMixin(RefererRedirectMixin):

    def format_hypervisor(self, row):
        return row

    def is_allowed(self, obj):
        return True

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_objects(self, *args, **kwargs):
        hypervisors, failed, instances = [], [], []
        for hypervisor_id in self.get_object_ids():
            try:
                obj = TNXHypervisorsClient.get(self.request, hypervisor_id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(err.message)
                continue

            hypervisor_name = getattr(obj, obj.NAME_ATTR, '')

            if not self.is_allowed(obj):
                failed.append(hypervisor_name)
                continue

            hypervisors.append(self.format_hypervisor(obj))

            servers = TNXServersClient.list(
                self.request,
                search_opts={'host': hypervisor_name, 'all_tenants': True})

            for res in servers:

                power_state_dict = dict(project_tables.POWER_DISPLAY_CHOICES)

                instances.append([
                    getattr(res, 'name', ''),
                    getattr(self.tenants.get(res.tenant_id), 'name', ''),
                    res.host_server,
                    power_state_dict.get(
                        project_tables.get_power_state(res), ''),
                    project_tables.get_ips(res),
                    VMTypes.TYPES.get(
                        str(getattr(res, 'tnx_type', '')), ''),
                ])

        if failed:
            messages.error(self.request, u'{0}: "{1}"'.format(
                self.failure_msg, u'", "'.join(failed)))

        return hypervisors, instances

    @memoized.memoized_method
    def get_all_tenants(self, request):
        try:
            tenants, _has_more = api.keystone.tenant_list(request)
        except Exception as err:
            logger.error(unicode(err))
            tenants = []
            msg = _('Unable to retrieve instance project information.')
            exceptions.handle(request, msg)
        return SortedDict([(t.id, t) for t in tenants])

    @property
    def tenants(self):
        return self.get_all_tenants(self.request)


class HypervisorShutdownView(HypervisorsViewMixin, LinkBatchFormView):
    """Shut down hypervisor view"""
    form_class = HypervisorShutdown
    modal_header = _('Shutdown hypervisor')
    template_name = 'hypervisors/shutdown.html'
    modal_id = "hypervisors_shutdown"
    submit_label = _('Shutdown')
    context_object_name = 'hypervisor'
    success_url = reverse_lazy(HYPERVISORS_INDEX)
    hidden_title = False
    failure_msg = _('"Shutdown" is not available for hypervisors')

    def format_hypervisor(self, row):
        return (
            row.id,
            getattr(row, row.NAME_ATTR, '')
        )

    def get_initial(self):

        hypervisors, instances = self.get_objects()

        # New modal header
        self.modal_header += ' {}'.format(','.join(n for i, n in hypervisors))

        return {'rows': instances,
                'hypervisors': hypervisors}

    def is_allowed(self, obj):
        allowed = False

        if obj and obj.tnx_devices:
            if obj.tnx_power_state is None:
                allowed = obj.state == STATE_UP
            else:
                allowed = obj.tnx_power_state == POWER_ON_STATE
        return allowed


class HypervisorPowerOffView(HypervisorShutdownView):
    """Power off hypervisor view"""
    form_class = HypervisorPowerOff
    modal_header = _('Power off hypervisor')
    template_name = 'hypervisors/power_off.html'
    modal_id = "hypervisors_power_off"
    submit_label = _('Power off')
    context_object_name = 'hypervisor'
    success_url = reverse_lazy(HYPERVISORS_INDEX)
    hidden_title = False
    failure_msg = _('"Power off" is not available for hypervisors')


class HypervisorRestartView(HypervisorShutdownView):
    """Restart hypervisor view"""
    form_class = HypervisorRestart
    modal_header = _('Restart hypervisors: ')
    template_name = 'hypervisors/restart.html'
    modal_id = "hypervisors_restart"
    submit_label = _('Restart')
    context_object_name = 'hypervisor'
    success_url = reverse_lazy(HYPERVISORS_INDEX)
    hidden_title = False
    failure_msg = _('"Restart" is not available for hypervisors')

    def get_initial(self):
        hypervisors, instances = self.get_objects()
        return {'rows': instances,
                'hypervisors': hypervisors}

    def get_context_data(self, **kwargs):
        context = super(HypervisorRestartView, self).get_context_data(**kwargs)
        context['hypervisors'] = []
        context['servers_count'] = 0

        form = kwargs.get('form')
        if form:
            hypervisors = form.initial.get('hypervisors', [])
            for hypervisor in hypervisors:
                _hypervisor = TNXHypervisorsClient.get(
                    self.request, hypervisor[0])
                _hypervisor.servers = []

                _hypervisor.tnx_power_state = dict(
                    POWER_STATE_DISPLAY_CHOICES
                ).get(_hypervisor.tnx_power_state)

                _hypervisor.state = dict(
                    STATE_DISPLAY_CHOICES
                ).get(_hypervisor.state)

                servers = TNXServersClient.list(
                    self.request,
                    search_opts={
                        'all_tenants': True,
                        'host': hypervisor[1]
                    })

                for server in servers:
                    addresses = server.addresses.values()
                    net = None
                    if addresses:
                        net = addresses[0]
                    if net is not None:
                        try:
                            internal_ip = net[0]['addr']
                        except (IndexError, KeyError):
                            internal_ip = ''
                    else:
                        internal_ip = ''
                    _hypervisor.servers.append({
                        'project': getattr(
                            self.tenants.get(server.tenant_id), 'name', ''),
                        'name': server.name,
                        'state': getattr(server, 'OS-EXT-STS:vm_state', ''),
                        'internal_ip': internal_ip,
                        'type': VMTypes.TYPES.get(str(server.tnx_type), ''),
                    })
                context['servers_count'] += len(_hypervisor.servers)
                context['hypervisors'].append(_hypervisor)

        context['hypervisors_count'] = len(context['hypervisors'])
        return context


class HypervisorMarkReservedView(HypervisorShutdownView):
    """Mark hypervisor reserved"""
    form_class = HypervisorMarkReserved
    modal_header = _('Mark hypervisors reserved: ')
    template_name = 'hypervisors/markreserved.html'
    submit_label = _("Shutdown and reserve")
    failure_msg = _('"Move to reserve" is not available for hypervisors')

    def is_allowed(self, obj):
        allowed = False

        if obj and obj.tnx_devices:
            # move to reserv only if device state is known
            allowed = obj.tnx_power_state == POWER_ON_STATE

        return allowed


def mark_device_default(request):
    TNXHypervisorsClient.set_default_power_device_for_node(
        request=request,
        node_id=request.POST['node_id'],
        power_device_id=int(request.POST['power_device_id'])
    )
    return HttpResponse()


class TnxAdminHypervisorsIndexView(ServerSortMultiSelect,
                                   admin_view.AdminIndexView):

    tab_group_class = TnxHypervisorHostTabs


class DetailContextDataMixin(object):
    actions_table = None

    # pylint: disable=not-callable
    def get_context_data(self, **kwargs):
        context = super(DetailContextDataMixin, self).get_context_data(
            **kwargs
        )
        hypervisor = self.get_data()
        context['hypervisor'] = hypervisor
        actions = []
        if hypervisor:
            table = self.actions_table(self.request)
            actions = table.render_row_actions(hypervisor)
            context["url"] = reverse(HYPERVISORS_INDEX)
        context['actions'] = actions
        return context


class HypervisorDetailView(ServerSortMultiSelect,
                           ServerFilterViewMixin,
                           DetailContextDataMixin,
                           views.TabbedTableView):
    tab_group_class = TnxHypervisorDetailTabs
    template_name = 'horizon/common/_detail.html'
    redirect_url = HYPERVISORS_INDEX
    page_title = _("Hypervisor Details: {{ hypervisor.page_title_name }}")
    scheduler_tab_url = 'horizon:admin:hypervisors:scheduler_tab'
    actions_table = TnxAdminHypervisorsTable

    @memoized.memoized_method
    def get_data(self):
        hypervisor_id = self.kwargs['hypervisor_id']

        try:
            hypervisor = TNXHypervisorsClient.get(self.request, hypervisor_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Unable to retrieve details for '
                                'hypervisor "%s".') % hypervisor_id,
                              redirect=redirect)
            # # Not all exception types handled above will result
            # in a redirect. Need to raise here just in case.
            raise exceptions.Http302(redirect)

        hypervisor.tnx_power_state_label = dict(
            POWER_STATE_DISPLAY_CHOICES
        )[unicode(hypervisor.tnx_power_state)]

        hypervisor.state_label = dict(
            STATE_DISPLAY_CHOICES
        )[unicode(hypervisor.state)]

        hypervisor.status_label = dict(
            STATUS_DISPLAY_CHOICES
        )[unicode(hypervisor.status)]

        hypervisor.page_title_name = (
            u'{0} ({1})'.format(hypervisor.hypervisor_hostname,
                                hypervisor.tnx_node_title)
            if hypervisor.tnx_node_title else hypervisor.hypervisor_hostname)

        return hypervisor

    def get_tabs(self, request, **kwargs):
        hypervisor = self.get_data()
        kwargs['obj_id'] = hypervisor.id
        kwargs['hypervisor'] = hypervisor
        kwargs['scheduler_tab_url'] = reverse(self.scheduler_tab_url,
                                              args=(hypervisor.id,))
        return self.tab_group_class(request, **kwargs)


class AdminDetailView(DetailContextDataMixin, views.TabbedTableView):
    tab_group_class = HypervisorDetailTabs
    template_name = 'horizon/common/_detail.html'
    actions_table = admin_tables.AdminHypervisorsTable

    @memoized.memoized_method
    def get_data(self):
        hypervisor = None

        try:
            _id = re.findall('^\d+', self.kwargs['hypervisor_id'])[0]
            client = novaclient(self.request)
            hypervisor = client.hypervisors.get(_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve hypervisor instances list.'))
        return hypervisor

    def get_tabs(self, request, **kwargs):
        hypervisor = self.get_data()
        kwargs['hypervisor'] = hypervisor
        return self.tab_group_class(request, **kwargs)


class HypervisorMultiAssignImageView(HypervisorsViewMixin,
                                     LinkBatchWorkflowViewMixin,
                                     workflows.WorkflowView):

    workflow_class = AssignImageWorkflow
    ajax_template_name = 'hypervisors/_assign_image.html'
    failure_msg = _('"Assign image" is not available for hypervisors')
    submit_url = "horizon:admin:hypervisors:assign_image"

    def format_hypervisor(self, row):
        hypervisor = (
            getattr(row, row.NAME_ATTR),
            row.tnx_node_title,
            row.hypervisor_type,
            row.tnx_image_name,
            dict(STATE_DISPLAY_CHOICES)[unicode(row.state)],
            dict(POWER_STATE_DISPLAY_CHOICES)[unicode(row.tnx_power_state)],
            filters.capfirst(filters.yesno(row.tnx_is_reserved)),
            row.running_vms,
        )

        if self.request.method == 'POST':
            hypervisor = (row.id, ) + hypervisor

        return hypervisor

    def get_initial(self):
        initial = super(HypervisorMultiAssignImageView, self).get_initial()
        hypervisors, instances = self.get_objects()
        initial.update(hypervisors=hypervisors, instances=instances)
        return initial


class AdminIndexView(admin_view.AdminIndexView):
    tab_group_class = HypervisorHostTabs
    template_name = 'hypervisors/index.html'


# pylint: disable=no-member
class HypervisorUpdateView(RefererRedirectMixin, LinkBatchFormView):
    """Edit hypervisor view"""
    form_class = HypervisorUpdateForm
    modal_header = _('Edit hypervisors: {0}')
    template_name = 'hypervisors/update.html'
    success_url = reverse_lazy(HYPERVISORS_INDEX)

    @memoized.memoized_method
    def get_objects(self):
        """Retrieve hypervisors"""
        hypervisors = []
        for hypervisor_id in self.get_object_ids():
            try:
                obj = TNXHypervisorsClient.get(self.request, hypervisor_id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(err.message)
                continue
            hypervisors.append(obj)
        return hypervisors

    def get_initial(self):
        """Initial data"""
        hypervisors = self.get_objects()

        initial = {'hypervisors': hypervisors}
        if len(hypervisors) == 1:
            initial.update(**{
                key: value
                for key, value in hypervisors[0].to_dict().items()
                if key not in ('id',)
            })

        self.modal_header = self.modal_header.format(
            ', '.join([getattr(hyp, hyp.NAME_ATTR, '') for hyp in hypervisors])
        )

        return initial


@rest_urls.register
class PowerDevicePorts(generic.View):
    """Power device ports.
    """
    url_regex = r'tnx/hypervisors/power_device_ports/$'

    @rest_utils.ajax()
    def get(self, request):
        power_device_id = request.GET.get('power_device_id')
        power_device_ports = []
        if power_device_id:
            try:
                power_device = TNXPowerDevicesClient.get(request,
                                                         power_device_id)
                assigned_nodes = (
                    TNXPowerDevicesClient.power_device_assigned_nodes(
                        request, power_device_id)
                )
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to get power control ports list'),
                                  redirect='')
            else:
                assigned_nodes = {
                    assign_node['port']: assign_node
                    for assign_node in assigned_nodes
                }

                power_device_ports = [
                    assigned_nodes.get(port, {'id': port,
                                              'port': port,
                                              'tnx_node_name': None,
                                              'is_default': None,
                                              'is_initialized': None})
                    for port in power_device.coil_map
                ]
        return {'available': power_device_ports, 'allocated': []}


class AssignPowerDeviceView(RefererRedirectMixin, WorkflowVerticalView):
    workflow_class = AssignPowerDeviceWorkflow
    success_url = reverse_lazy('horizon:admin:hypervisors:index')
    page_title = _("Assigning power control")

    @memoized.memoized_method
    def get_object(self):
        hypervisor_id = self.kwargs['hypervisor_id'] or ''
        if hypervisor_id.isdigit():
            hypervisor_id = int(hypervisor_id)

        hypervisor = None
        try:
            hypervisor = TNXHypervisorsClient.get(self.request,
                                                  hypervisor_id)
        except UrlNotAvailableError:
            raise
        except Exception as e:
            logger.error(e.message)
            msg = _('Unable to retrieve hypervisor details.')
            exceptions.handle(self.request, msg)

        return hypervisor

    def get_initial(self):
        initial = super(AssignPowerDeviceView, self).get_initial()
        initial.update(**self.kwargs)
        hypervisor = self.get_object()
        devices = hypervisor.devices
        if devices:
            device = devices[0]
            initial.update({'power_device': device.id,
                            'power_device_port': device.device_port})
        return initial
