# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import forms, workflows, messages, exceptions

from tionix_dashboard.clients import (
    TNXHypervisorsClient, TNXPowerDevicesClient)
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.pxe_images import TNXPXEImagesClient
from tionix_dashboard.dashboards.helpers import coerce_to_int
from tionix_dashboard.helpers.workflows import (
    TransferTableAction, TransferTableStep
)

logger = logging.getLogger(__name__)


class AssignImageFirstAction(workflows.Action):
    table_caption = _('Hypervisors list')

    columns = [
        _('Hostname'),
        pgettext_lazy('Hypervisor', 'Host title'),
        _('Type'),
        _('Image'),
        _('State'),
        _('Power state'),
        _('Reserved'),
        pgettext_lazy('Hypervisor', 'Instances')
    ]

    image = forms.ChoiceField(label=_("Image Name"), required=True)

    restart = forms.BooleanField(
        label=_('Restart Hypervisors'),
        required=False, widget=forms.CheckboxInput()
    )

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_image_choices(request, context):
        try:
            images = [
                (image.id, image.image_name)
                for image in
                TNXPXEImagesClient.list(request)]
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.debug(err)
            images = [('default', 'default')]
        return images


class AssignNodeFirstStep(workflows.Step):
    """Assign image first step"""

    action_class = AssignImageFirstAction
    template_name = 'hypervisors/_assign_image_first.html'
    contributes = ('image', 'restart')
    depends_on = ('id', 'hypervisors')

    def prepare_action_context(self, request, context):
        context['rows'] = context['hypervisors']
        return context


class AssignImageSecondAction(workflows.Action):
    table_caption = _('Instances list')

    columns = [
        _('Host name'),
        _('Project'),
        _('Name'),
        _('State'),
        _('Internal IP'),
        _('Type')
    ]


class AssignNodeSecondStep(workflows.Step):
    """Assign image first step"""

    action_class = AssignImageSecondAction
    template_name = 'hypervisors/_assign_image_second.html'
    depends_on = ('id', 'instances', 'restart',)

    def prepare_action_context(self, request, context):
        context['rows'] = context['instances']
        return context


class AssignImageWorkflow(workflows.Workflow):
    slug = 'assign_image'
    name = _('Assigning image')
    finalize_button_name = _('Assign Image')
    success_url = failure_url = 'horizon:admin:hypervisors:index'
    default_steps = (AssignNodeFirstStep, AssignNodeSecondStep)
    wizard = True
    template_name = 'hypervisors/_assign_image.html'
    success_hyps = None
    failed_hyps = None

    def get_absolute_url(self):
        return self.request.get_full_path()

    def handle(self, request, context):
        success_hyps, failed_hyps = [], []

        hypervisors = context['hypervisors']

        image_id = context['image']
        if image_id.isdigit():
            image_id = int(image_id)

        for hypervisor in hypervisors:
            try:
                TNXHypervisorsClient.assign_hypervisor_image(
                    request, hypervisor[0], image_id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(unicode(err))
                failed_hyps.append(hypervisor[1])
            else:
                success_hyps.append(hypervisor[1])

        if context['restart'] is True:
            could_not_restart = []

            for hypervisor in hypervisors:
                # skipping if image could be assigned
                if hypervisor[1] in failed_hyps:
                    continue
                # TODO fix tionix_api.reset_node for handling statuses
                if hypervisor[6] == '-':
                    could_not_restart.append(hypervisor[1])
                    continue

                try:
                    TNXHypervisorsClient.reset_hypervisor(
                        request, hypervisor[0])
                except UrlNotAvailableError:
                    raise
                except Exception as err:
                    logger.error(unicode(err))
                    could_not_restart.append(hypervisor[1])

            if could_not_restart:
                msg = _("Failed to restart hypervisors: {0}").format(
                    ', '.join(could_not_restart))
                messages.error(request, msg)

        self.success_hyps = set(success_hyps)
        self.failed_hyps = set(failed_hyps)

        return not failed_hyps

    def format_status_message(self, message):
        msg = super(AssignImageWorkflow, self).format_status_message(
            message)

        if self.failed_hyps:
            msg = _('Failed to assign image for hypervisors: '
                    '"{0}"').format('", "'.join(self.failed_hyps))

        elif self.success_hyps:
            msg = _('Successfully assigned image for hypervisors: '
                    '"{0}"').format('", "'.join(self.success_hyps))

        return msg


class SelectPowerDeviceAction(TransferTableAction):
    """Select power device action"""
    power_device = forms.ChoiceField(
        label=_('Power control'),
        widget=forms.SelectWidget(
            attrs={'ng-model': 'powerDevice'},
            data_attrs=('id', 'title', 'device_name', 'protocol_type',
                        'coils_count', 'used_coils_count'),
            transform=lambda x: x.name
        ),
        error_messages={'required': _('Power control is not chosen.')}
    )

    # noinspection PyUnusedLocal
    def populate_power_device_choices(self, request, context):
        power_devices = []
        try:
            power_devices = TNXPowerDevicesClient.list(request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to get power controls list'),
                              redirect='')
        else:
            power_devices = [(power_device.id, power_device)
                             for power_device in power_devices]
        if not power_devices:
            power_devices.insert(0, ('', _("No available power controls")))
        else:
            power_devices.insert(0, ('', _("Select a power control")))
        return power_devices

    class Meta(object):
        name = _("Power control")


class SelectPowerDeviceStep(TransferTableStep):
    action_class = SelectPowerDeviceAction
    template_name = 'hypervisors/_ip_addresses_step.html'
    controller = 'SelectPowerDeviceController'
    contributes = ('power_device', )
    depends_on = ('hypervisor_id', )


class SelectPowerDevicePortAction(TransferTableAction):
    """Select power device action"""
    power_device_port = forms.CharField(
        label=_('Power control port'),
        error_messages={'required': _('Power control port is not chosen.')},
        widget=forms.TextInput(attrs={'ng-model': 'powerDevicePort'})
    )

    class Meta(object):
        name = _("Power control port")


class SelectPowerDevicePortStep(TransferTableStep):
    action_class = SelectPowerDevicePortAction
    template_name = 'hypervisors/_ip_addresses_step.html'
    controller = 'SelectPowerDevicePortController'
    contributes = ('power_device_port', )
    depends_on = ('hypervisor_id', )


class AssignPowerDeviceWorkflow(workflows.Workflow):
    slug = 'assign_power_device'
    name = _('Assigning power control')
    template_name = "hypervisors/_assign_power_device.html"
    success_message = _("Successfully assigned power control.")
    failure_message = _("Unable to assign power control.")
    finalize_button_name = _('Assign Power Control')
    success_url = failure_url = 'horizon:admin:hypervisors:index'
    default_steps = (SelectPowerDeviceStep,
                     SelectPowerDevicePortStep)

    def handle(self, request, context):
        result = True
        power_device = coerce_to_int(context['power_device'])
        device_port = coerce_to_int(context['power_device_port'])

        try:
            TNXHypervisorsClient.assign_power_device(
                request,
                context['hypervisor_id'],
                power_device,
                device_port)
        except Exception as err:
            logger.error(unicode(err))
            result = False
        return result
