# coding: utf-8
import logging

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.images import views

from oslo_utils import units

from tionix_dashboard.dashboards.admin.images.forms import (
    MultiImageEditForm, AdminCreateImageForm, AdminUpdateImageForm)
from tionix_dashboard.dashboards.admin.images.tables import AdminImagesTable
from tionix_dashboard.dashboards.project.project_images import (
    views as project_views
)
from tionix_dashboard.helpers.utils import get_table_filter_action
from tionix_dashboard.helpers.views import LinkBatchFormView

logger = logging.getLogger(__name__)


class IndexView(views.IndexView):
    table_class = AdminImagesTable

    def get_filters(self):
        filters = {'is_public': None}
        filter_field = self.table.get_filter_field()
        filter_string = self.table.get_filter_string()
        filter_action = get_table_filter_action(self.table)
        if (filter_field and filter_string and
                filter_action.is_api_filter(filter_field)):
            if filter_field in ['size_min', 'size_max']:
                try:
                    # noinspection PyArgumentList
                    filter_string = long(float(filter_string) * units.Mi)
                    if filter_string >= 0:
                        filters[filter_field] = filter_string
                except ValueError:
                    pass
            elif (filter_field == 'disk_format' and
                  filter_string.lower() == 'docker'):
                filters['disk_format'] = 'raw'
                filters['container_format'] = 'docker'
            else:
                modifier = filter_action.filter_string_modifiers.get(
                    filter_field
                )
                if modifier is not None:
                    filter_string = modifier(filter_string)
                else:
                    filter_string = filter_string
                filters[filter_field] = filter_string
        return filters


class DetailView(project_views.DetailView):

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        context["url"] = reverse_lazy('horizon:admin:images:index')
        table = AdminImagesTable(self.request)
        context["actions"] = table.render_row_actions(context["image"])
        return context


class MultiImageEditView(LinkBatchFormView):
    """Multi edit for images"""
    form_class = MultiImageEditForm
    modal_header = _('Edit images')
    template_name = 'images/update.html'
    submit_url = "horizon:admin:images:multi_edit"
    success_url = reverse_lazy("horizon:admin:images:index")

    @memoized.memoized_method
    def get_objects(self):
        """Retrieve images"""
        images = []
        for image_id in self.get_object_ids():
            try:
                obj = api.glance.image_get(self.request, image_id)
            except Exception as err:
                logger.error(err)
                continue
            images.append(obj)
        return images

    def get_initial(self):
        """Initial data"""
        images = self.get_objects()
        return {'images': images,
                'images_names': [image.name for image in images]}


# noinspection PyClassHasNoInit
class CreateView(project_views.CreateImageView):
    template_name = 'admin/images/create.html'
    form_class = AdminCreateImageForm
    submit_url = reverse_lazy('horizon:admin:images:create')
    success_url = reverse_lazy('horizon:admin:images:index')


class UpdateView(project_views.UpdateImageView):
    form_class = AdminUpdateImageForm
    template_name = 'admin/images/update.html'
    submit_url = "horizon:admin:images:update"
    success_url = reverse_lazy('horizon:admin:images:index')
