# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import messages
from horizon import tables

from openstack_dashboard import policy
from openstack_dashboard.dashboards.admin.instances import (
    tables as admin_tables)
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions, constants)
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxUpdateAction, LinkBatchAction, TnxBatchActionMixin)
from tionix_dashboard.helpers.utils import get_filter_value_by_dict
from tionix_dashboard.helpers.vm_constants import ACTIVE, VMTypes

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class TnxAdminInstanceFilterAction(TnxFilterAction):
    """
    .. attribute:: filter_string_modifiers

        Modifies filter field value to make api filtering.
    """
    name = "filter_admin_instances"
    filter_type = "server"

    filter_choices = (('tenant', _("Project"), False),
                      ('name', _("Name"), True),
                      ('host', _("Host ="), True),
                      ('tnx_node_title', _("Host title"), False),
                      ('tnx_type', _("Type ="), True),
                      ('ip', _("IPv4 Address ="), True),
                      ('ip6', _("IPv6 Address ="), True),
                      ('status', _("Status ="), True),
                      ('power_state', _("Power state"), True),
                      ('uuid', _("Instance ID ="), True),
                      ('image', _("Image ID ="), True),
                      ('flavor', _("Flavor ID ="), True),
                      ('OS-EXT-AZ:availability_zone',
                       _("Availability zone"),
                       False),)
    filter_string_modifiers = {
        'power_state': project_actions.get_power_states_by_name,
        'tnx_type': get_filter_value_by_dict(VMTypes.TYPES,
                                             not_found_none=False),
        'status': project_actions.get_status_by_name
    }


# noinspection PyAbstractClass
class CreateImageBatchAction(TnxBatchActionMixin, tables.BatchAction):
    """
    Action power on selected hypervisor.
    """
    name = 'create_image'
    verbose_name = _(u'Create image')
    policy_rules = (('identity', 'admin_required'), )

    # noinspection PyUnusedLocal
    @staticmethod
    def action_present(count):
        return CreateImageBatchAction.verbose_name

    # noinspection PyUnusedLocal
    @staticmethod
    def action_past(count):
        return CreateImageBatchAction.verbose_name

    def action(self, request, datum_id):
        TNXServersClient.create_image_for_instance(request, datum_id)


# noinspection PyAbstractClass
class InstanceExtraAction(tables.LinkAction):
    """
    Action show extra information for selected instance.
    """
    name = 'instances'
    verbose_name = _('More')
    url = "horizon:admin:instances:extra"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )

    def get_link_url(self, datum=None):
        instance_id = datum.id if datum else None
        return reverse(self.url, args=[instance_id])


# noinspection PyAbstractClass
class TnxMigrateInstance(TnxBatchActionMixin, admin_tables.MigrateInstance):

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Migrate Instance",
            u"Migrate Instances",
            count
        )

    def allowed(self, request, instance):
        return (instance is None or
                ((instance.status in project_tables.ACTIVE_STATES or
                  instance.status == 'SHUTOFF') and
                 not project_tables.is_deleting(instance)))


class InstanceUpdateCell(TnxUpdateAction):
    """
    Action update instance name
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        result = True
        try:
            TNXServersClient.update(request, obj_id, name=new_cell_value)
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to update instance.'))
            result = False
        return result


# noinspection PyAbstractClass
class TnxStopInstance(TnxBatchActionMixin, project_tables.StopInstance):
    def allowed(self, request, instance):
        if instance is None:
            return True
        allow = super(TnxStopInstance, self).allowed(request, instance)
        return allow and instance.status.lower() == ACTIVE


# noinspection PyAbstractClass,PyClassHasNoInit
class ResetStateAction(project_actions.ResetStateAction):
    """ Action reset vm's state."""
    multiple_next_view_name = "horizon:admin:instances:reset_state"


# noinspection PyAbstractClass
class TnxLiveMigrateInstance(policy.PolicyTargetMixin, LinkBatchAction):
    name = "live_migrate"
    multiple_next_view_name = "horizon:admin:instances:live_migrate"
    classes = ("btn-migrate", "btn-danger")
    policy_rules = (
        ("compute", "compute_extension:admin_actions:migrateLive"),)

    def action_present(self, count):
        return (
            ungettext_lazy(
                u"Live Migrate Instance",
                u"Live Migrate Instances",
                count
            )
        )

    def action_past(self, count):
        return (
            ungettext_lazy(
                u"Live Migrated Instance",
                u"Live Migrated Instances",
                count
            )
        )

    def allowed(self, request, datum):
        allowed = LicenseController.is_valid(
            request, LicenseController.CLOUD_CONTROL
        )
        if (datum and
                (datum.status not in project_tables.ACTIVE_STATES or
                 project_tables.is_deleting(datum))):
            allowed = False
        return allowed


# noinspection PyAbstractClass
class TnxHypervisorLiveMigrateInstance(TnxLiveMigrateInstance):
    def get_next_view_kwargs(self, table, obj_ids, kwargs=None):
        kwargs = super(TnxHypervisorLiveMigrateInstance,
                       self).get_next_view_kwargs(table, obj_ids, kwargs)
        kwargs.update({'hypervisor_id': self.table.kwargs['hypervisor_id']})
        return kwargs


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxAdminConsoleLink(project_actions.TnxConsoleLink):
    url = 'horizon:admin:instances:console_tab'


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxAdminLogLink(project_actions.TnxLogLink):
    url = 'horizon:admin:instances:log_tab'


# noinspection PyAbstractClass,PyClassHasNoInit
class AdminEditInstance(project_actions.EditInstance):
    multiple_next_view_name = "horizon:admin:instances:update"


def get_task_by_name(value):
    """Return found task from display value"""
    if value is not None:
        value = unicode(value).lower()
        for actual_value, translation in constants.VM_TASK_DISPLAY_CHOICES:
            if value == unicode(translation).lower():
                value = actual_value
                break
        return value


# noinspection PyAbstractClass
class TnxHypervisorInstancesFilterAction(TnxFilterAction):
    name = "filter_hyp_instances"
    filter_type = "server"

    filter_choices = (('tenant', _("Project"), False),
                      ('name', _("Name"), True),
                      ('tnx_type', _("Type ="), True),
                      ('ip', _("IPv4 Address"), True),
                      ('ip6', _("IPv6 Address"), True),
                      ('flavor', _("Flavor ID ="), True),
                      ('status', _("Status ="), True),
                      ('task_state', _('Task ='), True),
                      ('power_state', _("Power state"), True))
    filter_string_modifiers = {
        'power_state': project_actions.get_power_states_by_name,
        'tnx_type': get_filter_value_by_dict(VMTypes.TYPES,
                                             not_found_none=False),
        'status': project_actions.get_status_by_name,
        'task_state': get_task_by_name
    }


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxDeleteInstance(project_actions.TnxDeleteInstance):
    """ Action to delete instance """
    multiple_next_view_name = "horizon:admin:instances:delete"
