# coding: utf-8

import logging

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from horizon import forms, messages
from horizon.utils import filters

from openstack_dashboard import api
from tionix_dashboard.dashboards.admin.instances.helpers import (
    get_admin_volumes)

from tionix_dashboard.dashboards.project.project_instances import (
    forms as project_forms)

logger = logging.getLogger(__name__)


class InstanceExtra(forms.Form):
    """Instance Extra information"""

    image_name = forms.CharField(label=_('Image Name'))
    created = forms.CharField(label=_('Time since created'))
    size = forms.CharField(label=_('Size'))

    field_value_filters = {
        'created': (filters.parse_isotime, filters.timesince_sortable)
    }

    def __init__(self, request, *args, **kwargs):
        super(InstanceExtra, self).__init__(request, *args, **kwargs)
        self._init_values()

    def _init_values(self):
        if self.initial:
            for key, field in self.fields.iteritems():
                if key in self.initial:
                    value = self.initial[key]
                    if key in self.field_value_filters:
                        for _filter in self.field_value_filters[key]:
                            value = _filter(value)
                    field.value = value


class TnxLiveMigrateForm(forms.SelfHandlingForm):
    current_host = forms.CharField(label=_("Current Host"),
                                   required=False,
                                   widget=forms.TextInput(
                                       attrs={'readonly': 'readonly'}))
    host = forms.CharField(label=_("New Host"),
                           required=False,
                           help_text=_("Choose a Host to migrate to."),
                           widget=forms.HiddenInput())

    disk_over_commit = forms.BooleanField(label=_("Disk Over Commit"),
                                          initial=False, required=False)
    block_migration = forms.BooleanField(label=_("Block Migration"),
                                         initial=False, required=False)

    def clean(self):
        cleaned_data = super(TnxLiveMigrateForm, self).clean()

        if not cleaned_data.get('host'):
            raise ValidationError(_('New host is not chosen.'))
        return cleaned_data

    def handle(self, request, data):
        success_instances, failed_instances = [], []
        for instance in self.initial.get('instances', []):
            try:
                block_migration = data['block_migration']
                disk_over_commit = data['disk_over_commit']
                api.nova.server_live_migrate(request,
                                             instance.id,
                                             data['host'],
                                             block_migration=block_migration,
                                             disk_over_commit=disk_over_commit)
                success_instances.append(instance.name)
            except Exception as err:
                logger.error(unicode(err))
                failed_instances.append(instance.name)

        if failed_instances:
            msg = _('Failed to live migrate instances:')
            messages.error(request, u'{0} "{1}"'.format(
                msg, u'", "'.join(failed_instances)))

        if success_instances:
            msg = _('Preparing to live migrate to a hypervisor of '
                    'the following instances:')
            messages.success(request, u'{0} "{1}"'.format(
                msg, u'", "'.join(success_instances)))

        return True


# noinspection PyAbstractClass,PyClassHasNoInit
class DeleteInstanceForm(project_forms.DeleteInstanceForm):
    """ Delete instances form """

    @staticmethod
    def _get_volumes(request):
        return get_admin_volumes(request)
