# coding: utf-8
from django.conf import settings
from django.core import urlresolvers
from django.core.urlresolvers import reverse
from django.template.defaultfilters import title
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon.utils import filters, memoized

from openstack_dashboard.dashboards.admin.instances import (
    tables as admin_tables)
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    InstanceShowMetricAction)

from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    InstanceSchedulerAction)
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import TnxPaginationDataTable
from tionix_dashboard.helpers.vm_constants import VMTypes
from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions, constants)
from tionix_dashboard.dashboards.admin.instances import (
    actions as admin_actions)
from tionix_dashboard.url_resolve_map import IDENTITY_PROJECT_DETAIL


def get_hypervisor_link(instance):
    link = None
    if getattr(instance, 'node_id', None):
        link = reverse('horizon:admin:hypervisors:detail',
                       args=(instance.node_id, ))
    return link


STATUS_CHOICES = constants.STATUS_CHOICES
TASK_STATUS_CHOICES = admin_tables.AdminInstancesTable.TASK_STATUS_CHOICES


def get_url_for_project(datum):
    return urlresolvers.reverse(IDENTITY_PROJECT_DETAIL,
                                args=(datum.tenant_id,))


class TnxAdminInstancesTable(TnxPaginationDataTable):
    tenant = tables.Column("tenant_name",
                           link=get_url_for_project,
                           verbose_name=_("Project"))

    name = tables.Column(
        "name",
        link="horizon:admin:instances:detail",
        verbose_name=_("Name"),
        update_action=admin_actions.InstanceUpdateCell,
        form_field=EmptyCharField(max_length=255))
    host = tables.Column("OS-EXT-SRV-ATTR:host",
                         link=get_hypervisor_link,
                         verbose_name=_("Host"),
                         classes=('nowrap-col',))
    node_title = tables.Column("tnx_node_title",
                               verbose_name=_("Host title"))
    type = tables.Column(lambda x: unicode(getattr(x, "tnx_type", '') or ''),
                         verbose_name=_("Type"),
                         display_choices=VMTypes.TYPES.items())
    ip = tables.Column(project_tables.get_ips,
                       verbose_name=_("IP Address"),
                       attrs={'data-type': "ip"})
    task = tables.Column("OS-EXT-STS:task_state",
                         verbose_name=_("Task"),
                         empty_value=project_tables.TASK_DISPLAY_NONE,
                         status=True,
                         status_choices=TASK_STATUS_CHOICES,
                         display_choices=constants.VM_TASK_DISPLAY_CHOICES)
    status = tables.Column(
        "status",
        filters=(title, filters.replace_underscores),
        verbose_name=_("Status"),
        status=True,
        status_choices=STATUS_CHOICES,
        display_choices=constants.VM_STATUS_DISPLAY_CHOICES)
    state = tables.Column(project_tables.get_power_state,
                          filters=(title, filters.replace_underscores),
                          verbose_name=_("Power State"),
                          display_choices=project_tables.POWER_DISPLAY_CHOICES)

    # noinspection PyTypeChecker
    class Meta(object):
        name = "instances"
        verbose_name = _("Instances")
        status_columns = ["status", "task"]
        row_class = project_actions.UpdateRow
        table_actions = (admin_actions.TnxAdminInstanceFilterAction, )
        table_actions_menu = (
            admin_actions.TnxDeleteInstance,
            admin_actions.ResetStateAction,
            InstanceSchedulerAction,
            admin_actions.CreateImageBatchAction,
            project_actions.TnxMultiToggleShelve,
            project_actions.TnxMultiToggleUnShelve,
            project_actions.StartInstance,
            project_actions.RebootInstance,
            admin_actions.AdminEditInstance)

        row_actions = (
            admin_actions.InstanceExtraAction,
            project_actions.ConfirmResize,
            project_tables.RevertResize,
            project_tables.CreateSnapshot,
            admin_actions.TnxAdminConsoleLink,
            admin_actions.TnxAdminLogLink,
            project_actions.TnxTogglePause,
            project_actions.ToggleSuspend,
            project_actions.TnxToggleShelve,
            admin_actions.TnxMigrateInstance,
            admin_actions.TnxLiveMigrateInstance,
            project_actions.SoftRebootInstance,
            project_actions.RebootInstance,
            admin_actions.TnxDeleteInstance,
            admin_actions.ResetStateAction,
            InstanceSchedulerAction,
            admin_actions.CreateImageBatchAction,
            admin_actions.TnxStopInstance,
            InstanceShowMetricAction,
            project_actions.StartInstance,
            admin_actions.AdminEditInstance)
        if getattr(settings, 'LAUNCH_INSTANCE_LEGACY_ENABLED', False):
            row_actions += (project_actions.CloneLink,)
        if getattr(settings, 'LAUNCH_INSTANCE_NG_ENABLED', True):
            row_actions += (project_actions.CloneNGLink,)


class AdminHypervisorInstancesTable(TnxPaginationDataTable):
    tenant = tables.Column("tenant_name", verbose_name=_("Project"))

    name = tables.Column("name",
                         link="horizon:admin:instances:detail",
                         verbose_name=_("Name"))
    type = tables.Column(lambda x: unicode(getattr(x, "tnx_type", '')),
                         verbose_name=_("Type"),
                         display_choices=VMTypes.TYPES.items())
    ip = tables.Column(project_tables.get_ips,
                       verbose_name=_("IP Address"),
                       attrs={'data-type': "ip"})
    size = tables.Column(project_tables.get_size,
                         verbose_name=_("Size"),
                         attrs={'data-type': 'size'})
    status = tables.Column(
        "status",
        filters=(title, filters.replace_underscores),
        verbose_name=_("Status"),
        status=True,
        status_choices=STATUS_CHOICES,
        display_choices=constants.VM_STATUS_DISPLAY_CHOICES)
    task = tables.Column("OS-EXT-STS:task_state",
                         verbose_name=_("Task"),
                         empty_value=project_tables.TASK_DISPLAY_NONE,
                         status=True,
                         status_choices=TASK_STATUS_CHOICES,
                         display_choices=constants.VM_TASK_DISPLAY_CHOICES)
    state = tables.Column(project_tables.get_power_state,
                          filters=(title, filters.replace_underscores),
                          verbose_name=_("Power State"),
                          display_choices=project_tables.POWER_DISPLAY_CHOICES)

    @memoized.memoized_method
    def get_extra_pagination_string(self):
        tab = self.request.GET.get('tab')
        return 'tab={0}'.format(tab) if tab else ''

    class Meta(object):
        name = "hypervisor_instances"
        verbose_name = _("Hypervisor Instances")
        status_columns = ["status", "task"]
        row_class = project_tables.UpdateRow
        multi_select = True
        table_actions = (admin_actions.TnxHypervisorInstancesFilterAction,)
        table_actions_menu = (InstanceSchedulerAction,
                              project_tables.StartInstance,
                              project_actions.TnxStopInstance,
                              project_tables.SoftRebootInstance,
                              project_actions.TnxMultiTogglePause,
                              project_actions.TnxMultiToggleUnPause,
                              project_actions.TnxMultiToggleSuspend,
                              project_actions.TnxMultiToggleResume,
                              project_tables.RebootInstance,
                              admin_actions.TnxHypervisorLiveMigrateInstance,
                              admin_actions.TnxMigrateInstance)


# noinspection PyClassHasNoInit
class AdminInstancesTable(admin_tables.AdminInstancesTable):
    class Meta(object):
        name = "instances"
        verbose_name = _("Instances")
        status_columns = ["status", "task"]
        row_class = admin_tables.AdminUpdateRow
        table_actions = (project_actions.TnxDeleteInstance,
                         admin_tables.AdminInstanceFilterAction)
        table_actions_menu = (InstanceSchedulerAction,)
        row_actions = (project_actions.ConfirmResize,
                       project_tables.RevertResize,
                       admin_tables.AdminEditInstance,
                       admin_actions.TnxAdminConsoleLink,
                       admin_actions.TnxAdminLogLink,
                       project_tables.CreateSnapshot,
                       project_tables.TogglePause,
                       project_tables.ToggleSuspend,
                       project_tables.ToggleShelve,
                       admin_tables.MigrateInstance,
                       admin_tables.LiveMigrateInstance,
                       project_tables.SoftRebootInstance,
                       project_tables.RebootInstance,
                       project_actions.TnxDeleteInstance,
                       InstanceShowMetricAction,
                       InstanceSchedulerAction)
