# coding: utf-8
# pylint: disable=no-member

import datetime
import mock

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, messages
from horizon.utils import filters

from openstack_dashboard import api

from requests.exceptions import ConnectionError

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.admin.instances import (
    actions, forms, tables, tabs, views, workflows)
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.tests.fake_cloud_state import (
    ALL_SERVERS_COUNT, cloud_state)
from tionix_dashboard.tests.helpers import (
    FakeUser, BaseAdminClientTestCase, patch_resource_get)


INDEX_URL = reverse('horizon:admin:instances:index')


class TnxAdminInstanceViewActionsTest(BaseAdminClientTestCase):

    def test_create_image_batch_action(self):
        for instance in self.tnx_servers.list():
            action = actions.CreateImageBatchAction()

            self.assertEqual(unicode(_(u'Create image')),
                             action.action_present(None))

            self.assertEqual(unicode(_(u'Create image')),
                             action.action_past(None))

            self.assertIsNone(action.action(self.request, instance.id))

    def test_extra_action(self):
        for instance in self.tnx_servers.list():
            action = actions.InstanceExtraAction()

            reset_action_url = reverse('horizon:admin:instances:extra',
                                       args=[instance.id])

            self.assertEqual(reset_action_url, action.get_link_url(instance))

    def test_tnx_migrate_action(self):
        instance1 = self.tnx_servers.list()[0]
        instance3 = self.tnx_servers.list()[2]

        action = actions.TnxMigrateInstance()

        self.assertTrue(action.allowed(self.request, instance1))

        self.assertFalse(action.allowed(self.request, instance3))

    @mock.patch.object(TNXServersClient, 'update', mock.Mock())
    def test_update_cell_action(self):
        for instance in self.tnx_servers.list():
            action = actions.InstanceUpdateCell()

            self.assertTrue(
                action.update_cell(self.request, instance, instance.id, None,
                                   None)
            )

    def test_check_console_action(self):
        """Check that ConsoleAction and ConsoleTab points to the same place."""
        for instance in self.tnx_servers.list():
            table = tables.TnxAdminInstancesTable(self.request)

            console_action = actions.TnxAdminConsoleLink(table=table)

            tab_query_string = tabs.TnxConsoleTab(
                tabs.TnxInstanceDetailTabs).get_query_string()
            action_link = console_action.get_link_url(instance).split('?')[1]
            self.assertEqual(tab_query_string, action_link)


class TnxAdminInstanceViewFormsTest(BaseAdminClientTestCase):

    def test_instance_extra_form(self):

        created_date = (datetime.datetime.today() -
                        datetime.timedelta(days=1)).isoformat()

        form = forms.InstanceExtra(
            self.request,
            initial={'created': created_date}
        )

        self.assertListEqual(
            form.fields.keys(),
            ['image_name', 'created', 'size']
        )

        self.assertEqual(
            form.fields['created'].value,
            filters.timesince_sortable(filters.parse_isotime(created_date))
        )

    @mock.patch.object(api.nova, 'server_live_migrate', mock.Mock())
    @mock.patch.object(messages, 'success')
    @mock.patch.object(messages, 'error')
    def test_live_migrate_form(self, error, success):
        for instance in self.tnx_servers.list():
            hosts = self.hosts.list()

            form = forms.TnxLiveMigrateForm(
                self.request,
                initial={'instances': [instance],
                         'hosts': hosts}
            )

            form.handle(self.request, data={'host': '',
                                            'block_migration': True,
                                            'disk_over_commit': True})
            self.assertTrue(success.called)
            form.handle(self.request, data={})
            self.assertTrue(error.called)


class TnxAdminInstanceViewTablesTest(BaseAdminClientTestCase):

    def test_tnx_instances_columns(self):

        table = tables.TnxAdminInstancesTable(self.request)

        self.assertListEqual(
            table.columns.keys(),
            ['multi_select', 'tenant', 'name', 'host', 'node_title',
             'type', 'ip', 'task', 'status', 'state', 'actions']
        )

    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_instances_actions(self):
        instances = self.servers.list()

        table = tables.TnxAdminInstancesTable(self.request, data=instances)

        self.assertListEqual(
            [action.name for action in table.get_table_actions()],
            ['filter_admin_instances', 'delete', 'reset_state',
             'instance_schedule', 'create_image', 'tnx_shelve',
             'tnx_unshelve', 'start', 'reboot', 'edit']
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    def test_tnx_instances_row_actions(self):
        table = tables.TnxAdminInstancesTable(self.request)

        instance = self.tnx_servers.list()[0]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instances', 'snapshot', 'edit', 'console', 'log', 'pause',
                    'suspend', 'shelve', 'migrate', 'live_migrate',
                    'soft_reboot', 'reboot', 'delete', 'instance_schedule',
                    'create_image', 'stop', 'show_instance_metrics',
                    'clone-ng'])
        )

        instance = self.tnx_servers.list()[2]
        self.assertListEqual(
            sorted([action.name for action
                    in table.get_row_actions(instance)]),
            sorted(['instances', 'edit', 'delete', 'reset_state',
                    'instance_schedule', 'create_image',
                    'show_instance_metrics'])
        )

        instance = self.tnx_servers.list()[3]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instances', 'edit', 'pause', 'delete', 'snapshot',
                    'instance_schedule', 'create_image',
                    'show_instance_metrics', 'clone-ng'])
        )

        instance = self.tnx_servers.list()[5]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instances', 'edit', 'shelve', 'delete',
                    'instance_schedule', 'create_image',
                    'show_instance_metrics', 'clone-ng'])
        )

        instance = self.tnx_servers.list()[8]
        self.assertListEqual(
            sorted([action.name for action
                    in table.get_row_actions(instance)]),
            sorted(['instances', 'confirm', 'revert', 'edit', 'delete',
                    'instance_schedule', 'create_image',
                    'show_instance_metrics', 'clone-ng'])
        )

        shutoff_instance = self.tnx_servers.list(
            search_opts={'status': 'SHUTOFF'})[0]

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(shutoff_instance)]),
            sorted(['instances', 'snapshot', 'edit', 'migrate', 'soft_reboot',
                    'reboot', 'delete', 'instance_schedule', 'create_image',
                    'show_instance_metrics', 'start', 'clone-ng'])
        )

        # instances without tnx fields

        instance = self.tnx_servers.list()[10]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instances', 'snapshot', 'edit', 'console', 'log', 'pause',
                    'suspend', 'shelve', 'migrate', 'live_migrate',
                    'soft_reboot', 'reboot', 'delete', 'instance_schedule',
                    'create_image', 'stop', 'show_instance_metrics',
                    'clone-ng'])
        )

        instance = self.tnx_servers.list()[11]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instances', 'edit', 'pause', 'delete', 'snapshot',
                    'instance_schedule', 'create_image',
                    'show_instance_metrics', 'clone-ng'])
        )

    def test_hyp_instances_columns(self):

        table = tables.AdminHypervisorInstancesTable(self.request)

        self.assertListEqual(
            table.columns.keys(),
            ['multi_select', 'tenant', 'name', 'type', 'ip', 'size',
             'status', 'task', 'state']
        )

    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(LicenseController, 'is_valid',
                       mock.Mock(return_value=True))
    def test_hyp_instances_actions(self):
        instances = self.servers.list()

        table = tables.AdminHypervisorInstancesTable(self.request,
                                                     data=instances)
        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['instance_schedule', 'start', 'stop', 'soft_reboot',
                    'tnx_pause', 'tnx_unpause', 'tnx_suspend', 'tnx_resume',
                    'reboot', 'live_migrate', 'migrate',
                    'filter_hyp_instances'])
        )

    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_instances_actions(self):
        instances = self.servers.list()

        table = tables.AdminInstancesTable(self.request, data=instances)

        self.assertListEqual(
            [action.name for action in table.get_table_actions()],
            ['delete', 'filter_admin_instances', 'instance_schedule']
        )

    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_instances_row_actions(self):
        table = tables.AdminInstancesTable(self.request)

        instance = self.servers.list()[0]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['console', 'delete', 'edit', 'instance_schedule',
                    'live_migrate', 'log', 'migrate', 'pause', 'reboot',
                    'shelve', 'show_instance_metrics', 'snapshot',
                    'soft_reboot', 'suspend'])
        )

        instance = self.servers.list()[1]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['delete', 'edit', 'instance_schedule',
                    'show_instance_metrics'])
        )

        instance = self.servers.list()[3]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['edit', 'snapshot', 'pause', 'delete',
                    'show_instance_metrics', 'instance_schedule'])
        )


class TnxAdminInstanceViewTabsTest(BaseAdminClientTestCase):

    @mock.patch.object(LicenseController, 'is_valid')
    @mock.patch('tionix_dashboard.dashboards.tionix.metrics.tabs.'
                'is_bareos_available', mock.Mock(return_value=True))
    def test_instance_detail_tabs(self, is_valid):
        is_valid.return_value = True

        tab_group = tabs.InstanceDetailTabs(self.request)

        self.assertListEqual(
            [tab.slug for tab in tab_group.get_tabs()],
            ['overview', 'log', 'console', 'audit', 'metrics',
             'bareos_templates_tab']
        )


class TnxAdminInstanceViews(BaseAdminClientTestCase):

    @mock.patch.object(api.network, 'servers_update_addresses', mock.Mock())
    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(api.glance, 'image_list_detailed')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.keystone.VERSIONS, 'get_project_manager')
    def test_tnx_inst_index_get_data(
            self, get_project_manager, flavor_list,
            flavor_get, image_list_detailed):
        tenants = self.tenants.list()
        flavors = self.tnx_flavors.list()
        images = self.tnx_images.list()
        instance = self.tnx_servers.list()[0]

        fake_project_manager = mock.Mock()
        fake_project_manager.list.return_value = tenants
        get_project_manager.return_value = fake_project_manager
        flavor_list.return_value = flavors
        image_list_detailed.return_value = images
        flavor_get.side_effect = patch_resource_get(self.tnx_flavors)

        res = self.client.get(INDEX_URL)

        table = res.context_data['table']

        instance_cells = [
            unicode(v.value) for k, v in table.get_rows()[0].cells.items()
            if k not in ('actions', 'multi_select')]

        self.assertEqual(table.total_count, ALL_SERVERS_COUNT)

        self.assertListEqual(
            instance_cells,
            [u'<a href="/identity/1/detail/" >test_tenant</a>',
             u'<a href="/admin/instances/%s/detail" >'
             u'fake_instance</a>' % instance.id,
             u'<a href="/admin/hypervisors/10/" >Host1</a>',
             u'Node1', u'Typical', u'\n\n', u'None', u'Active', u'Running']
        )

    def test_inst_extra_get_object(self):
        for instance in self.tnx_servers.list():
            view = views.InstanceExtraView()

            view.request = self.request
            view.kwargs = {
                'instance_id': instance.id
            }

            instance_from_view = view.get_object()

            self.assertIsNotNone(instance_from_view)

            self.assertEqual(instance.id, instance_from_view.id)

    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.glance, 'image_get')
    def test_inst_extra_get_initial(self, image_get, flavor_get):
        for instance in self.tnx_servers.list():
            flavor = self.tnx_flavors.get(instance.flavor['id'])
            image = self.tnx_images.get(instance.image['id'])
            flavor_get.return_value = flavor
            image_get.return_value = image
            instance.full_flavor = flavor

            view = views.InstanceExtraView()

            view.request = self.request
            view.kwargs = {
                'instance_id': instance.id
            }
            task_name = unicode(_('None'))
            image_name = [image for image in cloud_state['images']
                          if instance.image['id'] == image['id']][0]['name']
            self.assertDictEqual(
                view.get_initial(),
                {
                    'created': instance.created,
                    'image_name': image_name,
                    'size': views.get_size(instance),
                    'task': task_name,
                }
            )

    @mock.patch.object(messages, 'error', mock.Mock())
    def test_tnx_lv_mgrt_get_objects(self):

        view = views.TnxLiveMigrateView()
        instance = self.tnx_servers.list()[0]
        view.request = get_fake_request(
            get_params={
                'id': [instance.id, 'non-existing']
            }
        )

        self.assertListEqual(
            [obj.id for obj in view.get_objects()],
            [instance.id],
        )

    @mock.patch.object(api.nova, 'service_list')
    def test_tnx_lv_mgrt_get_initial(self, service_list):

        service_list.return_value = [Empty(host='Host2', zone='nova')]
        instance_id_first = self.tnx_servers.list()[0].id

        for instance in self.tnx_servers.list():
            view = views.TnxLiveMigrateView()
            view.request = get_fake_request(
                user=FakeUser(),
                get_params={
                    'id': [instance_id_first]
                }
            )
            view.kwargs = {
                'instance_id': instance.id
            }

            initial = view.get_initial()

            self.assertListEqual(
                sorted(initial.keys()),
                sorted(['instances', 'current_host', 'hypervisors', 'id'])
            )

    @mock.patch.object(api.nova, 'service_list')
    def test_tnx_lv_mgrt_get_initial_no_hosts(self, service_list):
        instance = self.tnx_servers.list()[0]

        service_list.return_value = []

        view = views.TnxLiveMigrateView()
        view.request = get_fake_request(
            user=FakeUser(),
            get_params={
                'id': [instance.id]
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            META={'HTTP_REFERER': ''}
        )
        view.kwargs = {
            'instance_id': instance.id
        }

        with self.assertRaises(exceptions.Http302):
            view.get_initial()

    @mock.patch.object(api.nova, 'service_list')
    def test_tnx_lv_mgrt_get_hosts(self, service_list):
        instance = self.tnx_servers.list()[0]
        service_list.return_value = [Empty(host='Host2', zone='nova')]

        view = views.TnxLiveMigrateView()
        view.request = get_fake_request(
            user=FakeUser(),
            get_params={
                'id': ['1']
            }
        )
        view.kwargs = {
            'instance_id': instance.id
        }

        hosts = view.get_hosts(
            getattr(instance, 'OS-EXT-SRV-ATTR:host', ''),
            instance.availability_zone)

        self.assertEqual(len(hosts), 1)

    def test_tnx_lv_mgrt_success_url(self):
        view = views.TnxLiveMigrateView()
        view.request = get_fake_request(
            user=FakeUser(),
            get_params={
                'hypervisor_id': 1
            }
        )
        self.assertEqual(view.get_success_url(), u'/admin/hypervisors/1/')

    @mock.patch.object(api.network, 'servers_update_addresses', mock.Mock())
    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(api.glance, 'image_list_detailed')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.keystone.VERSIONS, 'get_project_manager')
    @mock.patch.object(messages, 'error')
    @mock.patch.object(TNXServersClient, 'get_paginator')
    def test_tnx_inst_index_get_data_nova_unavailable(
            self, get_paginator, error, get_project_manager, flavor_list,
            flavor_get, image_list_detailed):
        tenants = self.tenants.list()
        flavors = self.tnx_flavors.list()
        images = self.tnx_images.list()

        get_paginator.side_effect = ConnectionError()

        fake_project_manager = mock.Mock()
        fake_project_manager.list.return_value = tenants
        get_project_manager.return_value = fake_project_manager
        flavor_list.return_value = flavors
        image_list_detailed.return_value = images
        flavor_get.side_effect = patch_resource_get(self.tnx_flavors)

        request = get_fake_request(user=FakeUser(), session={}, COOKIES={})

        view = views.TnxAdminInstancesIndexView()
        view.table = tables.TnxAdminInstancesTable(request)
        view.request = request
        view.get_data()
        self.assertTrue(error.called)


class ProjectInstanceWorkflowsTest(BaseAdminClientTestCase):
    @mock.patch('tionix_dashboard.dashboards.project.project_instances.'
                'workflows.get_security_groups_by_instance',
                mock.Mock(return_value=[]))
    @mock.patch('tionix_dashboard.dashboards.project.project_instances.'
                'workflows.get_security_groups', mock.Mock(return_value=[]))
    def test_update_instance_tabs_test(self):

        workflow = workflows.AdminUpdateInstance(self.request)

        self.assertListEqual(
            sorted([step.action.slug for step in workflow.steps]),
            sorted(['instance_info', 'update_security_groups'])
        )
