# coding: utf-8
import logging

from django import template
from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.templatetags import sizeformat
from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.instances import views as admin_views
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)
from openstack_dashboard.dashboards.project.instances.tables import (
    TASK_DISPLAY_NONE)
from tionix_dashboard.clients import TNXHypervisorsClient

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.admin.hypervisors.states import STATE_UP
from tionix_dashboard.dashboards.admin.instances.helpers import (
    get_instances_for_admin)
from tionix_dashboard.dashboards.admin.instances.tabs import (
    InstanceDetailTabs, TnxInstanceDetailTabs)
from tionix_dashboard.dashboards.admin.instances.workflows import (
    AdminUpdateInstance)
from tionix_dashboard.helpers.utils import get_resource_template
from tionix_dashboard.helpers.views import (
    FilterStringModifierMixin, RefererRedirectMixin, DTVTnxPaginationMixin,
    LinkBatchFormView)
from tionix_dashboard.dashboards.admin.instances.forms import (
    InstanceExtra, TnxLiveMigrateForm, DeleteInstanceForm)
from tionix_dashboard.dashboards.admin.instances.tables import (
    TnxAdminInstancesTable, AdminInstancesTable)
from tionix_dashboard.dashboards.project.project_instances import views
from tionix_dashboard.url_resolve_map import (
    ADMIN_SERVERS_INDEX, ADMIN_SERVERS_DETAIL)

logger = logging.getLogger(__name__)


class TnxAdminInstancesIndexView(FilterStringModifierMixin,
                                 DTVTnxPaginationMixin,
                                 admin_views.AdminIndexView):

    table_class = TnxAdminInstancesTable
    template_name = 'instances/index.html'

    def get_data(self):
        offset, limit = self.get_offset_limit()
        search_opts = self.get_filters({'sort_dirs': ('asc',),
                                        'sort_keys': ('created_at',)})

        instances = get_instances_for_admin(
            self.request,
            search_opts.get('sort_dir'),
            search_opts)

        total_count = len(instances)
        range_end = offset + limit
        instances = instances[offset:range_end]

        self.set_table_footer(offset, limit, total_count)

        return instances


class InstanceExtraView(forms.ModalFormView):
    """View represents instance's extra information"""
    form_class = InstanceExtra
    modal_header = _('Instance extra information')
    template_name = 'instances/extra.html'
    modal_id = "show_instances_extra_info"
    page_title = _("Instance extra information")
    cancel_label = _('Close')
    instance_fields = ['image_name', 'created']
    success_url = reverse_lazy(ADMIN_SERVERS_INDEX)

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_object(self, *args, **kwargs):
        instance, instance_id = None, self.kwargs['instance_id']
        try:
            instance = TNXServersClient.get(self.request, instance_id)
        except Exception as err:
            logger.error(err.message)
            msg = _('Unable to retrieve instance details.')
            exceptions.handle(self.request, msg)

        return instance

    def get_initial(self):
        instance, initial = self.get_object(), {}
        instance.full_flavor = api.nova.flavor_get(
            self.request, instance.flavor['id'])
        if instance is not None:
            initial = {
                field: getattr(instance, field, '') or ''
                for field in self.instance_fields
            }
            initial['size'] = get_size(instance)
            initial['task'] = _(
                getattr(instance, 'OS-EXT-STS:task_state',
                        TASK_DISPLAY_NONE) or u'None')
        return initial


class TnxDetailView(views.TnxDetailView):
    redirect_url = ADMIN_SERVERS_DETAIL
    scheduler_tab_url = 'horizon:admin:instances:scheduler_tab'
    tab_group_class = TnxInstanceDetailTabs
    template_name = "horizon/common/_detail.html"
    image_url = 'horizon:admin:images:detail'
    volume_url = 'horizon:admin:volumes:volumes:detail'
    actions_table = TnxAdminInstancesTable
    failure_url = ADMIN_SERVERS_INDEX


class TnxLiveMigrateView(RefererRedirectMixin, LinkBatchFormView):
    form_class = TnxLiveMigrateForm
    template_name = 'instances/live_migrate.html'
    context_object_name = 'instance'
    success_url = reverse_lazy(ADMIN_SERVERS_INDEX)
    page_title = _("Live Migrate")
    failure_msg = _('"Live Migrate" is not available for instances: ')

    @memoized.memoized_method
    def get_hosts(self, current_host, availability_zone):
        result = []
        try:
            hypervisors = TNXHypervisorsClient.list(self.request)
            hosts = api.nova.service_list(self.request, binary='nova-compute')
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve hypervisor information.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())

        else:
            hosts = {host.host: host.zone for host in hosts}

            for hypervisor in hypervisors:
                if (hypervisor.state != STATE_UP or
                        hypervisor.hypervisor_hostname == current_host or
                        not hypervisor.service or
                        (hosts.get(hypervisor.service['host']) !=
                         availability_zone)):
                    continue
                result.append({
                    "id": hypervisor.id,
                    "hostname": hypervisor.hypervisor_hostname,
                    "tnx_node_title": hypervisor.tnx_node_title or '-',
                    "availability_zone": hosts.get(hypervisor.service['host']),
                    "vcpus": get_resource_template(hypervisor.vcpus,
                                                   hypervisor.vcpus_used),
                    'ram': get_resource_template(hypervisor.memory_mb,
                                                 hypervisor.memory_mb_used),
                    'disk': get_resource_template(hypervisor.local_gb,
                                                  hypervisor.local_gb_used)
                })

            if not result:
                messages.error(self.request,
                               _('Available hosts are not found.'))
                raise exceptions.Http302(self.get_redirect_url())

        return result

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_objects(self, *args, **kwargs):
        instances, failed = [], []
        for instance_id in self.get_object_ids():
            try:
                instance = TNXServersClient.get(self.request, instance_id)
            except Exception as err:
                logger.error(err.message)
                continue

            if not (instance.status in project_tables.ACTIVE_STATES and
                    not project_tables.is_deleting(instance)):
                failed.append(instance.name)
            else:
                instances.append(instance)

        if failed:
            messages.error(self.request, u'{0} {1}'.format(
                self.failure_msg, u','.join(failed)))

        return instances

    def get_initial(self):
        initial = super(TnxLiveMigrateView, self).get_initial()
        instances = self.get_objects()
        if instances:
            first_instance = instances[0]
            current_host = getattr(first_instance, 'OS-EXT-SRV-ATTR:host', '')
            availability_zone = first_instance.availability_zone
            initial.update({
                'instances': instances,
                'current_host': current_host,
                'hypervisors': self.get_hosts(current_host, availability_zone)
            })
        return initial

    def get_success_url(self):
        hypervisor_id = self.request.GET.get('hypervisor_id')
        return (reverse('horizon:admin:hypervisors:detail',
                        args=[hypervisor_id])
                if hypervisor_id
                else super(TnxLiveMigrateView, self).get_success_url())


class AdminIndexView(admin_views.AdminIndexView):
    table_class = AdminInstancesTable


class DetailView(admin_views.DetailView):
    scheduler_tab_url = 'horizon:admin:instances:scheduler_tab'
    tab_group_class = InstanceDetailTabs

    def get_tabs(self, request, *args, **kwargs):
        instance = self.get_data()
        kwargs['obj_id'] = instance.id
        kwargs['instance'] = instance
        kwargs['scheduler_tab_url'] = reverse(self.scheduler_tab_url,
                                              args=(instance.id,))
        return self.tab_group_class(request, **kwargs)


def get_size(instance):
    if hasattr(instance, "full_flavor"):
        template_name = 'instances/_tnx_instance_flavor.html'
        size_ram = sizeformat.mb_float_format(instance.full_flavor.ram)
        if instance.full_flavor.disk > 0:
            size_disk = sizeformat.diskgbformat(instance.full_flavor.disk)
        else:
            size_disk = _("%s GB") % "0"
        context = {
            "name": instance.full_flavor.name,
            "size_disk": size_disk,
            "size_ram": size_ram,
            "vcpus": instance.full_flavor.vcpus,
            "flavor_id": instance.full_flavor.id
        }
        return template.loader.render_to_string(template_name, context)
    return _("Not available")


class FullScreenConsoleView(views.FullScreenConsoleView):
    return_url = ADMIN_SERVERS_DETAIL


class ResetStateView(views.ResetStateView):
    redirect_url = success_url = reverse_lazy("horizon:admin:instances:index")
    confirm_url = reverse_lazy("horizon:admin:instances:reset_state")


class AdminUpdateView(views.UpdateView):
    workflow_class = AdminUpdateInstance
    success_url = reverse_lazy(ADMIN_SERVERS_INDEX)
    submit_url = reverse_lazy('horizon:admin:instances:update')


class DeleteInstanceView(views.DeleteInstanceView):
    """ Delete instances view """
    form_class = DeleteInstanceForm
    success_url = reverse_lazy("horizon:admin:instances:index")
    confirm_url = reverse_lazy("horizon:admin:instances:delete")
