# coding: utf-8

from django.utils.translation import ugettext_lazy as _

from keystoneauth1 import exceptions as keystone_exceptions

from keystoneclient.v2_0 import client as v2_client
from keystoneclient.v3 import client as v3_client

from openstack_auth import exceptions, plugin as os_auth_plugins


class TnxPluginMixin(object):

    def list_projects(self, session, auth_plugin, auth_ref=None):
        """List the projects that are accessible to this plugin
        (+ filtering by auth_ref domain)

        Query the keystone server for all projects that this authentication
        token can be rescoped to.

        This function is overrideable by plugins if they use a non-standard
        mechanism to determine projects.

        :param session: A session object for communication:
        :type session: keystoneclient.session.Session
        :param auth_plugin: The auth plugin returned by :py:meth:`get_plugin`.
        :type auth_plugin: keystoneclient.auth.BaseAuthPlugin
        :param auth_ref: The current authentication data. This is optional as
                         future auth plugins may not have auth_ref data and all
                         the required information should be available via the
                         auth_plugin.
        :type auth_ref: keystoneclient.access.AccessInfo` or None.

        :raises: exceptions.KeystoneAuthException on lookup failure.

        :returns: A list of projects. This currently accepts returning both v2
                  or v3 keystoneclient projects objects.
        """
        try:
            if self.keystone_version >= 3:
                client = v3_client.Client(session=session, auth=auth_plugin)
                if auth_ref.is_federated:
                    return client.federation.projects.list()
                else:
                    return client.projects.list(
                        user=auth_ref.user_id,
                        domain=getattr(auth_ref, 'user_domain_id', None)
                    )

            else:
                client = v2_client.Client(session=session, auth=auth_plugin)
                return client.tenants.list()

        except (keystone_exceptions.ClientException,
                keystone_exceptions.AuthorizationFailure):
            msg = _('Unable to retrieve authorized projects.')
            raise exceptions.KeystoneAuthException(msg)


class TnxTokenPlugin(TnxPluginMixin, os_auth_plugins.TokenPlugin):
    pass


class TnxPasswordPlugin(TnxPluginMixin, os_auth_plugins.PasswordPlugin):
    pass
