# coding: utf-8

import logging
import sys

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, Panel

from openstack_dashboard import policy, api

from tionix_client.os_clients.keystone_client import get_admin_keystone_client
from tionix_client.helpers.journal import log_project_action

from tionix_dashboard.clients import TNXSessionsClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.journal import TNXJournalClient
from tionix_dashboard.clients.servers import TnxServer
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.helpers.vm_constants import ConnectionTypeEnum


logger = logging.getLogger(__name__)


class Empty(object):

    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)


class TionixPanel(Panel):
    """Extended panel with license check"""

    lic_type = None

    def __init__(self):
        super(TionixPanel, self).__init__()

        if self.lic_type:
            LicenseController.add_panel(self.lic_type,
                                        'tionix',
                                        self.__class__)

    def can_access(self, context):
        result = super(TionixPanel, self).can_access(context)
        if self.lic_type:
            request = context['request']
            result = result and LicenseController.is_valid(request,
                                                           self.lic_type)
        return result


def get_common_journal_data(request, **filters):
    journal = []
    try:
        journal = TNXJournalClient.list(
            request, limit=sys.maxint, **filters)
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _('Unable to retrieve common journal.'))

    return journal


def get_users_list(request, filters=None):
    users = []
    if policy.check((("identity", "identity:list_users"),), request):
        domain_context = api.keystone.get_effective_domain_id(request)
        try:
            users = api.keystone.user_list(request,
                                           domain=domain_context,
                                           filters=filters)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve user list.'))
    elif policy.check((("identity", "identity:get_user"),), request):
        try:
            user = api.keystone.user_get(request, request.user.id)

            users.append(user)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve user information.'))
    return users


def get_sessions_list(request, domain_id=None, limit=None, offset=None,
                      filters=None):
    sessions, total_count = [], None
    try:
        total_count = TNXSessionsClient.count(
            request, **filters
        )
        sessions = TNXSessionsClient.list(
            request,
            domain_id=domain_id,
            offset=offset,
            limit=limit,
            **filters
        )
    except UrlNotAvailableError:
        raise
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _('Unable to retrieve sessions list.'))

    return sessions, total_count


def get_connection_type_by_name(value):
    return [key for key, val in ConnectionTypeEnum.values.items()
            if value.lower() in val.lower()] or [None]


def coerce_to_int(value):
    # noinspection PyTypeChecker
    return (int(value)
            if isinstance(value, basestring) and value.isdigit()
            else value)


class CustomUnicodeString(unicode):
    """String that remembers it's encoding to propperly encode."""

    def __init__(self, string, encoding='utf-8', **kwargs):
        self.encoding = encoding
        super(CustomUnicodeString, self).__init__(string, encoding, **kwargs)

    def __str__(self):
        return self.encode()

    # noinspection PyMethodOverriding
    # pylint: disable=arguments-differ
    def encode(self, errors='strict'):
        return super(CustomUnicodeString, self).encode(self.encoding, errors)


def get_instances_list_by_domain(request, project_id=None, search_opts=None):
    instances = []
    search_opts = search_opts or {}

    domain_id = api.keystone.get_default_domain(request,
                                                get_name=False).get('id')
    search_opts.update({'domain': domain_id})
    try:
        instances = novaclient(request, project_id=project_id).servers.list(
            search_opts=search_opts
        )
    except UrlNotAvailableError:
        raise
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _('Unable to retrieve instance list.'))

    return [TnxServer(instance, request) for instance in instances]


def get_projects_assignments_roles_by_group(request, group_id):
    group_assignments = []
    if api.keystone.is_cloud_admin(request):
        group_assignments = api.keystone.role_assignments_list(
            request, group=group_id, include_subtree=False
        )
    elif api.keystone.is_domain_admin(request):
        keystone_client = get_admin_keystone_client()
        group_assignments = keystone_client.role_assignments.list(
            group=group_id
        )
    return group_assignments


def get_projects_assignments_roles_by_user(request, user_id):
    group_assignments = []
    if api.keystone.is_cloud_admin(request):
        group_assignments = api.keystone.role_assignments_list(
            request, user=user_id, include_subtree=False
        )
    elif api.keystone.is_domain_admin(request):
        keystone_client = get_admin_keystone_client()
        group_assignments = keystone_client.role_assignments.list(
            user=user_id
        )
    return group_assignments


def get_users_in_project(request, project_id):
    all_users = {user.id: user for user in get_users_list(request)}
    users_roles = api.keystone.get_project_users_roles(request, project_id)
    users = []
    for user_id in users_roles:
        if user_id in all_users:
            users.append(all_users[user_id])
        else:
            logger.info('User with id:"{0}" not found.'.format(user_id))
    return users


def log_project_login(tenant_id, user_id=None, user_name=None):
    log_project_action(
        object_id=tenant_id,
        message=u'User "{0}" was successfully '
                u'logged in.'.format(user_name),
        action='dashboard.user.login',
        user_id=user_id,
        user_name=user_name,
    )


def log_project_logout(tenant_id, user_id=None, user_name=None):
    log_project_action(
        object_id=tenant_id,
        message=u'User "{0}" was successfully '
                u'logged out.'.format(user_name),
        action='dashboard.user.logout',
        user_id=user_id,
        user_name=user_name,
    )
