# coding: utf-8
# pylint: disable=method-hidden

"""Tionix domains actions"""

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import exceptions
from horizon import messages
from horizon import tables

from openstack_dashboard import api, policy
from openstack_dashboard.api.keystone import is_multi_domain_enabled
from openstack_dashboard.dashboards.identity.domains import (
    constants as domain_constants, tables as domain_tables,
    workflows as domain_workflows)

from tionix_dashboard.clients.domain_config import TNXDomainConfigClient
from tionix_dashboard.dashboards.identity.domains import constants
from tionix_dashboard.dashboards.identity.domains.helpers import (
    get_domain_configuration)
from tionix_dashboard.dashboards.tionix.metrics.api import (
    is_bareos_available)
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.helpers.actions import TnxBatchActionMixin

logger = logging.getLogger()


class DomainEnabledAction(tables.UpdateAction):
    """ Action set enabled/disabled domain. """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """ Inline update of enabled-cell. """
        result = True
        if cell_name == 'enabled':
            try:
                api.keystone.domain_update(request,
                                           obj_id,
                                           enabled=new_cell_value)
            except Exception as err:
                result = False
                logger.error(unicode(err))
                messages.error(request, _('Unable to update domain.'))
        return result


class UpdateRow(tables.Row):
    """ Action update domain row in table row. """
    ajax = True

    def get_data(self, request, obj_id):
        """Retrieve domain object or None"""
        domain = None
        try:
            domain = api.keystone.domain_get(request, obj_id)
        except Exception as err:
            logger.error(err)
            exceptions.handle(
                request,
                _('Unable to get domain.'),
                redirect=reverse(constants.DOMAIN_INDEX_URL))
        return domain


class MultiDomainEnabledMixin(object):
    """Checks multi domain flag"""
    def allowed(self, request, domain):
        return is_multi_domain_enabled()


# noinspection PyAbstractClass
class AddConfigurationAction(MultiDomainEnabledMixin, tables.LinkAction):
    """Domain configuration add action."""
    name = 'add_config'
    verbose_name = _('Add configuration')
    url = constants.DOMAINS_EDIT_CONFIG_URL
    classes = ('ajax-modal',)

    policy_rules = (('identity', 'identity:create_domain_config'),)

    def allowed(self, request, domain):
        """Action is allowed if multi domain enabled and no config"""
        allowed = super(AddConfigurationAction, self).allowed(request, domain)
        return (get_domain_configuration(request, domain.id) is None and
                allowed)


# noinspection PyAbstractClass
class EditConfigurationAction(MultiDomainEnabledMixin, tables.LinkAction):
    """Domain configuration edit action."""
    name = 'edit_config'
    verbose_name = _('Edit configuration')
    url = constants.DOMAINS_EDIT_CONFIG_URL
    classes = ('ajax-modal',)

    policy_rules = (('identity', 'identity:update_domain_config'),)

    def allowed(self, request, domain):
        """Action is allowed if multi domain enabled and has config"""
        allowed = super(EditConfigurationAction, self).allowed(request, domain)
        return (get_domain_configuration(request, domain.id) is not None and
                allowed)


class DeleteDomainConfigurationAction(TnxBatchActionMixin,
                                      MultiDomainEnabledMixin,
                                      tables.DeleteAction):
    """Domain configuration delete action. """
    name = "delete_config"

    policy_rules = (('identity', 'identity:delete_domain_config'),)

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete configuration",
            u"Delete configurations",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted configuration",
            u"Deleted configurations",
            count
        )

    def allowed(self, request, domain):
        """Action is allowed for domains with configuration"""
        allowed = super(DeleteDomainConfigurationAction, self).allowed(request,
                                                                       domain)
        if domain:
            allowed &= get_domain_configuration(request, domain.id) is not None
        return allowed

    def delete(self, request, obj_id):
        """Deleting config"""
        TNXDomainConfigClient.delete(request, obj_id)


# pylint: disable=super-on-old-class
class UpdateDomainUsersAction(domain_workflows.UpdateDomainUsersAction):

    def __init__(self, request, *args, **kwargs):
        super(UpdateDomainUsersAction, self).__init__(request, *args, **kwargs)

        #  pylint:disable=no-member
        for field_name in self.fields:
            if field_name.startswith(self.slug):
                # sort by name
                self.fields[field_name].choices.sort(
                    key=lambda x: x[1].lower())

    class Meta(object):
        name = _("Domain Members")
        slug = domain_constants.DOMAIN_USER_MEMBER_SLUG


# noinspection PyAbstractClass,PyClassHasNoInit
class EditDomainLink(domain_tables.EditDomainLink):
    verbose_name = _("Edit domain")


# noinspection PyAbstractClass
class SetUpBackupAction(tables.LinkAction):
    """Domain setup backup action. """
    name = 'set_up_backup'
    verbose_name = _('Set Up Backup')
    url = constants.DOMAIN_SET_UP_BACKUP_URL
    classes = ('ajax-modal',)

    def allowed(self, request, datum):
        is_allowed = super(SetUpBackupAction, self).allowed(request, datum)
        if is_allowed:
            is_allowed = (
                is_bareos_available(request) and
                LicenseController.is_valid(
                    request,
                    LicenseController.MONITOR
                ) and (policy.check((("identity",
                                      "admin_and_matching_domain_id"),),
                                    request=request,
                                    target={'domain_id': datum.id}) or
                       api.keystone.is_cloud_admin(request))
            )
        return is_allowed
