# coding: utf-8

"""Tionix domains forms"""

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import forms
from horizon import messages
from tionix_dashboard.clients.backup_systems import TNXBackupSystemsClient

from tionix_dashboard.clients.domain_config import TNXDomainConfigClient
from tionix_dashboard.dashboards.helpers import coerce_to_int
from tionix_dashboard.dashboards.identity.domains.constants import (
    BackupSystemType)

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class EditDomainConfiguration(forms.SelfHandlingForm):
    """Domain configuration edit form"""

    url = forms.CharField(label='url', max_length=255)
    user = forms.CharField(label='user', max_length=255)
    user_name_attribute = forms.CharField(
        label='user_name_attribute',
        max_length=255)
    user_objectclass = forms.CharField(
        label='user_objectclass',
        max_length=255)
    user_tree_dn = forms.CharField(label='user_tree_dn', max_length=255)
    password = forms.CharField(
        label='password',
        widget=forms.PasswordInput(render_value=False),
        max_length=255)

    def handle(self, request, data):
        """Create/update domain configuration"""
        result, create = True, self.initial.get('create', True)
        action = (TNXDomainConfigClient.create
                  if create else TNXDomainConfigClient.update)
        try:
            action(request, self.initial['domain_id'], data)
        except Exception as err:
            logger.error(unicode(err))
            result = False
        else:
            if create:
                msg = _('Successfully added configuration.')
            else:
                msg = _('Successfully updated configuration.')
            messages.success(request, msg)
        return result


# noinspection PyAbstractClass
class SetUpBackupForm(forms.SelfHandlingForm):
    """Domain backup setup form"""

    backup_system = forms.TypedChoiceField(
        label=_("Backup system"),
        choices=BackupSystemType.TYPES.items(),
        coerce=coerce_to_int
    )
    hostname = forms.CharField(label=_('Hostname'), max_length=50)
    password = forms.CharField(
        label=_('Password'),
        widget=forms.PasswordInput(),
        max_length=50)
    private_key = forms.CharField(
        label=_("Private key"),
        widget=forms.widgets.Textarea(attrs={'rows': 6}))

    def __init__(self, request, *args, **kwargs):
        super(SetUpBackupForm, self).__init__(request, *args, **kwargs)
        self.is_create_action = not bool(self.initial.get('id'))
        self.fields['password'].required = self.is_create_action
        self.fields['private_key'].required = self.is_create_action

    def handle(self, request, data):
        """Setup backup"""
        result = True
        domain_id, backup_setup_id = (
            self.initial.get('domain_id'),
            self.initial.get('id')
        )
        try:
            if backup_setup_id:
                TNXBackupSystemsClient.update(request, backup_setup_id, **data)
            else:
                TNXBackupSystemsClient.create(request, domain_id, **data)
        except Exception as err:
            logger.error(unicode(err))
            result = False
            messages.error(request, _('Unable to update backup settings.'))
        else:
            messages.success(request,
                             _('Successfully updated backup settings.'))
        return result
