# coding: utf-8
# pylint: disable=no-member

import mock

from django.test import override_settings

from openstack_auth import policy as policy_backend

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients.domain_config import TNXDomainConfigClient
from tionix_dashboard.dashboards.identity.domains import actions, workflows
from tionix_dashboard.dashboards.identity.domains import forms
from tionix_dashboard.dashboards.identity.domains import tables, views
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, FakeUser, BaseAdminClientTestCase)


class DomainsTablesTest(BaseClientTestCase):

    def test_domains_table_actions(self):
        domains = self.domains.list()

        table = tables.DomainsTable(self.request, data=domains)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['create', 'delete', 'delete_config', 'disable', 'enable',
                    'filter'])
        )

    @override_settings(OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT=True)
    @mock.patch('tionix_dashboard.dashboards.identity.domains.actions.'
                'is_bareos_available', mock.Mock(return_value=True))
    def test_domains_table_row_actions(self):
        table = tables.DomainsTable(self.request)

        domain = self.tnx_domains.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(domain)]),
            sorted(['add_config', 'delete', 'disable', 'edit', 'groups',
                    'set_domain_context', 'users', 'set_up_backup']))

        domain = self.tnx_domains.list()[1]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(domain)]),
            sorted(['add_config', 'delete', 'enable', 'edit', 'groups',
                    'users', 'set_domain_context', 'set_up_backup']))


@override_settings(OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT=True)
class DomainsActionsTest(BaseClientTestCase):

    @mock.patch.object(TNXDomainConfigClient,
                       'get',
                       mock.Mock(side_effect=Exception))
    def test_add_config_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.AddConfigurationAction()

        self.assertEqual(action.allowed(self.request, domain), True)

    @mock.patch.object(TNXDomainConfigClient, 'get', mock.Mock())
    def test_add_config_not_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.AddConfigurationAction()

        self.assertEqual(action.allowed(self.request, domain), False)

    @mock.patch.object(TNXDomainConfigClient, 'get', mock.Mock())
    def test_edit_config_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.EditConfigurationAction()

        self.assertEqual(action.allowed(self.request, domain), True)

    @mock.patch.object(TNXDomainConfigClient,
                       'get',
                       mock.Mock(side_effect=Exception))
    def test_edit_config_not_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.EditConfigurationAction()

        self.assertEqual(action.allowed(self.request, domain), False)

    @mock.patch.object(TNXDomainConfigClient, 'get', mock.Mock())
    def test_delete_config_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.EditConfigurationAction()

        self.assertEqual(action.allowed(self.request, domain), True)

    @mock.patch.object(TNXDomainConfigClient,
                       'get',
                       mock.Mock(side_effect=Exception))
    def test_delete_config_not_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.EditConfigurationAction()

        self.assertEqual(action.allowed(self.request, domain), False)

    @override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
    def test_set_up_backup_not_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.SetUpBackupAction()

        self.assertEqual(action.allowed(self.request, domain), False)


@override_settings(OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT=True)
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class DomainsActionsAdminTest(BaseAdminClientTestCase):
    @mock.patch('tionix_dashboard.dashboards.identity.domains.actions.'
                'is_bareos_available', mock.Mock(return_value=True))
    def test_set_up_backup_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.SetUpBackupAction()

        self.assertEqual(action.allowed(self.request, domain), True)

        domain = self.tnx_domains.get('2')

        action = actions.SetUpBackupAction()

        self.assertEqual(action.allowed(self.request, domain), True)

    @mock.patch('tionix_dashboard.dashboards.identity.domains.actions.'
                'is_bareos_available', mock.Mock(return_value=False))
    def test_set_up_backup_not_allowed(self):
        domain = self.tnx_domains.get('1')

        action = actions.SetUpBackupAction()

        self.assertEqual(action.allowed(self.request, domain), False)


class DomainsViewsTest(BaseClientTestCase):

    @mock.patch.object(TNXDomainConfigClient,
                       'get',
                       mock.Mock(side_effect=Exception))
    def test_edit_config_view_1(self):

        domain = self.tnx_domains.get('1')

        view = views.EditConfigurationView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        view.kwargs = {'domain_id': domain.id}

        self.assertEqual(view.get_initial(),
                         {'domain_id': domain.id})

    @mock.patch.object(TNXDomainConfigClient, 'get')
    def test_edit_config_view_2(self, config_get):

        domain = self.tnx_domains.get('1')

        config_data = Empty(ldap={'config': 'data'})

        config_get.return_value = config_data

        view = views.EditConfigurationView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        view.kwargs = {'domain_id': domain.id}

        self.assertEqual(view.get_initial(),
                         {'domain_id': domain.id,
                          'create': False,
                          'config': 'data'})


class DomainsFormsTest(BaseClientTestCase):

    @mock.patch.object(TNXDomainConfigClient, 'create', mock.Mock())
    def test_create_configuration(self):

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.EditDomainConfiguration(request)
        form.initial = {'create': True, 'domain_id': None}

        form.handle(request, {})

        self.assertEqual(request.horizon['async_messages'][0][1],
                         u'Successfully added configuration.')

    @mock.patch.object(TNXDomainConfigClient, 'update', mock.Mock())
    def test_update_configuration(self):
        domain = self.tnx_domains.get('1')

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.EditDomainConfiguration(request)
        form.initial = {'create': False, 'domain_id': domain.id}

        form.handle(request, {})

        self.assertEqual(request.horizon['async_messages'][0][1],
                         u'Successfully updated configuration.')

    @mock.patch.object(TNXDomainConfigClient,
                       'create',
                       mock.Mock(side_effect=Exception))
    def test_create_configuration_error(self):

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.EditDomainConfiguration(request)
        form.initial = {'create': True, 'domain_id': None}

        self.assertFalse(form.handle(request, {}))

    @mock.patch.object(TNXDomainConfigClient,
                       'update',
                       mock.Mock(side_effect=Exception))
    def test_update_configuration_error(self):
        domain = self.tnx_domains.get('1')

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.EditDomainConfiguration(request)
        form.initial = {'create': False, 'domain_id': domain.id}

        self.assertFalse(form.handle(request, {}))


class DomainsWorkflowsmsTest(BaseClientTestCase):

    def test_create_domain_ascii_check(self):

        request = get_fake_request(
            is_ajax=True,
            method='POST',
            post_params={
                'name': u'Test name'
            },
            user=FakeUser()
        )

        workflow = workflows.CreateDomain(request)
        self.assertTrue(workflow.is_valid())

    def test_create_domain_ascii_check_error(self):

        request = get_fake_request(
            is_ajax=True,
            method='POST',
            post_params={
                'name': u'Тестовое имя'
            },
            user=FakeUser()
        )

        workflow = workflows.CreateDomain(request)
        self.assertFalse(workflow.is_valid())
