# coding: utf-8
# pylint: disable=no-member
"""Identity groups tabs"""

import logging
from collections import defaultdict

from django.utils.translation import ugettext_lazy as _

from horizon import tabs, exceptions

from openstack_dashboard import api

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.dashboards import utils
from tionix_dashboard.dashboards.helpers import (
    get_projects_assignments_roles_by_group)
from tionix_dashboard.dashboards.identity.groups import tables
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_vdi_instances_extra)
from tionix_dashboard.helpers.views import (
    ServerFilterTabMixin, DTVTnxPaginationMixin)

logger = logging.getLogger(__name__)


class OverviewTab(tabs.Tab):
    """Group overview tab"""
    name = _("Overview")
    slug = "overview"
    template_name = 'groups/_detail_overview.html'

    def get_context_data(self, request, **kwargs):
        context = super(OverviewTab, self).get_context_data(request, **kwargs)
        group = self.tab_group.kwargs['group']
        context["group"] = group
        return context


# noinspection PyAbstractClass
class InstancesTab(ServerFilterTabMixin, tabs.TableTab):
    """Tab for group's vdi instances"""
    table_classes = (tables.UserVDIInstancesTable,)
    name = _('Associated VDI instances')
    slug = 'instances_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_user_vdi_instances_data(self):
        search_opts = self.get_filters(
            {'tnx_group': self.tab_group.kwargs['group_id']}
        )

        instances = get_vdi_instances_extra(self.request, search_opts)
        return instances

    def allowed(self, request):
        return (api.keystone.is_domain_admin(request) and
                LicenseController.is_valid(request, LicenseController.VDI))


# noinspection PyAbstractClass
class ProjectsTab(ServerFilterTabMixin,
                  DTVTnxPaginationMixin,
                  tabs.TableTab):
    """Tab for group's projects"""
    table_classes = (tables.ProjectsTable,)
    name = _('Projects')
    slug = 'projects_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_projects_data(self):
        projects = []
        group_id = self.tab_group.kwargs['group_id']
        offset, limit = self.get_offset_limit()

        domain_id = api.keystone.get_default_domain(self.request,
                                                    get_name=False).id

        try:
            roles_dict = {
                role.id: role.name
                for role in api.keystone.role_list(self.request)
            }
            group_assignments_roles = get_projects_assignments_roles_by_group(
                self.request, group_id
            )

            group_projects_roles = defaultdict(set)
            for group_role in group_assignments_roles:
                project = group_role.scope.get('project')
                if project:
                    group_projects_roles[project['id']].add(
                        roles_dict.get(group_role.role['id'])
                    )
            if group_projects_roles:
                for project in TNXTenantClient.list(self.request,
                                                    domain=domain_id):
                    project_id = project.id
                    if project.id in group_projects_roles:
                        project.roles = group_projects_roles.get(project_id)
                        projects.append(project)

        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _('Unable to get projects list.'))

        if utils.is_domain_supported():
            domains_dict = utils.domain_lookup(self.request)
            if domains_dict:
                for project in projects:
                    project.domain_name = domains_dict.get(project.domain_id)

        self.table.kwargs.update(
            {'offset': offset, 'limit': limit}
        )
        return projects


# noinspection PyAbstractClass
class GroupDetailTabs(tabs.TabGroup):
    """Tionix group details tab group"""
    slug = "tnx_group_details"
    tabs = (OverviewTab,
            InstancesTab,
            ProjectsTab)
    sticky = True
