# coding: utf-8

import logging
from copy import copy

from django.utils.translation import (
    pgettext_lazy, ugettext_lazy as _, ungettext_lazy, npgettext_lazy)

from horizon import tables

from openstack_dashboard import api, policy
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.metrics.api import is_zabbix_available

from tionix_dashboard.dashboards.utils import is_domain_supported
from tionix_dashboard.helpers.actions import LinkBatchAction, TnxFilterAction

logger = logging.getLogger(__name__)


class TenantCheckAllowedMixin(policy.PolicyTargetMixin):
    """Tenant check policy mixin"""

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def allowed(self, request, project):
        """Checks if action is allowed"""
        if (api.keystone.is_multi_domain_enabled() and
                not api.keystone.is_domain_admin(request)):
            return False
        return api.keystone.keystone_can_edit_project()


# noinspection PyAbstractClass
class TnxDeleteTenant(TenantCheckAllowedMixin, LinkBatchAction):
    """ Action to delete projects """
    name = 'delete'
    classes = ("btn-danger",)
    multiple_next_view_name = "horizon:identity:projects:delete"
    policy_rules = (("identity", "identity:delete_project"),)
    policy_target_attrs = (("target.project.domain_id", "domain_id"),)

    def action_present(self, count):
        return (
            ungettext_lazy(
                u"Delete Project",
                u"Delete Projects",
                count
            )
        )

    def action_past(self, count):
        return (
            ungettext_lazy(
                u"Deleted Project",
                u"Deleted Projects",
                count
            )
        )


# noinspection PyAbstractClass
class TnxMultiUpdateTenant(TenantCheckAllowedMixin, LinkBatchAction):
    """ Group action to edit projects """
    name = 'multi_edit_projects'
    multiple_next_view_name = "horizon:identity:projects:multi_update"

    def action_present(self, count):
        return (
            npgettext_lazy(
                u"Multi edit",
                u"Edit Project",
                u"Edit Projects",
                count
            )
        )

    def action_past(self, count):
        return (
            npgettext_lazy(
                u"Multi edit",
                u"Edited Project",
                u"Edited Projects",
                count
            )
        )


# noinspection PyAbstractClass
class ManageHostAggregatesAction(tables.LinkAction):
    """Manage host aggregates action"""
    name = "manage_host_aggregate"
    verbose_name = _("Manage Host Aggregates")
    url = "horizon:identity:projects:manage_host_aggregates"
    classes = ("ajax-modal",)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)


# noinspection PyAbstractClass
class TenantFilterAction(TnxFilterAction):
    """Tenants filter"""
    filter_type = "server"

    filter_choices = [
        ('name', _("Name"), False),
        ('description', _("Description"), False),
        ('id', _("Project ID"), False),
        ('domain_name', _("Domain Name"), False),
        ('enabled',
         pgettext_lazy("Enabled field for project", u"Enabled"),
         False)
    ]
    if not is_domain_supported():
        filter_choices.remove(('domain_name', _("Domain Name"), False))


# noinspection PyAbstractClass
class TnxTenantFilterAction(TenantFilterAction):
    """Tenants filter with vdi"""
    filter_choices = copy(TenantFilterAction.filter_choices)

    filter_choices.insert(1, ('tnx_project_type', _("Type"), False))


# noinspection PyAbstractClass
class SetUpMonitoring(tables.LinkAction):
    """Set up monitoring action"""
    name = "set_up_monitoring"
    verbose_name = _("Set Up Monitoring")
    classes = ("ajax-modal",)
    url = "horizon:identity:projects:set_up_monitoring"
    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    def allowed(self, request, datum):
        return (
            LicenseController.is_valid(request, LicenseController.MONITOR) and
            is_zabbix_available(request)
        )
