# coding: utf-8

"""Identity projects forms"""
import json
import logging

from django import http
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from novaclient.exceptions import Conflict

from horizon import exceptions
from horizon import forms
from horizon import messages
from openstack_dashboard import api

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.clients.zabbix import TNXZabbixClient
from tionix_dashboard.dashboards.identity.projects.utils import (
    AGGREGATE_PROJECT_SEP)
from tionix_dashboard.dashboards.identity.projects.workflows import (
    post_remove_project_cache)
from tionix_dashboard.dashboards.tionix.scheduler import (
    api as scheduler_api)
from tionix_dashboard.dashboards.tionix.scheduler import (
    forms as schedule_forms)
from tionix_dashboard.url_resolve_map import IDENTITY_PROJECT_INDEX

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class ManageHostAggregatesForm(forms.SelfHandlingForm):
    """Manage host aggregates form"""

    name = _("Manage Host Aggregates")

    project_id = forms.CharField(widget=forms.HiddenInput)
    new_allocated = forms.CharField(widget=forms.HiddenInput, required=False)
    old_allocated = forms.CharField(widget=forms.HiddenInput, required=False)

    def get_json_clean_data(self, field):
        """Convert json"""
        try:
            value = json.loads(self.cleaned_data.get(field))
        except TypeError:
            value = []
        return value

    def clean_new_allocated(self):
        """Retrieve converted new_allocated value"""
        return self.get_json_clean_data('new_allocated')

    def clean_old_allocated(self):
        """Retrieve converted old_allocated value"""
        return self.get_json_clean_data('old_allocated')

    def clean(self):
        """Data validation"""
        cleaned_data = super(ManageHostAggregatesForm, self).clean()
        new_allocated = cleaned_data.get('new_allocated')
        old_allocated = cleaned_data.get('old_allocated')
        if new_allocated == old_allocated:
            msg = (
                _('Re-choose available host aggregates.')
                if new_allocated else _('Choose available host aggregates.')
            )
            raise ValidationError(msg)

    # pylint: disable=no-member
    def handle(self, request, data):
        """Sets host aggregates metadata"""

        new_allocated = {(obj['id'], obj['name'])
                         for obj in data['new_allocated']}
        old_allocated = {(obj['id'], obj['name'])
                         for obj in data['old_allocated']}
        project_id = self.initial['project_id']

        aggregates_diff = new_allocated.symmetric_difference(old_allocated)

        failure_aggregates, success_aggregates = [], []
        for aggregate in aggregates_diff:
            aggregate_project_ids = []
            aggregate_id, aggregate_name = aggregate
            try:
                filter_tenant_id = api.nova.aggregate_get(
                    request,
                    aggregate_id
                ).metadata.get('filter_tenant_id')
                if filter_tenant_id:
                    aggregate_project_ids = (
                        filter_tenant_id.split(AGGREGATE_PROJECT_SEP)
                    )
                if aggregate in new_allocated:
                    aggregate_project_ids.append(project_id)
                else:
                    aggregate_project_ids.remove(project_id)
                api.nova.aggregate_set_metadata(
                    request,
                    aggregate_id,
                    {'filter_tenant_id': AGGREGATE_PROJECT_SEP.join(
                        aggregate_project_ids)})
            except Exception as err:
                logger.error(unicode(err))
                failure_aggregates.append(aggregate_name)
            else:
                success_aggregates.append(aggregate_name)

        if success_aggregates:
            messages.success(
                request,
                _('Successfully updated host aggregates: '
                  '"{0}".').format(u'", "'.join(success_aggregates))
            )

        if failure_aggregates:
            messages.error(
                request,
                _('Unable to update host aggregates: '
                  '"{0}".').format(u'", "'.join(failure_aggregates))
            )

        return True


# noinspection PyAbstractClass, PyClassHasNoInit
class DeleteTenantForm(schedule_forms.DeleteObjectWithTasksForm):
    """ Delete openstack project form """

    object_type = ObjectTypeEnum.project

    @post_remove_project_cache
    def delete(self, request, object_id):
        api.keystone.tenant_delete(request, object_id)


# noinspection PyAbstractClass
class TnxDeleteTenantForm(forms.SelfHandlingForm):
    """ Delete tnx projects form """

    columns = [
        _('ID'), _('Name'), _('Action'), _('Task Type'),
        _('Status'), _('Object Type'), _('Object Name'),
    ]

    object_type = ObjectTypeEnum.project

    request = None

    # noinspection PyUnusedLocal
    @post_remove_project_cache
    def handle(self, request, data):
        action_success, action_failure = [], []

        tenants_with_instances = self.initial['tenants_with_instances']
        projects_groups = self.initial['projects_groups']
        redirect_url = self.initial['redirect_url']
        tasks_rows = self.initial['rows']

        for key, (tenant_id, name, project_group) in projects_groups.items():
            if tasks_rows.get(tenant_id):
                query_params = {
                    'subject_id': key,
                    'subject_type': ObjectTypeEnum.project,
                    'status': 'True',
                }
                try:
                    scheduler_api.delete_scheduled_tasks_list(
                        auth_token=request.user.token.id,
                        params=query_params)
                except Exception as err:
                    logger.error(unicode(err))
                    exceptions.handle(self.request,
                                      _('Unable to delete tasks for '
                                        'tenant "%s".') % key,
                                      redirect=redirect_url)

            try:
                if project_group == ObjectGroupEnum.vdi:
                    # tnx project
                    TNXTenantClient.delete(request, tenant_id)
                elif project_group == ObjectGroupEnum.identity:
                    # native openstack project
                    api.keystone.tenant_delete(request, tenant_id)
            except Exception as err:
                logger.error(unicode(err))
                action_failure.append(name)
            else:
                action_success.append(name)

        # Begin with success message class, downgrade to info if
        # problems.
        success_message_level = messages.success
        if tenants_with_instances:
            tenants_with_instances_str = u'", "'.join(
                tenants_with_instances)
            msg = _('Projects "{0}" contain instances, removing '
                    'is prohibited.').format(tenants_with_instances_str)
            messages.error(request, msg)
            success_message_level = messages.info
        if action_failure:
            msg = _('Unable to delete: "{0}"').format(
                u'", "'.join(action_failure))
            messages.error(request, msg)
            success_message_level = messages.info
        if action_success:
            msg = _('Deleted: "{0}"').format(u'", "'.join(
                action_success))
            success_message_level(request, msg)

        self.request = request

        if action_failure:
            return http.HttpResponseRedirect(redirect_url)

        return True


# noinspection PyAbstractClass
class SetUpMonitoringForm(forms.SelfHandlingForm):
    """Set up monitoring form"""
    zabbix_user_login = forms.CharField(label=_("Login"), max_length=100)
    zabbix_user_password = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput(render_value=True)
    )

    def handle(self, request, data):
        result = True

        try:
            TNXZabbixClient.setup_monitoring(request,
                                             self.initial['project_id'],
                                             **data)
            messages.success(request,
                             _(u'Monitoring setup successfully completed.'))
        except UrlNotAvailableError:
            raise
        except Conflict as err:
            logger.error(unicode(err))
            result = False
            self.add_error(
                None,
                _(u'Unable to set up monitoring. User with login "{0}" '
                  u'already exists.').format(data['zabbix_user_login']))
        except Exception as err:
            logger.error(unicode(err))
            result = False
            exceptions.handle(
                self.request,
                _(u'Unable to set up monitoring. {0}').format(err),
                redirect=reverse(IDENTITY_PROJECT_INDEX))

        return result
