# coding: utf-8

from django.core.urlresolvers import reverse
from django.template import defaultfilters as filters
from django.utils.translation import ugettext_lazy as _, pgettext_lazy
from tionix_client.journal.constants import ObjectType as JournalObjectType

from horizon import forms, tables

from openstack_dashboard import api, policy
from openstack_dashboard.dashboards.admin.aggregates.tables import (
    AggregateFilterAction)
from openstack_dashboard.dashboards.identity.projects import (
    tables as project_tables)

from tionix_dashboard import journal
from tionix_dashboard.dashboards.admin.aggregates.tables import (
    HostAggregatesTable)
from tionix_dashboard.dashboards.identity.projects import actions
from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ProjectSchedulerAction)
from tionix_dashboard.dashboards.tionix.vdi.instances import (
    tables as vdi_instances_tables)
from tionix_dashboard.dashboards.tionix.vdi.instances.actions import (
    VDIUsersFilterAction, VDIGroupsFilterAction)
from tionix_dashboard.dashboards.utils import is_domain_supported
from tionix_dashboard.helpers.tables import TnxLocalPaginationTabDataTable
from tionix_dashboard.url_resolve_map import (
    IDENTITY_PROJECT_DETAIL, ADMIN_SERVERS_DETAIL, PROJECT_SERVERS_DETAIL)


# noinspection PyClassHasNoInit
class TnxUpdateCell(project_tables.UpdateCell):
    def allowed(self, request, project, cell):
        policy_rule = (("identity", "identity:update_project"),)
        policy_target = {'target.project.domain_id': project.domain_id}
        return (
            (cell.column.name != 'enabled' or
             request.user.project_id != cell.datum.id) and
            api.keystone.keystone_can_edit_project() and
            policy.check(policy_rule, request, target=policy_target))

    def update_cell(self, request, datum, project_id,
                    cell_name, new_cell_value):
        # noinspection PyTypeChecker
        if isinstance(new_cell_value, basestring):
            new_cell_value = new_cell_value.strip()

        return super(TnxUpdateCell, self).update_cell(
            request, datum, project_id, cell_name, new_cell_value
        )


# noinspection PyAbstractClass, PyClassHasNoInit
class ModifyQuotas(project_tables.ModifyQuotas):
    def allowed(self, request, datum):
        if api.keystone.VERSIONS.active < 3:
            result = True
        else:
            if api.keystone.is_multi_domain_enabled():
                result = api.keystone.is_domain_admin(request)
            else:
                result = api.keystone.keystone_can_edit_project()
        return result


class TnxTenantsTable(tables.DataTable):
    """Tenants table with vdi enabled"""
    name = tables.Column(
        'name',
        verbose_name=_('Name'),
        link=IDENTITY_PROJECT_DETAIL,
        form_field=forms.CharField(max_length=64),
        update_action=TnxUpdateCell)
    tnx_project_type = tables.Column(
        lambda x: getattr(x, "tnx_project_type", '') or '',
        verbose_name=_("Type"),
        display_choices=TenantTypeEnum.values.items())
    description = tables.Column(
        lambda obj: getattr(obj, 'description', '').strip(),
        verbose_name=_('Description'),
        form_field=forms.CharField(
            widget=forms.Textarea(attrs={'rows': 4}),
            required=False),
        update_action=TnxUpdateCell)
    id = tables.Column('id', verbose_name=_('Project ID'))
    if is_domain_supported():
        domain_name = tables.Column('domain_name',
                                    verbose_name=_('Domain Name'))
    enabled = tables.Column(
        'enabled',
        verbose_name=pgettext_lazy("Enabled field for project", u"Enabled"),
        status=True,
        filters=(filters.yesno, filters.capfirst),
        form_field=forms.BooleanField(
            label=_('Enabled'),
            required=False),
        update_action=TnxUpdateCell)

    def get_project_detail_link(self, project):
        """return link to project"""
        # this method is an ugly monkey patch, needed because
        # the column link method does not provide access to the request
        policy_rule = (("identity", "identity:get_project"),)
        policy_target = {
            'project': project,
            'target.project.domain_id': project.domain_id,
        }
        if policy.check(policy_rule, self.request, target=policy_target):
            return reverse(IDENTITY_PROJECT_DETAIL,
                           args=(project.id,))
        return None

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(TnxTenantsTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)
        # see the comment above about ugly monkey patches
        self.columns['name'].get_link_url = self.get_project_detail_link

    class Meta(project_tables.TenantsTable.Meta):

        row_actions = (
            project_tables.UpdateMembersLink, project_tables.UpdateGroupsLink,
            project_tables.UpdateProject, ModifyQuotas,
            project_tables.RescopeTokenToProject, actions.TnxDeleteTenant,
            ProjectSchedulerAction, actions.ManageHostAggregatesAction,
            actions.SetUpMonitoring
        )
        table_actions = (
            actions.TnxTenantFilterAction, project_tables.CreateProject,
            actions.TnxDeleteTenant
        )
        table_actions_menu = (
            ProjectSchedulerAction,
            actions.TnxMultiUpdateTenant
        )


# noinspection PyClassHasNoInit
class TenantsTable(project_tables.TenantsTable):
    """Tenants table"""
    class Meta(project_tables.TenantsTable.Meta):

        row_actions = (
            project_tables.UpdateMembersLink,
            project_tables.UpdateGroupsLink,
            project_tables.UpdateProject,
            project_tables.UsageLink,
            ModifyQuotas,
            project_tables.RescopeTokenToProject,
            actions.TnxDeleteTenant,
            ProjectSchedulerAction,
            actions.ManageHostAggregatesAction,
            actions.SetUpMonitoring
        )
        table_actions = (
            actions.TenantFilterAction,
            project_tables.CreateProject,
            actions.TnxDeleteTenant
        )
        table_actions_menu = (
            ProjectSchedulerAction,
            actions.TnxMultiUpdateTenant
        )


def get_url_for_instance(datum, is_admin=False):
    link = None
    if is_admin and not getattr(datum, 'object_deleted', False):
        obj_id = getattr(datum, 'object_id', None)
        link = reverse("horizon:admin:instances:detail", args=(obj_id,))
    return link


class TnxInstancesAuditLog(TnxLocalPaginationTabDataTable,
                           journal.JournalBaseTable):

    @staticmethod
    def get_url_for_object(obj, is_admin=False):
        link = None
        if (not obj.object_deleted and
                obj.object_type == JournalObjectType.Server):
            link = reverse(
                (ADMIN_SERVERS_DETAIL if is_admin else PROJECT_SERVERS_DETAIL),
                args=(obj.object_id,))
        return link

    class Meta(object):
        name = 'audit'
        multi_select = False
        verbose_name = _('Instances Action List')
        table_actions = (journal.JournalFilterAction,)


class ProjectHostAggregatesTable(HostAggregatesTable):

    class Meta(object):
        name = "host_aggregates"
        verbose_name = _("Host Aggregates")
        columns = ('name', 'availability_zone', 'hosts', 'balancing')
        table_actions = (AggregateFilterAction,)
        multi_select = False


# noinspection PyAbstractClass,PyClassHasNoInit
class UsersTable(vdi_instances_tables.UsersTable):

    class Meta(object):
        name = 'users'
        verbose_name = _('Users')
        multi_select = False
        table_actions = (
            VDIUsersFilterAction,
        )


# noinspection PyAbstractClass,PyClassHasNoInit
class GroupsTable(vdi_instances_tables.GroupsTable):

    class Meta(object):
        name = 'groups'
        verbose_name = _('Groups')
        multi_select = False
        table_actions = (
            VDIGroupsFilterAction,
        )
