# coding: utf-8
"""Identity projects tabs"""

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs

from openstack_dashboard import api, usage as os_usage
from openstack_dashboard.dashboards.project.instances import (
    tabs as project_tabs)

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.dashboards import utils
from tionix_dashboard.dashboards.helpers import get_users_in_project
from tionix_dashboard.dashboards.identity.projects.tables import (
    ProjectHostAggregatesTable)
from tionix_dashboard.dashboards.identity.projects.utils import (
    AGGREGATE_PROJECT_SEP, get_host_aggregates)
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.dashboards.identity.projects import tables
from tionix_dashboard.helpers.views import (
    DTVTnxPaginationMixin, ServerFilterTabMixin)
from tionix_dashboard.journal.helpers import ProjectJournal

logger = logging.getLogger(__name__)


class OverviewTab(tabs.Tab):
    """Project overview tab"""
    name = _("Overview")
    slug = "overview"
    template_name = 'projects/_detail_overview.html'

    def get_context_data(self, request, **kwargs):
        """project context"""

        project = self.tab_group.kwargs['project']
        domain_enabled = utils.is_domain_supported()
        if domain_enabled:
            domains_dict = utils.domain_lookup(self.request)
            project.domain_name = domains_dict.get(project.domain_id, None)
        return {"project": project,
                "domain_enabled": domain_enabled,
                'up_time': '99,999',
                'recovery_time': '8',
                'movable': True}


# noinspection PyAbstractClass
class UsageTab(tabs.TableTab):
    """Project usage tab"""
    name = _("Usage")
    slug = "usage"
    template_name = 'projects/_detail_usage.html'
    table_classes = (os_usage.ProjectUsageTable,)
    usage_class = os_usage.ProjectUsage
    preload = False
    usage = None

    def get_context_data(self, request, **kwargs):
        context = super(UsageTab, self).get_context_data(request, **kwargs)
        context['table'].kwargs['usage'] = self.usage
        context['form'] = self.usage.form
        context['usage'] = self.usage

        try:
            context['simple_tenant_usage_enabled'] = (
                api.nova.extension_supported('SimpleTenantUsage', self.request)
            )
        except Exception as err:
            logger.error(unicode(err))
            context['simple_tenant_usage_enabled'] = True

        return context

    def get_project_usage_data(self):
        instances = []
        project_id = self.tab_group.kwargs.get('tenant_id',
                                               self.request.user.tenant_id)
        try:
            self.usage = self.usage_class(self.request, project_id)
            self.usage.summarize(*self.usage.get_date_range())
            instances = self.usage.get_instances()
        except Exception as err:
            logger.error(err)
            exceptions.handle(self.request,
                              _('Unable to retrieve usage information.'))
        return instances


# noinspection PyAbstractClass, PyClassHasNoInit
class InstancesAuditTab(ServerFilterTabMixin,
                        DTVTnxPaginationMixin,
                        project_tabs.AuditTab):
    """Tionix audit of instances tab"""

    table_classes = (tables.TnxInstancesAuditLog, )

    def get_audit_data(self):
        """Project instances audit data"""
        offset, limit = self.get_offset_limit()
        self.table.kwargs.update(
            {'offset': offset, 'limit': limit}
        )
        project = self.tab_group.kwargs['project']
        journal = ProjectJournal(self.request, project, include_instances=True)
        return journal.list()


# noinspection PyAbstractClass
class ProjectHostAggregatesTab(tabs.TableTab):
    """Host aggregates tab"""

    name = _("Host Aggregates")
    slug = "host_aggregates"
    template_name = 'horizon/common/_detail_table.html'
    table_classes = (ProjectHostAggregatesTable,)
    preload = False

    def get_host_aggregates_data(self):
        data = []
        project_id = self.tab_group.kwargs['tenant_id']
        try:
            aggregates = get_host_aggregates(self.request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve host aggregates list.'))
        else:
            for aggregate in aggregates:
                filter_tenant_id = aggregate.metadata.get('filter_tenant_id')
                if (filter_tenant_id and project_id in filter_tenant_id.split(
                        AGGREGATE_PROJECT_SEP)):
                    data.append(aggregate)

        return data


# noinspection PyAbstractClass
class GroupsTab(ServerFilterTabMixin, tabs.TableTab):
    name = _("Groups")
    slug = "groups"
    table_classes = (tables.GroupsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_groups_data(self):
        project_id = self.tab_group.kwargs['tenant_id']
        domain_id = api.keystone.get_effective_domain_id(self.request)
        groups = api.keystone.group_list(self.request, project=project_id,
                                         domain=domain_id)

        return groups

    def allowed(self, request):
        project = api.keystone.tenant_get(request,
                                          self.tab_group.kwargs['tenant_id'])
        domain_id = api.keystone.get_default_domain(request, False).get('id')
        return project.domain_id == domain_id


# noinspection PyAbstractClass
class UsersTab(ServerFilterTabMixin, tabs.TableTab):
    name = _("Users")
    slug = "users"
    table_classes = (tables.UsersTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_users_data(self):
        project_id = self.tab_group.kwargs['tenant_id']
        project_users = get_users_in_project(self.request, project_id)

        return project_users

    def allowed(self, request):
        project = api.keystone.tenant_get(request,
                                          self.tab_group.kwargs['tenant_id'])
        domain_id = api.keystone.get_default_domain(request, False).get('id')
        return project.domain_id == domain_id


# noinspection PyAbstractClass
class ProjectDetailTabs(tabs.TabGroup):
    """Tionix project details tab group"""
    slug = "tnx_project_details"
    tabs = (OverviewTab,
            UsageTab,
            InstancesAuditTab,
            ActiveTasksTab,
            ProjectHostAggregatesTab,
            GroupsTab,
            UsersTab,)
    sticky = True
    obj_type = ObjectTypeEnum.project
    obj_group = ObjectGroupEnum.identity
