# coding: utf-8
# pylint: disable=no-member
# pylint: disable=unused-variable
# pylint: disable=import-error

"""Identity projects tests"""
import json

import mock

from novaclient import exceptions as nova_exceptions

from django import http
from django.conf import settings
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from django.utils.http import urlencode

from openstack_auth import policy as policy_backend

from horizon import exceptions

from openstack_dashboard import api
from openstack_dashboard.usage import quotas

from tionix_scheduler_client.enums import ObjectGroupEnum

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.clients.zabbix import TNXZabbixClient
from tionix_dashboard.dashboards import utils
from tionix_dashboard.dashboards.identity.projects import (
    actions, tables, views, tabs)
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.identity.projects import forms
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, BaseAdminClientTestCase, FakeUser)


INDEX_URL = "horizon:identity:projects:index"
DETAIL_URL = "horizon:identity:projects:detail"
DELETE_URL = "horizon:identity:projects:delete"


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class TnxTenantsTablesTestCase(BaseClientTestCase):

    def test_tnx_tenants_table(self):

        table = tables.TnxTenantsTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'actions', 'name', 'tnx_project_type',
                    'description', 'id', 'enabled', 'domain_name'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_table_actions(self):
        tenants = self.tenants.list()

        table = tables.TnxTenantsTable(self.request, data=tenants)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_row_actions(self):
        table = tables.TnxTenantsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            [action.name for action in table.get_row_actions(tenant)], []
        )


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class TnxAdminTenantsTablesTestCase(BaseAdminClientTestCase):

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_table_actions(self):
        tenants = self.tnx_tenants.list()

        table = tables.TnxTenantsTable(self.request, data=tenants)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['create', 'delete', 'filter', 'multi_edit_projects',
                    'project_schedule'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_row_actions(self):
        table = tables.TnxTenantsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['delete', 'groups', 'project_schedule', 'quotas',
                    'update', 'users', 'manage_host_aggregate',
                    'set_up_monitoring'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(LicenseController, 'is_valid',
                       mock.Mock(return_value=False))
    def test_tnx_tenants_row_actions_monitor_lic_invalid(self):
        table = tables.TnxTenantsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['delete', 'groups', 'project_schedule', 'quotas',
                    'update', 'users', 'manage_host_aggregate'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXZabbixClient, 'get_templates',
                       mock.Mock(side_effect=nova_exceptions.Forbidden))
    def test_tnx_tenants_row_actions_monitor_zabbix_not_available(self):
        table = tables.TnxTenantsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['delete', 'groups', 'project_schedule', 'quotas',
                    'update', 'users', 'manage_host_aggregate'])
        )

    def test_project_aggregates_table(self):

        table = tables.ProjectHostAggregatesTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['name', 'availability_zone', 'hosts', 'balancing'])
        )

    def test_project_aggregates_table_actions(self):
        tenants = self.tenants.list()

        table = tables.ProjectHostAggregatesTable(self.request, data=tenants)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter'])
        )


class TnxTenantsViewsAdminTest(BaseClientTestCase):

    @mock.patch.object(TNXTenantClient, 'list')
    @mock.patch.object(utils, 'domain_lookup')
    @mock.patch.object(api.keystone, 'get_effective_domain_id')
    @mock.patch.object(api.keystone, 'tenant_list')
    def test_index_get_data(self, tenant_list, get_effective_domain_id,
                            domain_lookup, tnx_tenant_list):
        tenants = self.tenants.list()
        domain = self.domains.get(id="1")

        tenant_list.return_value = (tenants, None)
        get_effective_domain_id.return_value = domain.id
        domain_lookup.return_value = {domain.id: domain.name}
        tnx_tenant_list.return_value = tenants

        view = views.TnxIndexView()
        view.request = get_fake_request(**{
            'is_ajax': True,
            'session': {},
            'COOKIES': {},
            'horizon': {
                'async_messages': []
            },
            'user': Empty(roles=[{'name': 'admin'}])
        })

        data = view.get_data()

        self.assertEqual(len(data), len(tenants))

    @mock.patch.object(views.TnxDeleteTenantView,
                       'get_tasks_rows',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'tenant_delete',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXTenantClient, 'get')
    def test_delete_from_index(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]

        tenant_get.return_value = tenant

        url = '?'.join([reverse(DELETE_URL), urlencode({'id': tenant.id})])

        req = self.factory.post(url)

        index_url = reverse(INDEX_URL)
        req.META['HTTP_REFERER'] = ''.join([settings.TESTSERVER,
                                            settings.WEBROOT,
                                            index_url])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = views.TnxDeleteTenantView.as_view()(req)

        self.assertIn(index_url, response.url)

    @mock.patch.object(views.TnxDeleteTenantView,
                       'get_tasks_rows',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'tenant_delete',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXTenantClient, 'get')
    def test_delete_from_detail(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]

        tenant_get.return_value = tenant

        url = '?'.join([reverse(DELETE_URL), urlencode({'id': tenant.id})])

        req = self.factory.post(url)

        index_url = reverse(INDEX_URL)
        req.META['HTTP_REFERER'] = ''.join([
            settings.TESTSERVER,
            settings.WEBROOT,
            reverse(DETAIL_URL, args=(tenant.id,))])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = views.TnxDeleteTenantView.as_view()(req)

        self.assertIn(index_url, response.url)

    @mock.patch.object(api.base,
                       'is_service_enabled',
                       mock.Mock(return_value=False))
    @mock.patch.object(api.keystone, 'role_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'user_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'group_list', mock.Mock(return_value=[]))
    @mock.patch.object(quotas,
                       'get_disabled_quotas',
                       mock.Mock(return_value=[]))
    @mock.patch.object(quotas, 'get_default_quota_data',)
    @mock.patch.object(api.keystone, 'get_default_role')
    def test_tnx_project_create_view_layout(self, mocked_get_default_role,
                                            mocked_get_default_quota_data):
        role = self.roles.first()
        quota = self.quotas.first()

        mocked_get_default_role.return_value = role
        mocked_get_default_quota_data.return_value = quota

        view = views.TnxCreateProjectView()
        view.request = self.request
        view.kwargs = {}
        response = view.get(self.request)

        self.assertContains(response, 'workflow static_page vertical')

    @mock.patch.object(api.base,
                       'is_service_enabled',
                       mock.Mock(return_value=False))
    @mock.patch.object(api.keystone, 'group_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'role_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'user_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'get_project_groups_roles',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'get_project_users_roles',
                       mock.Mock(return_value=[]))
    @mock.patch.object(quotas,
                       'get_disabled_quotas',
                       mock.Mock(return_value=[]))
    @mock.patch.object(quotas, 'get_default_quota_data')
    @mock.patch.object(quotas, 'get_tenant_quota_data')
    @mock.patch.object(api.keystone, 'get_default_role')
    @mock.patch.object(api.keystone, 'domain_get')
    @mock.patch.object(api.keystone, 'tenant_get')
    def test_tnx_project_update_view_layout(self, mocked_tenant_get,
                                            mocked_domain_get,
                                            mocked_get_default_role,
                                            mocked_get_tenant_quota_data,
                                            mocked_get_default_quota_data):
        project = self.tnx_tenants.list()[0]
        role = self.roles.first()
        quota = self.quotas.first()
        domain = self.tnx_domains.list()[0]

        mocked_domain_get.return_value = domain
        mocked_tenant_get.return_value = project
        mocked_get_default_role.return_value = role
        mocked_get_default_quota_data.return_value = quota
        mocked_get_tenant_quota_data.return_value = quota

        view = views.TnxUpdateProjectView()
        view.request = self.request
        view.kwargs = {'tenant_id': project.id}
        response = view.get(self.request)

        self.assertContains(response, 'workflow static_page vertical')


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
class TnxTenantsActionsTestCase(BaseClientTestCase):

    def test_filter_action(self):

        fields, names, api_flags = zip(
            *actions.TenantFilterAction.filter_choices
        )

        self.assertListEqual(
            sorted(fields),
            sorted(['name', 'description', 'id', 'enabled', 'domain_name'])
        )

    def test_tnx_filter_action1(self):

        fields, names, api_flags = zip(
            *actions.TnxTenantFilterAction.filter_choices
        )

        self.assertListEqual(
            sorted(fields),
            sorted(['name', 'description', 'id', 'enabled', 'domain_name',
                    'tnx_project_type'])
        )

    def test_tnx_filter_action2(self):
        tenants = self.tnx_tenants.list()
        tenant = tenants[0]

        action = actions.TnxTenantFilterAction()

        fields, names, api_flags = zip(*action.filter_choices)

        for field in fields:
            table = tables.TenantsTable(
                request=get_fake_request(
                    session={
                        'tenants__filter__q_field': field
                    }
                )
            )

            filtered_data = action.filter(table,
                                          tenants,
                                          getattr(tenant, field, ''))

            self.assertEqual(len(filtered_data), 1)


class TnxTenantsTabsTestCase(BaseAdminClientTestCase):

    # noinspection PyUnusedLocal
    def fake_get_common_journal_data(self, data):
        # noinspection PyUnusedLocal
        def _(request, search_opts):
            project_id = search_opts.get('project_id')
            project_instances_ids = [instance.id
                                     for instance in instances
                                     if instance.tenant_id == project_id]
            return [d for d in data if d.object_id in project_instances_ids]

        instances = self.tnx_servers.list()

        return _

    @mock.patch.object(api.keystone, 'tenant_get')
    def test_project_details_tabs(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]
        tenant_get.return_value = tenant

        tab_group = tabs.ProjectDetailTabs(
            self.request, tenant_id=tenant.id
        )

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['audit', 'host_aggregates', 'overview', 'scheduler',
                    'usage'])
        )

    @mock.patch.object(TNXServersClient, 'list', mock.Mock(return_value=[]))
    @mock.patch('tionix_dashboard.journal.helpers.get_common_journal_data',
                mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'tenant_get')
    def test_audit_data_empty_instances(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]
        tenant_get.return_value = tenant

        tab_group = tabs.ProjectDetailTabs(
            self.request, tenant_id=tenant.id
        )
        tab_group.kwargs['project'] = tenant

        tab = tabs.InstancesAuditTab(tab_group=tab_group, request=self.request)

        self.assertEqual(len(tab.get_audit_data()), 0)

    @mock.patch('openstack_dashboard.policy.check', mock.Mock())
    @mock.patch('tionix_dashboard.dashboards.utils.user_list')
    @mock.patch('tionix_dashboard.journal.helpers.get_common_journal_data')
    @mock.patch.object(api.keystone, 'tenant_get')
    def test_audit_data(self, tenant_get, get_common_journal_data, user_list):
        tenant = self.tnx_tenants.list()[0]
        tenant_get.return_value = tenant
        instance = self.tnx_servers.list()[0]
        users = self.tnx_users.list()
        user = users[0]

        user_list.return_value = users

        get_common_journal_data.side_effect = (
            self.fake_get_common_journal_data(
                [Empty(object_id=instance.id,
                       user_id=user.id,
                       user_name=None,
                       event_time=None,
                       message=user.name)])
        )

        tab_group = tabs.ProjectDetailTabs(
            self.request, tenant_id=tenant.id
        )
        tab_group.kwargs['project'] = tenant

        tab = tabs.InstancesAuditTab(tab_group=tab_group, request=self.request)

        data = tab.get_audit_data()

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0].object_name, instance.name)
        self.assertEqual(data[0].user, user.name)

    @mock.patch('tionix_dashboard.dashboards.identity.projects.tabs.'
                'get_host_aggregates')
    @mock.patch.object(api.keystone, 'tenant_get')
    def test_project_host_aggregates_data(
            self, tenant_get, mocked_get_host_aggregates):
        tenant = self.tnx_tenants.list()[0]
        tenant_get.return_value = tenant

        aggregates = [
            Empty(id=1, metadata={'filter_tenant_id': tenant.id}),
            Empty(id=2, metadata={'filter_tenant_id': tenant.id}),
            Empty(id=3, metadata={})
        ]

        mocked_get_host_aggregates.return_value = aggregates

        tab_group = tabs.ProjectDetailTabs(
            self.request, tenant_id=tenant.id
        )

        tab = tabs.ProjectHostAggregatesTab(tab_group=tab_group,
                                            request=self.request)
        project_host_aggregates = tab.get_host_aggregates_data()
        self.assertEqual(2, len(project_host_aggregates))


@mock.patch('tionix_dashboard.dashboards.identity.projects.workflows.'
            'remove_project_cache', mock.Mock(return_value=True))
class TnxTenantsFormsTestCase(BaseClientTestCase):

    def test_manage_host_aggregate_valid(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.ManageHostAggregatesForm(
            self.request, data={
                'new_allocated': json.dumps([{'id': 1,
                                              'name': 'test aggregate'}]),
                'old_allocated': '[]',
                'project_id': tenant.id
            }
        )

        self.assertTrue(form.is_valid())

    def test_manage_host_aggregate_not_valid(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.ManageHostAggregatesForm(self.request,
                                              data={'new_allocated': '[]',
                                                    'old_allocated': '[]',
                                                    'project_id': tenant.id})

        self.assertFalse(form.is_valid())

    @mock.patch.object(api.nova, 'aggregate_set_metadata', mock.Mock())
    def test_manage_host_aggregate_handle(self):
        tenant = self.tnx_tenants.list()[0]

        data = {
            'new_allocated': [{'id': 2, 'name': 'aggregate 2'}],
            'old_allocated': [{'id': 1, 'name': 'aggregate'},
                              {'id': 2, 'name': 'aggregate 2'}],
            'project_id': tenant.id
        }
        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ManageHostAggregatesForm(request)
        form.initial = {'project_id': tenant.id}
        form.handle(request, data)

        self.assertTrue(request.horizon['async_messages'][0][1],
                        u'Successfully updated host aggregates: '
                        u'"test aggregate".')

    @mock.patch.object(api.nova,
                       'aggregate_set_metadata',
                       mock.Mock(side_effect=Exception))
    def test_manage_host_aggregate_handle_error(self):
        tenant = self.tnx_tenants.list()[0]

        data = {
            'new_allocated': [{'id': 2, 'name': 'aggregate 2'}],
            'old_allocated': [{'id': 1, 'name': 'aggregate'},
                              {'id': 2, 'name': 'aggregate 2'}],
            'project_id': tenant.id
        }
        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ManageHostAggregatesForm(request)
        form.initial = {'project_id': tenant.id}
        form.handle(request, data)

        self.assertTrue(request.horizon['async_messages'][0][1],
                        u'Unable to update host aggregates: "test aggregate".')

    @mock.patch.object(TNXTenantClient, 'delete')
    @mock.patch.object(api.keystone, 'tenant_delete')
    def test_tnx_delete_vdi_project(self, tenant_delete, delete):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': {}
        }

        request = get_fake_request(
            user=FakeUser(unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertTrue(delete.called)
        self.assertFalse(tenant_delete.called)

    @mock.patch.object(TNXTenantClient, 'delete')
    @mock.patch.object(api.keystone, 'tenant_delete')
    def test_tnx_delete_identity_project(self, tenant_delete, delete):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.identity)},
            'redirect_url': '',
            'rows': {}
        }

        request = get_fake_request(
            user=FakeUser(unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertFalse(delete.called)
        self.assertTrue(tenant_delete.called)

    @mock.patch('tionix_dashboard.dashboards.identity.projects.forms.'
                'scheduler_api.delete_scheduled_tasks_list')
    @mock.patch.object(TNXTenantClient, 'delete')
    def test_tnx_delete_with_tasks(self, delete, delete_scheduled_tasks_list):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': {tenant.id: [None]},
        }

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertTrue(delete.called)
        self.assertTrue(delete_scheduled_tasks_list.called)

    @mock.patch.object(TNXTenantClient, 'delete', mock.Mock())
    def test_tnx_delete_with_instances(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [tenant.id],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': {}
        }

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertTrue(request.horizon['async_messages'][0][1],
                        u'Projects "{0}" contain instances, removing '
                        u'is prohibited.'.format(tenant.id))
        self.assertTrue(request.horizon['async_messages'][0][0], 'info')

    @mock.patch.object(TNXTenantClient,
                       'delete',
                       mock.Mock(side_effect=Exception))
    def test_tnx_delete_redirect(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': {}
        }

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        result = form.handle(request, {})

        self.assertIsInstance(result, http.HttpResponseRedirect)

    def test_set_up_monitoring(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.SetUpMonitoringForm(self.request)
        form.initial = {
            'project_id': tenant.id
        }
        self.assertListEqual(sorted(form.fields.keys()),
                             sorted(['zabbix_user_login',
                                     'zabbix_user_password']))

    @mock.patch.object(TNXZabbixClient, 'setup_monitoring',
                       mock.Mock(side_effect=nova_exceptions.ClientException))
    @mock.patch.object(exceptions, 'handle')
    def test_set_up_monitoring_error(self, exceptions_handle):
        tenant = self.tnx_tenants.list()[0]

        form = forms.SetUpMonitoringForm(self.request)
        form.initial = {
            'project_id': tenant.id
        }
        form.cleaned_data = {}

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        result = form.handle(request, {'zabbix_user_login': 'login',
                                       'zabbix_user_password': 'password'})
        self.assertFalse(result)
        errors = form.errors.values()
        self.assertEqual(len(errors), 0)
        self.assertTrue(exceptions_handle.called)

    @mock.patch.object(TNXZabbixClient, 'setup_monitoring',
                       mock.Mock(side_effect=nova_exceptions.Conflict))
    @mock.patch.object(exceptions, 'handle')
    def test_set_up_monitoring_conflict_error(self, exceptions_handle):
        tenant = self.tnx_tenants.list()[0]

        form = forms.SetUpMonitoringForm(self.request)
        form.initial = {
            'project_id': tenant.id
        }
        form.cleaned_data = {}

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        result = form.handle(request, {'zabbix_user_login': 'login',
                                       'zabbix_user_password': 'password'})

        self.assertFalse(result)
        errors = form.errors.values()
        self.assertEqual(len(errors), 1)
        self.assertEqual(
            errors[0][0],
            u'Unable to set up monitoring. '
            u'User with login "login" already exists.'
        )
        self.assertFalse(exceptions_handle.called)


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                   mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class AdminTenantsTablesTestCase(BaseAdminClientTestCase):
    def test_tenants_table_actions(self):
        tenants = self.tenants.list()

        table = tables.TenantsTable(self.request, data=tenants)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['create', 'delete', 'filter', 'multi_edit_projects',
                    'project_schedule'])
        )

    def test_tenants_row_actions(self):
        table = tables.TenantsTable(self.request)

        tenant = self.tenants.first()
        tenant.tnx_project_type_name = None
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['delete', 'groups', 'project_schedule', 'quotas',
                    'update', 'users', 'manage_host_aggregate',
                    'set_up_monitoring', 'usage'])
        )

    @mock.patch.object(LicenseController, 'is_valid',
                       mock.Mock(return_value=False))
    def test_tenants_row_actions_monitor_lic_invalid(self):
        table = tables.TenantsTable(self.request)

        tenant = self.tenants.first()
        tenant.tnx_project_type_name = None
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['delete', 'groups', 'project_schedule', 'quotas',
                    'update', 'users', 'manage_host_aggregate', 'usage'])
        )
