# coding: utf-8

"""Projects utils"""

import logging
from collections import defaultdict

from django.core.cache import cache
from django.utils.translation import ugettext_lazy as _

from openstack_auth import utils

from tionix_client.os_clients.keystone_client import get_keystone_client

from tionix_scheduler_client.enums import ObjectGroupEnum

from horizon import exceptions, messages
from horizon.utils.memoized import memoized
from openstack_dashboard import api
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.helpers.utils import memoize_by_keyword_arg


logger = logging.getLogger(__name__)


AGGREGATE_PROJECT_SEP = ','


def remove_project_cache(token):
    """remove token cache"""
    cache.delete(token)


# noinspection PyUnusedLocal
@memoize_by_keyword_arg(('token', ))
def get_project_list(*args, **kwargs):
    """Return tenants list cached by token"""
    is_federated = kwargs.get('is_federated', False)
    auth_url = utils.fix_auth_url_version(kwargs['auth_url'])
    client = get_keystone_client(token=kwargs['token'], auth_url=auth_url)

    if utils.get_keystone_version() < 3:
        projects = client.tenants.list()
    elif is_federated:
        projects = client.federation.projects.list()
    else:
        projects = client.projects.list(
            domain=client.session.auth.auth_ref.user_domain_id,
            user=kwargs.get('user_id')
        )

    projects.sort(key=lambda project: project.name.lower())
    return projects


def patch_project_cache():
    """Patch to update projects cache"""
    utils.get_project_list = get_project_list


def get_project_group_for_schedule(project):
    """Return project type for schedule"""
    return (ObjectGroupEnum.vdi
            if project.tnx_project_type == TenantTypeEnum.VDI
            else ObjectGroupEnum.identity)


def get_host_aggregates(request):
    """Retrieve host aggregates"""
    return novaclient(request).aggregates.list()


@memoized
def get_role_list(request):
    members = []
    try:
        members = api.keystone.role_list(request)
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _('Unable to retrieve user list.'))
    return members


@memoized
def get_project_roles_list(request, project_id, member_field_getter):
    project_roles = defaultdict(list)
    if project_id:
        try:
            users_roles = api.keystone.get_project_users_roles(request,
                                                               project_id)
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to get users of project.'))
        else:
            for user_id in users_roles:
                roles_ids = users_roles[user_id]
                for role_id in roles_ids:
                    field_name = member_field_getter(role_id)
                    project_roles[field_name].append(user_id)
    return project_roles


@memoized
def get_project_groups_list(request, project_id, member_field_getter):
    project_groups = defaultdict(list)
    try:
        groups_roles = api.keystone.get_project_groups_roles(
            request, project_id)
    except Exception as err:
        logger.error(unicode(err))
        messages.error(request, _('Unable to get groups of project.'))
    else:
        for group_id in groups_roles:
            roles_ids = groups_roles[group_id]
            for role_id in roles_ids:
                field_name = member_field_getter(role_id)
                project_groups[field_name].append(group_id)

    return project_groups
