# coding: utf-8
# pylint: disable=import-error

"""Projects views"""

import logging

from django.core.urlresolvers import reverse_lazy, reverse
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import exceptions, forms, messages, tabs
from horizon.utils import memoized

from openstack_dashboard import api, policy
from openstack_dashboard.api.rest import (
    urls as rest_urls, utils as rest_utils)
from openstack_dashboard.dashboards.identity.projects import (
    views as project_views)
from openstack_dashboard.usage import quotas
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients.base import tenant_list
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards import utils
from tionix_dashboard.dashboards.identity.projects.tabs import (
    ProjectDetailTabs)
from tionix_dashboard.dashboards.identity.projects.utils import (
    get_project_group_for_schedule, AGGREGATE_PROJECT_SEP)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ObjectTasksView)
from tionix_dashboard.dashboards.identity.projects.forms import (
    ManageHostAggregatesForm, DeleteTenantForm, TnxDeleteTenantForm,
    SetUpMonitoringForm)
from tionix_dashboard.dashboards.identity.projects.workflows import (
    CreateProject, UpdateProject, TnxCreateProject, TnxUpdateProject,
    TnxMultiUpdateProject)
from tionix_dashboard.dashboards.identity.projects.tables import (
    TnxTenantsTable, TenantsTable)
from tionix_dashboard.dashboards.utils import get_tenants_kwargs
from tionix_dashboard.helpers.views import (
    ServerFilterViewMixin, ServerSortMultiSelect, LinkBatchWorkflowViewMixin,
    WorkflowWithLazyMembers)
from tionix_dashboard.helpers.workflows import (
    WorkflowVerticalViewMixin, WorkflowVerticalView)

logger = logging.getLogger(__name__)


# noinspection PyClassHasNoInit
class TnxIndexView(IdentityMixIn, project_views.IndexView):
    """"Tionix tenants list view"""
    table_class = TnxTenantsTable

    # noinspection PyAttributeOutsideInit,PyProtectedMember
    # pylint: disable=attribute-defined-outside-init,no-member
    def get_data(self):
        """return data for table"""
        tenants, self._more = [], False

        # pylint: disable=protected-access
        marker = self.request.GET.get(
            TnxTenantsTable._meta.pagination_param, None)

        kwargs = {
            'paginate': True,
            'marker': marker
        }

        # check the case when domain admin is not admin of a current project
        admin_roles = self.get_admin_roles()
        is_project_admin = any(role.get('name') in admin_roles
                               for role in self.request.user.roles)
        is_domain_admin = api.keystone.is_domain_admin(self.request)
        admin_check = (not is_domain_admin or is_domain_admin and
                       is_project_admin)

        if admin_check and policy.check(
                (("identity", "identity:list_projects"),), self.request):
            kwargs.update(
                {'domain': api.keystone.get_effective_domain_id(self.request)}
            )
        elif admin_check and policy.check(
                (("identity", "identity:list_user_projects"),), self.request):
            kwargs.update({
                'user': self.request.user.id,
                'admin': False
            })
        else:
            msg = _("Insufficient privilege level to view "
                    "project information.")
            messages.info(self.request, msg)
            if not admin_check:
                tenants = super(TnxIndexView, self).get_data()
            return tenants

        try:
            tenants, _more = TNXTenantClient.get_paginator(
                self.request, **kwargs)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _("Unable to retrieve VDI project list."))
            tenants = super(TnxIndexView, self).get_data()
        else:
            if api.keystone.VERSIONS.active >= 3:
                domains_dict = utils.domain_lookup(self.request)
                if domains_dict:
                    for tenant in tenants:
                        tenant.domain_name = domains_dict.get(tenant.domain_id)
        return tenants


class CreateProjectView(WorkflowVerticalViewMixin,
                        project_views.CreateProjectView):
    workflow_class = CreateProject


class UpdateProjectView(WorkflowVerticalViewMixin,
                        project_views.UpdateProjectView):
    workflow_class = UpdateProject


class TnxCreateProjectView(WorkflowWithLazyMembers, WorkflowVerticalView):
    """Tionix create project view"""
    workflow_class = TnxCreateProject

    def get_initial(self):
        # Overrides default horizon CreateProjectView.get_initial to allow
        # domain admin to set project quotas.

        initial = super(TnxCreateProjectView, self).get_initial()

        # Set the domain of the project
        domain = api.keystone.get_default_domain(self.request)
        initial["domain_id"] = domain.id
        initial["domain_name"] = domain.name
        # get initial quota defaults
        # here original is_cloud_admin check is replaced by is_domain_admin
        if api.keystone.is_domain_admin(self.request):
            try:
                quota_defaults = quotas.get_default_quota_data(self.request)

                try:
                    if (api.base.is_service_enabled(
                            self.request, 'network') and
                            api.neutron.is_quotas_extension_supported(
                                self.request)):
                        project_id = self.request.user.project_id
                        quota_defaults += api.neutron.tenant_quota_get(
                            self.request,
                            tenant_id=project_id)
                except Exception as err:
                    logger.error(unicode(err))
                    error_msg = _('Unable to retrieve default Neutron quota '
                                  'values.')
                    self.add_error_to_step(error_msg, 'create_quotas')

                for field in quotas.QUOTA_FIELDS:
                    initial[field] = quota_defaults.get(field).limit

            except Exception as err:
                logger.error(unicode(err))
                error_msg = _('Unable to retrieve default quota values.')
                self.add_error_to_step(error_msg, 'create_quotas')

        return initial


class TnxUpdateProjectView(WorkflowWithLazyMembers, WorkflowVerticalView):
    """Tionix update project view"""
    workflow_class = TnxUpdateProject

    def get_initial(self):
        # Overrides default horizon UpdateProjectView.get_initial to allow
        # domain admin to update project quotas.

        initial = super(TnxUpdateProjectView, self).get_initial()

        project_id = self.kwargs['tenant_id']
        initial['project_id'] = project_id

        try:
            # get initial project info
            project_info = api.keystone.tenant_get(self.request, project_id)
            for field in project_views.PROJECT_INFO_FIELDS:
                initial[field] = getattr(project_info, field, None)

            # Retrieve the domain name where the project belong
            if api.keystone.VERSIONS.active >= 3:
                try:
                    if policy.check((("identity", "identity:get_domain"),),
                                    self.request):
                        domain = api.keystone.domain_get(self.request,
                                                         initial["domain_id"])
                        initial["domain_name"] = domain.name

                    else:
                        domain = api.keystone.get_default_domain(self.request)
                        initial["domain_name"] = domain.name

                except Exception as err:
                    logger.error(unicode(err))
                    exceptions.handle(
                        self.request,
                        _('Unable to retrieve project domain.'),
                        redirect=reverse(project_views.INDEX_URL))

            # get initial project quota
            if api.keystone.is_domain_admin(self.request):
                quota_data = quotas.get_tenant_quota_data(self.request,
                                                          tenant_id=project_id)
                if (api.base.is_service_enabled(self.request, 'network') and
                        api.neutron.is_quotas_extension_supported(
                            self.request)):
                    quota_data += api.neutron.tenant_quota_get(
                        self.request, tenant_id=project_id)
                for field in quotas.QUOTA_FIELDS:
                    initial[field] = quota_data.get(field).limit
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve project details.'),
                              redirect=reverse(project_views.INDEX_URL))
        return initial


class TnxMultiUpdateProjectView(LinkBatchWorkflowViewMixin,
                                WorkflowWithLazyMembers,
                                WorkflowVerticalView):
    workflow_class = TnxMultiUpdateProject
    submit_url = reverse_lazy('horizon:identity:projects:multi_update')

    def get_initial(self):

        initial = super(TnxMultiUpdateProjectView, self).get_initial()

        request_tenants = self.get_object_ids()

        kwargs = get_tenants_kwargs(self.request)
        try:
            tenants, _has_more = api.keystone.tenant_list(
                self.request, **kwargs)
        except Exception as err:
            tenants = []
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve projects list.'),
                              redirect=reverse(
                                  'horizon:identity:projects:index'))
        else:
            tenants = {tenant.id: tenant.name for tenant in tenants
                       if tenant.id in request_tenants}

        initial['projects'] = tenants
        initial['domain_id'] = api.keystone.get_default_domain(
            self.request, get_name=False).id

        return initial


class IndexView(project_views.IndexView):
    """Tenants index view"""
    table_class = TenantsTable


class DetailProjectView(project_views.DetailProjectView):
    """Detail for project"""
    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(DetailProjectView, self).get_context_data(**kwargs)
        table = TenantsTable(self.request)
        context["actions"] = table.render_row_actions(context['project'])
        return context


class TnxDetailProjectView(ServerSortMultiSelect,
                           ServerFilterViewMixin,
                           tabs.TabbedTableView):
    """Tionix detail for project"""

    tab_group_class = ProjectDetailTabs
    template_name = 'horizon/common/_detail.html'
    redirect_url = project_views.INDEX_URL
    page_title = _("Project Details: {{ project.name }}")

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(TnxDetailProjectView, self).get_context_data(**kwargs)
        project = self.get_data()
        table = TnxTenantsTable(self.request)
        context["project"] = project
        context["url"] = reverse(project_views.INDEX_URL)
        context["actions"] = table.render_row_actions(context['project'])
        return context

    @memoized.memoized_method
    def get_data(self):
        project, project_id = None, self.kwargs['tenant_id']
        try:
            project = TNXTenantClient.get(self.request, project_id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve project details.'),
                              redirect=self.redirect_url)
        return project

    def get_tabs(self, request, **kwargs):
        kwargs['project'] = self.get_data()
        return self.tab_group_class(request, **kwargs)


class ManageHostAggregatesView(forms.ModalFormView):
    """Manage host aggregate view"""
    form_class = ManageHostAggregatesForm
    modal_header = _('Manage Host Aggregates')
    template_name = 'projects/manage_host_aggregates.html'
    modal_id = "manage_host_aggregates"
    success_url = reverse_lazy("horizon:identity:projects:index")

    def get_context_data(self, **kwargs):
        """context data"""
        context = super(ManageHostAggregatesView, self).get_context_data(
            **kwargs)
        context['project_id'] = self.kwargs['project_id']
        return context

    def get_initial(self):
        """initial data"""
        return self.kwargs


@rest_urls.register
class HostAggregates(generic.View):
    """API for nova aggregates.
    """
    url_regex = r'tnx/aggregates/(?P<project_id>[^/]+)/$'

    @rest_utils.ajax()
    def get(self, request, project_id):
        """Get a list of aggregates.
        The listing result is an object with property "items".
        """

        available, allocated = [], []
        try:
            aggregates = api.nova.aggregate_details_list(self.request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve host aggregates list.'))
        else:
            tenants = {}
            try:
                tenants = {tenant.id: tenant.name
                           for tenant in tenant_list(request)}
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to get projects list.'))

            for aggregate in aggregates:
                tenant_ids = []
                aggregate_dict = aggregate.to_dict()
                filter_tenant_id = aggregate.metadata.get('filter_tenant_id')
                if filter_tenant_id:
                    tenant_ids = filter_tenant_id.split(AGGREGATE_PROJECT_SEP)
                aggregate_dict['tenants'] = [tenants.get(tenant_id, tenant_id)
                                             for tenant_id in tenant_ids]
                if project_id in tenant_ids:
                    allocated.append(aggregate_dict)
                available.append(aggregate_dict)

        return {'available': available, 'allocated': allocated}


class DeleteTenantView(ObjectTasksView, forms.ModalFormView):
    """ Delete openstack tenants view """
    form_class = DeleteTenantForm
    modal_header = _('Delete Confirmation')
    template_name = "tionix_dashboard/delete_object.html"
    modal_id = "delete_tenant"
    submit_label = _('Delete')
    context_object_name = 'tenant'
    success_url = reverse_lazy("horizon:identity:projects:index")
    confirm_url = reverse_lazy("horizon:identity:projects:delete")
    hidden_title = False
    msg = _("Deleted projects are not recoverable.")

    def get_initial(self):
        names = {}
        ids = self.get_object_ids()
        for tenant_id in ids:
            try:
                tenant = api.keystone.tenant_get(self.request, tenant_id)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve details for '
                                    'tenant "%s".') % tenant_id,
                                  redirect=self.get_redirect_url())
                raise exceptions.Http302(self.get_redirect_url())
            else:
                names[tenant_id] = getattr(tenant, "name", None)

        query_params = {
            'subject_id': ids,
            'subject_type': ObjectTypeEnum.project,
            'subject_group': ObjectGroupEnum.identity,
            'status': True,
        }
        rows = self.get_tasks_rows(self.request, query_params)
        return {
            'rows': rows,
            'object_names_dict': names,
            'ids': ids,
            'msg': self.msg,
            'confirm_url': self.confirm_url,
            'redirect_url': self.get_redirect_url(),
        }

    def get_redirect_url(self):
        return self.request.META['HTTP_REFERER']


class TnxDeleteTenantView(DeleteTenantView):
    """ Delete tnx tenants view """
    form_class = TnxDeleteTenantForm

    def get_initial(self):
        ids = self.get_object_ids()
        names, tenants_with_instances = {}, []
        projects_groups_dict = {}

        for tenant_id in ids:

            try:
                tenant = TNXTenantClient.get(self.request, tenant_id)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve details for '
                                    'tenant "%s".') % tenant_id,
                                  redirect=self.get_redirect_url())
                raise exceptions.Http302(self.get_redirect_url())
            tenant_name, tenant_id = tenant.name, tenant.id
            project_group = get_project_group_for_schedule(tenant)
            names[tenant_id] = tenant_name

            if getattr(tenant, 'tnx_instances_count', None):
                tenants_with_instances.append(tenant_name)
            else:
                projects_groups_dict[tenant_id] = (tenant.id, tenant_name,
                                                   project_group)

        query_params = {
            'subject_id': ids,
            'subject_type': ObjectTypeEnum.project,
            'status': True,
        }
        rows = self.get_tasks_rows(self.request, query_params)
        return {
            'rows': rows,
            'object_names_dict': names,
            'ids': ids,
            'msg': self.msg,
            'confirm_url': self.confirm_url,
            'redirect_url': self.get_redirect_url(),
            'projects_groups': projects_groups_dict,
            'tenants_with_instances': tenants_with_instances,
        }


class SetUpMonitoringView(forms.ModalFormView):
    """Set up monitoring view"""
    form_class = SetUpMonitoringForm
    modal_header = _('Set up monitoring')
    template_name = "projects/set_up_monitoring.html"
    modal_id = "set_up_monitoring"
    submit_label = _('Set Up')
    success_url = reverse_lazy("horizon:identity:projects:index")

    def get_context_data(self, **kwargs):
        """context data"""
        context = super(SetUpMonitoringView, self).get_context_data(**kwargs)
        context['project_id'] = self.kwargs['project_id']
        return context

    def get_initial(self):
        """initial data"""
        return self.kwargs
