# coding: utf-8
# pylint: disable=too-many-lines

import functools
import logging
from collections import defaultdict

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.core import validators

from horizon import forms, workflows, exceptions, messages

from openstack_dashboard import api
from openstack_dashboard.dashboards.identity.projects import (
    workflows as proj_workflows)
from openstack_dashboard.usage import quotas

from tionix_dashboard.dashboards.identity.projects.utils import (
    remove_project_cache, get_role_list
)
from tionix_dashboard.helpers.fields import AutoGenNameCharField
from tionix_dashboard.helpers.workflows import (
    OptionalFieldsActionMixin, LazyMembershipAction, LazyUpdateMembersStep)


logger = logging.getLogger(__name__)


def post_remove_project_cache(f):

    @functools.wraps(f)
    def wrapped(self, *args, **kwargs):
        result = f(self, *args, **kwargs)
        if hasattr(self, 'request'):
            remove_project_cache(self.request.user.unscoped_token)
        return result

    return wrapped


# noinspection PyClassHasNoInit
class CreateProjectInfoAction(proj_workflows.CreateProjectInfoAction):
    name = AutoGenNameCharField(label=_("Name"), max_length=64)

    class Meta(object):
        name = _("Project Information")
        help_text = _("Create a project to organize users.")


# noinspection PyClassHasNoInit
class CreateProjectInfo(proj_workflows.CreateProjectInfo):
    action_class = CreateProjectInfoAction


# noinspection PyClassHasNoInit
class UpdateProjectInfoAction(proj_workflows.UpdateProjectInfoAction):
    name = AutoGenNameCharField(label=_("Name"), max_length=64)

    class Meta(object):
        name = _("Project Information")
        slug = 'update_info'
        help_text = _("Edit the project details.")


# noinspection PyClassHasNoInit
class UpdateProjectInfo(proj_workflows.UpdateProjectInfo):
    action_class = UpdateProjectInfoAction


class MultiUpdateProjectInfoAction(workflows.Action):
    description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)
    enabled = forms.BooleanField(label=_("Enabled"),
                                 required=False,
                                 initial=True)

    def clean(self):
        cleaned_data = super(MultiUpdateProjectInfoAction, self).clean()
        projects = self.initial['projects']
        current_project_id = self.request.user.tenant_id
        if current_project_id in projects and not cleaned_data['enabled']:
            raise forms.ValidationError(
                _('Current project "{0}" is selected. You cannot disable your '
                  'current project.').format(projects[current_project_id])
            )
        return cleaned_data

    class Meta(object):
        name = _("Project Information")
        slug = 'update_info'
        help_text = _("Edit the project details.")


class MultiUpdateProjectInfo(proj_workflows.UpdateProjectInfo):
    action_class = MultiUpdateProjectInfoAction
    template_name = proj_workflows.COMMON_HORIZONTAL_TEMPLATE
    depends_on = ("projects",)
    contributes = ("description", "enabled")


class TnxUpdateProjectMembersAction(LazyMembershipAction):

    def _populate_fields(self, request, workflow, **kwargs):
        err_msg = _('Unable to retrieve user list. Please try again later.')
        domain_id = (
            self.initial.get("domain_id", None) or
            request.POST.get('domain_id', None)
        )
        project_id = kwargs.get('tenant_id', None)

        # Get the default role
        try:
            default_role = api.keystone.get_default_role(request)
            # Default role is necessary to add members to a project
            if default_role is None:
                default = getattr(settings,
                                  "OPENSTACK_KEYSTONE_DEFAULT_ROLE", None)
                msg = (_('Could not find default role "%s" in Keystone') %
                       default)
                raise exceptions.WorkflowError(msg)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.WorkflowError(err_msg)
        default_role_name = self.get_default_role_field_name()
        self.fields[default_role_name] = forms.CharField(required=False)
        self.fields[default_role_name].initial = default_role.id

        # Get list of available users
        try:
            all_users = api.keystone.user_list(request, domain=domain_id)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.WorkflowError(err_msg)
        users_list = [(user.id, user.name) for user in all_users]

        # Get list of roles
        try:
            role_list = api.keystone.role_list(request)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.WorkflowError(err_msg)

        # Figure out users & roles
        project_roles = defaultdict(list)
        if project_id:
            try:
                users_roles = api.keystone.get_project_users_roles(request,
                                                                   project_id)
            except Exception as err:
                logger.error(unicode(err))
                messages.error(request, _('Unable to get users of project.'))
            else:
                for user_id in users_roles:
                    roles_ids = users_roles[user_id]
                    for role_id in roles_ids:
                        field_name = self.get_member_field_name(role_id)
                        project_roles[field_name].append(user_id)

        for role in role_list:
            field_name = self.get_member_field_name(role.id)
            label = role.name
            self.fields[field_name] = forms.MultipleChoiceField(required=False,
                                                                label=label)
            self.fields[field_name].choices = users_list
            session_members = request.session.get('selected_members', None)
            if session_members is not None and field_name in session_members:
                initial_members = session_members.get(field_name, [])
            else:
                initial_members = project_roles.get(field_name, [])
            self.fields[field_name].initial = initial_members

    class Meta(object):
        name = _("Project Members")
        slug = proj_workflows.PROJECT_USER_MEMBER_SLUG


class TnxUpdateProjectGroupsAction(LazyMembershipAction):
    def _populate_fields(self, request, workflow, **kwargs):
        err_msg = _('Unable to retrieve group list. Please try again later.')
        # Use the domain_id from the project
        domain_id = (
            self.initial.get("domain_id", None) or
            request.POST.get('domain_id', None)
        )
        project_id = kwargs.get('tenant_id', None)

        # Get the default role
        try:
            default_role = api.keystone.get_default_role(self.request)
            # Default role is necessary to add members to a project
            if default_role is None:
                default = getattr(settings,
                                  "OPENSTACK_KEYSTONE_DEFAULT_ROLE", None)
                msg = (_('Could not find default role "%s" in Keystone') %
                       default)
                raise exceptions.NotFound(msg)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.WorkflowError(err_msg)
        default_role_name = self.get_default_role_field_name()
        self.fields[default_role_name] = forms.CharField(required=False)
        self.fields[default_role_name].initial = default_role.id

        # Get list of available groups
        try:
            all_groups = api.keystone.group_list(request,
                                                 domain=domain_id)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.WorkflowError(err_msg)
        groups_list = [(group.id, group.name) for group in all_groups]

        # Get list of roles
        try:
            role_list = api.keystone.role_list(request)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.WorkflowError(err_msg)

        # Figure out groups & roles
        project_groups = defaultdict(list)
        if project_id:
            try:
                groups_roles = api.keystone.get_project_groups_roles(
                    request, project_id)
            except Exception as err:
                logger.error(unicode(err))
                messages.error(request, _('Unable to get groups of project.'))
            else:
                for group_id in groups_roles:
                    roles_ids = groups_roles[group_id]
                    for role_id in roles_ids:
                        field_name = self.get_member_field_name(role_id)
                        project_groups[field_name].append(group_id)

        for role in role_list:
            field_name = self.get_member_field_name(role.id)
            label = role.name
            self.fields[field_name] = forms.MultipleChoiceField(required=False,
                                                                label=label)
            self.fields[field_name].choices = groups_list
            session_members = request.session.get('selected_members', None)
            if session_members is not None and field_name in session_members:
                initial_members = session_members.get(field_name, [])
            else:
                initial_members = project_groups.get(field_name, [])
            self.fields[field_name].initial = initial_members

    class Meta(object):
        name = _("Project Groups")
        slug = proj_workflows.PROJECT_GROUP_MEMBER_SLUG


class TnxUpdateProjectMembers(LazyUpdateMembersStep):
    action_class = TnxUpdateProjectMembersAction
    available_list_title = _("All Users")
    members_list_title = _("Project Members")
    no_available_text = _("No users found.")
    no_members_text = _("No users.")

    def _get_members(self):
        return get_role_list(self.workflow.request)


# noinspection PyClassHasNoInit
class TnxUpdateProjectGroups(LazyUpdateMembersStep):
    action_class = TnxUpdateProjectGroupsAction
    available_list_title = _("All Groups")
    members_list_title = _("Project Groups")
    no_available_text = _("No groups found.")
    no_members_text = _("No groups.")

    def _get_members(self):
        return get_role_list(self.workflow.request)


# noinspection PyClassHasNoInit
class MultiUpdateProjectMembersStep(TnxUpdateProjectMembers):
    depends_on = ('domain_id',)


# noinspection PyClassHasNoInit
class MultiUpdateProjectGroupsStep(TnxUpdateProjectGroups):
    depends_on = ('domain_id',)


class ClearUserProjectCacheMixin(object):

    @post_remove_project_cache
    def handle(self, *args, **kwargs):
        return super(ClearUserProjectCacheMixin, self).handle(*args, **kwargs)


class ProjectQuotaActionMixin(object):

    def displayed_fields(self):
        return [self[name] for name, field in self.fields.items()
                if not isinstance(field.widget, forms.HiddenInput)]


class CreateProjectQuotaAction(ProjectQuotaActionMixin,
                               proj_workflows.CreateProjectQuotaAction):
    class Meta(object):
        name = _("Quota")
        slug = 'create_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.compute')


# noinspection PyClassHasNoInit
class CreateProjectQuota(proj_workflows.CreateProjectQuota):
    action_class = CreateProjectQuotaAction
    template_name = "projects/_quota_fields.html"


#  pylint:disable=non-parent-init-called,super-init-not-called
class CreateProject(proj_workflows.CreateProject):
    default_steps = (CreateProjectInfo,
                     proj_workflows.UpdateProjectMembers,
                     proj_workflows.CreateProjectQuota)

    # pylint:disable=keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  proj_workflows.UpdateProjectMembers,
                                  proj_workflows.UpdateProjectGroups,
                                  proj_workflows.CreateProjectQuota)

        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)

    def _update_project_members(self, request, data, project_id):
        # update project members
        users_to_add = 0

        available_roles = get_role_list(request)
        member_step = self.get_step(
            proj_workflows.PROJECT_USER_MEMBER_SLUG
        )

        for role in available_roles:
            field_name = member_step.get_member_field_name(role.id)
            role_list = data.get(field_name)
            if role_list is None:
                continue
            for user in role_list:
                try:
                    api.keystone.add_tenant_user_role(request,
                                                      project=project_id,
                                                      user=user,
                                                      role=role.id)
                except Exception as err:
                    logger.error(unicode(err))
                    users_to_add += 1
        if users_to_add:
            if proj_workflows.PROJECT_GROUP_ENABLED:
                group_msg = _(", add project groups")
            else:
                group_msg = ""
            exceptions.handle(request,
                              _('Failed to add %(users_to_add)s project '
                                'members%(group_msg)s and set project quotas.')
                              % {'users_to_add': users_to_add,
                                 'group_msg': group_msg})

    def _update_project_groups(self, request, data, project_id):
        # update project groups
        groups_to_add = 0

        available_roles = get_role_list(request)
        member_step = self.get_step(
            proj_workflows.PROJECT_GROUP_MEMBER_SLUG
        )

        for role in available_roles:
            field_name = member_step.get_member_field_name(role.id)
            role_list = data.get(field_name)
            if role_list is None:
                continue
            for group in role_list:
                try:
                    api.keystone.add_group_role(request,
                                                role=role.id,
                                                group=group,
                                                project=project_id)
                except Exception as err:
                    logger.error(unicode(err))
                    groups_to_add += 1
        if groups_to_add:
            exceptions.handle(request,
                              _('Failed to add %s project groups '
                                'and update project quotas.')
                              % groups_to_add)

    def handle(self, request, data):
        data['description'] = data.get('description', '').strip()
        return super(CreateProject, self).handle(request, data)


class UpdateProject(proj_workflows.UpdateProject):
    default_steps = (UpdateProjectInfo,
                     proj_workflows.UpdateProjectMembers,
                     proj_workflows.UpdateProjectQuota)

    # pylint:disable=keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (UpdateProjectInfo,
                                  proj_workflows.UpdateProjectMembers,
                                  proj_workflows.UpdateProjectGroups,
                                  UpdateProjectQuota)

        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)

    def handle(self, request, data):
        data['description'] = data.get('description', '').strip()
        return super(UpdateProject, self).handle(request, data)


class TnxCreateProject(ClearUserProjectCacheMixin, CreateProject):
    default_steps = (CreateProjectInfo,
                     TnxUpdateProjectMembers,
                     CreateProjectQuota)

    # pylint:disable=keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  TnxUpdateProjectMembers,
                                  TnxUpdateProjectGroups,
                                  CreateProjectQuota)
        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)

    def handle(self, request, data):  # pylint:disable=arguments-differ
        data['description'] = data.get('description', '').strip()
        project = self._create_project(request, data)
        if not project:
            return False
        project_id = project.id
        self._update_project_members(request, data, project_id)
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self._update_project_groups(request, data, project_id)
        if api.keystone.is_domain_admin(request):
            self._update_project_quota(request, data, project_id)
        return True


class UpdateProjectQuotaAction(ProjectQuotaActionMixin,
                               proj_workflows.UpdateProjectQuotaAction):
    class Meta(object):
        name = _("Quota")
        slug = 'update_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.compute')


# noinspection PyClassHasNoInit
class UpdateProjectQuota(proj_workflows.UpdateProjectQuota):
    action_class = UpdateProjectQuotaAction
    template_name = "projects/_quota_fields.html"


class MultiUpdateProjectQuotaAction(OptionalFieldsActionMixin,
                                    UpdateProjectQuotaAction):
    def clean(self):
        bad_projects = defaultdict(list)
        for project_id, project_name in self.initial['projects'].items():
            usages = quotas.tenant_quota_usages(self.request,
                                                tenant_id=project_id)
            # Validate the quota values before updating quotas.
            for key, value in self.cleaned_data.items():
                used = usages[key].get('used', 0)
                if value is not None and 0 <= value < used:
                    bad_projects[project_name].append(
                        _('{used} {key} used').format(
                            used=used, key=quotas.QUOTA_NAMES.get(key, key)
                        )
                    )
        if bad_projects:
            value_str = u''.join(
                u'<p><strong>{0}</strong>: '
                u'{1};</p>'.format(project, u', '.join(values))
                for project, values in bad_projects.items()
            )
            msg = _('Quota value(s) cannot be less than the current usage '
                    'value(s): {0}').format(value_str)
            raise forms.ValidationError(mark_safe(msg))
        return self.cleaned_data

    class Meta(object):
        name = _("Quota")
        slug = 'update_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.compute')


class MultiUpdateProjectQuota(UpdateProjectQuota):
    action_class = MultiUpdateProjectQuotaAction
    depends_on = ("projects",)


class TnxUpdateProject(ClearUserProjectCacheMixin,
                       proj_workflows.UpdateProject):
    default_steps = (UpdateProjectInfo,
                     TnxUpdateProjectMembers,
                     UpdateProjectQuota)

    # pylint:disable=keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (UpdateProjectInfo,
                                  TnxUpdateProjectMembers,
                                  TnxUpdateProjectGroups,
                                  UpdateProjectQuota)

        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)

    def _add_roles_to_users(self, request, data, project_id, user_id,
                            role_ids, available_roles):
        member_step = self.get_step(proj_workflows.PROJECT_USER_MEMBER_SLUG)
        current_role_ids = list(role_ids)

        for role in available_roles:
            field_name = member_step.get_member_field_name(role.id)
            field_data = data.get(field_name)
            if field_data is not None:
                # Check if the user is in the list of users with this role.
                if user_id in field_data:
                    if role.id not in current_role_ids:
                        # user role has changed
                        api.keystone.add_tenant_user_role(
                            request,
                            project=project_id,
                            user=user_id,
                            role=role.id)
                        continue
                else:
                    continue
            if role.id in current_role_ids:
                # User role is unchanged, so remove it from the
                # remaining roles list to avoid removing it later.
                index = current_role_ids.index(role.id)
                current_role_ids.pop(index)
        return current_role_ids

    def _update_project_members(self, request, data, project_id):
        # update project members
        result = True
        fail_message = _('Failed to modify %(users_to_modify)s project '
                         'members%(group_msg)s and update project quotas.')
        users_to_modify = 0
        # Project-user member step
        member_step = self.get_step(proj_workflows.PROJECT_USER_MEMBER_SLUG)

        # Get our role options
        available_roles = get_role_list(request)
        try:
            # Get the users currently associated with this project so we
            # can diff against it.
            users_roles = api.keystone.get_project_users_roles(
                request, project=project_id)
            users_to_modify = len(users_roles)

            # TODO(bpokorny): The following lines are needed to make sure we
            # only modify roles for users who are in the current domain.
            # Otherwise, we'll end up removing roles for users who have roles
            # on the project but aren't in the domain.  For now, Horizon won't
            # support managing roles across domains.  The Keystone CLI
            # supports it, so we may want to add that in the future.
            all_users = api.keystone.user_list(request,
                                               domain=data['domain_id'])
            users_dict = {user.id: user.name for user in all_users}

            for user_id in users_roles.keys():
                # Don't remove roles if the user isn't in the domain
                if user_id not in users_dict:
                    users_to_modify -= 1
                    continue

                # Check if there have been any changes in the roles of
                # Existing project members.
                current_role_ids = list(users_roles[user_id])
                modified_role_ids = self._add_roles_to_users(
                    request, data, project_id, user_id,
                    current_role_ids, available_roles)
                # Prevent admins from doing stupid things to themselves.
                removing_admin = self._is_removing_self_admin_role(
                    request, project_id, user_id, available_roles,
                    modified_role_ids)
                # Otherwise go through and revoke any removed roles.
                if not removing_admin:
                    self._remove_roles_from_user(request, project_id, user_id,
                                                 modified_role_ids)
                users_to_modify -= 1
        except Exception as err:
            result = False
            logger.error(unicode(err))
            if proj_workflows.PROJECT_GROUP_ENABLED:
                group_msg = _(", update project groups")
            else:
                group_msg = ""
            exceptions.handle(request,
                              fail_message
                              % {'users_to_modify': users_to_modify,
                                 'group_msg': group_msg})
        else:
            failed_users_count = 0
            # Grant new roles on the project.
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                field_data = data.get(field_name)
                if field_data is None:
                    continue
                for user_id in field_data:
                    if user_id in users_roles:
                        continue
                    try:
                        api.keystone.add_tenant_user_role(
                            request,
                            project=project_id,
                            user=user_id,
                            role=role.id)
                    except Exception as err:
                        logger.error(unicode(err))
                        failed_users_count += 1
            if failed_users_count:
                result = False
                if proj_workflows.PROJECT_GROUP_ENABLED:
                    group_msg = _(", update project groups")
                else:
                    group_msg = ""
                exceptions.handle(request,
                                  fail_message
                                  % {'users_to_modify': users_to_modify,
                                     'group_msg': group_msg})
        return result

    def _update_project_groups(self, request, data, project_id, domain_id):
        # update project groups
        result = True
        fail_message = _('Failed to modify %s project members, update '
                         'project groups and update project quotas.')
        groups_to_modify = 0
        member_step = self.get_step(proj_workflows.PROJECT_GROUP_MEMBER_SLUG)

        available_roles = get_role_list(request)
        try:
            # Get the groups currently associated with this project so we
            # can diff against it.
            project_groups = [
                group.id
                for group in api.keystone.group_list(request,
                                                     domain=domain_id,
                                                     project=project_id)
            ]
            groups_to_modify = len(project_groups)
            for group_id in project_groups:
                # Check if there have been any changes in the roles of
                # Existing project members.
                current_roles = api.keystone.roles_for_group(
                    self.request,
                    group=group_id,
                    project=project_id)
                current_role_ids = [role.id for role in current_roles]
                for role in available_roles:
                    # Check if the group is in the list of groups with
                    # this role.
                    field_name = member_step.get_member_field_name(role.id)
                    field_data = data.get(field_name)
                    if field_data is not None:
                        # Check if the group is in the list of groups with
                        # this role.
                        if group_id in field_data:
                            # Add it if necessary
                            if role.id not in current_role_ids:
                                # user role has changed
                                api.keystone.add_group_role(
                                    request,
                                    role=role.id,
                                    group=group_id,
                                    project=project_id)
                                continue
                        else:
                            continue
                    if role.id in current_role_ids:
                        # Group role is unchanged, so remove it from
                        # the remaining roles list to avoid removing it
                        # later.
                        index = current_role_ids.index(role.id)
                        current_role_ids.pop(index)

                # Revoke any removed roles.
                for id_to_delete in current_role_ids:
                    api.keystone.remove_group_role(request,
                                                   role=id_to_delete,
                                                   group=group_id,
                                                   project=project_id)
                groups_to_modify -= 1
        except Exception as err:
            result = False
            logger.error(unicode(err))
            exceptions.handle(request, fail_message % groups_to_modify)
        else:
            groups_to_add = 0
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                role_list = data.get(field_name)
                if role_list is None:
                    continue

                for group in role_list:
                    if group in project_groups:
                        continue
                    try:
                        api.keystone.add_group_role(request,
                                                    role=role.id,
                                                    group=group,
                                                    project=project_id)
                    except Exception as err:
                        logger.error(unicode(err))
                        groups_to_add += 1
                if groups_to_add:
                    result = False
                    exceptions.handle(request, fail_message % groups_to_add)
        return result

    def handle(self, request, data):  # pylint:disable=arguments-differ
        data['description'] = data.get('description', '').strip()
        project = self._update_project(request, data)
        if not project:
            return False

        project_id = data['project_id']
        # Use the domain_id from the project if available
        domain_id = getattr(project, "domain_id", '')

        ret = self._update_project_members(request, data, project_id)
        if not ret:
            return False

        if proj_workflows.PROJECT_GROUP_ENABLED:
            ret = self._update_project_groups(request, data,
                                              project_id, domain_id)
            if not ret:
                return False

        if api.keystone.is_domain_admin(request):
            ret = self._update_project_quota(request, data, project_id)
            if not ret:
                return False

        return True


class TnxMultiUpdateProject(ClearUserProjectCacheMixin,
                            proj_workflows.UpdateProject):
    name = _("Edit Projects")
    success_message = _('Successfully updated projects: "{0}".')
    failure_message = _('Unable to update projects "{0}".')

    # pylint:disable=keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):

        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (MultiUpdateProjectInfo,
                                  MultiUpdateProjectMembersStep,
                                  MultiUpdateProjectGroupsStep,
                                  MultiUpdateProjectQuota)
        else:
            self.default_steps = (MultiUpdateProjectInfo,
                                  MultiUpdateProjectMembersStep,
                                  MultiUpdateProjectQuota)
        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)

    def get_absolute_url(self):
        return self.request.get_full_path()

    def _update_project_members(self, request, data, project_id):
        """Updates members without deleting existing"""
        # update project members
        users_to_modify = 0
        # Project-user member step
        member_step = self.get_step(proj_workflows.PROJECT_USER_MEMBER_SLUG)
        result = True
        try:
            # Get our role options
            available_roles = self._get_available_roles(request)
            # Get the users currently associated with this project so we
            # can diff against it.
            users_roles = api.keystone.get_project_users_roles(
                request, project=project_id)
            users_to_modify = len(users_roles)

            # TODO(bpokorny): The following lines are needed to make sure we
            # only modify roles for users who are in the current domain.
            # Otherwise, we'll end up removing roles for users who have roles
            # on the project but aren't in the domain.  For now, Horizon won't
            # support managing roles across domains.  The Keystone CLI
            # supports it, so we may want to add that in the future.
            all_users = api.keystone.user_list(request,
                                               domain=data['domain_id'])
            users_dict = {user.id: user.name for user in all_users}

            for user_id in users_roles.keys():
                # Don't remove roles if the user isn't in the domain
                if user_id not in users_dict:
                    users_to_modify -= 1
                    continue

                # Check if there have been any changes in the roles of
                # Existing project members.
                current_role_ids = list(users_roles[user_id])
                self._add_roles_to_users(
                    request, data, project_id, user_id, current_role_ids,
                    available_roles)
                users_to_modify -= 1

            # Grant new roles on the project.
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                # Count how many users may be added for exception handling.
                users_to_modify += len(data.get(field_name) or [])
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                data_field = data.get(field_name)
                if data_field is not None:
                    users_added = 0
                    for user_id in data_field:
                        if user_id not in users_roles:
                            api.keystone.add_tenant_user_role(
                                request,
                                project=project_id,
                                user=user_id,
                                role=role.id)
                        users_added += 1
                    users_to_modify -= users_added
        except Exception as err:
            logger.error(unicode(err))
            group_msg = (
                _(", update project groups")
                if proj_workflows.PROJECT_GROUP_ENABLED else ''
            )
            exceptions.handle(request,
                              _('Failed to modify %(users_to_modify)s'
                                ' project members%(group_msg)s and '
                                'update project quotas.')
                              % {'users_to_modify': users_to_modify,
                                 'group_msg': group_msg})
            result = False
        return result

    def _update_project_groups(self, request, data, project_id, domain_id):
        """Updates groups without deleting existing"""
        # update project groups
        groups_to_modify = 0
        member_step = self.get_step(proj_workflows.PROJECT_GROUP_MEMBER_SLUG)
        result = True
        try:
            available_roles = self._get_available_roles(request)
            # Get the groups currently associated with this project so we
            # can diff against it.
            project_groups = api.keystone.group_list(request,
                                                     domain=domain_id,
                                                     project=project_id)
            groups_to_modify = len(project_groups)
            for group in project_groups:
                # Check if there have been any changes in the roles of
                # Existing project members.
                current_roles = api.keystone.roles_for_group(
                    self.request,
                    group=group.id,
                    project=project_id)
                current_role_ids = [role.id for role in current_roles]
                for role in available_roles:
                    # Check if the group is in the list of groups with
                    # this role.
                    field_name = member_step.get_member_field_name(role.id)
                    field_data = data.get(field_name)
                    if field_data is not None and group.id in field_data:
                        # Add it if necessary
                        if role.id not in current_role_ids:
                            # group role has changed
                            api.keystone.add_group_role(
                                request,
                                role=role.id,
                                group=group.id,
                                project=project_id)
                groups_to_modify -= 1

            # Grant new roles on the project.
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                # Count how many groups may be added for error handling.
                groups_to_modify += len(data.get(field_name) or [])
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                field_data = data.get(field_name)
                if field_data is not None:
                    groups_added = 0
                    for group_id in data.get(field_name):
                        if not [x for x in project_groups if group_id == x.id]:
                            api.keystone.add_group_role(request,
                                                        role=role.id,
                                                        group=group_id,
                                                        project=project_id)
                        groups_added += 1
                    groups_to_modify -= groups_added
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Failed to modify %s project '
                                'members, update project groups '
                                'and update project quotas.')
                              % groups_to_modify)
            result = False
        return result

    def _update_project_quota(self, request, data, project_id):
        """Updates only filled quotas"""
        # Update the project quota.
        nova_data = dict([(key, data[key])
                          for key in quotas.NOVA_QUOTA_FIELDS if key in data])
        api.nova.tenant_quota_update(request, project_id, **nova_data)

        if api.cinder.is_volume_service_enabled(request):
            cinder_data = dict([(key, data[key])
                                for key in quotas.CINDER_QUOTA_FIELDS
                                if key in data])
            api.cinder.tenant_quota_update(request, project_id, **cinder_data)

        if (api.base.is_service_enabled(request, 'network') and
                api.neutron.is_quotas_extension_supported(request)):
            disabled_quotas = quotas.get_disabled_quotas(request)
            neutron_data = dict(
                [(key, data[key])
                 for key in quotas.NEUTRON_QUOTA_FIELDS
                 if key in data and key not in disabled_quotas]
            )

            api.neutron.tenant_quota_update(request, project_id,
                                            **neutron_data)

    # pylint: disable=arguments-differ
    @post_remove_project_cache
    def handle(self, request, data):
        def _check_data(key, value):
            return not all([
                value in validators.EMPTY_VALUES,
                not key.startswith(proj_workflows.PROJECT_USER_MEMBER_SLUG),
                not key.startswith(proj_workflows.PROJECT_GROUP_MEMBER_SLUG)])

        data = {key: value for key, value in data.items()
                if _check_data(key, value)}
        success, failed = [], []
        for tenant_id, name in self.context_seed['projects'].items():
            try:
                data['project_id'] = tenant_id
                data['name'] = name
                if 'description' not in data.keys():
                    data['description'] = None
                self._update_project(request, data)
                ret = self._update_project_members(request, data, tenant_id)
                if not ret:
                    failed.append(name)

                if proj_workflows.PROJECT_GROUP_ENABLED:
                    ret = self._update_project_groups(request, data,
                                                      tenant_id,
                                                      data['domain_id'])
                    if not ret:
                        failed.append(name)

                if api.keystone.is_cloud_admin(request):
                    self._update_project_quota(request, data, tenant_id)
                success.append(name)
            except Exception as err:
                logger.error(unicode(err))
                failed.append(name)

        result = True
        if success:
            self.success_message = self.success_message.format(
                '", "'.join(success)
            )

        if failed:
            result = False
            self.failure_message = self.failure_message.format(
                '", "'.join(failed)
            )
        return result
