# coding: utf-8
# pylint: disable=duplicate-code

"""Identity users actions"""

from copy import copy
from django.utils.translation import (
    ugettext_lazy as _, ungettext_lazy, npgettext_lazy)

from horizon import tables

from openstack_dashboard import api, policy
from openstack_dashboard.dashboards.identity.users import (
    tables as users_tables)

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.helpers import get_connection_type_by_name
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxBatchActionMixin)
from tionix_dashboard.dashboards.identity.projects import (
    actions as project_actions)
from tionix_dashboard.journal import JournalFilterAction


# noinspection PyClassHasNoInit
class UpdateCell(users_tables.UpdateCell):
    def allowed(self, request, user, cell):
        policy_rule = (("identity", "identity:update_user"),)
        policy_target = {'target.user.domain_id': user.domain_id}
        return (
            api.keystone.keystone_can_edit_user() and
            policy.check(policy_rule, request, target=policy_target) and
            getattr(user, 'enabled', None) is not None
        )


# noinspection PyAbstractClass,PyClassHasNoInit
class EditUserLink(users_tables.EditUserLink):
    def allowed(self, request, user):
        return (
            api.keystone.keystone_can_edit_user() and
            getattr(user, 'enabled', None) is not None
        )


# noinspection PyClassHasNoInit
class DeleteUsersAction(users_tables.DeleteUsersAction):
    def allowed(self, request, datum):
        allowed = api.keystone.keystone_can_edit_user()
        if datum:
            allowed = (
                allowed and
                datum.id != request.user.id and
                getattr(datum, 'enabled', None) is not None
            )
        return allowed


class ToggleEnabled(users_tables.ToggleEnabled):

    def __init__(self, **kwargs):
        super(ToggleEnabled, self).__init__(**kwargs)
        self.enabled = None

    def allowed(self, request, user=None):
        allowed = api.keystone.keystone_can_edit_user()

        if user:
            self.enabled = getattr(user, 'enabled', None)
            allowed = allowed and self.enabled is not None
            if allowed:
                if self.enabled:
                    self.current_present_action = users_tables.DISABLE
                else:
                    self.current_present_action = users_tables.ENABLE
        return allowed


# noinspection PyAbstractClass,PyClassHasNoInit
class ChangePasswordLink(users_tables.ChangePasswordLink):
    def allowed(self, request, user):
        return (
            api.keystone.keystone_can_edit_user() and
            getattr(user, 'enabled', None) is not None
        )


# noinspection PyAbstractClass
class UserJournalTableFilter(JournalFilterAction):
    name = 'user_journal_filter'

    filter_choices = (
        ('request_id', _("Request ID"), False),
        ('parent_request_id', _("Parent request"), False),
        ('action', _("Action"), False),
        ('created_at', _("Start time"), False),
        ('object_type', _("Subject type"), False),
        ('object_name', _("Subject name"), False),
        ('status', _("Result"), False),
        ('result', _("Result details"), False),
    )


# noinspection PyAbstractClass
class UserSessionFilterAction(TnxFilterAction):
    """Session filter"""
    filter_type = "server"

    filter_choices = [
        ('project_name', _('Project name= '), True),
        ('instance_name', _('Instance name'), True),
        ('created_at', _('Created'), False),
        ('closed_at', _('Closed'), False),
        ('connection_type', _("Connection"), True),
        ('client_info', _("Client"), True),
        ('client_os', _("OS"), True),
        ('client_ip', _("IP"), True)
    ]

    filter_string_modifiers = {
        'connection_type': get_connection_type_by_name
    }


# noinspection PyAbstractClass
class UnassignUserAction(TnxBatchActionMixin, tables.BatchAction):
    """Unassign user from instance"""
    name = 'unassign'
    verbose_name = _('Unassign')
    classes = ("btn-danger",)
    permissions = ('openstack.roles.admin',)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    def confirm_header(self, count):
        return npgettext_lazy(
            "Action header",
            u"Unassign",
            u"Unassign",
            count
        )

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Unassign",
            u"Unassign",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Unassigned",
            u"Unassigned",
            count
        )

    def allowed(self, request, datum):
        allowed = super(UnassignUserAction, self).allowed(request, datum)
        if datum:
            instance_users = [user['id'] for user in datum.tnx_users or []]
            allowed &= self.table.kwargs['user_id'] in instance_users
        return allowed

    def action(self, request, datum_id):
        novaclient(request).tnx_servers.remove_user(
            datum_id, self.table.kwargs['user_id']
        )


# noinspection PyAbstractClass
class UserGroupTableFilter(TnxFilterAction):
    name = 'user_group_filter'
    filter_type = 'server'

    filter_choices = (
        ('name', _("Name"), False),
        ('description', _("Description"), False),
        ('id', _("Group ID"), False),
    )


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxUserTenantsFilterAction(project_actions.TnxTenantFilterAction):
    """Tenants filter with vdi"""
    filter_choices = copy(project_actions.TnxTenantFilterAction.filter_choices)

    filter_choices.append(('roles', _("Roles"), False))
