# coding: utf-8
"""Identity users tables"""
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, pgettext_lazy
from django.template import defaultfilters

from horizon import tables, forms

from openstack_dashboard import api
from openstack_dashboard.dashboards.identity.users import (
    tables as users_tables)

from tionix_client.journal.constants import ObjectType as JournalObjectType
from tionix_dashboard.dashboards.identity.users import actions as user_actions
from tionix_dashboard import journal
from tionix_dashboard.dashboards.tionix.scheduler.helpers import transform_date
from tionix_dashboard.dashboards.tionix.vdi.instances import (
    actions as vdi_actions, tables as vdi_tables)
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import (
    TnxPaginationTabDataTable, TnxLocalPaginationTabDataTable)
from tionix_dashboard.helpers.vm_constants import ConnectionTypeEnum
from tionix_dashboard.templatetags.tionix_tags import format_date
from tionix_dashboard.url_resolve_map import (
    VDI_INSTANCE_DETAILS, VDI_PROJECTS_DETAIL, IDENTITY_PROJECT_DETAIL,
    ADMIN_SERVERS_DETAIL, PROJECT_SERVERS_DETAIL)
from tionix_dashboard.dashboards.identity.projects import (
    tables as project_tables)


# noinspection PyClassHasNoInit
class UserVDIInstancesTable(vdi_tables.BaseVDIInstancesTable):
    """User's vdi instances table for users table"""

    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link=vdi_tables.get_url_for_details,
        form_field=EmptyCharField(max_length=255),
    )

    class Meta(object):
        name = "user_vdi_instances"
        verbose_name = _("Associated VDI instances")
        table_actions = (vdi_actions.VDIInstanceFilterAction,
                         user_actions.UnassignUserAction)
        row_actions = (user_actions.UnassignUserAction,)


class UsersTable(tables.DataTable):
    STATUS_CHOICES = (
        ("true", True),
        ("false", False)
    )
    name = tables.Column('name',
                         link="horizon:identity:users:detail",
                         verbose_name=_('User Name'),
                         form_field=forms.CharField(),
                         update_action=user_actions.UpdateCell)
    description = tables.Column(lambda obj: getattr(obj, 'description', None),
                                verbose_name=_('Description'),
                                hidden=users_tables.KEYSTONE_V2_ENABLED,
                                form_field=forms.CharField(
                                    widget=forms.Textarea(attrs={'rows': 4}),
                                    required=False),
                                update_action=user_actions.UpdateCell)
    email = tables.Column(lambda obj: getattr(obj, 'email', None),
                          verbose_name=_('Email'),
                          form_field=forms.EmailField(required=False),
                          update_action=user_actions.UpdateCell,
                          filters=(lambda v: users_tables.defaultfilters
                                   .default_if_none(v, ""),
                                   users_tables.defaultfilters.escape,
                                   users_tables.defaultfilters.urlize))

    id = tables.Column('id', verbose_name=_('User ID'),
                       attrs={'data-type': 'uuid'})
    enabled = tables.Column('enabled',
                            verbose_name=pgettext_lazy(
                                "Enabled field for identity", u"Enabled"),
                            status=True,
                            status_choices=STATUS_CHOICES,
                            filters=(users_tables.defaultfilters.yesno,
                                     users_tables.defaultfilters.capfirst),
                            empty_value="False")

    if api.keystone.VERSIONS.active >= 3:
        domain_name = tables.Column(
            'domain_name',
            verbose_name=_('Domain Name'),
            attrs={'data-type': 'uuid'})
        enabled = tables.Column('enabled',
                                verbose_name=pgettext_lazy(
                                    "Enabled field for identity", u"Enabled"),
                                status=True,
                                status_choices=STATUS_CHOICES,
                                filters=(users_tables.defaultfilters.yesno,
                                         users_tables.defaultfilters.capfirst),
                                empty_value="False")

    class Meta(object):
        name = "users"
        verbose_name = _("Users")
        row_actions = (
            user_actions.EditUserLink,
            user_actions.ChangePasswordLink,
            user_actions.ToggleEnabled,
            user_actions.DeleteUsersAction
        )
        table_actions = (
            users_tables.UserFilterAction,
            users_tables.CreateUserLink,
            user_actions.DeleteUsersAction
        )
        row_class = users_tables.UpdateRow


class UserJournalTable(journal.JournalBaseTable):
    @staticmethod
    def get_url_for_object(obj, is_admin=False):
        link = None
        if not obj.object_deleted:
            if obj.object_type == JournalObjectType.Project and is_admin:
                link = reverse(IDENTITY_PROJECT_DETAIL, args=(obj.object_id,))
            elif obj.object_type == JournalObjectType.Server:
                link = reverse((ADMIN_SERVERS_DETAIL
                                if is_admin else PROJECT_SERVERS_DETAIL),
                               args=(obj.object_id,))
        return link

    class Meta(object):
        name = "user_action_log"
        verbose_name = _("Action log")
        table_actions = (user_actions.UserJournalTableFilter,)
        css_classes = 'tablesorter tablesorter-server'
        columns = (
            'request_id', 'parent_request_id', 'action', 'start_time',
            'object_type', 'object_name', 'status', 'result'
        )


def get_url_for_instance(datum):
    project_id = getattr(datum, 'project_id', None)
    obj_id = getattr(datum, 'server_id', None)
    return (reverse(VDI_INSTANCE_DETAILS, args=(project_id, obj_id))
            if project_id else None)


def get_url_for_project(datum):
    project_id = getattr(datum, 'project_id', None)
    return reverse(VDI_PROJECTS_DETAIL, args=(project_id,))


class UserSessionTable(TnxPaginationTabDataTable):
    project_name = tables.Column(
        "project_name",
        verbose_name=_("Project name"),
        link=get_url_for_project,
    )
    instance_name = tables.Column(
        "instance_name",
        verbose_name=_("Instance name"),
        link=get_url_for_instance,
    )
    created_at = tables.Column(
        transform_date('created_at'),
        verbose_name=_("Created"),
        filters=(format_date, ))
    closed_at = tables.Column(
        transform_date('closed_at'),
        verbose_name=_("Closed"),
        filters=(format_date, ))
    connection_type = tables.Column(
        "connection_type",
        verbose_name=_('Connection'),
        display_choices=list(ConnectionTypeEnum.values.iteritems()))
    client_info = tables.Column("client_info", verbose_name=_('Client'))
    client_os = tables.Column("client_os", verbose_name=_('OS'))
    client_ip = tables.Column("client_ip", verbose_name=_('IP'))

    class Meta(object):
        name = "sessions"
        verbose_name = _("Sessions")
        multi_select = False
        table_actions = (user_actions.UserSessionFilterAction,)


# noinspection PyClassHasNoInit
class UserGroupsTable(TnxLocalPaginationTabDataTable):
    """User's groups table"""

    name = tables.Column('name', verbose_name=_('Name'))
    description = tables.Column(lambda obj: getattr(obj, 'description', None),
                                verbose_name=_('Description'))
    id = tables.Column('id', verbose_name=_('Group ID'))

    class Meta(object):
        name = "user_groups"
        verbose_name = _("Groups")
        table_actions = (user_actions.UserGroupTableFilter,)


# noinspection PyClassHasNoInit
class ProjectsTable(TnxLocalPaginationTabDataTable,
                    project_tables.TnxTenantsTable):
    name = tables.Column(
        'name',
        verbose_name=_('Name'),
        link=IDENTITY_PROJECT_DETAIL)
    description = tables.Column(
        lambda obj: getattr(obj, 'description', None),
        verbose_name=_('Description'))
    enabled = tables.Column(
        'enabled',
        verbose_name=pgettext_lazy("Enabled field for project", u"Enabled"),
        status=True,
        filters=(defaultfilters.yesno, defaultfilters.capfirst))
    roles = tables.Column('roles',
                          verbose_name=_('Roles'),
                          wrap_list=True,
                          filters=(defaultfilters.unordered_list,))

    class Meta(object):
        name = "projects"
        verbose_name = _("Projects")
        table_actions = (user_actions.TnxUserTenantsFilterAction,)
        multi_select = False
