# coding: utf-8
# pylint: disable=no-member
"""Identity users tabs"""

import logging
from operator import attrgetter
from collections import defaultdict

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs
from horizon.utils import memoized

from openstack_dashboard import api, policy

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.dashboards import utils
from tionix_dashboard.dashboards.helpers import (
    get_sessions_list, get_projects_assignments_roles_by_user)
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.identity.users import tables
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_vdi_instances_extra, get_vdi_instances)
from tionix_dashboard.dashboards.utils import get_tenants_kwargs
from tionix_dashboard.helpers.tabs import CloudAdminTabMixin
from tionix_dashboard.helpers.views import (
    ServerFilterTabMixin, DTVTnxPaginationMixin)
from tionix_dashboard.journal.helpers import UserJournal

logger = logging.getLogger(__name__)


class OverviewTab(tabs.Tab):
    """User overview tab"""
    name = _("Overview")
    slug = "overview"
    template_name = 'users/_detail_overview.html'

    def get_context_data(self, request, **kwargs):
        context = super(OverviewTab, self).get_context_data(request, **kwargs)
        user = self.tab_group.kwargs['user']
        domain_id = getattr(user, "domain_id", None)
        domain_name = ''
        if api.keystone.VERSIONS.active >= 3:
            try:
                if policy.check((("identity", "identity:get_domain"),),
                                self.request):
                    domain = api.keystone.domain_get(
                        self.request, domain_id)
                    domain_name = domain.name
                else:
                    domain = api.keystone.get_default_domain(self.request)
                    domain_name = domain.get('name')
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve project domain.'))
            context["description"] = getattr(user, "description", _("None"))

        context["user"] = user
        tenant = self.get_tenant(user.project_id)
        if tenant:
            context["tenant_name"] = tenant.name
        context["domain_id"] = domain_id
        if user.project_id:
            context["project_id_url"] = reverse(
                'horizon:identity:projects:detail', args=(user.project_id,))
        context["domain_name"] = domain_name
        return context

    @memoized.memoized_method
    def get_tenant(self, project_id):
        tenant = None
        if project_id:
            try:
                tenant = api.keystone.tenant_get(self.request, project_id)
            except Exception as err:
                msg = ('Failed to get tenant %(project_id)s: %(reason)s' %
                       {'project_id': project_id, 'reason': err})
                logger.error(msg)
        return tenant


# noinspection PyAbstractClass
class InstancesTab(CloudAdminTabMixin, ServerFilterTabMixin, tabs.TableTab):
    """Tab for user vdi instances"""
    table_classes = (tables.UserVDIInstancesTable,)
    name = _('Associated VDI instances')
    slug = 'instances_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_user_vdi_instances_data(self):
        search_opts = self.get_filters(
            {'tnx_user': self.tab_group.kwargs['user_id']}
        )

        instances = get_vdi_instances_extra(self.request, search_opts)
        return instances

    def allowed(self, request):
        allowed = super(InstancesTab, self).allowed(request)
        if allowed:
            allowed = LicenseController.is_valid(request,
                                                 LicenseController.VDI)
        return allowed


# noinspection PyAbstractClass
class GroupsTab(ServerFilterTabMixin, DTVTnxPaginationMixin, tabs.TableTab):
    """Tab for user vdi instances"""
    table_classes = (tables.UserGroupsTable,)
    name = _('Groups')
    slug = 'groups_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    # noinspection PyBroadException
    def allowed(self, request):
        user_id = self.tab_group.kwargs['user_id']
        user = self.tab_group.kwargs['user']
        domain_id = getattr(user, "domain_id", None)
        target = {"domain_id": domain_id, "user_id": user_id}
        is_cloud_admin = api.keystone.is_cloud_admin(request)
        if is_cloud_admin:
            allowed = policy.check((("identity", "identity:list_groups"),),
                                   self.request)
        else:
            allowed = policy.check(
                (("identity", "owner"),), request=self.request, target=target)

        return allowed

    def get_user_groups_data(self):
        offset, limit = self.get_offset_limit()
        user_id = self.tab_group.kwargs['user_id']

        user_groups = []
        is_reversed = False
        sort_dir = self.request.session.get(self.sort_dir_param, 'desc')
        sort_key = self.request.session.get(self.sort_key_param, 'name')

        if sort_key and sort_dir == 'desc':
            is_reversed = True
        domain_id = api.keystone.get_default_domain(self.request,
                                                    get_name=False).id
        is_cloud_admin = api.keystone.is_cloud_admin(self.request)
        if is_cloud_admin:
            groups = api.keystone.group_list(self.request, domain=domain_id)
            for group in groups:
                users = [user.id for user in api.keystone.user_list(
                    self.request, domain=domain_id, group=group.id)]
                if user_id in users:
                    user_groups.append(group)
        else:
            user_groups = api.keystone.group_list(self.request,
                                                  domain=domain_id,
                                                  user=user_id)

        user_groups.sort(key=attrgetter(sort_key), reverse=is_reversed)

        self.table.kwargs.update(
            {'offset': offset, 'limit': limit}
        )
        return user_groups


# noinspection PyAbstractClass
class ActionLogTab(ServerFilterTabMixin,
                   DTVTnxPaginationMixin,
                   tabs.TableTab):
    """Tab for user's action log"""
    table_classes = (tables.UserJournalTable,)
    name = _('Action log')
    slug = 'action_log_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_user_action_log_data(self):
        offset, limit = self.get_offset_limit()
        user_id = self.tab_group.kwargs['user_id']

        sort_dir = self.request.session.get(self.sort_dir_param, 'desc')
        sort_key = self.request.session.get(self.sort_key_param, 'event_time')

        search_opts = self.get_filters(include_not_api=True)
        search_opts.pop('marker', None)
        search_opts['all'] = True

        journal = UserJournal(self.request, user_id)
        journal_data = journal.list(search_opts,
                                    sort_dir=sort_dir,
                                    sort_key=sort_key)

        total_count = len(journal_data)

        if limit:
            journal_data = journal_data[offset:offset + limit]

        self.set_table_footer(offset, limit, total_count)
        return journal_data


# noinspection PyAbstractClass
class SessionTab(ServerFilterTabMixin, DTVTnxPaginationMixin, tabs.TableTab):
    """VDI session tab"""
    name = _("Sessions")
    slug = "session"
    table_classes = (tables.UserSessionTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_sessions_data(self):
        instances, tenants, total_count = {}, {}, 0
        offset, limit = self.get_offset_limit()
        user_id = self.tab_group.kwargs['user_id']
        filters = self.get_filters(filters={'user_id': user_id},
                                   with_marker=False)

        project_kwargs = {}
        project_filter = filters.pop('project_name', None)
        if project_filter:
            project_kwargs = {'filters': {'name': project_filter}}

        project_kwargs.update(**get_tenants_kwargs(self.request))
        try:
            tenants, _has_more = api.keystone.tenant_list(self.request,
                                                          **project_kwargs)
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve instance project information.')
            exceptions.handle(self.request, msg)
        else:
            tenants = {tenant.id: tenant.name for tenant in tenants}

        instance_filter = filters.pop('instance_name', None)
        search_opts = {}
        if instance_filter:
            search_opts = {'name': instance_filter}
        try:
            instances = get_vdi_instances(self.request, search_opts)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
        else:
            instances = {instance.id: instance for instance in instances}

        if project_filter:
            tenant_ids = tenants.keys()
            filters['server_id'] = [
                key for key, val in instances.items()
                if val.tenant_id in tenant_ids
            ] or [project_filter]
        if instance_filter:
            if instances:
                filters['server_id'] = instances.keys() or [instance_filter]
            else:
                filters['server_id__icontains'] = instance_filter

        sessions, total_count = get_sessions_list(
            self.request,
            limit=limit,
            offset=offset,
            filters=filters
        )

        for session in sessions:
            instance = instances.get(session.server_id)
            if instance:
                tenant_id = instance.tenant_id
                session.project_id = tenant_id
                session.project_name = tenants.get(tenant_id)
                session.instance_name = instance.name
            else:
                session.instance_name = session.server_id
        self.set_table_footer(offset, limit, total_count)
        return sessions

    def allowed(self, request):
        return LicenseController.is_valid(request, LicenseController.VDI)


# noinspection PyAbstractClass
class ProjectsTab(ServerFilterTabMixin, DTVTnxPaginationMixin, tabs.TableTab):
    """Tab for user's projects"""
    table_classes = (tables.ProjectsTable,)
    name = _('Projects')
    slug = 'projects_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_projects_data(self):
        projects = []
        user_id = self.tab_group.kwargs['user_id']
        offset, limit = self.get_offset_limit()

        domain_id = api.keystone.get_default_domain(self.request,
                                                    get_name=False).id

        try:
            roles_dict = {
                role.id: role.name
                for role in api.keystone.role_list(self.request)
            }
            user_assignments_roles = get_projects_assignments_roles_by_user(
                self.request, user_id
            )

            user_projects_roles = defaultdict(set)
            for user_role in user_assignments_roles:
                project = user_role.scope.get('project')
                if project:
                    user_projects_roles[project['id']].add(
                        roles_dict.get(user_role.role['id'])
                    )
            if user_projects_roles:
                for project in TNXTenantClient.list(self.request,
                                                    domain=domain_id):
                    project_id = project.id
                    if project.id in user_projects_roles:
                        project.roles = user_projects_roles.get(project_id)
                        projects.append(project)

        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _('Unable to get projects list.'))

        if utils.is_domain_supported():
            domains_dict = utils.domain_lookup(self.request)
            if domains_dict:
                for project in projects:
                    project.domain_name = domains_dict.get(project.domain_id)

        self.table.kwargs.update(
            {'offset': offset, 'limit': limit}
        )
        return projects


# noinspection PyAbstractClass
class UserDetailTabs(tabs.TabGroup):
    """Tionix user details tab group"""
    slug = "tnx_user_details"
    tabs = (OverviewTab,
            InstancesTab,
            GroupsTab,
            ActionLogTab,
            SessionTab,
            ProjectsTab)
    sticky = True
