# coding: utf-8
# pylint: disable=no-member

import mock

from openstack_dashboard import api
from openstack_dashboard.test.tests.templates import FakeUser

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.dashboards.identity.users import forms, tables, tabs
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, BaseAdminClientTestCase)


class UserInstancesTablesTest(BaseClientTestCase):

    def test_user_instances_columns(self):

        table = tables.UserVDIInstancesTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['tenant', 'name', 'ip', 'status', 'state',
                    'tnx_operation_mode', 'task', 'az', 'size', 'actions',
                    'multi_select'])
        )

    def test_user_instances_actions(self):
        instances = self.tnx_servers.list()

        table = tables.UserVDIInstancesTable(self.request, data=instances)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter_vdi_instances', 'unassign'])
        )

    def test_user_instances_row_actions(self):
        instance = self.tnx_servers.list()[0]

        table = tables.UserVDIInstancesTable(self.request)
        table.kwargs['user_id'] = instance.tnx_users[0]['id']

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['unassign'])
        )

        table.kwargs['user_id'] = 'invalid user'

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted([])
        )


class UsersTableTest(BaseClientTestCase):

    def test_users_actions(self):
        users = self.tnx_users.list()

        table = tables.UsersTable(self.request, data=users)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['create', 'delete', 'filter'])
        )

    def test_users_row_actions(self):
        user1, user2 = self.tnx_users.list()[:2]
        user1.enabled = True

        table = tables.UsersTable(self.request)

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(user1)]),
            sorted(['change_password', 'delete', 'edit', 'toggle'])
        )

        self.assertListEqual(
            [action.name for action in table.get_row_actions(user2)], []
        )


class UsersFormsTest(BaseClientTestCase):

    user_data = None

    # pylint: disable=bad-super-call
    def setUp(self):
        super(BaseClientTestCase, self).setUp()

        self.user_data = {'email': '', 'name': '', 'password': '',
                          'project': '', 'enabled': '', 'description': ''}

    @mock.patch.object(api.keystone,
                       'roles_for_user',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'add_domain_user_role')
    @mock.patch.object(api.keystone, 'role_get')
    @mock.patch.object(api.keystone, 'user_create')
    @mock.patch.object(api.keystone, 'get_default_domain')
    def test_tnx_create_user_admin(self, get_default_domain, create_user,
                                   role_get, add_domain_user_role):
        domain = self.tnx_domains.list()[0]
        user = self.tnx_users.list()[0]
        role = self.roles.list()[0]

        get_default_domain.return_value = domain
        create_user.return_value = user
        role_get.return_value = role

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.TnxCreateUserForm(request, initial={}, roles={})

        data = {'domain_id': domain.id, 'role_id': role.id}
        data.update(self.user_data)

        form.handle(request, data=data)
        self.assertTrue(add_domain_user_role.called)

    @mock.patch.object(api.keystone,
                       'roles_for_user',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'add_domain_user_role')
    @mock.patch.object(api.keystone, 'role_get')
    @mock.patch.object(api.keystone, 'user_create')
    @mock.patch.object(api.keystone, 'get_default_domain')
    def test_tnx_create_user_simple_user(self, get_default_domain, create_user,
                                         role_get, add_domain_user_role):
        domain = self.tnx_domains.list()[0]
        user = self.tnx_users.list()[0]
        role = self.roles.list()[1]

        get_default_domain.return_value = domain
        create_user.return_value = user
        role_get.return_value = role

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.TnxCreateUserForm(request, initial={}, roles={})

        data = {'domain_id': domain.id, 'role_id': role.id}
        data.update(self.user_data)

        form.handle(request, data=data)
        self.assertFalse(add_domain_user_role.called)


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
class TnxUsersTablesTestCase(BaseAdminClientTestCase):

    def test_projects_table(self):
        table = tables.ProjectsTable(self.request)
        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['name', 'tnx_project_type', 'description', 'id',
                    'enabled', 'domain_name', 'roles'])
        )


class TnxUsersTabsTestCase(BaseAdminClientTestCase):

    def test_user_details_tabs(self):
        user = self.users.first()

        tab_group = tabs.UserDetailTabs(
            self.request, user_id=user.id, user=user
        )

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['overview', 'instances_tab', 'groups_tab',
                    'action_log_tab', 'session', 'projects_tab'])
        )

    @mock.patch('tionix_dashboard.dashboards.identity.users.tabs.'
                'get_projects_assignments_roles_by_user')
    @mock.patch.object(api.keystone, 'role_list')
    def test_project_user_data(self, role_list, role_assignments_list):
        user = self.users.first()
        roles = self.roles.list()

        project = self.tnx_tenants.list()[0]
        role_list.return_value = roles
        role_assignments_list.return_value = [
            Empty(scope={'project': {'id': project.id}}, role={'id': role.id})
            for role in roles]

        tab_group = tabs.UserDetailTabs(
            self.request, user_id=user.id, user=user
        )

        tab = tabs.ProjectsTab(tab_group=tab_group,
                               request=self.request)

        self.assertEqual(len(tab.get_projects_data()), 1)
