# coding: utf-8

import logging

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, messages, tabs
from horizon.utils import memoized

from openstack_dashboard import api, policy
from openstack_dashboard.dashboards.identity.users import (
    views as users_views)

from tionix_dashboard.dashboards.identity.users.forms import (
    TnxCreateUserForm, TnxUpdateUserForm)
from tionix_dashboard.dashboards.identity.users.tables import UsersTable
from tionix_dashboard.dashboards.identity.users.tabs import UserDetailTabs
from tionix_dashboard.helpers.views import (
    ServerSortMultiSelect, ServerFilterViewMixin)

logger = logging.getLogger(__name__)


class TnxCreateView(users_views.CreateView):

    form_class = TnxCreateUserForm


class TnxUpdateView(users_views.UpdateView):

    form_class = TnxUpdateUserForm


class IndexView(users_views.IndexView):

    table_class = UsersTable

    def get_data(self):
        users = []

        if policy.check((("identity", "identity:list_users"),),
                        self.request):
            domain_context = api.keystone.get_effective_domain_id(self.request)
            try:
                users = api.keystone.user_list(self.request,
                                               domain=domain_context)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve user list.'))
        elif policy.check((("identity", "identity:get_user"),),
                          self.request):
            try:
                user = api.keystone.user_get(self.request,
                                             self.request.user.id)
                users.append(user)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve user information.'))
        else:
            msg = _("Insufficient privilege level to view user information.")
            messages.info(self.request, msg)

        if api.keystone.VERSIONS.active >= 3:
            domain_lookup = api.keystone.domain_lookup(self.request)
            if domain_lookup:
                for u in users:
                    u.domain_name = domain_lookup.get(u.domain_id)
        return users


class DetailView(ServerSortMultiSelect,
                 ServerFilterViewMixin,
                 tabs.TabbedTableView):
    """User details view"""
    tab_group_class = UserDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ user.name }}"
    redirect_url = reverse_lazy('horizon:identity:users:index')

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(DetailView, self).get_context_data(**kwargs)
        user = self.get_data()
        table = UsersTable(self.request)
        context["url"] = self.redirect_url
        context["actions"] = table.render_row_actions(user)
        context["user"] = user
        return context

    @memoized.memoized_method
    def get_data(self):
        user = None
        try:
            user_id = self.kwargs['user_id']
            user = api.keystone.user_get(self.request, user_id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve user details.'),
                              redirect=self.redirect_url)
        return user

    def get_tabs(self, request, **kwargs):
        kwargs['user'] = self.get_data()
        return self.tab_group_class(request, **kwargs)
