# coding: utf-8

import collections
import functools
import logging

from django.conf import settings
from django.core.urlresolvers import get_resolver
from django.utils.decorators import available_attrs

import horizon
from horizon import exceptions
from horizon.base import NotRegistered, _decorate_urlconf

from novaclient.exceptions import Unauthorized

from requests.exceptions import ConnectionError


logger = logging.getLogger(__name__)


def redirect_to_default(view_func, redirect_url):
    """Redirects unavailable urls
    """
    @functools.wraps(view_func, assigned=available_attrs(view_func))
    def dec(*args, **kwargs):
        raise exceptions.Http302(redirect_url)
    return dec


class LicenseController(object):

    NODE_CONTROL = 1
    CLOUD_CONTROL = 2
    MONITOR = 3
    VDI = 4

    LIC_TYPES = {
        NODE_CONTROL: 'NODE_CONTROL',
        CLOUD_CONTROL: 'CLOUD_CONTROL',
        MONITOR: 'MONITOR',
        VDI: 'VDI'
    }

    _licenses = None
    first_checked = False

    _panels = collections.defaultdict(
        lambda: collections.defaultdict(list)
    )

    _urls = collections.defaultdict(list)

    @classmethod
    def __collect_licenses(cls, request):
        from tionix_dashboard.clients import TNXLicensesClient
        result, licenses = True, {}
        try:
            licenses = {
                cls.NODE_CONTROL: (
                    TNXLicensesClient.get_node_control_license(request)),
                cls.CLOUD_CONTROL: (
                    TNXLicensesClient.get_cloud_control_license(request)),
                cls.MONITOR: (
                    TNXLicensesClient.get_monitor_license(request)),
                cls.VDI: (
                    TNXLicensesClient.get_vdi_server_license(request))
            }
            msg = (u'Lic middleware. '
                   u'Successfully collected licenses: {0}.'.format(licenses))
        except Unauthorized:
            msg = u'Lic middleware. Unauthorized error.'
            result = False
        except ConnectionError:
            msg = u'Lic middleware. Connection error.'
            result = False
        logger.debug(msg)
        cls._licenses = licenses
        return result

    @classmethod
    def _get_panel_url_patterns(cls, panel):
        return get_resolver(
            panel.__module__.replace('.panel', '.urls')
        ).url_patterns

    @classmethod
    def unregister(cls, lic_type):
        if lic_type == 'all':
            lic_type = cls.LIC_TYPES.keys()
        elif isinstance(lic_type, int):
            lic_type = (lic_type, )
        for lt in lic_type:
            for dash, panels in cls._panels[lt].iteritems():
                dash = horizon.get_dashboard(dash)
                for panel in panels:
                    try:
                        dash.unregister(panel)
                    except NotRegistered:
                        pass
                    # all urls decorate with redirect to default panel
                    try:
                        dash_absolute_url = dash.get_absolute_url()
                    except Exception as err:
                        logger.error(u'Unable to get default panel url: '
                                     u'{0}.'.format(err))
                        dash_absolute_url = settings.SITE_BRANDING_LINK
                    url_patterns = cls._get_panel_url_patterns(panel)
                    _decorate_urlconf(url_patterns,
                                      redirect_to_default,
                                      dash_absolute_url)
            for panel, urls in cls._urls[lt]:
                url_patterns = cls._get_panel_url_patterns(panel)
                for i in urls:
                    if i in url_patterns:
                        url_patterns.remove(i)

    @classmethod
    def is_valid(cls, request, lic_type):
        if not cls._licenses:
            cls.__collect_licenses(request)
        lic = cls._licenses.get(lic_type)
        if lic:
            is_valid = lic.verified
        else:
            is_valid = False
        return is_valid

    @classmethod
    def add_panel(cls, lic, dash, panel):
        cls._panels[lic][dash].append(panel)

    @classmethod
    def add_urls(cls, lic, panel, urls):
        cls._urls[lic].append((panel, urls))

    @classmethod
    def get_licenses(cls, request, recollect=False):
        if recollect or not cls._licenses:
            cls.__collect_licenses(request)
        return cls._licenses
