# coding: utf-8
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from horizon import forms

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.images.images import (
    forms as images_forms)
from openstack_dashboard.dashboards.project.images.snapshots import (
    forms as snapshots_forms)

from tionix_dashboard.dashboards.project.project_images.helpers import (
    get_image_file_from_cache, delete_image_file_from_cache)

from tionix_dashboard.helpers.base import BaseEnumerate
from tionix_dashboard.helpers.fields import AutoGenNameCharField, TnxFileWidget
from tionix_dashboard.helpers.constants import SCSI_PROPERTIES


class SourceType(BaseEnumerate):
    URL = 'url'
    FILE = 'file'

    values = {
        URL: _('Image Location'),
        FILE: _('Image File'),
    }


# noinspection PyAbstractClass
class CustomImageFieldsMixin(forms.Form):
    name = AutoGenNameCharField(max_length=255, label=_("Name"))
    disk_format = forms.ChoiceField(label=_('Image Format'),
                                    choices=[],
                                    widget=forms.Select(attrs={
                                        'class': 'switchable'
                                    }))


# noinspection PyClassHasNoInit
class CreateImageForm(CustomImageFieldsMixin, images_forms.CreateImageForm):
    minimum_disk = forms.IntegerField(
        label=_("Minimum Disk (GB)"),
        min_value=0,
        help_text=_('The minimum disk size required to boot the image. '
                    'If unspecified, this value defaults to 0 (no minimum).'),
        required=False)
    direct_scsi = forms.BooleanField(label="Direct SCSI", required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateImageForm, self).__init__(request, *args, **kwargs)
        self.fields['image_file'].widget = TnxFileWidget(
            attrs=self.image_attrs
        )

    @delete_image_file_from_cache
    def clean(self):
        image_file = self.cleaned_data.get('image_file', None)
        image_url = self.cleaned_data.get('image_url', None)
        source_type = self.cleaned_data.get('source_type', None)

        if source_type == SourceType.FILE:
            if not image_file:
                image_file = get_image_file_from_cache(self.request)
                if not image_file:
                    self.add_error('image_file', _('This field is required.'))
                else:
                    self.files['image_file'] = image_file
                    self.cleaned_data['image_file'] = image_file
        elif source_type == SourceType.URL:
            self.cleaned_data['image_file'] = None
            if not image_url:
                self.add_error('image_url', _('This field is required.'))

        if not image_url and not image_file:
            raise ValidationError(
                _("An image or external image location must be specified."))
        elif image_url and image_file:
            raise ValidationError(
                _("Can not specify both image and external image location."))
        return self.cleaned_data

    def handle(self, request, data):
        image = super(CreateImageForm, self).handle(request, data)
        if self.cleaned_data['direct_scsi']:
            api.glance.image_update_properties(
                request, image.id, **SCSI_PROPERTIES
            )
        return image


# noinspection PyClassHasNoInit
class UpdateImageForm(CustomImageFieldsMixin, images_forms.UpdateImageForm):
    minimum_disk = forms.IntegerField(
        label=_("Minimum Disk (GB)"),
        min_value=0,
        help_text=_('The minimum disk size required to boot the image. If '
                    'unspecified, this value defaults to 0 (no minimum).'),
        required=False)
    direct_scsi = forms.BooleanField(label="Direct SCSI", required=False)

    # noinspection PyBroadException
    def handle(self, request, data):
        image = super(UpdateImageForm, self).handle(request, data)

        if not self.cleaned_data['direct_scsi']:
            api.glance.image_update_properties(
                request, image.id, remove_props=SCSI_PROPERTIES
            )
        else:
            api.glance.image_update_properties(
                request, image.id, **SCSI_PROPERTIES
            )
        return image


# noinspection PyAbstractClass,PyClassHasNoInit
class CreateSnapshot(snapshots_forms.CreateSnapshot):
    name = AutoGenNameCharField(max_length=255, label=_("Snapshot Name"))
