# coding: utf-8
import functools
import os
import logging
import uuid

from django.core.cache import cache
from django.core.files.uploadedfile import TemporaryUploadedFile
from django.core.files.uploadhandler import StopFutureHandlers

logger = logging.getLogger(__name__)


FILE_IMAGE_CACHE_ATTR = 'image_file.{0}'


def gen_uploaded_file(request, field_name, file_name, content, content_type,
                      charset, content_type_extra=None, content_length=0):
    u"""Generates new uploaded file """

    file_object = None
    for handler in request.upload_handlers:
        try:
            handler.new_file(field_name, file_name,
                             content_type, content_length,
                             charset, content_type_extra)
        except StopFutureHandlers:
            continue
        handler.receive_data_chunk(content, 0)
        file_object = handler.file_complete(content_length)
        if file_object:
            break
    return file_object


def save_in_cache_image_file(request, image_file):
    """Saves image file to prevent repeated uploading by user"""
    image_file_cache_uuid = uuid.uuid4()
    request.session['image_file_cache_uuid'] = image_file_cache_uuid

    image_file_cache_dict = {
        'name': image_file.name,
        'size': image_file.size,
        'content_type': image_file.content_type,
        'charset': image_file.charset,
        'content_type_extra': image_file.content_type_extra
    }

    if isinstance(image_file, TemporaryUploadedFile):
        image_file_cache_dict['path'] = image_file.file.name
        # Otherwise will be closed by Django
        image_file.file.close_called = True
    else:
        image_file_cache_dict['content'] = image_file.read()

    cache.set(FILE_IMAGE_CACHE_ATTR.format(image_file_cache_uuid),
              image_file_cache_dict)


def get_image_file_from_cache(request):
    """Retrieve image file from cache"""
    content = uploaded_file = None
    image_file_cache_uuid = request.session.pop('image_file_cache_uuid', None)
    if image_file_cache_uuid:
        file_image_cached_uuid = FILE_IMAGE_CACHE_ATTR.format(
            image_file_cache_uuid
        )
        image_file_cache_dict = cache.get(file_image_cached_uuid)
        if image_file_cache_dict:
            cache.delete(file_image_cached_uuid)

            image_file_path = image_file_cache_dict.pop('path', None)
            if image_file_path:
                try:
                    old_file = open(image_file_path)
                    content = old_file.read()
                except Exception as err:
                    logger.error(
                        u'Unable to read the file "{0}" from cache: {1}. '
                        u'The File will be deleted.'.format(image_file_path,
                                                            unicode(err))
                    )
                finally:
                    try:
                        os.remove(image_file_path)
                    except Exception as err:
                        logger.error(unicode(err))
            else:
                content = image_file_cache_dict.get('content')
        if content is not None:
            content_type_extra = image_file_cache_dict.get(
                'content_type_extra'
            )
            uploaded_file = gen_uploaded_file(
                request, 'image_file', image_file_cache_dict.get('name'),
                content, image_file_cache_dict.get('content_type'),
                image_file_cache_dict.get('charset'),
                content_type_extra=content_type_extra,
                content_length=image_file_cache_dict.get('size'))
    return uploaded_file


def delete_image_file_from_cache(fn):
    """Decorator to delete image file cache"""
    @functools.wraps(fn)
    def wrapper(self):
        result = fn(self)
        image_file_cache_uuid = self.request.session.pop(
            'image_file_cache_uuid', None
        )
        if image_file_cache_uuid:
            file_image_cached_uuid = FILE_IMAGE_CACHE_ATTR.format(
                image_file_cache_uuid
            )
            image_file_cache_dict = cache.get(file_image_cached_uuid)
            if image_file_cache_dict:
                cache.delete(file_image_cached_uuid)

                image_file_path = image_file_cache_dict.pop('path', None)
                if image_file_path:
                    try:
                        os.remove(image_file_path)
                    except Exception as err:
                        logger.error(unicode(err))
        return result

    return wrapper
