# coding: utf-8

import logging
from collections import defaultdict

from django.core.urlresolvers import reverse
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils import functions

from openstack_dashboard import api
from openstack_dashboard.api import nova, cinder
from openstack_dashboard.dashboards.project.images import (
    utils as image_utils)
from openstack_dashboard.dashboards.project.instances import (
    forms as project_forms)

from tionix_scheduler_client.enums import ObjectTypeEnum

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    get_project_volumes)
from tionix_dashboard.dashboards.tionix.scheduler import (
    forms as schedule_forms)
from tionix_dashboard.dashboards.utils import TABLE_OBJECT_ID_SEP
from tionix_dashboard.helpers.fields import (
    BootstrapSliderInput, AutoGenNameCharField)
from tionix_dashboard.helpers.forms import MultiManagingForm
from tionix_dashboard.url_resolve_map import PROJECT_SERVERS_INDEX

logger = logging.Logger(__name__)


class SliderFlavorPartInput(BootstrapSliderInput):
    """Widget for slider flavor"""

    TICKS_DICT = None
    REVERSE_TICKS_DICT = None

    def __init__(self, ticks_dict, attrs=None):
        attrs = attrs or {}
        ticks, ticks_labels = ticks_dict.keys(), ticks_dict.values()
        attrs.update(**{
            'data-slider-ticks': ticks,
            'data-slider-ticks-labels': ticks_labels,
            'data-slider-value': ticks[0],
        })
        self.TICKS_DICT = ticks_dict
        self.REVERSE_TICKS_DICT = {v: k for k, v in ticks_dict.iteritems()}
        super(SliderFlavorPartInput, self).__init__(attrs)

    def get_value_for_render(self, value):
        """Return display value"""
        return self.REVERSE_TICKS_DICT.get(value, '')

    def value_from_datadict(self, data, files, name):
        """Return value from ticks dict"""
        value = data.get(name)
        return value and self.TICKS_DICT.get(int(value))

    def set_enabled(self, enabled=True):
        self.attrs['data-slider-enabled'] = str(enabled).lower()


# noinspection PyAbstractClass,PyClassHasNoInit
class DeleteInstanceForm(schedule_forms.DeleteObjectWithTasksForm):
    """ Delete instances form """

    object_type = ObjectTypeEnum.instance
    attached_volumes = None

    @staticmethod
    def _get_volumes(request):
        return get_project_volumes(request)

    def handle(self, request, data):
        ids = self.initial['ids']
        self.attached_volumes = defaultdict(set)
        volumes = self._get_volumes(request)
        for volume in volumes:
            for att in volume.attachments:
                server_id = att.get('server_id', None)
                if server_id is not None and server_id in ids:
                    self.attached_volumes[server_id].add(volume)
        return super(DeleteInstanceForm, self).handle(request, data)

    def delete(self, request, object_id):
        detached_volumes = []
        try:
            for volume in self.attached_volumes.get(object_id, []):
                api.nova.instance_volume_detach(request, object_id, volume.id)
                detached_volumes.append(volume.name)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _("Unable to detach volumes."))

        else:
            if detached_volumes:
                messages.success(request, _(
                    'Successfully detached volumes: "{0}".').format(
                        u'", "'.join(detached_volumes)))
            api.nova.server_delete(request, object_id)


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxDeleteInstanceForm(DeleteInstanceForm):
    """ Delete tnx instances form """

    @staticmethod
    def get_object_id(object_id):
        if TABLE_OBJECT_ID_SEP in object_id:
            _project_id, object_id = object_id.split(TABLE_OBJECT_ID_SEP)
        return object_id


# noinspection PyAbstractClass
class ResetStateForm(forms.SelfHandlingForm):
    """Reset vm's state form"""

    name = _("Reset state")
    reboot = forms.BooleanField(
        required=False,
        label=_('Reboot'),
        initial=True
    )

    def handle(self, request, data):
        initial = self.initial

        success_objects, failure_objects = [], []
        for instance in initial['instances']:
            try:
                TNXServersClient.reset_with_reboot(
                    request, instance.id, reboot=data['reboot'],
                    project_id=instance.tenant_id)
                success_objects.append(instance.name)
            except Exception as err:
                logger.error(unicode(err))
                failure_objects.append(instance.name)

        success_message_level = messages.success
        if failure_objects:
            messages.error(request,
                           _(u'Failed to reset states for instances: '
                             u'"{0}".'.format(u'", "'.join(failure_objects))))
            success_message_level = messages.info

        if success_objects:
            msg = _(u'Instances are preparing the state reset: '
                    u'"{0}".'.format(u'", "'.join(success_objects)))
            success_message_level(request, msg)
        return True


def get_pow_ticks(start, total):
    """
    Return dict where keys are ticks(from 0 to total) and values
    are label ticks (2**(from start to start + total))
    :param start: initial power
    :param total: total powers count
    :return: dict
    """
    return {i: 2 ** (i + start) for i in xrange(0, total)}


# values 1, 2, 4, 8, 16
VCPUS_DICT = get_pow_ticks(0, 5)
# values 512, 1024, 2048, 4096, 8192, 16384, 32768
RAM_DICT = get_pow_ticks(9, 7)
# values 50, 100, 150, 200, 250
DISK_DICT = {i: j for i, j in enumerate(range(50, 300, 50))}


def get_image_field_data(request):
    """Return images to choice field"""
    choices = []
    images = image_utils.get_available_images(request,
                                              request.user.tenant_id)
    if images:
        choices.append(("", _("Select Image")))
        for image in images:
            image.bytes = getattr(image, 'virtual_size', None) or image.size
            image.volume_size = max(
                image.min_disk, functions.bytes_to_gigabytes(image.bytes))
            choices.append((image.id, image))
    else:
        choices.append(("", _("No images available")))
    return choices


def get_network_field_data(request, include_empty_option=True):
    """Return networks to choice field"""
    networks = []

    try:
        networks = api.neutron.network_list_for_tenant(request,
                                                       request.user.tenant_id)
        networks.sort(key=lambda x: x.shared is True)
        networks = [(network.id, network.name)for network in networks]
    except Exception as err:
        logger.error(unicode(err))
        networks.append(("", _("No networks available")))
    else:
        if include_empty_option:
            networks.insert(0, ("", _("Select Network")))
    return networks


class LaunchBaseInstanceMixin(object):
    """Launch base instance mixin"""

    def __init__(self, request, *args, **kwargs):
        assert isinstance(self, forms.Form)
        super(LaunchBaseInstanceMixin, self).__init__(request, *args, **kwargs)
        image_field = self.fields.get('image') or self.fields.get('image_id')
        image_field.choices = get_image_field_data(request)

    def clean(self):
        cleaned_data = super(LaunchBaseInstanceMixin, self).clean()
        try:
            flavors = api.nova.flavor_list(self.request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, err.message,
                              redirect=PROJECT_SERVERS_INDEX)
        else:
            vcpus = cleaned_data['vcpus']
            ram = cleaned_data['ram']
            disk = cleaned_data['disk']
            for f in flavors:
                if vcpus == f.vcpus and ram == f.ram and disk == f.disk:
                    cleaned_data['flavor_id'] = f.id
                    break
            else:
                # pylint: disable=no-member
                raise forms.ValidationError(
                    _('Flavor with chosen parameters '
                      '(CPU - {vcpus}pcs., RAM - {ram}MB, '
                      'HDD - {disk}GB) is not found.').format(
                          vcpus=vcpus, ram=ram, disk=disk)
                )
        return cleaned_data


class LaunchBaseInstanceForm(LaunchBaseInstanceMixin, forms.SelfHandlingForm):
    """Launch base instance"""

    name = AutoGenNameCharField(label=_("Instance Name"), max_length=255)
    count = forms.IntegerField(
        label=_("Instance Count"),
        min_value=1,
        initial=1,
        help_text=_("Number of instances to launch."))
    image_id = forms.ChoiceField(
        label=_("Image Name"),
        widget=forms.SelectWidget(
            data_attrs=('volume_size',),
            transform=lambda x: ("%s (%s)" % (x.name,
                                              filesizeformat(x.bytes)))))
    network_id = forms.ChoiceField(label=_("Network"))
    vcpus = forms.IntegerField(
        label=_('CPU (pcs.)'),
        required=False,
        widget=SliderFlavorPartInput(
            ticks_dict=VCPUS_DICT,
            attrs={'data-slider-tooltip': 'hide'}
        )
    )
    ram = forms.IntegerField(
        label=_('RAM (MB)'),
        required=False,
        widget=SliderFlavorPartInput(
            ticks_dict=RAM_DICT,
            attrs={'data-slider-tooltip': 'hide'}
        )
    )
    disk = forms.IntegerField(
        label=_('HDD (GB)'),
        required=False,
        widget=SliderFlavorPartInput(
            ticks_dict=DISK_DICT,
            attrs={'data-slider-tooltip': 'hide'}
        )
    )

    def __init__(self, request, *args, **kwargs):
        super(LaunchBaseInstanceForm, self).__init__(request, *args, **kwargs)
        self.fields['network_id'].choices = get_network_field_data(request)

    # noinspection PyMethodMayBeStatic
    def handle(self, request, data):
        result = False
        nics = [{"net-id": data['network_id'], "v4-fixed-ip": ""}]
        try:
            TNXServersClient.create(request,
                                    data['name'],
                                    data['image_id'],
                                    data['flavor_id'],
                                    min_count=data['count'],
                                    nics=nics)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, err.message,
                              redirect=PROJECT_SERVERS_INDEX)
        else:
            result = True
            messages.success(request, _('Successfully created instance.'))
        return result


# noinspection PyClassHasNoInit
class AttachInterface(project_forms.AttachInterface):
    def handle(self, request, data):
        instance_id = data['instance_id']
        network = data.get('network')
        try:
            api.nova.interface_attach(request, instance_id, net_id=network)
            msg = _('Attaching interface for instance "%s".') % instance_id
            messages.success(request, msg)
        except Exception as err:
            logger.error(unicode(err))
            redirect = reverse('horizon:project:instances:index')
            exceptions.handle(request, _("Unable to attach interface."),
                              redirect=redirect)
        return True


# noinspection PyClassHasNoInit
class DetachInterface(project_forms.DetachInterface):
    def handle(self, request, data):
        instance_id = data['instance_id']
        port = data.get('port')
        try:
            api.nova.interface_detach(request, instance_id, port)
            msg = _('Detached interface %(port)s for instance '
                    '"%(instance)s".') % {'port': port,
                                          'instance': instance_id}
            messages.success(request, msg)
        except Exception as err:
            logger.error(unicode(err))
            redirect = reverse('horizon:project:instances:index')
            exceptions.handle(request, _("Unable to detach interface."),
                              redirect=redirect)
        return True


class AttachVolumeBaseForm(forms.SelfHandlingForm):
    """Manage volume attachments form"""

    name = _("Manage Volume Attachments")

    volume_id = forms.CharField(widget=forms.HiddenInput, required=False)
    instance_id = forms.CharField(widget=forms.HiddenInput, required=False)
    device = forms.CharField(
        label=_("Device Name"),
        widget=forms.TextInput(attrs={'placeholder': '/dev/vdc'}),
        required=False,
        help_text=_("Actual device name may differ due to hypervisor "
                    "settings. If not specified, then hypervisor will select "
                    "a device name."))

    def __init__(self, *args, **kwargs):
        super(AttachVolumeBaseForm, self).__init__(*args, **kwargs)

        # Hide the device field if the hypervisor doesn't support it.
        if not nova.can_set_mount_point():
            self.fields['device'].widget = forms.widgets.HiddenInput()


class AttachForm(MultiManagingForm):

    instance_id = forms.CharField(widget=forms.HiddenInput, required=False)

    device = forms.CharField(
        label=_("Device Name"),
        widget=forms.TextInput(attrs={'placeholder': '/dev/vdc'}),
        required=False,
        help_text=_("Actual device name may differ due to hypervisor "
                    "settings. If not specified, then hypervisor will select "
                    "a device name."))

    def __init__(self, *args, **kwargs):
        super(AttachForm, self).__init__(*args, **kwargs)

        # Hide the device field if the hypervisor doesn't support it.
        if not nova.can_set_mount_point():
            self.fields['device'].widget = forms.widgets.HiddenInput()

    def handle(self, request, data):
        new_allocated = {obj['id'] for obj in data['new_allocated']}
        old_allocated = set(data['old_allocated'])
        instance_id = self.initial['instance_id']
        # api requires non-empty device name or None
        device = data.get('device') or None

        volumes_diff = list(new_allocated.symmetric_difference(old_allocated))
        if not volumes_diff:
            return True

        failure_volumes, attached_volumes, detached_volumes = [], [], []

        volumes = []
        try:
            volumes = cinder.volume_list(request,
                                         search_opts={'id': volumes_diff})
        except Exception as err:
            logger.error(unicode(err))

        for volume in volumes:
            volume_id, volume_name = volume.id, volume.name
            try:
                if volume_id in old_allocated:
                    nova.instance_volume_detach(request,
                                                instance_id,
                                                volume_id)
                    detached_volumes.append(volume_name)
                else:
                    nova.instance_volume_attach(
                        request, volume_id, instance_id, device
                    )
                    attached_volumes.append(volume_name)
            except Exception as err:
                logger.error(unicode(err))
                failure_volumes.append(volume_name)

        if attached_volumes:
            messages.success(
                request,
                _('Successfully attached volumes: '
                  '"{0}".').format(u'", "'.join(attached_volumes))
            )

        if detached_volumes:
            messages.success(
                request,
                _('Successfully detached volumes: '
                  '"{0}".').format(u'", "'.join(detached_volumes))
            )

        if failure_volumes:
            messages.error(
                request,
                _('Unable attach/detach volumes: '
                  '"{0}".').format(u'", "'.join(failure_volumes))
            )

        return True
