# coding: utf-8

import logging

import time
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, messages
from horizon.utils.memoized import memoized

from openstack_dashboard import api, policy

from tionix_client.helpers.journal import log_server_action

from tionix_dashboard.clients.base import cinderclient
from tionix_dashboard.dashboards import utils
from tionix_dashboard.helpers.vm_constants import ACTIVE
from tionix_dashboard.settings import VOLUME_ATTACH_MAX_RETRIES

logger = logging.getLogger(__name__)


@memoized
def get_users(request):
    users_dict = {}
    if policy.check((("identity", "identity:list_users"),), request):
        try:
            domain = getattr(request.user, 'user_domain_id', None)
            users_dict = {obj.id: obj.name for obj in utils.user_list(
                request, domain=domain)}
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, _('Unable to retrieve user list.'))
    return users_dict


@memoized
def get_username(request, user_id):
    users = get_users(request)
    return users.get(user_id)


def get_instance_security_groups(request, instance_id):
    """
    Retrieve security groups ids by instance_id
    :param request:
    :param instance_id:
    :return:
    """
    security_groups = []
    try:
        security_groups = api.network.server_security_groups(request,
                                                             instance_id)
    except Exception as err:
        logger.error(err)
        messages.error(request, _('Unable to get security groups.'))
    else:
        security_groups = [group.id for group in security_groups]

    return security_groups


def get_instance_networks_and_ports(request, instance_id):
    """
    Retrieve networks and ports ids by instance_id
    :param request:
    :param instance_id:
    :return:
    """
    networks, ports = [], []
    try:
        _ports = api.neutron.list_resources_with_long_filters(
            api.neutron.port_list,
            'device_id',
            [instance_id],
            request=request
        )
    except Exception as err:
        logger.error(err)
        messages.error(request, _('Unable to connect to Neutron.'))
    else:
        ports = [port.id for port in _ports]
        networks = set([port.network_id for port in _ports])

    return networks, ports


def log_instance_group_added(instance_id, groups, result, res_msg=''):
    """Log add instance group action"""
    success = True
    if result:
        res_msg = ("New groups ID's: "
                   "{0}".format("\n" + ",\n".join(groups) if groups else "-"))
    else:
        res_msg = res_msg or '-'
        success = False
    action = 'dashboard.update_security_groups'
    log_server_action(
        object_id=instance_id, action=action, success=success,
        message=res_msg)


def delayed_volume_attach(request, instance_id, volume_id, device=None,
                          delay=5):
    retries = 0
    while retries < VOLUME_ATTACH_MAX_RETRIES:
        time.sleep(delay)
        try:
            instance = api.nova.server_get(request, instance_id)
            if instance.status.lower() == ACTIVE:
                api.nova.instance_volume_attach(
                    request, volume_id, instance_id, device
                )
                break
        except Exception as err:
            logger.error(unicode(err))
        retries += 1


def get_project_volumes(request, project_id=None):
    volumes = []
    try:
        volumes = cinderclient(request, project_id=project_id).volumes.list()
    except Exception as err:
        logger.error(unicode(err))
    return volumes
