# coding: utf-8

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon.utils import filters

from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables, audit_tables as project_audit_tables)

from tionix_dashboard.dashboards.project.project_instances.constants import (
    VM_STATUS_DISPLAY_CHOICES, VM_TASK_DISPLAY_CHOICES,
    ACTION_DISPLAY_CHOICES as INSTANCE_ACTIONS, STATUS_CHOICES)
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    InstanceShowMetricAction)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ProjectInstanceSchedulerAction)
from tionix_dashboard.dashboards.project.project_instances import actions
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import (
    DataTableFirstLastPagination, TnxColumn, ServerFilterTabTableMixin)
from tionix_dashboard.helpers.vm_constants import VMTypes
from tionix_dashboard.journal.tables import (
    STATUS_DISPLAY_CHOICES, ACTION_DISPLAY_CHOICES)
from tionix_dashboard.templatetags.tionix_tags import format_date
from tionix_dashboard.templatetags.tionix_tags import timesince_sortable
from tionix_dashboard.dashboards.tionix.metrics.bareos_templates import (
    tables as metrics_tables, actions as metrics_actions)


class TnxInstancesTable(DataTableFirstLastPagination, tables.DataTable):

    name = tables.Column(
        "name",
        link="horizon:project:instances:detail",
        verbose_name=_("Name"),
        form_field=EmptyCharField(max_length=255),
        update_action=actions.InstanceUpdateCell)
    image_name = tables.Column(
        "image_name",
        verbose_name=_("Image Name"))
    ip = tables.Column(
        project_tables.get_ips,
        verbose_name=_("IP Address"),
        attrs={'data-type': "ip"})
    size = tables.Column(
        project_tables.get_size,
        sortable=False,
        verbose_name=_("Size"))
    type = tables.Column(
        lambda x: unicode(getattr(x, "tnx_type", '') or ''),
        verbose_name=_("Type"),
        display_choices=VMTypes.TYPES.items())
    status = tables.Column(
        "status",
        filters=(project_tables.title, filters.replace_underscores),
        verbose_name=_("Status"),
        status=True,
        status_choices=STATUS_CHOICES,
        display_choices=VM_STATUS_DISPLAY_CHOICES)
    az = tables.Column(
        "availability_zone",
        verbose_name=_("Zone"))
    task = tables.Column(
        "OS-EXT-STS:task_state",
        verbose_name=_("Task"),
        empty_value=project_tables.TASK_DISPLAY_NONE,
        status=True,
        status_choices=project_tables.InstancesTable.TASK_STATUS_CHOICES,
        display_choices=VM_TASK_DISPLAY_CHOICES)
    state = tables.Column(
        project_tables.get_power_state,
        filters=(project_tables.title, filters.replace_underscores),
        verbose_name=_("Power State"),
        display_choices=project_tables.POWER_DISPLAY_CHOICES)
    created = tables.Column(
        "created",
        verbose_name=_("Time since created"),
        filters=(filters.parse_isotime,
                 timesince_sortable),
        attrs={'data-type': 'timesince'})

    # noinspection PyTypeChecker
    class Meta(object):
        name = "instances"
        verbose_name = _("Instances")
        status_columns = ["status", "task"]
        row_class = actions.UpdateRow
        launch_actions = (actions.LaunchBaseLink, )
        if getattr(settings, 'LAUNCH_INSTANCE_LEGACY_ENABLED', False):
            launch_actions = launch_actions + (project_tables.LaunchLink,)
        if getattr(settings, 'LAUNCH_INSTANCE_NG_ENABLED', True):
            launch_actions = launch_actions + (project_tables.LaunchLinkNG,)
        table_actions = launch_actions + (actions.TnxInstancesFilterAction, )

        table_actions_menu = (
            ProjectInstanceSchedulerAction,
            actions.TnxDeleteInstance,
            actions.StartInstance,
            actions.TnxStopInstance,
            actions.SoftRebootInstance,
            actions.ResetStateAction,
            actions.TnxMultiTogglePause,
            actions.TnxMultiToggleUnPause,
            actions.TnxMultiToggleSuspend,
            actions.TnxMultiToggleResume,
            actions.TnxMultiToggleShelve,
            actions.TnxMultiToggleUnShelve,
            actions.RebootInstance,
            actions.EditInstance
        )

        row_actions = (
            ProjectInstanceSchedulerAction,
            actions.StartInstance,
            actions.ConfirmResize,
            project_tables.RevertResize,
            project_tables.CreateSnapshot,
            project_tables.SimpleAssociateIP,
            project_tables.AssociateIP,
            project_tables.SimpleDisassociateIP,
            project_tables.AttachInterface,
            project_tables.DetachInterface,
            actions.EditInstance,
            project_tables.UpdateMetadata,
            project_tables.DecryptInstancePassword,
            actions.EditInstanceSecurityGroups,
            actions.TnxConsoleLink,
            actions.TnxLogLink,
            actions.TnxTogglePause,
            actions.ToggleSuspend,
            project_tables.ResizeLink,
            actions.LockInstance,
            actions.UnlockInstance,
            actions.SoftRebootInstance,
            actions.RebootInstance,
            actions.TnxStopInstance,
            project_tables.RebuildInstance,
            actions.TnxDeleteInstance,
            actions.ResetStateAction,
            actions.TnxToggleShelve,
            InstanceShowMetricAction,
            actions.ManageVolumeAttachmentAction)
        if getattr(settings, 'LAUNCH_INSTANCE_LEGACY_ENABLED', False):
            row_actions = row_actions + (actions.CloneLink,)
        if getattr(settings, 'LAUNCH_INSTANCE_NG_ENABLED', True):
            row_actions = row_actions + (actions.CloneNGLink,)


# noinspection PyClassHasNoInit
class InstancesTable(project_tables.InstancesTable):
    class Meta(project_tables.InstancesTable.Meta):

        table_actions_menu = (
            project_tables.InstancesTable.Meta.table_actions_menu +
            (ProjectInstanceSchedulerAction,)
        )

        row_actions = (
            project_tables.InstancesTable.Meta.row_actions +
            (ProjectInstanceSchedulerAction, InstanceShowMetricAction)
        )


class TnxAuditLog(ServerFilterTabTableMixin, tables.DataTable):

    request_id = tables.Column('request_id',
                               verbose_name=_('Request ID'))
    action = TnxColumn(
        'action', verbose_name=_('Action'),
        display_choices=INSTANCE_ACTIONS + ACTION_DISPLAY_CHOICES)
    created_at = tables.Column('event_time', verbose_name=_('Start time'),
                               filters=(format_date, ))
    user = tables.Column('user', verbose_name=_('User'))
    status = tables.Column('status', verbose_name=_('Result'),
                           display_choices=STATUS_DISPLAY_CHOICES)
    result = tables.Column('message', verbose_name=_('Result details'))

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instance Action List')
        multi_select = False
        table_actions = (actions.TnxAuditFilterAction,)


class AuditTable(ServerFilterTabTableMixin, project_audit_tables.AuditTable):

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instance Action List')
        table_actions = (actions.AuditFilterAction,)


# noinspection PyClassHasNoInit
class BareosTemplatesTable(metrics_tables.BareosTemplatesTable):

    class Meta(object):
        name = "bareos_templates"
        verbose_name = _("Bareos Templates")
        table_actions = (
            metrics_actions.TnxMetricsBareosTemplatesFilterAction,)
        row_actions = (actions.UnassignBareosTemplate,)
        no_data_message = _('No items to display.')
