# coding: utf-8

import logging

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs
from horizon.utils import functions as utils

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.instances import (
    tabs as project_tabs)

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients import TNXSessionsClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.project.project_instances import console
from tionix_dashboard.dashboards.project.project_instances import tables
from tionix_dashboard.dashboards.tionix.metrics.bareos_templates import (
    tabs as metrics_tabs)
from tionix_dashboard.dashboards.tionix.metrics.instances.tabs import (
    InstanceMetricsTab)
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.helpers.views import ServerFilterTabMixin
from tionix_dashboard.helpers.vm_constants import VMTypes, ConnectionTypeEnum
from tionix_dashboard.journal.helpers import ServerJournal
from tionix_dashboard.settings import LOG_CONSOLE_TIMEOUT
from tionix_dashboard.url_resolve_map import (
    PROJECT_ACCESS_SECURITY_GROUPS_DETAIL)

logger = logging.getLogger(__name__)


NO_STATE = 0


def is_instance_without_state(instance):
    return getattr(instance, 'OS-EXT-STS:power_state', NO_STATE) == NO_STATE


# noinspection PyClassHasNoInit
class OverviewTab(project_tabs.OverviewTab):
    """General instance details overview tab"""

    template_name = "project/project_instances/_detail_overview.html"
    is_vdi = False

    def get_context_data(self, request):
        instance = self.tab_group.kwargs['instance']
        project_id = instance.tenant_id
        description = getattr(instance, 'description', '') or ''
        project_name = ''
        is_admin = api.keystone.is_domain_admin(request)
        try:
            tenant = api.keystone.tenant_get(request, project=project_id,
                                             admin=is_admin)
            project_name = tenant.name
        except Exception as ex:
            logger.error(unicode(ex))

        for security_group in instance.security_groups:
            security_group.detail_url = reverse(
                PROJECT_ACCESS_SECURITY_GROUPS_DETAIL,
                args=(security_group.id,)
            )

        instance.tnx_type_name = VMTypes.TYPES.get(
            str(getattr(instance, 'tnx_type', '')), '')

        custom_titles = getattr(settings,
                                'INSTANCE_CUSTOM_PROPERTY_TITLES', {})
        instance_props = []
        metadata = getattr(instance, 'metadata', {})
        for key, value in metadata.items():
            prop = custom_titles.get(key, key)
            instance_props.append((prop, value))

        return {"instance": instance,
                "project_name": project_name,
                "description": description,
                "is_vdi": self.is_vdi,
                "is_superuser": request.user.is_superuser,
                "properties": instance_props,
                'up_time': '99,999', 'recovery_time': '8',
                'movable': getattr(instance, 'tnx_movable', '')}


# noinspection PyClassHasNoInit
class NonVDIOverviewTab(OverviewTab):
    """Instance details overview tab without vdi attributes"""

    def allowed(self, request):
        """Determines if vdi license is not available."""
        return not LicenseController.is_valid(request, LicenseController.VDI)


# noinspection PyClassHasNoInit
class VDIOverviewTab(OverviewTab):
    """Instance details overview tab with vdi attributes"""
    slug = "overview_vdi"
    is_vdi = True

    def allowed(self, request):
        """Determines if vdi license is available."""
        return LicenseController.is_valid(request, LicenseController.VDI)


# noinspection PyClassHasNoInit
class TnxLogTab(project_tabs.LogTab):

    template_name = "project/project_instances/_detail_log.html"

    def get_context_data(self, request):

        instance = self.tab_group.kwargs['instance']

        log_length = utils.get_log_length(request)

        is_without_state = is_instance_without_state(instance)

        if not is_without_state:
            client = novaclient(self.request, timeout=LOG_CONSOLE_TIMEOUT)
            try:
                data = client.servers.get_console_output(instance.id,
                                                         length=log_length)
            except Exception as err:
                logger.error(unicode(err))
                data = _('Unable to get log for instance "%s".') % instance.id
        else:
            data = _('Log is not available for an instance without a '
                     'state.')

        return {"instance": instance,
                "console_log": data,
                "buttons_disabled": is_without_state,
                "log_length": log_length}


# noinspection PyClassHasNoInit
class TnxConsoleTab(project_tabs.ConsoleTab):

    template_name = "project/project_instances/_detail_console.html"
    full_screen_console = 'horizon:project:instances:fullscreen_console'

    def get_full_screen_console(self, instance):
        return reverse(self.full_screen_console, args=(instance.id,))

    def get_context_data(self, request):
        instance = self.tab_group.kwargs['instance']

        console_type = getattr(settings, 'CONSOLE_TYPE', 'AUTO')
        console_url = None

        is_without_state = is_instance_without_state(instance)
        if not is_without_state:
            try:
                console_type, console_url = console.get_console(
                    request, console_type, instance)
                # For serial console, the url is different from VNC, etc.
                # because it does not include parms for title and token
                if console_type == "SERIAL":
                    console_url = reverse('horizon:project:instances:serial',
                                          args=[instance.id])
            except exceptions.NotAvailable:
                exceptions.handle(request, ignore=True, force_log=True)
            else:
                if str(getattr(instance, 'tnx_type', '')) == VMTypes.VDI:
                    user_agent = self.request.user_agent
                    client_os = "{family} {version}".format(
                        family=user_agent.os.family,
                        version=user_agent.os.version_string).strip()
                    client_info = "{family} {version}".format(
                        family=user_agent.browser.family,
                        version=user_agent.browser.version_string).strip()

                    try:
                        TNXSessionsClient.create(
                            request,
                            user_id=self.request.user.id,
                            instance_id=instance.id,
                            conn_type=ConnectionTypeEnum.DASHBOARD,
                            client_ip=self.request.META.get('REMOTE_ADDR'),
                            client_os=client_os,
                            client_info=client_info
                        )
                    except UrlNotAvailableError:
                        raise
                    except Exception as err:
                        logger.error(unicode(err))
                        exceptions.handle(self.request,
                                          _('Unable to create session.'))

        context = {
            'console_url': console_url,
            'instance_id': instance.id,
            'console_type': console_type,
            'fullscreen_console_url': self.get_full_screen_console(instance)}

        return context


# noinspection PyAbstractClass
class AuditTab(ServerFilterTabMixin, project_tabs.AuditTab):
    """Audit tab with filter"""

    table_classes = (tables.AuditTable, )


# noinspection PyAbstractClass
class TnxAuditTab(AuditTab):
    """Tionix audit tab"""

    table_classes = (tables.TnxAuditLog, )

    def get_audit_data(self):
        server_journal = ServerJournal(self.request,
                                       self.tab_group.kwargs['instance'])
        return server_journal.list()


# noinspection PyAbstractClass
class InstanceBareosTemplatesTab(metrics_tabs.InstanceBareosTemplatesTab):
    table_classes = (tables.BareosTemplatesTable,)


# noinspection PyAbstractClass
class TnxInstanceDetailTabs(tabs.TabGroup):
    """Tionix instance details tab group"""
    slug = "instance_details"
    tabs = (NonVDIOverviewTab,
            VDIOverviewTab,
            TnxLogTab,
            TnxConsoleTab,
            TnxAuditTab,
            ActiveTasksTab,
            InstanceMetricsTab,
            InstanceBareosTemplatesTab)
    sticky = True
    obj_type = ObjectTypeEnum.instance
    obj_group = ObjectGroupEnum.project


# noinspection PyAbstractClass,PyClassHasNoInit
class InstanceDetailTabs(project_tabs.InstanceDetailTabs):
    """Default instance details tab group"""
    tabs = (NonVDIOverviewTab,
            VDIOverviewTab,
            TnxLogTab,
            TnxConsoleTab,
            AuditTab,
            InstanceMetricsTab,
            InstanceBareosTemplatesTab)
    obj_type = ObjectTypeEnum.instance
    obj_group = ObjectGroupEnum.project
