# coding: utf-8
# pylint: disable=no-member, too-many-lines

import mock

from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.test import override_settings
from django.utils.http import urlencode

from openstack_auth import policy as policy_backend

from horizon import exceptions
from horizon import messages
from horizon.utils import functions as utils

from openstack_dashboard import api

from requests.exceptions import ConnectionError

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.project.project_instances import (
    actions, tables, tabs, views, forms, workflows
)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.tests import settings
from tionix_dashboard.tests.fake_cloud_state import ALL_SERVERS_COUNT
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, FakeUser, patch_resource_get,
    BaseAdminClientTestCase)


INDEX_URL = 'horizon:project:instances:index'
DETAIL_URL = 'horizon:project:instances:detail'
VOLUME_DETAIL_URL = 'horizon:project:volumes:volumes:detail'
DELETE_URL = "horizon:project:instances:delete"

GET_CONSOLE_PATH = (
    'tionix_dashboard.dashboards.project.project_instances.console.get_console'
)


@mock.patch.object(api.nova, 'extension_supported', mock.Mock())
class TnxProjectInstanceActionsTest(BaseClientTestCase):

    def test_tnx_toggle_mixin(self):
        for instance in self.tnx_servers.list():
            action = actions.TnxToggleMixin()
            self.assertTrue(action.allowed(self.request, instance))

    def test_tnx_multi_toggle_pause(self):
        instance1 = self.tnx_servers.list()[0]
        instance4 = self.tnx_servers.list()[3]

        action = actions.TnxMultiTogglePause()

        self.assertTrue(action.allowed(self.request, instance1))
        self.assertFalse(action.allowed(self.request, instance4))

        self.assertEqual(action.current_present_action, 0)

    def test_tnx_multi_toggle_unpause(self):
        instance1 = self.tnx_servers.list()[0]
        instance4 = self.tnx_servers.list()[3]

        action = actions.TnxMultiToggleUnPause()

        self.assertFalse(action.allowed(self.request, instance1))
        self.assertTrue(action.allowed(self.request, instance4))

        self.assertEqual(action.current_present_action, 0)

        self.assertEqual(action.action_present(1), u'Unpause Instance')
        self.assertEqual(action.action_present(2), u'Unpause Instances')
        self.assertEqual(action.action_past(1), u'Unpaused Instance')
        self.assertEqual(action.action_past(2), u'Unpaused Instances')

    def test_tnx_multi_toggle_suspend(self):
        instance1 = self.tnx_servers.list()[0]
        instance5 = self.tnx_servers.list()[4]

        action = actions.TnxMultiToggleSuspend()

        self.assertTrue(action.allowed(self.request, instance1))
        self.assertFalse(action.allowed(self.request, instance5))

        self.assertEqual(action.current_present_action, 0)

    def test_tnx_multi_toggle_resume(self):
        instance1 = self.tnx_servers.list()[0]
        instance5 = self.tnx_servers.list()[4]

        action = actions.TnxMultiToggleResume()

        self.assertFalse(action.allowed(self.request, instance1))
        self.assertTrue(action.allowed(self.request, instance5))

        self.assertEqual(action.current_present_action, 1)

        self.assertEqual(action.action_present(1), u'Resume Instance')
        self.assertEqual(action.action_present(2), u'Resume Instances')
        self.assertEqual(action.action_past(1), u'Resumed Instance')
        self.assertEqual(action.action_past(2), u'Resumed Instances')

    def test_tnx_multi_toggle_shelve(self):
        instance1 = self.tnx_servers.list()[0]
        instance6 = self.tnx_servers.list()[5]

        action = actions.TnxMultiToggleShelve()

        self.assertTrue(action.allowed(self.request, instance1))
        self.assertFalse(action.allowed(self.request, instance6))

        self.assertEqual(action.current_present_action, 0)

        paused_instance = self.tnx_servers.list()[6]
        suspended_instance = self.tnx_servers.list()[7]
        self.assertEqual(getattr(paused_instance, "OS-EXT-STS:power_state"), 3)
        self.assertEqual(getattr(suspended_instance,
                                 "OS-EXT-STS:power_state"), 7)
        self.assertFalse(action.allowed(self.request, paused_instance))
        self.assertFalse(action.allowed(self.request, suspended_instance))

    def test_tnx_multi_toggle_unshelve(self):
        instance1 = self.tnx_servers.list()[0]
        instance6 = self.tnx_servers.list()[5]

        action = actions.TnxMultiToggleUnShelve()

        self.assertFalse(action.allowed(self.request, instance1))
        self.assertTrue(action.allowed(self.request, instance6))

        self.assertEqual(action.current_present_action, 1)

        self.assertEqual(action.action_present(1), u'Unshelve Instance')
        self.assertEqual(action.action_present(2), u'Unshelve Instances')
        self.assertEqual(action.action_past(1), u'Unshelved Instance')
        self.assertEqual(action.action_past(2), u'Unshelved Instances')

    def test_tnx_toggle_pause(self):

        action = actions.TnxTogglePause()

        self.assertEqual(
            action.action_present(1),
            (u'Pause Instance', u'Unpause Instance')
        )
        self.assertEqual(
            action.action_present(2),
            (u'Pause Instances', u'Unpause Instances')
        )
        self.assertEqual(
            action.action_past(1),
            (u'Paused Instance', u'Unpaused Instance')
        )
        self.assertEqual(
            action.action_past(2),
            (u'Paused Instances', u'Unpaused Instances')
        )

    def test_tnx_stop_instance(self):
        instance1 = self.tnx_servers.list()[0]
        instance3 = self.tnx_servers.list()[2]

        action = actions.TnxStopInstance()

        self.assertTrue(action.allowed(self.request, None))

        self.assertTrue(action.allowed(self.request, instance1))

        self.assertFalse(action.allowed(self.request, instance3))

    @mock.patch.object(TNXServersClient, 'update', mock.Mock())
    def test_instance_update_cell(self):
        for instance in self.tnx_servers.list():
            action = actions.InstanceUpdateCell()
            self.assertTrue(
                action.update_cell(
                    self.request, instance, instance.id, 'name', 'new_name_val'
                )
            )

    @mock.patch.object(TNXServersClient,
                       'update',
                       mock.Mock(side_effect=Exception))
    def test_instance_update_cell_error(self):
        instance = self.tnx_servers.list()[0]

        action = actions.InstanceUpdateCell()

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        self.assertFalse(
            action.update_cell(
                request, instance, instance.id, 'name', 'new_name_val'
            )
        )

    @mock.patch.object(api.keystone, 'tenant_get')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.network, 'servers_update_addresses', mock.Mock())
    def test_update_action(self, flavor_get, tenant_get):
        instance = self.tnx_servers.list()[0]
        tenant = self.tenants.list()[0]
        flavor = self.flavors.list()[0]
        hypervisor = self.tnx_hypervisors.list()[0]

        tenant_get.return_value = tenant
        flavor_get.return_value = flavor

        action = actions.UpdateRow(table=tables.TnxInstancesTable)

        new_instance = action.get_data(self.request, instance.id)

        self.assertEqual(new_instance.id, instance.id)
        self.assertEqual(new_instance.full_flavor, flavor)
        self.assertEqual(new_instance.host_id, hypervisor.id)
        self.assertEqual(
            new_instance.node_title,
            getattr(hypervisor, 'tnx_node_title', '')
        )

    @mock.patch.object(api.keystone, 'tenant_get')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.network, 'servers_update_addresses', mock.Mock())
    def test_update_action_no_tnx(self, flavor_get, tenant_get):
        instance = self.tnx_servers.list()[10]
        tenant = self.tenants.list()[1]
        flavor = self.flavors.list()[0]
        hypervisor = self.tnx_hypervisors.list()[4]

        tenant_get.return_value = tenant
        flavor_get.return_value = flavor

        action = actions.UpdateRow(table=tables.TnxInstancesTable)

        new_instance = action.get_data(self.request, instance.id)

        self.assertEqual(new_instance.id, instance.id)
        self.assertEqual(new_instance.full_flavor, flavor)
        self.assertEqual(new_instance.host_id, hypervisor.id)
        self.assertEqual(
            new_instance.node_title,
            getattr(hypervisor, 'tnx_node_title', '')
        )

    @mock.patch.object(LicenseController, 'is_valid',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXServersClient, 'reset_with_reboot', mock.Mock())
    @mock.patch.object(messages, 'success', mock.Mock())
    def test_reset_state_action(self):
        instance1 = self.tnx_servers.list()[0]
        instance3 = self.tnx_servers.list()[2]

        action = actions.ResetStateAction()

        self.assertFalse(action.allowed(self.request, instance1))

        self.assertTrue(action.allowed(self.request, instance3))


@mock.patch.object(api.nova, 'extension_supported', mock.Mock())
class TnxProjectInstancesTablesTestCase(BaseClientTestCase):

    def test_tnx_instances_table(self):

        table = tables.TnxInstancesTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'name', 'image_name', 'ip', 'size',
                    'status', 'az', 'task', 'state', 'created', 'actions',
                    'type'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_instances_actions(self):
        servers = self.servers.list()

        table = tables.TnxInstancesTable(self.request, data=servers)

        actions_list = [action.name for action in table.get_table_actions()]

        self.assertEqual(
            len({'filter', 'instance_schedule', 'delete', 'start', 'stop',
                 'soft_reboot', 'reset_state', 'tnx_pause',
                 'tnx_unpause', 'tnx_suspend', 'tnx_resume', 'tnx_shelve',
                 'tnx_unshelve', 'launch_base', 'reboot'} - set(actions_list)),
            0
        )

        launch = getattr(settings, 'LAUNCH_INSTANCE_LEGACY_ENABLED', False)
        launch_ng = getattr(settings, 'LAUNCH_INSTANCE_NG_ENABLED', True)

        if launch and launch_ng:
            self.assertIn('launch', actions_list)
            self.assertIn('launch-ng', actions_list)
        elif launch:
            self.assertIn('launch', actions_list)
            self.assertNotIn('launch-ng', actions_list)
        elif launch_ng:
            self.assertIn('launch-ng', actions_list)
            self.assertNotIn('launch', actions_list)
        else:
            self.assertNotIn('launch-ng', actions_list)
            self.assertNotIn('launch', actions_list)

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.network, 'floating_ip_simple_associate_supported',
                       mock.Mock(return_value=True))
    @override_settings(
        OPENSTACK_ENABLE_PASSWORD_RETRIEVE=True)
    def test_tnx_instances_row_actions_1(self):
        table = tables.TnxInstancesTable(self.request)

        instance = self.tnx_servers.list()[0]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instance_schedule', 'snapshot', 'associate-simple',
                    'attach_interface', 'detach_interface', 'edit',
                    'update_metadata', 'decryptpassword', 'edit_secgroups',
                    'console', 'log', 'pause', 'suspend', 'resize', 'lock',
                    'unlock', 'soft_reboot', 'reboot', 'stop', 'rebuild',
                    'delete', 'shelve', 'show_instance_metrics', 'clone-ng',
                    'volume_attachment'])
        )

        instance = self.tnx_servers.list()[2]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instance_schedule', 'edit', 'lock', 'unlock', 'delete',
                    'reset_state', 'show_instance_metrics'])
        )

        instance = self.tnx_servers.list()[8]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instance_schedule', 'confirm', 'revert',
                    'associate-simple', 'edit', 'update_metadata', 'lock',
                    'unlock', 'stop', 'delete', 'show_instance_metrics',
                    'clone-ng'])

        )

        shutoff_instance = self.tnx_servers.list(
            search_opts={'status': 'SHUTOFF'})[0]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(shutoff_instance)]),
            sorted(['instance_schedule', 'start', 'snapshot',
                    'associate-simple', 'attach_interface', 'detach_interface',
                    'edit', 'update_metadata', 'decryptpassword', 'resize',
                    'lock', 'unlock', 'soft_reboot', 'reboot', 'stop',
                    'rebuild', 'delete', 'show_instance_metrics', 'clone-ng',
                    'volume_attachment'])
        )

        # instances without tnx fields

        instance = self.tnx_servers.list()[10]
        self.request.user.tenant_id = instance.tenant_id
        table = tables.TnxInstancesTable(self.request)

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['instance_schedule', 'snapshot', 'associate-simple',
                    'attach_interface', 'detach_interface', 'edit',
                    'update_metadata', 'decryptpassword', 'edit_secgroups',
                    'console', 'log', 'pause', 'suspend', 'resize', 'lock',
                    'unlock', 'soft_reboot', 'reboot', 'stop', 'rebuild',
                    'delete', 'shelve', 'show_instance_metrics', 'clone-ng',
                    'volume_attachment'])
        )

    def test_tnx_instances_row_actions_2(self):
        table = tables.TnxInstancesTable(self.request)

        instance = self.tnx_servers.list()[3]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['snapshot', 'edit', 'update_metadata', 'unlock', 'lock',
                    'pause', 'delete', 'show_instance_metrics', 'clone-ng',
                    'instance_schedule', 'volume_attachment'])

        )

        # instances without tnx fields

        instance = self.tnx_servers.list()[11]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['snapshot', 'edit', 'update_metadata', 'stop', 'unlock',
                    'lock', 'pause', 'delete', 'show_instance_metrics',
                    'clone-ng', 'instance_schedule', 'volume_attachment'])

        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_instances_actions(self):
        volumes = self.volumes.list()

        table = tables.InstancesTable(self.request, data=volumes)

        actions_list = [action.name for action in table.get_table_actions()]

        self.assertEqual(
            len({'instance_schedule'} - set(actions_list)), 0
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_instances_row_actions(self):

        table = tables.InstancesTable(self.request)

        instance = self.servers.first()
        actions_list = [action.name
                        for action in table.get_row_actions(instance)]
        self.assertEqual(
            len({'instance_schedule', 'show_instance_metrics'} -
                set(actions_list)),
            0
        )

    @override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
    def test_instance_detail_bareos_table(self):

        table = tables.BareosTemplatesTable(self.request)

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(Empty(id=None))]),
            sorted([])
        )


class TnxProjectInstancesTablesAdminTestCase(BaseAdminClientTestCase):
    @override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
    def test_instance_detail_bareos_table(self):

        table = tables.BareosTemplatesTable(self.request)

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(Empty(id=None))]),
            sorted(['unassign'])
        )


class TnxProjectInstancesTabsTestCase(BaseClientTestCase):

    def test_tnx_log_tab_context_data(self):
        instance = self.tnx_servers.list()[0]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxLogTab(tab_group=tab_group, request=self.request)

        self.assertDictEqual(
            tab.get_context_data(self.request),
            {
                'instance': instance,
                'log_length': utils.get_log_length(self.request),
                'buttons_disabled': False,
                'console_log': 'Test data'
            }
        )

    def test_tnx_log_tab_context_data_no_tnx(self):
        instance = self.tnx_servers.list()[11]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxLogTab(tab_group=tab_group, request=self.request)

        self.assertDictEqual(
            tab.get_context_data(self.request),
            {
                'instance': instance,
                'log_length': utils.get_log_length(self.request),
                'buttons_disabled': False,
                'console_log': 'Test data'
            }
        )

    def test_tnx_log_tab_context_data_console_error(self):
        instance = self.tnx_servers.list()[2]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxLogTab(tab_group=tab_group, request=self.request)

        context = tab.get_context_data(self.request)

        self.assertEqual(
            context['console_log'],
            u'Unable to get log for instance "{0}".'.format(instance.id)
        )

    @mock.patch.object(api.nova, 'server_console_output', mock.Mock())
    def test_tnx_log_tab_context_data_without_state(self):
        instance = self.tnx_servers.list()[1]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxLogTab(tab_group=tab_group, request=self.request)

        context = tab.get_context_data(self.request)

        self.assertTrue(context['buttons_disabled'])

        self.assertEqual(
            context['console_log'],
            u'Log is not available for an instance without a state.'
        )

    @mock.patch(
        GET_CONSOLE_PATH,
        mock.Mock(return_value=('TYPE', u'/console/url')))
    def test_tnx_console_tab_context_data(self):
        instance = self.tnx_servers.list()[0]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxConsoleTab(tab_group=tab_group, request=self.request)

        self.assertDictEqual(
            tab.get_context_data(self.request),
            {
                'instance_id': instance.id,
                'console_type': 'TYPE',
                'console_url': u'/console/url',
                'fullscreen_console_url': (
                    u'/project/instances/%s/fullscreen_console' % instance.id)
            }
        )

    @mock.patch(
        GET_CONSOLE_PATH,
        mock.Mock(return_value=('TYPE', u'/console/url')))
    def test_tnx_console_tab_context_data_no_tnx(self):
        instance = self.tnx_servers.list()[11]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxConsoleTab(tab_group=tab_group, request=self.request)

        self.assertDictEqual(
            tab.get_context_data(self.request),
            {
                'instance_id': instance.id,
                'console_type': 'TYPE',
                'console_url': u'/console/url',
                'fullscreen_console_url': (
                    u'/project/instances/%s/fullscreen_console' % instance.id)
            }
        )

    @mock.patch(
        GET_CONSOLE_PATH,
        mock.Mock(return_value=('SERIAL', u'/console/url')))
    def test_tnx_console_tab_context_data_serial(self):
        instance = self.tnx_servers.list()[0]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxConsoleTab(tab_group=tab_group, request=self.request)

        self.assertDictEqual(
            tab.get_context_data(self.request),
            {
                'instance_id': instance.id,
                'console_type': 'SERIAL',
                'console_url': u'/project/instances/{0}/serial'.format(
                    instance.id
                ),
                'fullscreen_console_url': (
                    u'/project/instances/%s/fullscreen_console' % instance.id)
            }
        )

    @mock.patch(GET_CONSOLE_PATH, mock.Mock())
    def test_tnx_console_tab_context_data_without_state(self):
        instance = self.tnx_servers.list()[1]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tab = tabs.TnxConsoleTab(tab_group=tab_group, request=self.request)

        self.assertDictEqual(
            tab.get_context_data(self.request),
            {
                'instance_id': instance.id,
                'console_type': 'AUTO',
                'console_url': None,
                'fullscreen_console_url': (
                    u'/project/instances/%s/fullscreen_console' % instance.id)
            }
        )

    @mock.patch(GET_CONSOLE_PATH, mock.Mock())
    @mock.patch.object(api.keystone, 'tenant_get')
    def test_tnx_overview_tab_context_data(self, tenant_get):
        instance = self.tnx_servers.list()[1]
        instance.security_groups = [Empty(id='test sec gorup')]

        tab_group = tabs.TnxInstanceDetailTabs(
            self.request, instance=instance
        )
        tenant = self.tenants.list()[0]
        tenant_get.return_value = tenant

        tab = tabs.VDIOverviewTab(tab_group=tab_group, request=self.request)

        context_data = tab.get_context_data(self.request)

        self.assertDictEqual(
            context_data,
            {
                'instance': instance,
                'is_superuser': False,
                'description': '',
                'is_vdi': True,
                'project_name': tenant.name,
                'properties': [],
                'up_time': '99,999',
                'recovery_time': '8',
                'movable': None
            }
        )

        self.assertTrue(
            hasattr(context_data['instance'].security_groups[0], 'detail_url')
        )


@mock.patch.object(LicenseController, 'is_valid', mock.Mock(return_value=True))
class TnxProjectInstanceTabsTest(BaseClientTestCase):

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=True))
    @mock.patch('tionix_dashboard.dashboards.tionix.metrics.tabs.'
                'is_bareos_available', mock.Mock(return_value=True))
    def test_tnx_base_instance_detail_tabs(self):

        tab_group = tabs.TnxInstanceDetailTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['overview_vdi', 'log', 'console', 'audit', 'scheduler',
                    'metrics', 'bareos_templates_tab'])
        )

    @mock.patch('tionix_dashboard.dashboards.tionix.metrics.tabs.'
                'is_bareos_available', mock.Mock(return_value=True))
    def test_instance_detail_tabs(self):

        tab_group = tabs.InstanceDetailTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['overview_vdi', 'log', 'console', 'audit', 'metrics',
                    'bareos_templates_tab'])
        )


# pylint: disable-msg=too-many-arguments
@mock.patch.object(api.network, 'floating_ip_supported', mock.Mock())
@mock.patch.object(api.network,
                   'floating_ip_simple_associate_supported',
                   mock.Mock())
@mock.patch.object(api.network, 'servers_update_addresses', mock.Mock())
@mock.patch.object(api.nova, 'extension_supported', mock.Mock())
@mock.patch.object(LicenseController, 'is_valid', mock.Mock(return_value=True))
class TnxProjectInstanceViewsTest(BaseClientTestCase):

    @mock.patch.object(api.nova, 'tenant_absolute_limits')
    @mock.patch.object(api.glance, 'image_list_detailed')
    @mock.patch.object(api.glance, 'image_get')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.keystone, 'tenant_list')
    def test_tnx_project_instances_index_view_get_data(
            self, tenant_list, flavor_list, flavor_get,
            image_get, image_list_detailed, tenant_absolute_limits):
        tenants = self.tenants.list()
        flavors = self.tnx_flavors.list()
        images = self.tnx_images.list()
        instance = self.tnx_servers.list()[0]
        instance.full_flavor = self.tnx_flavors.get(instance.flavor['id'])

        tenant_list.return_value = (tenants, False)
        flavor_list.return_value = flavors
        image_list_detailed.return_value = (images, None, None)
        flavor_get.side_effect = patch_resource_get(self.tnx_flavors)
        image_get.side_effect = patch_resource_get(self.tnx_images)
        tenant_absolute_limits.return_value = self.limits['absolute']

        view = views.TnxProjectInstancesIndexView()
        view.request = self.request
        view.kwargs = {}

        res = self.client.get(reverse(INDEX_URL))

        table = res.context_data['table']

        self.assertEqual(table.total_count, ALL_SERVERS_COUNT)

        self.assertContains(res, u'fake_instance')
        self.assertContains(res, u'Image 1')
        self.assertContains(res, u'Active')
        self.assertContains(res, u'nova')
        self.assertContains(res, u'Running')

    @mock.patch.object(api.nova, 'tenant_absolute_limits')
    @mock.patch.object(api.glance, 'image_list_detailed')
    @mock.patch.object(api.glance, 'image_get')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.keystone, 'tenant_list')
    @mock.patch.object(messages, 'error')
    @mock.patch.object(TNXServersClient, 'get_paginator')
    def test_tnx_project_instances_index_view_get_data_nova_unavailable(
            self, get_paginator, error, tenant_list, flavor_list, flavor_get,
            image_get, image_list_detailed, tenant_absolute_limits):
        tenants = self.tenants.list()
        flavors = self.tnx_flavors.list()
        images = self.tnx_images.list()
        instance = self.tnx_servers.list()[0]
        instance.full_flavor = self.tnx_flavors.get(instance.flavor['id'])

        get_paginator.side_effect = ConnectionError()

        tenant_list.return_value = (tenants, False)
        flavor_list.return_value = flavors
        image_list_detailed.return_value = (images, None, None)
        flavor_get.side_effect = patch_resource_get(self.tnx_flavors)
        image_get.side_effect = patch_resource_get(self.tnx_images)
        tenant_absolute_limits.return_value = self.limits['absolute']

        request = get_fake_request(
            user=FakeUser(),
            get_full_path=lambda: '',
            session={})

        view = views.TnxProjectInstancesIndexView()
        view.table = tables.TnxInstancesTable(request)
        view.request = request
        view.kwargs = {}
        view.get_data()
        self.assertTrue(error.called)

    @mock.patch.object(api.network, 'server_security_groups')
    @mock.patch.object(api.nova, 'instance_volumes_list')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.glance, 'image_get')
    @mock.patch.object(api.keystone, 'tenant_get')
    def test_tnx_detail_view_context_data(
            self, tenant_get, image_get, flavor_get, instance_volumes_list,
            server_security_groups):
        instance = self.tnx_servers.list()[0]
        flavor = self.tnx_flavors.get(instance.flavor['id'])
        image = self.tnx_images.get(instance.image['id'])
        volumes = self.volumes.list()[:1]
        tenant = self.tenants.list()[0]

        security_groups = self.security_groups.list()[:1]

        flavor_get.return_value = flavor
        image_get.return_value = image
        instance_volumes_list.return_value = volumes
        server_security_groups.return_value = security_groups
        tenant_get.return_value = tenant

        res = self.client.get(reverse(DETAIL_URL, args=(instance.id, )))

        self.assertContains(res, u'/project/images/1/')

        self.assertContains(
            res, u'/project/volumes/{0}/'.format(volumes[0].id)
        )

    def test_reset_state_view_get_objects(self):
        instance = self.tnx_servers.list()[2]

        view = views.ResetStateView()

        view.request = get_fake_request(
            get_params={
                'id': [instance.id]
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        instance_from_view = view.get_objects()[0]

        self.assertIsNotNone(instance_from_view)

        self.assertEqual(instance.id, instance_from_view.id)

    def test_reset_state_view_get_objects_no_tnx(self):
        instance = self.tnx_servers.list()[12]

        view = views.ResetStateView()

        view.request = get_fake_request(
            get_params={
                'id': [instance.id]
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        instance_from_view = view.get_objects()[0]

        self.assertIsNotNone(instance_from_view)

        self.assertEqual(instance.id, instance_from_view.id)

    def test_reset_state_view_get_objects_raise(self):
        instance = self.tnx_servers.list()[0]

        view = views.ResetStateView()

        view.request = get_fake_request(
            get_params={
                'id': [instance.id]
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        with self.assertRaises(exceptions.Http302):
            view.get_objects()

    def test_reset_state_view_get_initial(self):
        instance = self.tnx_servers.list()[2]

        view = views.ResetStateView()

        view.request = get_fake_request(
            get_params={
                'id': [instance.id]
            }
        )

        initial = view.get_initial()

        self.assertEqual(initial['instances'][0].id, instance.id)

    def test_reset_state_view_get_initial_no_tnx(self):
        instance = self.tnx_servers.list()[12]

        view = views.ResetStateView()

        view.request = get_fake_request(
            get_params={
                'id': [instance.id]
            }
        )

        initial = view.get_initial()

        self.assertEqual(initial['instances'][0].id, instance.id)

    @mock.patch.object(views.DeleteInstanceView,
                       'get_tasks_rows',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.nova, 'server_delete', mock.Mock(return_value=True))
    @mock.patch.object(api.nova, 'server_get')
    def test_delete_from_index(self, server_get):
        for instance in self.tnx_servers.list():
            server_get.return_value = instance

            url = '?'.join([reverse(DELETE_URL),
                            urlencode({'id': instance.id})])

            req = self.factory.post(url)

            index_url = reverse(INDEX_URL)
            req.META['HTTP_REFERER'] = ''.join([settings.TESTSERVER,
                                                settings.WEBROOT,
                                                index_url])
            req.META['HTTP_HOST'] = settings.TESTSERVER

            response = views.DeleteInstanceView.as_view()(req)

            self.assertIn(index_url, response.url)

    @mock.patch.object(views.DeleteInstanceView,
                       'get_tasks_rows',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.nova, 'server_delete', mock.Mock(return_value=True))
    @mock.patch.object(api.nova, 'server_get')
    def test_delete_from_detail(self, server_get):
        for instance in self.tnx_servers.list():
            server_get.return_value = instance

            url = '?'.join([reverse(DELETE_URL),
                            urlencode({'id': instance.id})])

            req = self.factory.post(url)

            index_url = reverse(INDEX_URL)
            req.META['HTTP_REFERER'] = ''.join([
                settings.TESTSERVER,
                settings.WEBROOT,
                reverse(DETAIL_URL, args=(instance.id,))])
            req.META['HTTP_HOST'] = settings.TESTSERVER

            response = views.DeleteInstanceView.as_view()(req)

            self.assertIn(index_url, response.url)

    @mock.patch.object(api.nova, 'server_get')
    @mock.patch.object(api.glance, 'image_get')
    @mock.patch.object(api.network, 'server_security_groups')
    @mock.patch.object(api.neutron, 'list_resources_with_long_filters')
    def test_clone_get_initial(self, list_resources_with_long_filters,
                               server_security_groups, image_get, server_get):
        instance = self.tnx_servers.list()[0]
        network = self.tnx_networks.list()[0]
        image = self.tnx_images.list()[0]
        port = Empty(id="port 1", network_id=network.id)
        security_group = Empty(id="group 1")

        list_resources_with_long_filters.return_value = [port]
        server_security_groups.return_value = [security_group]
        image_get.return_value = image
        server_get.return_value = instance

        view = views.CloneInstanceView()
        view.request = self.request
        view.kwargs = {'instance_id': instance.id}

        self.assertDictEqual(view.get_initial().dict(),
                             {u'user_id': self.request.user.id,
                              u'name': u'Clone of {0}'.format(instance.name),
                              u'availability_zone': instance.availability_zone,
                              u'source_type': 'image_id',
                              u'disk_config': getattr(instance,
                                                      'OS-DCF:diskConfig',
                                                      None),
                              u'network': {network.id},
                              u'keypair': instance.key_name,
                              u'groups': [security_group.id],
                              u'source_id': image.id,
                              u'flavor': instance.flavor['id'],
                              u'project_id': self.request.user.tenant_id,
                              u'config_drive': False,
                              u'description': 'desc'})

    @mock.patch.object(api.nova, 'server_get')
    @mock.patch.object(api.glance, 'image_get')
    @mock.patch.object(api.network, 'server_security_groups')
    @mock.patch.object(api.neutron, 'list_resources_with_long_filters')
    def test_clone_get_initial_no_tnx(self, list_resources_with_long_filters,
                                      server_security_groups, image_get,
                                      server_get):
        instance = self.tnx_servers.list()[10]
        network = self.tnx_networks.list()[0]
        image = self.tnx_images.list()[1]
        port = Empty(id="port 1", network_id=network.id)
        security_group = Empty(id="group 1")

        list_resources_with_long_filters.return_value = [port]
        server_security_groups.return_value = [security_group]
        image_get.return_value = image
        server_get.return_value = instance

        view = views.CloneInstanceView()
        view.request = self.request
        view.kwargs = {'instance_id': instance.id}

        self.assertDictEqual(view.get_initial().dict(),
                             {u'user_id': self.request.user.id,
                              u'name': u'Clone of {0}'.format(instance.name),
                              u'availability_zone': instance.availability_zone,
                              u'source_type': 'image_id',
                              u'disk_config': getattr(instance,
                                                      'OS-DCF:diskConfig',
                                                      None),
                              u'network': {network.id},
                              u'keypair': instance.key_name,
                              u'groups': [security_group.id],
                              u'source_id': image.id,
                              u'flavor': instance.flavor['id'],
                              u'project_id': self.request.user.tenant_id,
                              u'config_drive': False,
                              u'description': 'desc'})


class TnxProjectInstanceFormsTest(BaseClientTestCase):

    def test_get_pow_ticks(self):
        self.assertDictEqual(
            {0: 2, 1: 4, 2: 8, 3: 16, 4: 32},
            forms.get_pow_ticks(1, 5)
        )

    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.glance, 'image_list_detailed')
    def test_launch_base_instance_fields(self,
                                         image_list_detailed,
                                         flavor_list,
                                         network_list_for_tenant):

        images = self.tnx_images.list()
        flavors = self.tnx_flavors.list()
        networks = self.tnx_networks.list()

        image_list_detailed.return_value = (images, None, None)
        flavor_list.return_value = flavors
        network_list_for_tenant.return_value = networks

        form = forms.LaunchBaseInstanceForm(self.request)

        self.assertListEqual(
            sorted(form.fields.keys()),
            sorted(['name', 'image_id', 'network_id', 'vcpus', 'ram', 'disk',
                    'count'])
        )

    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.glance, 'image_list_detailed')
    def test_launch_base_instance_clean(self,
                                        image_list_detailed,
                                        flavor_list,
                                        network_list_for_tenant):

        images = self.tnx_images.list()
        flavors = self.tnx_flavors.list()
        networks = self.tnx_networks.list()

        image_list_detailed.return_value = (images, None, None)
        flavor_list.return_value = flavors
        network_list_for_tenant.return_value = networks

        form = forms.LaunchBaseInstanceForm(self.request)
        form.cleaned_data = {'vcpus': 1, 'ram': 512, 'disk': 50}

        cleaned_data = form.clean()

        self.assertEqual(cleaned_data['flavor_id'], flavors[0].id)

    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.glance, 'image_list_detailed')
    def test_launch_base_instance_clean_error(self,
                                              image_list_detailed,
                                              flavor_list,
                                              network_list_for_tenant):

        images = self.tnx_images.list()
        flavors = self.tnx_flavors.list()
        networks = self.tnx_networks.list()

        image_list_detailed.return_value = (images, None, None)
        flavor_list.return_value = flavors
        network_list_for_tenant.return_value = networks

        form = forms.LaunchBaseInstanceForm(self.request)
        form.cleaned_data = {'vcpus': 0, 'ram': 0, 'disk': 0}

        with self.assertRaises(ValidationError):
            form.clean()

    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.glance, 'image_list_detailed')
    @mock.patch.object(TNXServersClient, 'get_resource')
    def test_launch_base_instance_handle(self,
                                         get_resource,
                                         image_list_detailed,
                                         flavor_list,
                                         network_list_for_tenant):
        networks = self.tnx_networks.list()
        images = self.tnx_images.list()
        flavors = self.tnx_flavors.list()
        nova_client = novaclient(self.request)
        min_count = 2

        image_list_detailed.return_value = (images, None, None)
        flavor_list.return_value = flavors
        network_list_for_tenant.return_value = networks
        get_resource.return_value = nova_client.servers

        request = get_fake_request(
            user=FakeUser(tenant_id=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.LaunchBaseInstanceForm(request)

        result = form.handle(request, {'name': 'test instance',
                                       'count': min_count,
                                       'network_id': networks[0].id,
                                       'image_id': images[0].id,
                                       'flavor_id': flavors[0].id})

        self.assertTrue(result)
        self.assertEqual(len(nova_client.servers.list()),
                         ALL_SERVERS_COUNT + min_count)

    @mock.patch.object(exceptions, 'handle', mock.Mock())
    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.glance, 'image_list_detailed')
    def test_launch_base_instance_handle_error(self,
                                               image_list_detailed,
                                               flavor_list,
                                               network_list_for_tenant):
        networks = self.tnx_networks.list()
        images = self.tnx_images.list()
        flavors = self.tnx_flavors.list()

        image_list_detailed.return_value = (images, None, None)
        flavor_list.return_value = flavors
        network_list_for_tenant.return_value = networks
        fake_nova_client = mock.Mock()
        fake_nova_client.servers.create.side_effect = Exception

        request = get_fake_request(
            user=FakeUser(tenant_id=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.LaunchBaseInstanceForm(request)

        result = form.handle(request, {'name': 'test instance',
                                       'network_id': networks[0].id,
                                       'image_id': images[0].id,
                                       'flavor_id': flavors[0].id})

        self.assertFalse(result)

    @mock.patch.object(TNXServersClient, 'reset_with_reboot', mock.Mock())
    def test_reset_state(self):
        instances = self.tnx_servers.list()[:2]

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ResetStateForm(request)
        form.initial = {'instances': instances}

        form.handle(request, data={'reboot': True})

        success_message = request.horizon['async_messages'][0]

        self.assertEqual(success_message[0], 'success')

        self.assertTrue(
            success_message[1],
            u'Instances are preparing the state reset: '
            u'"{0}".'.format(u'", "'.join([i.name for i in instances])))

    @mock.patch.object(TNXServersClient,
                       'reset_with_reboot',
                       mock.Mock(side_effect=Exception))
    def test_reset_state_error(self):
        instances = self.tnx_servers.list()[:2]

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ResetStateForm(request)
        form.initial = {'instances': instances}

        form.handle(request, data={'reboot': True})

        self.assertTrue(
            request.horizon['async_messages'][0][1],
            u'Failed to reset states fot instances: '
            u'"{0}".'.format(u'", "'.join([i.name for i in instances])))

    @mock.patch.object(TNXServersClient, 'reset_with_reboot')
    def test_reset_state_mix(self, reset_with_reboot):
        instances = self.tnx_servers.list()[:2]

        # noinspection PyUnusedLocal
        def fake_reset_with_reboot(req, _id, *args, **kwargs):
            if _id != instances[0].id:
                raise Exception
            return None

        reset_with_reboot.side_effect = fake_reset_with_reboot

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ResetStateForm(request)
        form.initial = {'instances': instances}

        form.handle(request, data={'reboot': True})

        self.assertTrue(
            request.horizon['async_messages'][0][1],
            u'Failed to reset states fot instances: '
            u'"{0}".'.format(instances[1].name))

        success_message = request.horizon['async_messages'][1]

        self.assertEqual(success_message[0], 'info')

        self.assertTrue(
            success_message[1],
            u'Instances are preparing the state reset: '
            u'"{0}".'.format(instances[1].name))

    @mock.patch.object(api.nova, 'interface_attach', mock.Mock())
    @mock.patch('openstack_dashboard.dashboards.project.instances.utils.'
                'network_field_data')
    def test_attach_interface(self, network_field_data):
        instance = self.tnx_servers.list()[0]
        networks = self.networks.list()

        network_field_data.return_value = [(n.id, n.name_or_id)
                                           for n in networks]

        request = get_fake_request(
            user=FakeUser(tenant_id=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.AttachInterface(request)
        form.handle(request, data={'instance_id': instance.id,
                                   'network': 'network 1'})

        self.assertEqual(request.horizon['async_messages'][0][1],
                         u'Attaching interface for instance '
                         u'"{0}".'.format(instance.id))

    @mock.patch.object(api.nova,
                       'interface_attach',
                       mock.Mock(side_effect=Exception))
    @mock.patch('openstack_dashboard.dashboards.project.instances.utils.'
                'network_field_data')
    @mock.patch.object(exceptions, 'handle')
    def test_attach_interface_error(self, handle, network_field_data):
        instance = self.tnx_servers.list()[0]
        networks = self.networks.list()

        network_field_data.return_value = [(n.id, n.name_or_id)
                                           for n in networks]
        request = get_fake_request(
            user=FakeUser(tenant_id=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.AttachInterface(request)
        form.handle(request, data={'instance_id': instance.id,
                                   'network': 'network 1'})

        self.assertTrue(handle.called)

    @mock.patch.object(api.nova, 'interface_detach', mock.Mock())
    @mock.patch.object(api.neutron, 'port_list')
    def test_detach_interface(self, port_list):
        instance = self.tnx_servers.list()[0]
        ports = self.ports.list()

        port_list.return_value = ports

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        data = {'instance_id': instance.id, 'port': 'port 1'}

        form = forms.DetachInterface(request)
        form.handle(request, data=data)

        self.assertEqual(request.horizon['async_messages'][0][1],
                         u'Detached interface {port} for instance '
                         u'"{instance_id}".'.format(**data))

    @mock.patch.object(api.nova,
                       'interface_detach',
                       mock.Mock(side_effect=Exception))
    @mock.patch.object(api.neutron, 'port_list')
    @mock.patch.object(exceptions, 'handle')
    def test_detach_interface_error(self, handle, port_list):
        instance = self.tnx_servers.list()[0]
        ports = self.ports.list()

        port_list.return_value = ports

        request = get_fake_request(
            user=FakeUser(tenant_id=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        data = {'instance_id': instance.id, 'port': 'port 1'}

        form = forms.DetachInterface(request)
        form.handle(request, data=data)

        self.assertTrue(handle.called)

    @mock.patch('openstack_dashboard.dashboards.project.images.utils.'
                'get_available_images')
    def test_get_image_field_data(self, get_available_images):
        images = self.tnx_images.list()

        get_available_images.return_value = images

        image_field_data = forms.get_image_field_data(self.request)

        self.assertEqual(len(images) + 1, len(image_field_data))

    @mock.patch('openstack_dashboard.dashboards.project.images.utils.'
                'get_available_images')
    def test_get_image_field_data_no_data(self, get_available_images):

        get_available_images.return_value = []

        image_field_data = forms.get_image_field_data(self.request)

        self.assertEqual(image_field_data, [("", "No images available")])

    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    def test_get_network_field_data(self, network_list_for_tenant):
        networks = self.tnx_networks.list()

        network_list_for_tenant.return_value = networks

        network_field_data = forms.get_network_field_data(
            self.request, include_empty_option=False)

        self.assertEqual(len(networks), len(network_field_data))

    @mock.patch.object(api.neutron,
                       'network_list_for_tenant',
                       mock.Mock(side_effect=Exception))
    def test_get_network_field_data_error(self):

        network_field_data = forms.get_network_field_data(
            self.request, include_empty_option=False)

        self.assertEqual(len(network_field_data), 1)

    @mock.patch.object(api.neutron, 'network_list_for_tenant')
    def test_get_network_field_data_empty_option(self,
                                                 network_list_for_tenant):
        networks = self.tnx_networks.list()

        network_list_for_tenant.return_value = networks[:]

        network_field_data = forms.get_network_field_data(self.request)

        self.assertEqual(len(networks) + 1, len(network_field_data))


class ProjectInstanceWorkflowsTest(BaseClientTestCase):

    @mock.patch('tionix_dashboard.dashboards.project.project_instances.'
                'workflows.get_security_groups', mock.Mock(return_value=[]))
    @mock.patch('tionix_dashboard.dashboards.project.project_instances.'
                'workflows.get_security_groups_by_instance',
                mock.Mock(return_value=[]))
    def test_update_instance_tabs_test(self):

        workflow = workflows.UpdateInstance(self.request)

        self.assertListEqual(
            sorted([step.action.slug for step in workflow.steps]),
            sorted(['instance_info', 'update_security_groups'])
        )

    @mock.patch('tionix_dashboard.dashboards.project.project_instances.'
                'workflows.get_security_groups', mock.Mock(return_value=[]))
    @mock.patch('tionix_dashboard.dashboards.project.project_instances.'
                'workflows.get_security_groups_by_instance',
                mock.Mock(return_value=[]))
    def test_update_instance_fields_test(self):
        workflow = workflows.UpdateInstance(self.request)

        workflow_fields = []
        for step in workflow.steps:
            workflow_fields.extend(step.action.fields.keys())

        self.assertListEqual(
            sorted(workflow_fields),
            sorted(['name', 'description',
                    'default_update_security_groups_role',
                    'update_security_groups_role_member'])
        )
