# coding: utf-8
import json
import logging

import thread
from django.core.urlresolvers import reverse, reverse_lazy
from django.utils import http as utils_http
from django.utils.translation import (
    ugettext_lazy as _, ungettext_lazy, npgettext_lazy)
from django.shortcuts import redirect as redirect_to

from horizon import exceptions, forms, messages, tables, views
from horizon.tabs import views as tab_views
from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard.api.rest import (
    urls as rest_urls, utils as rest_utils, nova)
from openstack_dashboard.dashboards.project.images.images.tables import (
    get_image_type
)
from openstack_dashboard.dashboards.project.instances import (
    views as project_views)
from openstack_dashboard.dashboards.project.instances.tables import get_ips

from tionix_scheduler_client.enums import ObjectTypeEnum

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.project.project_instances.constants import (
    VM_STATUS_DISPLAY_CHOICES)
from tionix_dashboard.dashboards.project.project_instances.forms import (
    ResetStateForm, LaunchBaseInstanceForm, DeleteInstanceForm,
    TnxDeleteInstanceForm, AttachInterface, DetachInterface, AttachForm)
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    get_instance_security_groups, get_instance_networks_and_ports,
    delayed_volume_attach)
from tionix_dashboard.dashboards.project.project_instances.tables import (
    TnxInstancesTable, InstancesTable)
from tionix_dashboard.dashboards.project.project_instances.tabs import (
    TnxInstanceDetailTabs, InstanceDetailTabs)
from tionix_dashboard.dashboards.project.project_instances.workflows import (
    CloneInstance, LaunchInstance, UpdateInstance)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ObjectTasksView)
from tionix_dashboard.dashboards.utils import (
    update_instances_list, TABLE_OBJECT_ID_SEP)
from tionix_dashboard.helpers.utils import gen_uuid_object_name
from tionix_dashboard.helpers.views import (
    FilterStringModifierMixin, ServerFilterViewMixin, ServerSortMultiSelect,
    ServerPaginationViewMixin, RefererRedirectMixin, LinkBatchFormView,
    LinkBatchWorkflowViewMixin)
from tionix_dashboard.helpers.vm_constants import ERROR, VMTypes
from tionix_dashboard.helpers.workflows import WorkflowVerticalView
from tionix_dashboard.url_resolve_map import (
    PROJECT_SERVERS_INDEX, PROJECT_SERVERS_DETAIL)

logger = logging.getLogger(__name__)


class TnxProjectInstancesIndexView(FilterStringModifierMixin,
                                   ServerPaginationViewMixin,
                                   project_views.IndexView):
    table_class = TnxInstancesTable
    template_name = 'project_instances/index.html'

    def get_data(self):
        instances, total_count = [], 0
        self.marker_init()

        try:
            instances, self._more, self._prev, total_count = (
                TNXServersClient.get_paginator(
                    self.request,
                    search_opts=self.search_opts,
                    sort_dir=self.sort_dir,
                    last_page=self.is_last_page)
            )
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve instance list.'))

        instances = update_instances_list(self.request,
                                          instances,
                                          project=True)
        self.table.total_count = total_count

        return instances


class TnxIndexView(FilterStringModifierMixin, project_views.IndexView):
    table_class = TnxInstancesTable


class TnxDetailView(ServerFilterViewMixin,
                    ServerSortMultiSelect,
                    tab_views.TabbedTableView):
    tab_group_class = TnxInstanceDetailTabs
    template_name = 'horizon/common/_detail.html'
    failure_url = PROJECT_SERVERS_INDEX
    redirect_url = PROJECT_SERVERS_DETAIL
    page_title = _("Instance Details: {{ instance.name }}")
    image_url = 'horizon:project:images:images:detail'
    volume_url = 'horizon:project:volumes:volumes:detail'
    scheduler_tab_url = 'horizon:project:instances:scheduler_tab'
    actions_table = TnxInstancesTable
    _actions_table_instance = None

    def _get_url(self, instance):
        return reverse(self.redirect_url, args=[instance.id])

    def get_actions_table_instance(self, request, data):
        if self._actions_table_instance is None:
            self._actions_table_instance = self.actions_table(request, data)
        return self._actions_table_instance

    def get(self, request, *args, **kwargs):
        self.load_tabs()
        # Gather our table instances. It's important that they're the
        # actual instances and not the classes!
        table_instances = [t['table'] for t in self._table_dict.values()]
        # Early out before any tab or table data is loaded
        for table in table_instances:
            preempted = table.maybe_preempt()
            if preempted:
                return preempted

        # If we have an action, determine if it belongs to one of our tables.
        # We don't iterate through all of the tables' maybes_handle
        # methods; just jump to the one that's got the matching name.
        table_name, action, obj_id = tables.DataTable.check_handler(request)
        if table_name in self._table_dict:
            handled = self.handle_table(self._table_dict[table_name])
            if handled:
                return handled

        instance = self.get_data()
        # noinspection PyNoneFunctionAssignment
        table = self.get_actions_table_instance(self.request, [instance])
        actions = [table_action.name
                   for table_action in table.get_row_actions(instance)]
        if action in actions:
            handled = table.take_action(action, obj_id)
            if handled:
                return handled

        context = self.get_context_data(**kwargs)
        return self.handle_tabbed_response(context["tab_group"], context)

    def get_context_data(self, **kwargs):
        context = super(TnxDetailView, self).get_context_data(**kwargs)
        instance = self.get_data()
        if instance.image:
            instance.image_url = reverse(self.image_url,
                                         args=[instance.image['id']])
        instance.volume_url = self.volume_url
        context["instance"] = instance
        context["url"] = self._get_url(instance)
        context["actions"] = self._get_actions(instance)
        return context

    def _get_actions(self, instance):
        # noinspection PyNoneFunctionAssignment
        table = self.get_actions_table_instance(self.request, [instance])
        return table.render_row_actions(instance)

    def get_instance(self, instance_id):
        return TNXServersClient.get(self.request, instance_id)

    def get_security_groups(self, instance_id):
        return api.network.server_security_groups(self.request, instance_id)

    @memoized.memoized_method
    def get_data(self):
        instance_id = self.kwargs['instance_id']

        try:
            instance = self.get_instance(instance_id)
        except Exception as err:
            logger.error(unicode(err))
            redirect = reverse(self.failure_url)
            exceptions.handle(self.request,
                              _('Unable to retrieve details for '
                                'instance "%s".') % instance_id,
                              redirect=redirect)
            # Not all exception types handled above will result in a redirect.
            # Need to raise here just in case.
            raise exceptions.Http302(redirect)

        status_label = [label for (value, label) in VM_STATUS_DISPLAY_CHOICES
                        if value.lower() == (instance.status or '').lower()]
        if status_label:
            instance.status_label = status_label[0]
        else:
            instance.status_label = instance.status

        try:
            instance.volumes = api.nova.instance_volumes_list(self.request,
                                                              instance_id)
            # Sort by device name
            instance.volumes.sort(key=lambda vol: vol.device)
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve volume list for instance '
                    '"%(name)s" (%(id)s).') % {'name': instance.name,
                                               'id': instance_id}
            exceptions.handle(self.request, msg, ignore=True)

        try:
            instance.full_flavor = api.nova.flavor_get(
                self.request, instance.flavor["id"])
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve flavor information for instance '
                    '"%(name)s" (%(id)s).') % {'name': instance.name,
                                               'id': instance_id}
            exceptions.handle(self.request, msg, ignore=True)

        try:
            instance.security_groups = self.get_security_groups(instance_id)
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve security groups for instance '
                    '"%(name)s" (%(id)s).') % {'name': instance.name,
                                               'id': instance_id}
            exceptions.handle(self.request, msg, ignore=True)

        try:
            api.network.servers_update_addresses(self.request, [instance])
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve IP addresses from Neutron for '
                    'instance "%(name)s" (%(id)s).') % {'name': instance.name,
                                                        'id': instance_id}
            exceptions.handle(self.request, msg, ignore=True)

        return instance

    def get_tabs(self, request, **kwargs):
        instance = self.get_data()
        kwargs['obj_id'] = instance.id
        kwargs['instance'] = instance
        kwargs['scheduler_tab_url'] = reverse(self.scheduler_tab_url,
                                              args=(instance.id,))
        return self.tab_group_class(request, **kwargs)


# noinspection PyClassHasNoInit
class DetailView(project_views.DetailView):
    scheduler_tab_url = 'horizon:project:instances:scheduler_tab'

    tab_group_class = InstanceDetailTabs

    def get_tabs(self, request, *args, **kwargs):
        instance = self.get_data()
        kwargs['obj_id'] = instance.id
        kwargs['instance'] = instance
        kwargs['scheduler_tab_url'] = reverse(self.scheduler_tab_url,
                                              args=(instance.id,))
        return self.tab_group_class(request, **kwargs)


# noinspection PyClassHasNoInit
class IndexView(project_views.IndexView):
    table_class = InstancesTable


class FullScreenConsoleView(views.HorizonTemplateView):
    template_name = 'project/project_instances/_full_screen_console.html'
    return_url = PROJECT_SERVERS_DETAIL

    def get_context_data(self, **kwargs):
        return {'console_url': self.request.GET.get('console_url'),
                'return_url': reverse(self.return_url,
                                      args=(self.kwargs['instance_id'],))}


class DeleteInstanceView(ObjectTasksView, forms.ModalFormView):
    """ Delete instances view """
    form_class = DeleteInstanceForm
    modal_header = _('Delete Confirmation')
    template_name = "tionix_dashboard/delete_object.html"
    modal_id = "delete_instance"
    submit_label = _('Delete')
    context_object_name = 'instance'
    success_url = reverse_lazy("horizon:project:instances:index")
    confirm_url = reverse_lazy("horizon:project:instances:delete")
    hidden_title = False
    msg = _("Deleted instances are not recoverable.")

    def get_initial(self):
        ids, table = self.get_object_ids(), self.kwargs.get('table')
        names = {}

        for instance_id in ids:
            if table:
                instance = table.get_object_by_id(instance_id)
            else:
                try:
                    instance = api.nova.server_get(self.request, instance_id)
                except Exception as err:
                    logger.error(unicode(err))
                    redirect = self.get_redirect_url()
                    exceptions.handle(self.request,
                                      _('Unable to retrieve details for '
                                        'instance "%s".') % instance_id,
                                      redirect=redirect)
                    # Not all exception types handled above will result in a
                    # redirect. Need to raise here just in case.
                    raise exceptions.Http302(redirect)

            names[instance_id] = instance.name

        query_params = {
            'subject_id': ids,
            'subject_type': ObjectTypeEnum.instance,
            'status': True,
        }
        rows = self.get_tasks_rows(self.request, query_params)
        return {
            'rows': rows,
            'object_names_dict': names,
            'ids': ids,
            'msg': self.msg,
            'confirm_url': self.confirm_url,
            'redirect_url': self.get_redirect_url(),
        }


class TnxDeleteInstanceView(DeleteInstanceView):
    """ Delete tnx instances view """
    form_class = TnxDeleteInstanceForm

    def get_initial(self):
        # FIXME many requests by id
        ids = self.get_object_ids()
        names = {}

        # if POST method names are already on the form.
        if self.request.method != 'POST':
            table = self.kwargs.get('table', None)

            instances = {}
            if table:
                instances = {datum.id: datum for datum in table.data}

            for instance_id in ids:
                project_id = None
                if TABLE_OBJECT_ID_SEP in instance_id:
                    project_id, instance_id = instance_id.split(
                        TABLE_OBJECT_ID_SEP
                    )
                if api.keystone.is_domain_admin(self.request):
                    project_id = None
                instance = instances.get(instance_id)
                if not instance:
                    try:
                        instance = TNXServersClient.get(self.request,
                                                        instance_id,
                                                        project_id=project_id)
                    except Exception as err:
                        logger.error(unicode(err))
                        redirect = self.get_redirect_url()
                        exceptions.handle(self.request,
                                          _('Unable to retrieve details for '
                                            'instance "%s".') % instance_id,
                                          redirect=redirect)
                        # Not all exception types handled above will result in
                        # a redirect. Need to raise here just in case.
                        raise exceptions.Http302(redirect)

                names[instance_id] = instance.name

        query_params = {
            'subject_id': ids,
            'subject_type': ObjectTypeEnum.instance,
            'status': True,
        }
        rows = self.get_tasks_rows(self.request, query_params)
        return {
            'rows': rows,
            'object_names_dict': names,
            'ids': ids,
            'msg': self.msg,
            'confirm_url': self.confirm_url,
            'redirect_url': self.get_redirect_url(),
        }


class ResetStateView(LinkBatchFormView):
    """Reset state view"""
    form_class = ResetStateForm
    modal_header = _('Reset state')
    template_name = 'project_instances/reset_state.html'
    modal_id = "reset_state_id"
    submit_label = _('Reset state')
    context_object_name = 'instance'
    redirect_url = success_url = reverse_lazy(PROJECT_SERVERS_INDEX)
    confirm_url = reverse_lazy("horizon:project:instances:reset_state")
    hidden_title = False
    failure_msg = _('"Reset state" is not available for instances: "{0}"')
    states = (ERROR.upper(),)

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_objects(self, *args, **kwargs):
        instances, failed = [], []
        for instance_id in self.get_object_ids():
            try:
                instance = TNXServersClient.get(self.request, instance_id)
            except Exception as err:
                logger.error(err.message)
                continue

            if instance.status not in self.states:
                failed.append(instance.name)
            else:
                instances.append(instance)

        if failed:
            # pylint: disable=no-member
            failed_message = self.failure_msg.format(u'", "'.join(failed))
            messages.error(self.request, failed_message)

        if not instances:
            raise exceptions.Http302(self.redirect_url)

        return instances

    def get_initial(self):
        instances = self.get_objects()

        instances_names = [instance.name for instance in instances]

        return {'instances': instances,
                'instances_names': instances_names,
                'confirm_url': self.confirm_url,
                'redirect_url': self.redirect_url}


class LaunchBaseView(forms.ModalFormView):
    """Launch base instance view"""
    form_class = LaunchBaseInstanceForm
    modal_header = _('Launch Base Instance')
    template_name = 'project_instances/launch_base.html'
    modal_id = "launch_base_id"
    submit_label = _('Launch')
    context_object_name = 'instance'
    success_url = reverse_lazy(PROJECT_SERVERS_INDEX)


@rest_urls.register
class TnxServer(nova.Server):
    """Extended server api.
    """
    url_regex = r'tnx/servers/(?P<server_id>[^/]+|default)$'

    @rest_utils.ajax()
    def get(self, request, server_id):
        server = TNXServersClient.get(request, server_id).to_dict()

        networks, ports = get_instance_networks_and_ports(request, server_id)

        server.update(
            tnx_ports=ports,
            tnx_networks=networks,
            tnx_security_groups=get_instance_security_groups(request,
                                                             server_id)
        )
        return server


@rest_urls.register
class TnxServers(nova.Servers):
    """Extended servers api.
    """

    url_regex = r'tnx/servers/$'

    _optional_create = [
        'block_device_mapping', 'block_device_mapping_v2', 'nics', 'meta',
        'availability_zone', 'instance_count', 'admin_pass', 'disk_config',
        'config_drive', 'key_name', 'security_groups', 'description',
        'volumes_to_attach'
    ]

    @rest_utils.ajax()
    def get(self, request):
        """Get a list of servers.

        The listing result is an object with property "items". Each item is
        a server.

        Example GET:
        http://localhost/api/nova/servers
        """
        result_servers = []
        vm_statuses = dict(VM_STATUS_DISPLAY_CHOICES)

        filter_statuses = [
            status.lower() for status in request.GET.getlist('status', [])
        ]

        try:
            servers = TNXServersClient.list(request)
        except Exception as err:
            logger.error(unicode(err))
            raise rest_utils.AjaxError(400,
                                       _('Enable to retrieve instance list.'))
        for server in servers:
            status = (server.status or '').lower()
            if filter_statuses and status not in filter_statuses:
                continue
            s = server.to_dict()
            s['status_name'] = unicode(vm_statuses.get(status))
            s['ip_addresses'] = get_ips(server)
            s['tnx_type_name'] = unicode(
                VMTypes.TYPES.get(str(s.get('tnx_type', '')))
            )
            s['availability_zone'] = server.availability_zone
            s['is_direct_scsi'] = server.is_direct_scsi
            result_servers.append(s)
        return {'items': result_servers}

    @rest_utils.ajax(data_required=True)
    def post(self, request):
        """
        Create a server.
        :param request: request
        :return: new server object on success.
        """
        try:
            args = (
                request,
                request.DATA.get('name') or gen_uuid_object_name(),
                request.DATA['source_id'],
                request.DATA['flavor_id'],
                request.DATA['user_data'],
            )
        except KeyError as e:
            raise rest_utils.AjaxError(400, 'missing required parameter '
                                            "'%s'" % e.args[0])
        kw = {name: request.DATA[name] for name in self._optional_create
              if name in request.DATA}

        kw['min_count'] = kw.pop('instance_count')
        volumes_to_attach = kw.pop('volumes_to_attach', None)
        new = TNXServersClient.create(*args, **kw)

        if volumes_to_attach:
            for volume_id in volumes_to_attach:
                thread.start_new_thread(
                    delayed_volume_attach,
                    (request, new.id, volume_id)
                )
        return rest_utils.CreatedResponse(
            '/api/tnx/servers/%s' % utils_http.urlquote(new.id),
            new.to_dict()
        )


# noinspection PyClassHasNoInit
class CloneInstanceView(project_views.LaunchInstanceView):

    workflow_class = CloneInstance

    def get_initial(self):
        initial = super(CloneInstanceView, self).get_initial()

        instance_id = self.kwargs['instance_id']

        try:
            instance = TNXServersClient.get(self.request, instance_id)
        except Exception as err:
            logger.error(err)
        else:

            networks, _ports = get_instance_networks_and_ports(self.request,
                                                               instance_id)

            initial.update(
                name=u'Clone of {0}'.format(instance.name),
                availability_zone=instance.availability_zone,
                description=instance.description,
                disk_config=getattr(instance, 'OS-DCF:diskConfig', None),
                flavor=instance.flavor['id'],
                keypair=instance.key_name,
                groups=get_instance_security_groups(self.request, instance_id),
                network=networks
            )

            if instance.image:
                image_id = instance.image['id']
                source_type = "image_id"
                try:
                    image = api.glance.image_get(self.request, image_id)
                except Exception as err:
                    logger.error(err)
                else:
                    if get_image_type(image) != "image":
                        source_type = "instance_snapshot_id"

                initial.update(source_id=instance.image['id'],
                               source_type=source_type)

        return initial


# noinspection PyClassHasNoInit
class AttachInterfaceView(project_views.AttachInterfaceView):
    form_class = AttachInterface


# noinspection PyClassHasNoInit
class DetachInterfaceView(project_views.DetachInterfaceView):
    form_class = DetachInterface


# noinspection PyClassHasNoInit
class LaunchInstanceView(project_views.LaunchInstanceView):
    workflow_class = LaunchInstance


class UpdateView(RefererRedirectMixin,
                 LinkBatchWorkflowViewMixin,
                 WorkflowVerticalView):
    workflow_class = UpdateInstance
    success_url = reverse_lazy(PROJECT_SERVERS_INDEX)
    submit_url = reverse_lazy('horizon:project:instances:update')

    @staticmethod
    def modal_header(count):
        return npgettext_lazy(
            "Action to perform edit action",
            u"Edit Instance",
            u"Edit Instances",
            count
        )

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_object(self, *args, **kwargs):
        success_instances, failure_instances = [], []
        for instance_id in self.get_object_ids():
            try:
                success_instances.append(
                    novaclient(self.request).servers.get(instance_id)
                )
            except Exception as err:
                logger.error(unicode(err))
                failure_instances.append(instance_id)

        if failure_instances:
            messages.error(self.request,
                           ungettext_lazy(
                               u'Unable to retrieve instance: {0}',
                               u'Unable to retrieve instances: {0}',
                               len(failure_instances)
                           ).format(u'", "'.join(failure_instances)))

        return success_instances

    def get_initial(self):
        initial = super(UpdateView, self).get_initial()
        instances = self.get_object()
        if instances:
            project_ids = set(instance.tenant_id for instance in instances)
            initial.update({
                'instances': instances,
                'project_id': (
                    project_ids.pop() if len(project_ids) == 1 else None
                )
            })
            if len(instances) == 1:
                instance = instances[0]
                initial.update({'name': instance.name,
                                'description': instance.description})
        return initial


class ManageVolumeAttachmentsView(forms.ModalFormView):
    form_class = AttachForm
    form_id = "attach_volume"
    modal_header = _("Manage Volume Attachments")
    modal_id = "attach_volume_modal"
    template_name = 'project_instances/attach.html'
    submit_url = "horizon:project:instances:attach_volume"
    success_url = reverse_lazy("horizon:project:instances:index")
    page_title = _("Manage Volume Attachments")

    def get(self, request, *args, **kwargs):
        for volume in api.cinder.volume_list(
                request, search_opts={"status": "available"}):
            if not volume.attachments:
                break
        else:
            messages.warning(request, _("There are no volumes to attach to"
                                        " the instance in the project."))
            return redirect_to(self.get_cancel_url())
        return super(ManageVolumeAttachmentsView, self).get(request, *args,
                                                            **kwargs)

    @memoized.memoized_method
    def get_object(self):
        instance_id, instance = self.kwargs['instance_id'], None

        try:
            instance = TNXServersClient.get(self.request, instance_id)
            instance.attachments = api.nova.instance_volumes_list(
                self.request, instance_id
            )
        except Exception as err:
            logger.error(err)
            exceptions.handle(self.request,
                              _('Unable to retrieve instance information.'))

        return instance

    def get_initial(self):
        instance = self.get_object()
        return {'instance': instance,
                'instance_id': instance.id,
                'old_allocated': json.dumps(
                    [attach.id for attach in instance.attachments])}

    def get_context_data(self, **kwargs):
        context = super(ManageVolumeAttachmentsView, self).get_context_data(
            **kwargs
        )
        instance = self.get_object()
        context['instance'] = instance
        if self.request.is_ajax():
            context['hide'] = True
        return context
