# coding: utf-8

"""Project instance workflows"""
import logging

from collections import OrderedDict

from django.utils.text import normalize_newlines
from django.utils.translation import ugettext_lazy as _, ungettext_lazy
from django.views.decorators.debug import sensitive_variables

from horizon import exceptions, forms, workflows, messages

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.instances.workflows import (
    create_instance, update_instance)
from openstack_dashboard.utils import filters
from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.security_groups import (
    get_security_groups, get_security_groups_by_instance,
    server_update_security_groups)

from tionix_dashboard.helpers.fields import AutoGenNameCharField
from tionix_dashboard.helpers.views import RefererRedirectMixin
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    log_instance_group_added
)

logger = logging.getLogger(__name__)

COMMON_HORIZONTAL_TEMPLATE = "horizon/common/_common_horizontal_form.html"


def _cleanup_ports_on_failed_vm_launch(request, nics):
    ports_failing_deletes = []
    logger.debug(u'Cleaning up stale VM ports.')
    for nic in nics:
        try:
            logger.debug(u'Deleting port with id: {0}'.format(nic['port-id']))
            api.neutron.port_delete(request, nic['port-id'])
        except Exception as err:
            logger.error(unicode(err))
            ports_failing_deletes.append(nic['port-id'])
    return ports_failing_deletes


class SetInstanceDetailsAction(create_instance.SetInstanceDetailsAction):
    name = AutoGenNameCharField(label=_("Instance Name"), max_length=255)

    description = forms.CharField(
        label=_("Description"),
        max_length=255,
        widget=forms.Textarea(attrs={'rows': 4}),
        required=False,
        help_text=_("Edit the instance description.")
    )

    def __init__(self, request, context, *args, **kwargs):
        super(SetInstanceDetailsAction, self).__init__(
            request, context, *args, **kwargs
        )

        ordering = ["availability_zone", "name", "description", "flavor",
                    "count", "source_type", "instance_snapshot_id",
                    "volume_id", "volume_snapshot_id", "image_id",
                    "volume_size", "device_name",
                    "vol_delete_on_instance_delete"]

        self.fields = OrderedDict(
            (key, self.fields[key]) for key in ordering
        )

    class Meta(object):
        name = _("Details")
        help_text_template = "project/instances/_launch_details_help.html"


class SetInstanceDetails(create_instance.SetInstanceDetails):
    action_class = SetInstanceDetailsAction
    contributes = ("source_type", "source_id",
                   "availability_zone", "name", "count", "flavor",
                   "device_name",  # Can be None for an image.
                   "vol_delete_on_instance_delete", "description")


class LaunchInstance(create_instance.LaunchInstance):
    default_steps = (create_instance.SelectProjectUser,
                     SetInstanceDetails,
                     create_instance.SetAccessControls,
                     create_instance.SetNetwork,
                     create_instance.SetNetworkPorts,
                     create_instance.PostCreationStep,
                     create_instance.SetAdvanced)

    @sensitive_variables('context')
    def handle(self, request, context):
        dev_mapping_1 = dev_mapping_2 = None
        image_id = ''
        result = True

        # Determine volume mapping options
        source_type = context.get('source_type', None)
        if source_type in ['image_id', 'instance_snapshot_id']:
            image_id = context['source_id']
        elif source_type in ['volume_id', 'volume_snapshot_id']:
            try:
                if api.nova.extension_supported("BlockDeviceMappingV2Boot",
                                                request):
                    # Volume source id is extracted from the source
                    volume_source_id = context['source_id'].split(':')[0]
                    device_name = (
                        context.get('device_name', '').strip() or None
                    )
                    dev_source_type_mapping = {
                        'volume_id': 'volume',
                        'volume_snapshot_id': 'snapshot'
                    }
                    dev_mapping_2 = [
                        {'device_name': device_name,
                         'source_type': dev_source_type_mapping[source_type],
                         'destination_type': 'volume',
                         'delete_on_termination':
                             bool(context['vol_delete_on_instance_delete']),
                         'uuid': volume_source_id,
                         'boot_index': '0',
                         'volume_size': context['volume_size']}
                    ]
                else:
                    dev_mapping_1 = {
                        context['device_name']: '%s::%s' % (
                            context['source_id'],
                            bool(context['vol_delete_on_instance_delete']))
                    }
            except Exception as err:
                logger.error(unicode(err))
                msg = _('Unable to retrieve extensions information')
                exceptions.handle(request, msg)

        elif source_type == 'volume_image_id':
            device_name = context.get('device_name', '').strip() or None
            dev_mapping_2 = [
                {'device_name': device_name,  # None auto-selects device
                 'source_type': 'image',
                 'destination_type': 'volume',
                 'delete_on_termination':
                     bool(context['vol_delete_on_instance_delete']),
                 'uuid': context['source_id'],
                 'boot_index': '0',
                 'volume_size': context['volume_size']}
            ]

        netids = context.get('network_id', None)
        if netids:
            nics = [{"net-id": netid, "v4-fixed-ip": ""}
                    for netid in netids]
        else:
            nics = None

        avail_zone = context.get('availability_zone', None)

        port_profiles_supported = api.neutron.is_port_profiles_supported()

        if port_profiles_supported:
            nics = self.set_network_port_profiles(request,
                                                  context['network_id'],
                                                  context['profile_id'])

        ports = context.get('ports')
        if ports:
            if nics is None:
                nics = []
            nics.extend([{'port-id': port} for port in ports])

        try:
            TNXServersClient.create(
                request,
                context['name'],
                image_id,
                context['flavor'],
                description=context['description'],
                key_name=context['keypair_id'],
                user_data=normalize_newlines(
                    context.get('script_data', '')
                ),
                security_groups=context['security_group_ids'],
                block_device_mapping=dev_mapping_1,
                block_device_mapping_v2=dev_mapping_2,
                nics=nics,
                availability_zone=avail_zone,
                min_count=int(context['count']),
                admin_pass=context['admin_pass'],
                disk_config=context.get('disk_config'),
                config_drive=context.get('config_drive'))
        except Exception as err:
            logger.error(unicode(err))
            result = False
            if port_profiles_supported:
                ports_failing_deletes = _cleanup_ports_on_failed_vm_launch(
                    request, nics)
                if ports_failing_deletes:
                    ports_str = ', '.join(ports_failing_deletes)
                    msg = (_('Port cleanup failed for these port-ids (%s).')
                           % ports_str)
                    exceptions.handle(request, msg)
            exceptions.handle(request)
        return result


class CloneInstance(RefererRedirectMixin, LaunchInstance):
    slug = "clone_instance"
    name = _("Clone Instance")
    finalize_button_name = _("Clone")
    success_message = _('Cloned %(count)s named "%(name)s".')
    failure_message = _('Unable to clone %(count)s named "%(name)s".')
    success_url = "horizon:project:instances:index"

    def __init__(self, *args, **kwargs):
        super(CloneInstance, self).__init__(*args, **kwargs)

        for step in self.steps:
            for field_name, field in step.action.fields.items():
                value = kwargs.get('context_seed', {}).get(field_name)
                if value:
                    field.initial = value


class UpdateInstanceInfoAction(workflows.Action):
    name = AutoGenNameCharField(label=_("Name"), max_length=255)
    description = forms.CharField(
        label=_("Description"),
        max_length=255,
        widget=forms.Textarea(attrs={'rows': 4}),
        required=False,
        help_text=_("Edit the instance description.")
    )

    def __init__(self, request, context, *args, **kwargs):
        super(UpdateInstanceInfoAction, self).__init__(
            request, context, *args, **kwargs
        )
        instances_count = len(context.get('instances', []))
        if instances_count > 1:
            self.fields['name'] = forms.CharField(label=_("Name"),
                                                  max_length=255,
                                                  required=False)

    class Meta(object):
        name = _("Information")
        slug = 'instance_info'
        help_text = _("Edit the instance details.")


class UpdateInstanceInfo(update_instance.UpdateInstanceInfo):
    action_class = UpdateInstanceInfoAction
    contributes = ("name", "description")
    template_name = COMMON_HORIZONTAL_TEMPLATE
    depends_on = ("instances", "project_id")


class UpdateInstanceSecurityGroupsAction(workflows.MembershipAction):
    def __init__(self, request, context, *args, **kwargs):
        super(UpdateInstanceSecurityGroupsAction, self).__init__(request,
                                                                 context,
                                                                 *args,
                                                                 **kwargs)
        err_msg = _('Unable to retrieve security group list. '
                    'Please try again later.')
        instances = context.get('instances', [])
        project_id = context.get('project_id')

        default_role_name = self.get_default_role_field_name()
        self.fields[default_role_name] = forms.CharField(required=False)
        self.fields[default_role_name].initial = 'member'

        # Get list of available security groups
        all_groups = []
        try:
            all_groups = get_security_groups(request, project_id=project_id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, err_msg)
        groups_list = [(group.id, group.name) for group in all_groups]

        instance_groups = []
        if len(instances) == 1:
            try:
                instance_groups = get_security_groups_by_instance(
                    request, instances[0].id, project_id=project_id)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(request, err_msg)
        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)
        self.fields[field_name].choices = groups_list
        self.fields[field_name].initial = [group.id
                                           for group in instance_groups]

    class Meta(object):
        name = _("Security Groups")
        slug = update_instance.INSTANCE_SEC_GROUP_SLUG


class UpdateInstanceSecurityGroups(
        update_instance.UpdateInstanceSecurityGroups):
    action_class = UpdateInstanceSecurityGroupsAction
    depends_on = ("instances", "project_id")
    contributes = ()


class UpdateInstance(RefererRedirectMixin,
                     update_instance.UpdateInstance):
    template_name = "horizon/common/_workflow_vertical.html"
    default_steps = (UpdateInstanceInfo,
                     UpdateInstanceSecurityGroups)

    def get_absolute_url(self):
        return self.request.get_full_path()

    def format_status_message(self, message):
        return message

    @staticmethod
    def _update_instance(request, instance_id, data, project_id=None):
        instance = None
        kwargs = {}
        if not api.keystone.is_domain_admin(request):
            kwargs.update({'project_id': project_id})
        try:
            instance = novaclient(request,
                                  **kwargs).servers.update(instance_id, **data)
        except Exception as err:
            logger.error(unicode(err))
        return instance

    @staticmethod
    def _single_update_security_groups(request, instance_id, project_id,
                                       security_groups):
        result = True
        try:
            server_update_security_groups(request,
                                          instance_id,
                                          security_groups,
                                          project_id)
            log_instance_group_added(instance_id, security_groups, result)
        except Exception as err:
            err_text = unicode(err)
            logger.error(err_text)
            result = False
            log_instance_group_added(
                instance_id, security_groups, result, res_msg=err_text)
        return result

    @staticmethod
    def _multi_update_security_groups(request, instance_id, project_id,
                                      security_groups):
        result = True
        if security_groups:
            try:
                nova_client = novaclient(request, project_id=project_id)
                for security_group in security_groups:
                    nova_client.servers.add_security_group(instance_id,
                                                           security_group)
                log_instance_group_added(instance_id, security_groups, result)
            except Exception as err:
                result = False
                err_text = unicode(err)
                logger.error(err_text)
                log_instance_group_added(
                    instance_id, security_groups, result, res_msg=err_text)
        return result

    def handle(self, request, context):
        result = True
        success_instances, failure_instances, failure_sec_groups = [], [], []
        instances = context.pop('instances', [])
        context.pop('project_id', None)
        wanted_groups = [filters.get_int_or_uuid(group)
                         for group in context.pop('wanted_groups', [])]
        data = {key: value for key, value in context.items() if value}
        update_security_group_method = (
            self._multi_update_security_groups
            if len(instances) > 1
            else self._single_update_security_groups
        )
        for instance in instances:
            project_id = instance.tenant_id
            if data:
                updated_instance = self._update_instance(self.request,
                                                         instance.id,
                                                         data,
                                                         project_id=project_id)
                if updated_instance is None:
                    failure_instances.append(instance.name)
                else:
                    success_instances.append(updated_instance.name)
            else:
                success_instances.append(instance.name)
            if not update_security_group_method(
                    self.request, instance.id, project_id, wanted_groups):
                failure_sec_groups.append(instance.name)
        if failure_sec_groups:
            messages.info(
                request,
                ungettext_lazy(
                    u"Unable to update security groups of instance: {0}",
                    u"Unable to update security groups of instances: {0}",
                    len(failure_sec_groups)
                ).format(u'", "'.join(failure_sec_groups)))

        if failure_instances:
            result = False
            self.failure_message = ungettext_lazy(
                u'Unable to modify instance: "{0}".',
                u'Unable to modify instances: "{0}".',
                len(failure_instances)
            ).format(u'", "'.join(failure_instances))

        if success_instances:
            self.success_message = ungettext_lazy(
                u'Modified instance: "{0}".',
                u'Modified instances: "{0}".',
                len(failure_instances)
            ).format(u'", "'.join(success_instances))

        return result
