# coding: utf-8

"""Tnx project volumes helpers"""
import logging

import time

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon.utils import functions as utils

from openstack_dashboard.api import nova, cinder
from openstack_dashboard.api.cinder import (
    cinderclient, transfer_list, VERSIONS, Volume,
    VolumeSnapshot, VolumeBackup, update_pagination)

from tionix_dashboard.settings import VOLUME_ATTACH_MAX_RETRIES

logger = logging.getLogger(__name__)


def get_cinder_backups(request, search_opts=None):
    """Retrieve backups by volume"""
    backups = []
    cinder_client = cinderclient(request)
    if cinder_client:
        try:
            backups = cinder_client.backups.list(search_opts=search_opts)
        except Exception as err:
            logger.error(err)
            exceptions.handle(request,
                              _("Unable to retrieve volume backups."))
    return backups


def get_object_list_paged(request, object_manager, object_wrapper,
                          search_opts=None, marker=None, paginate=False,
                          sort_dir="desc"):
    page_size = utils.get_page_size(request)
    has_more_data = False
    has_prev_data = False
    volumes = []

    c_client = cinderclient(request)
    manager = getattr(c_client, object_manager, None)
    if c_client is None or manager is None:
        return volumes, has_more_data, has_prev_data

    # build a dictionary of volume_id -> transfer
    transfers = {t.volume_id: t
                 for t in transfer_list(request, search_opts=search_opts)}

    kwargs = {'search_opts': search_opts}

    if VERSIONS.active > 1:
        # sort_key and sort_dir deprecated in kilo, use sort
        # if pagination is true, we use a single sort parameter
        # by default, it is "created_at"
        sort = 'created_at:' + sort_dir
        kwargs.update({'marker': marker, 'sort': sort})
        if paginate:
            kwargs.update({'limit': page_size + 1})

    for v in manager.list(**kwargs):
        v.transfer = transfers.get(v.id)
        volumes.append(object_wrapper(v))

    if paginate:
        volumes, has_more_data, has_prev_data = update_pagination(
            volumes, page_size, marker, sort_dir)

    return volumes, has_more_data, has_prev_data


def volume_list_paged(request, search_opts=None, marker=None,
                      paginate=False, sort_dir="desc"):
    """To see all volumes in the cloud as an admin you can pass in a special
    search option: {'all_tenants': 1}
    """
    return get_object_list_paged(request, 'volumes', Volume, search_opts,
                                 marker, paginate, sort_dir)


def volume_snapshot_list_paged(request, search_opts=None, marker=None,
                               paginate=False, sort_dir="desc"):
    return get_object_list_paged(request, 'volume_snapshots', VolumeSnapshot,
                                 search_opts, marker, paginate, sort_dir,)


def volume_backup_list_paged(request, search_opts=None, marker=None,
                             paginate=False, sort_dir="desc"):
    return get_object_list_paged(request, 'backups', VolumeBackup, search_opts,
                                 marker, paginate, sort_dir)


def delayed_volume_attach(request, volume_id, instance_id, device, delay=5):
    retries = 0
    while retries < VOLUME_ATTACH_MAX_RETRIES:
        time.sleep(delay)
        try:
            volume = cinder.volume_get(request, volume_id)
            if volume.status == "available":
                nova.instance_volume_attach(
                    request, volume.id, instance_id, device
                )
                break
        except Exception as err:
            logger.error(unicode(err))
        retries += 1
