# coding: utf-8

import logging

from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.translation import (
    ugettext_lazy as _, string_concat, ungettext_lazy
)

from horizon import exceptions
from horizon import messages
from horizon import tables

from openstack_dashboard import api
from openstack_dashboard.api import cinder
from openstack_dashboard.dashboards.project.volumes.volumes import (
    tables as volumes_tables)

from tionix_dashboard.clients import TNXVolumesClient
from tionix_dashboard.dashboards.project.project_volumes.actions import (
    BaseVolumeFilterAction)
from tionix_dashboard.helpers.actions import (
    LinkBatchAction, TnxUpdateAction, TnxBatchActionMixin)
from tionix_dashboard.helpers.utils import generate_timestamp_name

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class TnxCreateSnapshot(volumes_tables.VolumePolicyTargetMixin,
                        LinkBatchAction):
    name = 'snapshots'
    multiple_next_view_name = 'horizon:project:volumes:volumes:create_snapshot'
    policy_rules = (('volume', 'volume:create_snapshot'),)

    def action_present(self, count):
        return ungettext_lazy(
            u"Create Snapshot",
            u"Create Snapshots",
            count
        )

    def action_past(self, count):
        return ungettext_lazy(
            u"Created Snapshot",
            u"Created Snapshots",
            count
        )

    def allowed(self, request, datum):
        try:
            limits = cinder.tenant_absolute_limits(request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve tenant limits.'))
            limits = {}

        snapshots_available = (
            limits.get('maxTotalSnapshots', float('inf')) -
            limits.get('totalSnapshotsUsed', 0)
        )

        if snapshots_available <= 0 and 'disabled' not in self.classes:
            self.classes = [c for c in self.classes] + ['disabled']
            self.verbose_name = string_concat(self.verbose_name, ' ',
                                              _('(Quota exceeded)'))
        return super(TnxCreateSnapshot, self).allowed(request, datum)


# noinspection PyAbstractClass
class TnxCreateBackup(volumes_tables.VolumePolicyTargetMixin,
                      TnxBatchActionMixin, tables.BatchAction):
    name = "backup"
    classes = ('btn-danger',)
    policy_rules = (("volume", "backup:create"),)
    success_url = reverse_lazy("horizon:project:volumes:backups_tab")

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Create Volume Backup",
            u"Create Volumes Backups",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Created Volume Backup",
            u"Created Volumes Backups",
            count
        )

    def allowed(self, request, datum=None):
        return (cinder.volume_backup_supported(request) and (
            datum is None or datum.status == "available"))

    def action(self, request, datum_id):

        try:
            volume = api.cinder.volume_get(request, datum_id)
        except Exception as err:
            logger.error(unicode(err))
        else:
            name = generate_timestamp_name(volume.name)
            api.cinder.volume_backup_create(request, datum_id, None, name,
                                            None)


class VolumeUpdateCell(TnxUpdateAction):
    """
    Action update volume cell
    """

    def update_cell(self, request, datum, obj_id, cell_name,
                    new_cell_value):
        """
        Update cell
        """
        result = True
        try:
            TNXVolumesClient.update(
                request, obj_id, {cell_name: new_cell_value})
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to update volume.'))
            result = False
        return result


class VolumeSetBootableAction(tables.UpdateAction):
    """ Action set volume as bootable. """

    def allowed(self, request, datum, cell):
        """
        check permission
        """
        return api.keystone.keystone_can_edit_project()

    def update_cell(self, request, datum, obj_id, cell_name,
                    new_cell_value):
        """ Inline update of bootable-cell. """
        if cell_name == 'bootable' and datum.is_bootable != new_cell_value:
            try:
                cinder.volume_set_bootable(request, obj_id,
                                           new_cell_value)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(
                    request,
                    _('Unable to set bootable flag on volume.'),
                    redirect=reverse("horizon:project:volumes:index")
                )


# noinspection PyAbstractClass
class TnxDeleteVolume(volumes_tables.VolumePolicyTargetMixin,
                      LinkBatchAction):
    """ Group action to delete instances """
    name = 'delete'
    classes = ("btn-danger",)
    multiple_next_view_name = "horizon:project:volumes:volumes:delete"
    policy_rules = (("volume", "volume:delete"),)

    # pylint: disable=method-hidden
    def action_present(self, count):
        return (
            ungettext_lazy(
                u"Delete Volume",
                u"Delete Volumes",
                count
            )
        )

    # pylint: disable=method-hidden
    def action_past(self, count):
        return (
            ungettext_lazy(
                u"Deleted Volume",
                u"Deleted Volumes",
                count
            )
        )

    # noinspection PyMethodMayBeStatic
    def delete(self, request, obj_id):
        cinder.volume_delete(request, obj_id)

    def allowed(self, request, datum=None):
        if datum:
            return (datum.status in volumes_tables.DELETABLE_STATES and
                    not getattr(datum, 'has_snapshot', False))
        return True


# noinspection PyAbstractClass
class VolumeFilterAction(BaseVolumeFilterAction):
    """Volume snapshots filter"""
    filter_type = "server"
    filter_choices = (BaseVolumeFilterAction.filter_choices +
                      (('volume_tape', _("Type"), False),
                       ('attachments', _("Attached To"), False),
                       ('availability_zone', _("Availability Zone"), False),
                       ('bootable', _("Bootable"), False),
                       ('encryption', _("Encrypted"), False)))
    escape_enabled = False
