# coding: utf-8

import logging

import thread
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import forms
from horizon import messages
from horizon import exceptions

from openstack_dashboard.api import cinder, nova
from openstack_dashboard.dashboards.project.volumes.volumes import (
    forms as volumes_forms)

from tionix_scheduler_client.enums import ObjectTypeEnum

from tionix_dashboard.dashboards.project.project_instances.forms import (
    AttachVolumeBaseForm)
from tionix_dashboard.dashboards.project.project_volumes.helpers import (
    delayed_volume_attach)
from tionix_dashboard.dashboards.tionix.scheduler.forms import (
    DeleteObjectWithTasksForm)
from tionix_dashboard.helpers.fields import AutoGenNameCharField

logger = logging.getLogger(__name__)


# noinspection PyClassHasNoInit,PyAbstractClass
class TnxCreateSnapshotForm(forms.SelfHandlingForm):
    name = AutoGenNameCharField(max_length=255, label=_("Snapshot Name"))
    description = forms.CharField(max_length=255,
                                  widget=forms.Textarea(
                                      attrs={'rows': 4}),
                                  label=_("Description"),
                                  required=False)

    def handle(self, request, data):
        failed, attached, success, volume = [], [], [], None
        for _id in self.initial.get('volumes_ids', []):
            try:
                volume = cinder.volume_get(request, _id)
                force = False
                if volume.status == 'in-use':
                    force = True
                    attached.append(volume.name)
                cinder.volume_snapshot_create(request,
                                              _id,
                                              data['name'],
                                              data['description'],
                                              force=force)
                success.append(volume.name)
            except Exception as err:
                logger.error(unicode(err))
                failed.append(volume.name if volume else _id)
                # pylint: disable=no-member
                if err.code == 413:
                    redirect = reverse(
                        "horizon:project:volumes:index")
                    msg = _(
                        'Requested snapshot would exceed the allowed quota.')
                    exceptions.handle(request,
                                      msg,
                                      redirect=redirect)
        if failed:
            msg = (_('Unable to create volume snapshot for %s.') %
                   ', '.join(failed))
            messages.error(request, msg)

        if attached:
            msg = _('Forcing to create snapshot '
                    'from attached volumes %s.') % ', '.join(attached)
            messages.info(request, msg)

        if success:
            msg = _('Creating volume snapshot for %s.') % ', '.join(
                success)
            messages.success(request, msg)

        return True


# noinspection PyClassHasNoInit,PyAbstractClass
class DeleteVolumeForm(DeleteObjectWithTasksForm):
    """ Delete volumes form """

    object_type = ObjectTypeEnum.volume

    def delete(self, request, object_id):
        cinder.volume_delete(request, object_id)


# noinspection PyClassHasNoInit
class CreateVolumeForm(volumes_forms.CreateForm):
    name = AutoGenNameCharField(max_length=255, label=_("Volume Name"),
                                required=False)
    thin_provisioning = forms.BooleanField(
        required=False,
        label=_('Thin provisioning'),
        initial=True
    )


# noinspection PyClassHasNoInit,PyAbstractClass
class CreateTransferForm(volumes_forms.CreateTransferForm):
    name = AutoGenNameCharField(max_length=255, label=_("Transfer Name"),
                                required=False)


class AttachForm(AttachVolumeBaseForm):

    def handle(self, request, data):
        result, message = True, None
        instance_id = data.get('instance_id')

        volume = self.initial['volume']
        # api requires non-empty device name or None
        device = data.get('device') or None
        try:
            if volume.attachments:
                attachment = volume.attachments[0]
                if instance_id != attachment['server_id']:
                    nova.instance_volume_detach(request,
                                                attachment['server_id'],
                                                attachment['id'])
                    if instance_id:
                        thread.start_new_thread(
                            delayed_volume_attach,
                            (request, volume.id, instance_id, device)
                        )
                        message = _('Started reattaching the volume.')
                    else:
                        message = _('Started detaching the volume.')
            elif instance_id:
                attach = nova.instance_volume_attach(
                    request, volume.id, instance_id, device
                )
                instance = nova.server_get(request, instance_id)
                message = _('Attaching volume %(vol)s to instance '
                            '%(inst)s on %(dev)s.') % {"vol": volume.name,
                                                       "inst": instance.name,
                                                       "dev": attach.device}
            if message:
                messages.success(request, message)
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to edit attachments.'))

        return result
