# coding: utf-8

from django.forms.fields import BooleanField
from django.template import defaultfilters as filters
from django.utils.translation import pgettext_lazy, ugettext_lazy as _

from horizon import tables

from openstack_dashboard.dashboards.project.volumes.volumes import (
    tables as volume_tables)

from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ProjectVolumeSchedulerAction)
from tionix_dashboard.dashboards.project.project_volumes.volumes import (
    actions as volumes_actions
)
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import TnxLocalPaginationTabDataTable
from tionix_dashboard.helpers.volume_constants import DELETING_STATUS


class FilterActionVolumesMixin(object):
    """Filtering actions by status"""

    def _filter_action(self, action, request, datum=None):
        """Filters actions"""
        result = True
        if datum:
            result = datum.status.lower() != DELETING_STATUS
        # noinspection PyProtectedMember
        return (result and
                super(FilterActionVolumesMixin, self)._filter_action(action,
                                                                     request,
                                                                     datum))


class TnxVolumesTableBase(FilterActionVolumesMixin,
                          TnxLocalPaginationTabDataTable,
                          volume_tables.VolumesTable):
    STATUS_DISPLAY_CHOICES = (
        ("available", pgettext_lazy("Current status of a Volume",
                                    u"Available")),
        ("in-use", pgettext_lazy("Current status of a Volume", u"In-use")),
        ("error", pgettext_lazy("Current status of a Volume", u"Error")),
        ("creating", pgettext_lazy("Current status of a Volume",
                                   u"Creating")),
        ("error_extending", pgettext_lazy("Current status of a Volume",
                                          u"Error Extending")),
        ("extending", pgettext_lazy("Current status of a Volume",
                                    u"Extending")),
        ("attaching", pgettext_lazy("Current status of a Volume",
                                    u"Attaching")),
        ("detaching", pgettext_lazy("Current status of a Volume",
                                    u"Detaching")),
        ("deleting", pgettext_lazy("Current status of a Volume",
                                   u"Deleting")),
        ("error_deleting", pgettext_lazy("Current status of a Volume",
                                         u"Error Deleting")),
        ("backing-up", pgettext_lazy("Current status of a Volume",
                                     u"Backing Up")),
        ("restoring-backup", pgettext_lazy("Current status of a Volume",
                                           u"Restoring Backup")),
        ("error_restoring", pgettext_lazy("Current status of a Volume",
                                          u"Error Restoring")),
        ("error_backing-up", pgettext_lazy("Current status of a Volume",
                                           u"Error Backing-up")),
        ("error_restoring", pgettext_lazy("Current status of a Volume",
                                          u"Error Restoring")),
        ("maintenance", pgettext_lazy("Current status of a Volume",
                                      u"Maintenance")),
        ("awaiting-transfer", pgettext_lazy("Current status of a Volume",
                                            u"Awaiting Transfer")),
        ("downloading", pgettext_lazy("Current status of a Volume",
                                      u"Downloading")),
        ("uploading", pgettext_lazy("Current status of a Volume",
                                    u"Uploading")),
        ("retyping", pgettext_lazy("Current status of a Volume",
                                   u"Retyping")),
        ("extending", pgettext_lazy("Current status of a Volume",
                                    u"Extending")),
    )

    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link="horizon:project:volumes:volumes:detail",
        form_field=EmptyCharField(max_length=255),
        update_action=volumes_actions.VolumeUpdateCell)

    description = tables.Column(
        "description",
        verbose_name=_("Description"),
        form_field=EmptyCharField(max_length=255),
        update_action=volumes_actions.VolumeUpdateCell)
    status = tables.Column(
        "status",
        verbose_name=_("Status"),
        status=True,
        status_choices=volume_tables.VolumesTable.STATUS_CHOICES,
        display_choices=STATUS_DISPLAY_CHOICES,
    )
    bootable = tables.Column(
        'is_bootable',
        verbose_name=_("Bootable"),
        filters=(filters.yesno, filters.capfirst),
        form_field=BooleanField(
            label=_('Bootable'),
            required=False),
        update_action=volumes_actions.VolumeSetBootableAction
    )

    class Meta(object):
        name = "volumes"
        verbose_name = _("Volumes")
        status_columns = ["status"]
        row_class = volume_tables.UpdateRow
        table_actions = (
            volume_tables.CreateVolume,
            volume_tables.AcceptTransfer,
            volumes_actions.VolumeFilterAction)
        table_actions_menu = (
            volumes_actions.TnxDeleteVolume,
            volumes_actions.TnxCreateSnapshot,
            volumes_actions.TnxCreateBackup,
            ProjectVolumeSchedulerAction)
        row_actions = (
            volume_tables.ExtendVolume,
            volume_tables.LaunchVolume,
            volume_tables.EditAttachments,
            volumes_actions.TnxCreateSnapshot,
            volumes_actions.TnxCreateBackup,
            volume_tables.RetypeVolume,
            volume_tables.UploadToImage,
            volume_tables.CreateTransfer,
            volume_tables.DeleteTransfer,
            volumes_actions.TnxDeleteVolume,
            ProjectVolumeSchedulerAction)
