# coding: utf-8
# pylint: disable=no-member

import mock

from novaclient.exceptions import ClientException

from horizon import messages
from horizon.exceptions import Http302

from openstack_dashboard import api
from openstack_dashboard.test import helpers as test
from openstack_dashboard.usage import quotas

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.clients import TNXVolumesClient
from tionix_dashboard.dashboards.project.project_volumes.volumes import (
    actions, forms, tables, views, tabs
)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.helpers.volume_constants import DELETING_STATUS
from tionix_dashboard.tests.helpers import FakeUser, patch_resource_get

VOLUME_DETAIL_URL = 'horizon:project:volumes:volumes:detail'


@mock.patch.object(messages, 'error', mock.Mock())
class ProjectVolumesActionsTestCase(test.TestCase):
    def test_tnx_create_backup_allowed(self):
        volume = self.volumes.list()[0]

        action = actions.TnxCreateBackup()

        self.assertTrue(action.allowed(self.request, volume))

    def test_tnx_create_backup_not_allowed(self):
        volume = self.volumes.list()[1]

        action = actions.TnxCreateBackup()

        self.assertFalse(action.allowed(self.request, volume))

    @mock.patch.object(api.cinder, 'volume_get')
    @mock.patch.object(api.cinder, 'volume_backup_create')
    def test_tnx_create_backup(self, volume_backup_create,
                               volume_get):
        volume = self.volumes.first()

        volume_get.return_value = volume

        action = actions.TnxCreateBackup()

        action.action(self.request, volume.id)

        self.assertTrue(volume_backup_create.called)

    @mock.patch.object(api.cinder,
                       'volume_get',
                       mock.Mock(side_effect=Exception))
    @mock.patch.object(api.cinder, 'volume_backup_create')
    def test_tnx_create_backup_error(self, volume_backup_create):
        action = actions.TnxCreateBackup()

        action.action(self.request, 10000)

        self.assertFalse(volume_backup_create.called)

    @mock.patch.object(TNXVolumesClient,
                       'update',
                       mock.Mock(return_value=None))
    def test_volume_update_cell(self):
        volume = self.volumes.first()

        action = actions.VolumeUpdateCell()

        self.assertTrue(
            action.update_cell(self.request, volume, volume.id,
                               'name', '')
        )

    @mock.patch.object(TNXVolumesClient,
                       'update',
                       mock.Mock(side_effect=Exception))
    def test_volume_update_cell_error(self):
        action = actions.VolumeUpdateCell()

        self.assertFalse(
            action.update_cell(self.request, None, 10000, 'name', '')
        )

    @mock.patch.object(api.cinder, 'volume_set_bootable')
    def test_set_bootable(self, volume_set_bootable):
        volume = self.cinder_volumes.first()

        set_bootable_action = actions.VolumeSetBootableAction()

        set_bootable_action.update_cell(
            request=None,
            datum=volume,
            obj_id=volume.id,
            cell_name='bootable',
            new_cell_value=not volume.is_bootable)

        self.assertTrue(volume_set_bootable.called)

    @mock.patch.object(api.cinder, 'volume_set_bootable')
    def test_set_bootable_not_changed_value(self,
                                            volume_set_bootable):
        volume = self.cinder_volumes.first()

        set_bootable_action = actions.VolumeSetBootableAction()

        set_bootable_action.update_cell(
            request=None,
            datum=volume,
            obj_id=volume.id,
            cell_name='bootable',
            new_cell_value=volume.is_bootable)

        self.assertFalse(volume_set_bootable.called)

    @mock.patch.object(api.cinder, 'volume_set_bootable')
    def test_set_bootable_another_call_name(self,
                                            volume_set_bootable):
        volume = self.cinder_volumes.first()

        set_bootable_action = actions.VolumeSetBootableAction()

        set_bootable_action.update_cell(
            request=None,
            datum=volume,
            obj_id=volume.id,
            cell_name='not_bootable',
            new_cell_value=volume.is_bootable)

        self.assertFalse(volume_set_bootable.called)


class ProjectVolumesFormsTestCase(test.TestCase):
    def test_create_snapshot_form_fields(self):
        form = forms.TnxCreateSnapshotForm(self.request)

        self.assertListEqual(
            form.fields.keys(),
            ['name', 'description']
        )

    @mock.patch.object(api.cinder,
                       'volume_snapshot_create',
                       mock.Mock(return_value=None))
    @mock.patch.object(api.cinder, 'volume_get')
    def test_create_snapshot_form_handle(self, volume_get):
        volumes = self.volumes.list()

        volume_get.side_effect = patch_resource_get(self.volumes,
                                                    volumes[0].id)

        form = forms.TnxCreateSnapshotForm(self.request)
        form.initial = {
            'volumes_ids': [volume.id for volume in volumes]
        }

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        result = form.handle(
            request=request,
            data={'name': 'name', 'description': 'description'}
        )

        self.assertTrue(result)

        self.assertEqual(
            request.horizon['async_messages'][0][1],
            u'Unable to create volume snapshot for {0}.'.format(
                volumes[0].id
            )
        )

        self.assertEqual(
            request.horizon['async_messages'][1][1],
            u'Forcing to create snapshot from attached volumes {0}.'.format(
                ', '.join([volume.name for volume in volumes[1:]
                           if volume.status == 'in-use'])
            )
        )

        self.assertEqual(
            request.horizon['async_messages'][2][1],
            u'Creating volume snapshot for {0}.'.format(
                ', '.join([volume.name for volume in volumes[1:]])
            )
        )

    @mock.patch.object(api.cinder,
                       'volume_get',
                       mock.Mock(
                           side_effect=ClientException(code=413)))
    def test_create_snapshot_form_error413(self):
        volume = self.volumes.first()

        form = forms.TnxCreateSnapshotForm(self.request)
        form.initial = {
            'volumes_ids': [volume.id]
        }

        with self.assertRaises(Http302) as cm:
            form.handle(
                request=get_fake_request(
                    user=FakeUser(),
                    is_ajax=True,
                    horizon={
                        'async_messages': []
                    }
                ),
                data={'name': 'name', 'description': 'description'}
            )

        self.assertEqual(u'/project/volumes/', cm.exception.location)
        self.assertEqual(302, cm.exception.status_code)


class ProjectVolumesTablesTestCase(test.TestCase):
    def test_tnx_volumes_table(self):
        table = tables.TnxVolumesTableBase(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'name', 'description', 'size',
                    'status',
                    'volume_type', 'attachments', 'availability_zone',
                    'bootable', 'encryption', 'actions'])
        )

    @mock.patch.object(api.cinder, 'tenant_absolute_limits')
    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_volumes_table_actions(self, tenant_absolute_limits):
        volumes = self.volumes.list()

        tenant_absolute_limits.return_value = self.limits['absolute']

        table = tables.TnxVolumesTableBase(self.request, data=volumes)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['create', 'accept_transfer', 'filter', 'delete',
                    'snapshots', 'backup', 'volume_schedule'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.cinder, 'tenant_absolute_limits')
    def test_tnx_volumes_table_row_actions(self, tenant_absolute_limits):
        table = tables.TnxVolumesTableBase(self.request)

        tenant_absolute_limits.return_value = self.limits['absolute']

        volume = self.volumes.first()

        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(volume)]),
            sorted(['extend', 'launch_volume', 'attachments',
                    'snapshots', 'backup', 'retype', 'upload_to_image',
                    'create_transfer', 'delete', 'volume_schedule'])
        )

    def test_tnx_volumes_table_deleting_status(self):
        volume = self.volumes.list()[0]

        table = tables.TnxVolumesTableBase(self.request)

        self.assertNotEqual(table.get_row_actions(volume), 0)

        volume.status = DELETING_STATUS

        self.assertEqual(len(table.get_row_actions(volume)), 0)


class ProjectVolumesViewsTestCase(test.TestCase):
    @mock.patch.object(quotas, 'tenant_limit_usages')
    @mock.patch.object(api.cinder, 'volume_get')
    def test_tnx_create_snapshot_view_context_data(
            self, volume_get, tenant_limit_usages):
        volumes = self.volumes.list()

        volume_get.side_effect = patch_resource_get(self.volumes)
        usage_limit = {
            'maxTotalVolumeGigabytes': 250,
            'gigabytesUsed': 20,
            'snapshotsUsed': len(self.cinder_volume_snapshots.list()),
            'maxTotalSnapshots': 6
        }
        tenant_limit_usages.return_value = usage_limit

        view = views.TnxCreateSnapshotView()
        view.request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            get_params={
                'id': [volume.id for volume in volumes]
            },
            META={},
            method='GET'
        )

        context = view.get_context_data(
            form=view.get_form(forms.TnxCreateSnapshotForm)
        )

        self.assertDictEqual(
            context['usages'],
            usage_limit
        )

        self.assertTrue(context['attached'])

        self.assertIsNotNone(context['form'].warnings)

    def test_tnx_create_snapshot_view_initial(self):
        volumes = self.volumes.list()

        view = views.TnxCreateSnapshotView()
        view.request = get_fake_request(
            get_params={
                'id': [volume.id for volume in volumes]
            },
            META={}
        )

        self.assertDictEqual(
            view.get_initial(),
            {'volumes_ids': [str(volume.id) for volume in volumes]}
        )

    @mock.patch.object(api.cinder, 'volume_get')
    @mock.patch.object(api.cinder, 'volume_snapshot_list')
    @mock.patch.object(api.nova, 'server_get')
    @mock.patch.object(api.cinder, 'tenant_absolute_limits')
    def test_detail_view_context_data(
            self, tenant_absolute_limits, server_get,
            volume_snapshot_list,
            volume_get):
        volume = self.volumes.first()

        volume_get.return_value = volume
        volume_snapshot_list.return_value = self.cinder_volume_snapshots.list()
        tenant_absolute_limits.return_value = self.limits['absolute']
        server_get.side_effect = patch_resource_get(self.servers)

        view = views.DetailView()
        view.request = self.request
        view.kwargs = {
            'volume_id': volume.id
        }

        context = view.get_context_data()
        table = tables.TnxVolumesTableBase(self.request)

        self.assertEqual(
            context['actions'], table.render_row_actions(volume)
        )


class ProjectVolumesTabsTestCase(test.TestCase):

    @mock.patch.object(api.cinder,
                       'volume_backup_supported',
                       mock.Mock(return_value=True))
    def test_volume_detail_tabs_backups_available(self):

        tab_group = tabs.VolumeDetailTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['overview', 'scheduler', 'snapshots_tab', 'backups_tab'])
        )

    @mock.patch.object(api.cinder,
                       'volume_backup_supported',
                       mock.Mock(return_value=False))
    def test_volume_detail_tabs_backups_not_available(self):

        tab_group = tabs.VolumeDetailTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['overview', 'scheduler', 'snapshots_tab'])
        )
