# coding: utf-8

import logging

from django.core.urlresolvers import reverse, reverse_lazy
from django.shortcuts import redirect as redirect_to
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, messages, tabs as horizon_tabs
from horizon.utils import memoized

from openstack_dashboard.api import cinder, nova
from openstack_dashboard.api.rest import (
    cinder as rest_cinder, urls, utils as rest_utils
)
from openstack_dashboard.usage import quotas
from openstack_dashboard.utils import filters
from openstack_dashboard.dashboards.project.volumes.volumes import (
    tables as volume_tables, views as volume_views)

from tionix_scheduler_client.enums import ObjectTypeEnum

from tionix_dashboard.clients.base import cinderclient
from tionix_dashboard.dashboards.project.project_volumes.volumes import (
    tables, tabs)
from tionix_dashboard.dashboards.project.project_volumes.volumes.forms import (
    TnxCreateSnapshotForm, DeleteVolumeForm, CreateVolumeForm,
    CreateTransferForm, AttachForm)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ObjectTasksView)
from tionix_dashboard.helpers import vm_constants
from tionix_dashboard.helpers.views import (
    ServerFilterViewMixin, ServerSortMultiSelect, LinkBatchFormView)

logger = logging.getLogger(__name__)


VOLUME_IN_USE_STATUS = 'in-use'


class TnxCreateSnapshotView(LinkBatchFormView):
    form_class = TnxCreateSnapshotForm
    modal_header = _("Create Volume Snapshot")
    template_name = 'project/volumes/volumes/create_snapshot.html'
    submit_url = "horizon:project:volumes:volumes:create_snapshot"
    success_url = reverse_lazy('horizon:project:volumes:snapshots_tab')
    page_title = _("Create Volume Snapshot")

    def get_context_data(self, **kwargs):
        context = super(TnxCreateSnapshotView, self).get_context_data(**kwargs)
        ids = context_ids = self.get_object_ids()
        attached_volumes, failed_volumes = [], []

        for _id in ids:
            try:
                volume = cinder.volume_get(self.request, _id)
                if volume.status in ("available", "in-use"):
                    if volume.status == 'in-use':
                        attached_volumes.append(volume.name)
                else:
                    failed_volumes.append(volume.name)
                    context_ids.remove(_id)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve volume information.'))

        try:
            context['usages'] = quotas.tenant_limit_usages(self.request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve tenant usages.'))

        if attached_volumes:
            context['attached'] = True
            context['form'].set_warning(
                _('Volumes %s is currently attached to an instance. '
                  'In some cases, creating a snapshot from an attached '
                  'volume can result in a corrupted snapshot.') %
                ','.join(attached_volumes)
            )

        if failed_volumes:
            msg = _('"Create Volume Snapshot" is '
                    'not available for volumes: "%s"')
            messages.error(self.request, msg % '", "'.join(failed_volumes))

        return context

    def get_initial(self):
        return {'volumes_ids': self.get_object_ids()}


class DetailView(ServerSortMultiSelect,
                 ServerFilterViewMixin,
                 horizon_tabs.TabbedTableView):

    tab_group_class = tabs.VolumeDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ volume.name|default:volume.id }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        volume = self.get_data()
        table = tables.TnxVolumesTableBase(self.request)
        context["volume"] = volume
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(volume)
        choices = table.STATUS_DISPLAY_CHOICES
        volume.status_label = filters.get_display_label(choices, volume.status)
        return context

    @memoized.memoized_method
    def get_data(self):
        volume = None
        volume_id = self.kwargs['volume_id']
        try:
            volume = cinder.volume_get(self.request, volume_id)
            snapshots = cinder.volume_snapshot_list(
                self.request, search_opts={'volume_id': volume.id})
            if snapshots:
                setattr(volume, 'has_snapshot', True)
            for att in volume.attachments:
                att['instance'] = nova.server_get(self.request,
                                                  att['server_id'])
        except Exception as err:
            logger.error(err)
            redirect = self.get_redirect_url()
            exceptions.handle(self.request,
                              _('Unable to retrieve volume details.'),
                              redirect=redirect)
        return volume

    def get_redirect_url(self):
        return reverse('horizon:project:volumes:index')

    def get_tabs(self, request, **kwargs):
        volume = self.get_data()
        return self.tab_group_class(request, volume=volume, **kwargs)


class DeleteVolumeView(ObjectTasksView, forms.ModalFormView):
    """ Delete volumes view """

    form_class = DeleteVolumeForm
    modal_header = _('Delete Confirmation')
    template_name = "tionix_dashboard/delete_object.html"
    modal_id = "delete_volume"
    submit_label = _('Delete')
    context_object_name = 'volume'
    success_url = reverse_lazy("horizon:project:volumes:index")
    confirm_url = reverse_lazy("horizon:project:volumes:volumes:delete")
    hidden_title = False
    failure_msg = _('"Delete" is not available for volume')
    msg = _("Deleted volumes are not recoverable. "
            "All data stored in the volume will be removed.")

    def get_initial(self):
        ids = self.get_object_ids()
        names = {}
        for volume_id in ids:
            try:
                volume = cinder.volume_get(self.request, volume_id)
            except Exception as err:
                logger.error(unicode(err))
                redirect = self.get_redirect_url()
                exceptions.handle(self.request,
                                  _('Unable to retrieve details for '
                                    'volume "%s".') % volume_id,
                                  redirect=redirect)
                # Not all exception types handled above will result in a
                # redirect. Need to raise here just in case.
                raise exceptions.Http302(redirect)
            else:
                names[volume_id] = volume.name

        query_params = {
            'subject_id': ids,
            'subject_type': ObjectTypeEnum.volume,
            'status': True,
        }
        rows = self.get_tasks_rows(self.request, query_params)
        return {
            'rows': rows,
            'object_names_dict': names,
            'ids': ids,
            'msg': self.msg,
            'confirm_url': self.confirm_url,
            'redirect_url': self.get_redirect_url(),
        }


# noinspection PyClassHasNoInit
class CreateVolumeView(volume_views.CreateView):
    form_class = CreateVolumeForm


# noinspection PyClassHasNoInit
class CreateTransferView(volume_views.CreateTransferView):
    form_class = CreateTransferForm


# noinspection PyClassHasNoInit
class AcceptTransferView(volume_views.AcceptTransferView):
    modal_header = _("Accept Volume Transfer")
    submit_label = _("Accept Volume Transfer")


class EditAttachmentsView(forms.ModalFormView):
    form_class = AttachForm
    form_id = "attach_volume_form"
    modal_header = _("Manage Volume Attachments")
    modal_id = "attach_volume_modal"
    template_name = 'project_volumes/volumes/attach.html'
    submit_url = "horizon:project:volumes:volumes:attach"
    success_url = reverse_lazy("horizon:project:volumes:index")
    page_title = _("Manage Volume Attachments")

    def get(self, request, *args, **kwargs):
        servers, _has_more = nova.server_list(
            request, search_opts={'status': vm_constants.ACTIVE}
        )
        if not servers:
            messages.warning(request, _("There are no instances to attach "
                                        "to the volume in the project."))
            return redirect_to(self.get_cancel_url())
        return super(EditAttachmentsView, self).get(request, *args, **kwargs)

    @memoized.memoized_method
    def get_object(self):
        volume_id, volume = self.kwargs['volume_id'], None
        try:
            volume = cinder.volume_get(self.request, volume_id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve volume information.'))
        return volume

    def get_initial(self):
        volume, instance_id = self.get_object(), None
        if volume.attachments:
            instance_id = volume.attachments[0]['server_id']
        return {'volume': volume,
                'volume_id': volume.id,
                'instance_id': instance_id}

    def get_context_data(self, **kwargs):
        context = super(EditAttachmentsView, self).get_context_data(**kwargs)
        volume = self.get_object()
        context['volume_id'] = volume.id
        if self.request.is_ajax():
            context['hide'] = True
        return context


@urls.register
class Volumes(rest_cinder.Volumes):
    """API for cinder volumes.
    """

    url_regex = r'tnx/volumes/$'

    @rest_utils.ajax()
    def get(self, request):

        volume_statuses = dict(
            volume_tables.VolumesTableBase.STATUS_DISPLAY_CHOICES
        )

        attached_instance_id_param = 'attached_instance_id'
        project_id_param = 'project_id'

        search_opts, kwargs = rest_utils.parse_filters_kwargs(
            request, (attached_instance_id_param, project_id_param)
        )

        attached_instance_id = kwargs.get(attached_instance_id_param)
        project_id = kwargs.get(project_id_param)

        if attached_instance_id:
            statuses = [VOLUME_IN_USE_STATUS]
            if 'status' in search_opts:
                statuses += [search_opts['status']]
            search_opts['status'] = statuses

        try:
            volumes = cinderclient(
                request, project_id=project_id
            ).volumes.list(search_opts=search_opts)
        except Exception as err:
            logger.error(unicode(err))
            raise rest_utils.AjaxError(400,
                                       _('Enable to retrieve volume list.'))

        result = []
        for volume in volumes:
            volume_attached_instances = [
                attachment['server_id'] for attachment in volume.attachments
            ]
            if (volume_attached_instances and
                    attached_instance_id not in volume_attached_instances):
                continue
            v = volume.to_dict()
            v['status_name'] = unicode(
                volume_statuses.get((v.get('status') or '').lower())
            )
            result.append(v)

        return {'items': result}
