# coding: utf-8

import unittest

from django.core.urlresolvers import get_resolver
from django.core.urlresolvers import RegexURLResolver, RegexURLPattern
from django.core.urlresolvers import resolve

import horizon
from horizon.base import Horizon
from horizon.base import NotRegistered

from openstack_dashboard.dashboards.admin.hypervisors import (
    urls as admin_hypervisors)
from openstack_dashboard.dashboards.admin.hypervisors.views import (
    AdminIndexView, AdminDetailView)
from openstack_dashboard.dashboards.admin.instances import (
    urls as admin_instances)
from openstack_dashboard.dashboards.project.instances import (
    urls as project_instances)
from openstack_dashboard.dashboards.project.instances.views import IndexView

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.tionix.vdi.panel import VDI
from tionix_dashboard.dashboards.admin.hypervisors.views import (
    TnxAdminHypervisorsIndexView, HypervisorDetailView)
from tionix_dashboard.dashboards.admin.instances import (
    views as tnx_admin_instances)
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.middleware import TionixMiddleware
from tionix_dashboard.dashboards.project.project_instances import (
    views as tnx_project_instances)
from tionix_dashboard.dashboards.tionix.dashboard import TionixDashboard
from tionix_dashboard.dashboards.tionix.metrics.panel import Metrics
from tionix_dashboard.tests.helpers import FakeUser


class LicenseTestMixin(object):

    def _get_app_view_names(self, urls):
        """
        Get all views associated with the url
        :param urls:
        :return:
        """
        _view_names = []

        def _get_all_view_names(view_names, urlpatterns):
            for pattern in urlpatterns:
                if isinstance(pattern, RegexURLResolver):
                    _get_all_view_names(view_names, pattern.url_patterns)
                elif isinstance(pattern, RegexURLPattern):
                    view_name = pattern.callback.func_name
                    view_names.append(view_name)

        _get_all_view_names(_view_names, urls.urlpatterns)
        return _view_names

    def tnx_hypervisors(self):
        views = self._get_app_view_names(admin_hypervisors)
        self.assertIn(TnxAdminHypervisorsIndexView.__name__, views)

        admin_hypervisors_view = resolve('/admin/hypervisors/')
        self.assertEqual(admin_hypervisors_view.func.func_name,
                         TnxAdminHypervisorsIndexView.__name__)

        admin_hypervisors_view = resolve('/admin/hypervisors/1/')
        self.assertEqual(admin_hypervisors_view.func.func_name,
                         HypervisorDetailView.__name__)

    def tnx_instances(self):

        views = self._get_app_view_names(project_instances)
        self.assertIn(
            tnx_project_instances.TnxProjectInstancesIndexView.__name__, views
        )

        project_instances_view = resolve('/project/instances/')
        self.assertEqual(
            project_instances_view.func.func_name,
            tnx_project_instances.TnxProjectInstancesIndexView.__name__)

        project_instances_view = resolve('/project/instances/1/')
        self.assertEqual(project_instances_view.func.func_name,
                         tnx_project_instances.TnxDetailView.__name__)

        views = self._get_app_view_names(admin_instances)
        self.assertIn(
            tnx_admin_instances.TnxAdminInstancesIndexView.__name__, views
        )

        project_instances_view = resolve('/admin/instances/')
        self.assertEqual(
            project_instances_view.func.func_name,
            tnx_admin_instances.TnxAdminInstancesIndexView.__name__)

    def os_hypervisors(self):

        views = self._get_app_view_names(admin_hypervisors)
        self.assertNotIn(TnxAdminHypervisorsIndexView.__name__, views)
        self.assertNotIn(HypervisorDetailView.__name__, views)

        admin_hypervisors_view = resolve('/admin/hypervisors/')
        self.assertEqual(admin_hypervisors_view.func.func_name,
                         AdminIndexView.__name__)

        admin_hypervisors_view = resolve('/admin/hypervisors/1/')
        self.assertEqual(admin_hypervisors_view.func.func_name,
                         AdminDetailView.__name__)

    def os_instances(self):

        views = self._get_app_view_names(project_instances)
        self.assertNotIn(
            tnx_project_instances.TnxProjectInstancesIndexView.__name__, views
        )

        project_instances_view = resolve('/project/instances/')
        self.assertEqual(project_instances_view.func.func_name,
                         IndexView.__name__)

        project_instances_view = resolve('/admin/instances/')
        self.assertIn(
            project_instances_view.func.func_name,
            [AdminIndexView.__name__,
             tnx_admin_instances.TnxAdminInstancesIndexView.__name__]
        )

    def os_instance_details(self):

        project_instance_console_tab_view = resolve(
            '/project/instances/1/?tab=instance_details__console'
        )
        self.assertEqual(project_instance_console_tab_view.func.func_name,
                         tnx_project_instances.DetailView.__name__)

        admin_instance_console_tab_view = resolve(
            '/admin/instances/1/detail?tab=instance_details__console'
        )
        self.assertIn(admin_instance_console_tab_view.func.func_name,
                      tnx_admin_instances.DetailView.__name__)

        admin_fullscreen_view = resolve(
            '/admin/instances/1/fullscreen_console'
        )
        self.assertEqual(
            admin_fullscreen_view.func.func_name,
            tnx_admin_instances.FullScreenConsoleView.__name__)

        project_fullscreen_view = resolve(
            '/project/instances/1/fullscreen_console'
        )
        self.assertEqual(
            project_fullscreen_view.func.func_name,
            tnx_admin_instances.FullScreenConsoleView.__name__)

    def tnx_instance_details(self):

        project_instance_console_tab_view = resolve(
            '/project/instances/1/?tab=instance_details__console'
        )
        self.assertEqual(project_instance_console_tab_view.func.func_name,
                         tnx_project_instances.TnxDetailView.__name__)

        admin_instance_console_tab_view = resolve(
            '/admin/instances/1/detail?tab=instance_details__console'
        )
        self.assertIn(admin_instance_console_tab_view.func.func_name,
                      tnx_admin_instances.TnxDetailView.__name__)

        admin_fullscreen_view = resolve(
            '/admin/instances/1/fullscreen_console'
        )
        self.assertEqual(
            admin_fullscreen_view.func.func_name,
            tnx_admin_instances.FullScreenConsoleView.__name__)

        project_fullscreen_view = resolve(
            '/project/instances/1/fullscreen_console'
        )
        self.assertEqual(
            project_fullscreen_view.func.func_name,
            tnx_admin_instances.FullScreenConsoleView.__name__)

    def tnx_panel(self):
        dash = Horizon.get_dashboard(TionixDashboard)
        self.assertTrue(dash.get_panels())

    def no_panel(self):
        dash = Horizon.get_dashboard(TionixDashboard)
        self.assertFalse(dash.get_panels())

    def tnx_metrics(self):
        dash = Horizon.get_dashboard(TionixDashboard)
        self.assertIsNotNone(dash.get_panel(Metrics))

    def no_metrics(self):
        dash = Horizon.get_dashboard(TionixDashboard)
        self.assertRaises(NotRegistered, dash.get_panel, Metrics)

    def tnx_vdi(self):
        dash = Horizon.get_dashboard(TionixDashboard)
        self.assertIsNotNone(dash.get_panel(VDI))

    def no_vdi(self):
        dash = Horizon.get_dashboard(TionixDashboard)
        self.assertRaises(NotRegistered, dash.get_panel, VDI)


class DashboardsTestCase(unittest.TestCase, LicenseTestMixin):

    licences = {
        LicenseController.NODE_CONTROL: True,
        LicenseController.CLOUD_CONTROL: True,
        LicenseController.MONITOR: True,
        LicenseController.VDI: True
    }

    def _is_valid(self, request, module):
        return self.licences[module]

    def _get_licenses(self, request, recollect=False):
        return {key: Empty(verified=value)
                for key, value in self.licences.items()}

    def _register(self):
        """
        register TIONIX again
        :return:
        """
        lic_type = LicenseController.LIC_TYPES.keys()

        for lt in lic_type:
            for dash, panels in LicenseController._panels[lt].iteritems():
                dash = horizon.get_dashboard(dash)
                for panel in panels:
                    try:
                        dash.register(panel)
                    except NotRegistered:
                        pass
            for panel, urls in LicenseController._urls[lt]:
                url_patterns = get_resolver(
                    panel.__module__.replace('.panel', '.urls')
                ).url_patterns
                _patterns = [
                    url_patterns.insert(0, i) for i in urls
                    if i not in url_patterns
                ]

    def _check_license_request(self):
        """
        recheck licence
        :return:
        """
        LicenseController.first_checked = False
        self.middleware.process_request(self.request)

    def _check_license_exception(self, lic_type):
        """
        Imitate 402 or 404 exception
        :return:
        """
        error = UrlNotAvailableError(None, lic_type)
        # following try/except necessary for the safe adding error
        # to sys.exc_info() traceback output
        try:
            raise error
        except Exception:
            pass
        self.middleware.process_exception(self.request, error)

    def setUp(self):
        self.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        self.licences = {key: True for key in self.licences}
        self._base_is_valid = LicenseController.is_valid
        self._base_get_licenses = LicenseController.get_licenses
        LicenseController.first_checked = False
        LicenseController.is_valid = self._is_valid
        LicenseController.get_licenses = self._get_licenses

        self.middleware = TionixMiddleware()

    def tearDown(self):
        LicenseController.is_valid = self._base_is_valid
        LicenseController.get_licenses = self._base_get_licenses
        self._register()

    def test_all_valid(self):

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

    def test_node_control_invalid(self):
        self.licences[LicenseController.NODE_CONTROL] = False
        self._check_license_request()

        self.os_hypervisors()
        self.tnx_instances()
        self.tnx_metrics()
        self.tnx_vdi()

    def test_cloud_control_invalid(self):
        self.licences[LicenseController.CLOUD_CONTROL] = False
        self._check_license_request()

        self.tnx_hypervisors()
        self.os_instances()
        self.tnx_instance_details()
        self.tnx_metrics()
        self.tnx_vdi()

    def test_all_invalid(self):
        self.licences = {key: False for key in self.licences}
        self._check_license_request()

        self.os_hypervisors()
        self.os_instances()
        self.os_instance_details()
        # TEMPORARY NO VERIFY
        # self.no_panel()

    def test_node_cloud_invalid(self):
        self.licences[LicenseController.CLOUD_CONTROL] = False
        self.licences[LicenseController.NODE_CONTROL] = False
        self._check_license_request()

        self.os_hypervisors()
        self.os_instances()
        self.tnx_instance_details()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

    def test_monitor_invalid(self):
        self.licences[LicenseController.MONITOR] = False
        self._check_license_request()

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.no_metrics()
        self.tnx_vdi()

    def test_vdi_invalid(self):
        self.licences[LicenseController.VDI] = False
        self._check_license_request()

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_instance_details()
        self.tnx_panel()
        self.tnx_metrics()
        self.no_vdi()

    def test_node_control_402_404(self):

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

        self._check_license_exception(LicenseController.NODE_CONTROL)

        self.os_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

    def test_cloud_control_402_404(self):

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

        self._check_license_exception(LicenseController.CLOUD_CONTROL)

        self.tnx_hypervisors()
        self.os_instances()
        self.tnx_instance_details()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

    def test_monitor_402_404(self):

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

        self._check_license_exception(LicenseController.MONITOR)

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.no_metrics()
        self.tnx_vdi()

    def test_vdi_402_404(self):

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

        self._check_license_exception(LicenseController.VDI)

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.no_vdi()

    def test_tionix_402_404(self):

        self.tnx_hypervisors()
        self.tnx_instances()
        self.tnx_panel()
        self.tnx_metrics()
        self.tnx_vdi()

        self._check_license_exception(LicenseController.NODE_CONTROL)
        self._check_license_exception(LicenseController.CLOUD_CONTROL)
        self._check_license_exception(LicenseController.MONITOR)
        self._check_license_exception(LicenseController.VDI)
        # TEMPORARY NO VERIFY
        # self.no_panel()
