# coding: utf-8
# pylint: disable=no-name-in-module

"""Balancing compute tables"""

from collections import defaultdict
import operator
import logging

from django.core.urlresolvers import reverse
from django.template import defaultfilters as filters
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils.datastructures import SortedDict
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables
from horizon.templatetags import sizeformat

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.aggregates.tables import (
    safe_unordered_list)
from tionix_dashboard.dashboards.project.project_instances import constants

from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.dashboards.admin.hypervisors import (
    actions as hyp_actions)
from tionix_dashboard.dashboards.tionix.balancing.compute import actions
from tionix_dashboard.helpers.tables import ServerFilterTableMixin
from tionix_dashboard.helpers.utils import get_resource_template
from tionix_dashboard.helpers.vm_constants import VMTypes
from tionix_dashboard.clients import TNXHypervisorsClient, TNXServersClient


logger = logging.getLogger(__name__)


def get_vcpu(hypervisor):
    return get_resource_template(hypervisor.vcpus, hypervisor.vcpus_used)


def get_ram(hypervisor):
    return get_resource_template(hypervisor.memory_mb,
                                 hypervisor.memory_mb_used)


def get_disk(hypervisor):
    return get_resource_template(hypervisor.local_gb, hypervisor.local_gb_used)


# pylint: disable=too-many-nested-blocks
def get_compute_hosts(request, obj_id=None):
    hypervisors = []
    try:
        if obj_id:
            hyps = (TNXHypervisorsClient.get(request, obj_id),)
        else:
            hyps = TNXHypervisorsClient.list(request)
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request,
                          _('Unable to retrieve hypervisor information.'))
    else:
        try:
            aggregates = api.nova.aggregate_details_list(request)
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve host aggregates list.')
            exceptions.handle(request, msg)
        else:
            host_aggregates = defaultdict(set)
            for aggregate in aggregates:
                if getattr(aggregate, 'tnx_allow_balancing', None):
                    for host in aggregate.hosts:
                        host_aggregates[host].add(aggregate.name)
            tenants, flavors = {}, {}
            try:
                tenants, _has_more = api.keystone.tenant_list(request)
            except Exception as err:
                logger.error(unicode(err))
                msg = _('Unable to retrieve instance project information.')
                exceptions.handle(request, msg)
            else:
                tenants = SortedDict([(t.id, t.name) for t in tenants])
            try:
                flavors = {flavor.id: (flavor.ram, flavor.vcpus, flavor.disk)
                           for flavor in api.nova.flavor_list(request)}

            except Exception as err:
                logger.error(unicode(err))

            for hyp in hyps:
                if hyp.service:
                    host_name = hyp.service.get('host')
                    tnx_power_state = getattr(hyp, 'tnx_power_state', None)
                    if (hyp.state == 'up' and
                            tnx_power_state in ('power_on', None) and
                            host_name in host_aggregates):
                        hyp.host_aggregates = host_aggregates[host_name]
                        hyp.ram = get_ram(hyp)

                        servers = TNXServersClient.list(
                            request, search_opts={'all_tenants': True,
                                                  'host': host_name})
                        hyp.servers = []
                        for server in servers:
                            server.tenant = tenants.get(server.tenant_id)
                            server.ram, server.vcpus, server.disk = (
                                flavors.get(server.flavor["id"],
                                            (None, None, None)))
                            addresses = server.addresses.values()
                            server.internal_ip = ''
                            if addresses:
                                address = addresses[0]
                                if address:
                                    try:
                                        server.internal_ip = (
                                            address[0]['addr'])
                                    except (IndexError, KeyError):
                                        pass
                        hyp.servers = servers
                        hyp.servers.sort(key=operator.attrgetter('ram'),
                                         reverse=True)
                        hypervisors.append(hyp)
    hypervisors.sort(key=operator.attrgetter('ram'), reverse=True)
    return hypervisors


class HostInstancesTable(tables.DataTable):
    tenant = tables.Column("tenant", verbose_name=_("Project"))
    name = tables.Column("name", verbose_name=_("Name"))
    status = tables.Column(
        "status",
        verbose_name=_("State"),
        status_choices=constants.STATUS_CHOICES,
        display_choices=constants.VM_STATUS_DISPLAY_CHOICES)
    internal_ip = tables.Column("internal_ip", verbose_name=_("Internal IP"))
    type = tables.Column(lambda x: unicode(getattr(x, "tnx_type", '')),
                         verbose_name=_("Type"),
                         display_choices=VMTypes.TYPES.items())
    ram = tables.Column('ram',
                        verbose_name=_('RAM'),
                        attrs={'data-type': 'size'},
                        filters=(sizeformat.mb_float_format,))
    vcpus = tables.Column('vcpus', verbose_name=_('VCPUs'))
    disk = tables.Column('disk',
                         verbose_name=_('Root Disk'),
                         attrs={'data-type': 'size'},
                         filters=(sizeformat.diskgbformat,))

    class Meta(object):
        name = "compute_host_instances"
        template = "balancing/compute/_data_table_instances.html"
        footer = False

        row_actions = (
            actions.TnxBalanceInstance,
        )


class ExpandableColumn(tables.Column):

    def get_raw_data(self, datum):
        return mark_safe(u"<i class='fa fa-chevron-right' "
                         u"hz-expand-detail='fa-chevron-right fa-chevron-down'"
                         u"duration='200'></i>")


class ExpandableRow(tables.Row):

    ajax = True

    def get_data(self, request, obj_id):
        return get_compute_hosts(request, obj_id)[0]

    def render(self):
        return render_to_string(
            'balancing/compute/_data_table_row_expandable.html',
            {"row": self}
        )

    def render_details(self):
        table = HostInstancesTable(self.table.request, data=self.datum.servers)
        return table.render()


def get_hostname_data(hypervisor):
    link_url = reverse("horizon:admin:hypervisors:detail",
                       args=(hypervisor.id,))
    return render_to_string("balancing/compute/_hostname.html",
                            {"hostname": hypervisor.hypervisor_hostname,
                             "servers_count": hypervisor.running_vms,
                             "link_url": link_url})


class ComputeHostsTable(ServerFilterTableMixin, tables.DataTable):
    """Compute hosts table"""
    expandable = ExpandableColumn("", verbose_name="")
    hostname = tables.Column(
        get_hostname_data,
        attrs={'data-type': 'naturalSort'},
        verbose_name=_("Hostname"))
    node_title = tables.Column(
        'tnx_node_title',
        form_field=EmptyCharField(max_length=100),
        verbose_name=_("Node title"),
        update_action=hyp_actions.TnxNodeUpdateCell)
    hypervisor_type = tables.Column(
        "hypervisor_type",
        verbose_name=_("Type"))
    host_aggregates = tables.Column(
        "host_aggregates",
        verbose_name=_('Host aggregates'),
        wrap_list=True,
        filters=(safe_unordered_list,))
    is_reserved = tables.Column(
        "tnx_is_reserved",
        verbose_name=_('Reserved'),
        filters=(filters.yesno, filters.capfirst))
    ram = tables.Column(
        "ram",
        verbose_name=_('RAM'),
        attrs={'data-type': 'percentbar'})
    vcpu = tables.Column(
        get_vcpu,
        verbose_name=_('vCPU'),
        attrs={'data-type': 'percentbar'})
    disk = tables.Column(
        get_disk,
        verbose_name=_('Disk'),
        attrs={'data-type': 'percentbar'})

    class Meta(object):
        name = "compute_hosts"
        verbose_name = _("Compute hosts")
        row_class = ExpandableRow
        css_classes = 'expandable'
        multi_select = False
        table_actions = (actions.ComputeHostsFilterAction,)

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(ComputeHostsTable, self).__init__(request, data=data,
                                                needs_form_wrapper=True,
                                                **kwargs)
