# coding: utf-8
# pylint: disable=no-member, method-hidden

import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import tables, exceptions
from novaclient.exceptions import Conflict

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.host_classes import TNXHostClassesClient
from tionix_dashboard.dashboards.tionix.infrastructure import constants
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxUpdateAction, TnxBatchActionMixin)

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class HostClassesFilterAction(TnxFilterAction):

    name = "filter_host_classes"
    filter_type = "server"

    filter_choices = (
        ('id', _("ID"), False),
        ('name', _("Class name"), False),
        ('host_types', _("Types"), False),
    )


# noinspection PyAbstractClass
class HostClassTypesFilterAction(TnxFilterAction):

    name = "filter_host_types"
    filter_type = "server"

    filter_choices = (
        ('id', _("ID"), False),
        ('name', _("Type name"), False),
    )


# noinspection PyAbstractClass
class CreateHostClassAction(tables.LinkAction):
    """
    Create new host class action
    """
    name = 'create_host_class'
    verbose_name = _('Create host class')
    url = constants.INFRASTRUCTURE_HOST_CLASS_CREATE
    classes = ('ajax-modal',)


# noinspection PyAbstractClass
class EditHostClassAction(tables.LinkAction):
    """
    Edit host class action
    """
    name = 'edit_host_class'
    verbose_name = _('Edit host class')
    url = constants.INFRASTRUCTURE_HOST_CLASS_UPDATE
    classes = ('ajax-modal',)

    def allowed(self, request, datum):
        allowed = True
        if datum:
            allowed = allowed if datum.name else False
        return allowed

    def get_link_url(self, datum=None):
        return reverse(self.url, args=(datum.id,))


# noinspection PyAbstractClass
class CreateHostTypeAction(tables.LinkAction):
    """
    Create new host type action
    """
    name = 'create_host_type'
    verbose_name = _('Create host type')
    url = constants.INFRASTRUCTURE_HOST_TYPE_CREATE
    classes = ('ajax-modal',)

    def get_link_url(self, datum=None):
        return reverse(self.url, args=(self.table.kwargs['host_class_id'],))


# noinspection PyAbstractClass
class EditHostTypeAction(tables.LinkAction):
    """
    Edit host type action
    """
    name = 'edit_host_type'
    verbose_name = _('Edit host type')
    url = constants.INFRASTRUCTURE_HOST_TYPE_UPDATE
    classes = ('ajax-modal',)

    def allowed(self, request, datum):
        allowed = True
        if datum:
            allowed = allowed if datum.name else False
        return allowed

    def get_link_url(self, datum=None):
        return reverse(self.url, args=(self.table.kwargs['host_class_id'],
                                       datum.id))


class DeleteHostTypeAction(TnxBatchActionMixin, tables.DeleteAction):
    """Delete host type(s) action"""

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Host Type",
            u"Delete Host Types",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Host Type",
            u"Deleted Host Types",
            count
        )

    def delete(self, request, obj_id):
        TNXHostClassesClient.delete_host_type(
            request,
            self.table.kwargs['host_class_id'],
            obj_id)


class HostClassUpdateRow(tables.Row):
    """
    Action that updates host classes rows in a table.
    """
    ajax = True

    def get_data(self, request, obj_id):
        try:
            host_class = TNXHostClassesClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.critical(err)
            raise ObjectDoesNotExist()
        return host_class


class HostTypeUpdateRow(tables.Row):
    """
    Action updated host type rows in table.
    """
    ajax = True

    def get_data(self, request, obj_id):
        host_class_type = None
        host_class = self.table.kwargs['host_class']
        try:
            host_class_type = host_class.get_host_type(obj_id)
        except ObjectDoesNotExist:
            exceptions.handle(
                request,
                _('Unable to get host type.'),
                redirect=reverse(constants.INFRASTRUCTURE_HOST_CLASS_DETAIL,
                                 args=(host_class.id,))
            )

        return host_class_type


class HostClassUpdateCell(TnxUpdateAction):
    """
    Action update host class
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update host class cell
        """
        try:
            TNXHostClassesClient.update(
                request,
                obj_id,
                **{cell_name: new_cell_value}
            )
        except Conflict as err:
            logger.error(unicode(err))
            exceptions.handle(
                request,
                _('Host class with name "{0}" already exists.').format(
                    new_cell_value))
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to edit host class "{0}".').format(
                                  datum.name))


class HostTypeUpdateCell(TnxUpdateAction):
    """
    Action update host type
    """

    def allowed(self, request, datum, cell):
        """
        check permission
        """
        allowed = super(HostTypeUpdateCell, self).allowed(request, datum, cell)
        if datum:
            allowed = allowed if datum.name else False
        return allowed

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        try:
            TNXHostClassesClient.update_host_type(
                request,
                datum.host_class_id, obj_id,
                **{cell_name: new_cell_value}
            )
        except Conflict as err:
            logger.error(unicode(err))
            exceptions.handle(
                request,
                _('Host type with name "{0}" already exists.').format(
                    new_cell_value))
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to edit host type "{0}".').format(
                                  datum.name))


class DeleteHostClassAction(TnxBatchActionMixin, tables.DeleteAction):
    """Delete host class(es) action"""

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Host Class",
            u"Delete Host Classes",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Host Class",
            u"Deleted Host Classes",
            count
        )

    def allowed(self, request, datum):
        """
        check permission
        """
        allowed = super(DeleteHostClassAction, self).allowed(request, datum)
        if datum:
            allowed = not datum.is_default and allowed
        return allowed

    def delete(self, request, obj_id):
        TNXHostClassesClient.delete(request, obj_id)
