# coding: utf-8
import logging

from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, tabs
from horizon.utils import memoized

from tionix_dashboard.clients.host_classes import TNXHostClassesClient
from tionix_dashboard.dashboards.tionix.infrastructure import constants
from tionix_dashboard.dashboards.tionix.infrastructure.host_classes import (
    tabs as host_classes_tabs)
from tionix_dashboard.dashboards.tionix.infrastructure.host_classes import (
    forms as host_classes_forms)
from tionix_dashboard.dashboards.tionix.infrastructure.host_classes import (
    tables as host_classes_tables)
from tionix_dashboard.helpers.views import (
    ServerFilterViewMixin, RefererRedirectMixin)

logger = logging.getLogger(__name__)


class DetailHostClassView(ServerFilterViewMixin, tabs.TabbedTableView):
    tab_group_class = host_classes_tabs.HostClassTabs
    template_name = 'horizon/common/_detail.html'
    page_title = _("Host Class Details: {{ host_class.name }}")

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(DetailHostClassView, self).get_context_data(**kwargs)
        host_class = self.get_data()
        context['host_class'] = host_class
        self.kwargs['host_class'] = host_class
        table = host_classes_tables.HostClassesTable(self.request)
        context["url"] = reverse(constants.INFRASTRUCTURE_HOST_CLASSES_TAB)
        context["actions"] = table.render_row_actions(host_class)
        return context

    @memoized.memoized_method
    def get_data(self):
        """Gets host class data"""
        host_class = None
        try:
            host_class = TNXHostClassesClient.get(
                self.request, self.kwargs['host_class_id']
            )
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve host class details.'),
                redirect=reverse(constants.INFRASTRUCTURE_HOST_CLASSES_TAB))
        return host_class

    def get_tabs(self, request, **kwargs):
        kwargs['host_class'] = self.get_data()
        return self.tab_group_class(request, **kwargs)


class CreateHostClassTypeView(forms.ModalFormView):
    form_class = host_classes_forms.CreateHostClassTypeForm
    template_name = 'infrastructure/host_types/create_host_type.html'
    modal_header = _('Create host type')
    modal_id = 'create_host_type'
    submit_label = _('Create host type')
    context_object_name = 'host_type'
    success_url = constants.INFRASTRUCTURE_HOST_CLASS_DETAIL

    def get_context_data(self, **kwargs):
        context = super(CreateHostClassTypeView, self).get_context_data(
            **kwargs)
        context.update(**self.kwargs)
        return context

    def get_success_url(self):
        return reverse(self.success_url, args=(self.kwargs['host_class_id'],))

    def get_initial(self):
        return self.kwargs


class UpdateHostClassTypeView(CreateHostClassTypeView):
    form_class = host_classes_forms.UpdateHostClassTypeForm
    template_name = 'infrastructure/host_types/update_host_type.html'
    modal_header = _('Edit host type')
    modal_id = 'edit_host_type'
    submit_label = _('Edit host type')

    @memoized.memoized_method
    def get_data(self):
        """Gets host class data"""
        host_class_type = None
        try:
            host_class_type = TNXHostClassesClient.get(
                self.request, self.kwargs['host_class_id']
            ).get_host_type(self.kwargs['host_type_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to get host type.'),
                redirect=reverse(constants.INFRASTRUCTURE_HOST_CLASS_DETAIL,
                                 args=(self.kwargs['host_class_id'],)))
        return host_class_type

    def get_initial(self):
        initial = super(UpdateHostClassTypeView, self).get_initial()
        host_class_type = self.get_data()
        initial.update(name=host_class_type.name)
        return initial


class CreateHostClassView(forms.ModalFormView):
    form_class = host_classes_forms.CreateHostClassForm
    template_name = 'infrastructure/host_classes/create_host_class.html'
    modal_header = _('Create host class')
    modal_id = 'create_host_class'
    submit_label = _('Create host class')
    context_object_name = 'host_class'
    success_url = reverse_lazy(constants.INFRASTRUCTURE_HOST_CLASSES_TAB)

    def get_initial(self):
        return self.kwargs


class UpdateHostClassView(RefererRedirectMixin, CreateHostClassView):
    form_class = host_classes_forms.UpdateHostClassForm
    template_name = 'infrastructure/host_classes/update_host_class.html'
    modal_header = _('Edit host class')
    modal_id = 'edit_host_class'
    submit_label = _('Edit host class')

    @memoized.memoized_method
    def get_data(self):
        """Gets host class data"""
        host_class = None
        try:
            host_class = TNXHostClassesClient.get(self.request,
                                                  self.kwargs['host_class_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to get host class.'),
                redirect=reverse(constants.INFRASTRUCTURE_HOST_CLASSES_TAB)
            )
        return host_class

    def get_initial(self):
        initial = super(UpdateHostClassView, self).get_initial()
        host_class = self.get_data()
        initial.update(name=host_class.name)
        return initial

    def get_context_data(self, **kwargs):
        context = super(UpdateHostClassView, self).get_context_data(
            **kwargs)
        context.update(**self.kwargs)
        return context
