# coding: utf-8
# pylint: disable=no-member, method-hidden

import logging

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import messages, tables

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.hypervisors import TNXHypervisorsClient
from tionix_dashboard.dashboards.tionix.infrastructure import constants
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, LinkBatchAction, TnxUpdateAction)

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class HostsFilterAction(TnxFilterAction):

    name = "filter_hosts"
    filter_type = "server"

    filter_choices = (
        ('id', _("ID"), False),
        ('hostname', _("Hostname"), False),
        ('node_title', _("Host title"), False),
        ('tnx_host_class', _("Class"), False),
        ('hypervisor_type', _("Type"), False),
        ('ips', _("IP addresses"), False),
        ('active', _("Active"), False),
        ('description', _("Description"), False)
    )


# noinspection PyAbstractClass
class CreateHostAction(tables.LinkAction):
    """
    Create new host action
    """
    name = 'create_host'
    verbose_name = _('Create host')
    url = constants.INFRASTRUCTURE_HOST_CREATE
    classes = ('ajax-modal',)


# noinspection PyAbstractClass
class EditHostAction(LinkBatchAction):
    """
    Edit host action
    """
    name = 'edit_host'
    single_next_view_name = constants.INFRASTRUCTURE_HOST_EDIT
    multiple_next_view_name = constants.INFRASTRUCTURE_HOST_MULTI_EDIT

    def action_present(self, count):
        return ungettext_lazy(
            u"Edit host",
            u"Edit hosts",
            count
        )

    def action_past(self, count):
        return ungettext_lazy(
            u"Edited host",
            u"Edited hosts",
            count
        )


class HostUpdateRow(tables.Row):
    """ Action update hosts row in grid. """
    ajax = True

    def get_data(self, request, obj_id):
        try:
            host = TNXHypervisorsClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.critical(err)
            raise ObjectDoesNotExist()
        return host


class UpdateHost(TnxUpdateAction):
    """ Action update host info. """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """ Inline update of enabled-cell. """
        result = True
        update_kwargs = {cell_name: new_cell_value}

        try:
            TNXHypervisorsClient.update(request, datum.id, **update_kwargs)
        except Exception as err:
            logger.critical(err)
            messages.error(request, _('Unable to update host.'))
            result = False

        return result


class DeleteHostAction(tables.DeleteAction):
    """Delete host(s) action"""

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Host",
            u"Delete Hosts",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Host",
            u"Deleted Hosts",
            count
        )

    def allowed(self, request, datum):
        allowed = True
        if datum:
            allowed = False if datum.tnx_node_id else allowed
        return allowed

    def delete(self, request, obj_id):
        TNXHypervisorsClient.delete(request, obj_id)
