# coding: utf-8

from django.forms import BooleanField
from django.template import defaultfilters as filters
from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import forms, tables

from tionix_dashboard.dashboards.tionix.infrastructure.constants import (
    INFRASTRUCTURE_HOST_DETAIL)
from tionix_dashboard.dashboards.tionix.infrastructure.hosts import actions
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import TnxPaginationDataTable


def get_ips(hypervisor):
    return [host_type['ip_address']
            for host_type in hypervisor.tnx_ip_addresses or []] or _('-')


def get_host_class_name(hypervisor):
    return (hypervisor.tnx_host_class['name']
            if hypervisor.tnx_host_class else None)


def get_host_type_name(hypervisor):
    return (hypervisor.tnx_host_type['name']
            if hypervisor.tnx_host_type else None)


class HostsTable(TnxPaginationDataTable):
    host_id = tables.Column("id", verbose_name=_("ID"))
    hostname = tables.Column(
        "hypervisor_hostname",
        attrs={'data-type': 'naturalSort'},
        link=INFRASTRUCTURE_HOST_DETAIL,
        verbose_name=_("Hostname")
    )
    node_title = tables.Column(
        "tnx_node_title",
        verbose_name=pgettext_lazy("Infrastructure", "Host title"),
        form_field=EmptyCharField(max_length=100),
        update_action=actions.UpdateHost,
    )
    host_class = tables.Column(
        get_host_class_name,
        verbose_name=_("Class"))
    hypervisor_type = tables.Column(
        get_host_type_name,
        verbose_name=_("Type"))
    ips = tables.Column(
        get_ips,
        verbose_name=_("IP addresses"),
        wrap_list=True,
        filters=(filters.unordered_list,)
    )
    active = tables.Column(
        "tnx_active",
        verbose_name=_('Active'),
        empty_value=False,
        filters=(filters.yesno, filters.capfirst),
        form_field=BooleanField(label=_('Active'), required=False),
        update_action=actions.UpdateHost,
    )
    description = tables.Column(
        "tnx_description",
        verbose_name=_("Description"),
        form_field=forms.CharField(
            max_length=255,
            widget=forms.Textarea(attrs={'rows': 4, 'columns': 4}),
            required=False),
        update_action=actions.UpdateHost,
    )

    class Meta(object):
        name = 'hosts'
        verbose_name = _('Hosts')
        row_class = actions.HostUpdateRow
        table_actions = (actions.HostsFilterAction, actions.CreateHostAction)
        table_actions_menu = (actions.EditHostAction,
                              actions.DeleteHostAction)
        row_actions = (actions.EditHostAction, actions.DeleteHostAction)
