# coding: utf-8
# pylint: disable=no-member, duplicate-code


import mock

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.dashboards.tionix.infrastructure.hosts import (
    actions, tables, views,
    workflows)
from tionix_dashboard.tests.helpers import FakeUser, BaseClientTestCase


class HostsTablesTestCase(BaseClientTestCase):
    def test_host_classes_actions(self):
        hypervisors = self.tnx_hypervisors.list()

        table = tables.HostsTable(self.request, data=hypervisors)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter_hosts', 'create_host', 'edit_host', 'delete'])
        )

    def test_host_classes_columns(self):
        table = tables.HostsTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'host_id', 'hostname', 'node_title',
                    'host_class', 'hypervisor_type', 'ips', 'description',
                    'actions', 'active'])
        )

    def test_power_devices_table_row_actions(self):
        table = tables.HostsTable(self.request)

        hyp_with_node_id = self.tnx_hypervisors.list()[0]

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(hyp_with_node_id)]),
            sorted(['edit_host'])
        )

        hyp_without_node_id = self.tnx_hypervisors.list()[1]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(hyp_without_node_id)]),
            sorted(['edit_host', 'delete'])
        )


class HostsActionsTestCase(BaseClientTestCase):
    @mock.patch.object(TNXHypervisorsClient,
                       'update',
                       mock.Mock(return_value=None))
    def test_host_update_cell_description(self):
        tnx_hypervisor = self.tnx_hypervisors.list()[0]

        action = actions.UpdateHost()

        self.assertTrue(
            action.update_cell(self.request, tnx_hypervisor, tnx_hypervisor.id,
                               'description', 'new_description_text')
        )

    @mock.patch.object(TNXHypervisorsClient,
                       'update',
                       mock.Mock(side_effect=Exception))
    def test_host_update_cell_error(self):
        action = actions.UpdateHost()

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        self.assertFalse(
            action.update_cell(request, None, 10000, 'description', '')
        )


class HostsViewsTestCase(BaseClientTestCase):

    def test_detail_context_data(self):
        host = self.tnx_hypervisors.get(1)

        view = views.DetailView()
        view.request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        kwargs = {
            'host_id': host.id
        }
        view.kwargs = kwargs

        result = view.get_context_data(**kwargs)
        _result_keys = ('host_id', 'tab_group', 'host', 'view', 'actions',
                        'url')

        self.assertEqual(len(result.keys()), len(_result_keys))
        for key in _result_keys:
            self.assertIn(key, result)

        self.assertEqual(result['host'].id, host.id)


class HostsWorkflowsTestCase(BaseClientTestCase):

    @mock.patch('tionix_dashboard.dashboards.tionix.infrastructure.hosts'
                '.workflows.get_host_classes', mock.Mock(return_value=[]))
    def test_create_workflow_field(self):

        workflow = workflows.CreateHost(self.request)

        workflow_fields = []

        for step in workflow.steps:
            workflow_fields.extend(step.action.fields.keys())

        self.assertListEqual(
            sorted(workflow_fields),
            sorted(['tnx_node_name', 'tnx_node_title', 'tnx_condition',
                    'tnx_cpu_amount', 'tnx_ram_mb', 'tnx_hdd_gb',
                    'tnx_description', 'tnx_active', 'tnx_dtc_number',
                    'tnx_row_number', 'tnx_rack_number', 'tnx_unit_number',
                    'tnx_inventory_code', 'tnx_host_class_id',
                    'tnx_host_type_id', 'tnx_ip_addresses', 'available_ips']))

    @mock.patch('tionix_dashboard.dashboards.tionix.infrastructure.hosts'
                '.workflows.get_host_classes', mock.Mock(return_value=[]))
    def test_update_workflow_field(self):

        workflow = workflows.UpdateHost(self.request)

        workflow_fields = []

        for step in workflow.steps:
            workflow_fields.extend(step.action.fields.keys())

        self.assertListEqual(
            sorted(workflow_fields),
            sorted(['tnx_node_name', 'tnx_node_title', 'tnx_condition',
                    'tnx_cpu_amount', 'tnx_ram_mb', 'tnx_hdd_gb',
                    'tnx_description', 'tnx_active', 'tnx_dtc_number',
                    'tnx_row_number', 'tnx_rack_number', 'tnx_unit_number',
                    'tnx_inventory_code', 'tnx_host_class_id',
                    'tnx_host_type_id', 'tnx_ip_addresses', 'available_ips']))

    @mock.patch('tionix_dashboard.dashboards.tionix.infrastructure.hosts'
                '.workflows.get_host_classes', mock.Mock(return_value=[]))
    def test_multi_update_workflow_field(self):

        workflow = workflows.MultiUpdateHost(self.request)

        workflow_fields = []

        for step in workflow.steps:
            workflow_fields.extend(step.action.fields.keys())

        self.assertListEqual(
            sorted(workflow_fields),
            sorted(['tnx_node_title', 'tnx_condition', 'tnx_cpu_amount',
                    'tnx_ram_mb', 'tnx_hdd_gb', 'tnx_description',
                    'tnx_active', 'tnx_dtc_number', 'tnx_row_number',
                    'tnx_rack_number', 'tnx_unit_number', 'tnx_inventory_code',
                    'tnx_host_class_id', 'tnx_host_type_id']))
