# coding: utf-8
# pylint: disable=no-member, arguments-differ

import logging

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs, forms
from horizon.utils import memoized

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.dashboards.tionix.infrastructure import constants
from tionix_dashboard.dashboards.tionix.infrastructure.hosts.forms import (
    CreateIPAddressForm)
from tionix_dashboard.dashboards.tionix.infrastructure.hosts.tabs import (
    HostDetailTabs)
from tionix_dashboard.dashboards.tionix.infrastructure.hosts.tables import (
    HostsTable)
from tionix_dashboard.dashboards.tionix.infrastructure.hosts.workflows import (
    CreateHost, UpdateHost, MultiUpdateHost)
from tionix_dashboard.helpers.views import LinkBatchWorkflowViewMixin
from tionix_dashboard.helpers.workflows import WorkflowVerticalView


logger = logging.getLogger(__name__)


class CreateView(WorkflowVerticalView):
    workflow_class = CreateHost
    success_url = reverse_lazy('horizon:tionix:infrastructure:index')
    page_title = _("Create Host")

    def get(self, request, *args, **kwargs):
        request.session.pop('temp_available_ips', None)
        return super(CreateView, self).get(request, *args, **kwargs)


class UpdateView(LinkBatchWorkflowViewMixin, CreateView):
    workflow_class = UpdateHost
    page_title = _("Edit Host")
    submit_url = constants.INFRASTRUCTURE_HOST_EDIT

    def get(self, request, *args, **kwargs):
        request.session.pop('temp_available_ips', None)
        return super(UpdateView, self).get(request, *args, **kwargs)

    def get_initial(self):
        host_ids = self.get_object_ids()
        initial = {}
        if host_ids:
            initial = {"host_id": host_ids[0]}

            try:
                initial.update(
                    **TNXHypervisorsClient.get(
                        self.request,
                        initial['host_id']).to_dict()
                )
            except Exception as err:
                logger.error(unicode(err))
            else:
                tnx_host_class = initial.get('tnx_host_class')
                if tnx_host_class:
                    initial.update({'tnx_host_class_id': tnx_host_class['id']})
                tnx_host_type = initial.get('tnx_host_type')
                if tnx_host_type:
                    initial.update({'tnx_host_type_id': tnx_host_type['id']})

        return initial


class MultiUpdateView(UpdateView):
    workflow_class = MultiUpdateHost
    page_title = _("Edit Hosts")
    submit_url = constants.INFRASTRUCTURE_HOST_MULTI_EDIT

    def get(self, request, *args, **kwargs):
        request.session.pop('temp_available_ips', None)
        return super(MultiUpdateView, self).get(request, *args, **kwargs)

    def get_initial(self):
        return {"host_ids": self.get_object_ids()}


class CreateIpAddressView(forms.ModalFormView):
    form_class = CreateIPAddressForm
    template_name = 'infrastructure/hosts/create_ip.html'
    modal_header = _('Create IP')
    modal_id = 'create_ip_address'
    submit_label = _('Create IP')
    context_object_name = 'ip_address'
    success_url = reverse_lazy(constants.INFRASTRUCTURE_INDEX)


class DetailView(tabs.TabView):
    tab_group_class = HostDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ host.tnx_node_name }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        host = self.get_data()
        context["host"] = host
        context["url"] = self.get_redirect_url()
        table = HostsTable(self.request)
        context["actions"] = table.render_row_actions(host)
        return context

    @staticmethod
    def get_redirect_url():
        return reverse_lazy(constants.INFRASTRUCTURE_INDEX)

    @memoized.memoized_method
    def get_data(self):
        host = None
        try:
            host = TNXHypervisorsClient.get(
                self.request, self.kwargs['host_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve host details.'),
                              redirect=self.get_redirect_url())

        return host

    def get_tabs(self, request, *args, **kwargs):
        host = self.get_data()
        return self.tab_group_class(request, host=host, **kwargs)
