# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import forms, workflows

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.dashboards.helpers import Empty, coerce_to_int
from tionix_dashboard.dashboards.tionix.infrastructure.constants import (
    INFRASTRUCTURE_HOST_IP_ADD, INFRASTRUCTURE_INDEX)
from tionix_dashboard.dashboards.tionix.infrastructure.helpers import (
    get_host_classes)
from tionix_dashboard.helpers.enums import ConditionTypes
from tionix_dashboard.helpers.fields import NullableCharField
from tionix_dashboard.helpers.views import RefererRedirectMixin
from tionix_dashboard.helpers.workflows import OptionalFieldsActionMixin
from tionix_dashboard.settings import MAX_INT

logger = logging.getLogger(__name__)

COMMON_HORIZONTAL_TEMPLATE = "horizon/common/_common_horizontal_form.html"


def get_condition_choices(include_empty=False):
    choices = ConditionTypes.get_choices()
    if include_empty:
        choices.insert(0, ("", _("Select condition")))
    return sorted(choices, key=lambda choice: unicode(choice[1]))


class CreateHostInfoAction(workflows.Action):

    tnx_node_name = forms.CharField(label=_("Hostname"),
                                    max_length=255)
    tnx_node_title = NullableCharField(
        label=pgettext_lazy("Infrastructure", "Host title"),
        max_length=100,
        required=False)
    tnx_condition = forms.TypedChoiceField(
        label=_("Condition"),
        initial=ConditionTypes.OUT_OF_SERVICE,
        coerce=coerce_to_int,
        help_text=_("Select the condition of operation of the host(s).")
    )
    tnx_cpu_amount = forms.IntegerField(label=_("Total vCPU"),
                                        min_value=0,
                                        max_value=99,
                                        required=False)
    tnx_ram_mb = forms.IntegerField(label=_("Total RAM (MB)"),
                                    min_value=0,
                                    max_value=MAX_INT,
                                    required=False)
    tnx_hdd_gb = forms.IntegerField(label=_("HDD"),
                                    min_value=0,
                                    max_value=MAX_INT,
                                    required=False)
    tnx_description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)
    tnx_active = forms.BooleanField(label=_("Active"),
                                    required=False,
                                    initial=True)

    @staticmethod
    def populate_tnx_condition_choices(request, context):
        return get_condition_choices()

    class Meta(object):
        name = _("Host Information")


class CreateHostInfoStep(workflows.Step):
    action_class = CreateHostInfoAction
    template_name = COMMON_HORIZONTAL_TEMPLATE
    contributes = ("tnx_node_name", "tnx_node_title", "tnx_cpu_amount",
                   "tnx_ram_mb", "tnx_hdd_gb", "tnx_description", "tnx_active",
                   "tnx_condition")


class CreateExtraAttributesAction(workflows.Action):

    tnx_dtc_number = forms.IntegerField(
        label=_("Data Center number"),
        min_value=1,
        max_value=99,
        required=False,
        help_text=_('Input data center number. Must be in the range '
                    'from 1 to 99.'))
    tnx_row_number = forms.IntegerField(
        label=_("Row number"),
        min_value=1,
        max_value=99,
        required=False,
        help_text=_('Input row number. Must be in the range from 1 to 99.'))
    tnx_rack_number = forms.IntegerField(
        label=_("Rack number"),
        min_value=1,
        max_value=99,
        required=False,
        help_text=_('Input rack number. Must be in the range from 1 to 99.'))
    tnx_unit_number = forms.IntegerField(
        label=_("Place in rack"),
        min_value=1,
        max_value=99,
        required=False,
        help_text=_('Input place number in rack. Must be in the range '
                    'from 1 to 99.'))
    tnx_inventory_code = NullableCharField(
        label=_("Inventory number"),
        max_length=128,
        required=False,
        help_text=_('Input inventory number of hypervisor. Inventory number '
                    'can contain letters and numbers, but no more than 128 '
                    'characters.'))

    class Meta(object):
        name = _("Extra Attributes")


class CreateExtraAttributesStep(workflows.Step):
    action_class = CreateExtraAttributesAction
    template_name = COMMON_HORIZONTAL_TEMPLATE
    contributes = ("tnx_dtc_number", "tnx_row_number", "tnx_rack_number",
                   "tnx_unit_number", "tnx_inventory_code")


class CreateClassAndTypeAction(workflows.Action):

    tnx_host_class_id = forms.TypedChoiceField(
        label=_("Host Class"),
        coerce=coerce_to_int,
        widget=forms.ChoiceField.widget(attrs={
            'class': 'filtering',
            'data-slug': 'tnx-host-class'
        })
    )
    tnx_host_type_id = forms.TypedChoiceField(
        label=_("Host Type"),
        coerce=coerce_to_int,
        widget=forms.SelectWidget(
            attrs={'class': 'filterable',
                   'data-filter-on': 'tnx-host-class',
                   'data-filter-attr': 'host-class-id'},
            data_attrs=('host-class-id',),
            transform=lambda x: x.name)
    )

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_tnx_host_class_id_choices(request, context):

        host_classes = [(host_class.id, host_class.name)
                        for host_class in get_host_classes(request)]
        if host_classes:
            host_classes.insert(0, ("", _("Select host class")))
        else:
            host_classes = [("", _("Host classes not found"))]

        return host_classes

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_tnx_host_type_id_choices(request, context):
        host_types = []
        for host_class in get_host_classes(request):
            host_types.extend([
                (host_type.id, Empty(**{'name': host_type.name,
                                        'host-class-id': host_class.id}))
                for host_type in host_class.host_types])
        if host_types:
            host_types.insert(0, ("", Empty(**{'name': _("Select host type"),
                                               'host-class-id': ""})))
        else:
            host_types = [("", _("Host types not found"))]
        return host_types

    class Meta(object):
        name = _("Class and type")


class CreateClassAndTypeStep(workflows.Step):
    action_class = CreateClassAndTypeAction
    template_name = COMMON_HORIZONTAL_TEMPLATE
    contributes = ("tnx_host_class_id", "tnx_host_type_id")


class CreateIpAddressesAction(workflows.Action):

    tnx_ip_addresses = forms.MultipleChoiceField(
        label=_('IP-addresses'),
        required=False
    )
    available_ips = forms.MultipleChoiceField(
        label=_('IP-addresses available'),
        required=False
    )

    def __init__(self, request, context, *args, **kwargs):
        super(CreateIpAddressesAction, self).__init__(
            request, context, *args, **kwargs
        )

        ip_addresses = [(addr['ip_address'], addr['ip_address'])
                        for addr in context.get('tnx_ip_addresses', [])]

        tmp_addresses = [
            (addr, addr)
            for addr in request.session.get('temp_available_ips', [])
        ]
        self.fields['available_ips'].choices = ip_addresses + tmp_addresses

        if request.method == 'POST':
            self.fields['tnx_ip_addresses'].choices = (
                ip_addresses + tmp_addresses
            )
        else:
            self.fields['tnx_ip_addresses'].choices = ip_addresses

    def clean(self):
        self.fields['tnx_ip_addresses'].choices = [
            (ip, ip) for ip in self.cleaned_data['tnx_ip_addresses']
        ]

    class Meta(object):
        name = _("IP-addresses")


class CreateIpAddressesStep(workflows.Step):
    action_class = CreateIpAddressesAction
    template_name = 'infrastructure/hosts/_ip_addresses_step.html'
    controller_name = 'HostsIPAddressesController as ctrl'
    contributes = ("tnx_ip_addresses",)

    @property
    def add_item_link(self):
        return '?host_id='.join([reverse(INFRASTRUCTURE_HOST_IP_ADD),
                                 self.workflow.context_seed.get('host_id',
                                                                '')])


class CreateHost(workflows.Workflow):
    slug = "create_host"
    name = _("Create Host")
    finalize_button_name = _("Create")
    success_message = _('Created host "%s".')
    failure_message = _('Unable to create host "%s".')
    success_url = INFRASTRUCTURE_INDEX
    default_steps = (CreateHostInfoStep,
                     CreateExtraAttributesStep,
                     CreateClassAndTypeStep,
                     CreateIpAddressesStep)

    def format_status_message(self, message):
        return message % self.context.get('tnx_node_name')

    @staticmethod
    def _get_prepared_data(data):
        """Trimming string values because selenium
        with phantomjs sometimes doesn`t trim whitespaces."""
        return {
            key.replace('tnx_', ''): (val.strip()
                                      if isinstance(val, (str, unicode,))
                                      else val)
            for key, val in data.items()
            if key not in ('tnx_node_id',)
        }

    def handle(self, request, context):
        result = True
        data = self._get_prepared_data(context)
        data['owner'] = request.user.id
        try:
            TNXHypervisorsClient.create(request, **data)
        except Exception as err:
            logger.error(unicode(err))
            result = False

        return result


class UpdateHostInfoAction(CreateHostInfoAction):

    def __init__(self, request, context, *args, **kwargs):
        super(UpdateHostInfoAction, self).__init__(
            request, context, *args, **kwargs
        )

        if context.get('tnx_node_id'):
            self.fields['tnx_node_name'].widget.attrs.update(
                **{'readonly': 'readonly'}
            )

    class Meta(object):
        name = _("Host Information")


class UpdateHostInfoStep(CreateHostInfoStep):
    action_class = UpdateHostInfoAction
    depends_on = ("host_id", "tnx_node_id")


class UpdateHost(RefererRedirectMixin, CreateHost):
    slug = "update_host"
    name = _("Edit Host")
    finalize_button_name = _("Save")
    success_message = _('Edited host "%s".')
    failure_message = _('Unable to edit host "%s".')
    success_url = INFRASTRUCTURE_INDEX
    default_steps = (UpdateHostInfoStep,
                     CreateExtraAttributesStep,
                     CreateClassAndTypeStep,
                     CreateIpAddressesStep)

    @staticmethod
    def _update_host(request, host_id, **data):
        result = True
        try:
            TNXHypervisorsClient.update(request, host_id, **data)
        except Exception as err:
            logger.error(unicode(err))
            result = False
        return result

    def handle(self, request, context):
        data = self._get_prepared_data(context)
        host_id = data.pop("host_id")
        return self._update_host(request, host_id, **data)


class MultiUpdateHostInfoAction(OptionalFieldsActionMixin,
                                CreateHostInfoAction):

    def __init__(self, request, context, *args, **kwargs):
        super(MultiUpdateHostInfoAction, self).__init__(
            request, context, *args, **kwargs
        )

        self.fields.pop('tnx_node_name')
        self.fields['tnx_active'].initial = False
        self.fields['tnx_condition'].initial = None

    @staticmethod
    def populate_tnx_condition_choices(request, context):
        return get_condition_choices(include_empty=True)

    class Meta(object):
        name = _("Host Information")


class MultiUpdateHostInfoStep(UpdateHostInfoStep):
    action_class = MultiUpdateHostInfoAction
    contributes = ("tnx_node_title", "tnx_cpu_amount", "tnx_ram_mb",
                   "tnx_hdd_gb", "tnx_description", "tnx_active",
                   "tnx_condition")
    depends_on = ("host_ids",)


class MultiUpdateExtraAttributesAction(OptionalFieldsActionMixin,
                                       CreateExtraAttributesAction):

    def __init__(self, request, context, *args, **kwargs):
        super(MultiUpdateExtraAttributesAction, self).__init__(
            request, context, *args, **kwargs
        )

        self.fields['tnx_inventory_code'].widget = forms.HiddenInput()

    class Meta(object):
        name = _("Extra Attributes")


class MultiUpdateExtraAttributesStep(CreateExtraAttributesStep):
    action_class = MultiUpdateExtraAttributesAction


class MultiUpdateClassAndTypeAction(OptionalFieldsActionMixin,
                                    CreateClassAndTypeAction):
    class Meta(object):
        name = _("Class and type")

    def clean(self):
        cleaned_data = super(MultiUpdateClassAndTypeAction, self).clean()
        tnx_host_class_id = cleaned_data.get('tnx_host_class_id')
        tnx_host_type_id = cleaned_data.get('tnx_host_type_id')
        if tnx_host_class_id and not tnx_host_type_id:
            self.add_error('tnx_host_type_id', _('This field is required.'))
        return cleaned_data


class MultiUpdateClassAndTypeStep(CreateClassAndTypeStep):
    action_class = MultiUpdateClassAndTypeAction


class MultiUpdateHost(UpdateHost):
    slug = "multi_update_host"
    success_message = _('Successfully edited hosts: "{0}".')
    failure_message = _('Unable to edit hosts: "{0}".')
    success_url = INFRASTRUCTURE_INDEX
    default_steps = (MultiUpdateHostInfoStep,
                     MultiUpdateExtraAttributesStep,
                     MultiUpdateClassAndTypeStep)

    def format_status_message(self, message):
        return message

    def get_absolute_url(self):
        return self.request.get_full_path()

    def handle(self, request, context):
        result = True
        data = {
            key.replace('tnx_', ''): val
            for key, val in context.items()
            if val and key not in ('tnx_node_id',)
        }
        host_ids = data.pop("host_ids")
        success_hosts, failed_hosts = [], []
        for host_id in host_ids:

            try:
                host = TNXHypervisorsClient.get(request, host_id)
            except Exception as err:
                logger.error(unicode(err))
                failed_hosts.append(host_id)
            else:
                if self._update_host(request, host_id, **data):
                    success_hosts.append(host.tnx_node_name)
                else:
                    failed_hosts.append(host.tnx_node_name)

        if failed_hosts:
            result = False
            self.failure_message = self.failure_message.format(
                '", "'.join(failed_hosts)
            )

        if success_hosts:
            self.success_message = self.success_message.format(
                '", "'.join(success_hosts)
            )

        return result
