# coding: utf-8

import datetime
import logging
import socket

from django import shortcuts
from django.conf import settings
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import messages
from horizon.tables import DataTableView

from tionix_client.helpers import PACKAGES
from tionix_client.helpers.packages_info import get_tionix_modules_info
from tionix_client.helpers import updates as tc_updates
from tionix_client.functional.writers import TestsRunningException

from tionix_scheduler_client.client import SCHEDULER_URL

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.tionix.licenses import (
    tables, constants)
from tionix_dashboard.dashboards.tionix.licenses.api import SelfDiagnostic
from tionix_dashboard.settings import (
    CHECK_UPDATES_TIMEOUT, MOD_VERSIONS_CACHE_KEY)


logger = logging.getLogger(__name__)


class LicensesIndexView(DataTableView):
    """Main page view."""
    table_class = tables.LicensesTable
    template_name = constants.LICENSES_TEMPLATE_NAME
    page_title = _('Overview')

    def get_data(self):
        nova_client = novaclient(self.request)
        modules = get_tionix_modules_info(
            nova_client,
            scheduler_url=SCHEDULER_URL,
            exclude_modules=[PACKAGES.TIONIX_VDIClient],
            local_distr=False
        )

        installed_packages = {tnx_module.pip_name: tnx_module.version
                              for tnx_module in modules}
        actual_versions = cache.get(MOD_VERSIONS_CACHE_KEY)
        if not actual_versions:
            actual_versions = tc_updates.get_actual_versions(
                installed_packages.keys()
            )
            cache.set(MOD_VERSIONS_CACHE_KEY, actual_versions,
                      CHECK_UPDATES_TIMEOUT)
        updates = tc_updates.check_updates(
            installed_packages, actual=actual_versions
        )

        result = []
        for i, tnx_module in enumerate(modules):
            tnx_module.id = i
            result.append(tnx_module)
            if tnx_module.version:
                actual_version = updates.get(tnx_module.pip_name)
                if tnx_module.pip_name in updates:
                    tnx_module.outdated = True if actual_version else None
                else:
                    tnx_module.outdated = False
                tnx_module.actual_version = actual_version
            else:
                tnx_module.outdated = tnx_module.actual_version = None

        return result


class RunSelfDiagnosticView(generic.View):

    redirect_url = 'horizon:tionix:licenses:index'

    # noinspection PyUnusedLocal
    def get(self, request, **response_kwargs):
        nova_client = novaclient(self.request)
        locale = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME,
                                     request.LANGUAGE_CODE)
        try:
            SelfDiagnostic.run(username=request.user.username,
                               nova_client=nova_client,
                               locale=locale)
        except TestsRunningException:
            messages.error(request, _('Self-diagnostic is running already.'))
        else:
            messages.success(request, _('Self-diagnostic is running.'))

        request.session['self-diagnostic'] = True

        return shortcuts.redirect(reverse(self.redirect_url))


class DownloadSelfDiagnosticView(generic.View):

    template = constants.TIONIX_TESTS_TEMPLATE_NAME

    redirect_url = 'horizon:tionix:licenses:index'

    # noinspection PyUnusedLocal
    def get(self, request, **response_kwargs):
        data = SelfDiagnostic.report(request.user.username)

        if data:
            response = shortcuts.render(
                request,
                self.template,
                {'data': data},
                content_type="text/plain"
            )
            hostname = socket.gethostname()
            response['Content-Disposition'] = (
                'attachment; filename="tionix_tests_{hostname}_{date:'
                '%Y-%m-%d_%H.%M}.txt"'.format(hostname=hostname,
                                              date=datetime.datetime.now())
            )
            response['Content-Length'] = str(len(response.content))
        else:

            messages.error(
                request, _('Self-diagnostic file is not found.')
            )
            response = shortcuts.redirect(reverse(self.redirect_url))

        return response
