# coding: utf-8
# pylint: disable=no-member, unused-variable

"""Metrics api"""

import logging
import operator

from collections import Counter
from datetime import timedelta
from itertools import izip, islice

from json import loads

import pytz

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from novaclient.exceptions import Forbidden, ClientException

from horizon import messages
from horizon.utils.memoized import memoized
from openstack_dashboard import api

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.backup_templates import TNXBackupTemplatesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.zabbix import TNXZabbixClient
from tionix_dashboard.dashboards.tionix.metrics.enums import (
    MetricsTypes, MetricsEnum)
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.helpers.utils import get_datetime_from_string


logger = logging.getLogger(__name__)


def format_meter_data(items, tz):
    """Format meters data for output"""
    interval_step = 5

    def round_interval(interval):
        """Round interval between points by interval step"""
        return int(round(float(interval) / interval_step)) * interval_step

    def get_metric_interval(points):
        """Find most common interval between points"""

        interval = None
        if len(points) > 1:
            counter = Counter((
                round_interval(abs(p1[0] - p0[0]).seconds)
                for p0, p1 in izip(points, islice(points, 1, len(points)))))
            (interval, _count), = counter.most_common(1)
        return interval

    label_format = '%H:%M:%S'
    data = []

    for item in items:
        date_string, value = item
        utc_date = get_datetime_from_string(date_string)
        date = pytz.utc.localize(utc_date, is_dst=None).astimezone(tz)
        if date:
            data.append((date, round(value, 2)))

    metric_interval = get_metric_interval(data)
    formatted_data = []
    prev_x = None

    for x, y in sorted(data, key=operator.itemgetter(0)):
        if prev_x is not None and metric_interval:
            delta = (x - prev_x).seconds
            if delta >= metric_interval * 2:
                cur_x = prev_x
                while delta > metric_interval:
                    cur_x += timedelta(seconds=metric_interval)
                    formatted_data.append((cur_x.strftime(label_format), None))
                    delta -= metric_interval

        formatted_data.append((x.strftime(label_format), y))
        prev_x = x

    return formatted_data


def get_instance_metric_by_id(request, instance_id, project_id=None):
    """Return metrics for instance"""

    result, metrics, metrics_names = {}, None, []

    try:
        user_metrics = UserMetrics.objects.get(
            user_id=request.user.id,
            metric_type=MetricsTypes.INSTANCE
        ).metrics_names
    except ObjectDoesNotExist:
        user_metrics = [MetricsEnum.CPU_UTIL, MetricsEnum.MEMORY_USAGE]

    try:
        metrics_names = TNXServersClient.get_metrics_names(request)
    except UrlNotAvailableError:
        raise
    except Exception as err:
        logger.error(unicode(err))
        messages.error(request, _('Unable to get available metrics list.'))

    try:
        instance_data = TNXServersClient.get_instance_metric(
            request, instance_id, project_id=project_id
        )
    except UrlNotAvailableError:
        raise
    except Exception as err:
        logger.error(unicode(err))
        messages.error(request, _('Unable to get metrics data.'))
    else:
        metrics = instance_data.get('metrics')

    metrics = loads(metrics) if metrics else {}

    tz = pytz.timezone(
        request.COOKIES.get('django_timezone', settings.TIME_ZONE))

    unavailable_metrics = []
    for user_metric in user_metrics:
        metric_name = unicode(MetricsEnum.values.get(user_metric))
        if user_metric not in metrics_names:
            unavailable_metrics.append(metric_name)
            continue
        line_data = {}
        metric = metrics.get(user_metric, {})
        if metric:
            data = format_meter_data(metric.get('items', []), tz)
            labels, points = zip(*data)
            line_data = {
                'labels': labels,
                'data': [{'label': metric_name,
                          'fill': False,
                          'borderColor': 'blue',
                          'backgroundColor': 'blue',
                          'pointBorderColor': 'blue',
                          'data': points}]
            }

        result[user_metric] = {
            'id': user_metric,
            'name': metric_name,
            'data': line_data
        }

    if unavailable_metrics:
        delete_url = reverse(
            'horizon:tionix:metrics:delete_unavailable_metrics',
            args=(MetricsTypes.INSTANCE,)
        )
        msg = _('Following metrics are not found in TIONIX.Monitor '
                'configuration file: "{0}". You can '
                '<a href="{1}">delete</a> unavailable '
                'metrics.').format('", "'.join(unavailable_metrics),
                                   delete_url)
        messages.error(request, msg, extra_tags='safe')

    return result


@memoized
def is_ceilometer_available(request):
    result = True
    try:
        TNXServersClient.get_metrics_names(request)
    except Forbidden:
        result = False
    except ClientException:
        pass
    except Exception as err:
        logger.error(unicode(err))
        result = False
    return result


@memoized
def is_zabbix_available(request):
    result = True
    try:
        TNXZabbixClient.get_templates(request, request.user.tenant_id)
    except Forbidden:
        result = False
    except ClientException:
        pass
    except Exception as err:
        logger.error(unicode(err))
        result = False
    return result


@memoized
def is_bareos_available(request):
    result = True

    domain_id = api.keystone.get_default_domain(request,
                                                get_name=False).get('id')

    try:
        TNXBackupTemplatesClient.list(request,
                                      search_opts={'domain_id': domain_id})
    except Forbidden:
        result = False
    except ClientException:
        pass
    except Exception as err:
        logger.error(unicode(err))
        result = False
    return result
