# coding: utf-8

from django.utils.translation import (
    ugettext_lazy as _, pgettext_lazy, ungettext_lazy, npgettext_lazy)

from horizon import tables
from horizon.utils import memoized

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.backup_templates import TNXBackupTemplatesClient
from tionix_dashboard.dashboards.tionix.metrics import constants
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxBatchActionMixin)


# noinspection PyAbstractClass
class TnxMetricsBareosTemplatesFilterAction(TnxFilterAction):
    """
    .. attribute:: filter_string_modifiers

        Modifies filter field value to make api filtering.
    """
    filter_type = "server"

    filter_choices = (('name', _("Name"), False),
                      ('description', _("Description"), False),
                      ('level', _("Level"), False),
                      ('job_defs', _("JobDefs"), False),
                      ('file_set', _("FileSet"), False),
                      ('schedule',
                       pgettext_lazy("Bareos templates", u"Schedule"), False))


# noinspection PyAbstractClass
class CreateBareosTemplateAction(tables.LinkAction):
    """Create bareos template action. """
    name = 'create'
    verbose_name = _('Create Bareos Template')
    url = constants.METRICS_BAREOS_TEMPLATE_CREATE
    classes = ('ajax-modal',)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)


class DeleteBareosTemplateAction(TnxBatchActionMixin, tables.DeleteAction):
    """Delete bareos template(s) action"""

    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def templates_with_instances(self, request):
        return set()

    # pylint: disable=method-hidden
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Bareos Template",
            u"Delete Bareos Templates",
            count
        )

    # pylint: disable=method-hidden
    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Bareos Template",
            u"Deleted Bareos Templates",
            count
        )

    def allowed(self, request, datum=None):
        if not datum or datum.assigned_servers:
            self.help_text = _("This action cannot be undone. All "
                               "assignments will be deleted.")
        else:
            self.help_text = _("This action cannot be undone.")

        return super(DeleteBareosTemplateAction, self).allowed(request,
                                                               datum=datum)

    def get_success_message(self, action_success):
        msg = super(DeleteBareosTemplateAction, self).get_success_message(
            action_success
        )
        templates_with_instances = self.templates_with_instances(
            self.table.request
        )
        if templates_with_instances:
            msg = u" ".join([
                msg,
                _(u'Deleted assignments for following templates: '
                  u'"%s".') % (u'", "'.join(templates_with_instances))
            ])
        return msg

    def delete(self, request, obj_id):
        obj = self.table.get_object_by_id(obj_id)
        for instance_id in obj.assigned_servers:
            self.templates_with_instances(request).add(
                self.table.get_object_display(obj) or obj_id
            )
            TNXServersClient.remove_backup_templates(request,
                                                     instance_id,
                                                     [obj_id])
        TNXBackupTemplatesClient.delete(request, obj_id)


class UnassignInstancesAction(TnxBatchActionMixin, tables.DeleteAction):
    """Unassign instances action"""

    name = "unassign"
    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    # pylint: disable=method-hidden
    @staticmethod
    def action_present(count):
        return npgettext_lazy(
            "Bareos templates",
            u"Unassign",
            u"Unassign",
            count
        )

    # pylint: disable=method-hidden
    @staticmethod
    def action_past(count):
        return npgettext_lazy(
            "Bareos templates",
            u"Unassigned",
            u"Unassigned",
            count
        )

    def allowed(self, request, datum=None):
        allowed = super(UnassignInstancesAction, self).allowed(request,
                                                               datum=datum)
        if datum:
            allowed &= bool(datum.assigned_servers)
        return allowed

    def delete(self, request, obj_id):
        obj = self.table.get_object_by_id(obj_id)
        for instance_id in obj.assigned_servers:
            TNXServersClient.remove_backup_templates(request,
                                                     instance_id,
                                                     [obj_id])
