# coding: utf-8
import logging

from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import workflows, forms, exceptions
from openstack_dashboard import api

from tionix_dashboard.clients.backup_templates import TNXBackupTemplatesClient
from tionix_dashboard.dashboards.helpers import coerce_to_int
from tionix_dashboard.dashboards.tionix.metrics import constants
from tionix_dashboard.dashboards.tionix.metrics.bareos_templates import helpers
from tionix_dashboard.dashboards.tionix.metrics.enums import LevelTypes
from tionix_dashboard.helpers.fields import AutoGenNameCharField

COMMON_HORIZONTAL_TEMPLATE = "horizon/common/_common_horizontal_form.html"
TRANSFER_TABLE_STEP_TEMPLATE = 'tionix_dashboard/_transfer_table_step.html'

logger = logging.getLogger(__name__)


class CreateDetailsAction(workflows.Action):

    name = AutoGenNameCharField(label=_('Template name'), max_length=100)
    level = forms.TypedChoiceField(
        label=_("Level"),
        initial=LevelTypes.INCREMENTAL,
        choices=LevelTypes.get_choices(),
        coerce=coerce_to_int)
    description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)

    class Meta(object):
        name = pgettext_lazy("Bareos templates", u"Details")
        help_text = _("Please provide the Bareos template's name, "
                      "the level and description")


class CreateDetailsStep(workflows.Step):
    action_class = CreateDetailsAction
    template_name = COMMON_HORIZONTAL_TEMPLATE
    contributes = ("name", "level", "description")

    def _verify_contributions(self, context):
        for key in self.contributes:
            # Make sure we don't skip steps based on weird behavior of
            # POST query dicts.
            field = self.action.fields.get(key, None)
            if (field and field.required and
                    context.get(key) in field.empty_values):
                context.pop(key, None)
        failed_to_contribute = set(self.contributes)
        failed_to_contribute -= set(context.keys())
        if failed_to_contribute:
            raise exceptions.WorkflowError("The following expected data was "
                                           "not added to the workflow context "
                                           "by the step %s: %s."
                                           % (self.__class__,
                                              failed_to_contribute))
        return True


class CreateJobDefsAction(workflows.Action):

    job_defs = forms.TypedChoiceField(label=_("JobDefs"))

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_job_defs_choices(request, context):
        jobdefs = [(jobdef, jobdef)
                   for jobdef in helpers.get_bareos_jobdefs(request)]
        jobdefs.insert(0, ("", _("Select jobDefs")))
        return jobdefs

    def clean(self):
        cleaned_data = super(CreateJobDefsAction, self).clean()
        if not cleaned_data.get('job_defs'):
            self.add_error(None, _('JobDefs must be selected.'))
        return cleaned_data

    class Meta(object):
        name = _("JobDefs")
        slug = 'jobdefs'


class CreateJobDefsStep(workflows.Step):
    action_class = CreateJobDefsAction
    template_name = TRANSFER_TABLE_STEP_TEMPLATE
    controller_name = 'BareosTemplateJobDefsController as ctrl'
    contributes = ("job_defs",)


class CreateFileSetAction(workflows.Action):

    file_set = forms.TypedChoiceField(
        label=_("FileSet"),
        required=False,
        empty_value=None
    )

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_file_set_choices(request, context):
        filesets = [(fileset, fileset)
                    for fileset in helpers.get_bareos_filesets(request)]
        filesets.insert(0, ("", _("Select fileSet")))
        return filesets

    class Meta(object):
        name = _("FileSet")
        slug = 'fileset'


class CreateFileSetStep(workflows.Step):
    action_class = CreateFileSetAction
    template_name = TRANSFER_TABLE_STEP_TEMPLATE
    controller_name = 'BareosTemplateFileSetController as ctrl'
    contributes = ("file_set",)


class CreateScheduleAction(workflows.Action):

    schedule = forms.TypedChoiceField(
        label=pgettext_lazy("Bareos templates", u"Schedule"),
        required=False,
        empty_value=None
    )

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_schedule_choices(request, context):
        schedules = [(schedule, schedule)
                     for schedule in helpers.get_bareos_schedules(request)]
        schedules.insert(0, ("", _("Select schedule")))
        return schedules

    class Meta(object):
        name = pgettext_lazy("Bareos templates", u"Schedule")
        slug = 'schedule'


class CreateScheduleStep(workflows.Step):
    action_class = CreateScheduleAction
    template_name = TRANSFER_TABLE_STEP_TEMPLATE
    controller_name = 'BareosTemplateScheduleController as ctrl'
    contributes = ("schedule",)


class CreateBareosTemplateWorkflow(workflows.Workflow):
    slug = "create_bareos_template"
    name = _("Create Bareos Template")
    finalize_button_name = _("Create")
    success_message = _('Created Bareos template "%s".')
    failure_message = _('Unable to create Bareos template "%s".')
    success_url = constants.METRICS_BAREOS_TEMPLATE_INDEX
    default_steps = (CreateDetailsStep,
                     CreateJobDefsStep,
                     CreateFileSetStep,
                     CreateScheduleStep)

    def format_status_message(self, message):
        return message % self.context.get('name', '')

    def handle(self, request, context):
        result = True

        domain_id = api.keystone.get_default_domain(self.request,
                                                    get_name=False).id

        try:
            TNXBackupTemplatesClient.create(request,
                                            domain_id=domain_id,
                                            **context)
        except Exception as err:
            logger.error(unicode(err))
            result = False
        return result
