# coding: utf-8
# pylint: disable=no-member

import json

import mock

from django.core.urlresolvers import reverse
from tionix_tests.helpers import get_fake_request, Empty

from horizon import exceptions
from openstack_dashboard.test import helpers as test
from openstack_dashboard import api

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.metrics.enums import MetricsTypes
from tionix_dashboard.dashboards.tionix.metrics.instances import (
    actions, tables, views, tabs
)
from tionix_dashboard.dashboards.tionix.metrics.constants import METRICS_INDEX
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    MetricsInstanceShowMetricAction)
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.tests.helpers import BaseClientTestCase, FakeUser


class TionixMetricsActionsTest(test.TestCase):

    def test_metrics_instance_filter(self):

        filter_action = actions.MetricsInstanceFilterAction()

        self.assertListEqual(
            sorted(zip(*filter_action.filter_choices)[0]),
            sorted(['tenant', 'name', 'host', 'tnx_node_title', 'tnx_type',
                    'ip', 'ip6', 'task', 'status', 'power_state'])
        )

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=True))
    def test_metrics_instances_show_allowed(self):

        action = actions.MetricsInstanceShowMetricAction()

        self.assertTrue(action.allowed(self.request, None))

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=False))
    def test_metrics_instances_show_not_allowed(self):

        action = actions.MetricsInstanceShowMetricAction()

        self.assertFalse(action.allowed(self.request, None))

    def test_metrics_instances_show_get_link(self):
        instance = self.servers.first()

        action = actions.MetricsInstanceShowMetricAction(
            table=tables.MetricsInstancesTable(self.request)
        )

        self.assertEqual(
            action.get_link_url(instance),
            u'/tionix/metrics/{0}_{1}/'.format(instance.id, instance.tenant_id)
        )

    @mock.patch.object(api.keystone,
                       'is_domain_admin',
                       mock.Mock(return_value=False))
    def test_metrics_instances_settings_not_allowed(self):
        action = actions.MetricsInstanceSettingsAction()

        self.assertFalse(action.allowed(self.request, None))


class TionixMetricsTablesTest(BaseClientTestCase):

    def test_metrics_instances_table(self):

        table = tables.MetricsInstancesTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'tenant', 'name', 'host', 'node_title',
                    'type', 'ip', 'task', 'status', 'state', 'actions'])
        )

    def test_metrics_instances_table_actions(self):
        instance = self.servers.list()[0]

        table = tables.MetricsInstancesTable(self.request, data=[instance])

        self.assertListEqual(
            [action.name for action in table.get_table_actions()],
            ['filter_metrics_instances', 'settings_instance_metrics',
             'manage_template']
        )

    @mock.patch.object(MetricsInstanceShowMetricAction,
                       'get_link_url',
                       mock.Mock(return_value=''))
    def test_metrics_instances_table_row_actions(self):
        table = tables.MetricsInstancesTable(self.request)

        instance = self.tnx_servers.list()[0]

        self.assertListEqual(
            [action.name for action in table.get_row_actions(instance)],
            ['show_instance_metrics', 'manage_template']
        )

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=True))
    def test_metrics_instances_full_url(self):

        url = '{0}?test=data'.format(reverse(METRICS_INDEX))

        res = self.client.get(url)

        tab = res.context_data['tab_group'].get_tab('instances_metrics_tab')
        table = tab.data['instances_metrics_table']

        self.assertEqual(
            table.get_full_url(),
            u'/tionix/metrics/'
        )


class TionixMetricsViewsTest(BaseClientTestCase):

    def tearDown(self):
        super(TionixMetricsViewsTest, self).tearDown()
        UserMetrics.objects.all().delete()

    @mock.patch.object(
        TNXServersClient,
        'get_instance_metric',
        mock.Mock(return_value={
            'message': u'OK',
            'metrics': u'{"disk.read.bytes.rate": {"items": '
                       u'[["2017-03-22T11:27:27.554000", 29.0]], '
                       u'"type": "cumulative", "unit": "B", "interval": "60"},'
                       u'"cpu_util": {"items": [["2017-03-22T11:27:27.291000",'
                       u'3.08]], "type": "gauge", "unit": "%", '
                       u'"interval": "60"}, "disk.write.bytes.rate": {"items":'
                       u'[["2017-03-22T11:27:27.372000", 28.0]], '
                       u'"type": "cumulative", "unit": "B", "interval": "60"},'
                       u'"network.incoming.packets.rate": {"items": '
                       u'[["2017-03-22T11:27:27.291000", 3.082715849239197]], '
                       u'"type": "gauge", "unit": "%", "interval": "60"}, '
                       u'"network.outgoing.packets.rate": {"items": '
                       u'[["2017-03-22T11:27:27.291000", 3.082715849239197]], '
                       u'"type": "gauge", "unit": "%", "interval": "60"}, '
                       u'"memory.usage": {"items": '
                       u'[["2017-03-22T11:27:27.291000", 3.082715849239197]], '
                       u'"type": "gauge", "unit": "%", "interval": "60"}}'
        }))
    @mock.patch.object(TNXServersClient,
                       'get_metrics_names',
                       mock.Mock(return_value=['cpu_util',
                                               'disk.read.bytes.rate']))
    def test_get_metrics_data(self):
        instance = self.tnx_servers.list()[0]

        self.request.COOKIES['django_timezone'] = 'UTC'
        UserMetrics.objects.create(user_id=self.request.user.id,
                                   metric_type=MetricsTypes.INSTANCE,
                                   metrics_names=[u'disk.read.bytes.rate',
                                                  u'cpu_util'])

        tab = tabs.InstanceMetricsTab(
            tab_group=Empty(kwargs={'instance_id': instance.id}),
            request=self.request
        )

        context = tab.get_context_data(self.request)

        self.assertJSONEqual(
            context['data'],
            json.dumps(
                {
                    'disk.read.bytes.rate': {
                        'data': {
                            'labels': [u'11:27:27'],
                            'data': [
                                {'borderColor': u'blue',
                                 'pointBorderColor': u'blue',
                                 'label': u'Rate of reads, B/s',
                                 'backgroundColor': u'blue',
                                 'data': [29.0],
                                 'fill': False}
                            ]
                        },
                        'id': u'disk.read.bytes.rate',
                        'name': u'Rate of reads, B/s'
                    },
                    'cpu_util': {
                        'data': {
                            'labels': [u'11:27:27'],
                            'data': [
                                {'borderColor': u'blue',
                                 'pointBorderColor': u'blue',
                                 'label': u'CPU utilization, %',
                                 'backgroundColor': u'blue',
                                 'data': [3.08],
                                 'fill': False}
                            ]
                        },
                        'id': u'cpu_util',
                        'name': u'CPU utilization, %'
                    }
                }
            )
        )

    @mock.patch.object(TNXServersClient,
                       'get_metrics_names',
                       mock.Mock(return_value=[]))
    @mock.patch.object(TNXServersClient,
                       'get_instance_metric',
                       mock.Mock(return_value={'metrics': []}))
    def test_get_metrics_data_unavailable_metrics(self):
        instance = self.tnx_servers.list()[0]

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(tenant_id=None, id=1),
            horizon={
                'async_messages': []
            },
            COOKIES={},
        )

        UserMetrics.objects.create(user_id=request.user.id,
                                   metric_type=MetricsTypes.INSTANCE,
                                   metrics_names=[u'disk.read.bytes.rate'])

        tab = tabs.InstanceMetricsTab(
            tab_group=Empty(kwargs={'instance_id': instance.id}),
            request=request
        )

        tab.get_context_data(request)

        self.assertEqual(
            request.horizon['async_messages'][0][1],
            u'Following metrics are not found in TIONIX.Monitor '
            u'configuration file: "Rate of reads, B/s". You can '
            u'<a href="/tionix/metrics/1/delete">delete</a> unavailable '
            u'metrics.'
        )

    @mock.patch.object(TNXServersClient, 'get_metrics_names')
    def test_instance_settings(self, get_metrics_names):

        metrics = [u'metric 1', u'metric 2']

        get_metrics_names.return_value = metrics

        view = views.MetricsInstanceSettingsView()
        view.request = self.request

        self.assertDictEqual(
            view.get_initial().dict(),
            {u'metric_type': 1,
             u'allocated_members': [],
             u'available_members': metrics}
        )

    @mock.patch.object(TNXServersClient, 'get_metrics_names')
    def test_instance_settings_with_user_metrics(self, get_metrics_names):

        metrics = [u'metric 1', u'metric 2', u'metric 3']

        get_metrics_names.return_value = metrics

        UserMetrics.objects.create(user_id=self.request.user.id,
                                   metric_type=MetricsTypes.INSTANCE,
                                   metrics_names=metrics[:2])

        view = views.MetricsInstanceSettingsView()
        view.request = self.request

        self.assertDictEqual(
            view.get_initial().dict(),
            {u'metric_type': MetricsTypes.INSTANCE,
             u'allocated_members': metrics[:2],
             u'available_members': metrics}
        )

    @mock.patch.object(TNXServersClient, 'get_metrics_names')
    def test_instance_settings_no_metrics(self, get_metrics_names):

        get_metrics_names.return_value = []

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(tenant_id=None),
            horizon={
                'async_messages': []
            }
        )

        view = views.MetricsInstanceSettingsView()
        view.request = request

        with self.assertRaises(exceptions.Http302):
            view.get_initial()

    @mock.patch.object(TNXServersClient, 'get_metrics_names',
                       mock.Mock(side_effect=Exception))
    def test_instance_settings_error(self):

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(tenant_id=None),
            horizon={
                'async_messages': []
            }
        )

        view = views.MetricsInstanceSettingsView()
        view.request = request

        with self.assertRaises(exceptions.Http302):
            view.get_initial()

    def test_instance_add_templates(self):
        instances = self.tnx_servers.list()[:2]
        instances_ids = ['{}_{}'.format(instance.tenant_id, instance.id)
                         for instance in instances]

        view = views.MetricsManageTemplateView()
        view.request = get_fake_request(
            is_ajax=True,
            get_params={'id': instances_ids},
            user=FakeUser(tenant_id=None),
            horizon={
                'async_messages': []
            }
        )

        initial = view.get_initial()
        initial_instances = initial.pop('instances')

        self.assertEqual(len(instances), len(initial_instances))
        self.assertDictEqual(
            initial,
            {'project_id': '1',
             'id': instances_ids}
        )

    def test_instance_add_templates_multi_projects(self):
        instances = self.tnx_servers.list()[7:9]
        instances_ids = ['{}_{}'.format(instance.tenant_id, instance.id)
                         for instance in instances]

        view = views.MetricsManageTemplateView()
        view.request = get_fake_request(
            is_ajax=True,
            get_params={'id': instances_ids},
            user=FakeUser(tenant_id=None),
            horizon={
                'async_messages': []
            }
        )

        with self.assertRaises(exceptions.Http302):
            view.get_initial()

        self.assertEqual(view.request.horizon['async_messages'][0][1],
                         u'Chosen instances are from different projects.')
