# coding: utf-8
# pylint: disable=no-member
import json
import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse_lazy, reverse
from django.utils.translation import ugettext_lazy as _
from openstack_dashboard import api

from horizon import exceptions, views
from horizon import messages
from horizon import workflows
from horizon.utils import memoized

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.tionix.metrics import constants
from tionix_dashboard.dashboards.tionix.metrics.api import (
    get_instance_metric_by_id)
from tionix_dashboard.dashboards.tionix.metrics.constants import METRICS_INDEX
from tionix_dashboard.dashboards.tionix.metrics.enums import MetricsTypes
from tionix_dashboard.dashboards.tionix.metrics.instances.workflows import (
    MetricsManageTemplateWorkflow)
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.dashboards.tionix.metrics.workflows import (
    MetricsSettingsWorkflow)
from tionix_dashboard.dashboards.utils import TABLE_OBJECT_ID_SEP
from tionix_dashboard.helpers.views import LinkBatchWorkflowViewMixin

logger = logging.getLogger(__name__)


class InstanceMetricsView(views.HorizonTemplateView):
    template_name = 'metrics/instances/metrics.html'
    page_title = "{{ instance.name }}"

    def get_context_data(self, **kwargs):
        context = super(InstanceMetricsView, self).get_context_data(**kwargs)
        instance = self.get_data()
        context["instance"] = instance
        context['data'] = json.dumps(
            get_instance_metric_by_id(self.request,
                                      instance.id,
                                      project_id=instance.tenant_id)
        )
        return context

    @memoized.memoized_method
    def get_data(self):
        instance = project_id = None
        instance_id = self.kwargs['instance_id']
        if TABLE_OBJECT_ID_SEP in instance_id:
            project_id, instance_id = instance_id.split(TABLE_OBJECT_ID_SEP)
        try:
            instance = TNXServersClient.get(self.request,
                                            instance_id,
                                            project_id=project_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(err)
            exceptions.handle(self.request,
                              _('Unable to retrieve instance details.'),
                              redirect=reverse(METRICS_INDEX))
        return instance


class MetricsInstanceSettingsView(workflows.WorkflowView):
    workflow_class = MetricsSettingsWorkflow
    success_url = reverse_lazy(constants.METRICS_INDEX)
    page_title = _("Settings")

    def get_initial(self):
        initial = super(MetricsInstanceSettingsView, self).get_initial()

        try:
            metrics_names = TNXServersClient.get_metrics_names(self.request)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to get available metrics list.'))
            raise exceptions.Http302(self.success_url)
        else:
            if not metrics_names:
                messages.error(self.request, _('No metrics found.'))
                raise exceptions.Http302(self.success_url)

        try:
            user_metrics_names = UserMetrics.objects.get(
                user_id=self.request.user.id,
                metric_type=MetricsTypes.INSTANCE
            ).metrics_names
        except ObjectDoesNotExist:
            user_metrics_names = []

        initial.update(metric_type=MetricsTypes.INSTANCE,
                       available_members=metrics_names,
                       allocated_members=user_metrics_names)
        return initial


class MetricsManageTemplateView(LinkBatchWorkflowViewMixin,
                                workflows.WorkflowView):
    workflow_class = MetricsManageTemplateWorkflow
    success_url = reverse_lazy(constants.METRICS_INDEX)
    submit_url = reverse_lazy(constants.METRICS_INSTANCE_MANAGE_TEMPLATE)
    page_title = _("Manage templates")

    @memoized.memoized_method
    def get_objects(self):
        instances, projects_ids = [], set()
        is_domain_admin = api.keystone.is_domain_admin(self.request)
        for instance_id in self.get_object_ids():
            project_id, instance_id = instance_id.split(TABLE_OBJECT_ID_SEP)
            projects_ids.add(project_id)
            if len(projects_ids) > 1:
                messages.error(
                    self.request,
                    _('Chosen instances are from different projects.')
                )
                raise exceptions.Http302(self.success_url)

            kwargs = {}
            if not is_domain_admin:
                kwargs = {'project_id': project_id}
            try:
                instance = TNXServersClient.get(self.request,
                                                instance_id,
                                                **kwargs)
            except Exception as err:
                logger.error(err.message)
                continue

            instances.append(instance)

            try:
                nova_client = novaclient(self.request, **kwargs)
                instance.host = nova_client.tnx_zabbix.get_host(instance_id)
            except Exception as err:
                logger.error(err.message)

        return instances

    def get_initial(self):
        initial = super(MetricsManageTemplateView, self).get_initial()
        instances = self.get_objects()
        initial.update(
            project_id=instances[0].tenant_id if instances else None,
            instances=instances
        )

        return initial
