# coding: utf-8
# pylint: disable=no-member

import logging

from django.utils.translation import ugettext_lazy as _, ungettext

from horizon import workflows, exceptions, messages
from horizon.templatetags.horizon import has_permissions

from openstack_dashboard import api

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.backup_templates import TNXBackupTemplatesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.zabbix import TNXZabbixClient
from tionix_dashboard.dashboards.helpers import coerce_to_int
from tionix_dashboard.dashboards.tionix.metrics import constants
from tionix_dashboard.dashboards.tionix.metrics.api import (
    is_zabbix_available, is_bareos_available)
from tionix_dashboard.dashboards.tionix.metrics.workflows import (
    BaseMembershipAction)

logger = logging.getLogger(__name__)


class SkipUpdatingException(Exception):
    pass


class MetricsZabbixTemplatesAction(BaseMembershipAction):

    def _get_choices(self):
        project_id = self.initial.get('project_id')
        try:
            templates = [
                (template['templateid'], template['name'])
                for template in TNXZabbixClient.get_templates(self.request,
                                                              project_id)]
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to get available Zabbix templates. '
                  '{0}').format(unicode(err)))
            templates = []

        return templates

    def _get_initial_choices(self):
        instance_templates, instances = [], self.initial.get('instances', [])
        if len(instances) == 1:
            host = getattr(instances[0], 'host', None)
            if host:
                instance_templates = [template['templateid']
                                      for template in host['parentTemplates']]
        return instance_templates

    class Meta(object):
        name = _("Zabbix templates")


class MetricsZabbixTemplatesStep(workflows.UpdateMembersStep):
    action_class = MetricsZabbixTemplatesAction
    available_list_title = _("All available Zabbix templates")
    members_list_title = _("Selected Zabbix templates")
    no_available_text = _("No Zabbix templates found.")
    no_members_text = _("No Zabbix template selected.")
    show_roles = False
    move_all_allowed = False
    contributes = ("available_members", "allocated_members")
    depends_on = ("instances", "project_id")

    def contribute(self, data, context):
        request = self.workflow.request
        if data:
            field_name = self.get_member_field_name('member')
            context["allocated_zabbix_members"] = (
                request.POST.getlist(field_name)
            )
        return context


class MetricsBareosTemplatesAction(BaseMembershipAction):

    def _get_choices(self):
        domain_id = api.keystone.get_default_domain(self.request,
                                                    get_name=False).id
        try:
            templates = [
                (template.id, template.name)
                for template in TNXBackupTemplatesClient.list(
                    self.request, search_opts={'domain_id': domain_id})]
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to get available Bareos templates. '
                  '{0}').format(unicode(err)))
            templates = []

        return templates

    def _get_initial_choices(self):
        instance_templates, instances = [], self.initial.get('instances', [])
        if len(instances) == 1:
            tnx_backup_templates = instances[0].tnx_backup_templates or []
            instance_templates = [
                template['id'] for template in tnx_backup_templates]
        return instance_templates

    class Meta(object):
        name = _("Bareos templates")


class MetricsBareosTemplatesStep(workflows.UpdateMembersStep):
    action_class = MetricsBareosTemplatesAction
    available_list_title = _("All available Bareos templates")
    members_list_title = _("Selected Bareos templates")
    no_available_text = _("No Bareos templates found.")
    no_members_text = _("No Bareos template selected.")
    show_roles = False
    move_all_allowed = False
    contributes = ("available_members", "allocated_members")
    depends_on = ("instances", "project_id")

    def contribute(self, data, context):
        request = self.workflow.request
        if data:
            field_name = self.get_member_field_name('member')
            context["allocated_bareos_members"] = (
                [coerce_to_int(i) for i in request.POST.getlist(field_name)]
            )
        return context


class MetricsManageTemplateWorkflow(workflows.Workflow):
    slug = "metrics_add_template"
    name = _("Manage templates")
    success_message = _("Templates were successfully updated.")
    failure_message = _("Unable to update templates.")
    success_url = constants.METRICS_INDEX

    def __init__(self, request, *args, **kwargs):

        if is_zabbix_available(request) and api.keystone.is_domain_admin(
                request):
            self.default_steps += (MetricsZabbixTemplatesStep,)
        if is_bareos_available(request):
            self.default_steps += (MetricsBareosTemplatesStep,)

        super(MetricsManageTemplateWorkflow, self).__init__(
            request, *args, **kwargs
        )

    def _gather_steps(self):
        self._ordered_steps = []
        for default_step in self.default_steps:
            self.register(default_step)
            default_step_instance = default_step(self)
            self._registry[default_step] = default_step_instance
            if has_permissions(self.request.user, default_step_instance):
                self._ordered_steps.append(default_step_instance)

    def get_absolute_url(self):
        return self.request.get_full_path()

    def _update_zabbix_templates(self, instance, templates,
                                 is_single_instance, **kwargs):
        result = True

        project_id = kwargs.get('project_id')

        instance_host = getattr(instance, 'host', None)
        host_method, host_args = None, ()
        if instance_host:
            if templates:
                host_method = 'update_host'
                if not is_single_instance:
                    templates |= {
                        template['templateid']
                        for template in instance_host['parentTemplates']
                    }
                host_args = (list(templates),)
            elif is_single_instance:
                host_method = 'delete_host'
        elif templates:
            host_method = 'create_host'
            host_args = (list(templates),)
        else:
            raise SkipUpdatingException()

        if host_method:
            try:
                getattr(TNXZabbixClient, host_method)(self.request,
                                                      instance.id,
                                                      project_id=project_id,
                                                      *host_args)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(unicode(err))
                result = False
        return result

    # noinspection PyMethodMayBeStatic, PyUnusedLocal
    def _update_bareos_templates(self, instance, templates,
                                 is_single_instance, **kwargs):
        result = True

        tnx_backup_templates = instance.tnx_backup_templates or []
        instance_templates = set(
            template['id'] for template in tnx_backup_templates
        )

        templates_to_add = list(
            templates.difference(instance_templates)
        )

        templates_to_remove = list(
            instance_templates.difference(templates)
        ) if is_single_instance else []

        try:
            if templates_to_add:
                TNXServersClient.add_backup_templates(self.request,
                                                      instance.id,
                                                      templates_to_add)
            if templates_to_remove:
                TNXServersClient.remove_backup_templates(self.request,
                                                         instance.id,
                                                         templates_to_remove)
        except Exception as err:
            logger.error(unicode(err))
            result = False

        return result

    def handle(self, request, context):
        result = True
        failure_zabbix, failure_bareos = [], []
        success_zabbix, success_bareos = [], []
        is_single_instance = len(context['instances']) == 1

        update_functions = []

        if is_bareos_available(request):
            allocated_bareos_members = set(context['allocated_bareos_members'])
            if allocated_bareos_members or is_single_instance:
                update_functions.append((self._update_bareos_templates,
                                         allocated_bareos_members,
                                         success_bareos,
                                         failure_bareos))

        if is_zabbix_available(request):
            allocated_zabbix_members = set(context['allocated_zabbix_members'])
            if allocated_zabbix_members or is_single_instance:
                update_functions.append((self._update_zabbix_templates,
                                         allocated_zabbix_members,
                                         success_zabbix,
                                         failure_zabbix))

        for instance in context['instances']:
            for func, members, successes, failures in update_functions:
                try:
                    if func(instance, members, is_single_instance,
                            project_id=context['project_id']):
                        successes.append(instance.name)
                    else:
                        failures.append(instance.name)
                except SkipUpdatingException:
                    pass
                except UrlNotAvailableError:
                    raise

        failure_msg = success_msg = u''

        if failure_zabbix:
            failure_msg += ungettext(
                u'Unable to update Zabbix templates for instance: {0}.',
                u'Unable to update Zabbix templates for instances: {0}.',
                len(failure_zabbix)
            ).format(u'", "'.join(failure_zabbix))

        if failure_bareos:
            failure_msg += ungettext(
                u'Unable to update Bareos templates for instance: {0}.',
                u'Unable to update Bareos templates for instances: {0}.',
                len(failure_bareos)
            ).format(u'", "'.join(failure_bareos))

        if success_zabbix:
            success_msg += ungettext(
                u'Successfully update Zabbix templates for instance: "{0}".',
                u'Successfully update Zabbix templates for instances: "{0}".',
                len(success_zabbix)
            ).format(u'", "'.join(success_zabbix))

        if success_bareos:
            success_msg += ungettext(
                u'Successfully update Bareos templates for instance: "{0}".',
                u'Successfully update Bareos templates for instances: "{0}".',
                len(success_bareos)
            ).format(u'", "'.join(success_bareos))

        if failure_msg:
            result = False
            self.failure_message = failure_msg
            if success_msg:
                messages.success(request, success_msg)
        elif success_msg:
            self.success_message = success_msg

        return result
