# coding: utf-8
import json
import logging

from django.utils.translation import ugettext_lazy as _
from horizon import exceptions, tabs
from openstack_dashboard import api

from tionix_dashboard.clients.backup_templates import TNXBackupTemplatesClient
from tionix_dashboard.dashboards.admin.instances.helpers import (
    get_instances_for_admin)
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.metrics.api import (
    is_bareos_available, is_ceilometer_available)
from tionix_dashboard.dashboards.tionix.metrics.bareos_templates import tables
from tionix_dashboard.dashboards.tionix.metrics.instances import (
    tables as instance_tables)
from tionix_dashboard.dashboards.tionix.metrics.instances.helpers import (
    get_user_instances
)
from tionix_dashboard.helpers.views import (
    DTVTnxPaginationMixin, ServerFilterTabMixin, ServerPaginationMixin
)

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class InstancesMetricsTab(ServerFilterTabMixin, DTVTnxPaginationMixin,
                          ServerPaginationMixin, tabs.TableTab):
    """Metrics instances tab."""
    name = _("Instances")
    slug = "instances_metrics_tab"
    template_name = "horizon/common/_detail_table.html"
    table_classes = (instance_tables.MetricsInstancesTable, )
    preload = False

    def get_instances_metrics_data(self):
        instances, total_count = [], 0
        is_admin = api.keystone.is_domain_admin(self.request)

        if is_admin:  # request all vms from all tenants, client pagination
            offset, limit = self.get_offset_limit()
            search_opts = self.get_filters()
            try:
                instances = get_instances_for_admin(
                    self.request,
                    search_opts=search_opts
                )
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve instance list.'))
            else:
                total_count = len(instances)
                range_end = offset + limit
                instances = instances[offset:range_end]
                self.set_table_footer(offset, limit, total_count)
        else:  # server-side pagination for user active tenant
            self.marker_init()
            try:
                instances, has_more, has_prev, total_count = (
                    get_user_instances(
                        self.request, self.search_opts, self.sort_dir,
                        self.is_last_page,
                    )
                )
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve instance list.'))
            else:
                self.table.total_count = total_count
                self._more, self._prev = has_more, has_prev

        return instances


# noinspection PyAbstractClass,PyCompatibility
class BareosTemplatesTab(ServerFilterTabMixin, tabs.TableTab):
    """Tab for bareos tempates"""
    table_classes = (tables.BareosTemplatesTable,)
    name = _('Bareos Templates')
    slug = 'bareos_templates_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_bareos_templates_data(self):

        templates = []
        domain_id = api.keystone.get_default_domain(self.request,
                                                    get_name=False).get('id')
        search_opts = {'domain_id': domain_id}
        try:
            templates = TNXBackupTemplatesClient.list(
                self.request, search_opts=search_opts)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve template list.'))
        return templates

    def allowed(self, request):
        return is_bareos_available(request) and api.keystone.is_domain_admin(
            self.request)


class MetricsTab(tabs.Tab):
    name = _("Metrics")
    slug = "metrics"
    template_name = "metrics/_detail_metrics.html"
    preload = False

    def allowed(self, request):
        return (
            LicenseController.is_valid(request, LicenseController.MONITOR) and
            is_ceilometer_available(request)
        )


class HypervisorMetricsTab(MetricsTab):

    def get_context_data(self, request, **kwargs):
        context = super(HypervisorMetricsTab, self).get_context_data(request,
                                                                     **kwargs)
        context['data'] = json.dumps({})
        return context


# noinspection PyAbstractClass
class MetricsTabs(tabs.TabGroup):
    slug = "metrics_tabs"
    tabs = (InstancesMetricsTab, BareosTemplatesTab,)
    sticky = True
    show_single_tab = True
