# coding: utf-8
# pylint: disable=no-member

import random

from datetime import datetime, timedelta

import mock
import pytz
from novaclient.exceptions import Forbidden

from openstack_dashboard import api  # pylint: disable=import-error
from tionix_dashboard.clients import TNXServersClient

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.metrics import (
    tabs, views, api as metric_api, workflows
)
from tionix_dashboard.dashboards.tionix.metrics.enums import MetricsTypes
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, patch_resource_get)
from tionix_dashboard.tests.fake_cloud_state import ALL_SERVERS_COUNT


class TionixMetricsTabsTest(BaseClientTestCase):

    @mock.patch.object(api.network, 'servers_update_addresses', mock.Mock())
    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(api.glance, 'image_list_detailed')
    @mock.patch.object(api.nova, 'flavor_get')
    @mock.patch.object(api.nova, 'flavor_list')
    @mock.patch.object(api.keystone.VERSIONS, 'get_project_manager')
    @mock.patch.object(api.keystone, 'get_default_domain')
    def test_tnx_admin_inst_index_view_get_data(
            self, get_default_domain, get_project_manager, flavor_list,
            flavor_get, image_list_detailed):
        tenants = self.tnx_tenants.list()
        flavors = self.tnx_flavors.list()
        images = self.tnx_images.list()
        domain = self.tnx_domains.list()[0]

        fake_project_manager = mock.Mock()
        fake_project_manager.list.return_value = tenants
        get_project_manager.return_value = fake_project_manager
        flavor_list.return_value = flavors
        image_list_detailed.return_value = images, False, False
        flavor_get.side_effect = patch_resource_get(self.tnx_flavors)
        get_default_domain.return_value = {'id': domain.id,
                                           'name': domain.name}

        tab = tabs.InstancesMetricsTab(
            tab_group=tabs.MetricsTabs(self.request),
            request=self.request
        )

        tab_instances = tab.get_instances_metrics_data()

        self.assertEqual(ALL_SERVERS_COUNT, len(tab_instances))

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=True))
    def test_metrics_tab_allowed(self):

        tab = tabs.MetricsTab(
            tab_group=tabs.MetricsTabs(self.request),
            request=self.request
        )

        self.assertTrue(tab.allowed(self.request))

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=False))
    def test_metrics_tab_license_no_valid(self):

        tab = tabs.MetricsTab(
            tab_group=tabs.MetricsTabs(self.request),
            request=self.request
        )

        self.assertFalse(tab.allowed(self.request))

    @mock.patch.object(TNXServersClient, 'get_metrics_names',
                       mock.Mock(side_effect=Forbidden))
    def test_metrics_tab_ceilometer_not_available(self):

        tab = tabs.MetricsTab(
            tab_group=tabs.MetricsTabs(self.request),
            request=self.request
        )

        self.assertFalse(tab.allowed(self.request))

    @mock.patch('tionix_dashboard.dashboards.tionix.metrics.tabs.'
                'is_bareos_available', mock.Mock())
    def test_metrics_tabs(self):

        tab_group = tabs.MetricsTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['bareos_templates_tab', 'instances_metrics_tab'])
        )

    @mock.patch('tionix_dashboard.dashboards.tionix.metrics.tabs.'
                'is_bareos_available', mock.Mock(return_value=False))
    def test_metrics_tabs_bareos_not_available(self):

        tab_group = tabs.MetricsTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['instances_metrics_tab'])
        )

    @mock.patch.object(api.keystone,
                       'is_domain_admin',
                       mock.Mock(return_value=False))
    def test_metrics_tabs_bareos_not_allowed(self):
        tab_group = tabs.MetricsTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['instances_metrics_tab']))


@mock.patch.object(TNXServersClient, 'get_metrics_names', mock.Mock())
class TionixMetricsViewsTest(BaseClientTestCase):

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=True))
    @mock.patch.object(tabs.InstancesMetricsTab,
                       'get_instances_metrics_data',
                       mock.Mock(return_value=[]))
    def test_metrics_index_view(self):

        view = views.MetricsIndexView()
        view.request = self.request
        view.kwargs = {}
        response = view.get(self.request)
        self.assertEqual(response.status_code, 200)

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=False))
    def test_metrics_view_not_available(self):

        view = views.MetricsIndexView()
        view.request = self.request
        view.kwargs = {}

        with self.assertRaises(UrlNotAvailableError):
            view.get(self.request)

    def test_format_meter_data(self):
        def get_meter_series(n, lost=0):
            delay = 90
            deviation = 10
            y_max = 100

            values = []
            new_time = datetime.utcnow()
            for _i in range(n):
                new_delay = delay + random.randint(-deviation, deviation)
                new_time += timedelta(seconds=new_delay)
                values.append(
                    (new_time.isoformat(), random.random() * y_max))
            for _i in range(lost):
                values.remove(random.choice(values[1:-1]))
            return values

        data = []
        formatted_data = metric_api.format_meter_data(data, tz=pytz.utc)
        self.assertEqual(len(data), len(formatted_data))

        total_points = 10
        for lost_points in range(5):
            data = get_meter_series(total_points, lost=lost_points)
            formatted_data = metric_api.format_meter_data(data, tz=pytz.utc)
            self.assertGreaterEqual(len(formatted_data), len(data))


class TionixMetricsWorkflowsTest(BaseClientTestCase):

    def setUp(self):
        super(TionixMetricsWorkflowsTest, self).setUp()

        self.metrics_names = ['metric', 'second_metric', 'metric 3']

    def tearDown(self):
        super(TionixMetricsWorkflowsTest, self).tearDown()
        UserMetrics.objects.all().delete()

    def test_metrics_settings_action(self):

        action = workflows.MetricsSettingsAction(
            self.request,
            context={'available_members': self.metrics_names,
                     'allocated_members': self.metrics_names[:2]}
        )

        field_name = action.get_member_field_name('member')

        self.assertListEqual(
            sorted(action.fields[field_name].choices),
            sorted([(metric_name.replace('_', '.'), metric_name)
                    for metric_name in self.metrics_names])
        )

        self.assertListEqual(
            sorted(action.fields[field_name].initial),
            sorted([metric_name.replace('_', '.')
                    for metric_name in self.metrics_names[:2]])
        )

    def test_metrics_settings_workflow(self):
        context = {
            'allocated_members': self.metrics_names[:2],
            'metric_type': MetricsTypes.INSTANCE
        }
        workflow = workflows.MetricsSettingsWorkflow(self.request)
        workflow.handle(request=self.request, context=context)

        self.assertEqual(UserMetrics.objects.count(), 1)

    def test_metrics_settings_workflow_no_metrics(self):
        context = {
            'allocated_members': [],
            'metric_type': MetricsTypes.INSTANCE
        }
        workflow = workflows.MetricsSettingsWorkflow(self.request)
        workflow.handle(request=self.request, context=context)

        self.assertEqual(UserMetrics.objects.count(), 0)

    def test_metrics_settings_workflow_delete_metrics(self):

        UserMetrics.objects.create(user_id=self.request.user.id,
                                   metric_type=MetricsTypes.INSTANCE,
                                   metrics_names=self.metrics_names)

        context = {
            'allocated_members': [],
            'metric_type': MetricsTypes.INSTANCE
        }
        workflow = workflows.MetricsSettingsWorkflow(self.request)
        workflow.handle(request=self.request, context=context)

        self.assertEqual(UserMetrics.objects.count(), 0)
