# coding: utf-8
# pylint: disable=no-member

import logging

from django import shortcuts
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.views import generic
from django.utils.translation import ugettext_lazy as _

from horizon import tabs, messages
from tionix_dashboard.clients import TNXServersClient

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.dashboards.tionix.metrics.tabs import MetricsTabs
from tionix_dashboard.helpers.views import RefererRedirectMixin

logger = logging.getLogger(__name__)


class MetricsIndexView(tabs.TabbedTableView):
    tab_group_class = MetricsTabs
    template_name = 'metrics/index.html'
    page_title = _("Metrics")

    def get(self, request, *args, **kwargs):
        if not LicenseController.is_valid(request, LicenseController.MONITOR):
            raise UrlNotAvailableError(
                redirect_to=settings.WEBROOT,
                lic_type=LicenseController.MONITOR)
        return super(MetricsIndexView, self).get(request, *args, **kwargs)


class DeleteUnavailableMetrics(RefererRedirectMixin, generic.View):

    def get(self, request, **kwargs):
        metric_type = kwargs['metric_type']

        try:
            metrics_names = TNXServersClient.get_metrics_names(request)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to get available metrics list.'))
        else:
            try:
                user_metric = UserMetrics.objects.get(
                    user_id=request.user.id,
                    metric_type=metric_type
                )
            except ObjectDoesNotExist:
                messages.error(request, _('Metrics are not found.'))
            else:
                user_metric.metrics_names = [
                    metric for metric in user_metric.metrics_names
                    if metric in metrics_names
                ]
                user_metric.save()
                messages.success(
                    request,
                    _('Successfully deleted unavailable metrics '
                      'from settings.'))
        return shortcuts.redirect(self.get_redirect_url())
