# coding: utf-8
# pylint: disable=no-member

import logging

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _

from horizon import forms, workflows  # pylint: disable=import-error

from tionix_dashboard.dashboards.tionix.metrics import constants
from tionix_dashboard.dashboards.tionix.metrics.enums import MetricsTypes
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.helpers.workflows import TnxUpdateMembersStep

logger = logging.getLogger(__name__)


METRICS_SEP = '_'
METRICS_FIELD_SEP = '.'


class BaseMembershipAction(workflows.MembershipAction):

    # noinspection PyMethodMayBeStatic
    def _get_choices(self):
        return []

    # noinspection PyMethodMayBeStatic
    def _get_initial_choices(self):
        return []

    def __init__(self, request, *args, **kwargs):
        super(BaseMembershipAction, self).__init__(request, *args, **kwargs)

        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)

        self.fields[field_name].choices = self._get_choices()

        self.fields[field_name].initial = self._get_initial_choices()


class MetricsSettingsAction(BaseMembershipAction):

    metric_type = forms.ChoiceField(choices=MetricsTypes.get_choices(),
                                    widget=forms.HiddenInput())

    def _get_choices(self):
        return [(member.replace(METRICS_SEP, METRICS_FIELD_SEP), member)
                for member in self.initial.get('available_members')]

    def _get_initial_choices(self):
        return [member.replace(METRICS_SEP, METRICS_FIELD_SEP)
                for member in self.initial.get('allocated_members')]

    class Meta(object):
        name = _("Settings")


class MetricsSettingsStep(TnxUpdateMembersStep):
    action_class = MetricsSettingsAction
    available_list_title = _("All available metrics")
    members_list_title = _("Selected metrics")
    no_available_text = _("No metrics found.")
    no_members_text = _("No metric selected.")
    show_roles = False
    contributes = ("available_members", "allocated_members", "metric_type")

    def contribute(self, data, context):
        if data:
            member_field_name = self.get_member_field_name('member')
            choices_dict = dict(self.action[member_field_name].field.choices)
            context['allocated_members'] = [
                choices_dict[value]
                for value in data.get(member_field_name, [])]
        return context


class MetricsSettingsWorkflow(workflows.Workflow):
    slug = "metrics_settings"
    name = _("Add/Remove metrics")
    finalize_button_name = _("Save")
    success_message = _("Settings was updated.")
    failure_message = _("Unable to update settings.")
    success_url = constants.METRICS_INDEX
    default_steps = (MetricsSettingsStep, )

    def handle(self, request, context):
        user_metrics_names = context['allocated_members']

        try:
            user_metrics = UserMetrics.objects.get(
                user_id=request.user.id,
                metric_type=context['metric_type']
            )
        except ObjectDoesNotExist:
            if user_metrics_names:
                UserMetrics.objects.create(
                    user_id=request.user.id,
                    metric_type=context['metric_type'],
                    metrics_names=user_metrics_names
                )
        else:
            if user_metrics_names:
                user_metrics.metrics_names = user_metrics_names
                user_metrics.save()
            else:
                user_metrics.delete()
        return True
