# coding: utf-8
import logging

from django.core.urlresolvers import reverse
from django.utils.translation import (
    ugettext_lazy as _, npgettext_lazy, ungettext_lazy)

from horizon import tables
from horizon import exceptions

from tionix_dashboard.clients import (
    TNXPowerDevicesClient, TNXHypervisorsClient)
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.helpers.actions import (
    LinkBatchAction, TnxUpdateAction, TnxFilterAction, TnxBatchActionMixin)
from tionix_dashboard.url_resolve_map import (
    POWER_DEVICES_INDEX, POWER_DEVICES_DELETE, POWER_DEVICES_CREATE,
    POWER_DEVICES_CLONE, POWER_DEVICES_UPDATE)


logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class DeletePowerDeviceAction(LinkBatchAction):
    """Power device delete action."""

    name = 'delete_power_device'
    multiple_next_view_name = POWER_DEVICES_DELETE
    classes = ('btn-danger', )

    def action_present(self, count):
        return ungettext_lazy(
            u"Delete power control",
            u"Delete power controls",
            count
        )

    def action_past(self, count):
        return ungettext_lazy(
            u"Deleted power control",
            u"Deleted power controls",
            count
        )


# noinspection PyAbstractClass
class CreatePowerDevice(tables.LinkAction):
    """Power device create action."""
    name = 'create'
    verbose_name = _('Create power control')
    url = POWER_DEVICES_CREATE
    classes = ('ajax-modal',)
    icon = 'plus'


# noinspection PyAbstractClass
class PowerDeviceFilterAction(TnxFilterAction):
    """Power device filter table action."""
    name = "filter_power_device"
    filter_type = "server"

    filter_choices = (
        ('id', _("ID"), False),
        ('title', _("Control title"), False),
        ('device_name', _("Control type"), False),
        ('protocol_type', _("Protocol type"), False),
        ('ports', _("Ports"), False),
        ('used_ports', _("Used ports"), False)
    )


class TnxInstancesUpdateRow(tables.Row):
    """
    Action updated power device rows in table.
    """
    ajax = True

    def get_data(self, request, obj_id):
        try:
            device = TNXPowerDevicesClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                request,
                _('Unable to get power control.'),
                redirect=reverse(POWER_DEVICES_INDEX))
            device = None

        return device


class DeviceTitleUpdateCell(TnxUpdateAction):
    """
    Action update device title
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        try:
            TNXPowerDevicesClient.update(
                request, obj_id, **{cell_name: new_cell_value})
        except Exception as err:
            exceptions.handle(
                request,
                _(unicode(err.message)))


# noinspection PyAbstractClass
class UpdateEditPowerDevice(tables.LinkAction):
    """Power device edit action."""
    name = 'update'
    verbose_name = _('Edit power control')
    url = POWER_DEVICES_UPDATE
    classes = ('ajax-modal',)


# noinspection PyAbstractClass
class ClonePowerDevice(tables.LinkAction):
    """Power device clone action."""
    name = 'clone'
    verbose_name = _('Clone power control')
    url = POWER_DEVICES_CLONE
    classes = ('ajax-modal',)


# noinspection PyAbstractClass
class UnassignHypervisorAction(TnxBatchActionMixin, tables.BatchAction):
    """Unasign hypervisor from power device"""
    name = 'unassign'
    verbose_name = _('Unassign hypervisor')
    classes = ("btn-danger",)
    permissions = ('openstack.roles.admin',)

    # pylint: disable=arguments-differ
    def confirm_header(self, items):
        return npgettext_lazy(
            "Action header",
            u"Unassign hypervisor",
            u"Unassign hypervisor",
            items
        )

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Unassign hypervisor",
            u"Unassign hypervisor",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Unassigned hypervisor",
            u"Unassigned hypervisor",
            count
        )

    def action(self, request, datum_id):
        TNXHypervisorsClient.unassign_power_device(
            request,
            datum_id,
            int(self.table.kwargs['power_device_id'])
        )
