# coding: utf-8

import logging

from django.forms import ValidationError
from django.utils.translation import ugettext_lazy as _
from horizon import forms, messages
from horizon.utils import validators

from tionix_dashboard.clients import TNXPowerDevicesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.helpers import Empty
from tionix_dashboard.helpers.fields import AutoGenNameCharField

logger = logging.getLogger(__name__)


class AuthTypes(object):
    PASSWORD = 'password'
    PK = 'private_key'

    PK_LABEL = _("Private key")
    PASSWORD_LABEL = _("Password")


class CreatePowerDeviceForm(forms.SelfHandlingForm):
    """Power device create form."""
    name = _('Create power control')
    protocol_type = forms.ChoiceField(
        label=_('Protocol type'),
        help_text=_('Choose a protocol type.'),
        widget=forms.ChoiceField.widget(
            attrs={
                'class': 'filtering',
                'data-slug': 'protocol-type',

            })
    )
    device_name = forms.ChoiceField(
        label=_('Control type'),
        help_text=_('Choose a control type.'),
        widget=forms.SelectWidget(
            attrs={
                'class': 'filterable',
                'data-filter-on': 'protocol-type',
                'data-filter-attr': 'protocol-type',
            },
            data_attrs=('protocol-type',),
            transform=lambda x: x.name)
    )
    title = AutoGenNameCharField(
        label=_('Control title'),
        help_text=_('If field is not filled in the object name will be '
                    'generated automatically. Maximum 50 symbols.'),
        max_length=50
    )
    host = forms.CharField(
        label=_('Host or ip'),
        help_text=_('Input host or ip of control. '
                    'Maximum 255 symbols, cyrillic characters not allowed'),
        validators=[validators.validators.URLValidator],
        max_length=255,
        widget=forms.TextInput(attrs={
            'onkeypress': 'return horizon.onPressKeyExcludeRus(event);'
        })
    )
    port_number = forms.IntegerField(
        label=_('Port'),
        help_text=_('Input port of control. Must be in the range '
                    'from 1 to 65535'),
        validators=[validators.validate_port_range],
        min_value=1,
        max_value=65535
    )
    auth_type = forms.ChoiceField(
        label=_('Authentication type'),
        choices=(
            (AuthTypes.PASSWORD, AuthTypes.PASSWORD_LABEL),
            (AuthTypes.PK, AuthTypes.PK_LABEL),
        ),
    )
    login = forms.CharField(
        label=_('Control login'),
        required=False,
        max_length=255,
    )
    password = forms.RegexField(
        label=_('Control password'), required=False,
        widget=forms.PasswordInput(
            render_value=True,
        ),
        regex=validators.password_validator(),
        error_messages={'invalid': validators.password_validator_msg()}
    )
    ssh_key = forms.CharField(
        label=_("Private key"),
        help_text=_("Private key for SSH connection. "
                    "Should be encrypted via RSA algorithm."),
        required=False,
        widget=forms.widgets.Textarea(
            attrs={
                'rows': 12,
            }
        ),
    )

    second_column_fields = {'login', 'password', 'ssh_key'}

    def __init__(self, request, *args, **kwargs):
        super(CreatePowerDeviceForm, self).__init__(request, *args, **kwargs)

        protocols, types = zip(*[
            ((protocol_name, protocol_name),
             (type_name, Empty(**{'name': type_name,
                                  'protocol-type': protocol_name})))
            for type_name, protocol_name in TNXPowerDevicesClient.types_list(
                request
            )
        ])

        self.fields['device_name'].choices = types
        self.fields['protocol_type'].choices = set(protocols)

        self._distribute_columns()

    def _distribute_columns(self):
        for name, field in self.fields.items():
            field.column = 2 if name in self.second_column_fields else 1

    def clean(self):
        data = super(CreatePowerDeviceForm, self).clean()
        protocol_type = data.get('protocol_type', None)
        password = data.get('password', None)
        password_required_types = ('ipmi', 'intel_amt')
        auth_type = data.pop('auth_type', None)

        if auth_type == AuthTypes.PASSWORD:
            data.pop('ssh_key')
        else:
            data.pop('password')

        if not password and protocol_type in password_required_types:
            raise ValidationError(_("'Control password' field is required when"
                                    " protocol type 'ipmi' or 'intel_amt'."))

        return data

    # noinspection PyMethodMayBeStatic
    def handle(self, request, data):
        result = True
        try:
            device = TNXPowerDevicesClient.create(request, **data)
            if device:
                msg = _('Control successfully created.')
                messages.success(request, msg)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            result = False
            self.add_error(None,
                           _('Device could not be added! {0}.').format(err))
        return result


# noinspection PyAbstractClass
class DeletePowerDeviceForm(forms.SelfHandlingForm):
    """Power device delete form"""

    # noinspection PyUnusedLocal
    def handle(self, request, data):
        power_devices = self.initial['power_devices']

        success, failed = [], []
        for power_device in power_devices:
            try:
                TNXPowerDevicesClient.power_device_delete(
                    request, power_device.id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.debug(unicode(err))
                failed.append(power_device.name)
                continue
            success.append(power_device.name)

        if failed:
            msg = _('Failed to delete power controls:')
            messages.error(
                request, '%s %s' % (unicode(msg), ', '.join(failed)))
        if success:
            msg = _('Successfully deleted power controls:')
            messages.success(
                request, '%s %s' % (unicode(msg), ', '.join(success)))

        return True


class UpdatePowerDeviceForm(CreatePowerDeviceForm):
    """Power device edit form."""

    def __init__(self, request, *args, **kwargs):
        super(UpdatePowerDeviceForm, self).__init__(request, *args, **kwargs)
        readonly_input = forms.TextInput(attrs={'readonly': 'readonly'})
        self.fields['device_name'].widget = readonly_input
        self.fields['protocol_type'].widget = readonly_input
        self.fields['title'].widget.attrs.update(**{'autofocus': 'autofocus'})

    def handle(self, request, data):
        result = True
        try:
            device = TNXPowerDevicesClient.update(request,
                                                  self.initial.get('id'),
                                                  **data)
            if device:
                msg = _('Control successfully updated.')
                messages.success(request, msg)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            result = False
            self.add_error(None,
                           _('Unable to update control! {0}.').format(err))
        return result
