# coding: utf-8

import logging

from collections import defaultdict

from django.core.urlresolvers import reverse_lazy, reverse
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import (
    exceptions, forms as horizon_forms, messages, tables as horizon_tables,
    tabs as horizon_tabs)
from horizon.utils import memoized

from tionix_dashboard.clients import TNXPowerDevicesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    POWER_STATE_DISPLAY_CHOICES)
from tionix_dashboard.dashboards.tionix.power_devices import (
    forms, tables, tabs)
from tionix_dashboard.dashboards.tionix.power_devices.tables import (
    PowerDevicesTable)
from tionix_dashboard.helpers.views import (
    ServerFilterViewMixin, LinkBatchFormView)
from tionix_dashboard.url_resolve_map import (
    POWER_DEVICES_INDEX, POWER_DEVICES_UPDATE, POWER_DEVICES_CREATE,
    POWER_DEVICES_DELETE)

logger = logging.getLogger(__name__)


# noinspection PyClassHasNoInit
class PowerDevicesIndexView(horizon_tables.DataTableView):
    """Main page view."""
    table_class = tables.PowerDevicesTable
    template_name = 'power_devices/index.html'
    page_title = _('Power controls')

    def get_data(self):
        request = self.request
        power_devices = []
        device_types = []
        protocols = []

        try:
            power_devices = TNXPowerDevicesClient.list(request)
            device_types, _protocols = zip(
                *TNXPowerDevicesClient.types_list(request)
            )
            protocols = TNXPowerDevicesClient.protocols_list(request)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve power controls list.'))

        invalid_devices = defaultdict(list)
        for device in power_devices:
            if device.device_name not in device_types:
                invalid_devices[device.title].append(
                    unicode(_("Control type")))
            if device.protocol_type not in protocols:
                invalid_devices[device.title].append(
                    unicode(_("Protocol type")))

        for device, fields in invalid_devices.items():
            messages.error(
                request,
                _('Following fields of the device "{0}"'
                  ' are invalid: {1}').format(device, ", ".join(fields)))

        return power_devices


# noinspection PyClassHasNoInit
class CreatePowerDeviceView(horizon_forms.ModalFormView):
    """Create power device view."""
    form_class = forms.CreatePowerDeviceForm
    template_name = 'power_devices/create.html'
    modal_header = _('Create power control')
    modal_id = 'create_power_device'
    submit_label = _('Create control')
    submit_url = reverse_lazy(POWER_DEVICES_CREATE)
    context_object_name = 'power_devices'
    success_url = reverse_lazy(POWER_DEVICES_INDEX)


# noinspection PyClassHasNoInit
class DeletePowerDeviceView(LinkBatchFormView):
    """Delete power device view."""
    form_class = forms.DeletePowerDeviceForm
    template_name = 'power_devices/delete.html'
    modal_id = "power_device_delete"
    submit_label = _('Delete')
    context_object_name = 'power_device'
    success_url = reverse_lazy(POWER_DEVICES_INDEX)
    submit_url = reverse_lazy(POWER_DEVICES_DELETE)
    hidden_title = False
    failure_msg = _('"Delete" is not available for power controls:')

    @staticmethod
    def modal_header(count):
        return ungettext_lazy(
            u'Confirm delete power control',
            u'Confirm delete power controls',
            count
        )

    @memoized.memoized_method
    def get_objects(self):
        devices, failed = [], []
        for power_device_id in self.get_object_ids():
            try:
                obj = TNXPowerDevicesClient.get(
                    self.request, power_device_id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(err.message)
                failed.append(power_device_id)
                continue
            devices.append(obj)

        if failed:
            messages.error(self.request, u'{0}: {1}'.format(
                self.failure_msg, u','.join(failed)))

        return devices

    def get_initial(self):
        devices = self.get_objects()

        hypervisors = []

        power_state_dict = dict(POWER_STATE_DISPLAY_CHOICES)

        for device in devices:
            try:
                hyps = TNXPowerDevicesClient.power_device_assigned_nodes(
                    self.request, device.id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(unicode(err))
            else:
                hypervisors.extend(hyps)

        for h in hypervisors:
            h.update(
                tnx_power_state=power_state_dict.get(
                    unicode(h['tnx_power_state'])))

        return {
            'power_device_names': [u'"{0}"'.format(d.name) for d in devices],
            'power_devices': devices,
            'hypervisors': hypervisors
        }


class PowerDeviceInitialMixin(object):

    kwargs = None
    request = None

    def get_initial(self):
        initial = super(PowerDeviceInitialMixin, self).get_initial()

        try:
            power_device_dict = TNXPowerDevicesClient.get(
                self.request, self.kwargs['power_device_id']
            ).to_dict()
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _('Unable to get power control.'))
        else:
            power_device_dict.pop('password')
            power_device_dict['port_number'] = power_device_dict['port']
            initial.update(power_device_dict)

        return initial


# noinspection PyClassHasNoInit
class UpdatePowerDeviceView(PowerDeviceInitialMixin, CreatePowerDeviceView):
    """Create power device view."""
    form_class = forms.UpdatePowerDeviceForm
    template_name = 'power_devices/update.html'
    modal_header = _('Edit power control')
    modal_id = 'edit_power_device'
    submit_label = _('Edit Control')
    submit_url = POWER_DEVICES_UPDATE

    def get_context_data(self, **kwargs):
        context = super(UpdatePowerDeviceView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        args=(self.kwargs['power_device_id'],))
        return context


# noinspection PyClassHasNoInit
class ClonePowerDeviceView(PowerDeviceInitialMixin, CreatePowerDeviceView):
    """Clone power device view."""
    modal_header = _('Clone power control')
    modal_id = 'clone_power_device'
    submit_label = _('Clone control')


class DetailPowerDeviceView(ServerFilterViewMixin,
                            horizon_tabs.TabbedTableView):
    tab_group_class = tabs.PowerDeviceDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = _("Power control details: {{ power_device.name }}")

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(DetailPowerDeviceView, self).get_context_data(**kwargs)
        power_device = self.get_data()
        context['power_device'] = power_device
        self.kwargs['power_device'] = power_device
        table = PowerDevicesTable(self.request)
        context["url"] = reverse_lazy(POWER_DEVICES_INDEX)
        context["actions"] = table.render_row_actions(power_device)
        return context

    @memoized.memoized_method
    def get_data(self):
        """Gets power device data"""
        power_device = None
        try:
            power_device = TNXPowerDevicesClient.get(
                self.request, self.kwargs['power_device_id']
            )
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve power control details.'),
                redirect=reverse(POWER_DEVICES_INDEX))
        return power_device

    def get_tabs(self, request, **kwargs):
        kwargs['power_device'] = self.get_data()
        return self.tab_group_class(request, **kwargs)
