# coding: utf-8
# pylint: disable=method-hidden, inconsistent-return-statements

import logging
from collections import defaultdict

from django.core.urlresolvers import (
    reverse, resolve, Resolver404, NoReverseMatch)
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import tables, exceptions
from horizon.utils.functions import get_page_size
from openstack_dashboard import api

from tionix_scheduler_client.client import Client as SchedulerClient
from tionix_scheduler_client.enums import (
    ObjectTypeEnum, ObjectGroupEnum, TaskResultEnum)

from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.dashboards.tionix.scheduler import (
    api as scheduler_api)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.helpers.actions import (
    LinkBatchAction, TnxFilterAction, TnxBatchActionMixin)
from tionix_dashboard.helpers.utils import get_http_referer_path
from tionix_dashboard.helpers.views import (
    LinkBatchFormView, RefererRedirectMixin)

logger = logging.getLogger(__name__)

EXECUTION_STATE_DISPLAY_CHOICES = (
    ('', _('-')),
    (TaskResultEnum.task_unknown, _('Unknown')),
    (TaskResultEnum.task_success, _('Success')),
    (TaskResultEnum.task_failure, _('Failure'))
)

TYPE_DISPLAY_CHOICES = (
    ('True', _('Periodic')),
    ('False', _('Once-only'))
)

SUBJECT_TYPE_DISPLAY_CHOICES = tuple(
    (k, _(v)) for k, v in ObjectTypeEnum.VERBOSE_NAMES.iteritems()
)


# noinspection PyAbstractClass
class AddTableData(tables.LinkAction):
    name = 'add'
    verbose_name = _('Add schedule')
    url = 'horizon:tionix:scheduler:add'
    classes = ('btn-launch', 'ajax-modal')


class DeleteTaskAction(TnxBatchActionMixin, tables.DeleteAction):

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Task",
            u"Delete Tasks",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Task",
            u"Deleted Tasks",
            count
        )

    def allowed(self, request, datum):
        """Action is allowed for not deleted tasks"""
        allowed = super(DeleteTaskAction, self).allowed(request, datum)
        if datum:
            allowed = allowed and not datum.deleted
        return allowed

    def delete(self, request, obj_id):
        scheduler_client = SchedulerClient(
            headers={'X-AUTH-TOKEN': request.user.token.id})
        scheduler_client.delete_timetable(obj_id)

    def get_success_url(self, request=None):
        _table, _action, obj_id = self.table.parse_action(
            request.POST["action"])
        is_multiple = obj_id is None
        if is_multiple:
            request.session.pop('object_ids')
        return self.table.get_redirect_url(is_multiple)


def get_display_key_by_value(display_values):
    """Return function for searching value in display values"""
    def func(value):
        reversed_status_dict = {unicode(v).lower(): k
                                for k, v in display_values}
        return reversed_status_dict.get(value.lower(), value)

    return func


def get_display_key_by_value_partial(display_values):
    """Return function for searching partial value in display values"""
    def func(value):
        value = value.lower()
        keys = [k for k, v in display_values if value in v.lower()]
        return keys or value

    return func


# noinspection PyAbstractClass
class FilterAction(TnxFilterAction, tables.FilterAction):
    """Filter action for scheduled tasks"""
    name = 'scheduler_filter'
    filter_type = 'server'

    filter_choices = (
        ('id', _("ID"), True),
        ('name', _("Task name"), True),
        ('action', _("Action"), True),
        ('type', _("Type ="), True),
        ('last_run_result', _("Last execution state"), True),
        ('subject_type', _("Subject type"), True),
        ('subject', _("Subject title"), True),
        ('subject_tenant', _("Project"), False)
    )

    filter_string_modifiers = {
        'last_run_result': (
            get_display_key_by_value_partial(EXECUTION_STATE_DISPLAY_CHOICES)
        ),
        'type': get_display_key_by_value(TYPE_DISPLAY_CHOICES),
        'subject_type': (
            get_display_key_by_value_partial(SUBJECT_TYPE_DISPLAY_CHOICES)
        )
    }

    def get_default_attrs(self):
        """Returns a list of the default HTML attributes
        for the action.
        """
        attrs = super(FilterAction, self).get_default_attrs()
        attrs.update({'name': 'filter',
                      'value': '{0}__{1}'.format(self.table.name,
                                                 self.name)})
        return attrs

    def get_param_name(self):
        obj_id = self.table.kwargs.get('obj_id')
        return "{0}__{1}__{2}".format(
            self.table.name,
            self.name,
            obj_id or self.param_name)


# noinspection PyAbstractClass
class TabFilterAction(FilterAction):

    filter_choices = (
        ('id', _("ID"), True),
        ('name', _("Task name"), True),
        ('action', _("Action"), True),
        ('type', _("Type ="), True),
        ('last_run_result', _("Last execution state"), True),
    )


class SchedulerUpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        result = {}
        msg = _('Unable to retrieve the task.')

        scheduler_client = SchedulerClient(
            headers={'X-AUTH-TOKEN': request.user.token.id})

        try:
            result = scheduler_client.get_timetable(obj_id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, msg)

        return result


class ObjectTasksView(RefererRedirectMixin, LinkBatchFormView):

    request = success_url = None

    def get_success_url(self):
        referer_path = get_http_referer_path(self.request)
        try:
            resolved = resolve(referer_path)
            if resolved.url_name == 'detail':
                namespaces = resolved.namespaces
                url = reverse('{0}:index'.format(':'.join(namespaces)))
            else:
                url = self.get_redirect_url()
        except (Resolver404, NoReverseMatch):
            url = self.success_url
        return url

    @staticmethod
    def get_display_value(choices, data):
        for (value, display) in choices:
            if value.lower() == (unicode(data) or u'').lower():
                return display

    def get_tasks_rows(self, request, query_params):
        limit = get_page_size(request)
        is_admin = api.keystone.is_domain_admin(request)
        query_params.update(all_tenants=is_admin)
        tasks, _prev, _more, _total = scheduler_api.get_scheduled_tasks(
            request, offset=0, limit=limit, query_params=query_params,
            use_post=True
        )
        rows = defaultdict(list)
        for task in tasks:
            rows[task.subject_id].append([
                task.id,
                task.name,
                task.action,
                self.get_display_value(TYPE_DISPLAY_CHOICES, task.repeat),
                self.get_display_value(EXECUTION_STATE_DISPLAY_CHOICES,
                                       task.last_run_result),
                self.get_display_value(SUBJECT_TYPE_DISPLAY_CHOICES,
                                       task.subject_type),
                task.subject
            ])
        return dict(rows)


# noinspection PyAbstractClass,PyUnusedLocal
class SchedulerLinkBatchAction(LinkBatchAction):
    name = 'schedule_calendar'
    policy_rules = (('identity', 'admin_required'),)
    req_param_name = 'object_id'
    object_type = None
    object_group = None
    url = None

    def action_present(self, count):
        return _('Schedule')

    def action_past(self, count):
        return _('Scheduled')

    def get_next_view_url(self, next_view_name):
        return reverse(next_view_name,
                       args=[self.object_type, self.object_group],
                       prefix='/')

    def allowed(self, request, datum):
        allowed = super(SchedulerLinkBatchAction, self).allowed(request, datum)
        allowed = allowed and SchedulingPreChecker.is_actions_not_empty(
            request, self.object_type, self.object_group
        )

        if datum:
            allowed = allowed and SchedulingPreChecker.is_schedulable(datum)
        return allowed


# noinspection PyAbstractClass
class HypervisorSchedulerAction(SchedulerLinkBatchAction):
    """
    Action to schedule any available task for hypervisors
    """
    name = 'hypervisor_schedule'
    multiple_next_view_name = "horizon:tionix:scheduler:calendar"

    object_type = ObjectTypeEnum.hypervisor
    object_group = ObjectGroupEnum.admin


# noinspection PyAbstractClass
class InstanceSchedulerAction(SchedulerLinkBatchAction):
    """
    Action to schedule any available task for instances
    """
    name = 'instance_schedule'
    multiple_next_view_name = "horizon:tionix:scheduler:calendar"

    object_type = ObjectTypeEnum.instance
    object_group = ObjectGroupEnum.admin


# noinspection PyAbstractClass
class ProjectInstanceSchedulerAction(InstanceSchedulerAction):
    object_group = ObjectGroupEnum.project


# noinspection PyAbstractClass
class VDIInstanceSchedulerAction(InstanceSchedulerAction):
    """
    Action to schedule any available task for vdi instances
    """

    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    object_type = ObjectTypeEnum.instance
    object_group = ObjectGroupEnum.vdi


# noinspection PyAbstractClass
class VolumeSchedulerAction(SchedulerLinkBatchAction):
    """
    Action to schedule any available task for volumes
    """
    name = 'volume_schedule'
    multiple_next_view_name = "horizon:tionix:scheduler:calendar"

    object_type = ObjectTypeEnum.volume
    object_group = ObjectGroupEnum.admin


# noinspection PyAbstractClass
class ProjectVolumeSchedulerAction(VolumeSchedulerAction):
    object_group = ObjectGroupEnum.project


# noinspection PyAbstractClass
class ShowTaskAction(tables.LinkAction):
    """
    Action show extra information for selected schedule.
    """
    name = 'show'
    verbose_name = _('More')
    url = 'horizon:tionix:scheduler:show'
    classes = ("ajax-modal",)
    policy_rules = (('identity', 'admin_required'),)


# noinspection PyAbstractClass
class ProjectSchedulerAction(SchedulerLinkBatchAction):
    """
    Action to schedule project`s any available task.
    """
    name = 'project_schedule'
    multiple_next_view_name = "horizon:tionix:scheduler:calendar"

    object_type = ObjectTypeEnum.project
    _object_group = None

    @property
    def object_group(self):
        if self._object_group is None:
            if getattr(self.datum, 'tnx_project_type_name',
                       None) == TenantTypeEnum.VDI:
                self._object_group = ObjectGroupEnum.vdi
            else:
                self._object_group = ObjectGroupEnum.identity
        return self._object_group


# noinspection PyAbstractClass
class VDIProjectSchedulerAction(SchedulerLinkBatchAction):
    """
    Action to schedule any available task for vdi projects
    """
    name = 'vdi_project_schedule'
    multiple_next_view_name = "horizon:tionix:scheduler:calendar"

    object_type = ObjectTypeEnum.project
    object_group = ObjectGroupEnum.vdi

    def allowed(self, request, datum):
        """Determine whether this action is allowed for the current request.

        Only for enabled projects.
        """
        return (datum.enabled if datum else True) and super(
            VDIProjectSchedulerAction, self).allowed(request, datum)


# noinspection PyAbstractClass
class RepeatTaskAction(tables.LinkAction):
    """Repeat task action"""
    name = 'repeat_schedule'
    verbose_name = _('Repeat task')
    classes = ("ajax-modal",)
    url = 'horizon:tionix:scheduler:repeat_schedule_task'

    def allowed(self, request, datum):
        allowed = super(RepeatTaskAction, self).allowed(request, datum)
        return allowed and datum.is_linkable

    def get_link_url(self, datum=None):
        params = urlencode({'object_id': datum.subject_id if datum else ''})
        args = [datum.timetable_id, datum.subject_type, datum.subject_group]
        return '?'.join([reverse(self.url, args=args), params])
