# coding: utf-8

import datetime
import json
import logging
import pytz

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from tionix_scheduler_client.client import Client as SchedulerClient
from tionix_scheduler_client.enums import ActionFieldsEnum

from tionix_dashboard.dashboards.helpers import Empty
from tionix_dashboard.dashboards.tionix.scheduler.enums import (
    TaskTypeEnum, RepeatingTypeEnum, EndTypeEnum,
    WeekDaysEnum)
from tionix_dashboard.helpers.utils import (
    memoize_by_keyword_arg, parse_datetime)

logger = logging.getLogger(__name__)


TASK_CREATION_TIME = 60 * 5

CRON_WORKING_DAYS_RANGE = '1-5'


# noinspection PyUnusedLocal
@memoize_by_keyword_arg(('type_name', 'group_name', 'cache_key'))
def get_available_actions(request, type_name, group_name, cache_key=None):
    """Retrieve for action type available tasks
    :param request: django request
    :param type_name: type of task subject
    :param group_name: group of task subject
    :param cache_key: key for cache
    :return: list of tasks id and name
    """
    scheduler_client = SchedulerClient(
        headers={'X-AUTH-TOKEN': request.user.token.id}
    )

    locale = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME,
                                 request.LANGUAGE_CODE)

    tasks = []
    try:
        tasks = scheduler_client.get_tasks_list(type_name, group_name, locale)
    except Exception as err:
        logger.debug(unicode(err))
    if tasks:
        tasks.sort(key=lambda item: item[1] if len(item) > 1 else '')

    return tasks


def get_cron_schedule(begin_date, task_type, repeat=None, interval=None,
                      week_days=None):
    every = '*'
    minute = begin_date.minute
    hour = begin_date.hour
    day_of_week = month_of_year = day_of_month = every

    if task_type == TaskTypeEnum.SINGLE:
        day_of_month = begin_date.day
        month_of_year = begin_date.month
    else:
        if repeat == RepeatingTypeEnum.DAYS:
            day_of_month = '{0}/{1}'.format(
                every, interval) if interval > 1 else every
        elif repeat == RepeatingTypeEnum.WORKING_DAYS:
            day_of_week = CRON_WORKING_DAYS_RANGE
        elif repeat == RepeatingTypeEnum.DAYS_OF_THE_WEEK:
            day_of_week = ','.join(week_days) if week_days else every
        elif repeat == RepeatingTypeEnum.YEAR:
            day_of_month = begin_date.day
            month_of_year = begin_date.month

    cron_dict = {
        'minute': str(minute),
        'hour': str(hour),
        'day_of_week': day_of_week,
        'day_of_month': str(day_of_month),
        'month_of_year': str(month_of_year),
    }

    return cron_dict


def get_interval_schedule(repeat=None, interval=None):
    interval_dict = {
        "every": interval,
        "period": repeat,
    }
    return interval_dict


def get_cron_str(cron_dict):
    return (
        '{minute} {hour} {day_of_month} '
        '{month_of_year} {day_of_week}').format(**cron_dict)


def get_task_expire_date(begin_date, task_type, end_type=None, end_date=None):
    if task_type == TaskTypeEnum.SINGLE:
        expire_date = datetime.datetime(
            begin_date.year, begin_date.month, begin_date.day, 23, 59, 59)
    elif task_type == TaskTypeEnum.REPEATING:
        if end_type in (EndTypeEnum.MAX_COUNT, EndTypeEnum.NEVER):
            expire_date = None
        elif end_type == EndTypeEnum.DATE:
            expire_date = datetime.datetime(
                end_date.year, end_date.month, end_date.day,
                23, 59, 59)
        else:
            raise ValueError(u'Invalid end_type: {0}'.format(end_type))
    else:
        raise ValueError(u'Invalid task_type: {0}'.format(task_type))

    return expire_date


def get_utc_datetime(local, date, time):
    native_datetime = datetime.datetime.combine(date, time)
    local_datetime = local.localize(native_datetime, is_dst=None)
    utc_datetime = local_datetime.astimezone(pytz.utc)
    return utc_datetime


def get_params_for_create(data):
    """
    Makes params dict for creating task.

    :param data: data for making dict
    :type data: dict

    :return: params
    :rtype: dict
    """
    begin_time = data.get('begin_time')
    begin_date = data.get('begin_date')
    if not begin_date:
        return False

    task_type = data.get('task_type')
    week_days = [
        WeekDaysEnum.cron_values.get(k)
        for k, v in WeekDaysEnum.form_values.iteritems() if data.get(v)
    ]
    end_date = data.get('end_date', '')
    run_count_limit = data.get('end_max_number', 0)

    subject_id = data.get('object_id', '')
    subject = data.get('object_name', '')
    subject_type = data.get('object_type', '')
    subject_group = data.get('object_group', '')
    unique_name = data.get('unique_name')

    local = pytz.timezone(data.get('timezone'))
    if begin_time is None:
        begin_time = (datetime.datetime.now() +
                      datetime.timedelta(seconds=TASK_CREATION_TIME)).time()

    utc_begin_date = get_utc_datetime(local, begin_date, begin_time)
    utc_end_date = (
        get_utc_datetime(local, end_date, begin_time) if end_date else None
    )

    params = {
        'author_name': data.get('author_name'),
        'tenant_id': data.get('tenant_id'),
        'unique_name': unique_name,
        'description': data.get('description'),
        'repeat': task_type == TaskTypeEnum.REPEATING,
        'subject_id': subject_id,
        'subject': subject,
        'subject_type': subject_type,
        'subject_group': subject_group,
        'task_type': task_type,
        'begin_date': utc_begin_date.strftime('%Y-%m-%d'),
        'begin_time': utc_begin_date.strftime('%H:%M:%S'),
        'timezone': data.get('timezone'),
        'arguments': json.dumps([
            data.get('user_name'),
            data.get('user_password'),
            data.get('tenant'),
            subject_id,
            subject,
            data.get('user_domain_name'),
        ]),
        'key_arguments': json.dumps(
            {field: data.get(field)
             for field in ActionFieldsEnum.fields.get(unique_name, [])}
        )
    }

    repeat = data.get('repeat', '')
    if task_type == TaskTypeEnum.SINGLE or repeat in RepeatingTypeEnum.CRONTAB:
        cron_dict = get_cron_schedule(begin_date=utc_begin_date,
                                      task_type=task_type,
                                      repeat=repeat,
                                      interval=data.get('interval'),
                                      week_days=week_days)
        params['crontab'] = cron_dict
    elif repeat in RepeatingTypeEnum.INTERVAL:
        interval_dict = get_interval_schedule(repeat=repeat,
                                              interval=data.get('interval'))
        params['interval'] = interval_dict
    else:
        raise TypeError(_('Unable to schedule task with unknown repeat type'))

    expire_date = get_task_expire_date(begin_date=utc_begin_date,
                                       task_type=task_type,
                                       end_type=data.get('end'),
                                       end_date=utc_end_date)
    params['expires'] = (
        expire_date.strftime('%Y-%m-%d %H:%M:%S') if expire_date else None
    )
    params['run_count_limit'] = run_count_limit if run_count_limit else None
    return params


def schedule_task(request, data):
    """
    Makes a schedule for task.

    :param request: django request
    :param data: data for making task
    :type data: dict

    :return: True if succeed
    :rtype: bool
    """
    scheduler_client = SchedulerClient(
        headers={'X-AUTH-TOKEN': request.user.token.id}
    )
    result = True
    try:
        params = get_params_for_create(data)
        scheduler_client.create_timetable(params)
    except Exception as err:
        logger.error(unicode(err))
        result = False

    return result


def get_scheduled_tasks_list(auth_token, params, use_post=False):
    """Retrieve all scheduled tasks"""
    scheduler_client = SchedulerClient(headers={'X-AUTH-TOKEN': auth_token})
    try:
        result = scheduler_client.get_monitoring_tasks_list(params,
                                                            use_post=use_post)
    except Exception as err:
        logger.debug(unicode(err))
        result = []

    return result


def get_task_results(auth_token, params):
    """Retrieve results of task"""
    total_count = 0
    formatted_task_results = []
    scheduler_client = SchedulerClient(headers={'X-AUTH-TOKEN': auth_token})
    try:
        response = scheduler_client.get_task_results(**params)
    except Exception as err:
        logger.debug(unicode(err))
    else:
        tasks = []
        # pylint: disable=no-member
        if isinstance(response, dict):
            tasks = response.get('rows', [])
            total_count = response.get('total_count', 0)

        formatted_task_results = [Empty(**task) for task in tasks]

    return formatted_task_results, total_count


def delete_scheduled_tasks_list(auth_token, params):
    """
    Delete scheduled tasks
    :raises tionix_scheduler_client.client.SchedulerClientHttpException
    """
    scheduler_client = SchedulerClient(headers={'X-AUTH-TOKEN': auth_token})
    scheduler_client.delete_monitoring_tasks_list(params)


def get_scheduled_tasks(request, offset, limit, query_params, sort_key=None,
                        use_post=False):
    """Return list of scheduled tasks"""

    if query_params is None:
        query_params = {}

    query_params.update({
        'offset': offset,
        'limit': limit,
        'locale': request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME,
                                      request.LANGUAGE_CODE),
        'sort_key': sort_key or '-id',
        'tenant_id': request.user.tenant_id,
    })

    response = get_scheduled_tasks_list(
        auth_token=request.user.token.id,
        params=query_params,
        use_post=use_post
    )

    tasks, total_count = [], 0
    # pylint: disable=no-member
    if isinstance(response, dict):
        tasks = response.get('rows', [])
        total_count = response.get('total_count', 0)

    formatted_tasks = [Empty(**task) for task in tasks]

    prev = more = True
    if offset == 0:
        prev = False
    if limit + offset >= total_count:
        more = False

    return formatted_tasks, prev, more, total_count


def parse_repeating_task(task, periodic_task):
    """
    Return fields value to fill repeat fields
    :param task: dict
    :param periodic_task: dict
    :return:
    """

    fields = {
        'task_type': TaskTypeEnum.REPEATING
    }

    # Interval fields
    crontab = periodic_task.get('crontab')
    interval = periodic_task.get('interval')
    if crontab:
        every = '*'

        day_of_week = crontab.get('day_of_week')
        day_of_month = crontab.get('day_of_month')
        month_of_year = crontab.get('month_of_year')

        if day_of_month != every and month_of_year != every:
            fields['repeat'] = RepeatingTypeEnum.YEAR
        elif day_of_week == CRON_WORKING_DAYS_RANGE:
            fields['repeat'] = RepeatingTypeEnum.WORKING_DAYS
        elif day_of_week != every:
            fields['repeat'] = RepeatingTypeEnum.DAYS_OF_THE_WEEK
            days = day_of_week.split(',')
            for day in days:
                value = WeekDaysEnum.reverse_cron_values.get(day)
                fields[WeekDaysEnum.form_values.get(value)] = True
        else:
            fields['repeat'] = RepeatingTypeEnum.DAYS
            if '/' in day_of_month:
                every, interval = day_of_month.split('/')
                fields['interval'] = interval
    elif interval and interval.get('period') in RepeatingTypeEnum.INTERVAL:
        fields['repeat'] = interval.get('period')
        fields['interval'] = interval.get('every')
    else:
        raise TypeError(_('Unable to open repeat task. Type is unknown.'))

    # End fields
    run_count_limit = task.get('run_count_limit')
    if run_count_limit:
        fields['end'] = EndTypeEnum.MAX_COUNT
        fields['end_max_number'] = run_count_limit
    else:
        expire_date = periodic_task.get('expires')
        if expire_date:
            fields['end'] = EndTypeEnum.DATE
            fields['end_date'] = parse_datetime(expire_date)
        else:
            fields['end'] = EndTypeEnum.NEVER

    return fields
