# coding: utf-8

import logging

from django import http
from django.utils.translation import ugettext_lazy as _

from jsonfield import forms as json_forms

from horizon import exceptions, forms, messages

from tionix_dashboard.dashboards.tionix.scheduler.api import (
    delete_scheduled_tasks_list)
from tionix_dashboard.helpers.fields import BootstrapSwitcherField
from tionix_dashboard.helpers.forms import JSONWidgetHidden

logger = logging.getLogger(__name__)


class ReadOnlyMixin(object):
    """Read only mixin"""
    def __init__(self, attrs=None):
        assert isinstance(self, forms.Widget)
        attrs = attrs or {}
        attrs['readonly'] = 'readonly'
        super(ReadOnlyMixin, self).__init__(attrs=attrs)


class ReadOnlyTextInput(ReadOnlyMixin, forms.TextInput):
    """TextInput readonly"""


class ReadOnlyTextArea(ReadOnlyMixin, forms.Textarea):
    """TextArea readonly"""


class DetailCharField(forms.CharField):
    """Disabled char field"""

    widget = ReadOnlyTextInput

    def __init__(self, *args, **kwargs):
        super(DetailCharField, self).__init__(required=False, *args, **kwargs)

    def prepare_value(self, value):
        """return value to display"""
        return value or _('-')


# noinspection PyAbstractClass
class TaskDetailsForm(forms.SelfHandlingForm):
    """
    Task details form.
    """
    id = DetailCharField(label=_('ID'))
    name = DetailCharField(label=_('Task name'))
    author_name = DetailCharField(label=_('Author'))
    created = DetailCharField(label=_('Created Date and Time'))
    end_time = DetailCharField(label=_('End Date and Time'))
    timezone = DetailCharField(label=_('Timezone'))
    schedule = DetailCharField(label=_('Schedule'))
    begin = DetailCharField(label=_('Start Date and Time'))
    total_run_count = DetailCharField(label=_('Total run count'))
    last_run_time = DetailCharField(label=_('Last run time'))
    next_run_time = DetailCharField(label=_('Next run time'))
    run_result = DetailCharField(label=_('Result'),
                                 widget=ReadOnlyTextArea(attrs={'rows': 2}))
    summary = DetailCharField(label=_('Summary'))

    def left_fields(self):
        return [self['name'],
                self['author_name'],
                self['begin'],
                self['timezone'],
                self['last_run_time']]

    def right_fields(self):
        return [self['id'],
                self['created'],
                self['end_time'],
                self['total_run_count'],
                self['next_run_time']]

    def common_fields(self):
        return [self['summary'],
                self['run_result']]

    # noinspection PyUnusedLocal,PyMethodMayBeStatic
    def handle(self, request, data):
        return True


# noinspection PyArgumentList
class SwitcherForm(forms.Form):
    switcher = forms.BooleanField(
        label='',
        required=False,
        widget=BootstrapSwitcherField(attrs={
            'data-provide-boostrapswitcher': 'switcher',
            'data-handle-width': '60px',
            'data-on-text': _('ON'),
            'data-off-text': _('OFF'),
        })
    )

    def __init__(self, initial_value, **kwargs):
        super(SwitcherForm, self).__init__(**kwargs)
        self.fields['switcher'].initial = initial_value


class SchedulerCalendarForm(forms.Form):
    """Scheduler calendar"""
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(SchedulerCalendarForm, self).__init__(*args, **kwargs)


# pylint: disable=no-member
# noinspection PyAbstractClass
class DeleteObjectWithTasksForm(forms.SelfHandlingForm):
    """ Delete object with scheduled tasks form"""

    columns = [
        _('ID'), _('Name'), _('Action'), _('Task Type'),
        _('Status'), _('Object Type'), _('Object Name'),
    ]

    object_type = None

    object_names_dict = json_forms.JSONFormField(widget=JSONWidgetHidden,
                                                 required=False)

    def delete(self, request, object_id):
        raise NotImplementedError()

    @staticmethod
    def get_object_id(object_id):
        return object_id

    # noinspection PyUnusedLocal
    def handle(self, request, data):
        redirect_url = self.initial['redirect_url']
        object_names_dict = (
            self.initial['object_names_dict'] or data['object_names_dict']
        )
        ids = self.initial['ids']
        tasks_rows = self.initial['rows']

        success_names, failed_names = [], []
        for object_id in ids:
            object_id = self.get_object_id(object_id)
            if tasks_rows.get(object_id):
                query_params = {
                    'subject_id': object_id,
                    'subject_type': self.object_type,
                }
                try:
                    delete_scheduled_tasks_list(
                        auth_token=request.user.token.id,
                        params=query_params)
                except Exception as err:
                    logger.error(unicode(err))
                    exceptions.handle(self.request,
                                      _('Unable to delete tasks for '
                                        'object "%s".') % object_id,
                                      redirect=redirect_url)
            object_name = object_names_dict[object_id]
            try:
                self.delete(request, object_id)
                success_names.append(object_name)
            except Exception as err:
                logger.error(unicode(err))
                failed_names.append(object_name)

        success_message_level = messages.success
        if failed_names:
            msg = _('Unable to delete: "{0}"').format(
                u'", "'.join(failed_names))
            messages.error(request, msg)
            success_message_level = messages.info
        if success_names:
            msg = _('Deleted: "{0}"').format(u'", "'.join(success_names))
            success_message_level(request, msg)

        if failed_names:
            return http.HttpResponseRedirect(redirect_url)

        return True
