# coding: utf-8

import pytz

from django import template
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon.utils import memoized

from openstack_dashboard import api

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.helpers.tables import (
    TnxPaginationDataTable, ColoredRowMixin)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    FilterAction, SchedulerUpdateRow, ShowTaskAction, DeleteTaskAction,
    EXECUTION_STATE_DISPLAY_CHOICES, TYPE_DISPLAY_CHOICES,
    SUBJECT_TYPE_DISPLAY_CHOICES, TabFilterAction, RepeatTaskAction)
from tionix_dashboard.dashboards.tionix.scheduler.forms import (
    SwitcherForm)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    localize_datetime, transform_date)
from tionix_dashboard.helpers.utils import add_kwargs
from tionix_dashboard.url_resolve_map import (
    IDENTITY_PROJECT_DETAIL, ADMIN_SERVERS_DETAIL, PROJECT_SERVERS_DETAIL,
    VDI_INSTANCE_DETAILS, SCHEDULER_TASK_DETAIL)


def get_url_for_task(datum):
    return (reverse(SCHEDULER_TASK_DETAIL, args=(datum.timetable_id,
                                                 datum.name))
            if datum.repeat else None)


class BaseTasksTable(TnxPaginationDataTable):
    """Base table for scheduled tasks"""

    timetable_id = tables.Column('timetable_id', verbose_name=_('ID'))
    name = tables.Column('name',
                         verbose_name=_('Task name'),
                         link=get_url_for_task,)
    action = tables.Column('action', verbose_name=_('Action'))
    type = tables.Column(lambda x: unicode(x.repeat),
                         verbose_name=_('Type'),
                         display_choices=TYPE_DISPLAY_CHOICES)
    last_run_result = tables.Column(
        'last_run_result',
        verbose_name=_('Last execution state'),
        display_choices=EXECUTION_STATE_DISPLAY_CHOICES)

    def get_redirect_url(self, first_page=False):
        self.request.session['is_redirect'] = True
        return self.get_first_page_url() if first_page else self.get_full_url()

    def get_first_page_url(self):
        """
        Returns url without page info params, with switcher
        """
        url = self.request.get_full_path().partition('?')[0]

        extra_string = self.get_extra_pagination_string()
        if extra_string:
            url = '{0}?{1}'.format(url, extra_string)

        return url

    def get_object_id(self, datum):
        return datum.timetable_id

    def get_object_display(self, datum):
        return getattr(datum, 'timetable_id', datum)

    # pylint: disable=no-init
    class Meta(object):
        name = 'scheduler'
        verbose_name = _('Schedule')
        row_class = SchedulerUpdateRow
        table_actions = (FilterAction, DeleteTaskAction)
        row_actions = (ShowTaskAction, DeleteTaskAction, RepeatTaskAction)
        css_classes = 'tablesorter tablesorter-server'


link_map = {
    (ObjectTypeEnum.hypervisor, ObjectGroupEnum.admin, True):
        'horizon:admin:hypervisors:detail',
    (ObjectTypeEnum.hypervisor, ObjectGroupEnum.admin, False):
        'horizon:admin:hypervisors:detail',
    (ObjectTypeEnum.instance, ObjectGroupEnum.admin, True):
        ADMIN_SERVERS_DETAIL,
    (ObjectTypeEnum.instance, ObjectGroupEnum.admin, False):
        PROJECT_SERVERS_DETAIL,
    (ObjectTypeEnum.volume, ObjectGroupEnum.admin, True):
        'horizon:admin:volumes:volumes:detail',
    (ObjectTypeEnum.volume, ObjectGroupEnum.admin, False):
        'horizon:project:volumes:volumes:detail',
    (ObjectTypeEnum.instance, ObjectGroupEnum.project, True):
        ADMIN_SERVERS_DETAIL,
    (ObjectTypeEnum.instance, ObjectGroupEnum.project, False):
        PROJECT_SERVERS_DETAIL,
    (ObjectTypeEnum.volume, ObjectGroupEnum.project, True):
        'horizon:admin:volumes:volumes:detail',
    (ObjectTypeEnum.volume, ObjectGroupEnum.project, False):
        'horizon:project:volumes:volumes:detail',
    (ObjectTypeEnum.instance, ObjectGroupEnum.vdi, True):
        VDI_INSTANCE_DETAILS,
    (ObjectTypeEnum.instance, ObjectGroupEnum.vdi, False):
        VDI_INSTANCE_DETAILS,
    (ObjectTypeEnum.project, ObjectGroupEnum.identity, True):
        'horizon:identity:projects:detail',
    (ObjectTypeEnum.project, ObjectGroupEnum.vdi, True):
        'horizon:tionix:vdi:projects:detail',
}


def get_subject_link(datum, **kwargs):
    is_admin = kwargs.get('is_admin', False)

    link = None
    link_key = (datum.subject_type, datum.subject_group, is_admin)
    if datum.is_linkable and link_key in link_map:
        link = link_map.get(link_key)

        if (datum.subject_type == ObjectTypeEnum.instance and
                datum.subject_group == ObjectGroupEnum.vdi):
            link = reverse(link, args=[datum.subject_tenant_id,
                                       datum.subject_id])
        else:
            link = reverse(link, args=[datum.subject_id])

    return link


def get_url_for_project(datum):
    tenant_id = getattr(datum, 'subject_tenant_id', None)
    return (reverse(IDENTITY_PROJECT_DETAIL, args=(tenant_id,))
            if tenant_id else None)


class IndexTasksTable(ColoredRowMixin, BaseTasksTable):
    """
    The main table for scheduler
    """

    STATUS_CLASSES = {
        True: 'gray_color'
    }

    subject_type = tables.Column(
        'subject_type',
        verbose_name=_('Subject type'),
        display_choices=SUBJECT_TYPE_DISPLAY_CHOICES)
    subject = tables.Column(
        'subject',
        verbose_name=_('Subject title'),
        link=get_subject_link)
    subject_tenant = tables.Column(
        'subject_tenant',
        verbose_name=_('Project'),
        link=get_url_for_project)
    deleted = tables.Column(
        'deleted',
        verbose_name=_('Deleted'),
        hidden=True)

    switcher_param = 'switcher'

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(IndexTasksTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)

        is_admin = api.keystone.is_domain_admin(request)
        self.columns['subject'].get_link_url = add_kwargs(get_subject_link,
                                                          is_admin=is_admin)

        # Project column is visible only for admin
        if not is_admin:
            self.columns.pop('subject_tenant')

    def render_switcher(self):
        switcher_template = template.loader.get_template(
            'scheduler/_switcher.html'
        )
        show_deleted = self.request.session.get(self.switcher_param)
        extra_context = {
            'form': SwitcherForm(show_deleted),
            'table': self
        }
        context = template.RequestContext(self.request, extra_context)
        return switcher_template.render(context)

    @memoized.memoized_method
    def get_extra_pagination_string(self):
        string = ''
        if self.switcher_param in self.request.GET:
            string = '{0}={1}'.format(
                self.switcher_param,
                self.request.GET.get(self.switcher_param))
        return string

    class Meta(BaseTasksTable.Meta):
        name = 'index_scheduler'
        template = 'scheduler/_scheduler_data_table.html'
        status_columns = ['deleted']


def transform_begin_datetime(task):
    begin_date = getattr(task, 'begin_date')
    begin_time = getattr(task, 'begin_time')
    timezone = getattr(task, 'timezone', 'UTC')
    tz = pytz.timezone(timezone)
    return localize_datetime(tz, begin_date, begin_time)


class TabTasksTable(BaseTasksTable):
    created = tables.Column(transform_date('created_at'),
                            verbose_name=_('Created Date and Time'))
    begin = tables.Column(transform_begin_datetime,
                          verbose_name=_('Start Date and Time'))

    def get_full_url(self):
        return (self.kwargs.get('scheduler_tab_url') or
                super(TabTasksTable, self).get_full_url())

    @memoized.memoized_method
    def get_extra_pagination_string(self):
        tab = self.request.GET.get('tab')
        return 'tab={0}'.format(tab) if tab else ''

    class Meta(BaseTasksTable.Meta):
        name = 'scheduler'
        table_actions = (TabFilterAction,
                         DeleteTaskAction)


class TnxTaskResultsTable(TnxPaginationDataTable):
    serial_number = tables.Column('serial_number',
                                  verbose_name=_('Number'))
    start_datetime = tables.Column(transform_date('start_datetime'),
                                   verbose_name=_('Start Date and Time'))
    end_datetime = tables.Column(transform_date('end_datetime'),
                                 verbose_name=_('End Date and Time'))
    status = tables.Column("task_status",
                           verbose_name=_("Status"),
                           display_choices=EXECUTION_STATE_DISPLAY_CHOICES)
    result_message = tables.Column("result_message",
                                   verbose_name=_("Result"),
                                   attrs={'style': 'width: 50%'})

    class Meta(object):
        name = "task_results"
        verbose_name = _("Task Results")
