# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import tabs, exceptions

from openstack_dashboard import api as os_api

from tionix_scheduler_client.enums import ObjectTypeEnum

from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.helpers.views import (
    DTVTnxPaginationMixin, ServerFilterTabMixin)
from tionix_dashboard.dashboards.tionix.scheduler import api
from tionix_dashboard.dashboards.tionix.scheduler.tables import (
    TabTasksTable, TnxTaskResultsTable)

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class ActiveTasksTab(ServerFilterTabMixin,
                     DTVTnxPaginationMixin,
                     tabs.TableTab):

    name = _("Scheduled Tasks")
    slug = "scheduler"
    table_classes = (TabTasksTable, )
    template_name = "horizon/common/_detail_table.html"
    preload = False

    sort_map = {
        'begin': ('begin_date', 'begin_time')
    }

    def get_scheduler_data(self):

        obj_type = getattr(self.tab_group, 'obj_type', '')
        obj = self.tab_group.kwargs.get(obj_type)

        offset, limit = self.get_offset_limit()

        sort_dir = self.request.session.get(self.sort_dir_param)
        sort_key = self.request.session.get(self.sort_key_param)

        if sort_key:
            sort_key = self.sort_map.get(sort_key, sort_key)
            if not isinstance(sort_key, (list, tuple)):
                sort_key = (sort_key, )
            sort_key = [''.join(['-' if sort_dir == 'desc' else '', key])
                        for key in sort_key]

        is_admin = os_api.keystone.is_domain_admin(self.request)

        query_params = self.get_filters()
        query_params.update(
            subject_id=obj.id if obj else '',
            subject_type=obj_type,
            all_tenants=is_admin
        )

        tasks, self._prev, self._more, self.table.total_count = (
            api.get_scheduled_tasks(self.request, offset, limit, query_params,
                                    sort_key)
        )

        is_linkable = SchedulingPreChecker.is_schedulable(obj)
        for task in tasks:
            task.is_linkable = is_linkable
            if task.subject_type == ObjectTypeEnum.instance:
                tenant_id = obj.tenant_id
            elif task.subject_type == ObjectTypeEnum.volume:
                tenant_id = getattr(obj, "os-vol-tenant-attr:tenant_id", None)
            else:
                continue
            task.subject_tenant_id = tenant_id

        # Fist, last pages identifiers
        self.table.first_marker, self.table.last_marker = (
            0, self.calc_last_marker(self.table.total_count, limit)
        )

        return tasks


# noinspection PyAbstractClass
class DetailTab(ServerFilterTabMixin, DTVTnxPaginationMixin, tabs.TableTab):
    name = _("Task Results")
    slug = "task_results"
    table_classes = (TnxTaskResultsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_task_results_data(self):
        results = []
        offset, limit = self.get_offset_limit()
        try:
            query_params = {
                'timetable_id': self.tab_group.kwargs['id'],
                'limit': limit,
                'offset': offset
            }
            results, self.table.total_count = api.get_task_results(
                auth_token=self.request.user.token.id, params=query_params)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request, _('Unable to retrieve task result list.'))
        else:
            if offset == 0:
                self._prev = False

            if (offset + len(results)) == self.table.total_count:
                self._more = False

            self.table.first_marker, self.table.last_marker = (
                0, self.calc_last_marker(self.table.total_count, limit)
            )

        return results


# noinspection PyAbstractClass
class TaskDetailTabs(tabs.TabGroup):
    slug = "task_details"
    tabs = (DetailTab,)
