# coding: utf-8

import datetime
import json
import logging
import uuid

import pytz

from cron_descriptor import FormatException, Options

from django import shortcuts
from django.conf import settings
from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _

from novaclient.exceptions import NotFound

from openstack_dashboard import api as os_api

from horizon import tables, workflows, forms, exceptions, messages, tabs

from tionix_scheduler_client.client import Client as SchedulerClient
from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients.base import novaclient, tenant_list
from tionix_dashboard.dashboards.helpers import Empty
from tionix_dashboard.dashboards.identity.projects.utils import (
    get_project_group_for_schedule)
from tionix_dashboard.dashboards.tionix.scheduler.api import (
    get_scheduled_tasks_list, parse_repeating_task)
from tionix_dashboard.dashboards.tionix.scheduler.enums import TaskTypeEnum
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker, localize_datetime, get_utc_date,
    get_pretty_timezone, get_task_summary, OBJECTS_CLIENT_MAP)
from tionix_dashboard.dashboards.utils import TABLE_OBJECT_ID_SEP
from tionix_dashboard.helpers.cron import get_description
from tionix_dashboard.helpers.utils import get_table_filter_action
from tionix_dashboard.helpers.views import (
    DTVTnxPaginationMixin, FilterStringModifierMixin, LinkBatchFormView)
from tionix_dashboard.dashboards.tionix.scheduler import api
from tionix_dashboard.dashboards.tionix.scheduler.forms import (
    TaskDetailsForm, SchedulerCalendarForm)
from tionix_dashboard.dashboards.tionix.scheduler.tables import (
    IndexTasksTable)
from tionix_dashboard.dashboards.tionix.scheduler.tabs import (
    TaskDetailTabs)
from tionix_dashboard.dashboards.tionix.scheduler.workflows import (
    SchedulerWorkflow, RepeatSchedulerWorkflow)
from tionix_dashboard.templatetags.tionix_tags import format_date

logger = logging.getLogger(__name__)

MIN_TASK_ARGS_COUNT = 3


# pylint: disable=too-many-statements
class SchedulerView(FilterStringModifierMixin,
                    DTVTnxPaginationMixin,
                    tables.DataTableView):
    """
    The index page for all scheduled tasks
    """
    table_class = IndexTasksTable
    template_name = 'scheduler/index.html'
    page_title = _('Scheduled Tasks')

    objects_map = OBJECTS_CLIENT_MAP

    sort_map = {
        'tenant': 'tenant_id'
    }

    # noinspection PyUnboundLocalVariable
    def get_data(self):
        """
        Add data to the context
        """
        switcher = self.request.session.get('switcher')
        sort_dir = self.request.session.get(self.sort_dir_param)
        sort_key = self.request.session.get(self.sort_key_param)

        if sort_key:
            sort_key = ''.join([
                '-' if sort_dir == 'desc' else '',
                self.sort_map.get(sort_key, sort_key)
            ])

        offset, limit = self.get_offset_limit()

        is_admin = os_api.keystone.is_domain_admin(self.request)

        query_params = self.get_filters()

        tenants = {}
        if is_admin:
            tenant_filter = query_params.pop('tenant', None)
            if os_api.keystone.VERSIONS.active >= 3 and tenant_filter:
                filters = {'name': tenant_filter}
            else:
                filters = {}
            try:
                tenants = {tenant.id: tenant.name for tenant in
                           tenant_list(self.request, filters=filters)}
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to get projects list.'))
            else:
                if tenant_filter:
                    query_params.update(
                        filter_tenants=tenants.keys() or [tenant_filter]
                    )

        if switcher:
            query_params.update(show_deleted=True)

        query_params.update(all_tenants=is_admin)

        tasks, self._prev, self._more, self.table.total_count = (
            api.get_scheduled_tasks(
                self.request, offset, limit, query_params, sort_key)
        )

        try:
            servers, _more = os_api.nova.server_list(self.request,
                                                     all_tenants=is_admin)
            servers = {server.id: server.tenant_id for server in servers}
        except Exception as err:
            logger.error(unicode(err))
            servers = {}
        try:
            volumes = os_api.cinder.volume_list(
                self.request,
                search_opts={'all_tenants': is_admin}
            )
            volumes = {
                volume.id: getattr(volume,
                                   "os-vol-tenant-attr:tenant_id",
                                   None)
                for volume in volumes
            }
        except Exception as err:
            logger.error(unicode(err))
            volumes = {}

        hypervisors = {}
        if is_admin:
            try:
                hypervisors = {
                    str(hypervisor.id): hypervisor
                    for hypervisor in os_api.nova.hypervisor_list(self.request)
                }
            except Exception as err:
                logger.error(unicode(err))

        for task in tasks:
            if task.subject_type == ObjectTypeEnum.instance:
                task.is_linkable = task.subject_id in servers
                tenant_id = servers.get(task.subject_id)
            elif task.subject_type == ObjectTypeEnum.volume:
                task.is_linkable = task.subject_id in volumes
                tenant_id = volumes.get(task.subject_id)
            elif task.subject_type == ObjectTypeEnum.hypervisor:
                task.is_linkable = is_admin and task.subject_id in hypervisors
                continue
            elif task.subject_type == ObjectTypeEnum.project:
                task.is_linkable = is_admin and task.subject_id in tenants
                continue
            task.subject_tenant_id = tenant_id
            task.subject_tenant = tenants.get(tenant_id, tenant_id)

        # Fist, last pages identifiers
        self.table.first_marker, self.table.last_marker = (
            0, self.calc_last_marker(self.table.total_count, limit)
        )

        return tasks

    def update_query_filter_action(self):
        filter_action = get_table_filter_action(self.get_table())
        if filter_action is None or filter_action.filter_type != 'query':
            return None
        param_name = filter_action.get_param_name()
        filter_string = self.request.GET.get(param_name)
        setattr(filter_action, 'filter_string', filter_string)

    def _get_data_dict(self):
        data = super(SchedulerView, self)._get_data_dict()
        self.update_query_filter_action()
        return data

    def post(self, request, *args, **kwargs):
        # If the server side table filter changed then go back to the first
        # page of data. Otherwise GET and POST handling are the same.
        if self.handle_server_filter(request):
            self.handler_server_params(request)
            request.session[self.redirect_param_name] = True
            return shortcuts.redirect(self.get_table().get_redirect_url(True))
        return self.get(request, *args, **kwargs)

    def handler_server_params(self, request):
        switcher = request.POST.get('switcher', '')
        if switcher:
            request.session['switcher'] = switcher
        elif ('switcher' in request.session and
              self.redirect_param_name not in request.session):
            request.session.pop('switcher')
        super(SchedulerView, self).handler_server_params(request)


class SchedulerWorkflowView(workflows.WorkflowView):
    """
    The workflow view to schedule tasks
    """

    workflow_class = SchedulerWorkflow
    template_name = 'scheduler/_workflow_base.html'
    ajax_template_name = 'scheduler/_workflow.html'

    def get_initial(self):
        initial = super(SchedulerWorkflowView, self).get_initial()
        tenant_id = self.request.user.tenant_id
        if tenant_id:
            initial.update({'tenant': tenant_id})
        initial.update(self.kwargs)
        return initial


class RetrieveTimetableMixin(object):
    redirect_url = 'horizon:tionix:scheduler:index'

    def get_object(self):
        timetable = {}
        if not hasattr(self, '_object'):
            msg = _('Unable to retrieve the task.')
            scheduler_client = SchedulerClient(
                headers={'X-AUTH-TOKEN': self.request.user.token.id}
            )
            try:
                timetable = scheduler_client.get_timetable(self.kwargs['id'])
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request, msg)
                raise exceptions.Http302(self.redirect_url)

        return timetable


class TnxTaskDetailView(RetrieveTimetableMixin,
                        tabs.TabbedTableView):
    tab_group_class = TaskDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ task_name }}"

    def get_object(self):
        timetable = super(TnxTaskDetailView, self).get_object()

        object_getter = OBJECTS_CLIENT_MAP.get(timetable['subject_type'])
        is_linkable = False
        if object_getter:
            try:
                obj = object_getter(self.request, timetable['subject_id'])
            except Exception as err:
                logger.error(unicode(err))
            else:
                is_linkable = SchedulingPreChecker.is_schedulable(obj)
        timetable['is_linkable'] = is_linkable
        return timetable

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(TnxTaskDetailView, self).get_context_data(**kwargs)
        timetable = self.get_object()
        context['timetable'] = timetable
        table = IndexTasksTable(self.request)
        context["url"] = reverse_lazy('horizon:tionix:scheduler:index')
        context["actions"] = table.render_row_actions(Empty(**timetable))
        return context


class ShowTaskDetailsView(forms.ModalFormView, RetrieveTimetableMixin):
    """
    Show details about task.
    """
    initial = {}
    form_class = TaskDetailsForm
    success_url = 'horizon:tionix:scheduler:index'

    modal_id = 'task_details_modal'
    modal_header = _('Show task details')

    form_id = 'task_details_form'

    submit_url = 'horizon:tionix:scheduler:index'
    submit_label = 'Ok'

    cancel_url = 'horizon:tionix:scheduler:index'
    cancel_label = _('Close')

    template_name = 'scheduler/show.html'

    page_title = _('Task details')

    def get_context_data(self, **kwargs):
        context = super(ShowTaskDetailsView, self).get_context_data(**kwargs)
        context['id'] = self.kwargs['id']
        return context

    def get_initial(self):
        task = self.get_object()
        periodic_task = task.get('periodic_task')
        interval = periodic_task.get('interval')
        crontab = periodic_task.get('crontab')
        total_run_count = periodic_task.get('total_run_count')
        run_count_limit = task.get('run_count_limit')
        last_run_time = periodic_task.get('last_run_at')
        next_run_time = periodic_task.get('next_run_time')

        if crontab is not None:

            code = self.request.COOKIES.get(
                settings.LANGUAGE_COOKIE_NAME,
                self.request.LANGUAGE_CODE)

            schedule_str = '{0} {1} {2} {3} {4}'.format(
                crontab.get('minute'),
                crontab.get('hour'),
                crontab.get('day_of_month'),
                crontab.get('month_of_year'),
                crontab.get('day_of_week')
            )

            options = Options()
            options.use_24hour_time_format = True
            try:
                schedule_desc = get_description(schedule_str,
                                                options=options,
                                                lang_code=code)
            except FormatException:
                schedule_desc = schedule_str
        else:
            schedule_desc = 'every {} {}'.format(
                interval.get('every'),
                interval.get('period'))

        if run_count_limit:
            schedule = '{0}, {1} {2}'.format(
                schedule_desc, run_count_limit, _('times')
            )
        else:
            schedule = schedule_desc

        timezone = task.get('timezone')
        tz = pytz.timezone(timezone)

        begin_date = task.get('begin_date')
        if begin_date is not None:
            begin_date = localize_datetime(tz, begin_date,
                                           task.get('begin_time'))

        if last_run_time:
            last_run_time = format_date(date_string=last_run_time, tz=tz)

        if next_run_time:
            next_run_time = format_date(date_string=next_run_time, tz=tz)

        try:
            summary = get_task_summary(task, tz)
        except Exception as err:
            logger.error(err)
            summary = err.message

        end_time = format_date(date_string=periodic_task.get('expires'), tz=tz)
        if not task.get('repeat'):
            end_time = '-'

        return {
            'id': self.kwargs['id'],
            'name': task.get('description'),
            'author_name': task.get('author_name'),
            'created': format_date(date_string=task.get('created_at'), tz=tz),
            'end_time': end_time,
            'timezone': get_pretty_timezone(timezone),
            'schedule': schedule,
            'begin': begin_date,
            'total_run_count': total_run_count,
            'last_run_time': last_run_time,
            'next_run_time': next_run_time,
            'run_result': task.get('last_run_message'),
            'summary': summary
        }


def get_tenant_object_group(request, tenant):
    """Return project type"""
    object_group = ObjectGroupEnum.identity
    tenant_id = tenant.id
    nova_client_kwargs = {}
    if not os_api.keystone.is_domain_admin(request):
        nova_client_kwargs['project_id'] = tenant_id
    nova_client = novaclient(request, **nova_client_kwargs)
    try:
        tenant = nova_client.tnx_tenants.get(tenant_id)
    except NotFound:
        pass
    else:
        object_group = get_project_group_for_schedule(tenant)
    return object_group


class SchedulerCalendarView(LinkBatchFormView):
    """
    View represents scheduler calendar
    """
    form_class = SchedulerCalendarForm
    modal_header = _('Schedule')
    template_name = 'scheduler/calendar.html'
    modal_id = "scheduler_calendar_id"
    page_title = _("Hypervisor extra information")
    cancel_label = _('Close')

    schedule_task_url = 'horizon:tionix:scheduler:schedule_task'

    objects_map = OBJECTS_CLIENT_MAP

    @staticmethod
    def get_begin_date(task):
        tz = pytz.timezone(task.get('timezone'))
        utc_datetime = get_utc_date(task.get('begin_date'),
                                    time=task.get('begin_time'))
        local_datetime = utc_datetime.astimezone(tz)

        return local_datetime.isoformat()

    def get_scheduler_task_list(self, object_type, object_group, object_ids):

        is_admin = os_api.keystone.is_domain_admin(self.request)

        query_params = {
            'subject_type': object_type,
            'subject_group': object_group,
            'subject_id': object_ids,
            'locale': self.request.COOKIES.get(
                settings.LANGUAGE_COOKIE_NAME,
                self.request.LANGUAGE_CODE),
            'tenant_id': self.request.user.tenant_id,
            'all_tenants': is_admin,
        }
        tasks = get_scheduled_tasks_list(
            auth_token=self.request.user.token.id,
            params=query_params
        )

        if tasks:
            tasks = [
                {'title': u'{0} ({1})'.format(task['action'], task['subject']),
                 'type': 'info',
                 'fullStartsAt': self.get_begin_date(task)}
                for task in tasks['rows']]

        return tasks

    def get_group_by_object(self, obj, object_type):
        """Return type by object"""
        objects_type_map = {
            ObjectTypeEnum.project: get_tenant_object_group,
        }

        if object_type in objects_type_map:
            return objects_type_map[object_type](self.request, obj)

        return None

    def validate_actions(self, object_type, object_group):
        """Check available actions for object"""
        user_action_uuid = uuid.uuid4()
        self.request.session['user_action_uuid'] = user_action_uuid
        if not SchedulingPreChecker.is_actions_not_empty(
                request=self.request, group_name=object_group,
                type_name=object_type):
            self.request.session.pop('user_action_uuid')
            messages.error(self.request,
                           message=_('Actions for object(s) not found.'))
            raise exceptions.Http302(self.get_redirect_url())

    # pylint: disable=attribute-defined-outside-init
    def get_object_ids_and_type(self):
        object_type = self.kwargs['object_type']
        default_object_group = self.kwargs['object_group']

        object_ids = self.get_object_ids()
        valid_ids, invalid_ids, object_groups = [], [], set()
        for object_id in object_ids:
            if (object_type == ObjectTypeEnum.instance and
                    default_object_group == ObjectGroupEnum.vdi):
                object_id = object_id.split(TABLE_OBJECT_ID_SEP)[1]
            if object_type in self.objects_map:
                try:
                    obj = self.objects_map[object_type](
                        self.request, object_id
                    )
                except Exception as err:
                    logger.error(unicode(err))
                else:

                    if not SchedulingPreChecker.is_schedulable(obj):
                        invalid_ids.append(obj.name or object_id)
                        continue

                    object_group = self.get_group_by_object(obj, object_type)

                    if object_group:
                        object_groups.add(object_group)

                        if len(object_groups) != 1:
                            messages.error(
                                self.request,
                                message=_('Scheduling is not available for'
                                          ' different types of objects.'))
                            raise exceptions.Http302(self.get_redirect_url())

            valid_ids.append(object_id)

        if invalid_ids:
            messages.error(
                self.request,
                # pylint: disable=no-member
                message=_('Scheduling is not available for '
                          'objects: "{0}"').format('", "'.join(invalid_ids)))
        if not valid_ids:
            logger.error(u'No valid object to schedule task.')
            raise exceptions.WorkflowValidationError
        return (valid_ids,
                object_type,
                object_groups.pop() if object_groups else default_object_group)

    def get_context_data(self, **kwargs):
        context = super(SchedulerCalendarView, self).get_context_data(**kwargs)
        objects_ids, object_type, object_group = self.get_object_ids_and_type()

        self.validate_actions(object_type, object_group)

        params = '&'.join(['object_id={0}'.format(_id) for _id in objects_ids])
        context['schedule_task_url'] = '?'.join([
            reverse(self.schedule_task_url, args=[object_type, object_group]),
            params
        ])
        context['schedule_task_list'] = json.dumps(
            self.get_scheduler_task_list(object_type,
                                         object_group,
                                         objects_ids)
        )
        return context

    def get_redirect_url(self):
        return self.request.META['HTTP_REFERER']


class RepeatSchedulerWorkflowView(SchedulerWorkflowView,
                                  RetrieveTimetableMixin):
    """
    The workflow view to repeat schedule tasks
    """

    workflow_class = RepeatSchedulerWorkflow

    def get_initial(self):
        self.request.session['user_action_uuid'] = uuid.uuid4()

        initial = super(RepeatSchedulerWorkflowView, self).get_initial()

        task = self.get_object()
        if task:
            begin_date = task.get('begin_date')
            begin_time = task.get('begin_time')
            if begin_date is not None and begin_time is not None:
                timezone = task.get('timezone')
                tz = pytz.timezone(timezone)
                utc_datetime = get_utc_date(begin_date, begin_time)
                local_datetime = utc_datetime.astimezone(tz)
                local_date = local_datetime.date()
                utcnow = datetime.datetime.utcnow()
                today_date = utcnow.date()
                finale_time = ''
                if local_date > today_date:
                    finale_date = local_date
                    finale_time = local_datetime.time()
                else:
                    finale_date = today_date
                    if utc_datetime.time() > utcnow.time():
                        finale_time = local_datetime.time()
                task['begin_date'] = unicode(finale_date)
                task['begin_time'] = unicode(finale_time)

            periodic_task = task.get('periodic_task', {})
            kwargs = json.loads(periodic_task.get('kwargs', '{}'))
            if kwargs:
                task.update(**kwargs)
            args = json.loads(periodic_task.get('args', '[]'))
            if args and args >= MIN_TASK_ARGS_COUNT:
                user_name, user_password, tenant = args[:MIN_TASK_ARGS_COUNT]
                task.update({
                    'user_name': user_name,
                    'user_password': user_password,
                    'tenant': tenant
                })
            if task.get('repeat'):
                try:
                    task.update(**parse_repeating_task(task, periodic_task))
                except TypeError as err:
                    redirect = reverse(self.redirect_url)
                    exceptions.handle(self.request, unicode(err))
                    raise exceptions.Http302(redirect)
            else:
                task['task_type'] = TaskTypeEnum.SINGLE

            initial.update(**task)

        return initial
