# coding: utf-8
# pylint: disable=super-on-old-class, too-many-lines, no-member

import datetime
import logging

import babel
import babel.dates
import pytz

from django.conf import settings
from django.forms import ValidationError
from django.forms.utils import ErrorDict
from django.template.defaultfilters import filesizeformat
from django.utils import translation
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, messages, workflows
from horizon.utils import validators

from openstack_dashboard import api as os_api
from openstack_dashboard.dashboards.identity.users.forms import (
    ADD_PROJECT_URL)
from openstack_dashboard.dashboards.project.instances import (
    utils as instance_utils)

from tionix_scheduler_client.enums import (
    ObjectTypeEnum, TASK_INTERVAL_RANGE, ActionFieldsEnum)

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.clients.base import neutronclient
from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.dashboards.project.project_instances.forms import (
    LaunchBaseInstanceMixin, SliderFlavorPartInput, RAM_DICT, VCPUS_DICT,
    DISK_DICT, get_image_field_data)
from tionix_dashboard.dashboards.tionix.scheduler import api
from tionix_dashboard.dashboards.tionix.scheduler.enums import (
    TaskTypeEnum, RepeatingTypeEnum, EndTypeEnum)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker, get_cache_key, OBJECTS_CLIENT_MAP)
from tionix_dashboard.dashboards.tionix.vdi.instances.forms import (
    RebuildInstanceMixin)
from tionix_dashboard.dashboards.utils import (
    quotas_usages_validation, get_available_image, get_available_flavor,
    get_available_network, image_choice_title)
from tionix_dashboard.helpers.fields import (
    BootstrapTimePickerField, TionixDateField, AutoGenNameCharField)
from tionix_dashboard.helpers.utils import get_sorted_timezones

logger = logging.getLogger(__name__)

MIN_INTERVAL, MAX_INTERVAL = TASK_INTERVAL_RANGE


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class SelectUserCredentialsAction(workflows.Action):
    user_name = forms.CharField(required=True, label=_("User name"))
    user_password = forms.CharField(
        label=_("User password"),
        widget=forms.PasswordInput(render_value=False))
    tenant = forms.DynamicChoiceField(
        label=_("Tenant"), required=True, add_item_link=ADD_PROJECT_URL)

    def populate_tenant_choices(self, request, context):
        """
        Populate project choices
        """
        user_id = request.user.id
        domain_id = request.user.domain_id
        is_admin = os_api.keystone.is_domain_admin(request)
        projects, _has_more = os_api.keystone.tenant_list(
            request, domain=domain_id, user=user_id, admin=is_admin)

        project_choices = []
        for project in projects:
            if project.enabled:
                project_choices.append((project.id, project.name))

        if not project_choices:
            project_choices.insert(0, ('', _("No available projects")))
        elif len(project_choices) > 1:
            project_choices.insert(0, ('', _("Select a project")))
        return project_choices

    class Meta(object):
        name = _('User credentials')


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class SelectUserCredentials(workflows.Step):
    """
    User credentials step
    """

    action_class = SelectUserCredentialsAction
    contributes = ('user_name', 'user_password', 'tenant')


# noinspection PyMethodMayBeStatic
class SelectActionNameAction(workflows.Action):
    description = AutoGenNameCharField(
        label=_('Task name'), required=True, max_length=80)

    unique_name = forms.ChoiceField(label=_("Action"), required=True)

    def populate_unique_name_choices(self, request, context):
        actions = api.get_available_actions(
            request=request,
            type_name=context.get('object_type'),
            group_name=context.get('object_group'),
            cache_key=request.session.get('user_action_uuid',
                                          get_cache_key(request)),
        )

        if not actions:
            actions = [(None, _('No tasks available'))]

        return actions

    class Meta(object):
        name = _('Select action')


class SelectAction(workflows.Step):
    """
    Name and action choosing step
    """

    action_class = SelectActionNameAction
    contributes = ('description', 'unique_name')

    template_name = 'scheduler/_select_action.html'

    def __init__(self, workflow):
        super(SelectAction, self).__init__(workflow)
        self.object_ids = workflow.object_ids
        self.object_type = workflow.object_type
        self.object_group = workflow.object_group

    def prepare_action_context(self, request, context):
        step_context = super(SelectAction, self).prepare_action_context(
            request, context)
        step_context.update({
            'object_ids': self.object_ids,
            'object_type': self.object_type,
            'object_group': self.object_group,
        })
        return step_context


def get_week_days_attrs(week_day_name):
    data_repeat_days_of_weak_str = (
        'data-repeat-{0}'.format(RepeatingTypeEnum.DAYS_OF_THE_WEEK)
    )
    return {
        'class': 'switched week-days-checkbox',
        'data-switch-on': 'repeat',
        data_repeat_days_of_weak_str: week_day_name
    }


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class SetAddTaskDetailsAction(workflows.Action):
    task_type = forms.ChoiceField(
        label=_("Type"),
        required=True,
        choices=TaskTypeEnum.get_choices(),
        widget=forms.ChoiceField.widget(attrs={
            'class': 'switchable',
            'data-slug': 'type'
        })
    )

    data_type_single_str = 'data-type-{0}'.format(TaskTypeEnum.SINGLE)
    data_type_repeat_str = 'data-type-{0}'.format(TaskTypeEnum.REPEATING)

    repeat = forms.ChoiceField(
        label=_("Repeating"),
        required=False,
        choices=RepeatingTypeEnum.get_choices(),
        widget=forms.ChoiceField.widget(attrs={
            'class': 'switched switchable',
            'data-slug': 'repeat',
            'data-switch-on': 'type',
            data_type_repeat_str: _("Repeat")
        })
    )

    _data_repeat = 'data-repeat-{0}'
    data_repeat_days_str = _data_repeat.format(RepeatingTypeEnum.DAYS)
    data_repeat_minutes_str = _data_repeat.format(RepeatingTypeEnum.MINUTES)
    data_repeat_hours_str = _data_repeat.format(RepeatingTypeEnum.HOURS)

    interval = forms.IntegerField(
        label=_('Repeat with interval'),
        required=False,
        initial=1,
        min_value=1,
        widget=forms.NumberInput(attrs={
            'class': 'switched',
            'data-switch-on': 'repeat',
            data_repeat_days_str: _('Repeat with interval'),
            data_repeat_minutes_str: _('Repeat with interval'),
            data_repeat_hours_str: _('Repeat with interval'),
        })
    )

    repeat_mon = forms.BooleanField(
        required=False,
        label=_("Mon"),
        widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Mon"))
        )
    )
    repeat_tue = forms.BooleanField(
        required=False,
        label=_("Tue"), widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Tue"))
        )
    )
    repeat_wed = forms.BooleanField(
        required=False,
        label=_("Wed"), widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Wed"))
        )
    )
    repeat_thu = forms.BooleanField(
        required=False,
        label=_("Thu"), widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Thu"))
        )
    )
    repeat_fri = forms.BooleanField(
        required=False,
        label=_("Fri"), widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Fri"))
        )
    )
    repeat_sat = forms.BooleanField(
        required=False,
        label=_("Sat"), widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Sat"))
        )
    )
    repeat_sun = forms.BooleanField(
        required=False,
        label=_("Sun"), widget=forms.CheckboxInput(
            attrs=get_week_days_attrs(_("Sun"))
        )
    )

    begin_date = TionixDateField(
        label=_('Start date'),
        required=True,
        widget=BootstrapTimePickerField(attrs={
            'class': 'switched',
            'data-switch-on': 'type',
            data_type_single_str: _('Start date'),
            data_type_repeat_str: _('Start date'),
            'data-provide-bootstrapdatetimepicker': 'datepicker'
        }),
        help_text=_('Starting date of task. Field format is dd.mm.yyyy.')
    )

    begin_time = forms.TimeField(
        label=_('Start time'),
        required=True,
        widget=BootstrapTimePickerField(attrs={
            'data-provide-bootstrapdatetimepicker': 'timepicker'
        }),
        help_text=_('Starting time of task. Field format is HH:MM.')
    )

    timezone = forms.ChoiceField(
        label=_('Timezone'),
        required=True,
        widget=forms.ChoiceField.widget(
            attrs={
                'class': 'switched',
                data_type_single_str: _('Timezone')}
        )
    )

    end = forms.ChoiceField(
        label=_('End'),
        required=False,
        initial=EndTypeEnum.NEVER,
        choices=EndTypeEnum.get_choices(),
        widget=forms.RadioSelect(
            attrs={
                'class': 'switched',
                'data-switch-on': 'type',
                data_type_repeat_str: _('End')}
        )
    )

    end_max_number = forms.IntegerField(
        required=False,
        min_value=1,
        widget=forms.NumberInput(
            attrs={
                'value': '1',
                'disabled': 'disabled',
                'class': 'switched',
                'data-switch-on': 'type',
                data_type_repeat_str: '',
            }
        )
    )

    end_date = TionixDateField(
        required=False,
        widget=forms.DateInput(
            attrs={
                'disabled': 'disabled',
                'class': 'switched',
                'data-switch-on': 'type',
                data_type_repeat_str: '',
                'data-date-format': 'dd.mm.yyyy',
                'data-provide': "datepicker"}
        )
    )

    MAX_TASK_INTERVAL = 20

    def __init__(self, request, context, *args, **kwargs):
        super(SetAddTaskDetailsAction, self).__init__(request, context, *args,
                                                      **kwargs)

        self.fields['timezone'].initial = request.session.get(
            'django_timezone',
            self.request.COOKIES.get('django_timezone', settings.TIME_ZONE))

    def populate_timezone_choices(self, request, context):
        """
        Populate Timezones
        """
        timezones = []
        language = translation.get_language()
        current_locale = translation.to_locale(language)
        babel_locale = babel.Locale.parse(current_locale)
        for tz, offset in get_sorted_timezones():
            try:
                utc_offset = _("UTC %(hour)s:%(min)s") % {"hour": offset[:3],
                                                          "min": offset[3:]}
            except Exception as err:
                logger.error(unicode(err))
                utc_offset = ""

            if tz == "UTC":
                tz_name = _("UTC")
            elif tz == "GMT":
                tz_name = _("GMT")
            else:
                tz_label = babel.dates.get_timezone_location(
                    tz, locale=babel_locale)
                # Translators:  UTC offset and timezone label
                tz_name = _("%(offset)s: %(label)s") % {"offset": utc_offset,
                                                        "label": tz_label}
            timezones.append((tz, tz_name))
        return timezones

    def clean(self):
        cleaned_data = super(SetAddTaskDetailsAction, self).clean()

        begin_time = cleaned_data.get('begin_time')
        if begin_time is None:
            delta = datetime.timedelta(seconds=api.TASK_CREATION_TIME)
            begin_time = (datetime.datetime.now() + delta).time()

        begin_date = cleaned_data.get('begin_date')
        if begin_date:
            naive = datetime.datetime.combine(begin_date, begin_time)

            local = pytz.timezone(cleaned_data.get('timezone'))
            local_dt = local.localize(naive, is_dst=None)

            now_time = datetime.datetime.now(pytz.utc)
            # If adding task for multi objects, then increase now time
            objects_count = len(self.request.GET.getlist('id'))
            if objects_count > 1:
                now_time += datetime.timedelta(
                    seconds=objects_count * self.MAX_TASK_INTERVAL
                )

            if local_dt.astimezone(pytz.utc) < now_time:
                msg = _('A task can not be scheduled in the past.')
                self.add_error('begin_time', _('This field is invalid.'))
                raise ValidationError(msg)

        if cleaned_data.get('task_type') == TaskTypeEnum.REPEATING:
            if not cleaned_data.get('interval'):
                self.add_error('interval', _('This field is required.'))

            end = cleaned_data.get('end')

            if (end == EndTypeEnum.MAX_COUNT and
                    not cleaned_data.get('end_max_number')):
                self.add_error('end_max_number', _('This field is required.'))
            elif end == EndTypeEnum.DATE:
                end_date = cleaned_data.get('end_date')
                if not end_date:
                    self.add_error('end_date', _('This field is required.'))
                elif end_date < begin_date:
                    self.add_error(
                        'end_date',
                        _('This field must be later than "Start date".')
                    )

        return cleaned_data

    class Meta(object):
        name = _('Details')


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class ActionDetailsBase(workflows.Action):
    """Base action for action's details"""

    def __init__(self, request, initial, *args, **kwargs):
        super(ActionDetailsBase, self).__init__(
            request, initial, *args, **kwargs)
        self._errors = None
        self.cleaned_data = None

        # If action is chosen and does not belong to current step,
        # reset initial values for step's fields
        unique_name = initial.get('unique_name')
        if unique_name and unique_name != self.slug:
            self.initial = {k: v for k, v in initial.items()
                            if k not in self.fields}

        self._check_initial_data(initial)

    def _check_initial_data(self, initial):
        pass

    def full_clean(self):
        # pylint: disable=no-member
        if self.data.get('unique_name') == self.slug:
            super(ActionDetailsBase, self).full_clean()
        else:
            self._errors = ErrorDict()
            self.cleaned_data = {}


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class RebuildDetailsAction(RebuildInstanceMixin, ActionDetailsBase):

    image_id = forms.ChoiceField(
        label=_("Select Image"),
        widget=forms.SelectWidget(attrs={'class': 'image-selector'},
                                  data_attrs=('size', 'display-name'),
                                  transform=image_choice_title)
    )
    action_password = forms.RegexField(
        label=_("Rebuild Password"),
        widget=forms.PasswordInput(render_value=False),
        required=False,
        regex=validators.password_validator(),
        error_messages={'invalid': validators.password_validator_msg()}
    )
    confirm_action_password = forms.CharField(
        label=_("Confirm Rebuild Password"),
        required=False,
        widget=forms.PasswordInput(render_value=False)
    )
    disk_config = forms.ChoiceField(label=_("Disk Partition"),
                                    required=False)

    class Meta(object):
        slug = ActionFieldsEnum.REBUILD
        name = _('Rebuild action details')

    def _check_initial_data(self, initial):

        image_id = initial.get('image_id')
        if image_id:
            if get_available_image(self.request, image_id) is None:
                self.add_error(
                    'image_id',
                    _(u"Image with id '{0}' not found.").format(image_id)
                )
                initial['image_id'] = ''


class ShtdownDetailsAction(ActionDetailsBase):

    allow_off_vm = forms.BooleanField(
        required=False,
        label=_('Allow to stop and migrate instance if live '
                'migration fails'))
    allow_evacuate = forms.BooleanField(
        required=False,
        label=_('Allow to evacuate instance if migration fails'))

    class Meta(object):
        slug = ActionFieldsEnum.SHUTDOWN
        name = _('Shutdown action details')


def get_tenant_network_field_data(request,
                                  tenant_ids,
                                  include_empty_option=True):
    """ Retrieve shared networks for selected projects.
    For multiple selection only shared nets, otherwise + tenant's nets """
    networks = (
        neutronclient(request).list_networks(shared=True).get('networks')
    )

    if len(tenant_ids) == 1:
        tenant_id = tenant_ids[0]
        neutron_client = neutronclient(request, project_id=tenant_id)
        networks += neutron_client.list_networks(
            tenant_id=tenant_id
        ).get('networks')

    networks = {
        (network['id'], network['name'])
        for network in sorted(networks, key=lambda x: x['shared'] is True)
    }

    if not networks:
        networks = [("", _("No networks available"))]
    elif include_empty_option:
        networks = [("", _("Select Network")), ] + list(networks)

    return networks


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class LaunchBaseDetailsAction(LaunchBaseInstanceMixin, ActionDetailsBase):
    """Action to launch base instance"""

    name = AutoGenNameCharField(label=_("Instance Name"), max_length=255)
    count = forms.IntegerField(
        label=_("Instance Count"),
        min_value=1,
        initial=1,
        help_text=_("Number of instances to launch."))
    image = forms.ChoiceField(
        label=_("Image Name"),
        widget=forms.SelectWidget(
            data_attrs=('volume_size',),
            transform=lambda x: ("%s (%s)" % (x.name,
                                              filesizeformat(x.bytes)))))
    network_id = forms.ChoiceField(label=_("Network"))
    vcpus = forms.IntegerField(
        label=_('CPU (pcs.)'),
        required=False,
        widget=SliderFlavorPartInput(
            ticks_dict=VCPUS_DICT,
            attrs={'data-slider-tooltip': 'hide'}
        )
    )
    ram = forms.IntegerField(
        label=_('RAM (MB)'),
        required=False,
        widget=SliderFlavorPartInput(
            ticks_dict=RAM_DICT,
            attrs={'data-slider-tooltip': 'hide'}
        )
    )
    disk = forms.IntegerField(
        label=_('HDD (GB)'),
        required=False,
        widget=SliderFlavorPartInput(
            ticks_dict=DISK_DICT,
            attrs={'data-slider-tooltip': 'hide'}
        )
    )

    class Meta(object):
        slug = ActionFieldsEnum.LAUNCH_BASE_VM
        name = _('Launch base instance details')

    def _check_initial_data(self, initial):
        flavor_id = initial.get('flavor_id')
        if flavor_id:
            flavor = get_available_flavor(self.request, flavor_id)
            if flavor:
                initial.update(vcpus=flavor.vcpus,
                               ram=flavor.ram,
                               disk=flavor.disk)
            else:
                self.add_error(
                    None,
                    _(u"Flavor with id '{0}' not found.").format(flavor_id)
                )
                initial['flavor_id'] = ''

        image_id = initial.get('image')
        if image_id:
            if get_available_image(self.request, image_id) is None:
                self.add_error(
                    'image',
                    _(u"Image with id '{0}' not found.").format(image_id)
                )
                initial['image'] = ''

        network_id = initial.get('network_id')
        if network_id:
            if get_available_network(self.request, network_id) is None:
                self.add_error(
                    'network_id',
                    _(u"Network with id '{0}' not found.").format(network_id)
                )
                initial['network_id'] = ''

    def populate_network_id_choices(self, request, context):
        return get_tenant_network_field_data(request, context['object_ids'])

    def clean(self):
        cleaned_data = super(LaunchBaseDetailsAction, self).clean()

        tenant_id = self.request.GET.get('id')
        instances_count = self.cleaned_data.get('count', 1)
        if tenant_id:
            quotas_usages_validation(self.request,
                                     flavor_id=cleaned_data.get('flavor_id'),
                                     tenant_id=tenant_id,
                                     instances_count=instances_count)

        return cleaned_data


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class LaunchInstanceDetailsAction(ActionDetailsBase):
    """Action to launch instance"""
    name = AutoGenNameCharField(label=_("Instance Name"), max_length=255)
    availability_zone = forms.ChoiceField(
        label=_("Availability Zone"),
        required=False)
    count = forms.IntegerField(
        label=_('Count'),
        min_value=1,
        initial=1,
    )
    image_id = forms.ChoiceField(
        label=_("Source"),
        widget=forms.SelectWidget(
            data_attrs=('volume_size',),
            transform=lambda x: ("%s (%s)" % (x.name,
                                              filesizeformat(x.bytes)))))
    flavor_id = forms.ChoiceField(label=_("Flavor"))
    networks = forms.MultipleChoiceField(
        label=_("Networks"),
        error_messages={
            'required': _("At least one network must be specified.")
        },
        help_text=_("Launch instance with these networks"))

    class Meta(object):
        slug = ActionFieldsEnum.LAUNCH_VM
        name = _('Launch instance details')

    def _check_initial_data(self, initial):
        flavor_id = initial.get('flavor_id')
        if flavor_id:
            if get_available_flavor(self.request, flavor_id) is None:
                self.add_error(
                    'flavor_id',
                    _(u"Flavor with id '{0}' not found.").format(flavor_id)
                )
                initial['flavor_id'] = ''

        image_id = initial.get('image_id')
        if image_id:
            if get_available_image(self.request, image_id) is None:
                self.add_error(
                    'image_id',
                    _(u"Image with id '{0}' not found.").format(image_id)
                )
                initial['image_id'] = ''

        networks = initial.get('networks')
        if networks:
            not_valid_networks = []
            for network in networks:
                if get_available_network(self.request, network) is None:
                    not_valid_networks.append(network)
            if not_valid_networks:
                self.add_error(
                    'networks',
                    _(u"Networks with ids {0} not found.").format(
                        u', '.join(not_valid_networks)
                    )
                )

    def populate_image_id_choices(self, request, context):
        return get_image_field_data(request)

    def populate_networks_choices(self, request, context):
        return get_tenant_network_field_data(request,
                                             context['object_ids'],
                                             include_empty_option=False)

    def populate_flavor_id_choices(self, request, context):
        return instance_utils.flavor_field_data(request,
                                                include_empty_option=True)

    def populate_availability_zone_choices(self, request, context):
        try:
            zones = os_api.nova.availability_zone_list(request)
        except Exception as err:
            logger.error(unicode(err))
            zones = []
            exceptions.handle(request,
                              _('Unable to retrieve availability zones.'))
        else:
            zones = [(zone.zoneName, zone.zoneName)
                     for zone in zones if zone.zoneState['available']]
            zones.sort()
        if not zones:
            zones.insert(0, ("", _("No availability zones found")))
        elif len(zones) > 1:
            zones.insert(0, ("", _("Any Availability Zone")))
        return zones

    def clean(self):
        cleaned_data = super(LaunchInstanceDetailsAction, self).clean()

        tenant_id = self.request.GET.get('id')
        instances_count = self.cleaned_data.get('count', 1)
        if tenant_id:
            quotas_usages_validation(self.request,
                                     flavor_id=cleaned_data.get('flavor_id'),
                                     tenant_id=tenant_id,
                                     instances_count=instances_count)

        return cleaned_data


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class LaunchVDIDetailsAction(ActionDetailsBase):
    """Action to launch vdi instance"""

    name = AutoGenNameCharField(label=_("Instance Name"), max_length=255)
    count = forms.IntegerField(
        label=_('Instance count'),
        min_value=1,
        initial=1,
    )

    class Meta(object):
        slug = ActionFieldsEnum.LAUNCH_VDI_VM
        name = _('Launch VDI instance details')

    def clean(self):
        cleaned_data = super(LaunchVDIDetailsAction, self).clean()

        tenant_id = self.request.GET.get('id')
        instances_count = self.cleaned_data.get('count', 1)
        if tenant_id:
            try:
                tenant = TNXTenantClient.get(self.request, tenant_id)
            except Exception as err:
                logger.error(err)
                raise ValidationError(_('Unable to get project.'))
            else:
                if tenant.tnx_project_type != TenantTypeEnum.VDI:
                    raise ValidationError(_('Project is not VDI.'))

            flavor_id = tenant.tnx_flavor_id
            quotas_usages_validation(self.request,
                                     flavor_id=flavor_id,
                                     tenant_id=tenant_id,
                                     instances_count=instances_count)

        return cleaned_data


class SetAddTaskDetails(workflows.Step):
    """Schedule params step"""

    action_class = SetAddTaskDetailsAction
    depends_on = ('user_name', 'user_password', 'tenant')

    # A list of keys which this step will contribute to the workflow's
    # context data. Optional keys should still be listed, even if their
    # values may be set to ``None``.
    contributes = ('task_type', 'repeat', 'interval',
                   'repeat_mon', 'repeat_tue', 'repeat_wed', 'repeat_thu',
                   'repeat_fri', 'repeat_sat', 'repeat_sun',
                   'begin_date', 'begin_time', 'timezone',
                   'end', 'end_max_number', 'end_date')

    template_name = 'scheduler/details.html'


class ActionDetailsBaseStep(workflows.Step):
    """Base class for action's details"""
    template_name = 'scheduler/_action_details.html'

    def get_id(self):
        """Returns the ID for this step. Suitable for use in HTML markup."""
        return "%s__action__%s" % (self.workflow.slug, self.slug)

    def _verify_contributions(self, context):
        """Verifying contribution only for chosen action step"""
        result = True
        if context.get('unique_name') == self.slug:
            result = super(ActionDetailsBaseStep, self)._verify_contributions(
                context
            )
        return result

    def __init__(self, workflow):
        super(ActionDetailsBaseStep, self).__init__(workflow)
        self.object_ids = workflow.object_ids

    def prepare_action_context(self, request, context):
        context = super(ActionDetailsBaseStep, self).prepare_action_context(
            request, context
        )
        context.update({'object_ids': self.object_ids})
        return context


class RebuildActionDetails(ActionDetailsBaseStep):
    """Action's details"""

    action_class = RebuildDetailsAction

    contributes = (
        'image_id', 'action_password', 'confirm_action_password',
        'disk_config',
    )


class ShutdownActionDetails(ActionDetailsBaseStep):
    """Action's details"""

    action_class = ShtdownDetailsAction

    contributes = ('allow_off_vm', 'allow_evacuate')


class LaunchBaseActionDetails(ActionDetailsBaseStep):
    """Action's details"""

    action_class = LaunchBaseDetailsAction

    contributes = ('name', 'image', 'network_id', 'flavor_id', 'count')

    template_name = "scheduler/_launch_base_step.html"


class LaunchInstanceActionDetails(ActionDetailsBaseStep):
    """Launch instance step"""

    action_class = LaunchInstanceDetailsAction

    contributes = ('availability_zone', 'count', 'image_id', 'flavor_id',
                   'networks', 'name')


class LaunchVDIActionDetails(ActionDetailsBaseStep):
    """Action's details"""

    action_class = LaunchVDIDetailsAction

    contributes = ('name', 'count')


class SchedulerWorkflow(workflows.Workflow):
    """ Scheduler workflow"""
    slug = 'schedule'
    name = _('Schedule')
    multipart = True
    finalize_button_name = _('Add schedule')
    success_message = _('Scheduling workflow completed.')
    success_url = failure_url = 'horizon:tionix:scheduler:index'
    template_name = 'scheduler/workflow.html'

    default_steps = (
        SelectAction,
        SelectUserCredentials,
        SetAddTaskDetails,
        RebuildActionDetails,
        ShutdownActionDetails,
        LaunchBaseActionDetails,
        LaunchVDIActionDetails,
        LaunchInstanceActionDetails,
    )

    objects_map = OBJECTS_CLIENT_MAP

    # pylint: disable=keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        self.object_ids = context_seed.getlist('object_id')
        self.object_type = context_seed.get('object_type')
        self.object_group = context_seed.get('object_group')

        super(SchedulerWorkflow, self).__init__(
            request, context_seed, entry_point, *args, **kwargs)

    @property
    def steps(self):
        """Steps filtered by actions"""
        steps = super(SchedulerWorkflow, self).steps
        tasks = api.get_available_actions(
            request=self.request,
            type_name=self.object_type,
            group_name=self.object_group,
            cache_key=self.request.session.get('user_action_uuid',
                                               get_cache_key(self.request)),
        )
        task_ids, _task_names = zip(*tasks) if tasks else ([], None)
        return [step for step in steps
                if (not isinstance(step, ActionDetailsBaseStep) or
                    step.action_class.slug in task_ids)]

    def format_status_message(self, message):
        return message % self.context.get('unique_name', 'unknown task')

    def handle(self, request, context):
        failed_objs, success_objs, ignored_objs = [], [], []
        for object_id in self.object_ids:
            try:
                obj = self.objects_map[self.object_type](request, object_id)
            except Exception as err:
                failed_objs.append(object_id)
                logger.error(unicode(err))
                continue

            obj.verbose_name = self.get_object_name(obj)
            verbose_name = obj.verbose_name.replace('%', '%%')

            if not SchedulingPreChecker.is_schedulable(obj):
                ignored_objs.append(obj.verbose_name or object_id)
                continue

            if not self.add_schedule(request, context, obj):
                failed_objs.append(object_id)
            else:
                success_objs.append(verbose_name or object_id)

        if success_objs:
            msg = _('Successfully added task "%s" for:')
            self.success_message = u'{0} "{1}"'.format(
                unicode(msg), '", "'.join(success_objs))

        if failed_objs:
            msg = _('Unable to schedule task "%s" for:')
            self.failure_message = u'{0} "{1}"'.format(
                unicode(msg), '", "'.join(failed_objs))
            if success_objs:
                messages.success(
                    request, self.format_status_message(self.success_message))

        if ignored_objs:
            msg = _('Skipped scheduling of tasks for:')
            messages.info(request, u'{0} "{1}"'.format(
                unicode(msg), '", "'.join(ignored_objs)))

        request.session.pop('user_action_uuid', None)

        return len(failed_objs) == 0

    def get_object_name(self, obj):
        """
        Return name of object depending on its type
        """
        if self.object_type == ObjectTypeEnum.hypervisor:
            name = (getattr(obj, 'node_title', None) or
                    getattr(obj, 'hypervisor_hostname', None))
        else:
            if hasattr(obj, '_apiresource'):
                # noinspection PyProtectedMember
                attr = obj._apiresource.NAME_ATTR
            else:
                attr = obj.NAME_ATTR

            name = getattr(obj, attr, None)

        return name or 'undefined'

    def add_schedule(self, request, context, obj):
        params = {
            'author_name': request.user.username,
            'tenant_id': request.user.tenant_id,
            'object_id': obj.id,
            'object_name': obj.verbose_name,
            'object_type': self.object_type,
            'object_group': self.object_group,
            'user_domain_name': request.user.user_domain_name
        }
        params.update(context)
        return api.schedule_task(request, data=params)

    def get_absolute_url(self):
        return self.request.get_full_path()


class SelectUserCredentialsActionRepeat(SelectUserCredentialsAction):
    """user credentials repeat form"""
    user_password = forms.CharField(
        label=_("User password"),
        widget=forms.PasswordInput(render_value=True))

    class Meta(object):
        name = _('User credentials')


class SelectUserCredentialsRepeat(SelectUserCredentials):
    """
    User credentials step for repeat form
    """
    action_class = SelectUserCredentialsActionRepeat


# noinspection PyArgumentList,PyUnusedLocal,PyMethodMayBeStatic
class SetAddTaskDetailsActionRepeat(SetAddTaskDetailsAction):
    """Schedule details for repeat form"""
    begin_date = TionixDateField(
        label=_('Start date'),
        required=True,
        widget=BootstrapTimePickerField(attrs={
            'class': 'switched',
            'data-switch-on': 'type',
            SetAddTaskDetailsAction.data_type_single_str: _('Start date'),
            SetAddTaskDetailsAction.data_type_repeat_str: _('Start date'),
            'data-provide-bootstrapdatetimepicker': 'datepicker'
        }),
        help_text=_('Starting date of task. Field format is dd.mm.yyyy.')
    )

    class Meta(object):
        name = _('Details')


class SetAddTaskDetailsRepeat(SetAddTaskDetails):
    """Schedule params step for repeat form"""

    action_class = SetAddTaskDetailsActionRepeat


class RepeatSchedulerWorkflow(SchedulerWorkflow):
    """ Repeat scheduler workflow"""
    finalize_button_name = _('Repeat schedule')

    default_steps = (
        SelectAction,
        SelectUserCredentialsRepeat,
        SetAddTaskDetailsRepeat,
        RebuildActionDetails,
        ShutdownActionDetails,
        LaunchBaseActionDetails,
        LaunchVDIActionDetails,
        LaunchInstanceActionDetails,
    )

    def __init__(self, *args, **kwargs):

        super(RepeatSchedulerWorkflow, self).__init__(*args, **kwargs)

        select_action_exclude = ('description', 'unique_name')

        user_credentials_exclude = ('user_name', 'user_password', 'tenant')

        details_exclude = ('task_type', 'repeat', 'timezone', 'end',
                           'end_max_number', 'end_date', 'begin_date',
                           'begin_time', 'interval', 'repeat_mon',
                           'repeat_tue', 'repeat_wed', 'repeat_thu',
                           'repeat_fri', 'repeat_sat', 'repeat_sun')

        exclude_field_names = (
            select_action_exclude + user_credentials_exclude + details_exclude
        )

        for action in [ActionFieldsEnum.LAUNCH_VDI_VM,
                       ActionFieldsEnum.LAUNCH_BASE_VM,
                       ActionFieldsEnum.LAUNCH_VM]:
            exclude_field_names = (
                exclude_field_names + ActionFieldsEnum.fields.get(action)
            )

        for step in self.steps:
            for fields_name, field in step.action.fields.items():
                widget = field.widget
                if fields_name not in exclude_field_names:
                    if isinstance(widget, (forms.CheckboxInput,
                                           forms.RadioSelect)):
                        widget.attrs['onclick'] = 'return false;'
                    elif isinstance(field, forms.DynamicChoiceField):
                        widget.attrs['disabled'] = 'disabled'
                        field.widget.add_item_link = None
                    else:
                        widget.attrs['readonly'] = 'readonly'
