# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon.utils.memoized import memoized

from openstack_dashboard import api, policy
from openstack_dashboard.api.glance import is_image_public
from openstack_dashboard.api.keystone import VERSIONS

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.clients.base import (
    neutronclient, novaclient, glanceclient)
from tionix_dashboard.dashboards.helpers import get_instances_list_by_domain
from tionix_dashboard.dashboards.tionix.vdi.constants import (
    VDI_TYPE, VDI_PROJECT_DETAILS)
from tionix_dashboard.dashboards.utils import (
    update_instances_list, get_tenants_kwargs, get_quota_available)

logger = logging.getLogger(__name__)


@memoized
def get_quota_by_tenant(request, tenant_id):
    """Retrieve tenant quota"""
    nova_client = novaclient(request, project_id=tenant_id)
    limits = nova_client.limits.get()
    quotas = limits.to_dict().get('absolute')
    return quotas


@memoized
def get_networks_by_tenant(request, tenant_id):
    """Retrieve tenant networks"""
    neutron_client = neutronclient(request, project_id=tenant_id)
    networks = neutron_client.list_networks().get('networks')
    return {network['id']: network for network in networks}


@memoized
def get_favor_ids_by_tenant(request, tenant_id):
    """Retrieve flavors ids available for tenant"""
    nova_client = novaclient(request, project_id=tenant_id)
    return [flavor.id for flavor in nova_client.flavors.list()]


@memoized
def get_image_by_tenant(request, tenant_id, image_id):
    """Retrieve tenant image"""
    glance_client = glanceclient(request, project_id=tenant_id)
    return glance_client.images.get(image_id)


def validate_tenant(request, tenant=None, user_tenants=None):
    """Check tenant by tnx attributes, quota and available network"""
    def _validate_tenant_fields(_tenant):
        fields = ['tnx_operation_mode', 'tnx_flavor_id', 'tnx_network_id',
                  'tnx_image_id']
        for field in fields:
            if not getattr(_tenant, field, None):
                return False
        return True

    validated = False
    tenant_id = tenant.id

    if _validate_tenant_fields(tenant):
        if tenant_id in user_tenants:
            try:
                quotas = get_quota_by_tenant(request, tenant_id=tenant_id)
                available = get_quota_available(quotas)
                instances_available = available.get('instances')
                validated = (
                    instances_available is None or
                    instances_available > 0)
            except Exception as err:
                logger.error(err)
    if validated:
        try:
            image = get_image_by_tenant(request,
                                        tenant_id,
                                        tenant.tnx_image_id)
        except Exception as err:
            logger.error(unicode(err))
            validated = False
        else:
            validated = (image.status.lower() != "deleted" and
                         (is_image_public(image) or image.owner == tenant_id))

    if validated:
        try:
            flavors = get_favor_ids_by_tenant(request, tenant_id)
        except Exception as err:
            logger.error(unicode(err))
            validated = False
        else:
            validated = tenant.tnx_flavor_id in flavors

    if validated:
        networks = get_networks_by_tenant(request, tenant_id)
        network = networks.get(tenant.tnx_network_id)
        validated = network and (network['tenant_id'] == tenant_id or
                                 network['shared'])

    return validated


@memoized
def get_projects_for_instance_launch(request):
    """Retrieve available projects for vdi instance"""

    user = request.user
    projects = []
    try:
        if policy.check((('identity', 'identity:list_role_assignments'),),
                        request):
            assignments = api.keystone.role_assignments_list(
                request, user=user, include_subtree=False)
            user_assignments = [assignment.scope['project']['id']
                                for assignment in assignments
                                if 'project' in assignment.scope]
        else:
            user_assignments = [project.id for project in
                                user.authorized_tenants]
    except Exception as err:
        logger.error(err)
        msg = _('Unable to get user assignments.')
        exceptions.handle(request, msg)
    else:

        kwargs = {'filters': {'enabled': True}}
        kwargs.update(get_tenants_kwargs(request))
        try:
            projects = [
                (tenant.id, tenant.name)
                for tenant in TNXTenantClient.list_vdi(request, **kwargs)
                if validate_tenant(request,
                                   tenant=tenant,
                                   user_tenants=user_assignments)
            ]

        except Exception as err:
            logger.error(err)
            msg = _('Failed to retrieve list of projects.')
            exceptions.handle(request, msg)

    return projects


@memoized
def get_project_users(request, project):
    """Retrieve project's users list"""

    def _filter(obj):
        return (hasattr(obj, 'user') and
                'project' in obj.scope and
                obj.scope['project']['id'] == project)

    if VERSIONS.active < 3:
        project_users = [user.id for user in
                         api.keystone.user_list(request, project=project)]
    else:
        project_role_assignments = api.keystone.role_assignments_list(
            request,
            project=project
        )
        project_users = list({role_assignment.user['id']
                              for role_assignment in project_role_assignments
                              if _filter(role_assignment)})
    return project_users


def get_user_vdi_projects(request):

    projects, kwargs = [], get_tenants_kwargs(request)

    try:
        projects = TNXTenantClient.list_vdi(request, **kwargs)
    except Exception as err:
        logger.error(unicode(err))

    return projects


def get_vdi_instances_list(request, project_id=None, search_opts=None):
    search_opts = search_opts or {}

    search_opts.update({'tnx_type': VDI_TYPE})

    return get_instances_list_by_domain(request,
                                        project_id,
                                        search_opts=search_opts)


def get_vdi_instances(request, search_opts, is_admin=None):
    """Retrieve vdi instances of all available projects"""
    instances = []

    if is_admin is None:
        is_admin = api.keystone.is_domain_admin(request)

    if is_admin:
        search_opts.update(all_tenants=True)
        instances = get_vdi_instances_list(request, search_opts=search_opts)
    else:
        search_opts.update(user=request.user.id)
        for project in get_user_vdi_projects(request):
            instances += get_vdi_instances_list(request,
                                                project.id,
                                                search_opts)

    return instances


def get_vdi_instances_extra(request, search_opts):
    """Retrieve vdi instances with extra fields"""

    tenants = []

    # Gather our tenants to correlate against IDs
    kwargs = get_tenants_kwargs(request)

    try:
        tenants, _has_more = api.keystone.tenant_list(request, **kwargs)
    except Exception as err:
        logger.error(unicode(err))
        msg = _('Unable to retrieve instance project information.')
        exceptions.handle(request, msg)

    instances = get_vdi_instances(request,
                                  search_opts,
                                  is_admin=kwargs['admin'])
    instances = update_instances_list(request, instances, tenants=tenants)

    return instances


@memoized
def get_user_roles_by_project(request, project_id):
    """
    Retrieve user's roles in project
    :param request: django request
    :param project_id: project ID
    :return:
    """
    roles = []
    try:
        roles = api.keystone.roles_for_user(request,
                                            request.user,
                                            project=project_id)
    except Exception as err:
        logger.error(unicode(err))
    return [role.name.lower() for role in roles]


def get_project_detail_link(request, project):
    """return link to project"""
    policy_rule = (("identity", "identity:get_project"),)
    policy_target = {
        'project': project,
        'target.project.domain_id': project.domain_id,
    }
    if policy.check(policy_rule, request, target=policy_target):
        return reverse(VDI_PROJECT_DETAILS, args=(project.id,))
