# coding: utf-8
# pylint: disable=duplicate-code

import logging

from django.core.urlresolvers import reverse
from django.utils.http import urlencode
from django.utils.translation import (
    ugettext_lazy as _, ungettext, ungettext_lazy)

from horizon import tables, exceptions, messages
from horizon.utils.memoized import memoized
from openstack_dashboard import policy

from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.servers import TnxServer, TNXServersClient
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.helpers import get_connection_type_by_name
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.utils import (
    update_instances_list, TABLE_OBJECT_ID_SEP)
from tionix_dashboard.helpers.actions import (
    LinkBatchAction, TnxFilterAction, TnxBatchActionMixin, TnxUpdateAction)
from tionix_dashboard.helpers.vm_constants import (
    ERROR, ACTIVE, PAUSED, STOPPED)

logger = logging.getLogger(__name__)


def get_operation_mode_by_name(value):
    """Return list of operation mode"""
    keys = [k for k, v in OperationModeEnum.values.items()
            if value.lower() in v.lower()]
    return ','.join(keys)


def get_instance_flavor_name(instance):
    """Return instance flavor name"""

    full_flavor, name = getattr(instance, 'full_flavor', None), ''
    if full_flavor:
        name = getattr(full_flavor, 'name', '')
    return name


class VDISingleActionMixin(object):

    url = None

    def get_link_url(self, instance=None):
        project_id, instance_id = ((instance.tenant_id, instance.id)
                                   if instance else ('', ''))
        params = urlencode({'id': TABLE_OBJECT_ID_SEP.join([project_id,
                                                            instance_id])})
        return '?'.join([reverse(self.url), params])


# noinspection PyAbstractClass
class VDIInstanceFilterAction(TnxFilterAction):
    """
    Filter VDI instances filter
    """
    name = "filter_vdi_instances"
    filter_type = "server"

    filter_choices = (
        ('name', _("Name"), True),
        ('tenant', _("Project"), False),
        ('ip', _("IP Address"), False),
        ('size', _("Size"), False),
        ('status', _("Status"), False),
        ('OS-EXT-AZ:availability_zone', _("Zone"), False),
        ('task', _("Task"), False),
        ('power_state', _("Power state"), True),
        ('tnx_operation_mode', _("Operation mode"), True),
        ('id', _("Instance ID"), False),
    )

    filter_string_modifiers = {
        'power_state': project_actions.get_power_states_by_name,
        'tnx_operation_mode': get_operation_mode_by_name
    }

    field_get_data = {
        'size': get_instance_flavor_name
    }


class LaunchVDIInstanceAction(tables.LinkAction):
    """
    Action to launch new instance
    """
    name = 'launch'
    verbose_name = _('Launch VDI Instance')
    classes = ('ajax-modal', )
    url = 'horizon:tionix:vdi:instances:launch'
    icon = "cloud-upload"
    policy_rules = (("compute", "compute:create"),)
    ajax = True

    def __init__(self, attrs=None, **kwargs):
        kwargs['preempt'] = True
        super(LaunchVDIInstanceAction, self).__init__(attrs, **kwargs)


@memoized
def is_user_in_project(request, tenant_id):
    return tenant_id in [tenant.id
                         for tenant in request.user.authorized_tenants]


# noinspection PyAbstractClass
class VDIInstanceManageUsers(tables.LinkAction, IdentityMixIn):
    """Action to manage instance users"""
    url = 'horizon:tionix:vdi:instances:users'

    name = 'vdi_assign_to_users'
    classes = ('ajax-modal', )
    verbose_name = _('Assign to users')

    def allowed(self, request, datum):
        allowed = datum and datum.status.lower() in (ACTIVE, PAUSED, STOPPED)
        if allowed:
            allowed = is_user_in_project(request, datum.tenant_id)
        return allowed


# noinspection PyAbstractClass
class RebuildInstance(policy.PolicyTargetMixin, LinkBatchAction):
    """ Action reset vm's state."""
    name = 'rebuild'
    classes = ('btn-rebuild', )
    multiple_next_view_name = "horizon:tionix:vdi:instances:rebuild"
    policy_rules = (("compute", "compute:rebuild"),)

    def action_present(self, count):
        return ungettext_lazy(
            u"Rebuild Instance",
            u"Rebuild Instances",
            count
        )

    def action_past(self, count):
        return ungettext_lazy(
            u"Rebuilt Instance",
            u"Rebuilt Instances",
            count
        )

    def allowed(self, request, datum):
        allowed = super(RebuildInstance, self).allowed(request, datum)
        if datum and allowed:
            allowed = (
                (datum.status in project_tables.ACTIVE_STATES or
                 datum.status == 'SHUTOFF')
                and not project_tables.is_deleting(datum)
            )
        return allowed


class InstanceUpdateCell(TnxUpdateAction):
    """
    Action update instance name
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        result = True
        project_id = None
        if TABLE_OBJECT_ID_SEP in obj_id:
            project_id, obj_id = obj_id.split(TABLE_OBJECT_ID_SEP)
        try:
            novaclient(
                request, project_id=project_id
            ).servers.update(obj_id, name=new_cell_value)
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to update instance.'))
            result = False
        return result


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        instance = project_id = None
        if TABLE_OBJECT_ID_SEP in obj_id:
            project_id, obj_id = obj_id.split(TABLE_OBJECT_ID_SEP)
        try:
            instance = novaclient(request,
                                  project_id=project_id).servers.get(obj_id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to get instance'), ignore=True)
        else:
            tenants = None
            try:
                tenants = [TNXTenantClient.get(request, instance.tenant_id)]
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(request,
                                  _('Unable to retrieve tenant information '
                                    'for instance "%s".') % obj_id,
                                  ignore=True)
            instance = update_instances_list(request,
                                             [TnxServer(instance, request)],
                                             tenants=tenants)
        return instance.pop() if instance else None


# noinspection PyAbstractClass, PyClassHasNoInit
class ResetStateAction(project_actions.ResetStateAction):
    """ Action reset vm's state."""
    multiple_next_view_name = 'horizon:tionix:vdi:instances:reset_state'


# noinspection PyAbstractClass, PyClassHasNoInit
class VDIStopInstance(project_actions.TnxStopInstance):

    def allowed(self, request, instance):
        allowed = super(VDIStopInstance, self).allowed(request, instance)
        if instance:
            allowed &= instance.status.lower() not in (ERROR, )
        return allowed


# noinspection PyAbstractClass
class SessionFilterAction(TnxFilterAction):
    """Session filter"""
    filter_type = "server"

    filter_choices = [('user_name', _("User Name"), True),
                      ('created_at', _('Created'), True),
                      ('closed_at', _('Closed'), True),
                      ('connection_type', _("Connection"), True),
                      ('client_info', _("Client"), True),
                      ('client_os', _("OS"), True),
                      ('client_ip', _("IP"), True)]

    filter_string_modifiers = {
        'connection_type': get_connection_type_by_name
    }


# noinspection PyAbstractClass
class CloneVDILink(tables.LinkAction):
    """Clone instance action"""
    name = "clone"
    verbose_name = verbose_name_base = _("Clone Instance")
    url = "horizon:tionix:vdi:instances:clone"
    classes = ("ajax-modal",)
    policy_rules = (("compute", "compute:create"),)
    ajax = True

    def allowed(self, request, datum):
        allowed = super(CloneVDILink, self).allowed(request, datum)
        return (allowed and
                LicenseController.is_valid(request, LicenseController.VDI) and
                datum.status.lower() != ERROR)

    def get_link_url(self, datum=None):
        instance_id, project_id = (
            (datum.id, datum.tenant_id) if datum
            else (None, None)
        )
        return reverse(self.url, args=(project_id, instance_id))


# noinspection PyAbstractClass,PyUnusedLocal
class MembersAndGroupsManageAction(LinkBatchAction):
    """Multi members and groups manage"""
    name = "members_groups_update"
    multiple_next_view_name = (
        "horizon:tionix:vdi:instances:members_groups_manage"
    )

    def action_present(self, count):
        return _("Manage Members and Groups")

    def allowed(self, request, datum):
        allowed = True
        if datum:
            allowed = (datum.status.lower() in (ACTIVE, PAUSED, STOPPED) and
                       is_user_in_project(request, datum.tenant_id))
        return allowed


# noinspection PyAbstractClass, PyClassHasNoInit
class VDIEditInstance(project_actions.EditInstance):
    multiple_next_view_name = 'horizon:tionix:vdi:instances:update'


class UnassignVDIUserAction(TnxBatchActionMixin, tables.DeleteAction):

    # pylint: disable=method-hidden
    @staticmethod
    def action_present(count):
        return ungettext(
            u"Unassign user",
            u"Unassign users",
            count
        )

    # pylint: disable=method-hidden
    @staticmethod
    def action_past(count):
        return ungettext(
            u"Unassigned user",
            u"Unassigned users",
            count
        )

    def delete(self, request, obj_id):
        instance_id = self.table.kwargs['obj_id']
        TNXServersClient.remove_user(request, instance_id, obj_id)


class UnassignVDIGroupAction(TnxBatchActionMixin, tables.DeleteAction):
    permissions = ('openstack.roles.admin',)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    # pylint: disable=method-hidden
    @staticmethod
    def action_present(count):
        return ungettext(
            u"Unassign user group",
            u"Unassign user groups",
            count
        )

    # pylint: disable=method-hidden
    @staticmethod
    def action_past(count):
        return ungettext(
            u"Unassigned user group",
            u"Unassigned user groups",
            count
        )

    def delete(self, request, obj_id):
        instance_id = self.table.kwargs['obj_id']
        TNXServersClient.remove_user_group(request, instance_id, obj_id)


# noinspection PyAbstractClass
class VDIUsersFilterAction(TnxFilterAction):
    """Filter for VDI instance users"""
    name = "filter_vdi_users"
    filter_type = "server"

    filter_choices = [
        ('name', _("User Name"), False),
        ('description', _('Description'), False),
        ('id', _('User ID'), False),
    ]


# noinspection PyAbstractClass
class VDIGroupsFilterAction(TnxFilterAction):
    """Filter for VDI instance groups"""
    name = "filter_vdi_groups"
    filter_type = "server"

    filter_choices = [
        ('name', _("Name"), False),
        ('description', _('Description'), False),
        ('id', _('Group ID'), False),
    ]


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxVDIDeleteInstance(project_actions.TnxDeleteInstance):
    multiple_next_view_name = "horizon:tionix:vdi:instances:delete"
