# coding: utf-8
# pylint: disable=no-member

import logging

from django.utils.translation import ugettext_lazy as _
from django.views.decorators.debug import sensitive_variables

from horizon import exceptions, forms, messages
from horizon.utils import validators

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.images import (
    utils as image_utils
)

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.admin.instances.helpers import (
    get_admin_volumes)
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    get_project_volumes)
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_user_vdi_projects)
from tionix_dashboard.dashboards.utils import image_choice_title
from tionix_dashboard.dashboards.project.project_instances import (
    forms as project_forms)

logger = logging.getLogger(__name__)


class RebuildInstanceMixin(object):

    def __init__(self, request, *args, **kwargs):
        assert isinstance(self, forms.Form)
        super(RebuildInstanceMixin, self).__init__(request, *args, **kwargs)

        images = image_utils.get_available_images(request,
                                                  request.user.tenant_id)
        choices = [(image.id, image) for image in images]
        if choices:
            choices.insert(0, ("", _("Select Image")))
        else:
            choices.insert(0, ("", _("No images available")))
        self.fields['image_id'].choices = choices

        if not api.nova.can_set_server_password():
            self.fields.pop('action_password')
            self.fields.pop('confirm_action_password')

        try:
            disk_config_supported = api.nova.extension_supported("DiskConfig",
                                                                 request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, _('Unable to retrieve extensions '
                                         'information.'))
        else:

            if not disk_config_supported:
                self.fields.pop('disk_config')
            else:
                # Set our disk_config choices
                config_choices = [("AUTO", _("Automatic")),
                                  ("MANUAL", _("Manual"))]
                self.fields['disk_config'].choices = config_choices

    def clean(self):
        cleaned_data = super(RebuildInstanceMixin, self).clean()
        if 'action_password' in cleaned_data:
            passwd = cleaned_data.get('action_password')
            confirm = cleaned_data.get('confirm_action_password')
            if passwd != confirm:
                raise forms.ValidationError(_("Passwords do not match."))
        return cleaned_data


class TnxRebuildInstanceForm(RebuildInstanceMixin,
                             forms.SelfHandlingForm):
    image_id = forms.ChoiceField(
        label=_("Select Image"),
        widget=forms.SelectWidget(attrs={'class': 'image-selector'},
                                  data_attrs=('size', 'display-name'),
                                  transform=image_choice_title))
    action_password = forms.RegexField(
        label=_("Rebuild Password"),
        required=False,
        widget=forms.PasswordInput(),
        regex=validators.password_validator(),
        error_messages={'invalid': validators.password_validator_msg()})
    confirm_action_password = forms.CharField(
        label=_("Confirm Rebuild Password"),
        required=False,
        widget=forms.PasswordInput())
    disk_config = forms.ChoiceField(label=_("Disk Partition"),
                                    required=False)

    # We have to protect the entire "data" dict because it contains the
    # password and confirm_password strings.
    @sensitive_variables('data', 'password')
    def handle(self, request, data):
        image_id = data.get('image_id')
        action_password = data.get('action_password') or None
        disk_config = data.get('disk_config', None)
        instances = self.initial['instances']

        success_list, failed_list = [], []
        for instance in instances:
            try:
                novaclient(
                    request, project_id=instance.tenant_id
                ).servers.rebuild(
                    instance, image_id, action_password, disk_config,
                )
                success_list.append(instance.name)
            except Exception as err:
                logger.error(unicode(err))
                failed_list.append(instance.name)

        if failed_list:
            msg = _('Enable to rebuild instances: "{0}"')

            messages.error(request, msg.format('", "'.join(failed_list)))

        if success_list:
            msg = _('Rebuilding instances: "{0}"')
            messages.success(request, msg.format('", "'.join(success_list)))

        return True


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxDeleteInstanceForm(project_forms.TnxDeleteInstanceForm):
    """ Delete instances form """

    @staticmethod
    def _get_volumes(request):
        volumes = []
        if api.keystone.is_domain_admin(request):
            volumes = get_admin_volumes(request)
        else:
            for project in get_user_vdi_projects(request):
                volumes += get_project_volumes(request, project_id=project.id)

        return volumes
