# coding: utf-8
from django.core.urlresolvers import reverse
from django.template.defaultfilters import title
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon.utils import filters

from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)

from tionix_dashboard.dashboards.admin.instances.tables import (
    STATUS_CHOICES, TASK_STATUS_CHOICES)
from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions, constants)
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    MetricsInstanceShowMetricAction)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    VDIInstanceSchedulerAction)
from tionix_dashboard.dashboards.tionix.vdi.constants import (
    VDI_INSTANCE_DETAILS, VDI_PROJECT_DETAILS)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_project_detail_link)
from tionix_dashboard.dashboards.tionix.vdi.instances import actions
from tionix_dashboard.dashboards.tionix.vdi.instances.actions import (
    MembersAndGroupsManageAction)
from tionix_dashboard.dashboards.tionix.vdi.tables import BaseSessionTable
from tionix_dashboard.dashboards.utils import TABLE_OBJECT_ID_SEP
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import TnxPaginationDataTable


def get_url_for_project(datum):
    link = VDI_PROJECT_DETAILS
    obj_id = datum.tenant_id
    return reverse(link, args=(obj_id,))


def get_url_for_details(datum):
    link = VDI_INSTANCE_DETAILS
    return reverse(link, args=(datum.tenant_id, datum.id))


class BaseVDIInstancesTable(TnxPaginationDataTable):
    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link=get_url_for_details,
        form_field=EmptyCharField(max_length=255),
        update_action=actions.InstanceUpdateCell,
    )
    tenant = tables.Column(
        "tenant_name",
        link=get_url_for_project,
        verbose_name=_("Project")
    )
    ip = tables.Column(
        project_tables.get_ips,
        verbose_name=_("IP Address"),
        attrs={'data-type': "ip"}
    )
    size = tables.Column(
        project_tables.get_size,
        verbose_name=_("Size")
    )
    status = tables.Column(
        "status",
        filters=(title, filters.replace_underscores),
        verbose_name=_("Status"),
        status=True,
        status_choices=STATUS_CHOICES,
        display_choices=constants.VM_STATUS_DISPLAY_CHOICES
    )
    az = tables.Column(
        "availability_zone",
        verbose_name=_("Zone")
    )
    task = tables.Column(
        "OS-EXT-STS:task_state",
        verbose_name=_("Task"),
        empty_value=project_tables.TASK_DISPLAY_NONE,
        status_choices=TASK_STATUS_CHOICES,
        display_choices=constants.VM_TASK_DISPLAY_CHOICES
    )
    state = tables.Column(
        project_tables.get_power_state,
        filters=(title, filters.replace_underscores),
        verbose_name=_("Power State"),
        display_choices=project_tables.POWER_DISPLAY_CHOICES
    )
    tnx_operation_mode = tables.Column(
        "tnx_operation_mode",
        verbose_name=_("Operation mode"),
        display_choices=OperationModeEnum.get_choices()
    )

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(BaseVDIInstancesTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)
        self.columns['tenant'].get_link_url = self.get_project_detail_link

    def get_project_detail_link(self, instance):
        if instance.tenant:
            return get_project_detail_link(self.request, instance.tenant)
        return None


class VDIInstancesTable(BaseVDIInstancesTable):

    def get_object_id(self, datum):
        return ''.join([datum.tenant_id, TABLE_OBJECT_ID_SEP, datum.id])

    class Meta(object):
        name = "vdi_instances"
        verbose_name = _("Instances")
        row_class = actions.UpdateRow
        status_columns = ["status", "task"]
        table_actions = (
            actions.VDIInstanceFilterAction,
            actions.LaunchVDIInstanceAction,
        )

        table_actions_menu = (
            VDIInstanceSchedulerAction,
            actions.ResetStateAction,
            project_actions.TnxMultiTogglePause,
            project_actions.TnxMultiToggleUnPause,
            project_actions.TnxMultiToggleShelve,
            project_actions.TnxMultiToggleUnShelve,
            project_actions.StartInstance,
            actions.VDIStopInstance,
            actions.RebuildInstance,
            actions.TnxVDIDeleteInstance,
            MembersAndGroupsManageAction,
            actions.VDIEditInstance,
        )

        row_actions = (
            project_actions.StartInstance,
            actions.VDIInstanceManageUsers,
            project_actions.TnxTogglePause,
            actions.RebuildInstance,
            project_actions.TnxToggleShelve,
            actions.VDIStopInstance,
            actions.ResetStateAction,
            VDIInstanceSchedulerAction,
            MetricsInstanceShowMetricAction,
            actions.TnxVDIDeleteInstance,
            actions.CloneVDILink,
            actions.VDIEditInstance
        )


class SessionTable(BaseSessionTable):

    class Meta(object):
        name = "sessions"
        verbose_name = _("Sessions")
        multi_select = False
        table_actions = (actions.SessionFilterAction,)


class UsersTable(TnxPaginationDataTable, tables.DataTable):

    name = tables.Column('name',
                         link="horizon:identity:users:detail",
                         verbose_name=_('User Name'))
    description = tables.Column(lambda obj: getattr(obj, 'description', None),
                                verbose_name=_('Description'))
    user_id = tables.Column('id', verbose_name=_('User ID'))

    class Meta(object):
        name = 'users'
        verbose_name = _('Users')
        multi_select = True
        table_actions = (
            actions.VDIUsersFilterAction,
            actions.UnassignVDIUserAction,
        )
        row_actions = (
            actions.UnassignVDIUserAction,
        )


class GroupsTable(TnxPaginationDataTable, tables.DataTable):

    name = tables.Column('name',
                         link="horizon:identity:groups:detail",
                         verbose_name=_('Name'))
    description = tables.Column(lambda obj: getattr(obj, 'description', None),
                                verbose_name=_('Description'))
    user_id = tables.Column('id', verbose_name=_('Group ID'))

    class Meta(object):
        name = 'groups'
        verbose_name = _('Groups')
        multi_select = True
        table_actions = (
            actions.VDIGroupsFilterAction,
            actions.UnassignVDIGroupAction,
        )
        row_actions = (
            actions.UnassignVDIGroupAction,
        )
