# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from tionix_client.os_clients.keystone_client import get_admin_keystone_client

from horizon import tabs
from openstack_dashboard import api
from openstack_dashboard.api.keystone import get_effective_domain_id
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.dashboards.helpers import (get_sessions_list,
                                                 get_users_list)
from tionix_dashboard.dashboards.project.project_instances import (
    tabs as tnx_project_tabs)
from tionix_dashboard.dashboards.tionix.metrics.instances.tabs import (
    InstanceMetricsTab)
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_user_roles_by_project)
from tionix_dashboard.dashboards.tionix.vdi.instances.tables import (
    GroupsTable, UsersTable, SessionTable)
from tionix_dashboard.helpers.views import (
    DTVTnxPaginationMixin, ServerFilterTabMixin)

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class SessionTab(ServerFilterTabMixin, DTVTnxPaginationMixin, tabs.TableTab):
    """VDI Instance session tab"""
    name = _("Sessions")
    slug = "session"
    table_classes = (SessionTable, )
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_sessions_data(self):
        instance = self.tab_group.kwargs['instance']
        offset, limit = self.get_offset_limit()
        filters = self.get_filters(filters={'server_id': instance.id},
                                   with_marker=False)
        try:
            tenant = TNXTenantClient.get(self.request, instance.tenant_id)
        except Exception as err:
            logger.error(unicode(err))
            domain_id = get_effective_domain_id(self.request)
        else:
            domain_id = tenant.domain_id

        user_name = filters.pop('user_name', None)
        if user_name:
            filters['user_id'] = [
                user.id
                for user in get_users_list(self.request,
                                           filters={'name': user_name})
            ] or [user_name]

        sessions, total_count = get_sessions_list(
            self.request,
            domain_id=domain_id,
            limit=limit,
            offset=offset,
            filters=filters
        )
        self.set_table_footer(offset, limit, total_count)
        return sessions


# noinspection PyClassHasNoInit
class TnxConsoleTab(tnx_project_tabs.TnxConsoleTab):
    """VDI instance console tab"""
    full_screen_console = 'horizon:tionix:vdi:instances:fullscreen_console'

    def get_full_screen_console(self, instance):
        return reverse(self.full_screen_console, args=(instance.tenant_id,
                                                       instance.id,))


# noinspection PyClassHasNoInit
class OverviewTab(tnx_project_tabs.OverviewTab):
    """VDI instance details overview tab with vdi attributes"""

    is_vdi = True
    template_name = "vdi/instances/_detail_overview.html"


# noinspection PyAbstractClass
class UsersTab(ServerFilterTabMixin, IdentityMixIn, tabs.TableTab):
    name = _("Users")
    slug = "users"
    table_classes = (UsersTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_users_data(self):
        users = []
        tenant_id = self.tab_group.kwargs['tenant_id']
        user_roles = set(get_user_roles_by_project(self.request, tenant_id))
        is_project_admin = user_roles & set(self.get_admin_roles())
        keystone_client = get_admin_keystone_client()
        instance_users = self.tab_group.kwargs['instance'].tnx_users
        if is_project_admin:
            users = [keystone_client.users.get(user['id'])
                     for user in instance_users]
        elif self.request.user.id in [user['id'] for user in instance_users]:
            users = [keystone_client.users.get(self.request.user.id)]

        return users


# noinspection PyAbstractClass
class GroupsTab(ServerFilterTabMixin, IdentityMixIn, tabs.TableTab):
    name = _("Groups")
    slug = "groups"
    table_classes = (GroupsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def allowed(self, request):
        tenant_id = self.tab_group.kwargs['tenant_id']
        user_roles = set(get_user_roles_by_project(request, tenant_id))
        allowed = user_roles & set(self.get_admin_roles())
        return allowed

    def get_groups_data(self):
        groups = self.tab_group.kwargs['instance'].tnx_groups

        return [
            api.keystone.group_get(self.request, grp['id']) for grp in groups
        ]


# noinspection PyAbstractClass, PyClassHasNoInit
class VDIInstanceDetailTabs(tnx_project_tabs.TnxInstanceDetailTabs):
    """Instance details tab group"""
    tabs = (
        OverviewTab,
        tnx_project_tabs.TnxLogTab,
        TnxConsoleTab,
        tnx_project_tabs.TnxAuditTab,
        ActiveTasksTab,
        InstanceMetricsTab,
        SessionTab,
        GroupsTab,
        UsersTab,
    )
