# coding: utf-8
# pylint: disable=no-member

import mock

from django.conf import settings
from django.core.urlresolvers import reverse
from django.test import override_settings
from django.utils.http import urlencode

from horizon import messages
from horizon.utils import validators

from openstack_auth import policy as policy_backend

from openstack_dashboard.dashboards.project.images import (
    utils as image_utils
)
from openstack_dashboard import api

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    MetricsInstanceShowMetricAction)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.dashboards.tionix.vdi import helpers
from tionix_dashboard.dashboards.tionix.vdi.instances import (
    forms, tables, tabs, views)
from tionix_dashboard.dashboards.tionix.vdi.instances import workflows
from tionix_dashboard.dashboards.tionix.vdi.instances.tables import (
    VDIInstancesTable)
from tionix_dashboard.dashboards.tionix.vdi.instances.views import (
    TnxVDIDeleteInstanceView)
from tionix_dashboard.dashboards.utils import TABLE_OBJECT_ID_SEP
from tionix_dashboard.helpers import vm_constants
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, FakeUser, BaseAdminClientTestCase)

DELETE_URL = "horizon:project:instances:delete"
DETAIL_URL = "horizon:tionix:vdi:instances:detail"
INDEX_URL = reverse("horizon:tionix:vdi:instances_tab")
CLONE_URL = "horizon:tionix:vdi:instances:clone"


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
@mock.patch.object(api.nova, 'extension_supported', mock.Mock())
class VDIInstancesTablesTest(BaseClientTestCase):

    def test_vdi_instances_columns(self):

        table = tables.VDIInstancesTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'tenant', 'name', 'ip',
                    'status', 'state', 'tnx_operation_mode', 'actions', 'task',
                    'az', 'size'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_vdi_instances_actions(self):
        instances = getattr(self, 'servers').list()

        table = tables.VDIInstancesTable(self.request, data=instances)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted([
                'filter_vdi_instances',
                'launch',
                'tnx_pause',
                'tnx_unpause',
                'tnx_shelve',
                'tnx_unshelve',
                'start',
                'stop',
                'rebuild',
                'delete',
                'edit',
                'members_groups_update'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(VDIInstancesTable, 'get_object_id',
                       mock.Mock(return_value=''))
    @mock.patch.object(MetricsInstanceShowMetricAction,
                       'get_link_url',
                       mock.Mock(return_value=''))
    def test_vdi_instance_row_actions(self):
        table = tables.VDIInstancesTable(self.request)

        instance = self.tnx_servers.list(
            search_opts={'status': vm_constants.ACTIVE.upper()})[0]
        # FIXME test aren't correct! Check the action list
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['pause', 'rebuild', 'shelve', 'stop',
                    'show_instance_metrics', 'delete', 'clone', 'edit',
                    'vdi_assign_to_users'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(VDIInstancesTable, 'get_object_id',
                       mock.Mock(return_value=''))
    @mock.patch.object(MetricsInstanceShowMetricAction,
                       'get_link_url',
                       mock.Mock(return_value=''))
    def test_vdi_error_instance_row_actions(self):
        table = tables.VDIInstancesTable(self.request)

        instance = self.tnx_servers.list(
            search_opts={'status': vm_constants.ERROR.upper()})[0]
        # FIXME test aren't correct! Need more actions here
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['show_instance_metrics', 'delete', 'edit'])
        )

    def test_session_table_columns(self):

        table = tables.SessionTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['user_name', 'created_at', 'closed_at', 'client_os',
                    'connection_type', 'client_ip', 'client_info'])
        )


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
@mock.patch.object(api.nova, 'extension_supported', mock.Mock())
class AdminVDIInstancesTablesTest(BaseAdminClientTestCase):

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @mock.patch.object(VDIInstancesTable, 'get_object_id',
                       mock.Mock(return_value=''))
    @mock.patch.object(MetricsInstanceShowMetricAction,
                       'get_link_url',
                       mock.Mock(return_value=''))
    @mock.patch.object(api.keystone, 'roles_for_user')
    def test_vdi_instances_row_actions(self, roles_for_user):
        table = tables.VDIInstancesTable(self.request)

        roles_for_user.return_value = [Empty(name="user")]

        instance = self.tnx_servers.list()[0]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['pause', 'rebuild', 'shelve', 'stop', 'clone',
                    'instance_schedule', 'show_instance_metrics', 'delete',
                    'edit', 'vdi_assign_to_users'])
        )

        instance = self.tnx_servers.list()[2]
        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['delete', 'instance_schedule', 'reset_state',
                    'show_instance_metrics', 'edit'])
        )

        instance = self.tnx_servers.list()[9]

        roles_for_user.return_value = [
            Empty(name="user"), Empty(name="admin")
        ]

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(instance)]),
            sorted(['delete', 'instance_schedule', 'rebuild', 'clone',
                    'show_instance_metrics', 'start', 'stop', 'edit'])
        )


class VDIInstancesTabsTestCase(BaseClientTestCase):

    @mock.patch.object(LicenseController,
                       'is_valid',
                       mock.Mock(return_value=True))
    @mock.patch.object(tabs.GroupsTab, 'allowed',
                       mock.Mock(return_value=True))
    @mock.patch.object(tabs.UsersTab, 'allowed',
                       mock.Mock(return_value=True))
    def test_vdi_tabs(self):

        tab_group = tabs.VDIInstanceDetailTabs(self.request)

        self.assertListEqual(
            sorted([tab.slug for tab in tab_group.get_tabs()]),
            sorted(['overview', 'log', 'console', 'audit', 'scheduler',
                    'session', 'metrics', 'groups', 'users'])
        )


class VDIInstancesFormsTest(BaseClientTestCase):
    @mock.patch.object(api.nova,
                       'server_rebuild',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(image_utils,
                       'get_available_images',
                       mock.Mock(return_value=[]))
    @mock.patch.object(messages, 'success')
    @mock.patch.object(messages, 'error')
    def test_rebuild_form_success(self, error, success):
        instances = self.tnx_servers.list()[:2]

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(tenant_id=None),
            horizon={
                'async_messages': []
            }
        )

        form = forms.TnxRebuildInstanceForm(request)
        form.initial = {
            'instances': instances
        }

        form.handle(request, {})

        self.assertTrue(success.called)

        self.assertFalse(error.called)

    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.forms'
                '.novaclient', mock.Mock(side_effect=Exception))
    @mock.patch.object(api.nova, 'extension_supported', mock.Mock())
    @mock.patch.object(image_utils,
                       'get_available_images',
                       mock.Mock(return_value=[]))
    @mock.patch.object(messages, 'success')
    @mock.patch.object(messages, 'error')
    def test_rebuild_form_error(self, error, success):
        instances = self.tnx_servers.list()[:2]

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(tenant_id=None),
            horizon={
                'async_messages': []
            }
        )

        form = forms.TnxRebuildInstanceForm(request)
        form.initial = {
            'instances': instances
        }

        form.handle(request, {})

        self.assertFalse(success.called)

        self.assertTrue(error.called)

    @mock.patch.object(image_utils,
                       'get_available_images',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.nova,
                       'can_set_server_password',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.nova,
                       'extension_supported',
                       mock.Mock(return_value=True))
    def test_rebuild_form_fields(self):
        form = forms.TnxRebuildInstanceForm(self.request)

        self.assertListEqual(
            sorted(form.fields.keys()),
            sorted(['image_id', 'action_password',
                    'confirm_action_password', 'disk_config'])
        )

    @mock.patch.object(image_utils,
                       'get_available_images',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.nova,
                       'can_set_server_password',
                       mock.Mock(return_value=False))
    @mock.patch.object(api.nova,
                       'extension_supported',
                       mock.Mock(return_value=False))
    def test_rebuild_form_fields_2(self):
        form = forms.TnxRebuildInstanceForm(self.request)

        self.assertListEqual(form.fields.keys(), ['image_id'])

    @mock.patch.object(image_utils,
                       'get_available_images',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.nova,
                       'can_set_server_password',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.nova,
                       'extension_supported',
                       mock.Mock(return_value=True))
    def test_rebuild_form_clean(self):
        image = self.tnx_images.list()[0]

        data = {'image': image.id,
                'config_disk': 'config_disk',
                'action_password': 'pass',
                'confirm_action_password': 'pass 2'}

        form = forms.TnxRebuildInstanceForm(self.request)
        form.cleaned_data = data

        with self.assertRaises(validators.ValidationError):
            form.clean()


class VDIInstancesViewsTest(BaseClientTestCase):

    def test_rebuild_get_objects(self):
        instances = self.tnx_servers.list()[:3]

        view = views.TnxRebuildView()

        view.request = get_fake_request(
            get_params={
                'id': [''.join([instance.tenant_id,
                                TABLE_OBJECT_ID_SEP,
                                str(instance.id)])
                       for instance in instances]
            },
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        instances_from_view = view.get_objects()

        self.assertIsNotNone(instances_from_view)

        self.assertEqual(len(instances_from_view), 2)

    @mock.patch.object(TnxVDIDeleteInstanceView,
                       'get_tasks_rows',
                       mock.Mock(return_value={}))
    @mock.patch.object(api.nova, 'server_delete', mock.Mock(return_value=True))
    @mock.patch.object(api.nova, 'server_get')
    def test_delete_from_index(self, server_get):
        instance = self.tnx_servers.list()[0]

        server_get.return_value = instance

        url = '?'.join([reverse(DELETE_URL), urlencode({'id': instance.id})])

        req = self.factory.post(url)

        req.META['HTTP_REFERER'] = ''.join([settings.TESTSERVER,
                                            settings.WEBROOT,
                                            INDEX_URL])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = TnxVDIDeleteInstanceView.as_view()(req)

        self.assertIn(INDEX_URL, response.url)

    @mock.patch.object(TnxVDIDeleteInstanceView,
                       'get_tasks_rows',
                       mock.Mock(return_value={}))
    @mock.patch.object(api.nova, 'server_delete', mock.Mock(return_value=True))
    @mock.patch.object(api.nova, 'server_get')
    def test_delete_from_detail(self, server_get):
        instance = self.tnx_servers.list()[0]

        server_get.return_value = instance

        url = '?'.join([reverse(DELETE_URL), urlencode({'id': instance.id})])

        req = self.factory.post(url)

        req.META['HTTP_REFERER'] = ''.join([
            settings.TESTSERVER,
            settings.WEBROOT,
            reverse(DETAIL_URL, args=(instance.tenant_id, instance.id))])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = TnxVDIDeleteInstanceView.as_view()(req)

        self.assertIn(INDEX_URL, response.url)

    @mock.patch('api.keystone.get_default_domain')
    def test_clone_get_initial(self, get_default_domain):
        instance = self.tnx_servers.list()[0]
        domain = self.tnx_domains.list()[0]

        get_default_domain.return_value = domain

        view = views.CloneVDIInstanceView()
        view.request = self.request
        view.kwargs = {'instance_id': instance.id,
                       'tenant_id': instance.tenant_id}

        initial = view.get_initial().dict()
        initial_instance = initial.pop('instance')

        self.assertEqual(initial_instance.id, instance.id)

        self.assertDictEqual(initial,
                             {u'project': instance.tenant_id,
                              u'domain_id': domain.id,
                              u'name': u'Clone of {}'.format(instance.name)})

    @mock.patch.object(api.keystone,
                       'is_domain_admin',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.keystone, 'group_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'role_assignments_list',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'user_list')
    @mock.patch.object(api.keystone, 'get_effective_domain_id')
    @mock.patch.object(api.keystone, 'get_default_domain')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_project_users')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_user_roles_by_project')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_projects_for_instance_launch')
    def test_clone(self, get_projects, get_user_roles_by_project,
                   get_project_users, get_default_domain,
                   get_effective_domain_id, user_list):
        instance = self.tnx_servers.list()[0]
        users = self.tnx_users.list()
        tenants = self.tnx_tenants.list()
        domain = self.tnx_domains.get(1)
        user = users[0]

        get_projects.return_value = [(tenant.id, tenant.name)
                                     for tenant in tenants]
        get_user_roles_by_project.return_value = [user.id]
        get_project_users.return_value = [user.id]
        get_effective_domain_id.return_value = domain.id
        user_list.return_value = users
        get_projects.return_value = [(tenant.id, tenant.name)
                                     for tenant in tenants]
        get_default_domain.return_value = domain
        get_effective_domain_id.return_value = domain.id

        response = self.client.get(reverse(CLONE_URL, args=(instance.tenant_id,
                                                            instance.id)))

        self.assertIn(u'Clone of {0}'.format(instance.name), response.content)


class VDIInstanceWorkflowsTest(BaseClientTestCase):
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.helpers.policy')
    @mock.patch('api.keystone.get_default_domain', new=mock.Mock())
    @mock.patch.object(api.keystone, 'tenant_list')
    @mock.patch.object(helpers, 'get_favor_ids_by_tenant')
    @mock.patch.object(helpers, 'get_image_by_tenant')
    @mock.patch.object(helpers, 'get_networks_by_tenant')
    @mock.patch.object(helpers, 'get_quota_by_tenant')
    def test_launch_project_not_enabled(self, get_quota, network_list,
                                        get_image, get_favor_ids_by_tenant,
                                        tenant_list, policy):
        tenants = getattr(self, 'tenants').list()
        flavors = self.tnx_flavors.list()
        tenant_ids = [tenant.id for tenant in tenants if tenant.enabled]

        get_quota.return_value = {
            'totalInstancesUsed': 1,
            'maxTotalInstances': 10,
        }

        tenant_list.return_value = (tenants, False)
        policy.check.return_value = False
        get_image.side_effect = lambda req, id1, id2: self.tnx_images.get(id2)
        get_favor_ids_by_tenant.return_value = [flavor.id
                                                for flavor in flavors]
        network_list.return_value = {
            network.id: network.to_dict()
            for network in self.tnx_networks.list()
        }

        request = get_fake_request(
            is_ajax=True,
            method='GET',
            user=FakeUser(tenant_id=None, token=None,
                          authorized_tenants=tenants),
            horizon={
                'async_messages': []
            }
        )

        action = workflows.VDIInstanceLaunchDetailsAction(request, context={})

        choices = [project_id for project_id, _project_name in
                   action.fields['project'].choices if project_id]

        self.assertEqual(len(choices), 2)
        self.assertListEqual(tenant_ids, choices)

    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.helpers.policy')
    @mock.patch.object(api.keystone, 'tenant_list')
    @mock.patch.object(helpers, 'get_favor_ids_by_tenant')
    @mock.patch.object(helpers, 'get_image_by_tenant')
    @mock.patch.object(helpers, 'get_networks_by_tenant')
    @mock.patch.object(helpers, 'get_quota_by_tenant')
    def test_launch_projects_no_auth(self, get_quota, network_list, get_image,
                                     get_favor_ids_by_tenant,
                                     tenant_list, policy):
        tenants = getattr(self, 'tenants').list()
        flavors = self.tnx_flavors.list()
        tenant_list.return_value = (tenants, False)
        policy.check.return_value = False
        tenant_ids = [tenants[0].id]

        get_quota.return_value = {
            'totalInstancesUsed': 1,
            'maxTotalInstances': 10,
        }
        get_image.side_effect = lambda req, id1, id2: self.tnx_images.get(id2)
        get_favor_ids_by_tenant.return_value = [flavor.id
                                                for flavor in flavors]
        network_list.return_value = {
            network.id: network.to_dict()
            for network in self.tnx_networks.list()
        }

        request = get_fake_request(
            is_ajax=True,
            method='GET',
            user=FakeUser(tenant_id=None, token=None,
                          authorized_tenants=[tenants[0]]),
            horizon={
                'async_messages': []
            },
            session={
                'domain_context': 1,
                'domain_context_name': 'domain 1',
            }
        )

        action = workflows.VDIInstanceLaunchDetailsAction(request, context={})

        choices = [project_id for project_id, _project_name in
                   action.fields['project'].choices if project_id]

        self.assertEqual(len(choices), 1)

        self.assertListEqual(tenant_ids, choices)

    @mock.patch('api.keystone.get_default_domain', mock.Mock())
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.helpers.policy')
    @mock.patch.object(api.keystone, 'tenant_list')
    @mock.patch.object(helpers, 'get_favor_ids_by_tenant')
    @mock.patch.object(helpers, 'get_image_by_tenant')
    @mock.patch.object(helpers, 'get_networks_by_tenant')
    @mock.patch.object(helpers, 'get_quota_by_tenant')
    def test_launch_project_broken(self, get_quota, network_list, get_image,
                                   get_favor_ids_by_tenant,
                                   tenant_list, policy):
        tenants = self.tnx_tenants.list()
        flavors = self.tnx_flavors.list()
        tenant_ids = [tenant.id for tenant in tenants
                      if tenant.id in ['1', '3']]

        tenant_list.return_value = (tenants, False)
        policy.check.return_value = False
        get_image.side_effect = lambda req, id1, id2: self.tnx_images.get(id2)
        get_favor_ids_by_tenant.return_value = [flavor.id
                                                for flavor in flavors]
        network_list.return_value = {
            network.id: network.to_dict()
            for network in self.tnx_networks.list()
        }

        get_quota.return_value = {
            'totalInstancesUsed': 1,
            'maxTotalInstances': 10,
        }

        request = get_fake_request(
            is_ajax=True,
            method='GET',
            user=FakeUser(tenant_id=None, token=None,
                          authorized_tenants=tenants),
            horizon={
                'async_messages': []
            }
        )

        action = workflows.VDIInstanceLaunchDetailsAction(request, context={})

        choices = [project_id for project_id, _project_name in
                   action.fields['project'].choices if project_id]

        self.assertEqual(len(choices), 2)

        self.assertListEqual(tenant_ids, choices)

    @mock.patch.object(api.keystone,
                       'is_domain_admin',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.keystone, 'group_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'role_assignments_list',
                       mock.Mock(return_value=[]))
    @mock.patch.object(workflows.CloneVDIInstance,
                       'is_project_admin',
                       mock.Mock(return_value=True))
    @mock.patch.object(api.keystone, 'user_list')
    @mock.patch.object(api.keystone, 'get_effective_domain_id')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_project_users')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_user_roles_by_project')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_projects_for_instance_launch')
    def test_clone_admin(self, get_projects, get_user_roles_by_project,
                         get_project_users, get_effective_domain_id,
                         user_list):
        instance = self.tnx_servers.list()[0]
        users = self.tnx_users.list()
        tenants = self.tnx_tenants.list()
        domain = self.tnx_domains.get(1)

        get_projects.return_value = [(tenant.id, tenant.name)
                                     for tenant in tenants]
        get_user_roles_by_project.return_value = [users[0].id]
        get_project_users.return_value = [users[0].id]
        get_effective_domain_id.return_value = domain.id
        user_list.return_value = users

        request = get_fake_request(
            method='GET',
            user=FakeUser(tenant_id=None, token=None),
            horizon={
                'async_messages': []
            },
            session={}
        )

        workflow = workflows.CloneVDIInstance(
            request, context_seed={'instance': instance})

        self.assertEqual(len(workflow.default_steps), 4)

    @mock.patch.object(api.keystone,
                       'is_domain_admin',
                       mock.Mock(return_value=False))
    @mock.patch.object(api.keystone, 'group_list', mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'role_assignments_list',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'user_list')
    @mock.patch.object(api.keystone, 'get_effective_domain_id')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_project_users')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_user_roles_by_project')
    @mock.patch('tionix_dashboard.dashboards.tionix.vdi.instances.workflows'
                '.get_projects_for_instance_launch')
    def test_clone_not_admin(self, get_projects, get_user_roles_by_project,
                             get_project_users, get_effective_domain_id,
                             user_list):
        instance = self.tnx_servers.list()[0]
        users = self.tnx_users.list()
        tenants = self.tnx_tenants.list()
        domain = self.tnx_domains.get(1)

        get_projects.return_value = [(tenant.id, tenant.name)
                                     for tenant in tenants]
        get_user_roles_by_project.return_value = [users[0].id]
        get_project_users.return_value = [users[0].id]
        get_effective_domain_id.return_value = domain.id
        user_list.return_value = users

        request = get_fake_request(
            method='GET',
            user=FakeUser(tenant_id=None, token=None),
            horizon={
                'async_messages': []
            },
            session={}
        )

        workflow = workflows.CloneVDIInstance(
            request, context_seed={'instance': instance})

        self.assertEqual(len(workflow.default_steps), 3)
