# coding: utf-8

import logging

from django.core.urlresolvers import reverse_lazy, reverse
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import exceptions, messages, views, workflows
from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard.api.keystone import is_domain_admin

from openstack_dashboard.dashboards.project.instances import (
    views as project_views)
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.security_groups import (
    get_security_groups_by_instance)
from tionix_dashboard.dashboards.project.project_instances import (
    views as project_instances_view
)
from tionix_dashboard.dashboards.tionix.vdi.constants import (
    VDI_INSTANCE_DELETE)
from tionix_dashboard.dashboards.tionix.vdi.instances.forms import (
    TnxRebuildInstanceForm, TnxDeleteInstanceForm)
from tionix_dashboard.dashboards.tionix.vdi.instances.tables import (
    VDIInstancesTable)
from tionix_dashboard.dashboards.tionix.vdi.instances.tabs import (
    VDIInstanceDetailTabs)
from tionix_dashboard.dashboards.tionix.vdi.instances.workflows import (
    UpdateVDIInstanceMembers, VDIInstanceLaunchWorkflow, CloneVDIInstance,
    MultiUpdateMembersWithGroups, VDIUpdateInstance)
from tionix_dashboard.dashboards.utils import TABLE_OBJECT_ID_SEP
from tionix_dashboard.helpers.views import (
    RefererRedirectMixin, LinkBatchWorkflowViewMixin, LinkBatchFormView,
    WorkflowWithLazyMembers)
from tionix_dashboard.helpers.vm_constants import ACTIVE, SHUTOFF, ERROR
from tionix_dashboard.url_resolve_map import (
    VDI_INSTANCE_DETAILS, VDI_INSTANCES_INDEX)

logger = logging.getLogger(__name__)


# noinspection PyClassHasNoInit
class DetailView(project_instances_view.TnxDetailView):

    tab_group_class = VDIInstanceDetailTabs
    actions_table = VDIInstancesTable
    redirect_url = VDI_INSTANCE_DETAILS
    failure_url = VDI_INSTANCES_INDEX
    scheduler_tab_url = 'horizon:tionix:vdi:instances:scheduler_tab'

    def _get_url(self, instance):
        return reverse(self.redirect_url,
                       args=[instance.tenant_id, instance.id])

    def get_instance(self, instance_id):
        tenant_id = (None
                     if api.keystone.is_domain_admin(self.request)
                     else self.kwargs['tenant_id'])

        return TNXServersClient.get(self.request, instance_id, tenant_id)

    def get_security_groups(self, instance_id):
        return get_security_groups_by_instance(
            self.request, instance_id, project_id=self.kwargs['tenant_id'])

    def get_tabs(self, request, **kwargs):
        instance = self.get_data()
        kwargs['obj_id'] = instance.id
        kwargs['instance'] = instance
        kwargs['scheduler_tab_url'] = reverse(self.scheduler_tab_url,
                                              args=(instance.tenant_id,
                                                    instance.id,))
        return self.tab_group_class(request, **kwargs)


class VDIInstanceLaunchView(WorkflowWithLazyMembers, workflows.WorkflowView):
    """Vdi instance launch workflow"""
    ajax_template_name = 'vdi/instances/_launch_workflow.html'
    workflow_class = VDIInstanceLaunchWorkflow

    def get_initial(self):
        initial = super(VDIInstanceLaunchView, self).get_initial()

        domain = api.keystone.get_default_domain(self.request, get_name=False)
        initial["domain_id"] = domain.id
        return initial


class ManageMembersView(RefererRedirectMixin,
                        WorkflowWithLazyMembers,
                        workflows.WorkflowView):

    workflow_class = UpdateVDIInstanceMembers

    @memoized.memoized_method
    def _get_object(self):
        """Retrieve instance object"""
        _project_id, instance_id = self.kwargs['instance_id'].split(
            TABLE_OBJECT_ID_SEP
        )
        try:
            server = novaclient(self.request).tnx_servers.get(instance_id)
        except Exception as err:
            logger.error(unicode(err))
            raise exceptions.Http302(self.get_redirect_url())
        return server

    def get_initial(self):
        initial = super(ManageMembersView, self).get_initial()

        domain = api.keystone.get_default_domain(self.request, get_name=False)
        instance = self._get_object()
        initial.update({'instance': instance,
                        'project_id': instance.id,
                        'domain_id': domain.id})
        return initial


# noinspection PyClassHasNoInit
class RebuildView(project_views.RebuildView):
    template_name = 'vdi/instances/rebuild.html'
    success_url = reverse_lazy('horizon:tionix:vdi:index')


class FullScreenConsoleView(views.HorizonTemplateView):
    template_name = 'project/project_instances/_full_screen_console.html'
    return_url = VDI_INSTANCE_DETAILS

    def get_context_data(self, **kwargs):
        return {'console_url': self.request.GET.get('console_url'),
                'return_url': reverse(self.return_url,
                                      args=(self.kwargs['tenant_id'],
                                            self.kwargs['instance_id'],))}


# noinspection PyClassHasNoInit
class ResetStateView(project_instances_view.ResetStateView):
    redirect_url = success_url = reverse_lazy("horizon:tionix:vdi:index")
    confirm_url = reverse_lazy("horizon:tionix:vdi:instances:reset_state")

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_objects(self, *args, **kwargs):
        instances, failed = [], []
        for instance_id in self.get_object_ids():
            project_id, instance_id = instance_id.split(TABLE_OBJECT_ID_SEP)
            try:
                instance = TNXServersClient.get(self.request,
                                                instance_id,
                                                project_id=project_id)
            except Exception as err:
                logger.error(err.message)
                continue

            if instance.status not in self.states:
                failed.append(instance.name)
            else:
                instances.append(instance)

        if failed:
            # pylint: disable=no-member
            failed_message = self.failure_msg.format(u'", "'.join(failed))
            messages.error(self.request, failed_message)

        if not instances:
            raise exceptions.Http302(self.redirect_url)

        return instances


class TnxRebuildView(RefererRedirectMixin, LinkBatchFormView):
    form_class = TnxRebuildInstanceForm
    template_name = 'vdi/instances/rebuild.html'
    modal_header = _('Rebuild Instances: "{0}"')
    submit_label = _('Rebuild')
    redirect_url = success_url = reverse_lazy("horizon:tionix:vdi:index")
    failure_msg = _('"Rebuild Instance" is not available for instances: "{0}"')
    states = (ACTIVE, SHUTOFF)

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_objects(self, *args, **kwargs):
        instances, failed = [], []
        for instance_id in self.get_object_ids():
            project_id, instance_id = instance_id.split(TABLE_OBJECT_ID_SEP)
            try:
                instance = TNXServersClient.get(self.request,
                                                instance_id,
                                                project_id=project_id)
            except Exception as err:
                logger.error(err.message)
                continue

            if instance.status.lower() not in self.states:
                failed.append(instance.name)
            else:
                instances.append(instance)

        if failed:
            # pylint: disable=no-member
            failed_message = self.failure_msg.format(u'", "'.join(failed))
            messages.error(self.request, failed_message)

        if not instances:
            raise exceptions.Http302(self.redirect_url)

        return instances

    def get_initial(self):
        instances = self.get_objects()

        instances_names = [instance.name for instance in instances]

        # New modal header
        self.modal_header = (
            self.modal_header.format('", "'.join(instances_names))
        )

        return {'instances': instances,
                'instances_names': instances_names,
                'redirect_url': self.redirect_url}


class CloneVDIInstanceView(RefererRedirectMixin, VDIInstanceLaunchView):

    workflow_class = CloneVDIInstance

    def get_initial(self):
        initial = super(CloneVDIInstanceView, self).get_initial()

        project_id = (self.kwargs['tenant_id']
                      if not is_domain_admin(self.request) else None)
        instance_id = self.kwargs['instance_id']
        try:
            nova_client = novaclient(self.request, project_id=project_id)
            instance = nova_client.tnx_servers.get(instance_id)
        except Exception as err:
            logger.error(unicode(err))
        else:
            initial.update(
                instance=instance,
                name=u'Clone of {0}'.format(instance.name),
                project=instance.tenant_id
            )
        return initial


class MultiManageMembersView(RefererRedirectMixin,
                             LinkBatchWorkflowViewMixin,
                             WorkflowWithLazyMembers,
                             workflows.WorkflowView,
                             IdentityMixIn):
    """Multi users assigning"""
    workflow_class = MultiUpdateMembersWithGroups
    submit_url = "horizon:tionix:vdi:instances:members_groups_manage"

    @memoized.memoized_method
    def _get_objects(self):
        """Retrieve instance objects and project"""
        first_project_id = None
        success_instances, failure_instances = [], []
        for instance_id in self.get_object_ids():
            project_id, instance_id = instance_id.split(TABLE_OBJECT_ID_SEP)
            if is_domain_admin(self.request):
                project_id = None
            try:
                instance = novaclient(
                    self.request, project_id=project_id
                ).tnx_servers.get(instance_id)
            except Exception as err:
                logger.error(unicode(err))
                raise exceptions.Http302(self.get_redirect_url())
            else:
                if first_project_id is None:
                    first_project_id = project_id
                elif project_id != first_project_id:
                    messages.error(
                        self.request,
                        _('Selected instances are taken from '
                          'different project.')
                    )
                    raise exceptions.Http302(self.get_redirect_url())

                if instance.status.lower() in (ERROR,):
                    failure_instances.append(instance.name)

        if failure_instances:
            messages.error(
                self.request,
                message=_('Action is not available for objects: '
                          '"{0}"').format('", "'.join(failure_instances)))
            if not success_instances:
                raise exceptions.Http302(self.get_redirect_url())

        return success_instances, first_project_id

    def get_initial(self):
        """Retrieve initial"""
        initial = super(MultiManageMembersView, self).get_initial()
        domain = api.keystone.get_default_domain(self.request, get_name=False)
        instances, project_id = self._get_objects()
        initial.update(domain_id=domain.id,
                       instances=instances,
                       project_id=project_id)
        return initial


# noinspection PyClassHasNoInit
class VDIUpdateView(project_instances_view.UpdateView):
    workflow_class = VDIUpdateInstance
    success_url = reverse_lazy(VDI_INSTANCES_INDEX)
    submit_url = reverse_lazy('horizon:tionix:vdi:instances:update')

    # noinspection PyUnusedLocal
    @memoized.memoized_method
    def get_object(self, *args, **kwargs):
        success_instances, failure_instances = [], []
        for instance_id in self.get_object_ids():
            project_id, instance_id = instance_id.split(TABLE_OBJECT_ID_SEP)
            if is_domain_admin(self.request):
                project_id = None
            try:
                success_instances.append(
                    novaclient(
                        self.request,
                        project_id=project_id
                    ).tnx_servers.get(instance_id)
                )
            except Exception as err:
                logger.error(unicode(err))
                failure_instances.append(instance_id)

        if failure_instances:
            messages.error(self.request,
                           ungettext_lazy(
                               u'Unable to retrieve instance: {0}',
                               u'Unable to retrieve instances: {0}',
                               len(failure_instances)
                           ).format(u'", "'.join(failure_instances)))

        return success_instances


# noinspection PyClassHasNoInit
class TnxVDIDeleteInstanceView(project_instances_view.TnxDeleteInstanceView):
    """ Delete tnx instances view """
    form_class = TnxDeleteInstanceForm
    success_url = reverse_lazy(VDI_INSTANCES_INDEX)
    confirm_url = reverse_lazy(VDI_INSTANCE_DELETE)
