# coding: utf-8
# pylint: disable=super-on-old-class, len-as-condition

import logging

import thread
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, messages, workflows
from horizon.utils.memoized import memoized_method

from openstack_dashboard import api
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.admin.instances import (
    workflows as admin_workflows)
from tionix_dashboard.dashboards.helpers import Empty
from tionix_dashboard.dashboards.project.project_instances import (
    workflows as project_workflows)
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    delayed_volume_attach)
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_INDEX
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_projects_for_instance_launch, get_quota_by_tenant,
    get_project_users, get_user_roles_by_project)
from tionix_dashboard.dashboards.tionix.vdi.projects.workflows import (
    CloneStepMixin)
from tionix_dashboard.dashboards.utils import (
    quotas_usages_validation, is_domain_supported, quota_map)
from tionix_dashboard.helpers.fields import AutoGenNameCharField
from tionix_dashboard.helpers.forms import MultiManagingFormMixin
from tionix_dashboard.helpers.views import RefererRedirectMixin
from tionix_dashboard.helpers.vm_constants import VMTypes
from tionix_dashboard.helpers.workflows import (
    LazyUpdateMembersStep, LazyMembershipAction)
from tionix_dashboard.url_resolve_map import VDI_INSTANCES_INDEX

logger = logging.getLogger(__name__)


UPDATE_USERS_ACTION_SLUG = 'update_users'
UPDATE_GROUPS_ACTION_SLUG = 'update_groups'

PROJECT_GROUP_ENABLED = is_domain_supported()


class VDIIdentityMixIn(IdentityMixIn):

    @memoized_method
    def is_project_admin(self, request, project_id):
        return bool(set(self.get_admin_roles()) &
                    set(get_user_roles_by_project(request, project_id)))


class FieldGroup(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)


class UpdateVDIInstanceMembersMixin(object):
    """Updates vdi instance members"""

    is_group_editing_enabled = False

    @classmethod
    def _unregister(cls, step):
        return step

    def _update_members(self, instance, data, delete_old_records=True):

        step = self.get_step(UPDATE_USERS_ACTION_SLUG)

        user_list = data.get(
            step.get_member_field_name(step.action.role.id)
        )

        if user_list is not None:
            tnx_users = instance.tnx_users or []
            instance_users = [user['id'] for user in tnx_users]

            for user in user_list:
                if user not in instance_users:
                    instance.add_user(user)
            if delete_old_records:
                for user in instance_users:
                    if user not in user_list:
                        instance.remove_user(user)

        if self.is_group_editing_enabled:
            step = self.get_step(UPDATE_GROUPS_ACTION_SLUG)

            group_list = data.get(
                step.get_member_field_name(step.action.role.id)
            )

            if group_list is not None:
                instance_groups = [group['id']
                                   for group in instance.tnx_groups or []]

                for group in group_list:
                    if group not in instance_groups:
                        instance.add_user_group(group)
                if delete_old_records:
                    for group in instance_groups:
                        if group not in group_list:
                            instance.remove_user_group(group)


class InstanceMembershipAction(LazyMembershipAction):

    role = None

    def _get_membership_data(self, request):
        return []

    def _populate_fields(self, request, workflow, **kwargs):

        default_field_name = self.get_default_role_field_name()
        self.fields[default_field_name] = forms.CharField(required=False)
        self.fields[default_field_name].initial = self.role.id

        field_name = self.get_member_field_name(self.role.id)
        self.fields[field_name] = forms.MultipleChoiceField(
            required=False, label=self.role.name
        )

        self.fields[field_name].choices = self._get_membership_data(request)


class InstanceUpdateMembersStep(LazyUpdateMembersStep):

    template_name = 'vdi/instances/_launch_users_step.html'
    show_roles = False

    def _get_members(self):
        return [self.action_class.role]


class CreateUsersAction(InstanceMembershipAction, VDIIdentityMixIn):

    role = FieldGroup(id=u'1', name=u'instance_users')

    def _get_users_by_project(self, request, domain_id, project_id):
        users_list = []
        if self.is_project_admin(request, project_id):
            try:
                all_users = {
                    user.id: user.name
                    for user in api.keystone.user_list(request,
                                                       domain=domain_id)
                }
                users_list = [
                    (user_id, all_users.get(user_id))
                    for user_id in get_project_users(request, project_id)
                    if user_id in all_users
                ]
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve user list.'))
        else:
            users_list.append((self.request.user.id,
                               self.request.user.username))

        return users_list

    def _get_membership_data(self, request):
        domain_id = api.keystone.get_effective_domain_id(self.request)
        project_id = (self.initial.get("project_id") or
                      request.GET.get('project'))

        return self._get_users_by_project(request, domain_id, project_id)

    class Meta(object):
        name = _("Instance users")
        slug = UPDATE_USERS_ACTION_SLUG


class UpdateUsersAction(CreateUsersAction):

    role = FieldGroup(id=u'1', name=u'instance_users')

    def _get_membership_data(self, request):
        domain_id = api.keystone.get_effective_domain_id(self.request)
        project_id = getattr(self.initial['instance'], 'tnx_tenant_id', None)
        return self._get_users_by_project(request, domain_id, project_id)

    def _populate_fields(self, request, workflow, **kwargs):
        super(UpdateUsersAction, self)._populate_fields(
            request, workflow, **kwargs
        )
        instance_users = [
            user['id']
            for user in getattr(self.initial["instance"], 'tnx_users', [])]

        # pylint: disable=no-member
        field_name = self.get_member_field_name(self.role.id)
        session_members = request.session.get('selected_members', None)
        if session_members is not None and field_name in session_members:
            initial_members = session_members.get(field_name, [])
        else:
            initial_members = instance_users
        self.fields[field_name].initial = initial_members

    class Meta(object):
        name = _("Instance users")
        slug = UPDATE_USERS_ACTION_SLUG


class CreateUsersStep(InstanceUpdateMembersStep):
    action_class = CreateUsersAction

    available_list_title = _("All Users")
    members_list_title = _("Instance users")
    no_available_text = _("No users found.")
    no_members_text = _("No users.")


class UpdateUsersStep(CreateUsersStep):
    action_class = UpdateUsersAction
    template_name = "horizon/common/_workflow_step_update_members.html"
    depends_on = ('instance', 'project_id')


class CloneUsersStep(CloneStepMixin, UpdateUsersStep):
    depends_on = ('instance', 'project')

    def get_initial_data(self):
        instance = self.workflow.context_seed['instance']
        return {self.get_member_field_name(self.action.role.id): [
            user['id'] for user in getattr(instance, 'tnx_users', [])
        ]}


class CreateGroupsAction(InstanceMembershipAction, VDIIdentityMixIn):

    role = FieldGroup(id=u'1', name=u'instance_groups')

    # pylint: disable=too-many-nested-blocks
    def _get_membership_data(self, request):
        all_groups, available_groups = [], []
        domain_id = self.initial.get("domain_id", None)
        project_id = (self.initial.get("project_id") or
                      request.GET.get('project'))

        if api.keystone.policy.check(
                (("identity", "identity:list_groups"),), request):
            kwargs = {'domain': domain_id}
        else:
            kwargs = {'user': request.user.id}

        # pylint: disable=too-many-nested-blocks
        try:
            all_groups = {
                group.id: group.name
                for group in api.keystone.group_list(request, **kwargs)
            }
        except Exception as err:
            logger.error(unicode(err))
        else:
            available_groups = api.keystone.get_project_groups_roles(
                request, project_id
            ).keys()

        # groups from project
        groups_list = [(user_id, all_groups.get(user_id))
                       for user_id in available_groups]

        return groups_list

    class Meta(object):
        name = _("Instance groups")
        slug = UPDATE_GROUPS_ACTION_SLUG


class CreateGroupsStep(InstanceUpdateMembersStep):
    action_class = CreateGroupsAction
    template_name = 'vdi/instances/_launch_groups_step.html'

    depends_on = ('domain_id',)

    available_list_title = _("All Groups")
    members_list_title = _("Instance groups")
    no_available_text = _("No groups found.")
    no_members_text = _("No groups.")


class UpdateGroupsAction(CreateGroupsAction):

    def _get_membership_data(self, request):
        project_groups = []
        domain_id = self.initial.get("domain_id", None)
        project_id = self.initial['instance'].tenant_id

        kwargs = {'project': project_id}
        if api.keystone.policy.check(
                (("identity", "identity:list_groups"),), request):
            kwargs['domain'] = domain_id
        else:
            kwargs['user'] = request.user.id

        try:
            project_groups = [
                (group.id, group.name)
                for group in api.keystone.group_list(request, **kwargs)
            ]
        except Exception as err:
            logger.error(unicode(err))

        return project_groups

    def _populate_fields(self, request, workflow, **kwargs):
        super(UpdateGroupsAction, self)._populate_fields(
            request, workflow, **kwargs
        )

        instance = self.initial["instance"]
        instance_groups = [
            group['id'] for group in getattr(instance, 'tnx_groups', [])
        ]

        # pylint: disable=no-member
        field_name = self.get_member_field_name(self.role.id)
        session_members = request.session.get('selected_members', None)
        if session_members is not None and field_name in session_members:
            initial_members = session_members.get(field_name, [])
        else:
            initial_members = instance_groups
        self.fields[field_name].initial = initial_members

    class Meta(object):
        name = _("Instance groups")
        slug = UPDATE_GROUPS_ACTION_SLUG


class UpdateGroupsStep(CreateGroupsStep):
    template_name = "horizon/common/_workflow_step_update_members.html"
    action_class = UpdateGroupsAction


class CloneGroupsStep(CloneStepMixin, UpdateGroupsStep):
    def get_initial_data(self):
        instance = self.workflow.context_seed['instance']
        return {self.get_member_field_name(self.action.role.id): [
            group['id'] for group in getattr(instance, 'tnx_groups', [])
        ]}


class UpdateVDIInstanceMembers(RefererRedirectMixin,
                               UpdateVDIInstanceMembersMixin,
                               VDIIdentityMixIn,
                               workflows.Workflow):
    success_message = _('Instance user list updated.')
    failure_message = _('Unable to update user list of instance.')

    success_url = "horizon:tionix:vdi:index"
    name = _("Assign to users")

    slug = 'vdi_instances_members'

    default_steps = (UpdateUsersStep,)

    # pylint: disable=no-member
    def __init__(self, request, *args, **kwargs):
        context_seed = kwargs.get('context_seed', {})
        project_id = context_seed['instance'].tenant_id
        self.is_group_editing_enabled = (
            PROJECT_GROUP_ENABLED and self.is_project_admin(request,
                                                            project_id)
        )
        if self.is_group_editing_enabled:
            self.default_steps += (UpdateGroupsStep,)
        elif UpdateGroupsStep in self._cls_registry:
            self.unregister(UpdateGroupsStep)

        super(UpdateVDIInstanceMembers, self).__init__(
            request, *args, **kwargs
        )

    def handle(self, request, context):
        self._update_members(self.context['instance'], context)
        return True


class VDIInstanceLaunchDetailsAction(workflows.Action, VDIIdentityMixIn):
    """Instance launch details action"""

    project = forms.ChoiceField(
        label=_('VDI Project'),
        widget=forms.SelectWidget(
            attrs={'ng-model': 'currentProject'},
            data_attrs=('user_is_admin',),
            transform=lambda x: x.name
        )
    )
    instances_count = forms.IntegerField(
        label=_('Instance count'),
        min_value=1,
        initial=1,
        widget=forms.NumberInput(attrs={
            'ng-model': 'currentInstanceCount',
            'validate-number-min': '1',
        }),
    )
    name = AutoGenNameCharField(
        label=_('Instance name'),
        max_length=255,
        required=False
    )

    def __init__(self, request, context, *args, **kwargs):
        super(VDIInstanceLaunchDetailsAction, self).__init__(request,
                                                             context,
                                                             *args,
                                                             **kwargs)
        try:
            projects = self._populate_project_choices(request)
        except ValueError as err:
            messages.info(request, unicode(err))
            raise
        else:
            self.fields['project'].choices = projects

        self._populate_quotas(request)

    def _populate_quotas(self, request):
        self.quotas = []
        tenant_ids = [tenant_id for tenant_id, _tenant_name in
                      self.fields['project'].choices if tenant_id]

        for tenant_id in tenant_ids:
            try:
                tenant = {'id': tenant_id}
                quotas = get_quota_by_tenant(request, tenant_id=tenant_id)
                instances_quota = quotas.get(quota_map['instances']['total'])
                instances_usage = quotas.get(quota_map['instances']['used'])
                tenant.update({'instances_quota': instances_quota,
                               'instances_usage': instances_usage,
                               'instances_available': (
                                   instances_quota - instances_usage)})

                self.quotas.append(tenant)
            except Exception as err:
                logger.error(err)

    def _populate_project_choices(self, request):
        projects = get_projects_for_instance_launch(request)
        projects = [
            (project_id,
             Empty(name=project_name,
                   user_is_admin=self.is_project_admin(request,
                                                       project_id)))
            for project_id, project_name in projects
        ]

        if projects:
            projects = [('', Empty(name=_('Select project'),
                                   user_is_admin=True))] + projects
        else:
            raise ValueError(_('No VDI projects available.'))

        return projects

    def clean_instances_count(self):
        instances_count = self.cleaned_data['instances_count']
        tenant_id = self.cleaned_data.get('project')
        if tenant_id is None:
            return instances_count

        nova_cl = novaclient(self.request, project_id=tenant_id)
        try:
            tenant = nova_cl.tnx_tenants.get(tenant_id)
        except Exception as err:
            logger.error(err)
            raise ValidationError(_('Unable to get VDI project.'))

        flavor_id = tenant.tnx_flavor_id

        quotas_usages_validation(self.request, flavor_id=flavor_id,
                                 tenant_id=tenant_id,
                                 instances_count=instances_count)

        return instances_count

    class Meta(object):
        name = _("Details")


class VDIInstanceCloneDetailsAction(VDIInstanceLaunchDetailsAction):

    project = forms.ChoiceField(
        label=_('VDI Project'),
        widget=forms.SelectWidget(
            attrs={'ng-model': 'currentProject',
                   'disabled': 'disabled'},
            data_attrs=('user_is_admin',),
            transform=lambda x: x.name
        )
    )

    class Meta(object):
        name = _("Details")


class VDIInstanceLaunchDetailsStep(workflows.Step):
    """Instance launch details step"""

    action_class = VDIInstanceLaunchDetailsAction
    contributes = ('project', 'instances_count', 'name')
    template_name = 'vdi/instances/_launch_details_step.html'


class VDIInstanceCloneDetailsStep(VDIInstanceLaunchDetailsStep):
    """Instance clone details step"""

    action_class = VDIInstanceCloneDetailsAction


class CreateVolumesAction(MultiManagingFormMixin,
                          workflows.Action):

    new_allocated = forms.CharField(widget=forms.HiddenInput, required=False)
    old_allocated = forms.CharField(widget=forms.HiddenInput, required=False)
    device = forms.CharField(
        label=_("Device Name"),
        widget=forms.TextInput(attrs={'placeholder': '/dev/vdc'}),
        required=False,
        help_text=_("Actual device name may differ due to hypervisor "
                    "settings. If not specified, then hypervisor will select "
                    "a device name."))

    def __init__(self, *args, **kwargs):
        super(CreateVolumesAction, self).__init__(*args, **kwargs)

        # Hide the device field if the hypervisor doesn't support it.
        if not api.nova.can_set_mount_point():
            self.fields['device'].widget = forms.widgets.HiddenInput()

    class Meta(object):
        name = _("Volumes")
        slug = 'update_volumes'


class CreateVolumesStep(workflows.Step):
    """Instance launch volumes step"""

    action_class = CreateVolumesAction
    contributes = ('new_allocated', 'old_allocated', 'device')
    template_name = 'vdi/instances/_launch_volumes_step.html'
    controller_name = 'LaunchVDIInstanceVolumesController as ctrl'


class VDIInstanceLaunchWorkflow(UpdateVDIInstanceMembersMixin,
                                workflows.Workflow,
                                VDIIdentityMixIn):
    """Instance launch workflow"""
    success_message = _('Successfully launched instance(s).')
    failure_message = _('Unable to launch instance(s).')
    success_url = "horizon:tionix:vdi:index"
    name = _("Launch VDI Instance")
    slug = 'vdi_instances_launch'

    if PROJECT_GROUP_ENABLED:
        default_steps = (VDIInstanceLaunchDetailsStep,
                         CreateUsersStep,
                         CreateGroupsStep,
                         CreateVolumesStep)
    else:
        default_steps = (VDIInstanceLaunchDetailsStep,
                         CreateUsersStep,
                         CreateVolumesStep)

    def handle(self, request, context):
        result = True

        project_id = context['project']
        instances_count = int(context['instances_count'])

        nova_client = novaclient(request, project_id=project_id)

        instance_name = context['name']
        volumes_to_attach = context.pop('new_allocated', None)

        try:
            server = nova_client.servers.create(
                name=instance_name,
                image=None,
                flavor=None,
                min_count=instances_count,
                max_count=instances_count,
                tnx_type=int(VMTypes.VDI),
                tnx_tenant_id=project_id
            )
        except Exception as err:
            result = False
            logger.error(unicode(err))
            exceptions.handle(request, err.message, redirect=VDI_INDEX)
        else:
            reservation_id = getattr(server,
                                     'OS-EXT-SRV-ATTR:reservation_id',
                                     None)
            try:
                if reservation_id:
                    instances = novaclient(request).tnx_servers.list(
                        search_opts={'all_tenants': True,
                                     'reservation_id': reservation_id}
                    )
                else:
                    instances = [
                        novaclient(request).tnx_servers.get(server.id)
                    ]
            except Exception as err:
                logger.error(unicode(err))
                instances = []

            for instance in instances:
                self._update_members(instance, context)

            # Volume attach is available only for single server launch
            if instances_count == 1 and volumes_to_attach:
                volume_ids = {obj['id'] for obj in volumes_to_attach}
                for volume_id in volume_ids:
                    # Starting attach in a new thread because vm's status
                    # must be "ACTIVE"
                    thread.start_new_thread(
                        delayed_volume_attach,
                        (request, server.id, volume_id)
                    )
        return result


class CloneVDIInstance(RefererRedirectMixin, VDIInstanceLaunchWorkflow):
    """Clone vdi instance workflow"""
    slug = "clone_vdi_instance"
    name = _("Clone VDI Instance")
    finalize_button_name = _("Clone")
    success_message = _('Successfully cloned instance.')
    failure_message = _('Unable to clone instance.')
    success_url = "horizon:tionix:vdi:index"

    default_steps = (VDIInstanceCloneDetailsStep, CloneUsersStep)

    # pylint: disable=no-member
    def __init__(self, request, *args, **kwargs):
        context_seed = kwargs.get('context_seed', {})

        self.is_group_editing_enabled = (
            PROJECT_GROUP_ENABLED and
            self.is_project_admin(request, context_seed.get('project'))
        )

        if self.is_group_editing_enabled:
            self.default_steps += (CloneGroupsStep,)
        elif CloneGroupsStep in self._cls_registry:
            self.unregister(CloneGroupsStep)
        self.default_steps += (CreateVolumesStep,)

        super(CloneVDIInstance, self).__init__(request, *args, **kwargs)

        for step in self.steps:
            for field_name, field in step.action.fields.items():
                value = context_seed.get(field_name)
                if value:
                    field.initial = value


class MultiCreateUsersStep(CreateUsersStep):
    depends_on = ('domain_id', 'project_id', 'instances')
    template_name = "horizon/common/_workflow_step_update_members.html"


class MultiCreateGroupsStep(CreateGroupsStep):
    depends_on = ('domain_id', 'project_id', 'instances')
    template_name = "horizon/common/_workflow_step_update_members.html"


class MultiUpdateMembersWithGroups(RefererRedirectMixin,
                                   UpdateVDIInstanceMembersMixin,
                                   VDIIdentityMixIn,
                                   workflows.Workflow):
    success_message = _('Instances user and group lists updated.')
    failure_message = _('Unable to update user and group lists of instances.')

    success_url = "horizon:tionix:vdi:index"
    name = _("Assign to users")

    slug = 'multi_members_groups'

    default_steps = (MultiCreateUsersStep,)

    # pylint: disable=no-member
    def __init__(self, request, *args, **kwargs):
        context_seed = kwargs.get('context_seed', {})
        project_id = context_seed['project_id']

        self.is_group_editing_enabled = (
            PROJECT_GROUP_ENABLED and self.is_project_admin(request,
                                                            project_id)
        )

        if self.is_group_editing_enabled:
            self.default_steps += (MultiCreateGroupsStep,)
        elif MultiCreateGroupsStep in self._cls_registry:
            self.unregister(MultiCreateGroupsStep)

        super(MultiUpdateMembersWithGroups, self).__init__(
            request, *args, **kwargs
        )

    # pylint: disable=arguments-differ
    def get_absolute_url(self):
        return self.request.get_full_path()

    def handle(self, request, context):
        for instance in self.context['instances']:
            self._update_members(instance, context, delete_old_records=False)
        return True


# noinspection PyClassHasNoInit
class VDIUpdateInstanceInfoAction(project_workflows.UpdateInstanceInfoAction):

    def __init__(self, request, context, *args, **kwargs):
        super(VDIUpdateInstanceInfoAction, self).__init__(
            request, context, *args, **kwargs
        )
        instances_count = len(context.get('instances', []))
        if instances_count > 1:
            self.fields['name'].widget.attrs.update({'disabled': 'disabled'})

    class Meta(object):
        name = _("Information")
        slug = 'instance_info'


# noinspection PyClassHasNoInit
class VDIUpdateInstanceInfo(project_workflows.UpdateInstanceInfo):
    action_class = VDIUpdateInstanceInfoAction


# noinspection PyClassHasNoInit
class VDIUpdateInstance(admin_workflows.AdminUpdateInstance):
    success_url = VDI_INSTANCES_INDEX
    default_steps = (VDIUpdateInstanceInfo,)
