# coding: utf-8
# pylint: disable=arguments-differ,duplicate-code
import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, npgettext_lazy
from horizon import tables, exceptions
from openstack_dashboard import api, policy

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.helpers import get_connection_type_by_name
from tionix_dashboard.dashboards.identity.projects import (
    actions as identity_actions)
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_INDEX
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxUpdateAction, LinkBatchAction)
from tionix_dashboard.helpers.utils import (
    get_yes_no_filter_value, get_filter_value_by_dict)
from tionix_dashboard.url_resolve_map import (
    VDI_PROJECTS_CREATE, VDI_PROJECTS_CLONE, VDI_PROJECTS_MULTI_UPDATE,
    VDI_PROJECTS_UPDATE)


logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class CreateVDIProjectAction(tables.LinkAction):
    """
    Action to create vdi project
    """
    name = 'create'
    verbose_name = _('Create VDI Project')
    classes = ('ajax-modal',)
    url = VDI_PROJECTS_CREATE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


# noinspection PyAbstractClass
class EditVDIProjectMultiAction(LinkBatchAction):
    """
    Action to edit project
    """
    name = 'multi_vdi_edit_project'
    multiple_next_view_name = VDI_PROJECTS_MULTI_UPDATE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    def action_present(self, count):
        return npgettext_lazy(
            u"Multi edit",
            u"Edit project",
            u"Edit projects",
            count
        )

    def action_past(self, count):
        return npgettext_lazy(
            u"Multi edit",
            u"Edited project",
            u"Edited projects",
            count
        )


# noinspection PyAbstractClass
class EditVDIProjectAction(tables.LinkAction):
    """
    Action to edit project
    """
    name = 'vdi_edit_project'
    verbose_name = _('Edit Project')
    classes = ('ajax-modal',)
    url = VDI_PROJECTS_UPDATE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


# noinspection PyAbstractClass
class CloneVDIProjectAction(tables.LinkAction):
    """
    Action to clone project
    """
    name = 'vdi_clone_project'
    verbose_name = _('Clone Project')
    classes = ('ajax-modal',)
    url = VDI_PROJECTS_CLONE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


# noinspection PyAbstractClass
class UsageLink(tables.LinkAction):
    name = "usage"
    verbose_name = _("View Usage")
    url = "horizon:identity:projects:usage"
    icon = "stats"
    policy_rules = (("compute", "compute_extension:simple_tenant_usage:show"),)

    def allowed(self, request, project):
        return request.user.is_superuser


# noinspection PyAbstractClass
class VDIProjectFilterAction(TnxFilterAction):
    """
    Filter VDI projects filter
    """
    name = "filter_vdi_projects"
    filter_type = "server"

    filter_choices = (
        ('name', _("Name"), False),
        ('description', _("Description"), False),
        ('domain_name', _("Domain Name"), False),
        ('enabled', _("Enabled"), False),
        ('tnx_operation_mode', _("Operation Mode"), True),
    )

    filter_string_modifiers = {
        'enabled': get_yes_no_filter_value(),
        'tnx_operation_mode': (
            get_filter_value_by_dict(OperationModeEnum.values,
                                     not_found_none=False))
    }


class UpdateCell(TnxUpdateAction):
    """Inline project update"""

    def allowed(self, request, project, cell):
        policy_rule = (("identity", "identity:update_project"),)
        policy_target = {'target.project.domain_id': project.domain_id}
        return (
            (cell.column.name != 'enabled' or
             request.user.project_id != cell.datum.id) and
            api.keystone.keystone_can_edit_project() and
            policy.check(policy_rule, request, target=policy_target))

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        result = True
        # Server couldn't pass boolean false type
        if isinstance(new_cell_value, bool):
            new_cell_value = unicode(new_cell_value)
        try:
            TNXTenantClient.update(request,
                                   obj_id,
                                   **{cell_name: new_cell_value})
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, ignore=True)
            result = False
        return result


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        try:
            tenant = TNXTenantClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            tenant = None
            exceptions.handle(request,
                              _('Unable to get tenant.'),
                              redirect=reverse(VDI_INDEX))

        return tenant


# noinspection PyAbstractClass
class ProjectSessionFilterAction(TnxFilterAction):
    """Session filter"""
    filter_type = "server"

    filter_choices = [
        ('instance_name', _('Instance name'), True),
        ('user_name', _("User Name"), True),
        ('created_at', _('Created'), True),
        ('closed_at', _('Closed'), True),
        ('connection_type', _("Connection"), True),
        ('client_info', _("Client"), True),
        ('client_os', _("OS"), True),
        ('client_ip', _("IP"), True),
    ]

    filter_string_modifiers = {
        'connection_type': get_connection_type_by_name
    }


# noinspection PyAbstractClass,PyClassHasNoInit
class TnxVDIDeleteProject(identity_actions.TnxDeleteTenant):
    multiple_next_view_name = "horizon:tionix:vdi:projects:delete"
