# coding: utf-8
from django.core.urlresolvers import reverse
from django.template import defaultfilters as filters
from django.utils.translation import ugettext_lazy as _

from horizon import forms, tables

from tionix_client.journal.constants import ObjectType as JournalObjectType

from tionix_dashboard import journal
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    VDIProjectSchedulerAction)
from tionix_dashboard.dashboards.tionix.vdi.constants import (
    VDI_PROJECT_DETAILS)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_project_detail_link)
from tionix_dashboard.dashboards.tionix.vdi.projects import actions
from tionix_dashboard.dashboards.tionix.vdi.projects.actions import (
    ProjectSessionFilterAction)
from tionix_dashboard.dashboards.tionix.vdi.tables import BaseSessionTable
from tionix_dashboard.dashboards.utils import is_domain_supported
from tionix_dashboard.helpers.tables import (
    TnxLocalPaginationTabDataTable)
from tionix_dashboard.url_resolve_map import VDI_INSTANCE_DETAILS


class VDIProjectsTable(tables.DataTable):
    name = tables.Column(
        'name',
        verbose_name=_('Name'),
        link=VDI_PROJECT_DETAILS,
        form_field=forms.CharField(max_length=64),
        update_action=actions.UpdateCell
    )
    description = tables.Column(
        lambda obj: getattr(obj, 'description', None),
        verbose_name=_('Description'),
        form_field=forms.CharField(
            widget=forms.Textarea(attrs={'rows': 4}),
            required=False),
        update_action=actions.UpdateCell
    )

    if is_domain_supported():
        domain_name = tables.Column(
            'domain_name', verbose_name=_('Domain Name'))
    enabled = tables.Column(
        'enabled',
        verbose_name=_('Enabled'),
        filters=(filters.yesno, filters.capfirst),
        form_field=forms.BooleanField(
            label=_('Enabled'),
            required=False),
        update_action=actions.UpdateCell
    )
    tnx_operation_mode = tables.Column(
        'tnx_operation_mode',
        verbose_name=_('Operation Mode'),
        display_choices=OperationModeEnum.get_choices()
    )

    class Meta(object):
        name = "vdi_projects"
        verbose_name = _("Projects")
        row_class = actions.UpdateRow
        table_actions = (
            actions.VDIProjectFilterAction,
            actions.CreateVDIProjectAction,
            actions.TnxVDIDeleteProject
        )

        table_actions_menu = (
            VDIProjectSchedulerAction,
            actions.EditVDIProjectMultiAction
        )

        row_actions = (
            actions.CloneVDIProjectAction,
            actions.TnxVDIDeleteProject,
            actions.EditVDIProjectAction,
            VDIProjectSchedulerAction
        )

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(VDIProjectsTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)
        self.columns['name'].get_link_url = self.get_project_detail_link

    def get_project_detail_link(self, project):
        return get_project_detail_link(self.request, project)


def get_url_for_instance(datum):
    link = None
    if (not getattr(datum, 'object_deleted', False) and
            getattr(datum, 'object_type', None) == JournalObjectType.Server):
        project_id = getattr(datum, 'project_id', None)
        obj_id = getattr(datum,
                         'object_id',
                         getattr(datum, 'server_id', None))
        link = reverse(VDI_INSTANCE_DETAILS, args=(project_id, obj_id,))
    return link


class TnxVDIInstancesAuditLog(TnxLocalPaginationTabDataTable,
                              journal.JournalBaseTable):

    # noinspection PyUnusedLocal
    @staticmethod
    def get_url_for_object(obj, is_admin=False):
        return get_url_for_instance(obj)

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instances Action List')
        multi_select = False
        table_actions = (journal.JournalFilterAction,)


class ProjectsSessionTable(BaseSessionTable):

    instance_name = tables.Column(
        "instance_name",
        link=get_url_for_instance,
        verbose_name=_("Instance name"),
    )

    class Meta(object):
        name = "project_sessions"
        verbose_name = _("Project Sessions")
        multi_select = False
        table_actions = (ProjectSessionFilterAction,)
        columns = ('instance_name', 'user_name', 'created_at', 'closed_at',
                   'connection_type', 'client_info', 'client_os', 'client_ip')
