# coding: utf-8

"""Vdi projects tabs"""

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs

from openstack_dashboard import api
from openstack_dashboard.api import keystone, neutron, nova

from tionix_scheduler_client.enums import ObjectGroupEnum, ObjectTypeEnum

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.helpers import get_sessions_list
from tionix_dashboard.dashboards.identity.projects import tabs as projects_tabs
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.projects import tables
from tionix_dashboard.dashboards.tionix.vdi.projects.helpers import (
    VDIProjectUsage, get_flavor_details)
from tionix_dashboard.helpers.views import (
    DTVTnxPaginationMixin, ServerFilterTabMixin)
from tionix_dashboard.helpers.vm_constants import DELETED
from tionix_dashboard.url_resolve_map import (
    ADMIN_IMAGE_DETAIL, ADMIN_NETWORK_DETAIL)

logger = logging.getLogger(__name__)


# noinspection PyClassHasNoInit
class OverviewTab(projects_tabs.OverviewTab):
    """Project overview tab"""
    template_name = 'vdi/projects/_detail_overview.html'

    def get_context_data(self, request, **kwargs):
        """project context"""

        context = super(OverviewTab, self).get_context_data(request, **kwargs)
        project = context['project']

        project.tnx_operation_mode_name = OperationModeEnum.values.get(
            project.tnx_operation_mode
        )
        try:
            project.tnx_image_name = api.glance.image_get(
                request, project.tnx_image_id).name
        except Exception as err:
            logger.error(unicode(err))
            project.tnx_image_name = project.tnx_image_id
        else:
            if keystone.is_domain_admin(request):
                context['image_url'] = reverse(ADMIN_IMAGE_DETAIL,
                                               args=(project.tnx_image_id,))
        try:
            flavor = nova.flavor_get(self.request, project.tnx_flavor_id)
        except Exception as err:
            logger.error(unicode(err))
            project.tnx_flavor = project.tnx_flavor_id
        else:
            project.tnx_flavor = get_flavor_details(flavor)

        try:
            project.tnx_network_name = neutron.network_get(
                self.request, project.tnx_network_id).name
        except Exception as err:
            logger.error(unicode(err))
            project.tnx_network_name = project.tnx_network_id
        else:
            if keystone.is_domain_admin(request):
                context['network_url'] = reverse(
                    ADMIN_NETWORK_DETAIL, args=(project.tnx_network_id,))

        return context


# noinspection PyAbstractClass, PyClassHasNoInit
class VDIUsageTab(projects_tabs.UsageTab):
    """Project usage of vdi instances tab"""
    usage_class = VDIProjectUsage


# noinspection PyAbstractClass, PyClassHasNoInit
class InstancesAuditTab(projects_tabs.InstancesAuditTab):
    """Tionix audit of instances tab"""

    table_classes = (tables.TnxVDIInstancesAuditLog,)


# noinspection PyAbstractClass
class SessionTab(ServerFilterTabMixin, DTVTnxPaginationMixin, tabs.TableTab):
    """VDI Project session tab"""
    name = _("Sessions")
    slug = "session"
    table_classes = (tables.ProjectsSessionTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def get_project_sessions_data(self):
        offset, limit = self.get_offset_limit()
        instances, user_ids, sessions, total_count = [], [], [], 0
        filters = self.get_filters(with_marker=False)

        project_id = self.tab_group.kwargs['tenant_id']
        domain_id = self.tab_group.kwargs['project'].domain_id
        search_opts = {'project_id': project_id}
        is_domain = api.keystone.is_domain_admin(self.request)

        instance_name = filters.pop('instance_name', None)
        if instance_name:
            search_opts['name'] = instance_name
        if is_domain:
            search_opts.update({'all_tenants': True})
        try:
            instances = TNXServersClient.list(self.request,
                                              search_opts=search_opts)
            if is_domain:
                search_opts.update({'deleted': True})
                instances += TNXServersClient.list(self.request,
                                                   search_opts=search_opts)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to get project instances list.'))
        else:
            instances = {
                instance.id: (instance.name,
                              instance.status.lower() == DELETED)
                for instance in instances}
        username = filters.pop('user_name', None)
        if username:
            try:
                user_ids = [
                    user.id
                    for user in keystone.user_list(
                        self.request,
                        domain=domain_id,
                        filters={'name': username})
                ] or [username]
            except Exception as err:
                logger.error(unicode(err))
        if instances:
            session_filters = {'project_id': project_id,
                               'user_id': user_ids}
            session_filters.update(**filters)
            sessions, total_count = get_sessions_list(
                self.request,
                domain_id=self.tab_group.kwargs['project'].domain_id,
                limit=limit,
                offset=offset,
                filters=session_filters
            )
            for session in sessions:
                session.instance_name, session.object_deleted = (
                    instances.get(session.server_id,
                                  (session.server_id, True))
                )
                session.project_id = project_id
        self.set_table_footer(offset, limit, total_count)
        return sessions


# noinspection PyAbstractClass
class ProjectDetailTabs(tabs.TabGroup):
    """Tionix project details tab group"""
    slug = "tnx_project_details"
    tabs = (OverviewTab,
            VDIUsageTab,
            InstancesAuditTab,
            ActiveTasksTab,
            SessionTab,
            projects_tabs.GroupsTab,
            projects_tabs.UsersTab,)

    sticky = True
    obj_type = ObjectTypeEnum.project
    obj_group = ObjectGroupEnum.vdi
