# coding: utf-8
# pylint: disable=no-member
import mock

from django.conf import settings
from django.core.urlresolvers import reverse
from django.test import override_settings

from openstack_dashboard import api

from openstack_auth import policy as policy_backend

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.dashboards.tionix.vdi.projects import tables, workflows
from tionix_dashboard.dashboards.tionix.vdi.projects.views import (
    TnxDeleteTenantView)
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, BaseAdminClientTestCase, FakeUser)


INDEX_URL = "horizon:tionix:vdi:projects_tab"
DETAIL_URL = "horizon:tionix:vdi:projects:detail"
DELETE_URL = "horizon:identity:projects:delete"


@mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                   mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class VDIProjectsTablesTest(BaseClientTestCase):
    def test_vdi_projects_columns(self):
        table = tables.VDIProjectsTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'name', 'description', 'enabled',
                    'tnx_operation_mode', 'actions', 'domain_name'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_vdi_projects_actions(self):
        projects = self.tnx_servers.list()

        table = tables.VDIProjectsTable(self.request, data=projects)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter_vdi_projects'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    @override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
    def test_vdi_projects_row_actions(self):
        table = tables.VDIProjectsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted([])
        )


@mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                   mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class VDIProjectsAdminTablesTest(BaseAdminClientTestCase):

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_vdi_projects_actions(self):
        projects = self.tnx_servers.list()

        table = tables.VDIProjectsTable(self.request, data=projects)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter_vdi_projects', 'create', 'multi_vdi_edit_project',
                    'delete', 'vdi_project_schedule'])
        )

    def test_vdi_projects_row_actions(self):
        table = tables.VDIProjectsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['vdi_clone_project', 'delete',
                    'vdi_edit_project', 'vdi_project_schedule'])
        )


class VDIProjectsViewsTest(BaseClientTestCase):

    @mock.patch.object(TnxDeleteTenantView,
                       'get_tasks_rows',
                       mock.Mock(return_value={}))
    @mock.patch.object(api.keystone,
                       'tenant_delete',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXTenantClient, 'get')
    def test_delete_from_index(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]

        tenant_get.return_value = tenant

        url = reverse(DELETE_URL)

        req = self.factory.post(url, data={'id': tenant.id})

        index_url = reverse(INDEX_URL)
        req.META['HTTP_REFERER'] = ''.join([settings.TESTSERVER,
                                            settings.WEBROOT,
                                            index_url])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = TnxDeleteTenantView.as_view()(req)

        self.assertIn(index_url, response.url)

    @mock.patch.object(TnxDeleteTenantView,
                       'get_tasks_rows',
                       mock.Mock(return_value={}))
    @mock.patch.object(api.keystone,
                       'tenant_delete',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXTenantClient, 'get')
    def test_delete_from_detail(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]

        tenant_get.return_value = tenant

        url = reverse(DELETE_URL)

        req = self.factory.post(url, data={'id': tenant.id})

        index_url = reverse(INDEX_URL)
        req.META['HTTP_REFERER'] = ''.join([
            settings.TESTSERVER,
            settings.WEBROOT,
            reverse(DETAIL_URL, args=(tenant.id,))])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = TnxDeleteTenantView.as_view()(req)

        self.assertIn(index_url, response.url)


class VDIProjectsWorkflowTest(BaseClientTestCase):

    @mock.patch.object(api.nova,
                       'availability_zone_list',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'get_project_users_roles',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'get_project_groups_roles',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone, 'get_default_role')
    @mock.patch.object(api.keystone, 'role_list')
    def test_update_workflow_az_invalid(self, role_list, get_default_role):
        roles = self.roles.list()

        role_list.return_value = roles
        get_default_role.return_value = roles[0]

        context_seed = {'project_id': 'test_project',
                        'availability_zone': 'test_zone'}

        request = get_fake_request(
            is_ajax=True,
            method='GET',
            user=FakeUser(project_id=None, token=None,
                          authorized_tenants=[], service_catalog=None),
            horizon={
                'async_messages': []
            }
        )

        workflow = workflows.UpdateVDIProject(request,
                                              context_seed=context_seed)

        self.assertFalse(workflow.is_valid())
        self.assertIn('availability_zone', workflow.steps[0].action.errors)


class VDIProjectSessionsTableTest(BaseClientTestCase):
    def test_vdi_project_session_columns(self):
        """Test if all columns are still in place"""

        table = tables.ProjectsSessionTable(self.request)

        self.assertEqual(
            sorted(table.columns.keys()),
            sorted(['instance_name', 'user_name', 'created_at', 'client_info',
                    'closed_at', 'client_os', 'connection_type', 'client_ip']))

    def test_vdi_project_session_action(self):
        """Test whether all required filters are available"""
        projects = self.tnx_servers.list()

        table = tables.ProjectsSessionTable(self.request, data=projects)

        filters_choices = []
        for action in table.get_table_actions():
            if hasattr(action, "filter_choices"):
                filters_choices.append(action.filter_choices)

        filter_names = []
        for filter_choices in filters_choices:
            name, _, _ = zip(*filter_choices)
            filter_names.extend(name)

        self.assertListEqual(
            sorted(filter_names),
            sorted(['instance_name', 'user_name', 'client_info', 'created_at',
                    'closed_at', 'client_os', 'client_ip', 'connection_type'])
        )
