# coding: utf-8
import json
import logging

from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import exceptions, workflows
from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard.api import neutron
from openstack_dashboard.api.rest import (
    urls as rest_urls, utils as rest_utils)
from openstack_dashboard import policy
from openstack_dashboard.dashboards.identity.projects.views import (
    PROJECT_INFO_FIELDS)
from openstack_dashboard.dashboards.project.instances.utils import flavor_list
from openstack_dashboard.usage import quotas

from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.identity.projects import (
    views as projects_views)
from tionix_dashboard.dashboards.tionix.vdi import constants
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_PROJECT_INDEX
from tionix_dashboard.dashboards.tionix.vdi.projects import (
    workflows as project_workflows)
from tionix_dashboard.dashboards.tionix.vdi.projects.helpers import (
    get_available_images)
from tionix_dashboard.dashboards.tionix.vdi.projects.tables import (
    VDIProjectsTable
)
from tionix_dashboard.dashboards.tionix.vdi.projects.tabs import (
    ProjectDetailTabs)
from tionix_dashboard.dashboards.tionix.vdi.projects.workflows import (
    CreateDefaultNetworkWorkflow)
from tionix_dashboard.dashboards.utils import get_tenants_kwargs
from tionix_dashboard.helpers.views import (
    LinkBatchWorkflowViewMixin, WorkflowWithLazyMembers)
from tionix_dashboard.helpers.workflows import WorkflowVerticalView

logger = logging.getLogger(__name__)


class CreateVDIProjectView(WorkflowWithLazyMembers, WorkflowVerticalView):
    """Create VDI project view"""
    workflow_class = project_workflows.CreateVDIProject

    def get(self, request, *args, **kwargs):
        request.session.pop('vdi_default_network', None)
        return super(CreateVDIProjectView, self).get(request, *args, **kwargs)

    def get_initial(self):
        initial = super(CreateVDIProjectView, self).get_initial()

        # Set the domain of the project
        domain = api.keystone.get_default_domain(self.request)
        initial["domain_id"] = domain.id
        initial["domain_name"] = domain.name

        # get initial quota defaults
        if is_project_quotas_disabled(self.request):
            self.workflow_class = project_workflows.CreateVDIProjectNoQuota
        elif api.keystone.is_domain_admin(self.request):
            try:
                quota_defaults = quotas.get_default_quota_data(self.request)

                try:
                    if (api.base.is_service_enabled(
                            self.request, 'network') and
                            api.neutron.is_quotas_extension_supported(
                                self.request)):
                        project_id = self.request.user.project_id
                        quota_defaults += api.neutron.tenant_quota_get(
                            self.request, tenant_id=project_id)
                except Exception as err:
                    logger.error(unicode(err))
                    error_msg = _('Unable to retrieve default Neutron quota '
                                  'values.')
                    self.add_error_to_step(error_msg, 'create_quotas')

                for field in quotas.QUOTA_FIELDS:
                    initial[field] = quota_defaults.get(field).limit

            except Exception as err:
                logger.error(unicode(err))
                error_msg = _('Unable to retrieve default quota values.')
                self.add_error_to_step(error_msg, 'create_quotas')

        return initial


def is_project_quotas_disabled(request):
    """Checks if quotas enabled"""
    return (api.keystone.is_multi_domain_enabled() and
            not api.keystone.is_domain_admin(request))


class UpdateVDIProjectView(WorkflowWithLazyMembers, WorkflowVerticalView):
    """Update VDI project view"""
    workflow_class = project_workflows.UpdateVDIProject

    def get(self, request, *args, **kwargs):
        request.session.pop('vdi_default_network', None)
        return super(UpdateVDIProjectView, self).get(request, *args, **kwargs)

    def get_initial(self):
        initial = super(UpdateVDIProjectView, self).get_initial()

        project_id = self.kwargs['tenant_id']
        initial['project_id'] = project_id

        try:
            # get initial project info
            project_info = api.keystone.tenant_get(self.request, project_id)
            for field in PROJECT_INFO_FIELDS:
                initial[field] = getattr(project_info, field, None)

            # Retrieve the domain name where the project belong
            if api.keystone.VERSIONS.active >= 3:
                try:
                    if policy.check((("identity", "identity:get_domain"),),
                                    self.request):
                        domain = api.keystone.domain_get(self.request,
                                                         initial["domain_id"])
                        initial["domain_name"] = domain.name

                    else:
                        domain = api.keystone.get_default_domain(self.request)
                        initial["domain_name"] = domain.name

                except Exception as err:
                    logger.error(unicode(err))
                    exceptions.handle(self.request,
                                      _('Unable to retrieve project domain.'),
                                      redirect=reverse(VDI_PROJECT_INDEX))

            if is_project_quotas_disabled(self.request):
                self.workflow_class = project_workflows.UpdateVDIProjectNoQuota
            elif api.keystone.is_domain_admin(self.request):
                # get initial project quota
                quota_data = quotas.get_tenant_quota_data(self.request,
                                                          tenant_id=project_id)
                if (api.base.is_service_enabled(self.request, 'network') and
                        api.neutron.is_quotas_extension_supported(
                            self.request)):
                    quota_data += api.neutron.tenant_quota_get(
                        self.request, tenant_id=project_id)
                for field in quotas.QUOTA_FIELDS:
                    initial[field] = quota_data.get(field).limit
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve project details.'),
                              redirect=reverse(VDI_PROJECT_INDEX))

        try:
            tenant = TNXTenantClient.get(self.request, initial['project_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve VDI project information.'))
        else:
            initial.update(
                default_flavor=tenant.tnx_flavor_id,
                default_image=tenant.tnx_image_id,
                tnx_operation_mode=tenant.tnx_operation_mode,
                default_network=tenant.tnx_network_id,
                availability_zone=tenant.tnx_availability_zone
            )

        return initial


class MultiUpdateVDIProjectView(LinkBatchWorkflowViewMixin,
                                WorkflowWithLazyMembers,
                                WorkflowVerticalView):
    workflow_class = project_workflows.MultiUpdateVDIProject
    submit_url = 'horizon:tionix:vdi:projects:multi_update_projects'

    def get(self, request, *args, **kwargs):
        request.session.pop('vdi_default_network', None)
        return super(MultiUpdateVDIProjectView, self).get(request,
                                                          *args,
                                                          **kwargs)

    def get_initial(self):

        initial = super(MultiUpdateVDIProjectView, self).get_initial()

        if is_project_quotas_disabled(self.request):
            self.workflow_class = (
                project_workflows.MultiUpdateVDIProjectNoQuota
            )

        request_tenants = self.get_object_ids()

        # Gather our tenants to correlate against IDs
        kwargs = get_tenants_kwargs(self.request)
        try:
            tenants, _has_more = api.keystone.tenant_list(
                self.request, **kwargs)
        except Exception as err:
            tenants = []
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve projects list.'),
                              redirect=reverse(VDI_PROJECT_INDEX))
        else:
            tenants = {tenant.id: tenant.name for tenant in tenants
                       if tenant.id in request_tenants}
        initial['projects'] = tenants
        initial['domain_id'] = api.keystone.get_default_domain(
            self.request, get_name=False).id

        return initial


class CloneVDIProjectView(UpdateVDIProjectView):
    """Clone VDI project view"""
    workflow_class = project_workflows.CloneVDIProject

    def get_initial(self):
        initial = super(CloneVDIProjectView, self).get_initial()

        if 'name' in initial:
            initial['name'] = u'Clone of {0}'.format(initial['name'])

        return initial


class CreateVDIProjectNetworkView(workflows.WorkflowView):
    """Create VDI project network view"""
    workflow_class = CreateDefaultNetworkWorkflow
    ajax_template_name = 'vdi/_create_network.html'


# noinspection PyClassHasNoInit
class DetailVDIProjectView(projects_views.TnxDetailProjectView):
    """VDI project's details view"""
    tab_group_class = ProjectDetailTabs
    redirect_url = VDI_PROJECT_INDEX
    page_title = _("VDI Project Details: {{ project.name }}")

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(DetailVDIProjectView, self).get_context_data(**kwargs)
        project = self.get_data()
        table = VDIProjectsTable(self.request)
        context["project"] = project
        context["url"] = reverse(constants.VDI_PROJECT_INDEX)
        context["actions"] = table.render_row_actions(project)
        return context

    @memoized.memoized_method
    def get_data(self):
        """Gets project data"""
        project = None
        try:
            project = TNXTenantClient.get(self.request,
                                          self.kwargs['tenant_id'],
                                          only_vdi_vm=True)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve project details.'),
                              redirect=reverse(constants.VDI_PROJECT_INDEX))
        return project


@rest_urls.register
class ImagesForProject(generic.View):
    """Images api.
    """
    url_regex = r'tnx/vdi/images/$'

    @rest_utils.ajax()
    def get(self, request):
        project_id = request.GET.get('project_id')
        images = get_available_images(request, project_id=project_id)
        return {'available': [i.to_dict() for i in images], 'allocated': []}


@rest_urls.register
class FlavorsForProject(generic.View):
    """Flavors api.
    """
    url_regex = r'tnx/vdi/flavors/$'

    @rest_utils.ajax()
    def get(self, request):
        return {'available': [i.to_dict() for i in flavor_list(request)],
                'allocated': []}


@rest_urls.register
class NetworksForProject(generic.View):
    """Networks api.
    """
    url_regex = r'tnx/vdi/networks/$'

    @rest_utils.ajax()
    def get(self, request):
        project_id = request.GET.get('project_id')

        networks = []
        try:
            networks = [
                network.to_dict()
                for network in neutron.network_list_for_tenant(request,
                                                               project_id)]
        except Exception as err:
            logger.error(err)
            exceptions.handle(request,
                              _('Unable to retrieve shared network list'))

        vdi_default_networks = request.session.get('vdi_default_network', None)
        if vdi_default_networks:
            networks.extend([
                {'id': key,
                 'name': net['net_name'],
                 'shared': net['shared'],
                 'admin_state': net['admin_state'],
                 'subnets': ([{'name': net['subnet_name']}]
                             if net['with_subnet'] else []),
                 'status': 'DRAFT'}
                for key, net in json.loads(vdi_default_networks).iteritems()
            ])

        return {'available': networks, 'allocated': []}


# noinspection PyClassHasNoInit
class TnxDeleteTenantView(projects_views.TnxDeleteTenantView):
    """ Delete tnx tenants view """
    success_url = reverse_lazy("horizon:tionix:vdi:projects_tab")
    confirm_url = reverse_lazy("horizon:tionix:vdi:projects:delete")
