# coding: utf-8
# pylint: disable=no-member,super-on-old-class,non-parent-init-called
# pylint: disable=too-many-lines, duplicate-code

import copy
import json
import logging
import uuid

from django.core import validators
from django.core.urlresolvers import reverse_lazy
from django.forms import ValidationError
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import workflows

from neutronclient.common import exceptions as neutron_exceptions
from novaclient import exceptions as nova_exceptions

from openstack_dashboard.api import (
    keystone, neutron, nova, cinder, base)
from openstack_dashboard import policy
from openstack_dashboard.dashboards.identity.projects import (
    workflows as proj_workflows)
from openstack_dashboard.dashboards.project.networks import (
    workflows as network_workflows)
from openstack_dashboard.usage import quotas

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.identity.projects.utils import (
    get_project_roles_list, get_project_groups_list)

from tionix_dashboard.dashboards.identity.projects.workflows import (
    post_remove_project_cache, CreateProjectQuota, UpdateProjectQuota,
    MultiUpdateProjectQuota, TnxUpdateProjectMembersAction,
    TnxUpdateProjectMembers, TnxUpdateProjectGroups, TnxUpdateProject,
    CreateProject)
from tionix_dashboard.dashboards.project.project_networks import (
    workflows as tnx_net_workflows)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.exceptions import (
    UpdateVDIProjectNetworkException)
from tionix_dashboard.dashboards.tionix.vdi.projects.helpers import (
    ProjectNetworkMixin, get_availability_zone_data)
from tionix_dashboard.dashboards.utils import (
    get_available_image, is_domain_supported)
from tionix_dashboard.helpers.fields import AutoGenNameCharField
from tionix_dashboard.helpers.constants import VDI_PROJECT_TYPE
from tionix_dashboard.helpers.workflows import (
    OptionalFieldsActionMixin, TransferTableAction, TransferTableStep)
from tionix_dashboard.url_resolve_map import VDI_CREATE_NETWORK

logger = logging.getLogger(__name__)

PROJECT_GROUP_ENABLED = is_domain_supported()


class NetworkObject(object):
    def __init__(self, object_id, name):
        super(NetworkObject, self).__init__()
        self.id = object_id
        self.name = name


class CreateSubnetInfoAction(tnx_net_workflows.NetworkCreateSubnetInfoAction):
    class Meta(object):
        name = _("Subnet")
        help_text = _('Creates a subnet associated with the network.'
                      ' You need to enter a valid "Network Address"'
                      ' and "Gateway IP". If you did not enter the'
                      ' "Gateway IP", the first value of a network'
                      ' will be assigned by default. If you do not want'
                      ' gateway please check the "Disable Gateway" checkbox.'
                      ' Advanced configuration is available by clicking on'
                      ' the "Subnet Details" tab.')

    def __init__(self, request, context, *args, **kwargs):
        super(CreateSubnetInfoAction, self).__init__(
            request, context, *args, **kwargs)
        self.fields.pop('with_subnet', None)
        self.context = context

    def clean(self):
        cleaned_data = super(CreateSubnetInfoAction, self).clean()
        with_subnet = self.context.get('with_subnet', False)
        if not with_subnet:
            return cleaned_data
        self._check_subnet_data(cleaned_data)
        return cleaned_data


# noinspection PyClassHasNoInit
class CreateSubnetInfo(network_workflows.CreateSubnetInfo):
    action_class = CreateSubnetInfoAction


# noinspection PyClassHasNoInit
class CreateNetwork(tnx_net_workflows.CreateNetwork):
    default_steps = (tnx_net_workflows.CreateNetworkInfo,
                     CreateSubnetInfo,
                     network_workflows.CreateSubnetDetail)


# noinspection PyClassHasNoInit,PyCompatibility
class CreateDefaultNetworkWorkflow(CreateNetwork):
    success_message = _('Network "%s" will be assigned '
                        'as default for project.')
    wizard = False

    def handle(self, request, data):
        temp_network_id = unicode(uuid.uuid4())
        self.object = NetworkObject(temp_network_id, data['net_name'])
        vdi_default_network = request.session.get('vdi_default_network', '{}')
        vdi_default_network = json.loads(vdi_default_network)
        vdi_default_network[temp_network_id] = data
        request.session['vdi_default_network'] = (
            json.dumps(vdi_default_network))
        return True


class UpdateVDIProjectMembersAction(TnxUpdateProjectMembersAction):
    def __init__(self, request, *args, **kwargs):
        super(UpdateVDIProjectMembersAction, self).__init__(request, *args,
                                                            **kwargs)
        project_id = self.initial.get('project_id')

        if project_id:
            try:
                users_roles = keystone.get_project_users_roles(request,
                                                               project_id)
            except Exception as err:
                exceptions.handle(request, 'Unable to retrieve user list. '
                                           'Please try again later.')
                logger.error(err)
                users_roles = {}
            for user_id in users_roles:
                roles_ids = users_roles[user_id]
                for role_id in roles_ids:
                    field_name = self.get_member_field_name(role_id)
                    self.fields[field_name].initial.append(user_id)

    class Meta(object):
        name = _("Project Members")
        slug = proj_workflows.PROJECT_USER_MEMBER_SLUG


# noinspection PyClassHasNoInit
class MultiUpdateVDIProjectMembersStep(TnxUpdateProjectMembers):
    depends_on = ('domain_id',)


# noinspection PyClassHasNoInit
class MultiUpdateVDIProjectGroupsStep(TnxUpdateProjectGroups):
    depends_on = ('domain_id',)


class CreateProjectInfoAction(workflows.Action):
    # Hide the domain_id and domain_name by default
    domain_id = forms.CharField(label=_("Domain ID"),
                                required=False,
                                widget=forms.HiddenInput())
    domain_name = forms.CharField(label=_("Domain Name"),
                                  required=False,
                                  widget=forms.HiddenInput())
    availability_zone = forms.ChoiceField(label=_("Availability Zone"),
                                          required=False)
    tnx_operation_mode = forms.ChoiceField(
        label=_("Operation mode"),
        choices=OperationModeEnum.get_choices())
    name = AutoGenNameCharField(label=_("Name"), max_length=64)
    description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateProjectInfoAction, self).__init__(request, *args, **kwargs)
        # For keystone V3, display the two fields in read-only
        if keystone.VERSIONS.active >= 3:
            readonly_input = forms.TextInput(attrs={'readonly': 'readonly'})
            self.fields["domain_id"].widget = readonly_input
            self.fields["domain_name"].widget = readonly_input

    class Meta(object):
        name = _("Project Information")
        help_text = _("Create a project to organize users.")

    def _get_projects_by_name(self, project_name):
        kwargs = {
            'filters': {
                'name': project_name
            }
        }
        if policy.check((("identity", "identity:list_projects"),),
                        self.request):
            kwargs.update(
                {'domain': keystone.get_effective_domain_id(self.request)}
            )
        elif policy.check((("identity", "identity:list_user_projects"),),
                          self.request):
            kwargs.update({
                'user': self.request.user.id,
                'admin': False
            })
        return TNXTenantClient.list(self.request, **kwargs)

    def clean_name(self):
        project_name = self.cleaned_data['name']
        projects = self._get_projects_by_name(project_name)
        if projects:
            failure_message = _(u'Project name "{0}" is already used.'
                                u'').format(project_name)
            raise ValidationError(failure_message)
        return project_name

    # noinspection PyUnusedLocal
    @staticmethod
    def populate_availability_zone_choices(request, context):
        return get_availability_zone_data(request)


# noinspection PyClassHasNoInit
class CreateProjectInfo(proj_workflows.CreateProjectInfo):
    action_class = CreateProjectInfoAction
    contributes = ("domain_id", "domain_name", "project_id", "name",
                   "description", "enabled", "availability_zone",
                   "tnx_operation_mode")


class UpdateProjectInfoAction(CreateProjectInfoAction):
    project_id = forms.CharField(label=_("Project ID"),
                                 required=False,
                                 widget=forms.HiddenInput())
    enabled = forms.BooleanField(required=False, label=_("Enabled"))
    tnx_operation_mode = forms.ChoiceField(
        label=_("Operation mode"),
        choices=OperationModeEnum.get_choices(),
        widget=forms.Select(attrs={'disabled': 'disabled'})
    )

    def __init__(self, request, initial, *args, **kwargs):
        super(UpdateProjectInfoAction, self).__init__(
            request, initial, *args, **kwargs)
        if initial['project_id'] == request.user.project_id:
            self.fields['enabled'].widget.attrs['disabled'] = True
            self.fields['enabled'].help_text = _(
                'You cannot disable your current project')

        if request.method == 'GET':
            self._check_tenant_defaults(initial)

    def clean(self):
        cleaned_data = super(UpdateProjectInfoAction, self).clean()
        # NOTE(tsufiev): in case the current project is being edited, its
        # 'enabled' field is disabled to prevent changing the field value
        # which is always `True` for the current project (because the user
        # logged in it). Since Django treats disabled checkbox as providing
        # `False` value even if its initial value is `True`, we need to
        # restore the original `True` value of 'enabled' field here.
        if self.fields['enabled'].widget.attrs.get('disabled', False):
            cleaned_data['enabled'] = True
        return cleaned_data

    def clean_name(self):
        project_name = self.cleaned_data['name']
        project_id = self.initial.get('project_id')
        projects = self._get_projects_by_name(project_name)
        if projects and projects[0].id != project_id:
            failure_message = _(u'Project name "{0}" is already used.'
                                u'').format(project_name)
            raise ValidationError(failure_message)
        return project_name

    class Meta(object):
        name = _("Project Information")
        slug = 'update_info'
        help_text = _("Edit the project details.")

    def _check_tenant_defaults(self, initial):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        availability_zone = initial.get('availability_zone')
        if availability_zone:
            availability_zone_valid = False
            zones = get_availability_zone_data(self.request)
            if zones:
                zone_names = zip(*zones)[0]
                availability_zone_valid = availability_zone in zone_names

            if not availability_zone_valid:
                self.add_error(
                    'availability_zone',
                    _(u"Availability zone is not available. "
                      u"Please choose valid availability zone.")
                )
                initial['availability_zone'] = ''


# noinspection PyClassHasNoInit
class UpdateProjectInfo(proj_workflows.UpdateProjectInfo):
    action_class = UpdateProjectInfoAction
    contributes = ("domain_id", "domain_name", "availability_zone", "name",
                   "description", "enabled", "tnx_operation_mode")


class CreateProjectImageAction(TransferTableAction):
    default_image = forms.CharField(
        label=_('Default image'),
        widget=forms.HiddenInput(),
        error_messages={'required': _('Default image is not chosen.')}
    )

    class Meta(object):
        name = _("Default image")


class CreateProjectImageStep(TransferTableStep):
    action_class = CreateProjectImageAction
    controller = 'CreateProjectImageController'
    contributes = ('default_image', )


class UpdateProjectImageAction(CreateProjectImageAction):

    def __init__(self, request, initial, *args, **kwargs):
        super(UpdateProjectImageAction, self).__init__(
            request, initial, *args, **kwargs)

        if self.request.method == "GET":
            self._check_tenant_defaults(initial)

    def _check_tenant_defaults(self, initial):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        default_image = initial.get('default_image')

        if default_image:
            if get_available_image(self.request, default_image) is None:
                self.add_error(
                    None,
                    _(u"Image with id '{0}' not found. Please choose valid "
                      u"image.").format(default_image)
                )
                initial['default_image'] = ''

    class Meta(object):
        name = _("Default image")


class UpdateProjectImageStep(CreateProjectImageStep):
    action_class = UpdateProjectImageAction


class CreateProjectFlavorAction(TransferTableAction):
    default_flavor = forms.CharField(
        label=_('Default flavor'),
        widget=forms.HiddenInput(),
        error_messages={'required': _('Default flavor is not chosen.')}
    )

    class Meta(object):
        name = _("Default flavor")


class CreateProjectFlavorStep(TransferTableStep):
    action_class = CreateProjectFlavorAction
    controller = 'CreateProjectFlavorController'
    contributes = ('default_flavor', )


class UpdateProjectFlavorAction(CreateProjectFlavorAction):

    def __init__(self, request, initial, *args, **kwargs):
        super(UpdateProjectFlavorAction, self).__init__(
            request, initial, *args, **kwargs)

        if self.request.method == "GET":
            self._check_tenant_defaults(initial)

    def _check_tenant_defaults(self, initial):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        default_flavor = initial.get('default_flavor')

        if default_flavor:
            try:
                # flavors.get returns deleted flavors. Using get_keys()
                # to verify that flavor is not deleted
                nova.flavor_get(self.request, default_flavor).get_keys()
            except nova_exceptions.NotFound:
                self.add_error(
                    None,
                    _(u"Flavor with id '{0}' not found. Please choose valid"
                      u" flavor.").format(default_flavor)
                )
                initial['default_flavor'] = ''

    class Meta(object):
        name = _("Default flavor")


class UpdateProjectFlavorStep(CreateProjectFlavorStep):
    action_class = UpdateProjectFlavorAction


class CreateProjectNetworkAction(TransferTableAction):
    default_network = forms.CharField(
        label=_('Default network'),
        widget=forms.HiddenInput(),
        error_messages={'required': _('Default network is not chosen.')}
    )

    class Meta(object):
        name = _("Default network")


class CreateProjectNetworkStep(TransferTableStep):
    action_class = CreateProjectNetworkAction
    template_name = 'vdi/projects/_network_step.html'
    controller = 'CreateProjectNetworkController'
    contributes = ('default_network', )
    add_item_link = reverse_lazy(VDI_CREATE_NETWORK)


# noinspection PyCompatibility,PyAttributeOutsideInit
class UpdateProjectNetworkAction(CreateProjectNetworkAction):

    def __init__(self, request, initial, *args, **kwargs):
        super(UpdateProjectNetworkAction, self).__init__(
            request, initial, *args, **kwargs)

        project_id = initial.get('project_id')
        if self.request.method == "GET":
            try:
                networks = neutron.network_list_for_tenant(request, project_id)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(request, _('Unable to retrieve VDI '
                                             'project information.'))
            else:
                networks = [(network.id, network.name) for network in networks]
                self.check_tenant_defaults(initial, networks)

    def check_tenant_defaults(self, initial, networks):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        default_network = initial.get('default_network')

        if default_network:
            try:
                neutron.network_get(self.request, default_network)

            except neutron_exceptions.NotFound as err:
                logger.error(unicode(err))
                self.add_error(
                    None,
                    _(u"Network with id '{0}' not found. Please choose valid"
                      u" network.").format(default_network)
                )
                initial['default_network'] = ''
            else:
                if default_network not in zip(*networks)[0]:
                    self.add_error(
                        None,
                        _(u"Network with id '{0}' is not available. Please "
                          u"choose a valid network.").format(default_network)
                    )

    class Meta(object):
        name = _("Default network")


# noinspection PyCompatibility
class CloneProjectNetworkAction(UpdateProjectNetworkAction):
    def check_tenant_defaults(self, initial, networks):
        super(CloneProjectNetworkAction, self).check_tenant_defaults(
            initial, networks)
        default_network = initial.get('default_network')

        if default_network:
            chosen_network = neutron.network_get(self.request, default_network)
            if not chosen_network['shared']:
                initial['default_network'] = ''

    class Meta(object):
        name = _("Default network")


class UpdateProjectNetworkStep(CreateProjectNetworkStep):
    action_class = UpdateProjectNetworkAction


class CloneProjectNetworkStep(CreateProjectNetworkStep):
    action_class = CloneProjectNetworkAction


# noinspection PyCompatibility
class CreateVDIProject(ProjectNetworkMixin,
                       CreateProject):
    success_message = _('Created new project "%s".')
    failure_message = _('Error creating project "%s".')
    success_url = "horizon:tionix:vdi:index"
    name = _("Create VDI Project")
    template_name = "vdi/projects/_workflow.html"

    default_steps = (CreateProjectInfo,
                     CreateProjectImageStep,
                     CreateProjectFlavorStep,
                     CreateProjectNetworkStep,
                     TnxUpdateProjectMembers,
                     CreateProjectQuota)

    # pylint: disable=super-init-not-called,keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):

        if PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  CreateProjectImageStep,
                                  CreateProjectFlavorStep,
                                  CreateProjectNetworkStep,
                                  TnxUpdateProjectMembers,
                                  TnxUpdateProjectGroups,
                                  CreateProjectQuota)
        workflows.Workflow.__init__(self, request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)

    @post_remove_project_cache
    def handle(self, request, data):
        saving_network_id = data['default_network']
        vdi_default_networks = request.session.pop('vdi_default_network', None)

        nova_client = novaclient(request)

        data.pop('domain_name')
        data['project_type'] = VDI_PROJECT_TYPE
        data['network_id'] = None
        data['flavor_id'] = data.pop('default_flavor', None)
        data['image_id'] = data.pop('default_image', None)
        data['description'] = data.get('description', '').strip()

        try:
            response = nova_client.tnx_tenants.create(**data)
        except nova_exceptions.Conflict:
            self.failure_message = _('Project name "{0}" is already '
                                     'used.').format(data['name'])
            result = False
        except Exception as err:
            logger.error(unicode(err))
            result = False
        else:
            _code, tenant = response
            tenant = tenant['tenant']
            tenant_id = tenant['id']

            self._update_project_members(request, data, tenant_id)
            if PROJECT_GROUP_ENABLED:
                self._update_project_groups(request, data, tenant_id)
            if keystone.is_domain_admin(request):
                self._update_project_quota(request, data, tenant_id)

            network_id = self._get_network_id(
                request,
                network_id=saving_network_id,
                network_list=vdi_default_networks,
                tenant_id=tenant_id)

            if network_id:
                result = self._update_tenant_network(request,
                                                     novaclient=nova_client,
                                                     tenant_id=tenant_id,
                                                     network_id=network_id)
            else:
                result = False
                exceptions.handle(
                    request,
                    _(
                        'Unable to create network for VDI project "{0}".'
                    ).format(data['name']))

        return result


class CreateVDIProjectNoQuota(CreateVDIProject):

    default_steps = (CreateProjectInfo,
                     CreateProjectImageStep,
                     CreateProjectFlavorStep,
                     CreateProjectNetworkStep,
                     TnxUpdateProjectMembers)

    # pylint: disable=super-init-not-called,keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):

        if PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  CreateProjectImageStep,
                                  CreateProjectFlavorStep,
                                  CreateProjectNetworkStep,
                                  TnxUpdateProjectMembers,
                                  TnxUpdateProjectGroups)

        workflows.Workflow.__init__(self, request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)


# noinspection PyCompatibility
class UpdateVDIProject(ProjectNetworkMixin,
                       TnxUpdateProject):
    success_message = _('Project "%s" has been updated.')
    failure_message = _('Error updating project "%s".')

    success_url = "horizon:tionix:vdi:index"
    name = _("Update VDI Project")

    if PROJECT_GROUP_ENABLED:
        default_steps = (UpdateProjectInfo,
                         UpdateProjectImageStep,
                         UpdateProjectFlavorStep,
                         UpdateProjectNetworkStep,
                         TnxUpdateProjectMembers,
                         TnxUpdateProjectGroups,
                         UpdateProjectQuota)
    else:
        default_steps = (UpdateProjectInfo,
                         UpdateProjectImageStep,
                         UpdateProjectFlavorStep,
                         UpdateProjectNetworkStep,
                         TnxUpdateProjectMembers,
                         UpdateProjectQuota)

    # pylint: disable=super-init-not-called,keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        workflows.Workflow.__init__(self,
                                    request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)

    def get_absolute_url(self):
        return self.request.get_full_path()

    # pylint: disable=too-many-arguments
    def _update_vdi_project(self, request, nova_client, tenant_id,
                            vdi_default_networks, default_network_id,
                            tenant_data, data):
        nova_client.tnx_tenants.update(tenant_id, **tenant_data)

        self._update_project_members(request, data, tenant_id)
        if PROJECT_GROUP_ENABLED:
            self._update_project_groups(request,
                                        data,
                                        tenant_id,
                                        data['domain_id'])
        if keystone.is_domain_admin(request):
            self._update_project_quota(request, data, tenant_id)

        if default_network_id:
            network_id = self._get_network_id(
                request,
                network_id=default_network_id,
                network_list=vdi_default_networks,
                tenant_id=tenant_id)

            if network_id:
                self._update_tenant_network(
                    request,
                    novaclient=nova_client,
                    tenant_id=tenant_id,
                    network_id=network_id)
            else:
                raise UpdateVDIProjectNetworkException()

    @post_remove_project_cache
    def handle(self, request, data):
        vdi_default_networks = request.session.pop('vdi_default_network', None)

        nova_client = novaclient(request)

        data['enabled'] = unicode(data['enabled'])
        data['flavor_id'] = data.pop('default_flavor', None)
        data['image_id'] = data.pop('default_image', None)
        data['description'] = data.get('description', '').strip()
        tenant_data = copy.copy(data)
        tenant_data.pop('domain_id')
        tenant_data.pop('domain_name')
        tenant_id = tenant_data.pop('project_id')
        default_network_id = data.pop('default_network')

        result = True
        try:
            self._update_vdi_project(request,
                                     nova_client,
                                     tenant_id,
                                     vdi_default_networks,
                                     default_network_id,
                                     tenant_data,
                                     data)
        except nova_exceptions.Conflict:
            self.failure_message = _('Project name "{0}" is already '
                                     'used.').format(data['name'])
            result = False
        except UpdateVDIProjectNetworkException:
            messages.info(request,
                          _('Unable to create network for VDI project '
                            '"{0}".').format(data['name']))
        except Exception as err:
            logger.error(unicode(err))
            result = False

        return result


class MultiUpdateProjectInfoAction(workflows.Action):

    description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)
    enabled = forms.BooleanField(label=_("Enabled"),
                                 required=False,
                                 initial=True)

    def clean(self):
        cleaned_data = super(MultiUpdateProjectInfoAction, self).clean()
        projects = self.initial['projects']
        current_project_id = self.request.user.tenant_id
        if current_project_id in projects and not cleaned_data['enabled']:
            raise forms.ValidationError(
                _('Current project "{0}" is selected. You cannot disable your '
                  'current project.').format(projects[current_project_id])
            )
        return cleaned_data

    class Meta(object):
        name = _("Project Information")
        slug = 'update_info'
        help_text = _("Edit the project details.")


# noinspection PyClassHasNoInit
class MultiUpdateProjectInfo(proj_workflows.UpdateProjectInfo):
    action_class = MultiUpdateProjectInfoAction
    template_name = proj_workflows.COMMON_HORIZONTAL_TEMPLATE
    depends_on = ("projects",)
    contributes = ("description",
                   "enabled")


class MultiUpdateProjectImageAction(OptionalFieldsActionMixin,
                                    CreateProjectImageAction):

    class Meta(object):
        name = _("Default image")


class MultiUpdateProjectImageStep(CreateProjectImageStep):
    action_class = MultiUpdateProjectImageAction


class MultiUpdateProjectFlavorAction(OptionalFieldsActionMixin,
                                     CreateProjectFlavorAction):

    class Meta(object):
        name = _("Default flavor")


class MultiUpdateProjectFlavorStep(CreateProjectFlavorStep):
    action_class = MultiUpdateProjectFlavorAction


class MultiUpdateProjectNetworkAction(OptionalFieldsActionMixin,
                                      CreateProjectNetworkAction):

    class Meta(object):
        name = _("Default network")


class MultiUpdateProjectNetworkStep(CreateProjectNetworkStep):
    action_class = MultiUpdateProjectNetworkAction


# noinspection PyCompatibility
class MultiUpdateVDIProject(UpdateVDIProject):
    success_message = _('Successfully updated projects: "{0}".')
    failure_message = _('Unable to update projects "{0}".')

    name = _("Update VDI Projects")

    if PROJECT_GROUP_ENABLED:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectImageStep,
                         MultiUpdateProjectFlavorStep,
                         MultiUpdateProjectNetworkStep,
                         MultiUpdateVDIProjectMembersStep,
                         MultiUpdateVDIProjectGroupsStep,
                         MultiUpdateProjectQuota)
    else:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectImageStep,
                         MultiUpdateProjectFlavorStep,
                         MultiUpdateProjectNetworkStep,
                         MultiUpdateVDIProjectMembersStep,
                         MultiUpdateProjectQuota)

    def _update_project_quota(self, request, data, project_id):
        """Updates only filled quotas"""
        # Update the project quota.
        nova_data = dict([(key, data[key])
                          for key in quotas.NOVA_QUOTA_FIELDS if key in data])
        nova.tenant_quota_update(request, project_id, **nova_data)

        if cinder.is_volume_service_enabled(request):
            cinder_data = dict([(key, data[key])
                                for key in quotas.CINDER_QUOTA_FIELDS
                                if key in data])
            cinder.tenant_quota_update(request,
                                       project_id,
                                       **cinder_data)

        if (base.is_service_enabled(request, 'network') and
                neutron.is_quotas_extension_supported(request)):
            disabled_quotas = quotas.get_disabled_quotas(request)
            neutron_data = dict(
                [(key, data[key])
                 for key in quotas.NEUTRON_QUOTA_FIELDS
                 if key in data and key not in disabled_quotas]
            )

            neutron.tenant_quota_update(request, project_id, **neutron_data)

    @post_remove_project_cache
    def handle(self, request, data):
        def _check_data(key, value):
            return not all([
                value in validators.EMPTY_VALUES,
                not key.startswith(proj_workflows.PROJECT_USER_MEMBER_SLUG),
                not key.startswith(proj_workflows.PROJECT_GROUP_MEMBER_SLUG)])

        vdi_default_networks = request.session.pop('vdi_default_network', None)

        nova_client = novaclient(request)

        data['enabled'] = unicode(data['enabled'])
        data['flavor_id'] = data.pop('default_flavor', None)
        data['image_id'] = data.pop('default_image', None)
        default_network_id = data.pop('default_network', None)
        # Remove all empty fields
        data = {key: value for key, value in data.items()
                if _check_data(key, value)}
        tenant_data = copy.copy(data)
        tenant_data.pop('domain_id')

        success, network_failed, failed = [], [], []
        for tenant_id, name in self.context_seed['projects'].items():

            try:
                self._update_vdi_project(request,
                                         nova_client,
                                         tenant_id,
                                         vdi_default_networks,
                                         default_network_id,
                                         tenant_data,
                                         data)
                success.append(name)
            except UpdateVDIProjectNetworkException:
                network_failed.append(name)
            except Exception as err:
                logger.error(unicode(err))
                failed.append(name)

        result = True
        if success:
            self.success_message = self.success_message.format(
                '", "'.join(success)
            )

        if failed:
            result = False
            self.failure_message = self.failure_message.format(
                '", "'.join(failed)
            )

        if network_failed:
            messages.info(
                request,
                _('Unable to create network for VDI '
                  'projects: "{0}".').format('", "'.join(failed)))

        return result


class CloneStepMixin(object):

    def contribute(self, data, context):
        members = self._get_members()
        if data and members:
            post = self.workflow.request.POST
            default_role_field = self.action.get_default_role_field_name()
            if default_role_field in post:
                members_data = dict(self.workflow.request.POST)
            else:
                members_data = self.get_initial_data()

            for member in members:
                field = self.get_member_field_name(member.id)
                context[field] = members_data.get(field, [])
        return context


class TnxCloneProjectMembers(CloneStepMixin, TnxUpdateProjectMembers):

    def get_initial_data(self):
        return get_project_roles_list(
            self.action.request,
            self.workflow.context_seed.get('project_id'),
            self.action.get_member_field_name
        )


class TnxCloneProjectGroups(CloneStepMixin, TnxUpdateProjectGroups):

    def get_initial_data(self):
        return get_project_groups_list(
            self.action.request,
            self.workflow.context_seed.get('project_id'),
            self.action.get_member_field_name
        )


class CloneVDIProject(CreateVDIProject):
    success_message = _('Project "%s" has been cloned.')
    failure_message = _('Error cloning project "%s".')

    success_url = "horizon:tionix:vdi:index"
    name = _("Clone VDI Project")

    finalize_button_name = _("Clone Project")

    default_steps = (CreateProjectInfo,
                     UpdateProjectImageStep,
                     UpdateProjectFlavorStep,
                     CloneProjectNetworkStep,
                     TnxCloneProjectMembers,
                     CreateProjectQuota)

    # pylint: disable=super-init-not-called,keyword-arg-before-vararg
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  UpdateProjectImageStep,
                                  UpdateProjectFlavorStep,
                                  CloneProjectNetworkStep,
                                  TnxCloneProjectMembers,
                                  TnxCloneProjectGroups,
                                  CreateProjectQuota)

        workflows.Workflow.__init__(self, request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)

    def handle(self, request, data):
        data['project_id'] = None
        return super(CloneVDIProject, self).handle(request, data)


class UpdateVDIProjectNoQuota(UpdateVDIProject):

    if PROJECT_GROUP_ENABLED:
        default_steps = (UpdateProjectInfo,
                         UpdateProjectImageStep,
                         UpdateProjectFlavorStep,
                         UpdateProjectNetworkStep,
                         TnxUpdateProjectMembers,
                         TnxUpdateProjectGroups)
    else:
        default_steps = (UpdateProjectInfo,
                         UpdateProjectImageStep,
                         UpdateProjectFlavorStep,
                         UpdateProjectNetworkStep,
                         TnxUpdateProjectMembers)


class MultiUpdateVDIProjectNoQuota(UpdateVDIProject):

    if PROJECT_GROUP_ENABLED:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectImageStep,
                         MultiUpdateProjectFlavorStep,
                         MultiUpdateProjectNetworkStep,
                         MultiUpdateVDIProjectMembersStep,
                         MultiUpdateVDIProjectGroupsStep)
    else:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectImageStep,
                         MultiUpdateProjectFlavorStep,
                         MultiUpdateProjectNetworkStep,
                         MultiUpdateVDIProjectMembersStep)
