# coding: utf-8

import logging

from django.forms import ValidationError
from django.template.defaultfilters import filesizeformat
from django.utils.datastructures import SortedDict
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions

from openstack_dashboard import api, policy
from openstack_dashboard.api import keystone
from openstack_dashboard.api.keystone import keystoneclient

from keystoneauth1.exceptions import Forbidden

from neutronclient.common import exceptions as neutron_exceptions

from novaclient import exceptions as nova_exceptions

from tionix_dashboard.clients.base import novaclient

logger = logging.getLogger(__name__)


TABLE_OBJECT_ID_SEP = '_'


def update_instances_list(request, instances, tenants=None,
                          project=False, filter_by_current_domain=False):

    updated_instances = []

    if instances:
        domain_id = None
        if filter_by_current_domain:
            domain_id = api.keystone.get_default_domain(
                request, get_name=False).get('id')

        if tenants is None:
            tenants = []
            # Gather our tenants to correlate against IDs
            try:
                tenants, _has_more = api.keystone.tenant_list(request,
                                                              admin=False)
            except Forbidden:
                pass

            except Exception as err:
                logger.error(unicode(err))
                msg = _(
                    'Unable to retrieve instance project information.')
                exceptions.handle(request, msg)

        # Gather our flavors to correlate against IDs
        try:
            flavors = api.nova.flavor_list(request)
        except Exception as err:
            logger.error(unicode(err))
            # If fails to retrieve flavor list, creates an empty list.
            flavors = []

        if project:
            try:
                images, _more, _prev = api.glance.image_list_detailed(
                    request)
            except Exception as err:
                logger.error(unicode(err))
                image_map = {}
            else:
                image_map = SortedDict([(str(image.id), image)
                                        for image in images])

        full_flavors = SortedDict([(f.id, f) for f in flavors])
        tenant_dict = SortedDict([(t.id, t) for t in tenants])

        # Loop through instances to get flavor and tenant info.
        for inst in instances:
            inst_project = tenant_dict.get(inst.tenant_id)
            if (filter_by_current_domain and
                    (inst_project is None or
                     inst_project.domain_id != domain_id)):
                continue

            if project and hasattr(inst, 'image'):
                # Instance from image returns dict
                if isinstance(inst.image, dict):
                    if inst.image.get('id') in image_map:
                        image = image_map[inst.image['id']]
                        inst.image = image

            flavor_id = inst.flavor["id"]
            try:
                if flavor_id in full_flavors:
                    inst.full_flavor = full_flavors[flavor_id]
                else:
                    # If the flavor_id is not in full_flavors list,
                    # gets it via nova api.
                    inst.full_flavor = api.nova.flavor_get(request,
                                                           flavor_id)
            except Exception as err:
                logger.error(unicode(err))
                msg = _(
                    'Unable to retrieve instance size information.')
                exceptions.handle(request, msg)

            tenant = tenant_dict.get(inst.tenant_id, None)
            inst.tenant = tenant
            inst.tenant_name = getattr(tenant, "name", None)
            updated_instances.append(inst)

        try:
            api.network.servers_update_addresses(request, instances,
                                                 all_tenants=True)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                request,
                message=_(
                    'Unable to retrieve IP addresses from Neutron.'),
                ignore=True)

    return updated_instances


quota_map = {
    'instances': {
        'used': 'totalInstancesUsed',
        'total': 'maxTotalInstances',
    },
    'cores': {
        'used': 'totalCoresUsed',
        'total': 'maxTotalCores',
    },
    'ram': {
        'used': 'totalRAMUsed',
        'total': 'maxTotalRAMSize',
    }
}


def get_quota_available(quota):
    available = {}
    for key, params in quota_map.items():
        used = quota.get(params['used'])
        total = quota.get(params['total'])
        if used is not None and total not in (None, -1):
            available[key] = total - used
    return available


# pylint: disable=no-member
def quotas_usages_validation(request, flavor_id=None, tenant_id=None,
                             instances_count=1):
    is_admin = keystone.is_domain_admin(request)
    project_id = tenant_id if not is_admin else None
    nova_cl = novaclient(request, project_id=project_id)
    try:
        flavor = nova_cl.flavors.get(flavor_id)
    except nova_exceptions.NotFound:
        raise ValidationError(_('Unable to get flavor '
                                'with id {0}.').format(flavor_id))
    try:
        if is_admin:
            limits = nova_cl.limits.get(tenant_id=tenant_id)
        else:
            # if not admin, tenant_id parameter is not allowed by policy,
            # even if tenant_id is authorized tenant
            limits = nova_cl.limits.get()
    except Exception as err:
        logger.error(unicode(err))
        raise ValidationError(_('Failed to retrieve quota information.'))
    else:
        cores_count = instances_count * flavor.vcpus
        ram_count = instances_count * flavor.ram

        _quotas = limits.to_dict()
        quotas = _quotas.get('absolute')
        available = get_quota_available(quotas)
        logger.debug(u"Project '{0}' quota available: {1}".format(
            tenant_id, available))

        instances_available = available.get('instances')
        cores_available = available.get('cores')
        ram_available = available.get('ram')

        if (instances_available is not None and
                instances_available < instances_count):
            raise ValidationError(
                _('Not enough quota for instances count.'))
        elif cores_available is not None and cores_available < cores_count:
            raise ValidationError(_('Not enough quota for cores.'))
        elif ram_available is not None and ram_available < ram_count:
            raise ValidationError(_('Not enough quota for ram.'))


def is_domain_supported():
    """Checks domain support"""
    return api.keystone.VERSIONS.active >= 3


def user_list(request, project=None, domain=None, group=None, filters=None):
    """Retrieve users list"""
    if is_domain_supported():
        kwargs = {
            "project": project,
            "domain": domain,
            "group": group
        }

    else:
        kwargs = {"tenant_id": project}
    if filters is not None:
        kwargs.update(filters)
    users = keystoneclient(
        request, admin=keystone.is_domain_admin(request)
    ).users.list(**kwargs)
    return [api.keystone.VERSIONS.upgrade_v2_user(user) for user in users]


def get_available_image(request, image_id):
    """
    Gets available image
    :param request: django request
    :param image_id: uuid
    :return:
    """
    try:
        image = api.glance.image_get(request, image_id)
        if image.status.lower() == u'deleted':
            image = None
    except nova_exceptions.NotFound:
        image = None
    return image


def get_available_flavor(request, flavor_id):
    """
    Gets available flavor
    :param request: django request
    :param flavor_id: uuid
    :return:
    """
    try:
        flavor = api.nova.flavor_get(request, flavor_id)
        # flavors.get returns deleted flavors. Using get_keys()
        # to verify that flavor is not deleted
        flavor.get_keys()
    except nova_exceptions.NotFound:
        flavor = None
    return flavor


def get_available_network(request, network_id):
    """
    Gets available network
    :param request: django request
    :param network_id: uuid
    :return:
    """
    try:
        network = api.neutron.network_get(request, network_id)
    except neutron_exceptions.NotFound:
        network = None
    return network


def image_choice_title(img):
    gb = filesizeformat(img.size)
    return '%s (%s)' % (img.name or img.id, gb)


def get_tenants_kwargs(request):
    is_admin = api.keystone.is_domain_admin(request)
    kwargs = {
        'admin': is_admin,
    }
    if is_admin:
        domain = api.keystone.get_default_domain(request, get_name=False)
        kwargs['domain'] = domain.get('id')
    else:
        kwargs['user'] = request.user.id
    return kwargs


def domain_lookup(request):
    """Return dict of available domains"""
    domain_dict = {}
    if policy.check((("identity", "identity:list_domains"),), request):
        try:
            domains = api.keystone.domain_list(request)
        except Exception as err:
            logger.error(unicode(err))
            logger.warn("Pure project admin doesn't have a domain token")
        else:
            domain_dict = {domain.id: domain.name for domain in domains}
    else:

        get_name = policy.check((("identity", "identity:get_domain"),),
                                request)

        domain = api.keystone.get_default_domain(request, get_name=get_name)
        domain_dict = {domain.id: domain.name}
    return domain_dict
