#!/usr/bin/env python

import os
import sys

from getpass import getuser

import django

from tionix_dashboard import decorate_secret_key_generator, check_configuration
from tionix_dashboard.helpers.i18 import ugettext as _


def get_migrations_count(recorder):
    return recorder.migration_qs.count()


def set_os_dashboard_settings():
    openstack_dashboard_path = '/usr/share/openstack-dashboard'
    if openstack_dashboard_path not in sys.path:
        sys.path.append(openstack_dashboard_path)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE",
                          "openstack_dashboard.settings")


def main(migration=None, verbose=False):
    migrate_args = None

    set_os_dashboard_settings()
    decorate_secret_key_generator()
    django.setup()

    # noinspection PyTypeChecker
    if isinstance(migration, basestring):
        migrate_args = migration.split()
        if len(migrate_args) != 2:
            return (
                os.EX_SOFTWARE,
                _(u'Invalid migration arguments: "{0}". '
                  u'Provide arguments as follows:'
                  u' "app_name migration_name"').format(migration)
            )

    # --traceback prevents calling sys.exit in execute_from_command_line
    # if CommandError exception raised
    args = [os.path.abspath(__file__), 'migrate', '--traceback']
    if migrate_args:
        args.extend(migrate_args)

    if not check_configuration():
        msg = _(u'Utility could not be executed by user '
                u'"{0}"').format(getuser())
        result = os.EX_SOFTWARE, msg
    else:
        from django.core.management import execute_from_command_line
        from django.db import connection
        from django.db.utils import ProgrammingError
        try:
            from django.db.migrations.recorder import MigrationRecorder
        except IOError as ex:
            return os.EX_SOFTWARE, unicode(ex)

        msg = _(u'Unknown DB Error. Please contact your administrator.')

        recorder = MigrationRecorder(connection)
        try:
            old_migrations_count = get_migrations_count(recorder)
        except ProgrammingError:
            old_migrations_count = 0
        except Exception as ex:
            if verbose:
                msg = _(u'Error while getting migrate versions: '
                        u'{0}').format(unicode(ex))
            return os.EX_SOFTWARE, msg

        try:
            execute_from_command_line(args)
        except Exception as ex:
            if verbose:
                msg = _(u'Error while db migrate: {0}').format(unicode(ex))
            return os.EX_SOFTWARE, msg

        try:
            new_migrations_count = get_migrations_count(recorder)
        except Exception as ex:
            if verbose:
                msg = _(u'Error while getting migrate versions: '
                        u'{0}').format(unicode(ex))
            result = os.EX_SOFTWARE, msg
        else:
            if new_migrations_count == old_migrations_count:
                msg = _(u'No changes')
            else:
                msg = u'{0} -> {1}'.format(old_migrations_count,
                                           new_migrations_count)
            result = os.EX_OK, msg

    return result
