# coding: utf-8
# pylint: disable=no-member

import logging
import collections

from django import shortcuts
from django.core.urlresolvers import reverse, resolve
from django.utils.translation import ugettext_lazy as _

from horizon import messages, tables
from horizon.utils import functions

from openstack_dashboard import api

logger = logging.getLogger(__name__)


# noinspection PyAbstractClass
class TnxFilterAction(tables.FilterAction):
    """
    Filters by filter_field from table
    """

    field_get_data = None

    def filter(self, table, data, filter_string):
        filter_field = table.get_filter_field()
        if all([filter_field,
                filter_string,
                not self.is_api_filter(filter_field)]):
            q = filter_string.lower()
            get_data = None
            # pylint: disable=unsupported-membership-test
            # pylint: disable=unsubscriptable-object
            if self.field_get_data and filter_field in self.field_get_data:
                get_data = self.field_get_data[filter_field]
            else:
                column = table.columns.get(filter_field)
                if column:
                    get_data = column.get_data

            def comp(obj):
                obj_data = unicode(
                    get_data(obj)
                    if get_data else getattr(obj, filter_field, '')
                )
                return q in obj_data.lower()

            data = filter(comp, data)

            table.total_count = len(data)
        return data


class TnxUpdateAction(tables.UpdateAction):
    """
    Base action for update cells strips trailing and leading
    whitespaces
    """

    def allowed(self, request, datum, cell):
        """
        check permission
        """
        return api.keystone.keystone_can_edit_project()

    def action(self, request, datum, obj_id, cell_name, new_cell_value):
        # noinspection PyTypeChecker
        if isinstance(new_cell_value, basestring):
            new_cell_value = new_cell_value.strip()

        return super(TnxUpdateAction, self).action(
            request, datum, obj_id, cell_name, new_cell_value
        )


# noinspection PyProtectedMember
class TnxBatchActionMixin(object):
    """Tionix Batch Action with quotes in messages"""

    classes = []
    verbose_name = None
    verbose_name_plural = None
    datum = False
    datum_id_attr = 'id'

    def set_danger_btn_class(self, flag=False):
        """Add or remove btn-danger class"""

        if flag and "btn-danger" not in self.classes:
            self.classes += ("btn-danger",)
        elif not flag and "btn-danger" in self.classes:
            self.classes = [c for c in self.classes if c != "btn-danger"]

    def get_default_attrs(self):
        """Returns a list of the default HTML attributes for the action."""
        attrs = super(TnxBatchActionMixin, self).get_default_attrs()
        confirm_header = (
            self._get_confirm_header('plural') if self.datum is None
            else self._get_confirm_header()
        )
        attrs.update(
            {'data-confirm-header': confirm_header}
        )
        return attrs

    def confirm_header(self, count):
        verbose_name = (self.verbose_name
                        if count == 1
                        else self.verbose_name_plural)

        return verbose_name() if callable(verbose_name) else verbose_name

    def _get_confirm_header(self, items=None):
        count = len(items) if isinstance(items, collections.Sized) else 1
        return self.confirm_header(count)

    def get_success_message(self, action_success):
        msg = u'%(action)s: "%(objs)s".'
        params = {"action":
                  self._get_action_name(action_success, past=True),
                  "objs": functions.lazy_join(u'", "', action_success)}
        return msg % params

    def handle(self, table, request, obj_ids):
        action_success = []
        action_failure = []
        action_not_allowed = []
        for datum_id in obj_ids:
            datum = table.get_object_by_id(datum_id)
            datum_display = table.get_object_display(datum) or datum_id
            datum_real_id = getattr(datum, self.datum_id_attr, datum_id)
            if not table._filter_action(self, request, datum):
                action_not_allowed.append(datum_display)
                msg = u'Permission denied to %s: "%s"' % (
                    self._get_action_name(past=True).lower(), datum_display
                )
                logger.warning(msg)
                continue
            try:
                self.action(request, datum_real_id)
                # Call update to invoke changes if needed
                self.update(request, datum)
                action_success.append(datum_display)
                self.success_ids.append(datum_real_id)
                logger.info(u'%s: "%s"', self._get_action_name(past=True),
                            datum_display)
            except Exception as ex:
                # Handle the exception but silence it since we'll display
                # an aggregate error message later. Otherwise we'd get
                # multiple error messages displayed to the user.
                action_failure.append(u'"{name}"({reason})'.format(
                    name=datum_display, reason=unicode(ex)))
                action_description = (
                    self._get_action_name(past=True).lower(), datum_display)
                logger.warning(
                    u'Action %(action)s Failed for %(reason)s',
                    {'action': action_description, 'reason': ex}
                )

        # Begin with success message class, downgrade to info if problems.
        success_message_level = messages.success
        if action_not_allowed:
            msg = _('You are not allowed to %(action)s: "%(objs)s".')
            params = {"action":
                      self._get_action_name(action_not_allowed).lower(),
                      "objs": functions.lazy_join(u'", "', action_not_allowed)}
            messages.error(request, msg % params)
            success_message_level = messages.info
        if action_failure:
            msg = _('Unable to %(action)s: "%(objs)s".')
            params = {"action": self._get_action_name(action_failure).lower(),
                      "objs": functions.lazy_join(u', ', action_failure)}
            messages.error(request, msg % params)
            success_message_level = messages.info
        if action_success:
            success_message_level(request,
                                  self.get_success_message(action_success))

        return shortcuts.redirect(self.get_success_url(request))


# noinspection PyAbstractClass,PyProtectedMember
class LinkBatchAction(tables.BatchAction):
    single_next_view_name = None
    multiple_next_view_name = None

    def __init__(self, **kwargs):
        super(LinkBatchAction, self).__init__(**kwargs)
        if not (self.multiple_next_view_name or self.single_next_view_name):
            class_name = self.__class__.__name__
            raise NotImplementedError(
                "You must define a multiple_next_view_name "
                "attribute on %s" % class_name
            )

    def action_present(self, count):
        return self.name

    def action_past(self, count):
        return self.action_present(count)

    # noinspection PyMethodMayBeStatic
    def get_next_view_url(self, next_view_name):
        return reverse(next_view_name, prefix='/')

    def get_default_attrs(self):
        """Returns a list of the default HTML attributes for the action."""
        attrs = super(LinkBatchAction, self).get_default_attrs()
        attrs.update({'ng-click': ''})
        return attrs

    # noinspection PyMethodMayBeStatic
    def get_next_view_kwargs(self, table, obj_ids, kwargs=None):
        kwargs = kwargs or {}
        kwargs.update({'table': table, 'ids': obj_ids})
        return kwargs

    def get_next_view_instance(self, request, next_view_name, table, obj_ids):
        # We change POST->GET to escape form validation
        request.method = 'GET'
        resolved_next_view = resolve(self.get_next_view_url(next_view_name))
        next_view_func = resolved_next_view.func
        next_view_kwargs = resolved_next_view.kwargs
        return next_view_func(request,
                              **self.get_next_view_kwargs(table,
                                                          obj_ids,
                                                          next_view_kwargs))

    # pylint: disable=method-hidden
    def handle(self, table, request, obj_ids):
        available_obj_ids, action_not_allowed = [], []
        for datum_id in obj_ids:
            datum = table.get_object_by_id(datum_id)
            datum_display = table.get_object_display(datum) or datum_id
            if not table._filter_action(self, request, datum):
                action_not_allowed.append(datum_display)
                msg = u'Permission denied to %s: "%s"' % (
                    self._get_action_name(past=True).lower(), datum_display
                )
                logger.warning(msg)
                continue
            available_obj_ids.append(datum_id)

        if action_not_allowed:
            msg = _('You are not allowed to %(action)s: "%(objs)s".')
            params = {"action":
                      self._get_action_name(action_not_allowed).lower(),
                      "objs": functions.lazy_join(u'", "', action_not_allowed)}
            messages.error(request, msg % params)

        if available_obj_ids:
            if len(obj_ids) == 1:
                next_view_name = (
                    self.single_next_view_name or self.multiple_next_view_name
                )
            else:
                next_view_name = (
                    self.multiple_next_view_name or self.single_next_view_name
                )
            return self.get_next_view_instance(request,
                                               next_view_name,
                                               table,
                                               available_obj_ids)

        return shortcuts.redirect(self.get_success_url(request))

    def get_default_classes(self):
        classes = super(LinkBatchAction, self).get_default_classes()
        classes += ("btn-link-batch",)
        return classes
