# coding: utf-8
# pylint: disable=no-init
"""
base form components
"""

from django import forms
from django.utils.encoding import smart_text
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from tionix_dashboard.helpers.utils import (
    get_datetime_value, gen_uuid_object_name)


class BootstrapInputMixin(object):

    def render(self, name, value, attrs=None):
        safe_string = super(BootstrapInputMixin, self).render(name,
                                                              value,
                                                              attrs)
        safe_string = mark_safe(u'{1}\n{0}'.format(self.media, safe_string))
        return safe_string


class BootstrapTimePickerField(BootstrapInputMixin, forms.DateTimeInput):
    """
    time picker field

    us:
        widget=BootstrapTimePickerField(
            attrs={
                'data-provide-bootstrapdatetimepicker': 'timepicker'
            }
        ),
    """

    class Media(object):
        css = {
            'all': ('tionix/css/bootstrap-datetimepicker.min.css', )
        }
        js = (
            'tionix/js/moment-with-locales.min.js',
            'tionix/js/bootstrap-datetimepicker.min.js',
            'tionix/js/BootstrapTimePickerField.js')


class EmptyCharField(forms.CharField):
    """
    field allow '' in value
    """
    empty_values = (None, [], (), {})


class TionixDateField(forms.DateField):
    """
    date in format dd.mm.yyyy
    """

    input_formats = ('%d.%m.%Y', )

    def prepare_value(self, value):
        return get_datetime_value(value, date_format=self.input_formats[0])


class BootstrapSwitcherField(BootstrapInputMixin, forms.CheckboxInput):
    """"
    Switcher field
    """

    class Media(object):
        css = {
            'all': ('tionix/css/bootstrap-switch.min.css', )
        }

        js = ('tionix/js/bootstrap-switch.min.js',
              'tionix/js/bootstrapSwitcherField.js')


class BootstrapSliderInput(BootstrapInputMixin, forms.TextInput):
    """Slider widget"""

    def __init__(self, attrs=None):
        attrs = attrs or {}
        attrs.update(**{
            'data-provide': 'slider'
        })
        super(BootstrapSliderInput, self).__init__(attrs=attrs)

    def get_value_for_render(self, value):
        return value

    def render(self, name, value, attrs=None):
        if value:
            self.attrs['data-slider-value'] = self.get_value_for_render(value)
        return super(BootstrapSliderInput, self).render(name, value, attrs)

    class Media(object):
        css = {
            'all': ('tionix/css/bootstrap-slider.min.css', )
        }

        js = ('tionix/js/BootstrapSliderField.js', )


class AutoGenNameFieldMixin(object):

    empty_values = forms.CharField.empty_values
    max_length = None
    help_text = _('If field is not filled in the object name will be '
                  'generated automatically.')

    def __init__(self, *args, **kwargs):
        assert isinstance(self, forms.CharField)
        kwargs.pop('required', None)
        help_text = kwargs.pop('help_text', None) or self.help_text
        super(AutoGenNameFieldMixin, self).__init__(required=False,
                                                    help_text=help_text,
                                                    *args,
                                                    **kwargs)

    def to_python(self, value):
        """Returns a Unicode object."""
        if value in self.empty_values:
            return gen_uuid_object_name(self.max_length)
        return smart_text(value)


class AutoGenNameCharField(AutoGenNameFieldMixin, forms.CharField):
    pass


class AutoGenNameRegexField(AutoGenNameFieldMixin, forms.RegexField):
    pass


class NullableCharField(forms.CharField):
    """
    If field has empty value is None
    """
    def to_python(self, value):
        if value in self.empty_values:
            return None
        return smart_text(value)


class TnxFileWidget(forms.ClearableFileInput):
    template_with_initial = (
        u'%(initial_text)s: <span>%(initial)s</span>'
        u'<input type="hidden" name="initial_path" value="%(initial_path)s">'
        u'%(clear_template)s<br />%(input_text)s: %(input)s'
    )

    def is_initial(self, value):
        """
        Return whether value is considered to be initial value.
        """
        return bool(value and hasattr(value, 'file'))

    def get_template_substitution_values(self, value):
        """
        Return value-related substitutions.
        """
        return {
            'initial': conditional_escape(value),
            'initial_path': conditional_escape(value.name),
        }
