# coding: utf-8

import functools
import uuid
from datetime import datetime

import iso8601
import pytz

from django import template
from django.conf import settings
from django.core.cache import cache
from django.utils import decorators
from django.utils.translation import pgettext_lazy, ugettext_lazy as _
from horizon.tables import FilterAction
from horizon.utils.memoized import memoized

TRANSLATIONS = (
    _("Identity"),
    _("Log In"),
    _('Sign In'),
    _('Action Log'),
    _('User Name'),
    _('Compute'),
    _('TIONIX'),
    _('Migrate Host'),
    pgettext_lazy("Admin state of a Network", u"UP"),
    pgettext_lazy("Admin state of a Network", u"DOWN"),
    _("Download Juju Environment File"),
    _("Flavor's disk is too small for requested image. Flavor disk "
      "is %(flavor_size)i bytes, image is %(image_size)i bytes."),
    _("Configuration Drive"),
    _("Launch Instance"),
    _("Associate Floating IP"),
    _("Disassociate Floating IP"),
    _("Console"),
    _("Edit Instance"),
    _("Rebuild Instance"),
    _("Resize Instance"),
    _("To:"),
    _("""
      QoS Specs can be associated with volume types.
      It is used to map to a set of quality of service capabilities requested
      by the volume owner. This is equivalent to the
      <tt>cinder qos-create</tt> command. Once the QoS Spec gets created,
      click the \"Manage Specs\" button to manage the key-value specs for \
the QoS Spec.
      <br>
      <br>
      Each QoS Specs entity will have a \"Consumer\" value which indicates \
where the
      administrator would like the QoS policy to be enforced. This \
value can be \"front-end\"
      (Nova Compute), \"back-end\" (Cinder back-end), or \"both\".\n  """),
    _("back-end"),
    _("front-end"),
    _("both"),
)


def get_filter_value_by_dict(values_dict, not_found_none=True):
    """Return function to get key as filtering value from dict by filtering
    string
    :param values_dict: dict with values.
    :param not_found_none: flag to use None if key is not found, else
    filtering string.
    """

    def _inner(value):
        new_value = None if not_found_none else value
        for key, val in values_dict.iteritems():
            if value.lower() in val.lower():
                new_value = key
                break

        return unicode(new_value)

    return _inner


def get_yes_no_filter_value():
    yes_no_dict = {
        True: _('Yes'),
        False: _('No'),
    }

    return get_filter_value_by_dict(yes_no_dict)


def memoize_by_keyword_arg(kw_keys):
    """Memoize a function using the list of keyword argument name as its key.

    Wrap a function so that results for any keyword argument tuple are stored
    in 'inner_cache'. Note that the keyword args to the function must be usable
    as dictionary keys. Keys store in django cache

    :param kw_keys: List of keyword arguments names. The values are used
                    for generating the key in the cache.
    """

    inner_cache = {}

    unique_cache_id = str(uuid.uuid4())

    def _decorator(func):
        @functools.wraps(func, assigned=decorators.available_attrs(func))
        def wrapper(*args, **kwargs):
            mem_args = [kwargs[key] for key in kw_keys if key in kwargs]
            mem_args = '__'.join(str(mem_arg) for mem_arg in mem_args)
            mem_args_cache = cache.get(mem_args) or []
            if not mem_args:
                return func(*args, **kwargs)
            if unique_cache_id in mem_args_cache and mem_args in inner_cache:
                return inner_cache[mem_args]
            result = func(*args, **kwargs)
            inner_cache[mem_args] = result
            mem_args_cache.append(unique_cache_id)
            cache.set(mem_args, mem_args_cache, settings.SESSION_TIMEOUT)
            return result

        return wrapper

    return _decorator


def generate_timestamp_name(object_name):
    return u'{object_name}_{date_time}'.format(
        object_name=object_name,
        date_time=datetime.now().strftime('%Y-%m-%d_%H:%M:%S')
    )


def parse_datetime(value):
    try:
        date = iso8601.parse_date(value)
    except (iso8601.ParseError, TypeError):
        date = None
    return date


def get_datetime_value(date, date_format='%d.%m.%Y %H:%M'):
    if not isinstance(date, datetime):
        try:
            date = datetime.strptime(date, date_format)
        except (TypeError, ValueError):
            date = parse_datetime(date)

    return date.strftime(date_format) if date else ''


def add_kwargs(func, **kwargs):
    @functools.wraps(func)
    def wrapped(*args):
        return func(*args, **kwargs)

    return wrapped


def get_http_referer_path(request):
    """
    Return http referer path
    :param request: django request
    :return:
    """
    result = ''
    http_host = request.META['HTTP_HOST'] + settings.WEBROOT
    http_referer = request.META['HTTP_REFERER']
    if http_host in http_referer:
        result = http_referer.split(http_host)[1]
        if not result.startswith('/'):
            result = '/%s' % result
    return result


def get_full_match_filter_by_dict(values_dict, not_found_none=True):
    """Return function to get key as filtering value from dict by filtering
    string
    :param values_dict: dict with values.
    :param not_found_none: flag to use None if key is not found, else
    filtering string.
    """

    def _inner(value):
        new_value = None if not_found_none else value
        for key, val in values_dict.iteritems():
            if value.lower() == val.lower():
                new_value = key
                break

        return unicode(new_value)

    return _inner


def get_datetime_from_string(date_string):

    string_formats = [
        "%d.%m.%Y %H:%M:%S", "%Y-%m-%dT%H:%M:%S.%f", "%Y-%m-%d %H:%M:%S.%f",
        "%Y-%m-%dT%H:%M:%S", "%Y-%m-%d %H:%M:%S", "%Y-%m-%dT%H:%M:%SZ",
        "%Y-%m-%dT%H:%M:%S.%fZ", "%Y-%m-%d %H:%M:%S.%fZ",
        "%Y-%m-%d %H:%M:%SZ",
    ]

    date = None

    for string_format in string_formats:
        try:
            date = datetime.strptime(date_string, string_format)
        except (TypeError, ValueError):
            continue
        break
    return date


def get_resource_template(total, used):
    template_name = "tionix_dashboard/_resource_limit.html"

    context = {
        "resource_value": int(float(used) / total * 100)
    }
    return template.loader.render_to_string(template_name, context)


@memoized
def get_table_filter_action(table):
    """Retrieve first table's filtering action"""
    filter_action = None
    for action in table.get_table_actions():
        if isinstance(action, FilterAction):
            filter_action = action
            break
    return filter_action


def gen_uuid_object_name(max_length=None):
    """
    Generates uuid name
    :param max_length: Max letters count
    :return:
    """
    return unicode(uuid.uuid4())[:max_length]


def dict_fetchall(cursor):
    """
    Returns all rows from a cursor as a dict
    :param cursor: django db cursor
    :return:
    """
    desc = cursor.description
    return [
        dict(zip([col[0] for col in desc], row))
        for row in cursor.fetchall()
    ]


def get_sorted_timezones():
    d = datetime.now()
    zones = [(tz, pytz.timezone(tz).localize(d).strftime('%z'))
             for tz in pytz.common_timezones]
    zones.sort(key=lambda zone: int(zone[1]))
    return zones


def get_load_member_step_slug(request):
    return request.META.get('HTTP_X_HORIZON_LOAD_MEMBERS_STEP', '')
