# coding: utf-8
# pylint: disable=no-member

import re
from collections import OrderedDict

import logging
from django import shortcuts, http
from django.core import urlresolvers
from django.http.response import HttpResponseRedirectBase
from django.template.response import TemplateResponse

from horizon import tabs as horizon_tabs, forms, exceptions
from horizon.utils import memoized
from horizon.utils.functions import get_page_size

from tionix_dashboard.helpers.utils import (
    get_table_filter_action, get_load_member_step_slug)
from tionix_dashboard.helpers.workflows import LazyMembershipAction

logger = logging.getLogger(__name__)


class FilterStringModifierMixin(object):
    """
    Modifiers filter string before filtering
    """
    escape_fields = [u'name', ]
    escape_enabled = True
    is_last_page = False

    def get_filters(self, filters=None, include_not_api=False,
                    with_marker=True):
        filters = filters or {}
        filter_action = get_table_filter_action(self.table)
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field) or include_not_api:
                filter_string = self.table.get_filter_string()
                drop_marker = self.update_actual_filter(
                    filter_field, filter_string)
                search_action = self.request.POST.get('search_action', [])
                if (drop_marker and
                        'pagination' not in search_action and
                        with_marker):
                    filters['marker'] = None
                if (getattr(filter_action, 'escape_enabled', True) and
                        filter_field in self.escape_fields):
                    filter_string = re.escape(filter_string)
                if filter_field and filter_string:
                    filter_string_modifier = getattr(filter_action,
                                                     'filter_string_modifiers',
                                                     None)
                    if filter_string_modifier:
                        modifier = filter_action.filter_string_modifiers.get(
                            filter_field
                        )
                        if modifier is not None:
                            filter_string = modifier(filter_string)
                    filters[filter_field] = filter_string
        return filters

    # noinspection PyProtectedMember
    def update_actual_filter(self, filter_field, filter_string):
        actual_filter_field = getattr(self.table._meta, 'filter_field', '')
        actual_filter_string = getattr(self.table._meta, 'filter_string', '')
        self.table._meta.filter_field = filter_field
        self.table._meta.filter_string = filter_string
        if (actual_filter_field != filter_field or
                actual_filter_string != filter_string):
            self.is_last_page = False
            return True


# pylint: disable=expression-not-assigned
class ServerSortMultiSelect(object):
    object_ids_param = 'object_ids'
    sort_dir_param = 'sort_dir'
    sort_key_param = 'sort_key'
    redirect_param_name = 'is_redirect'

    params_tuple = (
        object_ids_param,
        sort_dir_param,
        sort_key_param
    )

    def handler_server_params(self, request):
        if (request.method == 'POST' or
                self.redirect_param_name in request.session):
            session_dict = {i: request.POST.get(i)
                            for i in self.params_tuple[1:]
                            if i in request.POST}
            object_ids = request.POST.getlist(self.object_ids_param)
            if object_ids:
                session_dict.update(**{self.object_ids_param: object_ids})

            request.session.update(session_dict)

        else:
            [request.session.pop(i)
             for i in self.params_tuple if i in request.session]

        if self.redirect_param_name in request.session:
            request.session.pop(self.redirect_param_name)

    # pylint: disable=no-member
    def get(self, request, *args, **kwargs):
        self.handler_server_params(request)

        _super = super(ServerSortMultiSelect, self)
        if hasattr(_super, 'get'):
            return _super.get(request, *args, **kwargs)


# noinspection PyUnusedLocal
class DTVTnxPaginationMixin(ServerSortMultiSelect):
    """
    Data table view mixin for offset/limit pagination
    """
    _more = True
    _prev = True

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more

    # noinspection PyProtectedMember
    def get_offset_limit(self):

        limit = get_page_size(self.request)

        marker = self.request.GET.get(
            self.table._meta.prev_pagination_param, '')

        if not marker:
            marker = self.request.GET.get(
                self.table._meta.pagination_param, '')

        offset = int(marker) if marker.isdigit() else 0

        return offset, limit

    @staticmethod
    def calc_last_marker(total, page_size):
        total_pages = total / page_size
        if total % page_size == 0:
            total_pages -= 1
        return total_pages * page_size

    def post(self, request, *args, **kwargs):
        # If the server side table filter changed then go back to the first
        # page of data. Otherwise GET and POST handling are the same.
        # FIXME: why TnxInstancesTab not have handle_server_filter
        if (hasattr(self, 'handle_server_filter') and
                self.handle_server_filter(request)):
            self.handler_server_params(request)
            request.session[self.redirect_param_name] = True
            return shortcuts.redirect(self.get_table().get_absolute_url())
        return self.get(request, *args, **kwargs)

    def set_table_footer(self, offset, limit, total_count):
        self.table.total_count = total_count

        if offset == 0:
            self._prev = False
        if limit + offset >= total_count:
            self._more = False

        # Fist, last pages identifiers
        self.table.first_marker, self.table.last_marker = (
            0, self.calc_last_marker(total_count, limit)
        )


class ServerFilterTabMixin(FilterStringModifierMixin):
    """
    Server filter for tables in tab
    """

    table_classes = None

    # noinspection PyProtectedMember
    def __init__(self, tab_group, request):
        assert isinstance(self, horizon_tabs.Tab)
        super(ServerFilterTabMixin, self).__init__(tab_group, request)
        if not self.table_classes:
            class_name = self.__class__.__name__
            raise NotImplementedError("You must define a table_class "
                                      "attribute on %s" % class_name)
        # Instantiate our table classes but don't assign data yet
        kwargs = tab_group.kwargs
        kwargs.update(tab_group=tab_group)
        table_instances = [(table._meta.name, table(request, **kwargs))
                           for table in self.table_classes]
        self._tables = OrderedDict(table_instances)
        self._table_data_loaded = False

    @property
    def table(self):
        return self._tables.values()[0]

    def post(self, request, *args, **kwargs):
        # If the server side table filter changed then go back to the first
        # page of data. Otherwise GET and POST handling are the same.
        if self.handle_server_filter(request):
            return shortcuts.redirect(self.table.get_absolute_url())
        super(ServerFilterTabMixin, self).post(request, *args, **kwargs)

    def get_server_filter_info(self, request):
        filter_action = get_table_filter_action(self.table)
        if filter_action is None or filter_action.filter_type != 'server':
            return None
        param_name = filter_action.get_param_name()
        filter_string = request.POST.get(param_name)
        filter_string_session = request.session.get(param_name, "")
        changed = (filter_string is not None and
                   filter_string != filter_string_session)
        if filter_string is None:
            filter_string = filter_string_session
        filter_field_param = param_name + '_field'
        filter_field = request.POST.get(filter_field_param)
        filter_field_session = request.session.get(filter_field_param)
        if filter_field is None and filter_field_session is not None:
            filter_field = filter_field_session
        filter_info = {
            'action': filter_action,
            'value_param': param_name,
            'value': filter_string,
            'field_param': filter_field_param,
            'field': filter_field,
            'changed': changed
        }
        return filter_info

    def handle_server_filter(self, request):
        """
        Update the table server filter information in the session and
        determine if the filter has been changed.
        """
        filter_info = self.get_server_filter_info(request)
        if filter_info is None:
            return False
        request.session[filter_info['value_param']] = filter_info['value']
        if filter_info['field_param']:
            request.session[filter_info['field_param']] = filter_info['field']
        return filter_info['changed']

    def update_server_filter_action(self):
        """
        Update the table server side filter action based on the current
        filter. The filter info may be stored in the session and this will
        restore it.
        """
        filter_info = self.get_server_filter_info(self.request)
        if filter_info is not None:
            action = filter_info['action']
            setattr(action, 'filter_string', filter_info['value'])
            if filter_info['field_param']:
                setattr(action, 'filter_field', filter_info['field'])

    def get_context_data(self, request, **kwargs):
        self.update_server_filter_action()
        return super(ServerFilterTabMixin, self).get_context_data(request,
                                                                  **kwargs)


class ServerFilterViewMixin(object):
    """
    Server filter for tabbedviews
    """
    def post(self, request, *args, **kwargs):
        if 'search_action' in request.POST and not request.is_ajax():
            if 'tab' in request.GET:
                targetslug = request.GET['tab'].split('__')[1]
            else:
                targetslug = None
            tabs = self.get_tabs(self.request, **self.kwargs).get_tabs()
            for tab in tabs:
                if targetslug is None or tab.slug == targetslug:
                    response = tab.post(request, *args, **kwargs)
                    if isinstance(response, HttpResponseRedirectBase):
                        return response
                    break

        return super(ServerFilterViewMixin, self).post(request,
                                                       *args, **kwargs)


# noinspection PyUnusedLocal
class ServerPaginationMixin(object):
    """
    Pagination in instance view
    """
    _prev = False
    _more = False
    instances = []
    search_opts = {}
    sort_dir = 'desc'
    is_last_page = False

    def has_more_data(self, table):
        return self._more

    def has_prev_data(self, table):
        return self._prev

    # noinspection PyProtectedMember
    def marker_init(self):
        prev_marker = self.request.GET.get(
            self.table._meta.prev_pagination_param, None)

        first_marker = last_marker = 0
        marker = None
        if prev_marker is not None:
            if prev_marker != str(last_marker):
                marker = prev_marker
                self.sort_dir = 'asc'
        else:
            marker = self.request.GET.get(
                self.table._meta.pagination_param, None)
            if marker == str(last_marker):
                marker = None
                self.sort_dir = 'asc'
                self.is_last_page = True

        self.search_opts = self.get_filters({'marker': marker,
                                             'paginate': True})

        self.table.first_marker = first_marker
        self.table.last_marker = last_marker


class ServerPaginationViewMixin(ServerPaginationMixin):
    def post(self, request, *args, **kwargs):
        # redirect if search field reset
        # escape redirect if pagination use
        if 'search_action' in request.POST:
            search_action = request.POST['search_action']
            if (search_action == 'restart' or
                    (search_action == 'button' and
                     self.handle_server_filter(request))):
                return shortcuts.redirect(self.get_table().get_absolute_url())

        return self.get(request, *args, **kwargs)


class RefererRedirectMixin(object):
    """View redirect to http_referer from request"""

    def get_redirect_url(self):
        return self.request.META['HTTP_REFERER']

    def get_success_url(self):
        return self.get_redirect_url()


class LinkBatchViewMixin(object):
    request = None
    kwargs = None
    submit_url = None
    modal_header = None

    @memoized.memoized_method
    def get_object_ids(self):
        ids = []
        if self.kwargs:
            ids = self.kwargs.get('ids')
        if not ids:
            if self.request.method == 'POST':
                ids = self.request.POST.getlist('id', [])
            else:
                ids = self.request.GET.getlist('id', [])

        return ids

    def get_submit_url(self):
        submit_url = self.submit_url
        try:
            submit_url = urlresolvers.reverse(submit_url)
        except urlresolvers.NoReverseMatch:
            pass
        return submit_url

    # pylint: disable=not-callable
    def get_modal_header(self):
        modal_header = self.modal_header
        if callable(self.modal_header):
            count = len(self.get_object_ids())
            modal_header = modal_header(count)
        return modal_header

    def get_initial(self):
        initial = super(LinkBatchViewMixin, self).get_initial()
        initial['id'] = self.get_object_ids()
        return initial

    def get_context_data(self, **kwargs):
        """Returns the template context, including the workflow class.

        This method should be overridden in subclasses to provide additional
        context data to the template.
        """
        context = super(LinkBatchViewMixin, self).get_context_data(
            **kwargs
        )
        context['modal_header'] = self.get_modal_header()
        context['submit_url'] = self.get_submit_url()
        return context


class LinkBatchFormView(LinkBatchViewMixin, forms.ModalFormView):

    id_field_name = 'id'

    # pylint: disable=arguments-differ
    def get_form(self, *args, **kwargs):
        """Returns an instance of the form to be used in this view."""
        form = super(LinkBatchFormView, self).get_form(*args, **kwargs)
        ids = self.get_object_ids()
        form.fields[self.id_field_name] = forms.MultipleChoiceField(
            widget=forms.MultipleHiddenInput(),
            required=False,
            initial=ids,
            choices=[(obj_id, obj_id) for obj_id in ids]
        )

        return form

    def form_valid(self, form):
        form.cleaned_data.pop(self.id_field_name, None)
        return super(LinkBatchFormView, self).form_valid(form)


class LinkBatchWorkflowViewMixin(LinkBatchViewMixin):

    def get_workflow(self):
        """Returns the instantiated workflow class."""
        workflow = super(LinkBatchWorkflowViewMixin, self).get_workflow()
        for step in workflow.steps:
            if 'id' not in step.action.fields:
                ids = self.get_object_ids()
                step.action.fields['id'] = forms.MultipleChoiceField(
                    widget=forms.MultipleHiddenInput(),
                    required=False,
                    initial=ids,
                    choices=[(obj_id, obj_id) for obj_id in ids]
                )
            break
        return workflow


class WorkflowWithLazyMembers(object):
    """
    Allow to load members in steps dynamically
    """

    request = None

    # noinspection PyUnresolvedReferences
    def get_members_data_for_step(self, request, workflow, step):
        data = {}
        try:
            data['result'] = step.action.populate_members(request,
                                                          workflow,
                                                          **self.kwargs)
        except exceptions.WorkflowError as err:
            data['error'] = unicode(err.message)
        return data

    def get(self, request, *args, **kwargs):

        # There are valid LOAD_MEMBER_STEP* headers, so load members
        # for the specified steps and return results.
        load_member_step_slug = get_load_member_step_slug(request)
        if load_member_step_slug:
            context = self.get_context_data(**kwargs)
            workflow = context[self.context_object_name]
            load_member_step = workflow.get_step(load_member_step_slug)
            if load_member_step:
                data = self.get_members_data_for_step(request,
                                                      workflow,
                                                      load_member_step)
                return http.JsonResponse(data, content_type="application/json")
        else:
            request.session.pop('selected_members', None)

        return super(WorkflowWithLazyMembers, self).get(request,
                                                        *args,
                                                        **kwargs)

    def post(self, request, *args, **kwargs):
        """Handler for HTTP POST requests."""
        response = super(WorkflowWithLazyMembers, self).post(request,
                                                             *args,
                                                             **kwargs)
        if isinstance(response, TemplateResponse):
            # Form is not valid, so save in session selected members
            workflow = response.context_data.get('workflow')
            if workflow and not workflow.is_valid():
                selected_members = {}
                for step in workflow.steps:
                    action = step.action
                    if isinstance(action, LazyMembershipAction):
                        field_name = action.get_member_field_name('')
                        selected_members.update({
                            key: value
                            for key, value in workflow.context.items()
                            if value and key.startswith(field_name)})
                request.session['selected_members'] = selected_members
            else:
                request.session.pop('selected_members', None)
        return response
