# coding: utf-8

from operator import attrgetter

import json
import logging

import collections

from django.utils.translation import ugettext_lazy as _
from openstack_dashboard import api

from tionix_client.journal.constants import ObjectType

from tionix_dashboard.dashboards.helpers import get_common_journal_data
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    get_username)
from tionix_dashboard.helpers.vm_constants import DELETED
from tionix_dashboard.journal.utils import get_instances, get_tenants

logger = logging.getLogger(__name__)


class Journal(object):

    sort_dir = 'desc'
    sort_key = 'event_time'

    def __init__(self, request):

        self.request = request
        self.is_domain_admin = api.keystone.is_domain_admin(request)

    # noinspection PyMethodMayBeStatic
    def _get_search_opts(self, search_opts=None):
        search_opts = search_opts or {}
        return search_opts

    def get_data(self, search_opts=None):
        search_opts = self._get_search_opts(search_opts)

        return self.format_data(
            get_common_journal_data(self.request, search_opts=search_opts)
        )

    # noinspection PyMethodMayBeStatic
    def format_data(self, data):
        for row in data:
            action_arguments = getattr(row, 'action_arguments', None)
            message = getattr(row, 'message', None)
            if action_arguments and not message:
                try:
                    params = json.loads(row.action_arguments)
                except ValueError:
                    continue
                row.message = _(u'Action parameters: {0}').format(u', '.join(
                    [u'{0}={1}'.format(k, v) for k, v in params.items()]))
        return data

    def list(self, search_opts=None, sort_dir=None, sort_key=None):
        sort_dir = sort_dir or self.sort_dir
        sort_key = sort_key or self.sort_key
        is_reversed = sort_dir == 'desc'

        data = self.get_data(search_opts)
        data.sort(key=attrgetter(sort_key), reverse=is_reversed)

        for num, journal in enumerate(data):
            journal.id = num

        return data


class ObjectJournal(Journal):

    object_type = None

    def __init__(self, request, objects):
        super(ObjectJournal, self).__init__(request)

        if not isinstance(objects, collections.Iterable):
            objects = [objects]

        self.objects = objects
        self.objects_ids = [obj.id for obj in objects]

    def _get_search_opts(self, search_opts=None):
        search_opts = search_opts or {}

        search_opts.update({'all': True, 'object_type': self.object_type})

        if 'project_id' not in search_opts:
            search_opts.update({'object_ids': self.objects_ids})

        return search_opts


class HypervisorJournal(ObjectJournal):

    object_type = ObjectType.Node


class ServerJournal(ObjectJournal):

    object_type = ObjectType.Server

    def format_data(self, data):
        data = super(ServerJournal, self).format_data(data)
        obj_dict = {obj.id: (obj.name, obj.status.lower() == DELETED)
                    for obj in self.objects}

        for row in data:
            row_id = row.object_id
            row.object_name, row.object_deleted = (
                obj_dict.get(row_id, (row_id, True))
            )
            row.user = row.user_name or get_username(self.request, row.user_id)

        return data


class ProjectJournal(ObjectJournal):

    object_type = ObjectType.Project

    def __init__(self, request, object_ids, include_instances=False):
        super(ProjectJournal, self).__init__(request, object_ids)
        self.instances = self.get_instances() if include_instances else None

    def format_data(self, data):
        data = super(ProjectJournal, self).format_data(data)
        obj_dict = {obj.id: obj.name for obj in self.objects}

        for row in data:
            row_id = row.object_id
            row.object_deleted = False
            row.object_name = obj_dict.get(row_id, row_id)
            row.user = row.user_name or get_username(self.request, row.user_id)

        return data

    def get_instances(self):
        instances = []

        for object_id in self.objects_ids:
            instances += get_instances(self.request,
                                       object_id,
                                       with_deleted=self.is_domain_admin)

        return instances

    def get_data(self, search_opts=None):
        data = super(ProjectJournal, self).get_data(search_opts)

        if self.instances:
            server_journal = ServerJournal(self.request, self.instances)
            for object_id in self.objects_ids:
                data += server_journal.get_data(
                    search_opts={'project_id': object_id}
                )

        return data


class UserJournal(Journal):
    def __init__(self, request, user_id):
        super(UserJournal, self).__init__(request)

        self.user_id = user_id

    def _get_search_opts(self, search_opts=None):
        search_opts = search_opts or {}
        search_opts.update({'user_id': self.user_id})
        return search_opts

    def format_data(self, data):
        data = super(UserJournal, self).format_data(data)
        if self.is_domain_admin:
            search_opts = {'all_tenants': True}
        else:
            search_opts = {'project_id': self.request.user.project_id}

        instances_dict = {
            instance.id: (instance.name, instance.status.lower() == DELETED)
            for instance in get_instances(self.request,
                                          search_opts=search_opts,
                                          with_deleted=self.is_domain_admin)
        }

        tenants_dict = {
            tenant.id: tenant.name
            for tenant in get_tenants(self.request,
                                      is_admin=self.is_domain_admin)
        }

        for row in data:
            row_id = row.object_id
            row.object_deleted = False
            if row.object_type == ObjectType.Server:
                row.object_name, row.object_deleted = (
                    instances_dict.get(row_id, (row_id, True))
                )
            elif row.object_type == ObjectType.Project:
                row.object_name = tenants_dict.get(row_id, row_id)
        return data
