# coding: utf-8
import functools

from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import tables
from horizon.utils import filters

from openstack_dashboard import api

from tionix_dashboard.dashboards.admin.hypervisors import (
    constants as hypervisor_constants)
from tionix_dashboard.dashboards.identity.projects import (
    constants as project_constants)
from tionix_dashboard.dashboards.project.project_instances import (
    constants as instances_constants)
from tionix_dashboard.helpers.actions import TnxFilterAction
from tionix_dashboard.helpers.tables import TnxPaginationTabDataTable


JOURNAL_TYPES_DISPLAY_CHOICES = (
    ("server", _('Instance')),
    ("node", _('Hypervisor')),
    ("project", _('Project')),
)


ACTION_DISPLAY_CHOICES = (
    ("dashboard.user.login", pgettext_lazy("Action log", "Log in")),
    ("dashboard.user.logout", pgettext_lazy("Action log", "Log out")),
    ("keystone.created.project",
     pgettext_lazy("Action log", "Create project")),
    ("keystone.created.role_assignment",
     pgettext_lazy("Action log", "Assign role")),
    ("keystone.deleted.project",
     pgettext_lazy("Action log", "Delete project")),
    ("keystone.deleted.role_assignment",
     pgettext_lazy("Action log", "Unassign role")),
    ("keystone.updated.project",
     pgettext_lazy("Action log", "Update project")),
    ("node_control.attach_host", pgettext_lazy("Action log", "Attach host")),
    ("node_control.evacuate", pgettext_lazy("Action log", "Evacuation")),
    ("node_control.move_from_reserve",
     pgettext_lazy("Action log", "Move from reserve")),
    ("node_control.power_off", pgettext_lazy("Action log", "Power off")),
    ("node_control.power_on", pgettext_lazy("Action log", "Power on")),
    ("node_control.reset", pgettext_lazy("Action log", "Restart")),
    ("nova.create.servers", pgettext_lazy("Action log", "Create instance")),
    ("nova.create.servers.metadata",
     pgettext_lazy("Action log", "Set metadata")),
    ("nova.create.servers.os_interface",
     pgettext_lazy("Action log", "Attach interface")),
    ("nova.create.servers.os_volume_attachments",
     pgettext_lazy("Action log", "Attach volume")),
    ("nova.create.servers.remote_consoles",
     pgettext_lazy("Action log", "Create remote console")),
    ("nova.create.tnx_hypervisors", pgettext_lazy("Action log", "Create")),
    ("nova.create.tnx_tenants",
     pgettext_lazy("Action log", "Create VDI project")),
    ("nova.delete.servers", pgettext_lazy("Action log", "Delete instance")),
    ("nova.delete.servers.metadata",
     pgettext_lazy("Action log", "Delete metadata")),
    ("nova.delete.servers.os_interface",
     pgettext_lazy("Action log", "Detach interface")),
    ("nova.delete.tnx_hypervisors", pgettext_lazy("Action log", "Delete")),
    ("nova.delete.tnx_tenants",
     pgettext_lazy("Action log", "Delete VDI project")),
    ("nova.servers.add_scsi_meta",
     pgettext_lazy("Action log", "Set SCSI metadata")),
    ("nova.update.os_aggregates.add_host",
     pgettext_lazy("Action log", "Add host to aggregate")),
    ("nova.update.os_aggregates.remove_host",
     pgettext_lazy("Action log", "Remove host from aggregate")),
    ("nova.update.os_quota_sets",
     pgettext_lazy("Action log", "Update project quotas")),
    ("nova.update.servers",
     pgettext_lazy("Action log", "Update instance info")),
    ("nova.update.servers.add_user",
     pgettext_lazy("Action log", "Assign user")),
    ("nova.update.servers.add_user_group",
     pgettext_lazy("Action log", "Assign group")),
    ("nova.update.servers.addfixedip",
     pgettext_lazy("Action log", "Add fixed ip")),
    ("nova.update.servers.addfloatingip",
     pgettext_lazy("Action log", "Associate floating IP")),
    ("nova.update.servers.addsecuritygroup",
     pgettext_lazy("Action log", "Add security group")),
    ("nova.update.servers.assign_backup_templates",
     pgettext_lazy("Action log", "Assign backup templates")),
    ("nova.update.servers.balance",
     pgettext_lazy("Action log", "Balance instance")),
    ("nova.update.servers.changepassword",
     pgettext_lazy("Action log", "Change Password")),
    ("nova.update.servers.confirmresize",
     pgettext_lazy("Action log", "Confirm resize/migrate")),
    ("nova.update.servers.createbackup",
     pgettext_lazy("Action log", "Create backup")),
    ("nova.update.servers.createimage",
     pgettext_lazy("Action log", "Create image")),
    ("nova.update.servers.evacuate",
     pgettext_lazy("Action log", "Evacuation")),
    ("nova.update.servers.forcedelete",
     pgettext_lazy("Action log", "Force delete")),
    ("nova.update.servers.injectnetworkinfo",
     pgettext_lazy("Action log", "Inject network info")),
    ("nova.update.servers.lock", pgettext_lazy("Action log", "Lock instance")),
    ("nova.update.servers.migrate",
     pgettext_lazy("Action log", "Migrate instance")),
    ("nova.update.servers.movable",
     pgettext_lazy("Action log", "Change movable flag")),
    ("nova.update.servers.os_getconsoleoutput",
     pgettext_lazy("Action log", "Get output console")),
    ("nova.update.servers.os_getrdpconsole",
     pgettext_lazy("Action log", "Get RDP console")),
    ("nova.update.servers.os_getserialconsole",
     pgettext_lazy("Action log", "Get serial console")),
    ("nova.update.servers.os_getspiceconsole",
     pgettext_lazy("Action log", "Get SPICE console")),
    ("nova.update.servers.os_getvncconsole",
     pgettext_lazy("Action log", "Get VNC console")),
    ("nova.update.servers.os_migratelive",
     pgettext_lazy("Action log", "Live migrate instance")),
    ("nova.update.servers.os_resetstate",
     pgettext_lazy("Action log", "Reset state")),
    ("nova.update.servers.os_start",
     pgettext_lazy("Action log", "Start instance")),
    ("nova.update.servers.os_stop",
     pgettext_lazy("Action log", "Shut off instance")),
    ("nova.update.servers.pause",
     pgettext_lazy("Action log", "Pause instance")),
    ("nova.update.servers.reboot",
     pgettext_lazy("Action log", "Reboot instance")),
    ("nova.update.servers.rebuild",
     pgettext_lazy("Action log", "Rebuild instance")),
    ("nova.update.servers.recover",
     pgettext_lazy("Action log", "Recover instance")),
    ("nova.update.servers.remove_user",
     pgettext_lazy("Action log", "Unassign user")),
    ("nova.update.servers.remove_user_group",
     pgettext_lazy("Action log", "Unassign group")),
    ("nova.update.servers.removefixedip",
     pgettext_lazy("Action log", "Remove fixed ip")),
    ("nova.update.servers.removefloatingip",
     pgettext_lazy("Action log", "Disassociate floating IP")),
    ("nova.update.servers.removesecuritygroup",
     pgettext_lazy("Action log", "Remove security group")),
    ("nova.update.servers.repair",
     pgettext_lazy("Action log", "Repair instance")),
    ("nova.update.servers.rescue", pgettext_lazy("Action log", "Rescue")),
    ("nova.update.servers.reset_with_reboot",
     pgettext_lazy("Action log", "Reset and reboot instance")),
    ("nova.update.servers.resetnetwork",
     pgettext_lazy("Action log", "Reset network")),
    ("nova.update.servers.resize",
     pgettext_lazy("Action log", "Resize instance")),
    ("nova.update.servers.restore", pgettext_lazy("Action log", "Restore")),
    ("nova.update.servers.resume",
     pgettext_lazy("Action log", "Resume instance")),
    ("nova.update.servers.revertresize",
     pgettext_lazy("Action log", "Revert resize/migrate")),
    ("nova.update.servers.shelve",
     pgettext_lazy("Action log", "Shelve instance")),
    ("nova.update.servers.shelveoffload",
     pgettext_lazy("Action log", "Shelve offload")),
    ("nova.update.servers.suspend",
     pgettext_lazy("Action log", "Suspend instance")),
    ("nova.update.servers.unassign_backup_templates",
     pgettext_lazy("Action log", "Unassign backup templates")),
    ("nova.update.servers.unlock",
     pgettext_lazy("Action log", "Unlock instance")),
    ("nova.update.servers.unpause",
     pgettext_lazy("Action log", "Unpause instance")),
    ("nova.update.servers.unrescue", pgettext_lazy("Action log", "Unrescue")),
    ("nova.update.servers.unshelve",
     pgettext_lazy("Action log", "Unshelve instance")),
    ("nova.update.tnx_hypervisors", pgettext_lazy("Action log", "Update")),
    ("nova.update.tnx_hypervisors.assign_image",
     pgettext_lazy("Action log", "Assign image")),
    ("nova.update.tnx_hypervisors.assign_node_reserve",
     pgettext_lazy("Action log", "Assign reserve node")),
    ("nova.update.tnx_hypervisors.assign_power_device",
     pgettext_lazy("Action log", "Assign power control")),
    ("nova.update.tnx_hypervisors.attach_host",
     pgettext_lazy("Action log", "Attach host")),
    ("nova.update.tnx_hypervisors.detach_host",
     pgettext_lazy("Action log", "Detach host")),
    ("nova.update.tnx_hypervisors.power_manage.power_off",
     pgettext_lazy("Action log", "Power off")),
    ("nova.update.tnx_hypervisors.power_manage.power_on",
     pgettext_lazy("Action log", "Power on")),
    ("nova.update.tnx_hypervisors.power_manage.reset",
     pgettext_lazy("Action log", "Restart")),
    ("nova.update.tnx_hypervisors.reserve.from_reserve",
     pgettext_lazy("Action log", "Move from reserve")),
    ("nova.update.tnx_hypervisors.reserve.to_reserve",
     pgettext_lazy("Action log", "Move to reserve")),
    ("nova.update.tnx_hypervisors.set_default_power_device_for_node",
     pgettext_lazy("Action log", "Set default power control")),
    ("nova.update.tnx_hypervisors.set_power_device_port_for_node",
     pgettext_lazy("Action log", "Set power control port")),
    ("nova.update.tnx_hypervisors.unassign_power_device",
     pgettext_lazy("Action log", "Unassign power control")),
    ("nova.update.tnx_os_hosts.tnx_cc_evacuate",
     pgettext_lazy("Action log", "Evacuate instances from host")),
    ("nova.update.tnx_os_hosts.tnx_cc_shutdown",
     pgettext_lazy("Action log", "Shutdown host")),
    ("nova.update.tnx_tenants",
     pgettext_lazy("Action log", "Update VDI project")),
    ("nova.update.tnx_tenants.add_security_group",
     pgettext_lazy("Action log", "Add security group")),
    ("nova.update.tnx_tenants.delete_security_group",
     pgettext_lazy("Action log", "Remove security group")),
    ("nova.update.tnx_zabbix.create_host",
     pgettext_lazy("Action log", "Create Zabbix host")),
    ("nova.update.tnx_zabbix.delete_host",
     pgettext_lazy("Action log", "Delete Zabbix host")),
    ("nova.update.tnx_zabbix.setup_monitoring",
     pgettext_lazy("Action log", "Set up monitoring")),
    ("nova.update.tnx_zabbix.update_host",
     pgettext_lazy("Action log", "Update Zabbix host")),
    ("vdi_server.assign", pgettext_lazy("Action log", "Assign user")),
    ("vdi_server.get_vm_url", pgettext_lazy("Action log", "Get instance url")),
    ("vdi_server.request_vm",
     pgettext_lazy("Action log", "Request of an instance")),
)

JOURNAL_ACTION_DISPLAY_CHOICES = (
    hypervisor_constants.ACTION_DISPLAY_CHOICES +
    instances_constants.ACTION_DISPLAY_CHOICES +
    project_constants.ACTION_DISPLAY_CHOICES +
    ACTION_DISPLAY_CHOICES
)


STATUS_DISPLAY_CHOICES = (
    ("Success", _('Success')),
    ("Failure", _('Failure')),
    ("Unknown", _('Unknown'))
)


# noinspection PyAbstractClass
class JournalFilterAction(TnxFilterAction):
    """Journal filter"""
    filter_type = "server"

    filter_choices = [
        ('request_id', _("Request ID"), False),
        ('parent_request_id', _("Parent request"), False),
        ('action', _("Action"), False),
        ('start_time', _("Start time"), False),
        ('user', _("User"), False),
        ('object_type', _("Subject type"), False),
        ('object_name', _("Subject name"), False),
        ('status', _("Result"), False),
        ('message', _("Result details"), False),

    ]


class JournalBaseTable(TnxPaginationTabDataTable):
    request_id = tables.Column('request_id', verbose_name=_('Request ID'))
    parent_request_id = tables.Column('parent_request_id',
                                      verbose_name=_('Parent request'))
    action = tables.Column('action', verbose_name=_('Action'),
                           display_choices=JOURNAL_ACTION_DISPLAY_CHOICES)
    start_time = tables.Column('event_time', verbose_name=_('Start time'),
                               filters=[filters.parse_isotime])
    user = tables.Column('user', verbose_name=_('User'))
    object_type = tables.Column('object_type', verbose_name=_('Subject type'),
                                display_choices=JOURNAL_TYPES_DISPLAY_CHOICES)
    object_name = tables.Column('object_name', verbose_name=_('Subject name'))
    status = tables.Column('status', verbose_name=_('Result'),
                           display_choices=STATUS_DISPLAY_CHOICES)
    result = tables.Column('message', verbose_name=_('Result details'))

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(JournalBaseTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)

        is_admin = api.keystone.is_domain_admin(request)
        self.columns['object_name'].link = functools.partial(
            self.get_url_for_object, is_admin=is_admin
        )

    # noinspection PyUnusedLocal
    @staticmethod
    def get_url_for_object(obj, is_admin=False):
        return None
