# coding: utf-8

import logging.config
import os
from importlib import import_module

import pkg_resources
import horizon
import openstack_auth
import pymysql

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from tionix_client.settings.loader import Config

from tionix_dashboard.validators import validators_map


# use pymysql instead of MySQLdb driver
pymysql.install_as_MySQLdb()

MODULE_NAME = 'tionix_dashboard'

DEFAULT_CONF = {
    'LOG_LEVEL': 'INFO',
    'DB': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'tionix_dash',
        'USER': 'tionix',
        'PASSWORD': 'password',
        'HOST': 'localhost',
        'PORT': '3306',
    },
    'KEYSTONE': {
        'auth_version': '3.0',
        'auth_url': 'http://localhost:5000',
        'auth_user': 'admin',
        'auth_password': 'admin',
        'auth_tenant': 'admin',
        'compute_service_name': 'compute',
        'volume_service_name': 'volumev2',
        'network_service_name': 'network',
        'image_service_name': 'image',
        'identity_service_name': 'identity',
        'user_domain_name': 'default',
        'project_domain_name': 'default'
    },
    'NOVA_VERSION': 2.25,
    'NEUTRON_VERSION': 2,
    'GLANCE_VERSION': 1,
    'TRACEBACK_ENABLED': False,
}

immutable_variables = ['NOVA_VERSION']
unused_variables = ['CINDER_VERSION', 'RABBIT_QUEUES',
                    'CEILOMETER_SERVICE_NAME', 'NOVA_RABBIT_VHOST',
                    'KEYSTONE_RABBIT_VHOST', 'RAVEN']

base_config_path = '/etc/tionix/tionix.yaml'
module_config_path = '/etc/tionix/dashboard.yaml'

CONF = Config(
    config_files=[base_config_path, module_config_path],
    default_settings=DEFAULT_CONF,
    validators=validators_map,
    immutable_variables=immutable_variables,
    unused_variables=unused_variables,
    module_name=MODULE_NAME)

TNX_DASH_BASE_DIR = os.path.dirname(__file__)
TIONIX_DOCUMENTATION_LINK = 'http://docs.tionix.ru/'
TIONIX_DOCUMENTATION_DASH_SECTION = ('/tionix_modules/tionix-dash-docs/'
                                     'tionix_dashboard/functional.html')

ROOT_URLCONF = 'tionix_dashboard.dashboards.urls'


# pylint: disable=no-member
def get_help_url():
    help_url = TIONIX_DOCUMENTATION_LINK
    dash_version = None

    # noinspection PyBroadException
    try:
        dash_distribution = pkg_resources.get_distribution('tionix-dashboard')
        dash_version = dash_distribution.version
    except Exception:
        pass

    if dash_version is not None:
        dash_version = dash_version.split('.dev')[0]

        if not dash_version.endswith('0'):
            dash_version = '.'.join(dash_version.split('.')[:2])

        help_url = help_url + dash_version + TIONIX_DOCUMENTATION_DASH_SECTION

    return help_url


def populate_tnx_dashboards_static(installed_apps, horizon_config):
    from horizon.utils import file_discovery as fd

    for installed_app in installed_apps:
        installed_module = import_module(installed_app)
        installed_module_path = os.path.join(installed_module.__path__[0],
                                             'static/')
        sources, _mocks, _specs, _template = fd.discover_static_files(
            installed_module_path
        )
        horizon_config.setdefault('tionix_js_files', []).extend(sources)


# Enable Tionix theme
try:
    from tionix_dashboard_theme import *
except ImportError:
    pass

TIONIX_DASHBOARDS_APPS = [
    'tionix_dashboard.dashboards.project.access_and_security',
    'tionix_dashboard.dashboards.project.project_images',
    'tionix_dashboard.dashboards.project.project_instances',
    'tionix_dashboard.dashboards.project.network_topology',
    'tionix_dashboard.dashboards.project.project_networks',
    'tionix_dashboard.dashboards.project.project_routers',
    'tionix_dashboard.dashboards.project.project_volumes',
    'tionix_dashboard.dashboards.admin.overview',
    'tionix_dashboard.dashboards.admin.aggregates',
    'tionix_dashboard.dashboards.admin.flavors',
    'tionix_dashboard.dashboards.admin.hypervisors',
    'tionix_dashboard.dashboards.admin.metadata_defs',
    'tionix_dashboard.dashboards.admin.instances',
    'tionix_dashboard.dashboards.admin.volumes',
    'tionix_dashboard.dashboards.admin.images',
    'tionix_dashboard.dashboards.admin.networks',
    'tionix_dashboard.dashboards.admin.routers',
    'tionix_dashboard.dashboards.identity.domains',
    'tionix_dashboard.dashboards.identity.groups',
    'tionix_dashboard.dashboards.identity.projects',
    'tionix_dashboard.dashboards.identity.users',
    'tionix_dashboard.dashboards.settings.user',
    'tionix_dashboard.dashboards.tionix.balancing',
    'tionix_dashboard.dashboards.tionix.licenses',
    'tionix_dashboard.dashboards.tionix.infrastructure',
    'tionix_dashboard.dashboards.tionix.power_devices',
    'tionix_dashboard.dashboards.tionix.scheduler',
    'tionix_dashboard.dashboards.tionix.metrics',
    'tionix_dashboard.dashboards.tionix.vdi',
    'tionix_dashboard.dashboards.tionix.frames',
]

ADD_INSTALLED_APPS = [
    'tionix_dashboard',
    'tionix_dashboard.dashboards',
    'tionix_dashboard.management.commands.collectstatic',
] + TIONIX_DASHBOARDS_APPS

AUTHENTICATION_PLUGINS = [
    'tionix_dashboard.dashboards.auth_plugins.TnxPasswordPlugin',
    'tionix_dashboard.dashboards.auth_plugins.TnxTokenPlugin'
]

# Time out for checking licenses
MAX_LICENSE_TIME_OUT = 60

OPENSTACK_AUTH_BASE_DIR = os.path.dirname(openstack_auth.__file__)

HORIZON_BASE_DIR = os.path.dirname(horizon.__file__)

LOCALE_PATHS = [
    os.path.join(TNX_DASH_BASE_DIR, 'locale'),
    os.path.join(OPENSTACK_AUTH_BASE_DIR, 'locale'),
    os.path.join(HORIZON_BASE_DIR, 'locale'),
]

try:
    import tionix_dashboard_theme
except ImportError:
    pass
else:

    TIONIX_DASHBOARD_THEME_BASE_DIR = os.path.dirname(
        tionix_dashboard_theme.__file__
    )

    LOCALE_PATHS.append(os.path.join(TIONIX_DASHBOARD_THEME_BASE_DIR,
                                     'locale'))

try:
    import openstack_dashboard
except ImportError:
    pass
else:
    from openstack_dashboard.settings import MIDDLEWARE_CLASSES, HORIZON_CONFIG

    OS_BASE_DIR = os.path.dirname(openstack_dashboard.__file__)

    LOCALE_PATHS.append(os.path.join(OS_BASE_DIR, 'locale'))

    MIDDLEWARE_CLASSES += (
        'django_user_agents.middleware.UserAgentMiddleware',
        'tionix_dashboard.dashboards.middleware.TionixMiddleware',
        'django_tools.middlewares.ThreadLocal.ThreadLocalMiddleware',
    )
    HORIZON_CONFIG.update(help_url=get_help_url())

    populate_tnx_dashboards_static(TIONIX_DASHBOARDS_APPS, HORIZON_CONFIG)

try:
    from openstack_dashboard.local.local_settings import (
        LOGGING, OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT
    )
except ImportError:
    pass
else:
    LOGGING['formatters'] = {
        'WithExceptions': {
            '()': 'tionix_client.helpers.log.FormatterWithExceptions',
            'show_traceback': CONF.TRACEBACK_ENABLED,
        },
    }
    LOGGING['handlers'].update({
        'consoleTraceback': {
            'class': 'logging.StreamHandler',
            'formatter': 'WithExceptions',
        },
    })
    LOGGING['loggers'].update({
        'tionix_dashboard': {
            'handlers': ['consoleTraceback'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'tionix_client': {
            'handlers': ['consoleTraceback'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'tionix_scheduler_client': {
            'handlers': ['consoleTraceback'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'urllib3.connectionpool': {
            'handlers': ['console'],
            'level': 'ERROR',
            'propagate': False,
        },
        'keystoneauth': {
            'handlers': ['console'],
            'propagate': False,
        },
        'oslo_policy': {
            'handlers': ['console'],
            'propagate': False,
        },
    })

    # From django_openstack_auth/openstack_auth/backend.py:
    #
    # check django session engine, if using cookies, this will not
    # work, as it will overflow the cookie so don't add domain
    # scoped token to the session and put error in the log
    #
    # if utils.using_cookie_backed_sessions():
    #     LOG.error('Using signed cookies as SESSION_ENGINE with '
    #               'OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT is '
    #               'enabled. This disables the ability to '
    #               'perform identity operations due to cookie size '
    #               'constraints.')
    # else:
    #     request.session['domain_token'] = domain_auth_ref
    #
    # Because of domain_token doesn't exist, there are http 403 errors
    # while horizon actions (for example, on projects) when multi
    # domain support enabled.

    if OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT:
        SESSION_ENGINE = 'django.contrib.sessions.backends.cache'

TEMPLATE_DIRS = (
    os.path.join(OPENSTACK_AUTH_BASE_DIR, 'templates'),
    os.path.join(TNX_DASH_BASE_DIR, 'templates'),
)

SITE_BRANDING_LINK = reverse_lazy('horizon:tionix:licenses:index')

# Timeout for log and console rest api
LOG_CONSOLE_TIMEOUT = 5

DATABASES = {'default': dict(**CONF.DB)}

CHECK_UPDATES_TIMEOUT = 3600
MOD_VERSIONS_CACHE_KEY = 'tionix_modules_versions'

if os.environ.get('DJANGO_SETTINGS_MODULE') == 'openstack_dashboard.settings':
    logging.config.dictConfig(LOGGING)
    logging.getLogger().setLevel(CONF.LOG_LEVEL)

# Max integer value from nova
MAX_INT = 0x7FFFFFFF

OPENSTACK_IMAGE_BACKEND = {
    'image_formats': [
        ('', _('Select format')),
        ('aki', _('AKI - Amazon Kernel Image')),
        ('ami', _('AMI - Amazon Machine Image')),
        ('ari', _('ARI - Amazon Ramdisk Image')),
        ('docker', _('Docker - Docker container Image')),
        ('iso', _('ISO - Optical Disk Image')),
        ('ova', _('OVA - Open Virtual Appliance')),
        ('qcow2', _('QCOW2 - QEMU Emulator')),
        ('raw', _('RAW - unstructured format RAW disk')),
        ('vdi', _('VDI - Virtual Disk Image')),
        ('vhd', _('VHD - Virtual Hard Disk')),
        ('vmdk', _('VMDK - Virtual Machine Disk')),
    ]
}

IMAGE_CUSTOM_PROPERTY_TITLES = {
    "architecture": _("Architecture"),
    "kernel_id": _("Kernel ID"),
    "ramdisk_id": _("Ramdisk ID"),
    "image_state": _("Euca2ools state"),
    "project_id": _("Project ID"),
    "image_type": _("Image Type"),
    "base_image_ref": _("Base Image ID"),
    "image_location": _("Image Location"),
    "instance_uuid": _("Instance ID"),
    "owner_id": _("Project ID"),
    "user_id": _("User ID"),
    "hw_cdrom_bus": _("Hardware CDROM bus"),
    "hw_disk_bus": _("Hardware disk Bus"),
    "hw_floppy_bus": _("Hardware floppy bus"),
    "hw_qemu_guest_agent": _("Hardware QEMU guest agent"),
    "hw_rng": _("Hardware RNG"),
    "hw_scsi_model": _("Hardware SCSI model"),
    "hw_video_model": _("Hardware video model"),
    "hw_video_ram": _("Hardware video RAM"),
    "hw_vif_model": _("Hardware NIC model"),
    "hw_watchdog_action": _("Hardware watchdog action"),
    "os_command_line": _("OS command line"),
    "vmware_adartertype": _("VMware adapter type"),
    "vmware_disktype": _("VMware disk type"),
    "vmware_image_version": _("VMware image version"),
    "vmware_linked_clone": _("VMware linked clone"),
    "vmware_ostype": _("VMware OS type"),
    "auto_disk_config": _("Auto disk config"),
    "ipxe_boot": _("iPXE boot"),
    "os_type": _("OS type"),
    "xenapi_device_id": _("XenAPI device id"),
    "xenapi_image_compression_level": _("XenAPI image compression level"),
    "xenapi_use_agent": _("XenAPI use agent"),
    "xenapi_skip_agent_inject_ssh": _("XenAPI skip agent inject ssh"),
    "xenapi_skip_agent_inject_files_at_boot": _(
        "XenAPI skip agent inject ssh"),
    "cache_in_nova": _("Cache in Nova"),
    "vm_mode": _("Instance mode"),
    "bittorrent": _("Bittorrent")
}

IMAGE_CUSTOM_PROPERTY_VALUES = {
    "available": _("Available"),
    "snapshot": _("Snapshot"),
    "image": _("Image")
}

INSTANCE_CUSTOM_PROPERTY_TITLES = {
    "image_hw_disk_bus": _("Hardware disk Bus"),
    "image_hw_scsi_model": _("Hardware SCSI model")
}

VOLUME_ATTACH_MAX_RETRIES = 5
