/**
 * Created by user on 31.01.17.
 */
(function () {
  'use strict';

  angular
    .module('horizon.app.core.openstack-service-api')
    .factory('horizon.app.core.openstack-service-api.tnx', tnxAPI);

  tnxAPI.$inject = [
    'horizon.framework.util.http.service',
    'horizon.framework.widgets.toast.service'
  ];

  function tnxAPI(apiService, toastService) {
    var service = {
      getTnxServers: getTnxServers,
      getTnxServer: getTnxServer,
      createTnxServer: createTnxServer,
      getTnxVolumes: getTnxVolumes,
      getAggregatesByProject: getAggregatesByProject,
      getImagesForProject: getImagesForProject,
      getFlavorsForProject: getFlavorsForProject,
      getNetworksForProject: getNetworksForProject,
      getPowerDevicePorts: getPowerDevicePorts
    };

    return service;

    /**
     * @name horizon.app.core.openstack-service-api.tnx.getTnxServers
     * @description
     * Get a extended list of servers.
     */
    function getTnxServers(searchOpts) {

      var config = {'params': {}};

      if (searchOpts) {
        angular.copy(searchOpts, config['params']);
      }

      return apiService.get('/api/tnx/servers/', config)
        .error(function () {
          toastService.add('error', gettext('Unable to retrieve instances.'));
        });
    }

    /**
     * @name horizon.app.core.openstack-service-api.tnx.getTnxServer
     * @description
     * Get a single tnx extended server by ID
     * @param {string} serverId
     * Specifies the id of the server to request.
     */
    function getTnxServer(serverId) {
      return apiService.get('/api/tnx/servers/' + serverId)
        .error(function () {
          toastService.add('error', gettext('Unable to retrieve the instance.'));
        });
    }

    /**
     * @name horizon.app.core.openstack-service-api.tnx.createTnxServer
     * @description
     * Create a server using the parameters supplied in the
     * newServer. The required parameters:
     *
     * "name", "source_id", "flavor_id", "key_name", "user_data"
     *     All strings
     * "security_groups"
     *     An array of one or more objects with a "name" attribute.
     *
     * Other parameters are accepted as per the underlying novaclient:
     * "block_device_mapping", "block_device_mapping_v2", "nics", "meta",
     * "availability_zone", "instance_count", "admin_pass", "disk_config",
     * "config_drive"
     *
     * This returns the new server object on success.
     */
    function createTnxServer(newServer) {
      return apiService.post('/api/tnx/servers/', newServer)
        .error(function () {
          toastService.add('error', gettext('Unable to create the instance.'));
        });
    }

    /**
     * @name horizon.app.core.openstack-service-api.tnx.getTnxServer
     * @description
     * Get list of aggregates by project ID
     * @param {string} projectId
     * Specifies the id of the server to request.
     */
    function getAggregatesByProject(projectId) {
      return apiService.get('/api/tnx/aggregates/' + projectId + '/')
        .error(function () {
          var msg = gettext('Unable to retrieve the aggregates.');
          toastService.add('error', msg);
        });
    }

    /**Images
     * @description
     * Get list of images
     * @param {string} projectId
     * Specifies the id of the project.
     */
    function getImagesForProject(projectId) {
      var config = {'params': {}};

      if (projectId) {
        config.params['project_id'] = projectId;
      }

      return apiService.get('/api/tnx/vdi/images/', config)
        .error(function () {
          var msg = gettext('Unable to retrieve images list.');
          toastService.add('error', msg);
        });
    }

    /**Flavors
     * @description
     * Get list of flavors
     * @param {string} projectId
     * Specifies the id of the project.
     */
    function getFlavorsForProject(projectId) {
      var config = {'params': {}};

      if (projectId) {
        config.params['project_id'] = projectId;
      }

      return apiService.get('/api/tnx/vdi/flavors/', config)
        .error(function () {
          var msg = gettext('Unable to retrieve flavors list.');
          toastService.add('error', msg);
        });
    }

    /**Networks
     * @description
     * Get list of networks
     * @param {string} projectId
     * Specifies the id of the project.
     */
    function getNetworksForProject(projectId) {
      var config = {'params': {}};

      if (projectId) {
        config.params['project_id'] = projectId;
      }

      return apiService.get('/api/tnx/vdi/networks/', config)
        .error(function () {
          var msg = gettext('Unable to retrieve networks list.');
          toastService.add('error', msg);
        });
    }

    /**Power device ports
     * @description
     * Get list of power device ports
     * @param {string} powerDeviceId
     * Specifies the id of the power device.
     */
    function getPowerDevicePorts(powerDeviceId) {
      var config = {'params': {}};

      if (powerDeviceId) {
        config.params['power_device_id'] = powerDeviceId;
      }

      return apiService.get('/api/tnx/hypervisors/power_device_ports/', config)
        .error(function () {
          var msg = gettext('Unable to retrieve power control ports list.');
          toastService.add('error', msg);
        });
    }

    /**Extended volumes list
     * @description
     * Get list of volumes
     * @param {Object} params
     * Query parameters. Optional.
     */
    function getTnxVolumes(params) {
      var config = params ? {'params': params} : {};
      return apiService.get('/api/tnx/volumes/', config)
        .error(function () {
          toastService.add('error', gettext('Unable to retrieve the volumes.'));
        });
    }

  }

}());