# coding: utf-8

import functools
import logging
import uuid

from keystoneclient import exceptions as keystone_exceptions

from novaclient import exceptions as nova_exceptions

from neutronclient.common import exceptions as neutron_exceptions

from tionix_client.os_clients.neutron_client import get_neutron_client
from tionix_client.os_clients.nova_client import get_nova_client


logger = logging.getLogger(__name__)


def add_nova_client(fn):
    """Decorator to add nova client"""
    @functools.wraps(fn)
    def wrapper(username, password, tenant_id, object_id, object_name,
                user_domain_name, *args, **kwargs):
        try:
            nova_client = get_nova_client(username=username,
                                          password=password,
                                          tenant_id=object_id,
                                          user_domain_name=user_domain_name)
        except (nova_exceptions.Unauthorized,
                keystone_exceptions.Unauthorized):
            logger.error(u'Error occurred while executing task. '
                         u'Task was created with wrong authorization data.')
            return u'Unable to get nova client.', False

        return fn(nova_client, object_name, *args, **kwargs)

    return wrapper


def add_nova_neutron_clients(fn):
    """Decorator to add nova and neutron clients"""
    @functools.wraps(fn)
    def wrapper(username, password, tenant_id, object_id, object_name,
                user_domain_name, *args, **kwargs):
        try:
            nova_client = get_nova_client(username=username,
                                          password=password,
                                          tenant_id=object_id,
                                          user_domain_name=user_domain_name)
        except (nova_exceptions.Unauthorized,
                keystone_exceptions.Unauthorized):
            logger.error(u'Error occurred while executing task. '
                         u'Task was created with wrong authorization data.')
            return u'Unable to get nova client.', False

        try:
            neutron_client = get_neutron_client(
                username=username, password=password,
                tenant_id=object_id, user_domain_name=user_domain_name)
        except (neutron_exceptions.Unauthorized,
                keystone_exceptions.Unauthorized):
            logger.error(u'Error occurred while executing task. '
                         u'Task was created with wrong authorization data.')
            return u'Unable to get neutron client.', False

        return fn(nova_client, neutron_client, *args, **kwargs)

    return wrapper


def generate_uuid_name(name):
    return u'{name}_{uuid}'.format(name=name or u'object', uuid=uuid.uuid4())


# pylint: disable=too-many-arguments
def launch_instances(nova_client, image_id, flavor_id, nics, instances_count,
                     server_descr, instance_name=None, availability_zone=None):

    instance_name = generate_uuid_name(instance_name)
    try:
        server = nova_client.servers.create(
            instance_name,
            image_id,
            flavor_id,
            min_count=instances_count,
            availability_zone=availability_zone,
            nics=nics,
            description=server_descr)
    except Exception as err:
        logger.error(unicode(err))
        result = (
            u'Unable to launch instance "{0}": '
            u'{1}'.format(instance_name, err),
            False)
    else:
        if instances_count > 1:
            reservation_id = getattr(server,
                                     'OS-EXT-SRV-ATTR:reservation_id')
            # getting newly created servers list by reservation_id
            servers = nova_client.servers.list(
                search_opts={'reservation_id': reservation_id}
            )

            result = (
                u'Successfully launched {0} instances with IDs: '
                u'{1}'.format(instances_count, u', '.join([s.id
                                                           for s in servers])),
                True)
        else:
            result = (u'Successfully launched instance with ID: {0}'
                      u''.format(server.id), True)

    return result
