# coding: utf-8
# pylint: disable=import-error

from datetime import datetime
import pytz
from babel import dates

from django import template
from django.forms import widgets
from django.conf import settings
from django.utils import timezone
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django_tools.middlewares import ThreadLocal

from tionix_dashboard.helpers.timesince import timesince
from tionix_dashboard.helpers.utils import parse_datetime

register = template.Library()


@register.filter
def valid(value):
    statuses = {
        True: _('Valid'),
        False: _('Invalid'),
        None: '-'
    }
    return statuses.get(value)


@register.filter
def filter_version(value):
    return value or _('Not Available')


@register.filter
def format_date(date_string, format_pattern="medium", tz=None, locale=None):
    """
    Tries to parses a date-like input string into a timezone aware Python
        datetime.
    :param date_string: datetime string
    :param format_pattern: one of "full", "long", "medium", or "short", or a
            custom date/time pattern
    :param tz: timezone
    :param locale: locale
    """
    request = ThreadLocal.get_current_request()

    tz = tz or pytz.timezone(request.COOKIES.get('django_timezone',
                                                 settings.TIME_ZONE))
    if not locale:
        locale = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME,
                                     request.LANGUAGE_CODE)

    string_formats = [
        "%d.%m.%Y %H:%M:%S", "%Y-%m-%dT%H:%M:%S.%f", "%Y-%m-%d %H:%M:%S.%f",
        "%Y-%m-%dT%H:%M:%S", "%Y-%m-%d %H:%M:%S", "%Y-%m-%dT%H:%M:%SZ",
        "%Y-%m-%dT%H:%M:%S.%fZ", "%Y-%m-%d %H:%M:%S.%fZ",
        "%Y-%m-%d %H:%M:%SZ",
    ]
    date = date_string
    for string_format in string_formats:
        try:
            date = datetime.strptime(date_string, string_format)
        except (TypeError, ValueError):
            pass
        else:
            # noinspection PyTypeChecker
            if isinstance(date, basestring):
                date = parse_datetime(date)
            if not timezone.is_aware(date):
                date = timezone.make_aware(date, pytz.timezone('UTC'))
            date = dates.format_datetime(date, format=format_pattern,
                                         tzinfo=tz, locale=locale)
            break

    return date


@register.filter
def timesince_sortable(dt):
    delta = timezone.now() - dt
    # timedelta.total_seconds() not supported on python < 2.7
    seconds = delta.seconds + (delta.days * 24 * 3600)
    return mark_safe("<span data-seconds=\"%d\">%s</span>" %
                     (seconds, timesince(dt)))


@register.filter(is_safe=False)
def actions_sort(actions, is_plural=False):
    """
    Takes a list of actions, sorts by verbose_name or verbose_name_plural.
    """

    def _key(action):
        verbose_name = None

        if getattr(action, 'on_the_top', False):
            return True

        if is_plural and action.handles_multiple:
            verbose_name = getattr(action, 'verbose_name_plural')

        verbose_name = verbose_name or action.verbose_name

        if callable(verbose_name):
            verbose_name = verbose_name()

        return verbose_name

    return sorted(actions, key=_key)


@register.filter(is_safe=False)
def default_if_empty(value, arg):
    """If value is empty, use given default."""
    if value in [None, '']:
        return arg
    return value


@register.filter(is_safe=False)
def div(value, arg):
    """"Divides the value by the arg"""
    return int(value) / int(arg)


@register.filter(is_safe=False)
def progress_context(value):
    """"Retrieve progress bar context """
    value = int(value)
    if value < 30:
        return_value = "success"
    elif value < 70:
        return_value = "warning"
    else:
        return_value = "danger"
    return return_value


# noinspection PyTypeChecker
@register.filter(is_safe=False)
def empty_string_to_nbsp_in_list(values):
    return [format_html('&nbsp;')
            if isinstance(value, basestring) and not value else value
            for value in values] if values else values


@register.filter
def add_html_required(field):
    """Add a "required" attribute to input field."""
    if not isinstance(field.field.widget, (widgets.CheckboxInput,
                                           widgets.CheckboxSelectMultiple,
                                           widgets.RadioSelect,
                                           str)) and field.field.required:
        field.field.widget.attrs['required'] = True
    return field


@register.assignment_tag()
def is_javascript_link(value):
    return 'javascript' in value if value else False
