# coding: utf-8

import json
import re
import mock

from django import shortcuts

from horizon import tables as horizon_tables, tabs as horizon_tabs, messages
from horizon.tables.actions import FilterAction

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.dashboards.helpers import Empty
from tionix_dashboard.helpers import actions, base, tables, tabs, views
from tionix_dashboard.helpers.utils import get_table_filter_action
from tionix_dashboard.tests.helpers import BaseClientTestCase, FakeUser


# noinspection PyAbstractClass
class TestFilterAction(FilterAction):

    filter_type = None

    filter_string_modifiers = {
        'filter_field': lambda x: x + '_modified'
    }

    def get_param_name(self):
        return 'filter_param_name'


# noinspection PyAbstractClass
class TestServerFilterAction(TestFilterAction):

    filter_type = 'server'

    filter_choices = ()


# noinspection PyClassHasNoInit
class TestTable(horizon_tables.DataTable):

    name = horizon_tables.Column("name", verbose_name='Test_column')

    class Meta(object):
        table_actions = (TestFilterAction, )


# noinspection PyClassHasNoInit
class TestTableWithServerFilter(TestTable):

    class Meta(object):
        table_actions = (TestServerFilterAction, )


# noinspection PyAbstractClass, PyClassHasNoInit
class TestTabGroup(horizon_tabs.TabGroup):
    slug = 'test_tab_group'
    tabs = []


class FakeTab(Empty):

    def post(self, request, *args, **kwargs):
        pass


class FakeView(Empty):

    def post(self, request, *args, **kwargs):
        pass


# noinspection PyAbstractClass
class TestServerFilterTabMixin(views.ServerFilterTabMixin, horizon_tabs.Tab):
    name = 'test_tab'
    slug = 'test_tab'
    table_classes = (TestTable, )


# noinspection PyAbstractClass
class TestServerFilterTabMixin2(views.ServerFilterTabMixin, horizon_tabs.Tab):
    name = 'test_tab'
    slug = 'test_tab'
    table_classes = (TestTableWithServerFilter, )


# noinspection PyAbstractClass
class TestTnxBatchAction(actions.TnxBatchActionMixin,
                         horizon_tables.BatchAction):
    name = 'test'

    # noinspection PyUnusedLocal
    @staticmethod
    def action_present(count):
        return 'test'

    # noinspection PyUnusedLocal
    @staticmethod
    def action_past(count):
        return 'test'


class ActionsTestCase(BaseClientTestCase):

    @mock.patch.object(horizon_tables.DataTable,
                       'get_filter_field',
                       mock.Mock(return_value='name'))
    def test_tnx_filter_action(self):

        action = actions.TnxFilterAction(
            verbose_name='',
        )

        data = [
            Empty(**{'name': 'Test data 1', 'id': 1}),
            Empty(**{'name': 'data 2', 'id': 2}),
            Empty(**{'name': 'data for test 3', 'id': 3})
        ]

        self.assertListEqual(
            action.filter(
                table=TestTable(self.request),
                data=data,
                filter_string='test'
            ),
            data[::2]
        )

    def test_tnx_update_action(self):

        action = actions.TnxUpdateAction()

        arguments = []

        action.update_cell = lambda *x: arguments.extend(x)

        action.action(self.request, None, None, '', ' new value ')

        self.assertEqual(arguments[4], 'new value')

    def test_tnx_batch_action_danger_button(self):

        action = TestTnxBatchAction()

        action.set_danger_btn_class(True)

        self.assertListEqual(action.classes, ['btn-danger'])

        action.set_danger_btn_class()

        self.assertListEqual(action.classes, [])

    def test_tnx_batch_action_default_attrs(self):

        verbose_name = 'test verbose_name'
        verbose_name_callback = 'test verbose_name_callback'

        action = TestTnxBatchAction(verbose_name=verbose_name)
        action.datum = Empty(id=None)
        action.table = TestTable(self.request)

        default_attrs = action.get_default_attrs()

        self.assertTrue('data-confirm-header' in default_attrs)
        self.assertEqual(default_attrs['data-confirm-header'],
                         verbose_name)

        action.verbose_name = lambda: verbose_name_callback

        default_attrs = action.get_default_attrs()

        self.assertTrue('data-confirm-header' in default_attrs)
        self.assertEqual(default_attrs['data-confirm-header'],
                         verbose_name_callback)

    def test_tnx_batch_action_get_confirm_header(self):

        verbose_name = 'test verbose_name'
        verbose_name_plural = 'test verbose_name_plural'

        action = TestTnxBatchAction(verbose_name=verbose_name,
                                    verbose_name_plural=verbose_name_plural)

        self.assertEqual(action._get_confirm_header([Empty(id=None)]),
                         verbose_name)

        self.assertEqual(action._get_confirm_header(5),
                         verbose_name)

        self.assertEqual(action._get_confirm_header('test'),
                         verbose_name_plural)

    @mock.patch.object(messages, 'error')
    @mock.patch.object(messages, 'info')
    @mock.patch.object(messages, 'success')
    def test_tnx_batch_action_handle_success(self, success_handler,
                                             info_handler, error_handler):
        test_data_ids = range(3)
        test_data = [Empty(id=i) for i in test_data_ids]

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={'async_messages': []}
        )

        action = TestTnxBatchAction()
        action.get_success_url = lambda *args: Empty(
            get_absolute_url=lambda: None
        )
        action.table = TestTable(request, data=test_data)

        action.handle(action.table, request, test_data_ids)
        self.assertTrue(success_handler.called)
        self.assertFalse(info_handler.called)
        self.assertFalse(error_handler.called)

    @mock.patch.object(messages, 'error')
    @mock.patch.object(messages, 'info')
    @mock.patch.object(messages, 'success')
    def test_tnx_batch_action_handle_error(self, success_handler, info_handler,
                                           error_handler):
        test_data_ids = range(3)
        test_data = [Empty(id=i) for i in test_data_ids]

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={'async_messages': []}
        )

        action = TestTnxBatchAction()
        action.get_success_url = lambda *args: Empty(
            get_absolute_url=lambda: None
        )
        action.table = TestTable(request, data=test_data)

        action.action = mock.Mock(side_effect=Exception)
        action.handle(action.table, request, test_data_ids)
        self.assertFalse(success_handler.called)
        self.assertFalse(info_handler.called)
        self.assertTrue(error_handler.called)

    @mock.patch.object(messages, 'error')
    @mock.patch.object(messages, 'info')
    @mock.patch.object(messages, 'success')
    def test_tnx_batch_action_handle_filter_error(self, success_handler,
                                                  info_handler, error_handler):
        test_data_ids = range(3)
        test_data = [Empty(id=i) for i in test_data_ids]

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={'async_messages': []}
        )

        action = TestTnxBatchAction()
        action.get_success_url = lambda *args: Empty(
            get_absolute_url=lambda: None
        )
        action.table = TestTable(request, data=test_data)

        action.table._filter_action = lambda table, req, x: x.id == 1
        action.handle(action.table, request, test_data_ids)
        self.assertFalse(success_handler.called)
        self.assertTrue(info_handler.called)
        self.assertTrue(error_handler.called)


class BaseTestCase(BaseClientTestCase):

    def test_base_enumerate(self):

        enum_dict = {
            1: 'test 1',
            2: 'test 2'
        }

        enum = base.BaseEnumerate
        enum.values = enum_dict

        self.assertSequenceEqual(
            enum.get_choices(), list(enum_dict.iteritems())
        )

        self.assertDictEqual(
            enum.get_constant_value_by_name('values'), enum_dict
        )

        with self.assertRaises(ValueError):
            enum.get_constant_value_by_name('')

        with self.assertRaises(TypeError):
            enum.get_constant_value_by_name(None)


class TablesTestCase(BaseClientTestCase):

    class TestTnxPaginationDataTable(tables.TnxPaginationDataTable):
        class Meta(object):
            table_actions = (TestServerFilterAction, )

    @mock.patch('tionix_dashboard.helpers.tables.get_page_size',
                mock.Mock(return_value=0))
    def test_dt_tnx_pagination_get_prev_marker(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                get_params={
                    'prev_marker': '1'

                }
            )
        )

        self.assertEqual(table.get_prev_marker(), '1')

    @mock.patch('tionix_dashboard.helpers.tables.get_page_size',
                mock.Mock(return_value=0))
    def test_dt_tnx_pagination_get_marker(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                get_params={
                    'marker': '1'
                }
            )
        )

        self.assertEqual(table.get_marker(), '1')

    def test_dt_tnx_pagination_get_checked_ids(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                session={
                    'object_ids': ['1', '2', '3']
                }
            )
        )

        self.assertJSONEqual(table.get_checked_ids(), json.dumps([1, 2, 3]))

    def test_colored_row_mixin_1(self):

        mixin = tables.ColoredRowMixin()

        self.assertEqual(mixin.get_row_status_class(1), '')

    def test_colored_row_mixin_2(self):

        mixin = tables.ColoredRowMixin()
        mixin.STATUS_CLASSES = {
            1: 'class 1',
            2: 'class 2'
        }

        self.assertEqual(mixin.get_row_status_class(1), 'class 1')

    def test_tnx_pagination_filter_string_1(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                post_params={'filter_param_name': 'post'},
                session={'filter_param_name': 'session'}
            )
        )

        self.assertEqual(table.get_filter_string(), 'post')

    def test_tnx_pagination_filter_string_2(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                session={
                    'filter_param_name': 'session'
                }
            )
        )

        self.assertEqual(table.get_filter_string(), 'session')

    def test_tnx_pagination_filter_field_1(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                post_params={'filter_param_name_field': 'post'},
                session={'filter_param_name_field': 'session'}
            )
        )

        self.assertEqual(table.get_filter_field(), 'post')

    def test_tnx_pagination_filter_field_2(self):

        table = self.TestTnxPaginationDataTable(
            request=get_fake_request(
                session={
                    'filter_param_name_field': 'session'
                }
            )
        )

        self.assertEqual(table.get_filter_field(), 'session')


class TabsTestCase(BaseClientTestCase):

    # noinspection PyAbstractClass
    class TestDynamicTabGroup(tabs.DynamicTabGroup):
        tabs = []
        slug = 'test_dynamic_tab_group'
        delete_url = 'delete'

    def test_dynamic_tab_group_context_data(self):

        tab_group = self.TestDynamicTabGroup(self.request)

        context_data = tab_group.get_context_data()

        self.assertListEqual(
            sorted(context_data.keys()),
            sorted(['new_tab', 'tab_group', u'csrf_token'])
        )

    def test_dynamic_tab_group_render(self):

        tab_group = self.TestDynamicTabGroup(self.request)

        self.assertTrue(
            'DynamicTabController' in tab_group.render()
        )


# pylint: disable=no-member
class ViewsTestCase(BaseClientTestCase):

    class TestServerPaginationMixin(views.ServerPaginationViewMixin,
                                    horizon_tables.DataTableView):

        @staticmethod
        def get_filters(filters):
            return filters

    class TestServerFilterViewMixin(views.ServerFilterViewMixin, FakeView):
        pass

    @mock.patch.object(FilterAction,
                       'is_api_filter',
                       mock.Mock(return_value=True))
    def test_filter_string_modifier_mixin_1(self):

        view = views.FilterStringModifierMixin()
        view.table = TestTableWithServerFilter(
            request=get_fake_request(
                session={
                    'filter_param_name': 'filter_string',
                    'filter_param_name_field': 'filter_field'
                }
            )
        )
        view.request = get_fake_request()

        self.assertDictEqual(
            view.get_filters(),
            {'filter_field': 'filter_string_modified', 'marker': None}
        )

    @mock.patch.object(FilterAction,
                       'is_api_filter',
                       mock.Mock(return_value=True))
    def test_filter_string_modifier_mixin_2(self):

        view = views.FilterStringModifierMixin()
        view.table = TestTableWithServerFilter(
            request=get_fake_request(
                session={
                    'filter_param_name': 'filter_string',
                    'filter_param_name_field': 'name'
                }
            )
        )
        view.request = get_fake_request()

        self.assertDictEqual(
            view.get_filters(),
            {'marker': None, 'name': re.escape('filter_string')}
        )

    def test_server_sort_multi_select_1(self):

        view = views.ServerSortMultiSelect()

        request = get_fake_request(
            session={
                'object_ids': '',
                'sort_dir': '',
                'sort_key': ''
            },
            method='GET'
        )

        view.handler_server_params(request)

        self.assertDictEqual(request.session, {})

    def test_server_sort_multi_select_2(self):

        view = views.ServerSortMultiSelect()

        params = {
            'object_ids': [1, 2, 3],
            'sort_dir': 'desc',
            'sort_key': 'key'
        }

        request = get_fake_request(
            session={},
            post_params=params,
            method='POST'
        )

        view.handler_server_params(request)

        self.assertDictEqual(request.session, params)

    def test_server_sort_multi_select_get(self):

        view = views.ServerSortMultiSelect()

        params = {
            'object_ids': [1, 2, 3],
            'sort_dir': 'desc',
            'sort_key': 'key'
        }

        request = get_fake_request(
            session={
                'is_redirect': True
            },
            post_params=params,
            method='GET'
        )

        view.get(request)

        self.assertDictEqual(request.session, params)

    def test_dtv_tnx_pagination_mixin_has_prev_data(self):

        view = views.DTVTnxPaginationMixin()

        self.assertTrue(view.has_prev_data(None))

    def test_dtv_tnx_pagination_mixin_has_more_data(self):

        view = views.DTVTnxPaginationMixin()

        self.assertTrue(view.has_more_data(None))

    def test_dtv_tnx_pagination_mixinget_offset_limit_1(self):

        view = views.DTVTnxPaginationMixin()
        view.table = Empty(_meta=Empty(prev_pagination_param='prev_param'))
        view.request = get_fake_request(
            session={'horizon_pagesize': 5},
            get_params={'prev_param': '5'},
            COOKIES={}
        )

        self.assertSequenceEqual(view.get_offset_limit(), (5, 5))

    def test_dtv_tnx_pagination_mixinget_offset_limit_2(self):

        view = views.DTVTnxPaginationMixin()
        view.table = Empty(_meta=Empty(prev_pagination_param='prev_param',
                                       pagination_param='param'))
        view.request = get_fake_request(
            session={'horizon_pagesize': 5},
            get_params={'param': '5'},
            COOKIES={}
        )

        self.assertSequenceEqual(view.get_offset_limit(), (5, 5))

    def test_dtv_tnx_pagination_mixin_calc_last_marker(self):

        view = views.DTVTnxPaginationMixin()

        self.assertEqual(view.calc_last_marker(total=10, page_size=5), 5)

        self.assertEqual(view.calc_last_marker(total=14, page_size=5), 10)

    @mock.patch.object(shortcuts, 'redirect', mock.Mock(return_value=True))
    def test_dtv_tnx_pagination_mixin_post_1(self):

        view = views.DTVTnxPaginationMixin()
        view.handle_server_filter = lambda x: True
        view.get_table = lambda: Empty(get_absolute_url=lambda: 'test_url')

        request = get_fake_request(
            session={},
            method='GET'
        )

        result = view.post(request)

        self.assertTrue(result)

        self.assertDictEqual(request.session, {'is_redirect': True})

    def test_dtv_tnx_pagination_mixin_post_2(self):

        view = views.DTVTnxPaginationMixin()

        request = get_fake_request(
            session={},
            method='GET'
        )

        view.post(request)

        self.assertDictEqual(request.session, {})

    def test_server_filter_tab_mixin_filter_info_1(self):

        view = TestServerFilterTabMixin(TestTabGroup(self.request),
                                        self.request)

        self.assertIsNone(view.get_server_filter_info(self.request))

    def test_server_filter_tab_mixin_filter_info_2(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        res = view.get_server_filter_info(
            request=get_fake_request(
                post_params={
                    'filter_param_name': 'test',
                    'filter_param_name_field': 'test_field'
                },
                session={}
            )
        )

        filter_action = get_table_filter_action(view.table)

        self.assertDictEqual(
            res,
            {
                'value_param': 'filter_param_name',
                'field_param': 'filter_param_name_field',
                'changed': True,
                'value': 'test',
                'field': 'test_field',
                'action': filter_action
            }
        )

    def test_server_filter_tab_mixin_filter_info_3(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        res = view.get_server_filter_info(
            request=get_fake_request(
                post_params={
                    'filter_param_name': 'test',
                    'filter_param_name_field': 'test_field'
                },
                session={'filter_param_name': 'test'}
            )
        )

        self.assertFalse(res['changed'])

    def test_server_filter_tab_mixin_filter_info_4(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        res = view.get_server_filter_info(
            request=get_fake_request(
                post_params={
                    'filter_param_name_field': 'test_field'
                },
                session={'filter_param_name': 'session test'}
            )
        )

        self.assertEqual(res['value'], 'session test')

    def test_server_filter_tab_mixin_filter_info_5(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        res = view.get_server_filter_info(
            request=get_fake_request(
                session={
                    'filter_param_name_field': 'session test field'
                }
            )
        )

        self.assertEqual(res['field'], 'session test field')

    def test_handle_server_filter_1(self):

        view = TestServerFilterTabMixin(TestTabGroup(self.request),
                                        self.request)

        self.assertFalse(view.handle_server_filter(self.request))

    def test_handle_server_filter_2(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        request = get_fake_request(
            session={
                'filter_param_name_field': 'field'
            }
        )

        res = view.handle_server_filter(request)

        self.assertFalse(res)
        self.assertEqual(request.session['filter_param_name'], '')
        self.assertEqual(request.session['filter_param_name_field'], 'field')

    def test_handle_server_filter_3(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        res = view.handle_server_filter(
            request=get_fake_request(
                post_params={
                    'filter_param_name': 'test',
                    'filter_param_name_field': 'test_field'
                },
                session={}
            )
        )

        self.assertTrue(res)

    def test_update_server_filter_action_1(self):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)
        view.request = get_fake_request(
            post_params={
                'filter_param_name': 'test',
                'filter_param_name_field': 'test_field'
            },
            session={}
        )

        view.update_server_filter_action()

        filter_action = get_table_filter_action(view.table)

        self.assertEqual(
            getattr(filter_action, 'filter_string', ''),
            'test'
        )

        self.assertEqual(
            getattr(filter_action, 'filter_field', ''),
            'test_field'
        )

    def test_update_server_filter_action_2(self):

        view = TestServerFilterTabMixin(TestTabGroup(self.request),
                                        self.request)
        view.request = self.request

        filter_action = get_table_filter_action(view.table)

        self.assertFalse(
            hasattr(filter_action, 'filter_string')
        )

        self.assertFalse(
            hasattr(filter_action, 'filter_field')
        )

    @mock.patch.object(shortcuts, 'redirect')
    def test_server_filter_tab_mixin_post_1(self, redirect):

        view = TestServerFilterTabMixin2(TestTabGroup(self.request),
                                         self.request)

        view.post(
            request=get_fake_request(
                post_params={
                    'filter_param_name': 'test',
                    'filter_param_name_field': 'test_field'
                },
                session={}
            )
        )

        self.assertTrue(redirect.called)

    @mock.patch.object(shortcuts, 'redirect')
    def test_server_filter_tab_mixin_post_2(self, redirect):

        view = TestServerFilterTabMixin(TestTabGroup(self.request),
                                        self.request)

        view.post(self.request)

        self.assertFalse(redirect.called)

    @mock.patch.object(FakeTab, 'post')
    def test_server_filter_view_mixin_1(self, post):

        request = get_fake_request(
            get_params={
                'tab': 'tab__test',
            },
            post_params={
                'search_action': 'test'
            }
        )

        view = self.TestServerFilterViewMixin()
        view.kwargs = {}
        view.request = request
        view.get_tabs = (
            lambda *x: Empty(get_tabs=lambda: [FakeTab(slug='test')])
        )

        view.post(request)

        self.assertTrue(post.called)

    @mock.patch.object(FakeTab, 'post')
    def test_server_filter_view_mixin_2(self, get_tabs):

        view = self.TestServerFilterViewMixin()
        view.get_tabs = (
            lambda *x: Empty(get_tabs=lambda: [FakeTab(slug='test')])
        )

        view.post(self.request)

        self.assertFalse(get_tabs.called)

    def test_server_pagination_mixin_mark_init_1(self):

        view = self.TestServerPaginationMixin()
        view.request = get_fake_request(
            get_params={
                'prev_marker': '5'
            }
        )
        view.table = TestTable(self.request)

        view.marker_init()

        self.assertEqual(view.sort_dir, 'asc')

    def test_server_pagination_mixin_mark_init_2(self):

        view = self.TestServerPaginationMixin()
        view.request = get_fake_request(
            get_params={
                'marker': '0'
            }
        )
        view.table = TestTable(self.request)

        view.marker_init()

        self.assertEqual(view.sort_dir, 'asc')
        self.assertEqual(view.is_last_page, True)

    def test_server_pagination_mixin_mark_init_3(self):

        view = self.TestServerPaginationMixin()
        view.request = get_fake_request(
            get_params={
                'marker': '5'
            }
        )
        view.table = TestTable(self.request)

        view.marker_init()

        self.assertEqual(view.sort_dir, 'desc')
        self.assertEqual(view.is_last_page, False)

    @mock.patch.object(shortcuts, 'redirect')
    def test_server_pagination_mixin_post_1(self, redirect):

        request = get_fake_request(
            user=FakeUser(),
            post_params={
                'search_action': 'restart'
            },
            get_full_path=lambda: '',
            method='POST'
        )

        view = self.TestServerPaginationMixin(
            request=request,
            kwargs={},
            table_class=TestTable
        )

        view.post(request)

        self.assertTrue(redirect.called)

    @mock.patch.object(shortcuts, 'redirect')
    def test_server_pagination_mixin_post_2(self, redirect):

        request = get_fake_request(
            user=FakeUser(),
            post_params={
                'search_action': 'button',
                'filter_param_name': 'new test',
                'filter_param_name_field': 'test_field'

            },
            session={'filter_param_name': 'test'},
            get_full_path=lambda: ''
        )

        view = self.TestServerPaginationMixin(
            request=request,
            kwargs={},
            table_class=TestTableWithServerFilter
        )

        view.post(request)

        self.assertTrue(redirect.called)

    @mock.patch.object(shortcuts, 'redirect')
    @mock.patch.object(horizon_tables.DataTableView, 'get')
    def test_server_pagination_mixin_post_3(self, get, redirect):

        request = get_fake_request(
            user=FakeUser(),
            post_params={
                'search_action': 'button',
                'filter_param_name': 'test',
                'filter_param_name_field': 'test_field'

            },
            session={'filter_param_name': 'test'},
            get_full_path=lambda: ''
        )

        view = self.TestServerPaginationMixin(
            request=request,
            kwargs={},
            table_class=TestTableWithServerFilter
        )

        view.post(request)

        self.assertTrue(get.called)

        self.assertFalse(redirect.called)

    @mock.patch.object(shortcuts, 'redirect')
    @mock.patch.object(horizon_tables.DataTableView, 'get')
    def test_server_pagination_mixin_post_4(self, get, redirect):

        request = get_fake_request(
            user=FakeUser(),
            get_full_path=lambda: '',
            session={}
        )

        view = self.TestServerPaginationMixin(
            request=request,
            kwargs={},
            table_class=TestTableWithServerFilter
        )

        view.post(request)

        self.assertTrue(get.called)

        self.assertFalse(redirect.called)
